!=================================================!
!The following modules contains the subroutines 
!that run the calculation of the different modules
!=================================================!

MODULE MIMOSIS_SUBROUTINES

    USE MIMOSIS_DATA
    IMPLICIT NONE
    
    CONTAINS

!===============================================!
!Read in C:\MIMOSIS_LOCPARAM.txt
!===============================================!

SUBROUTINE MIMOSIS_START

    INTEGER:: I   
    CHARACTER(len=150) :: MIMOSIS_STARTPAR
    CHARACTER(len=3)   :: READVAR
    INTEGER            :: ierror

    !Search for the location of the parameter file
    MIMOSIS_STARTPAR = 'C:\FOD_MIMS2009_LOCPARAM.txt'
    OPEN (UNIT=4, FILE=MIMOSIS_STARTPAR, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') MIMOSIS_STARTPAR
        PAUSE
        STOP
    END IF

    IF (ierror.NE.0) THEN
        WRITE (*,*) 'Could not find '//MIMOSIS_STARTPAR
        WRITE(*,*) 'Type the location name of the file that contains the location of the MIMOSIS parameter file'
        WRITE(*,*) 'The preferred location of this file is C:\FOD_MIMS2009_LOCPARAM.txt'
        READ (*,'(a150)') MIMOSIS_STARTPAR
        OPEN (UNIT=4, FILE=MIMOSIS_STARTPAR, STATUS='OLD', ACTION='READ', IOSTAT=ierror)        
        IF (ierror.NE.0) THEN
            WRITE(*,'("Could not find ", a100)') MIMOSIS_STARTPAR
            PAUSE
            STOP
        END IF
    END IF
    READ (4,'(a150)') MIMOSIS_PARLPRE    
    READ (4,'(a150)') MIMOSIS_PARLPOST            
    READ (4,'(a3)')   READVAR
    READ (4,'(a150)') MIMOSIS_RUN_LOG
    READ (4,'(a150)') EVAL_START
    DO I=1,16
       !line 9 contains control yes or no in the first 3 columns
        READ (4,'(a3)') READVAR
    END DO
    READ (4,'(a3)')   CONTROL_TYPESIM(1)
    READ (4,'(a3)')   CONTROL_TYPESIM(2)
    READ (4,'(a3)')   CONTROL_TYPESIM(3)
    !Read number of units of observation
    READ (4,'(i10)')  N_REC
    CLOSE(UNIT=4)

    CALL CONTROL_CONVERT(CONTROL_TYPESIM(1)) 
    CALL CONTROL_CONVERT(CONTROL_TYPESIM(2)) 
    CALL CONTROL_CONVERT(CONTROL_TYPESIM(3))                 

    !Define the number of lines to be read from the MIMOSIS parameter file into the vector CONTROL_IO    
    N_LOCLINES = 236  

END SUBROUTINE MIMOSIS_START

!===============================================!
!Read in all parameters from the parameter file with the basic locations!                         
!===============================================!

SUBROUTINE MIMOSIS_PARAM

    INTEGER:: I   
    !Local parameters for locating output of the global budget files
    CHARACTER(len=150)                                      :: GLOBBUD_EMPL_FILE                    !Name of output file for employees
    CHARACTER(len=150)                                      :: GLOBBUD_SELFEMPL_FILE                !Name of output file for employees
    !Remark: the output path and the result variable are defined as PUBLIC
    !Tools to read file locations
    CHARACTER(len=1)                                        :: MIMOSIS_PAR_BREAK='@'                !Character used to identify end of string
    INTEGER,DIMENSION(3)                                    :: MIMOSIS_PAR_BREAK_POINT              !Vector registering end of string
    INTEGER                                                 :: ierror
    
    !Check control 
    !If no ask to continue
    IF (CONTROL=='no'.OR.CONTROL=='No'.OR.CONTROL=='NO')THEN         
        WRITE (*,*) 'You did answer earlier that you did not want to run this simulation. Do you want to continue? (yes,no)'
        READ (*,'(a3)') CONTROL
        CALL CONTROL_CONVERT(CONTROL) 
        CALL CONTROL_CYCLE(CONTROL)
    END IF

    !If control is yes, read in parameters
    !else stop
    IF (CONTROL=='yes') THEN
        !open parameter file after the location has been found            
        OPEN (UNIT=3, FILE=MIMOSIS_PARLOC, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
        IF (ierror.NE.0) THEN
            WRITE(*,'("Could not find ", a100)') MIMOSIS_PARLOC
            PAUSE
            STOP
        END IF

        READ (3,'(a150)')       MIMRUN_HEADING
        WRITE (*,'(1X,a150)')   MIMRUN_HEADING                     
        !Read the inflation factor
        DO i=1,3
            READ (3,'(f10.6)')      INFLATE_NOMVAL(i)
            !inflation factor for wages
            !inflation factor for pensions
            !inflation factor for nominal amounts other than wages and pensions
        END DO    
        !Read simulation year
        READ (3,'(i10)')        SIMYEAR
!        WRITE (*,'(f10.6)')   INFLATE_NOMVAL                     
        !Allocate the control input output vector
        !ALLOCATE(CONTROL_IO(N_LOCLINES))          
        !deze allocate in combinatie met PUBLIC in data module werkte niet, Hoe komt dat?
        !read in parameters for path and name of input files  
        DO i=1,n_loclines
            !read in parameters for path and name of input files  
            READ(3,'(a150)') CONTROL_IO(i)  
        END DO
        CLOSE(UNIT=3)
        !Determine the location of the global budget files
        DO i=1,n_loclines/2
            !read location of parameter-files of different modules
            IF (CONTROL_IO(2*i-1)=='TESTREAD') THEN
                TESTREAD = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='PIT_PARLOC') THEN     
                PIT_PARLOC = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='PIT_LOG') THEN        
                PIT_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='PIT_PRERUN_LOG') THEN             
                PIT_PRERUN_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_PARLOC') THEN      
                FAMAL_PARLOC = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_LOG') THEN         
                FAMAL_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_PRERUN_LOG') THEN             
                FAMAL_PRERUN_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='PENSWELF_PARLOC') THEN      
                PENSWELF_PARLOC = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='PENSWELF_LOG') THEN         
                PENSWELF_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='PENSWELF_PRERUN_LOG') THEN             
                PENSWELF_PRERUN_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_PARLOC') THEN              
                CONTRIB_PARLOC = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_LOG') THEN       
                CONTRIB_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_PRERUN_LOG') THEN           
                CONTRIB_PRERUN_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='UNEM_PARLOC') THEN      
                UNEM_PARLOC = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='UNEM_LOG') THEN         
                UNEM_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='UNEM_PRERUN_LOG') THEN             
                UNEM_PRERUN_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='SICK_PARLOC') THEN              
                SICK_PARLOC = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='SICK_LOG') THEN           
                SICK_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='SICK_PRERUN_LOG') THEN 
                SICK_PRERUN_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_PARLOC') THEN              
                SOCBEN_PARLOC = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_LOG') THEN           
                SOCBEN_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_PRERUN_LOG') THEN         
                SOCBEN_PRERUN_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_PARLOC') THEN              
                FAMREL_PARLOC = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_LOG') THEN           
                FAMREL_LOG = CONTROL_IO(2*i)
            ELSE IF (CONTROL_IO(2*i-1)=='WORKHOUR_PARLOC') THEN 
                WORKHOUR_PARLOC = CONTROL_IO(2*i)
            !Read path names of output files
            ELSE IF (CONTROL_IO(2*i-1)=='OUTPUT_PATH')THEN
                OUTPUT_PATH = CONTROL_IO(2*i)   
            !Read specific file-names of global budget files    
            ELSE IF (CONTROL_IO(2*i-1)=='GLOBBUD_EMPL_FILE')THEN
                GLOBBUD_EMPL_FILE = CONTROL_IO(2*i)   
            ELSE IF (CONTROL_IO(2*i-1)=='GLOBBUD_SELFEMPL_FILE') THEN
                GLOBBUD_SELFEMPL_FILE = CONTROL_IO(2*i)
            END IF 
        END DO
        !check for length of path and file name
        MIMOSIS_PAR_BREAK_POINT(1)=INDEX(OUTPUT_PATH,MIMOSIS_PAR_BREAK)
        MIMOSIS_PAR_BREAK_POINT(2)=INDEX(GLOBBUD_EMPL_FILE,MIMOSIS_PAR_BREAK)                          
        MIMOSIS_PAR_BREAK_POINT(3)=INDEX(GLOBBUD_SELFEMPL_FILE,MIMOSIS_PAR_BREAK)                          
        !merge infomration (path+name) to create default output file name     
        GLOBBUD_EMPL=OUTPUT_PATH(1:MIMOSIS_PAR_BREAK_POINT(1)-1)//&
        &GLOBBUD_EMPL_FILE(1:MIMOSIS_PAR_BREAK_POINT(2)-1)
        GLOBBUD_SELFEMPL=OUTPUT_PATH(1:MIMOSIS_PAR_BREAK_POINT(1)-1)//&
        &GLOBBUD_SELFEMPL_FILE(1:MIMOSIS_PAR_BREAK_POINT(3)-1)        
        CALL TRIM@(GLOBBUD_EMPL)
        CALL TRIM@(GLOBBUD_SELFEMPL)

        !write names to log file
        OPEN (UNIT=15, FILE=TESTREAD, IOSTAT=ierror)
        IF (ierror.NE.0) THEN
            WRITE(*,'("Could not find ", a100)') TESTREAD
            PAUSE
            STOP
        END IF
        WRITE (15,*) 'GLOBBUD_EMPL    = ',     GLOBBUD_EMPL
        WRITE (15,*) 'GLOBBUD_SELFEMPL= ',     GLOBBUD_SELFEMPL
        CLOSE(UNIT=15)

        !Open and clean the existing global budget-files
        !and write general headers to these files
        OPEN (UNIT=16, FILE=GLOBBUD_EMPL, IOSTAT=ierror)    
        IF (ierror.NE.0) THEN
            WRITE(*,'("Could not find ", a100)') GLOBBUD_EMPL
            PAUSE
            STOP
        END IF
        Write(16,'(''Statistics computed to reconstruct some concepts of the global social security budget of employees'')')
        Write(16,'(''                                                                                                    &
        &          AmountEuro       NumIndividual   Average      %Pop GlobClass'')')
        CLOSE(16)

        OPEN (UNIT=17, FILE=GLOBBUD_SELFEMPL, IOSTAT=ierror)        
        IF (ierror.NE.0) THEN
            WRITE(*,'("Could not find ", a100)') GLOBBUD_SELFEMPL
            PAUSE
            STOP
        END IF
        Write(17,'(''Statistics computed to reconstruct some concepts of the global social security budget of self employed'')')
        Write(17,'(''                                                                                                    &
        &          AmountEuro       NumIndividual   Average      %Pop GlobClass'')')
        CLOSE(17)
        !Write to screen
        WRITE(*,*) 'FILE MIMOSIS_PARAMETERS.txt.............................read correctly'  
        WRITE(*,*) ' ' 
    ELSE
        PAUSE
        STOP    
    END IF

    !test reading of parameter locations
    !open parameter file             
    OPEN(unit=15,FILE=TESTREAD,POSITION='APPEND',ACTION='WRITE')    
    !check for status of parameter file
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') TESTREAD
        PAUSE
        STOP
    END IF
    
    WRITE(15,'(a150)')  MIMRUN_HEADING
    !WRITE number of parameters
    WRITE(15,'(i10)')   N_REC
    DO i=1,3
        WRITE(15,'(f10.6)') INFLATE_NOMVAL(i)
    END DO
    WRITE(15,'(i10)')   SIMYEAR
    !WRITE in parameters for path and name of input files  
    DO i=1,N_LOCLINES
        !WRITE in parameters for path and name of input files  
        WRITE(15,'(a100)') CONTROL_IO(i)  
    END DO
    CLOSE(UNIT=15)

END SUBROUTINE MIMOSIS_PARAM

!Subroutine to convert control to proper string for later use
SUBROUTINE CONTROL_CONVERT(STRINGCONTROL)

    CHARACTER(len=3), Intent (Inout) :: STRINGCONTROL

    IF (STRINGCONTROL=='yes'.OR.STRINGCONTROL=='no'.OR.STRINGCONTROL=='Yes'.OR.STRINGCONTROL=='No'.OR.&
        &STRINGCONTROL=='YES'.OR.STRINGCONTROL=='NO')THEN 
        IF (STRINGCONTROL=='yes'.OR.STRINGCONTROL=='Yes'.OR.STRINGCONTROL=='YES') THEN             
            STRINGCONTROL='yes'
        ELSE
            STRINGCONTROL='no'            
        END IF
    END IF

END SUBROUTINE CONTROL_CONVERT

!Subroutine to cycle until the proper string is entered
SUBROUTINE CONTROL_CYCLE(STRINGCONTROL)

    CHARACTER(len=3), Intent (Inout) :: STRINGCONTROL
    
    DO 
        IF (STRINGCONTROL.EQ.'yes'.OR.STRINGCONTROL.EQ.'no') THEN
            EXIT
        ELSE IF (STRINGCONTROL.NE.'yes'.AND.STRINGCONTROL.NE.'no') THEN
            WRITE (*,*) 'Did you change the parameters in any of the other modules? (yes,no)'
            READ (*,'(a3)') STRINGCONTROL
            CALL CONTROL_CONVERT(STRINGCONTROL)             
            CYCLE
        END IF
    END DO
    
END SUBROUTINE CONTROL_CYCLE

!===============================================!
!Subroutine to read working hours of both spouses and location of disposable income
!===============================================!

SUBROUTINE WORKHOUR_FILEREAD

    INTEGER:: I   
    !Local parameters for locating output of the global budget files
    CHARACTER(len=150)                                      :: HEADER                               !read in header of the file (=first line)
    INTEGER                                                 :: ierror

    !open parameter file where working hours and location are stored            
    OPEN (UNIT=3, FILE=WORKHOUR_PARLOC, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    !check for status of parameter file
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') WORKHOUR_PARLOC
        PAUSE
        STOP
    END IF

    READ (3,'(a150)')   HEADER
    !read location of file where net disposable income per individual needs to be stored
    READ(3,'(a150)')    NDISPINC_PARLOC
    !read hours worked by spouse 1 and spouse 2
    READ(3,'(i10)')     WORKHOUR_SPOUSE1     
    READ(3,'(i10)')     WORKHOUR_SPOUSE2   
    
    !test reading of this parameter file
    !open parameter file             
    OPEN(unit=15,FILE=TESTREAD,POSITION='APPEND',ACTION='WRITE')    
    !check for status of parameter file
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') TESTREAD
        PAUSE
        STOP
    END IF

    WRITE(15,'(a150)')  HEADER
    WRITE(15,'(a150)')  NDISPINC_PARLOC
    !write hours worked by spouse 1 and spouse 2
    WRITE(15,'(i10)')   WORKHOUR_SPOUSE1     
    WRITE(15,'(i10)')   WORKHOUR_SPOUSE2   
    

END SUBROUTINE WORKHOUR_FILEREAD

!===============================================!
!Subroutine to run calculation of welfare adaptations
!===============================================!

SUBROUTINE PENSWELF

    USE PENSWELF_DATA
    USE PENSWELF_SUBROUTINES
    
    CALL PENSWELF_CALC

END SUBROUTINE PENSWELF

!===============================================!
!Subroutine to run pre run calculation of welfare adaptations
!===============================================!

SUBROUTINE PENSWELF_PRERUN

    USE PENSWELF_PRERUN_DATA
    USE PENSWELF_PRERUN_SUBROUTINES
    
    CALL PENSWELF_PRERUN_CALC

END SUBROUTINE PENSWELF_PRERUN

!===============================================!
!Subroutine to run calculation of family allowances
!===============================================!

SUBROUTINE FAMAL

    USE FAMAL_DATA
    USE FAMAL_SUBROUTINES
    
    CALL FAMAL_CALC

END SUBROUTINE FAMAL

!===============================================!
!Subroutine to run pre run calculation of family allowances
!===============================================!

SUBROUTINE FAMAL_PRERUN

    USE FAMAL_PRERUN_DATA
    USE FAMAL_PRERUN_SUBROUTINES
    
    CALL FAMAL_PRERUN_CALC

END SUBROUTINE FAMAL_PRERUN

!===============================================!
!Subroutine to run calculation of social security contributions
!===============================================!

SUBROUTINE CONTRIB

    USE CONTRIB_DATA
    USE CONTRIB_SUBROUTINES

    CALL CONTRIB_CALC
    
END SUBROUTINE CONTRIB

!===============================================!
!Subroutine to run pre run of calculation of social security contributions
!===============================================!

SUBROUTINE CONTRIB_PRERUN

    USE CONTRIB_PRERUN_DATA
    USE CONTRIB_PRERUN_SUBROUTINES

    CALL CONTRIB_PRERUN_CALC
    
END SUBROUTINE CONTRIB_PRERUN

!===============================================!
!Subroutine to run calculation of personal income taxes
!===============================================!

SUBROUTINE PIT

    USE PIT_DATA
    USE PIT_SUBROUTINES

    CALL PIT_CALC
    
END SUBROUTINE PIT

!===============================================!
!Subroutine to run pre run of calculation of personal income taxes
!===============================================!

SUBROUTINE PIT_PRERUN

    USE PIT_PRERUN_DATA
    USE PIT_PRERUN_SUBROUTINES

    CALL PIT_PRERUN_CALC
    
END SUBROUTINE PIT_PRERUN

!===============================================!
!Subroutine to run calculation of sickness and invalidity simulation
!===============================================!

SUBROUTINE SICK

    USE SICK_DATA
    USE SICK_SUBROUTINES

    CALL SICK_CALC
    
END SUBROUTINE SICK

!===============================================!
!Subroutine to run pre run of calculation of sickness and invalidity simulation
!===============================================!

SUBROUTINE SICK_PRERUN

    USE SICK_PRERUN_DATA
    USE SICK_PRERUN_SUBROUTINES

    CALL SICK_PRERUN_CALC
    
END SUBROUTINE SICK_PRERUN

!===============================================!
!Subroutine to run calculation of unemployment simulation
!===============================================!

SUBROUTINE UNEM

    USE UNEM_DATA
    USE UNEM_SUBROUTINES

    CALL UNEM_CALC
    
END SUBROUTINE UNEM

!===============================================!
!Subroutine to run pre run of calculation of unemployment simulation
!===============================================!

SUBROUTINE UNEM_PRERUN

    USE UNEM_PRERUN_DATA
    USE UNEM_PRERUN_SUBROUTINES

    CALL UNEM_PRERUN_CALC
    
END SUBROUTINE UNEM_PRERUN

!===============================================!
!Subroutine to run eval module
!===============================================!

SUBROUTINE EVAL

    USE EVAL_DATA
    USE EVAL_SUBROUTINES

    CALL EVAL_CALC
    
END SUBROUTINE EVAL

!===============================================!
!Subroutine to run SOCBEN module
!===============================================!

SUBROUTINE SOCBEN

    USE SOCBEN_DATA
    USE SOCBEN_SUBROUTINES

    CALL SOCBEN_CALC
    
END SUBROUTINE SOCBEN

!===============================================!
!Subroutine for pre run SOCBEN module
!===============================================!

SUBROUTINE SOCBEN_PRERUN

    USE SOCBEN_PRERUN_DATA
    USE SOCBEN_PRERUN_SUBROUTINES

    CALL SOCBEN_PRERUN_CALC
    
END SUBROUTINE SOCBEN_PRERUN

!===============================================!
!Subroutine to run famrel module
!===============================================!

SUBROUTINE FAMREL

    USE FAMREL_DATA
    USE FAMREL_SUBROUTINES

    CALL FAMREL_CALC
    
END SUBROUTINE FAMREL

END MODULE MIMOSIS_SUBROUTINES
