!===================================================================!
!  Record of revisions:                                             !
!                                                                   !
!    Date      Programmer               Description of changes      !
!  ========    ====================     ======================      !
!  01/03/06    Consortium AG/01/086     Original code               !
!  01/10/11    FPS Social Security      Revision                    !
!===================================================================!

!==========================================================!
! This module declare all variables used in the program    !
!  - Parameters                                            !
!  - Exogenous Variables                                   !
!  - Endogeneous Variables                                 !
!==========================================================!

MODULE CONTRIB_DATA

    !========================================================================!
    ! Database specific parameters:                                          !
    ! number of lines in parameters file                                     !
    INTEGER, PARAMETER::n_par=326
    ! year of database                                                       !
    INTEGER, PARAMETER::MIMOSIS_YEAR=2008
    !(first quarter=1, second quarter=4, third quarter=9, fourth quarter=12) !
    INTEGER, PARAMETER::month=12                                             !
    !Define kind of Integer*4 in a portable way                              !                       
    INTEGER, PARAMETER :: INT4 = SELECTED_INT_KIND(18)                       !                       
    !Define Double precision, i.e. REAL*8 in a portable way                  !
    INTEGER, PARAMETER :: DPRECI = SELECTED_REAL_KIND(15,307)                !
    !Define Single precision, i.e. REAL*4 in a portable way                  !
    INTEGER, PARAMETER :: SPRECI = SELECTED_REAL_KIND(6,37)                  !
    !Define parameter that stores the precision value used for large vectors !
    INTEGER, PARAMETER :: KIND_REAL = SPRECI                                 !   
    !==========================================================================

    !==========================================================================
    !Define variables to read locations and input parameters
    !Variables for locating input 
    CHARACTER(len=150)                                      :: CONTRIB_INPUT_PATH                   !Path name where file with CONTRIB input data is stored
    CHARACTER(len=150)                                      :: CONTRIB_INPUT_FILE                   !File name of file with CONTRIB input data
    CHARACTER(len=150)                                      :: CONTRIB_INPUT                        !Internal name to store Path+Name of file with CONTRIB input data
    CHARACTER(len=150)                                      :: CONTRIB_PENSWELF_FILE                !File name of file with PENSWELF input data
    CHARACTER(len=150)                                      :: CONTRIB_PENSWELF                     !Internal name to store Path+Name of file with PENSWELF input data
    CHARACTER(len=150)                                      :: CONTRIB_UNEM_FILE                    !File name of file with UNEM input data
    CHARACTER(len=150)                                      :: CONTRIB_UNEM                         !Internal name to store Path+Name of file with FAMREL input data
    CHARACTER(len=150)                                      :: CONTRIB_FAMAL_FILE                   !File name of file with FAMAL input data
    CHARACTER(len=150)                                      :: CONTRIB_FAMAL                        !Internal name to store Path+Name of file with FAMREL input data
    CHARACTER(len=150)                                      :: CONTRIB_SICK_FILE                    !File name of file with SICK input data
    CHARACTER(len=150)                                      :: CONTRIB_SICK                         !Internal name to store Path+Name of file with FAMREL input data
    CHARACTER(len=150)                                      :: CONTRIB_PIT_FILE                     !File name of file with PIT input data
    CHARACTER(len=150)                                      :: CONTRIB_PIT                          !Internal name to store Path+Name of file with FAMREL input data
    CHARACTER(len=150)                                      :: CONTRIB_CONSTRUCT_FILE               !File name of file with CONSTRUCT input data
    CHARACTER(len=150)                                      :: CONTRIB_CONSTRUCT                    !Internal name to store Path+Name of file with CONSTRUCT input data
    CHARACTER(len=150)                                      :: FAMREL_INPUT_PATH                    !Path name where file with FAMREL input data is stored
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_FILE                  !File name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL                       !Internal name to store Path+Name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: CONSTANT_INPUT_PATH                  !Path name where file with MIMOSIS Weights is stored
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG_FILE                !File name of file with MIMOSIS Weights                    
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG                     !Internal name to store Path+Name of file with MIMOSIS Weights
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT_FILE                  !File name of file with MIMOSIS gross wages
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT                       !Internal name to store Path+Name of file with MIMOSIS gross wages
    !Variables for locating output (local variables)
    CHARACTER(len=150)                                      :: PIT_CONTRIB_FILE                     !Name of file created with CONTRIB module for data used as input of the PIT module
    CHARACTER(len=150)                                      :: PIT_CONTRIB                          !Path+Name of the file created with CONTRIB module for data used as input of the PIT module
    CHARACTER(len=150)                                      :: SOCBEN_CONTRIB_FILE                  !Name of file created with CONTRIB module for data used as input of the MINIMEX module
    CHARACTER(len=150)                                      :: SOCBEN_CONTRIB                       !Path+Name of the file created with CONTRIB module for data used as input of the MINIMEX module
    CHARACTER(len=150)                                      :: EVAL_CONTRIB_FILE                    !Name of file created with CONTRIB module for data used as input of the EVAL module   
    CHARACTER(len=150)                                      :: EVAL_CONTRIB                         !Path+Name of the file created with CONTRIB module for data used as input of the EVAL module
    !Tools to read input, output and parameters    
    CHARACTER(len=20)                                       :: CONTRIB_PAR_FORMAT                   !Format of parameter read in from parameter file
    CHARACTER(len=1)                                        :: CONTRIB_PAR_BREAK='@'                !Character used to identify end of string
    CHARACTER(len=10)                                       :: CONTRIB_PAR_CURR                     !Currency of the parameter read in from the parameter file
    CHARACTER(len=40),ALLOCATABLE,DIMENSION(:)              :: CONTRIB_PAR_NAMES                    !Vector of parameter names
    INTEGER                                                 :: CONTRIB_PAR_L                        !Number of lines (if parameter is a matrix)_
    INTEGER                                                 :: CONTRIB_PAR_C                        !Number of columnss (if parameter is a matrix)
    INTEGER,DIMENSION(17)                                   :: CONTRIB_PAR_BREAK_POINT              !number of break points to be determined when combining path names and file names
    !=======================================================================!
    !Parameters of the CONTRIB module
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_PERSALB_QT                   !Percentage used to increase income of blue collar workers to determine common social security contributions
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SHBLUE_QT                    !Percentage used to determine single holiday earnings of blue collar workers
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_DHBLUE_QT                    !Percentage used to determine double holiday earnings of blue collar workers
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_DHWHITE_QT                   !Percentage used to determine double holiday earnings of white collar workers
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_DHCSFIX_QT                   !Fixed part of double holiday earnings of civil servants
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_DHCSVAR_QT                   !Variable part of double holiday earnings of civil servants
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_WEARCOM_QT                   !Percentages used to determine common wage earner contributions of employee and employer
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_WEARHOLA_QT                  !Percentage used to determine contribution for holiday earnings of blue collar workers
    REAL(KIND=KIND_REAL)                                    :: MIMOSIS_PRIINDEX_ANN                 !Price index used to deflate annual income to previous year
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_WEARHOLD_QT                  !Percentage used to determine contribution on double holiday earnings in wage earner regime
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_SD1_QT                       !Percentages used to determine structural deduction of type 1
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_SD2_QT                       !Percentages used to determine structural deduction of type 2
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_SD3_QT                       !Percentages used to determine structural deduction of type 3
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_SD4_QT                       !Percentages used to determine structural deduction of type 4
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SOCMAR_QT                    !Fixed deduction in case of social Maribel
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_YWDED_QT                     !Percentage used to determine deduction for young wage earner
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PLUSDEDU_QT                  !Percentages used to determine deductions for plus plans
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_UNPROFDP_QT                  !Percentages used to determine deduction in case of part time career interruption
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_UNPROFDF_QT                  !Percentages used to determine deduction in case of full time career interruption
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_UNPRERDP_QT                  !Percentages used to determine deduction in case of half time pre retirement with part time employment
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_UNPRERDF_QT                  !Percentages used to determine deduction in case of half time pre retirement with half time employment
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_4DAYDEDA_QT                  !Percentages used to determine deduction in case of 4-days week deduction
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_4DAYDEDB_QT                  !Maximal amount used to determine deduction in case of 4-days week deduction
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_38HDEDA_QT                   !Fixed amount applied as deduction in case of switch from 39 to 38 hours
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_JOBSDEDA_QT                  !Percentage used to determine deductions for job seekers
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PDEDBL_MONTH                 !Monthly personal deduction for blue collar workers with low wages
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PDEDWI_MONTH                 !Monthly personal deduction for white collar workers with low wages
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_CSCOMLOC_QT                  !Percentages used to determine common contributions of civil servants working for a local authority
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_CSCOMFED_QT                  !Percentages used to determine common contributions of civil servants working for the federal state
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_CSDHCON_QT                   !Percentage used to determine contributions on double holiday earnings of civil servants
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_LSALLSE_QT                   !Lump sum contribution to be paid by all self employed
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_SELFPAA_QT                   !Percentages used to determine social security contributions of self employed in principal activity
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SELFPAB_QT                   !Lump sum amounts of self employed in their first three years of activity for self employed in principal activity
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SELFPAC_QT                   !Values used to determine contributions of self employed with survival pension in principal activity
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SELFCAA_QT                   !Lump sum amounts used to determine contributions of self employed in complementary activity
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_SELFARA_QT                   !Percentages used to determine social security contributions of self employed who are retired and have a retirement pension
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_SELFARB_QT                   !Percentages used to determine social security contributions of self employed who are retired and do not have a retirement pension
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SELFARC_QT                   !Lump sum amounts used to determine contributions of self employed after retirement age
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_SELFASA_QT                   !Percentages used to determine contributions of self employed assistant
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SELFASB_QT                   !Lump sum amounts used to determine contributions of self employed assistant
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SELFDEDA_QT                  !Percentage used to determine deductions of self employed contributions
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SELFDEDB_QT                  !Lump sum amount used to limit the deduction of self employed contributions
    INTEGER                                                 :: MIMOSIS_REFYEAR_ANN                  !Year for which the data are registered
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_PELIMBA_MONTH                !Lump sum amounts used to determine the threshold of pension on which contributions have to be paid
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_PENSPERC_MONTH               !Percentage used to determine social security contributions on pension benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PELIMNOC_MONTH               !Percentages used to determine solidarity contributions on pensions for pensioner without dependent children
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PELIMCHI_MONTH               !Percentages used to determine solidarity contributions on pensions for pensioner with dependent children
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_EARPERC_QT                   !Percentages used to determine contributions on early retirement benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_EALIMFT_MONTH                !Lump sum amounts used to determine the threshold of early retirement benefits on which contributions have to be paid (benefits of full time early retired)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_EALIMPT_MONTH                !Lump sum amounts used to determine the threshold of early retirement benefits on which contributions have to be paid (benefits of part time early retired)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_DISABIM_DAY                  !Lump sum amounts used to determine the threshold of disability benefits on which contributions have to be paid
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_DISAPERC_DAY                 !Percentage used to determine social security contributions on disability benefits paid by wage earners
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_INDOCCPC_QT                  !Percentage used to determine social security contributions on benefits in case of industrial accidents of occupational diseases
    INTEGER                                                 :: CONTRIB_HOURSIM                      !Indicator saying that private labour market income must be computed as the product of hours worked per week and a wage rate
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_WAGEINC                      !Parameter used to increase the hourly wage earned           
    INTEGER                                                 :: CONTRIB_SEXINCR                      !Parameter used to identify whether the increase should be applied on male income (=1) or on female income (=2)
    INTEGER                                                 :: CONTRIB_DUURP                        !Maximum number of months in unemployment for which the unemployed can receive a benefit    
    INTEGER                                                 :: CONTRIB_SIMUNEM                      !Identifier to start the routine to compute the labour income of unemployed who drop out of the system
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_HOURUNEM                     !Number of hours per week the drop-out unemployed are assumed to work
    INTEGER                                                 :: CONTRIB_SIMNRMT                      !Number used to identify the for which individual the revenues have to be simulated at different hours of work (marginal tax computation)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_STRU2008_QT                  !Parameters required to compute the structural deduction with elements introduced after the 2005 reform
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_CEPWEARN_YEAR                !vector with ceilings applied on labour income to test for possible reduction of pensions (legislation on allowed labour income in combination with pensions)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_RAPWEARN_YEAR                !percentage used to deteremine the amount of income with which one can exceed the labour income ceiling in case of pensions (legislation on allowed labour income in combination with pensions)
    !=======================================================================!  
    !Exogenous variables
    !CONTRIB_INPUT_Write.sas
    !/*Variabelen uit PopChar-bestand*/
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: GECODEERD_INSZ_CHAR                  ! Geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_INPUT                 ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_HH_ID                       ! Geanonimiseerd identificatienummer van het gezin waartoe het individu behoort
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NAREGNIS_DATBIRTH                    ! Geboortedatum van het individu in jaar en maand
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_SEX                         ! Geslacht van het individu op 1 januari 2009
    !/*HistorInfo*/
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: LABMSTAT_PRIV                        ! Meest voorkomende arbeidsmarktstatus op de private arbeidsmarkt in de periode 1997 tot en met eindjaar 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: LABMSTAT_PUB                         ! Meest voorkomende arbeidsmarktstatus op de publieke arbeidsmarkt in de periode 1997 tot en met eindjaar
    !/*Variabelen uit RSZ-bestand*/
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: tauxpt_20084                         ! percentage parttime
    CHARACTER(len=1),ALLOCATABLE,DIMENSION(:)               :: t_prest_20084                        ! type prestatie
    CHARACTER(len=1),ALLOCATABLE,DIMENSION(:)               :: secemp_20084                         ! sector (1-prive 2-openbare)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: codtra_20084                         ! werknemerscode
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: clatra_20084                         ! werknemersklasse
    CHARACTER(len=2),ALLOCATABLE,DIMENSION(:)               :: clatr2_20084                         ! speciale klasse werknemer
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: codasm_20084                         ! hoofdcode van de gelijkgestelde dagen
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CATTRA_20084                         ! Werkgeverskengetal
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GROEP_VG                             ! Groep bepaling vakantiegeld
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: sal100_agg                           ! loon 100% (aggregaat op kwartaalbasis)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: primes_agg                           ! premies (aggregaat op kwartaalbasis)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: preavi_agg                           ! vooropzeg (aggregaat op kwartaalbasis)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: salatt_agg                           ! wachtloon (aggregaat op kwartaalbasis)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: salfor_agg                           ! forfaitair loon (aggregaat op kwartaalbasis)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: DeductionCode_Pers_1                 ! Verminderingscode persoonlijke bijdragevermindering
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: DeductionCode_Pat_1                  ! Verminderingscode patronale bijdragevermindering: code 1
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: DeductionCode_Pat_2                  ! Verminderingscode patronale bijdragevermindering: code 2
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: DeductionCode_Pat_3                  ! Verminderingscode patronale bijdragevermindering: code 3
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SumSal_2008                          ! Som gewone bezoldigingen, loon bestuurders wachttijden, premies en verbrekingsvergoedingen (som voor jaar = 2008) 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: RSZ_LABMSTAT_PRIV_2008               ! Labour market status of a wage earner on the private labour market in 2008                                        
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: RSZ_LABMSTAT_PUB_2008                ! Labour market status of a wage earner on the public labour market in 2008                                         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InRSZ                                ! Individu geselecteerd uit basisdata RSZ voor verwerking steekproef                                                
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RATE_FICTIF_RSZ                      ! Fractie die gebruikt zal worden om het fictief inkomen te bepalen bij de eventuele berekening van het vakantiegeld
    !/*Variabelen uit RSZPPO-bestand*/
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: apltauxpt_20084                      ! Percentage deeltijds
    CHARACTER(len=1),ALLOCATABLE,DIMENSION(:)               :: APL_t_prest_20084                    ! Code (voltijds, deeltijds, speciaal) die informatie verstrekt over het arbeidsregime op de laatste dag van het kwartaal
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: APL_cattra_20084                     ! Werkgeverscategorie (code voor vakantieregime niet-vastbenoemden, code voor pensioenstelsel vastbenoemden).
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: APL_codtra_20084                     ! Met het oog op een correcte inning van de socialezekerheidsbijdragen maakt deze variabele een onderscheid tussen handarbeiders, hoofdarbeiders, gesubsidieerde contractuelen, vastbenoemden, 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: clatraApl_20084                      ! Groepering van de werknemerscodes in een aantal werknemersklassen.
    CHARACTER(len=2),ALLOCATABLE,DIMENSION(:)               :: clatr2Apl_20084                      ! Gedetailleerde groepering van de werknemerscodes.
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: APL_GROEP_EV                         ! Groep bepaling enkel vakantiegeld
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: APL_GROEP_DV                         ! Groep bepaling dubbel vakantiegeld
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: APL_sal100_agg                       ! Bedrag van de gewone bezoldiging in het kwartaal. (aggregaat op kwartaalbasis)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: APL_salcot_agg                       ! Bedrag van de bijdrageplichtige loonmassa inzake sociale zekerheid in het kwartaal (aggregaat op kwartaalbasis)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: APL_primes_agg                       ! Het bedrag van de bezoldigingen in het kwartaal die niet rechtstreeks verbonden zijn met de prestaties gedurende een bepaald kwartaal. (aggregaat op kwartaalbasis)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: APL_DeductionCode_Pers_1             ! Verminderingscode persoonlijke bijdragevermindering
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: APL_SumSal_2008                      ! Som gewone bezoldigingen, premies en verbrekingsvergoedingen (som voor jaar = 2008)                               
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: RSZPPO_LABMSTAT_PUB_2008             ! Labour market status of a wage earner on the public labour market in 2008                                         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InRSZPPO                             ! Individu geselecteerd uit basisdata RSZPPO voor verwerking steekproef                                             
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RATE_FICTIF_RSZPPO                   ! Fractie die gebruikt zal worden om het fictief inkomen te bepalen bij de eventuele berekening van het vakantiegeld
    !/*Variabelen uit RSVZ-bestand*/
    CHARACTER(len=1),ALLOCATABLE,DIMENSION(:)               :: HOEDRSVZ_2008                        ! Code van hoedanigheid bij RSVZ (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: BEGAANSC_2008                        ! Aanvangsdatum van de aansluiting bij de RSVZ (2008)
    CHARACTER(len=1),ALLOCATABLE,DIMENSION(:)               :: BIJDCAT_2008                         ! Code van de bijdragecategorie bij RSVZ (in hoofdberoep, bijberoep, na pensioenleeftijd, ...) (2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: inkomens_2008                        ! Inkomen op jaarbasis na aanvulling van ontbrekende waarden en uitgedrukt in prijzen van 2008
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SelfStat_2008                        ! Aanduiding of het individu voorkomt in het bestand met inkomens en/of aansluitingsgegevens RSVZ
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NoIncomeLine_RSVZ                    ! Variabele die aangeeft of voor een gegeven lijn geen inkomen gereconstrueerd moest worden
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: Bron_Aanv                            ! Bron op basis waarvan de ontbrekende inkomensgegevens werden aangevuld
    !CONTRIB_CONSTRUCT_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_CONSTRUCT             ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: MIMOSIS_FLEXWEAR                     ! Identification whether the individual is available for the labour market as wage earner
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_HOUR                         ! Hours worked as wage earner per week (estimate based on income earned as wage earner divided by wage)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: MIMOSIS_NUMHHMEM                     ! Number of household members for which marginal taxes have to be simulated
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: MIMOSIS_SIMNUMMT                     ! Unique identification number identifying the position of the individual in the household for the computations of the marginal taxes
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: MIMOSIS_SIMSAMMT                     ! Identification whether the individual should be simulated for the computation of the marginal taxes
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_HOURPRED                     ! Hours worked as wage earner per week (estimate based on income earned as wage earner divided by wage predicted with wage equation)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_WAGEPRED                     ! Wages predicted with wage equation (estimated with data based on real hours worked)
    !Variables read from file CONTRIB_PENSWELF.txt                                                       
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_PENSWELF              !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_RETPENS_QT                  !Retirement pension received in a given quarter
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_SURPENS_QT                  !Survival pension received in a given quarter
    !Variables read from file CONTRIB_UNEM.txt                                                       
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_UNEM                  !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENUNS1_QT                      !Early retirement benefit paid by the RVA to unemployed not in search of work received in a given quarter
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_ADDUNS_QT                       !Early retirement benefit paid by the employer to unemployed not in search of work received in a given quarter
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMP1_QT                      !Early retirement benefit paid by the RVA to a part time employee received in a given quarter
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_ADDEMP_QT                       !Early retirement benefit paid by the employer to a part time employee received in a given quarter
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_OTHERUN_QT                      !Unemployment benefits, other than early retirement benefits and benefits for older unemployed with seniority supplement received in a given quarter
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_OLDUNSS_QT                      !Unemployment benefits for older unemployed with seniority supplement paid in a given quarter
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_STATUS_QT                       !Indicator whether an individual is entitled to any sort of benefit paid by the RVA          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_NUMMINU_QT                      !Number of months the unemployed is already in unemployment          
    !Variables read from file CONTRIB_FAMAL.txt                                                      
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMAL                 !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_AMOUNTF_QT                     !Total amount of family allowances received in a given quarter by the recipient
    !Variables read from file CONTRIB_SICK.txt                                                       
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_SICK                  !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_IND_QT                          !Disability benefits received by wage earners because of industrial accidents (benefits for help of a third person excluded)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_OCC_QT                          !Disability benefits received by wage earners because of occupational diseases (benefits for help of a third person excluded)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_DISABWEA_QT                     !Disability benefits received by wage earners on the private or public labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_ALOTHBEN_QT                     !Benefits in case of sickness or disability other than those covered by SICK_IND_QT, SICK_OCC_QT and SICK_DISABWEA_QT
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPLBPRI_QT                     !Benefits in case of sickness or disability paid by the employer in the private sector
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPLBPUB_QT                     !Benefits in case of sickness or disability paid by the employer in the public sector
    !Variables read from file CONTRIB_PIT.txt                                                        
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_PIT                   !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_DEPCHILD_ANN                     !Number of dependent children in the tax unit of the individual in a given year
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMREL                !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMILY                        !Unique identifier identifying the family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_RELATION                      !Relation of the individual with respect to the head of the family
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMTYPE                       !Type of family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_COUPLE                        !Type of couple to which the individual belongs
    !WAGE_CONSTRUCT_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WAGE                  ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SALJRS_LAST_CORR                     ! Dagloon in laatste jaar met positieve waarde gecorrigeerd dagloon (in prijzen van 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_HOUR                   ! Uurloon gecorrigeerd voor minimum onder minimumgrens
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_QT                     ! Raming van het kwartaalinkomen in het laatst geobserveerde kwartaal
    !MIMOSIS_SAMPWEIG_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WEIGHT                ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_WEIGHT                       ! Gewicht dat corrigeert voor niet random selectie van de observatie-eenheden
    !=======================================================================!
    !Endogeneous variables
    !to rank variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NEW_RANK                             !New rank after sorting and re-ranking       
    !to construct age variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_YEAR_AGE                     !Age of the individual expressed in years
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_MONTH_AGE                    !Age of the individual expressed in moths
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_MONTH_BIRTH                  !Month of birth(mm)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_YEAR_BIRTH                   !Year of birth (yyyy)
    !to construct information at the level of the sociological household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_HH_SIZE                      !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_START_HH                     !Variable stores an identification of the first household member (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_END_HH                       !Variable stores an identification of the last household member (maximal size vector = number of individuals)   
    INTEGER                                                 :: N_HH_CONTRIB                         !number of households analysed
    !To construct variables listed in the module note
    !Income concepts             
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_PRIV_QT             !Labour market status of a wage earner on the private labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_INCPRIV_QT                   !Labour income, earned as wage earner on the private labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SHPRIV_QT                    !Single holiday earnings earned on the private labour market         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_DHPRIV_QT                    !Single holiday earnings earned on the private labour market         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_INCSPRIV_QT                  !Supplement to real earned labour income, other than holiday earnings, obtained as wage earner on the private labour market
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_PUB_QT              !Labour market status of a wage earner on the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_INCPUB_QT                    !Labour income, earned as wage earner on the public labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SHPUB_QT                     !Single holiday earnings earned on the public labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_DHPUB_QT                     !Single holiday earnings earned on the public labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_INCSPUB_QT                   !Supplement to real earned labour income, other than holiday earnings, obtained as wage earner on the public labour market
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_SELF_QT             !Labour market status of a self employed         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_INCSELF_QT                   !Labour income, earned as self employed and subjected to contribution laws        
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SELFEXTR_QT                  !Labour income, earned as self employed and not subjected to contribution laws (=extra amount imputed on the basis of external sources)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_PENS_QT                      !Pension benefits on which social security contributions have to be paid         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_UNEMFT_QT                    !Early retirement benefits of a full time unemployed on which social security contributions have to be paid          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_UNEMPT_QT                    !Early retirement benefits of a part time unemployed on which social security contributions have to be paid          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_DISABWEA_QT                  !Disability benefits received by wage earners on the private or public labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_INDOCC_QT                    !Disability benefits received by wage earners because of industrial accidents or occupational diseases (benefits for help of a third person excluded)
    !Contributions of wage earners on the private labour market              
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_SIZECOMP_QT                  !Company size of the company the employee is working         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REMBPRIV_QT                  !Basic remuneration concept used to determine social security contributions of wage earners on the private labour market         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_WEARCOM_EMEE_QT              !Common social security contributions paid by the employee working as wage earner on the private labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_WEARCOM_EMER_QT              !Common social security contributions paid by the employer for an employee working as wage earner on the private labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_WEARHOLA_EMER_QT             !Social security contribution paid for holiday earnings of blue collar workers on the private labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_WEARHOLD_EMEE_QT             !Social security contributions paid on double holiday earnings of wage earners on the private labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_WEARDEDU_EMER_QT             !Deductions of employer social security contributions permitted for wage earners on the private labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_WEARDEDU_EMEE_QT             !Deductions of employee social security contributions permitted for wage earners on the private labour market            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_STRUCDED_QT                  !Identifier whether the employer can claim a structural deduction            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_TAUXPT_QT                    !Fraction of time performed as a part time worker, compared to a full time worker            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_STRUCCAT_QT                  !Type of structural deduction that employer is allowed to apply for the employee         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_YOUNGW_QT                    !Identification whether an individual is a young worker after his first job agreement yes or no          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_PLUSPLAN_QT                  !Identification that the employee is a plus-plan         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_WORKTIME_QT                  !Number of quarters the employee is employed         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_YOUNGJTR_QT                  !The individual is young worker in a job training program yes or no          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_DOMESDED_QT                  !Individual is a domestic worker yes or no           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_UNPROFC_QT                   !The employee was unemployed and is employed part time or full time          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_UNPRERET_QT                  !The employee was unemployed and is employed part time or full time and replaces a pre retired person            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_SCREADED_QT                  !The employer is allowed to reduce social security contributions because of hiring additional scientific workers         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_PUBSEDED_QT                  !The employee is civil servant that replaces other civil servants who reduce their work time         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_UNACTDED_QT                  !The employee was an unemployed that generates a deduction of employer social security contributions         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_4DAYDED_QT                   !The employee enters a 4 day week regime, which entitles the employer a deduction because of job creation            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_38HDED_QT                    !The employee generates a deduction of social security contributions because he switches from a 39 hours week to a 38 hours week         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_UNSMEDED_QT                  !The employee was a long term unemployed that was difficult to place         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_DRTUGDED_QT                  !Employee generates deduction for activity in the dragging and tugging sector            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_SUBCDED_QT                   !Employee is subsidised contract worker          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_JOBSDED_QT                   !Employee is employed as job seeker yes or no            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_JOBSMONT_QT                  !Number of months the job seeker was in a given status before he became employed         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_SOCMARDE_QT                  !Employee generates the right on the social Maribel yes or no            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPRIV_EMEE_QT               !Employee social security contributions paid for employee in the private labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPRIV_EMER_QT               !Employer social security contributions paid for employee in the private labour market           
    !Contributions of wage earners on the public labour market               
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REMBPUB_QT                   !Basic remuneration concept used to determine social security contributions of people on the public labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PUBNSCOM_EMEE_QT             !Common social security contributions paid by the employee working as wage earner on the public labour market but with non statutory service         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PUBNSCOM_EMER_QT             !Common social security contributions paid by the employer for an employee working as wage earner on the public labour market but with non statutory service            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_PUBNSHOL_EMER_QT             !Social security contribution paid for holiday earnings of blue collar workers on the public labour market but with non statutory service           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_PUBNSHOL_EMEE_QT             !Social security contributions paid on double holiday earnings of wage earners on the public labour market but with non statutory service           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PUBNSDED_EMER_QT             !Deductions of employer social security contributions permitted for wage earners on the public labour market but with non statutory service             
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_PUBNSDED_EMEE_QT             !Deductions of employee social security contributions permitted for wage earners on the public labour market but with non statutory service             
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_CSLOCFED_QT                  !Indicator whether civil servant works for a local authority or for the federal government           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PUBSCOM_EMEE_QT              !Common social security contributions paid by the employee working as wage earner on the public labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PUBSCOM_EMER_QT              !Common social security contributions paid by the employer for an employee working as wage earner on the public labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_PUBSHOL_EMEE_QT              !Social security contributions paid on double holiday earnings of wage earners on the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: CONTRIB_PUBSDED_EMER_QT              !Deductions of employer social security contributions permitted for wage earners on the public labour market         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_PUBSDED_EMEE_QT              !Social security contributions paid on double holiday earnings of wage earners on the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPUB_EMEE_QT                !Employee social security contributions paid for employee in the public labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPUB_EMER_QT                !Employer social security contributions paid for employee in the public labour market            
    !Contributions of self employed              
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_SELFYEAR_QT                  !Number of years the self employed is self employed          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SELFCOM_EMER_QT              !Common social security contributions paid by the self employed          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_SELFPCR_QT                   !Status of the self employed         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_SELFFIRS_QT                  !Self employed establishes his activity as self employed for the first time          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SELFDEDU_EMER_QT             !Deduction of social security contributions of self employed         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSSELF_EMER_QT               !Social security contributions net of deductions paid by self employed           
    !Contributions on non labour market income               
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_FAMCHARG_QT                  !A pensioner is identified to have family charge yes or no           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPEBASI_QT                  !The basic social security contributions paid on pension benefits            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPESOLI_QT                  !Solidarity contributions paid on pension benefits           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_EARSTART_QT                  !Identification whether early retirement  is of the old or the new type          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSEARBEN_QT                  !Social security contributions paid on early retirement benefits         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSDISAB_QT                   !Social security contributions paid on disability benefits by wage earners
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSINDOCC_QT                  !Social security contributions paid on benefits for industrial accidents or occupational diseases
    !Gross taxable income concepts               
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPRIV_QT                   !Gross taxable labour income earned as wage earner on the private labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPUB_QT                    !Gross taxable labour income earned as wage earner on the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISELF_QT                   !Gross taxable labour income earned as self employed         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPENS_QT                   !Gross taxable pension benefits          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEARNE_QT                  !Gross taxable early retirement benefits of the new type         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEAROL_QT                  !Gross taxable early retirement benefits of the old type         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOLDUN_QT                  !Gross taxable unemployment benefits of older unemployed with seniority supplement          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOTHUN_QT                  !Gross taxable unemployment benefits, other than early retirement benefits and unemployment benefits of older unemployed with seniority supplement
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIDISAB_QT                  !Gross taxable disability benefits of wage earners
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTINDOCC_QT                  !Gross taxable benefits in case of industrial accidents or occupational diseases (benefits for help of a third person excluded)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISICK_QT                   !Gross taxable benefits in case of sickness and disability not covered by CONTRIB_GTIDISAB_QT and CONTRIB_GTINDOCC_QT          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIFAMAL_QT                  !Gross taxable income from family allowances         
    !Other exchange concepts             
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REVPRIV_QT                   !Sum of gross labour income, holiday earnings and other supplements, earned as wage earner on the private labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REVPUB_QT                    !Sum of gross labour income, holiday earnings and other supplements, earned as wage earner on the public labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GREMPLOY_QT                  !Gross income from employment        
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSEMPLOY_QT                  !Social security contributions paid on income from employment        
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSREPLAC_QT                  !Social security contributions paid on all sorts of replacement income

    !Instruments
    INTEGER                                                 :: pers_j,hh_i,i,j,k
    INTEGER                                                 :: ierror
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_WORKHOUR                     !Vector with hours that the individual is assumed to work per week
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_FULLPART_QT                  !Vector that contains time worked expressed as a fraction of a full time worker
    
END MODULE CONTRIB_DATA

!==========================================!
! This module contains all the subroutines !
! and functions used in the CONTRIB MODULE !
!==========================================!
!List of subroutine names
!SUBROUTINE CONTRIB_ALLOC_EXOG(DIM_ALLOC)
!SUBROUTINE CONTRIB_ALLOC_ENDO(DIM_ALLOC)
!SUBROUTINE CONTRIB_DEALLOC
!SUBROUTINE CONTRIB_INITENDO(DIM_ALLOC)
!SUBROUTINE CONTRIB_RANKANDSORT
!SUBROUTINE CONTRIB_PARAMETER_FILEREAD
!SUBROUTINE CONTRIB_INPUT_FILEREAD
!SUBROUTINE CONTRIB_PENSWELF_FILEREAD
!SUBROUTINE CONTRIB_UNEM_FILEREAD
!SUBROUTINE CONTRIB_FAMAL_FILEREAD
!SUBROUTINE CONTRIB_SICK_FILEREAD
!SUBROUTINE CONTRIB_PIT_FILEREAD
!SUBROUTINE CONTRIB_FAMREL_FILEREAD
!SUBROUTINE CONTRIB_WAGE_FILEREAD
!SUBROUTINE CONTRIB_WEIGHT_FILEREAD
!SUBROUTINE CONTRIB_CONSISTENCY_CHECK
!SUBROUTINE CONTRIB_HOUSEHOLD_SIZE
!SUBROUTINE CONTRIB_AGE
!SUBROUTINE CONTRIB_LABMSTAT
!SUBROUTINE CONTRIB_LABMINC
!SUBROUTINE CONTRIB_NONLAINC
!SUBROUTINE CONTRIB_CHARAC
!SUBROUTINE CONTRIB_SSBEFDED_PRIVATE(pers_j)
!SUBROUTINE CONTRIB_SSDEDU_PRIVATE(pers_j)
!SUBROUTINE CONTRIB_SSBEFDED_PUBNSTAT(pers_j) 
!SUBROUTINE CONTRIB_SSDEDU_PUBNSTAT(pers_j)
!SUBROUTINE CONTRIB_SSBEFDED_PUBSTAT(pers_j)
!SUBROUTINE CONTRIB_SSDEDU_PUBSTAT(pers_j)
!SUBROUTINE CONTRIB_SSBEFDED_SELF(pers_j,hh_i)
!SUBROUTINE CONTRIB_SSDEDU_SELF(pers_j)
!SUBROUTINE CONTRIB_SSBEFDED_NONLAB(pers_j)
!SUBROUTINE CONTRIB_EXCHANGE
!SUBROUTINE CONTRIB_OUTPUT_FILEWRIT
!SUBROUTINE CONTRIB_TEST
!SUBROUTINE CONTRIB_RECODE
!List of functions
!FUNCTION CONTRIB_LSAMOUNT_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV)                            RESULT (OUTPUT_AMOUNT)
!FUNCTION CONTRIB_BRACRATE_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV)                            RESULT (OUTPUT_AMOUNT)
!FUNCTION CONTRIB_BRACRATE_4COLUMNS_MIN(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV)                        RESULT (OUTPUT_AMOUNT)
!FUNCTION CONTRIB_BRACRATE_5COLUMNS_PLUS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV)                       RESULT (OUTPUT_AMOUNT)
!FUNCTION CONTRIB_BRACRATE_5COLUMNS_MIN(INPUT_AMOUNTA,INPUT_AMOUNTB,NLINES_BRACRATE,MAT_BRACRATE,LINENUM,INDIV) RESULT (OUTPUT_AMOUNT)
!FUNCTION CONTRIB_SELPERC_4COLUMNS_2STATINM(STATUS1,STATUS2,NLINES_PERCENT,MAT_PERCENT,INDIV)                   RESULT (OUTPUT_PERCENT)
!FUNCTION CONTRIB_SELPERC_4COLUMNS_1STATINM(STATUS1,STATUS2,NLINES_PERCENT,MAT_PERCENT,INDIV)                   RESULT (OUTPUT_PERCENT)
!FUNCTION CONTRIB_SELPERC_3COLUMNS_1STATINM(STATUS1,NLINES_PERCENT,MAT_PERCENT,INDIV)                           RESULT (OUTPUT_PERCENT)
!FUNCTION CONTRIB_SELPERC_5COLUMNS_3STATINM(STATUS1,STATUS2,STATUS3,NLINES_PERCENT,MAT_PERCENT,INDIV)           RESULT (OUTPUT_PERCENT)

MODULE CONTRIB_SUBROUTINES

    USE CONTRIB_DATA
    USE MIMOSIS_DATA    
    !use routine for sorting and ranking data
    USE RANKING_SORTING_ROUTINES

    IMPLICIT NONE
    CONTAINS

!=====================================================!
!Allocate all exogenous vectors of dimension DIM_ALLOC
!=====================================================!

SUBROUTINE CONTRIB_ALLOC_EXOG(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !=======================================================================!  
    !Exogenous variables
    !CONTRIB_INPUT_Write.sas
    !/*Variabelen uit PopChar-bestand*/
    ALLOCATE(GECODEERD_INSZ_CHAR(DIM_ALLOC))
    ALLOCATE(GECODEERD_INSZ_INPUT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_HH_ID(DIM_ALLOC))
    ALLOCATE(NAREGNIS_DATBIRTH(DIM_ALLOC))
    ALLOCATE(NAREGNIS_SEX(DIM_ALLOC))
    !/*HistorInfo*/
    ALLOCATE(LABMSTAT_PRIV(DIM_ALLOC))
    ALLOCATE(LABMSTAT_PUB(DIM_ALLOC))
    !/*Variabelen uit RSZ-bestand*/
    ALLOCATE(tauxpt_20084(DIM_ALLOC))
    ALLOCATE(t_prest_20084(DIM_ALLOC))
    ALLOCATE(secemp_20084(DIM_ALLOC))
    ALLOCATE(codtra_20084(DIM_ALLOC))
    ALLOCATE(clatra_20084(DIM_ALLOC))
    ALLOCATE(clatr2_20084(DIM_ALLOC))
    ALLOCATE(codasm_20084(DIM_ALLOC))
    ALLOCATE(CATTRA_20084(DIM_ALLOC))
    ALLOCATE(GROEP_VG(DIM_ALLOC))
    ALLOCATE(sal100_agg(DIM_ALLOC))
    ALLOCATE(primes_agg(DIM_ALLOC))
    ALLOCATE(preavi_agg(DIM_ALLOC))
    ALLOCATE(salatt_agg(DIM_ALLOC))
    ALLOCATE(salfor_agg(DIM_ALLOC))
    ALLOCATE(DeductionCode_Pers_1(DIM_ALLOC))
    ALLOCATE(DeductionCode_Pat_1(DIM_ALLOC))
    ALLOCATE(DeductionCode_Pat_2(DIM_ALLOC))
    ALLOCATE(DeductionCode_Pat_3(DIM_ALLOC))
    ALLOCATE(SumSal_2008(DIM_ALLOC))
    ALLOCATE(RSZ_LABMSTAT_PRIV_2008(DIM_ALLOC))
    ALLOCATE(RSZ_LABMSTAT_PUB_2008(DIM_ALLOC))
    ALLOCATE(InRSZ(DIM_ALLOC))
    ALLOCATE(RATE_FICTIF_RSZ(DIM_ALLOC))
    !/*Variabelen uit RSZPPO-bestand*/
    ALLOCATE(apltauxpt_20084(DIM_ALLOC))
    ALLOCATE(APL_t_prest_20084(DIM_ALLOC))
    ALLOCATE(APL_cattra_20084(DIM_ALLOC))
    ALLOCATE(APL_codtra_20084(DIM_ALLOC))
    ALLOCATE(clatraApl_20084(DIM_ALLOC))
    ALLOCATE(clatr2Apl_20084(DIM_ALLOC))
    ALLOCATE(APL_GROEP_EV(DIM_ALLOC))
    ALLOCATE(APL_GROEP_DV(DIM_ALLOC))
    ALLOCATE(APL_sal100_agg(DIM_ALLOC))
    ALLOCATE(APL_salcot_agg(DIM_ALLOC))
    ALLOCATE(APL_primes_agg(DIM_ALLOC))
    ALLOCATE(APL_DeductionCode_Pers_1(DIM_ALLOC))
    ALLOCATE(APL_SumSal_2008(DIM_ALLOC))
    ALLOCATE(RSZPPO_LABMSTAT_PUB_2008(DIM_ALLOC))
    ALLOCATE(InRSZPPO(DIM_ALLOC))
    ALLOCATE(RATE_FICTIF_RSZPPO(DIM_ALLOC))
    !/*Variabelen uit RSVZ-bestand*/
    ALLOCATE(HOEDRSVZ_2008(DIM_ALLOC))
    ALLOCATE(BEGAANSC_2008(DIM_ALLOC))
    ALLOCATE(BIJDCAT_2008(DIM_ALLOC))
    ALLOCATE(inkomens_2008(DIM_ALLOC))
    ALLOCATE(SelfStat_2008(DIM_ALLOC))
    ALLOCATE(NoIncomeLine_RSVZ(DIM_ALLOC))
    ALLOCATE(Bron_Aanv(DIM_ALLOC))
    !CONTRIB_CONSTRUCT_Write.sas
    ALLOCATE(GECODEERD_INSZ_CONSTRUCT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_FLEXWEAR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_HOUR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_NUMHHMEM(DIM_ALLOC))
    ALLOCATE(MIMOSIS_SIMNUMMT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_SIMSAMMT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_HOURPRED(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WAGEPRED(DIM_ALLOC))
    !Variables read from file CONTRIB_PENSWELF.txt                                             
    ALLOCATE(GECODEERD_INSZ_PENSWELF(DIM_ALLOC))
    ALLOCATE(PENSWELF_RETPENS_QT(DIM_ALLOC))
    ALLOCATE(PENSWELF_SURPENS_QT(DIM_ALLOC))
    !Variables read from file CONTRIB_UNEM.txt                                             
    ALLOCATE(GECODEERD_INSZ_UNEM(DIM_ALLOC))
    ALLOCATE(UNEM_BENUNS1_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ADDUNS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMP1_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ADDEMP_QT(DIM_ALLOC))
    ALLOCATE(UNEM_OTHERUN_QT(DIM_ALLOC))
    ALLOCATE(UNEM_OLDUNSS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_STATUS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_NUMMINU_QT(DIM_ALLOC))
    !Variables read from file CONTRIB_FAMAL.txt                                            
    ALLOCATE(GECODEERD_INSZ_FAMAL(DIM_ALLOC))
    ALLOCATE(FAMAL_AMOUNTF_QT(DIM_ALLOC))
    !Variables read from file CONTRIB_SICK.txt                                             
    ALLOCATE(GECODEERD_INSZ_SICK(DIM_ALLOC))
    ALLOCATE(SICK_IND_QT(DIM_ALLOC))
    ALLOCATE(SICK_OCC_QT(DIM_ALLOC))
    ALLOCATE(SICK_ALOTHBEN_QT(DIM_ALLOC))
    ALLOCATE(SICK_DISABWEA_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPLBPRI_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPLBPUB_QT(DIM_ALLOC))
    !Variables read from file CONTRIB_PIT.txt                                              
    ALLOCATE(GECODEERD_INSZ_PIT(DIM_ALLOC))
    ALLOCATE(PIT_DEPCHILD_ANN(DIM_ALLOC))
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    ALLOCATE(GECODEERD_INSZ_FAMREL(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMILY(DIM_ALLOC))
    ALLOCATE(FAMREL_RELATION(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMTYPE(DIM_ALLOC))
    ALLOCATE(FAMREL_COUPLE(DIM_ALLOC))
    !WAGE_CONSTRUCT_Write.sas
    ALLOCATE(GECODEERD_INSZ_WAGE(DIM_ALLOC))
    ALLOCATE(SALJRS_LAST_CORR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_HOUR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_QT(DIM_ALLOC))
    !MIMOSIS_SAMPWEIG_Write.sas
    ALLOCATE(GECODEERD_INSZ_WEIGHT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WEIGHT(DIM_ALLOC))

END SUBROUTINE CONTRIB_ALLOC_EXOG

!=====================================================!
!Allocate all endogenous vectors of dimension DIM_ALLOC
!=====================================================!

SUBROUTINE CONTRIB_ALLOC_ENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !Endogeneous variables
    !to rank variables
    ALLOCATE(NEW_RANK(DIM_ALLOC))
    !to construct age variables
    ALLOCATE(CONTRIB_YEAR_AGE(DIM_ALLOC))
    ALLOCATE(CONTRIB_MONTH_AGE(DIM_ALLOC))
    ALLOCATE(CONTRIB_MONTH_BIRTH(DIM_ALLOC))
    ALLOCATE(CONTRIB_YEAR_BIRTH(DIM_ALLOC))
    !to construct information at the level of the sociological household
    ALLOCATE(CONTRIB_HH_SIZE(DIM_ALLOC))
    ALLOCATE(CONTRIB_START_HH(DIM_ALLOC))
    ALLOCATE(CONTRIB_END_HH(DIM_ALLOC))
    !To construct variables listed in the module note
    !Income concepts             
    ALLOCATE(CONTRIB_LABMSTAT_PRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_INCPRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SHPRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_DHPRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_INCSPRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_LABMSTAT_PUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_INCPUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SHPUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_DHPUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_INCSPUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_LABMSTAT_SELF_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_INCSELF_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SELFEXTR_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_PENS_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_UNEMFT_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_UNEMPT_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_DISABWEA_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_INDOCC_QT(DIM_ALLOC))
    !Contributions of wage earners on the private labour market              
    ALLOCATE(CONTRIB_SIZECOMP_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_REMBPRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_WEARCOM_EMEE_QT(DIM_ALLOC,21))
    ALLOCATE(CONTRIB_WEARCOM_EMER_QT(DIM_ALLOC,21))
    ALLOCATE(CONTRIB_WEARHOLA_EMER_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_WEARHOLD_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_WEARDEDU_EMER_QT(DIM_ALLOC,21))
    ALLOCATE(CONTRIB_WEARDEDU_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_STRUCDED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_TAUXPT_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_STRUCCAT_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_YOUNGW_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_PLUSPLAN_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_WORKTIME_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_YOUNGJTR_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_DOMESDED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_UNPROFC_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_UNPRERET_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SCREADED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_PUBSEDED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_UNACTDED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_4DAYDED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_38HDED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_UNSMEDED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_DRTUGDED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SUBCDED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_JOBSDED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_JOBSMONT_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SOCMARDE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPRIV_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPRIV_EMER_QT(DIM_ALLOC))
    !Contributions of wage earners on the public labour market  
    ALLOCATE(CONTRIB_REMBPUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_PUBNSCOM_EMEE_QT(DIM_ALLOC,21))
    ALLOCATE(CONTRIB_PUBNSCOM_EMER_QT(DIM_ALLOC,21))
    ALLOCATE(CONTRIB_PUBNSHOL_EMER_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_PUBNSHOL_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_PUBNSDED_EMER_QT(DIM_ALLOC,21))
    ALLOCATE(CONTRIB_PUBNSDED_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_CSLOCFED_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_PUBSCOM_EMEE_QT(DIM_ALLOC,6))
    ALLOCATE(CONTRIB_PUBSCOM_EMER_QT(DIM_ALLOC,6))
    ALLOCATE(CONTRIB_PUBSHOL_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_PUBSDED_EMER_QT(DIM_ALLOC,7))
    ALLOCATE(CONTRIB_PUBSDED_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPUB_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPUB_EMER_QT(DIM_ALLOC))
    !Contributions of self employed              
    ALLOCATE(CONTRIB_SELFYEAR_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SELFCOM_EMER_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SELFPCR_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SELFFIRS_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SELFDEDU_EMER_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSSELF_EMER_QT(DIM_ALLOC))
    !Contributions on non labour market income               
    ALLOCATE(CONTRIB_FAMCHARG_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPEBASI_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPESOLI_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_EARSTART_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSEARBEN_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSDISAB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSINDOCC_QT(DIM_ALLOC))
    !Gross taxable income concepts               
    ALLOCATE(CONTRIB_GTIPRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTISELF_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPENS_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIEARNE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIEAROL_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIOLDUN_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIOTHUN_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIDISAB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTINDOCC_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTISICK_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIFAMAL_QT(DIM_ALLOC))
    !Other exchange concepts             
    ALLOCATE(CONTRIB_REVPRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_REVPUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GREMPLOY_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSEMPLOY_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSREPLAC_QT(DIM_ALLOC))
    !Instruments
    ALLOCATE(CONTRIB_WORKHOUR(DIM_ALLOC))
    ALLOCATE(CONTRIB_FULLPART_QT(DIM_ALLOC))
    
END SUBROUTINE CONTRIB_ALLOC_ENDO

SUBROUTINE CONTRIB_DEALLOC

    !=======================================================================!  
    !Exogenous variables
    !CONTRIB_INPUT_Write.sas
    !/*Variabelen uit PopChar-bestand*/
    DEALLOCATE(GECODEERD_INSZ_CHAR)
    DEALLOCATE(GECODEERD_INSZ_INPUT)
    DEALLOCATE(NAREGNIS_HH_ID)
    DEALLOCATE(NAREGNIS_DATBIRTH)
    DEALLOCATE(NAREGNIS_SEX)
    !/*HistorInfo*/
    DEALLOCATE(LABMSTAT_PRIV)
    DEALLOCATE(LABMSTAT_PUB)
    !/*Variabelen uit RSZ-bestand*/
    DEALLOCATE(tauxpt_20084)
    DEALLOCATE(t_prest_20084)
    DEALLOCATE(secemp_20084)
    DEALLOCATE(codtra_20084)
    DEALLOCATE(clatra_20084)
    DEALLOCATE(clatr2_20084)
    DEALLOCATE(codasm_20084)
    DEALLOCATE(CATTRA_20084)
    DEALLOCATE(GROEP_VG)
    DEALLOCATE(sal100_agg)
    DEALLOCATE(primes_agg)
    DEALLOCATE(preavi_agg)
    DEALLOCATE(salatt_agg)
    DEALLOCATE(salfor_agg)
    DEALLOCATE(DeductionCode_Pers_1)
    DEALLOCATE(DeductionCode_Pat_1)
    DEALLOCATE(DeductionCode_Pat_2)
    DEALLOCATE(DeductionCode_Pat_3)
    DEALLOCATE(SumSal_2008)
    DEALLOCATE(RSZ_LABMSTAT_PRIV_2008)
    DEALLOCATE(RSZ_LABMSTAT_PUB_2008)
    DEALLOCATE(InRSZ)
    DEALLOCATE(RATE_FICTIF_RSZ)
    !/*Variabelen uit RSZPPO-bestand*/
    DEALLOCATE(apltauxpt_20084)
    DEALLOCATE(APL_t_prest_20084)
    DEALLOCATE(APL_cattra_20084)
    DEALLOCATE(APL_codtra_20084)
    DEALLOCATE(clatraApl_20084)
    DEALLOCATE(clatr2Apl_20084)
    DEALLOCATE(APL_GROEP_EV)
    DEALLOCATE(APL_GROEP_DV)
    DEALLOCATE(APL_sal100_agg)
    DEALLOCATE(APL_salcot_agg)
    DEALLOCATE(APL_primes_agg)
    DEALLOCATE(APL_DeductionCode_Pers_1)
    DEALLOCATE(APL_SumSal_2008)
    DEALLOCATE(RSZPPO_LABMSTAT_PUB_2008)
    DEALLOCATE(InRSZPPO)
    DEALLOCATE(RATE_FICTIF_RSZPPO)
    !/*Variabelen uit RSVZ-bestand*/
    DEALLOCATE(HOEDRSVZ_2008)
    DEALLOCATE(BEGAANSC_2008)
    DEALLOCATE(BIJDCAT_2008)
    DEALLOCATE(inkomens_2008)
    DEALLOCATE(SelfStat_2008)
    DEALLOCATE(NoIncomeLine_RSVZ)
    DEALLOCATE(Bron_Aanv)
    !CONTRIB_CONSTRUCT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_CONSTRUCT)
    DEALLOCATE(MIMOSIS_FLEXWEAR)
    DEALLOCATE(MIMOSIS_HOUR)
    DEALLOCATE(MIMOSIS_NUMHHMEM)
    DEALLOCATE(MIMOSIS_SIMNUMMT)
    DEALLOCATE(MIMOSIS_SIMSAMMT)
    DEALLOCATE(MIMOSIS_HOURPRED)
    DEALLOCATE(MIMOSIS_WAGEPRED)
    !Variables read from file CONTRIB_PENSWELF.txt                                             
    DEALLOCATE(GECODEERD_INSZ_PENSWELF)
    DEALLOCATE(PENSWELF_RETPENS_QT)
    DEALLOCATE(PENSWELF_SURPENS_QT)
    !Variables read from file CONTRIB_UNEM.txt                                             
    DEALLOCATE(GECODEERD_INSZ_UNEM)
    DEALLOCATE(UNEM_BENUNS1_QT)
    DEALLOCATE(UNEM_ADDUNS_QT)
    DEALLOCATE(UNEM_BENEMP1_QT)
    DEALLOCATE(UNEM_ADDEMP_QT)
    DEALLOCATE(UNEM_OTHERUN_QT)
    DEALLOCATE(UNEM_OLDUNSS_QT)
    DEALLOCATE(UNEM_STATUS_QT)
    DEALLOCATE(UNEM_NUMMINU_QT)
    !Variables read from file CONTRIB_FAMAL.txt                                            
    DEALLOCATE(GECODEERD_INSZ_FAMAL)
    DEALLOCATE(FAMAL_AMOUNTF_QT)
    !Variables read from file CONTRIB_SICK.txt                                             
    DEALLOCATE(GECODEERD_INSZ_SICK)
    DEALLOCATE(SICK_IND_QT)
    DEALLOCATE(SICK_OCC_QT)
    DEALLOCATE(SICK_ALOTHBEN_QT)
    DEALLOCATE(SICK_DISABWEA_QT)
    DEALLOCATE(SICK_EMPLBPRI_QT)
    DEALLOCATE(SICK_EMPLBPUB_QT)
    !Variables read from file CONTRIB_PIT.txt                                              
    DEALLOCATE(GECODEERD_INSZ_PIT)
    DEALLOCATE(PIT_DEPCHILD_ANN)
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    DEALLOCATE(GECODEERD_INSZ_FAMREL)
    DEALLOCATE(FAMREL_FAMILY)
    DEALLOCATE(FAMREL_RELATION)
    DEALLOCATE(FAMREL_FAMTYPE)
    DEALLOCATE(FAMREL_COUPLE)
    !WAGE_CONSTRUCT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WAGE)
    DEALLOCATE(SALJRS_LAST_CORR)
    DEALLOCATE(MIMOSIS_GRINC_HOUR)
    DEALLOCATE(MIMOSIS_GRINC_QT)
    !MIMOSIS_SAMPWEIG_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WEIGHT)
    DEALLOCATE(MIMOSIS_WEIGHT)

    !=======================================================================!  
    !Endogeneous variables
    !to rank variables
    DEALLOCATE(NEW_RANK)
    !to construct age variables
    DEALLOCATE(CONTRIB_YEAR_AGE)
    DEALLOCATE(CONTRIB_MONTH_AGE)
    DEALLOCATE(CONTRIB_MONTH_BIRTH)
    DEALLOCATE(CONTRIB_YEAR_BIRTH)
    !to construct information at the level of the sociological household
    DEALLOCATE(CONTRIB_HH_SIZE)
    DEALLOCATE(CONTRIB_START_HH)
    DEALLOCATE(CONTRIB_END_HH)
    !To construct variables listed in the module note
    !Income concepts             
    DEALLOCATE(CONTRIB_LABMSTAT_PRIV_QT)
    DEALLOCATE(CONTRIB_INCPRIV_QT)
    DEALLOCATE(CONTRIB_SHPRIV_QT)
    DEALLOCATE(CONTRIB_DHPRIV_QT)
    DEALLOCATE(CONTRIB_INCSPRIV_QT)
    DEALLOCATE(CONTRIB_LABMSTAT_PUB_QT)
    DEALLOCATE(CONTRIB_INCPUB_QT)
    DEALLOCATE(CONTRIB_SHPUB_QT)
    DEALLOCATE(CONTRIB_DHPUB_QT)
    DEALLOCATE(CONTRIB_INCSPUB_QT)
    DEALLOCATE(CONTRIB_LABMSTAT_SELF_QT)
    DEALLOCATE(CONTRIB_INCSELF_QT)
    DEALLOCATE(CONTRIB_SELFEXTR_QT)
    DEALLOCATE(CONTRIB_PENS_QT)
    DEALLOCATE(CONTRIB_UNEMFT_QT)
    DEALLOCATE(CONTRIB_UNEMPT_QT)
    DEALLOCATE(CONTRIB_DISABWEA_QT)
    DEALLOCATE(CONTRIB_INDOCC_QT)
    !Contributions of wage earners on the private labour market              
    DEALLOCATE(CONTRIB_SIZECOMP_QT)
    DEALLOCATE(CONTRIB_REMBPRIV_QT)
    DEALLOCATE(CONTRIB_WEARCOM_EMEE_QT)
    DEALLOCATE(CONTRIB_WEARCOM_EMER_QT)
    DEALLOCATE(CONTRIB_WEARHOLA_EMER_QT)
    DEALLOCATE(CONTRIB_WEARHOLD_EMEE_QT)
    DEALLOCATE(CONTRIB_WEARDEDU_EMER_QT)
    DEALLOCATE(CONTRIB_WEARDEDU_EMEE_QT)
    DEALLOCATE(CONTRIB_STRUCDED_QT)
    DEALLOCATE(CONTRIB_TAUXPT_QT)
    DEALLOCATE(CONTRIB_STRUCCAT_QT)
    DEALLOCATE(CONTRIB_YOUNGW_QT)
    DEALLOCATE(CONTRIB_PLUSPLAN_QT)
    DEALLOCATE(CONTRIB_WORKTIME_QT)
    DEALLOCATE(CONTRIB_YOUNGJTR_QT)
    DEALLOCATE(CONTRIB_DOMESDED_QT)
    DEALLOCATE(CONTRIB_UNPROFC_QT)
    DEALLOCATE(CONTRIB_UNPRERET_QT)
    DEALLOCATE(CONTRIB_SCREADED_QT)
    DEALLOCATE(CONTRIB_PUBSEDED_QT)
    DEALLOCATE(CONTRIB_UNACTDED_QT)
    DEALLOCATE(CONTRIB_4DAYDED_QT)
    DEALLOCATE(CONTRIB_38HDED_QT)
    DEALLOCATE(CONTRIB_UNSMEDED_QT)
    DEALLOCATE(CONTRIB_DRTUGDED_QT)
    DEALLOCATE(CONTRIB_SUBCDED_QT)
    DEALLOCATE(CONTRIB_JOBSDED_QT)
    DEALLOCATE(CONTRIB_JOBSMONT_QT)
    DEALLOCATE(CONTRIB_SOCMARDE_QT)
    DEALLOCATE(CONTRIB_SSPRIV_EMEE_QT)
    DEALLOCATE(CONTRIB_SSPRIV_EMER_QT)
    !Contributions of wage earners on the public labour market               
    DEALLOCATE(CONTRIB_REMBPUB_QT)
    DEALLOCATE(CONTRIB_PUBNSCOM_EMEE_QT)
    DEALLOCATE(CONTRIB_PUBNSCOM_EMER_QT)
    DEALLOCATE(CONTRIB_PUBNSHOL_EMER_QT)
    DEALLOCATE(CONTRIB_PUBNSHOL_EMEE_QT)
    DEALLOCATE(CONTRIB_PUBNSDED_EMER_QT)
    DEALLOCATE(CONTRIB_PUBNSDED_EMEE_QT)
    DEALLOCATE(CONTRIB_CSLOCFED_QT)
    DEALLOCATE(CONTRIB_PUBSCOM_EMEE_QT)
    DEALLOCATE(CONTRIB_PUBSCOM_EMER_QT)
    DEALLOCATE(CONTRIB_PUBSHOL_EMEE_QT)
    DEALLOCATE(CONTRIB_PUBSDED_EMER_QT)
    DEALLOCATE(CONTRIB_PUBSDED_EMEE_QT)
    DEALLOCATE(CONTRIB_SSPUB_EMEE_QT)
    DEALLOCATE(CONTRIB_SSPUB_EMER_QT)
    !Contributions of self employed              
    DEALLOCATE(CONTRIB_SELFYEAR_QT)
    DEALLOCATE(CONTRIB_SELFCOM_EMER_QT)
    DEALLOCATE(CONTRIB_SELFPCR_QT)
    DEALLOCATE(CONTRIB_SELFFIRS_QT)
    DEALLOCATE(CONTRIB_SELFDEDU_EMER_QT)
    DEALLOCATE(CONTRIB_SSSELF_EMER_QT)
    !Contributions on non labour market income               
    DEALLOCATE(CONTRIB_FAMCHARG_QT)
    DEALLOCATE(CONTRIB_SSPEBASI_QT)
    DEALLOCATE(CONTRIB_SSPESOLI_QT)
    DEALLOCATE(CONTRIB_EARSTART_QT)
    DEALLOCATE(CONTRIB_SSEARBEN_QT)
    DEALLOCATE(CONTRIB_SSDISAB_QT)
    DEALLOCATE(CONTRIB_SSINDOCC_QT)
    !Gross taxable income concepts               
    DEALLOCATE(CONTRIB_GTIPRIV_QT)
    DEALLOCATE(CONTRIB_GTIPUB_QT)
    DEALLOCATE(CONTRIB_GTISELF_QT)
    DEALLOCATE(CONTRIB_GTIPENS_QT)
    DEALLOCATE(CONTRIB_GTIEARNE_QT)
    DEALLOCATE(CONTRIB_GTIEAROL_QT)
    DEALLOCATE(CONTRIB_GTIOLDUN_QT)
    DEALLOCATE(CONTRIB_GTIOTHUN_QT)
    DEALLOCATE(CONTRIB_GTIDISAB_QT)
    DEALLOCATE(CONTRIB_GTINDOCC_QT)
    DEALLOCATE(CONTRIB_GTISICK_QT)
    DEALLOCATE(CONTRIB_GTIFAMAL_QT)
    !Other exchange concepts             
    DEALLOCATE(CONTRIB_REVPRIV_QT)
    DEALLOCATE(CONTRIB_REVPUB_QT)
    DEALLOCATE(CONTRIB_GREMPLOY_QT)
    DEALLOCATE(CONTRIB_SSEMPLOY_QT)
    DEALLOCATE(CONTRIB_SSREPLAC_QT)
    !Instruments
    DEALLOCATE(CONTRIB_WORKHOUR)
    DEALLOCATE(CONTRIB_FULLPART_QT)

    !=======================================================================!
    !Deallocate parameters
    DEALLOCATE(CONTRIB_WEARCOM_QT)
    DEALLOCATE(CONTRIB_SD1_QT)
    DEALLOCATE(CONTRIB_SD2_QT)
    DEALLOCATE(CONTRIB_SD3_QT)
    DEALLOCATE(CONTRIB_SD4_QT)
    DEALLOCATE(CONTRIB_PLUSDEDU_QT)
    DEALLOCATE(CONTRIB_UNPROFDP_QT)
    DEALLOCATE(CONTRIB_UNPROFDF_QT)
    DEALLOCATE(CONTRIB_UNPRERDP_QT)
    DEALLOCATE(CONTRIB_UNPRERDF_QT)
    DEALLOCATE(CONTRIB_4DAYDEDA_QT)
    DEALLOCATE(CONTRIB_JOBSDEDA_QT)
    DEALLOCATE(CONTRIB_PDEDBL_MONTH)
    DEALLOCATE(CONTRIB_PDEDWI_MONTH)
    DEALLOCATE(CONTRIB_CSCOMLOC_QT)
    DEALLOCATE(CONTRIB_CSCOMFED_QT)
    DEALLOCATE(CONTRIB_LSALLSE_QT)
    DEALLOCATE(CONTRIB_SELFPAA_QT)
    DEALLOCATE(CONTRIB_SELFPAB_QT)
    DEALLOCATE(CONTRIB_SELFPAC_QT)
    DEALLOCATE(CONTRIB_SELFCAA_QT)
    DEALLOCATE(CONTRIB_SELFARA_QT)
    DEALLOCATE(CONTRIB_SELFARB_QT)
    DEALLOCATE(CONTRIB_SELFARC_QT)
    DEALLOCATE(CONTRIB_SELFASA_QT)
    DEALLOCATE(CONTRIB_SELFASB_QT)
    DEALLOCATE(CONTRIB_PELIMBA_MONTH)
    DEALLOCATE(CONTRIB_PELIMNOC_MONTH)
    DEALLOCATE(CONTRIB_PELIMCHI_MONTH)
    DEALLOCATE(CONTRIB_EARPERC_QT)
    DEALLOCATE(CONTRIB_EALIMFT_MONTH)
    DEALLOCATE(CONTRIB_EALIMPT_MONTH)
    DEALLOCATE(CONTRIB_DISABIM_DAY)
    DEALLOCATE(CONTRIB_STRU2008_QT)
    DEALLOCATE(CONTRIB_CEPWEARN_YEAR)
    DEALLOCATE(CONTRIB_RAPWEARN_YEAR)
    
END SUBROUTINE CONTRIB_DEALLOC

!===============================================!
!Initialise endogenous variables                        
!===============================================!

SUBROUTINE CONTRIB_INITENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors
    INTEGER                                                 :: pers_j                               !index for do loop

    DO pers_j=1,DIM_ALLOC
        !to rank variables
        NEW_RANK(pers_j) = 0
        !to construct age variables
        CONTRIB_YEAR_AGE(pers_j) = 0
        CONTRIB_MONTH_AGE(pers_j) = 0
        CONTRIB_MONTH_BIRTH(pers_j) = 0
        CONTRIB_YEAR_BIRTH(pers_j) = 0
        !to construct information at the level of the sociological household
        CONTRIB_HH_SIZE(pers_j) = 0
        CONTRIB_START_HH(pers_j) = 0
        CONTRIB_END_HH(pers_j) = 0
        !To construct variables listed in the module note
        !Income concepts             
        CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 0
        CONTRIB_INCPRIV_QT(pers_j) = 0
        CONTRIB_SHPRIV_QT(pers_j) = 0
        CONTRIB_DHPRIV_QT(pers_j) = 0
        CONTRIB_INCSPRIV_QT(pers_j) = 0
        CONTRIB_LABMSTAT_PUB_QT(pers_j) = 0
        CONTRIB_INCPUB_QT(pers_j) = 0
        CONTRIB_SHPUB_QT(pers_j) = 0
        CONTRIB_DHPUB_QT(pers_j) = 0
        CONTRIB_INCSPUB_QT(pers_j) = 0
        CONTRIB_LABMSTAT_SELF_QT(pers_j) = 0
        CONTRIB_INCSELF_QT(pers_j) = 0
        CONTRIB_SELFEXTR_QT(pers_j) = 0
        CONTRIB_PENS_QT(pers_j) = 0
        CONTRIB_UNEMFT_QT(pers_j) = 0
        CONTRIB_UNEMPT_QT(pers_j) = 0
        CONTRIB_DISABWEA_QT(pers_j) = 0
        CONTRIB_INDOCC_QT(pers_j) = 0
        !Contributions of wage earners on the private labour market              
        CONTRIB_SIZECOMP_QT(pers_j) = 0
        CONTRIB_REMBPRIV_QT(pers_j) = 0
        CONTRIB_WEARCOM_EMEE_QT(pers_j,1:21) = 0
        CONTRIB_WEARCOM_EMER_QT(pers_j,1:21) = 0
        CONTRIB_WEARHOLA_EMER_QT(pers_j) = 0
        CONTRIB_WEARHOLD_EMEE_QT(pers_j) = 0
        CONTRIB_WEARDEDU_EMER_QT(pers_j,1:21) = 0
        CONTRIB_WEARDEDU_EMEE_QT(pers_j) = 0
        CONTRIB_STRUCDED_QT(pers_j) = 0
        CONTRIB_TAUXPT_QT(pers_j) = 0
        CONTRIB_STRUCCAT_QT(pers_j) = 0
        CONTRIB_YOUNGW_QT(pers_j) = 0
        CONTRIB_PLUSPLAN_QT(pers_j) = 0
        CONTRIB_WORKTIME_QT(pers_j) = 0
        CONTRIB_YOUNGJTR_QT(pers_j) = 0
        CONTRIB_DOMESDED_QT(pers_j) = 0
        CONTRIB_UNPROFC_QT(pers_j) = 0
        CONTRIB_UNPRERET_QT(pers_j) = 0
        CONTRIB_SCREADED_QT(pers_j) = 0
        CONTRIB_PUBSEDED_QT(pers_j) = 0
        CONTRIB_UNACTDED_QT(pers_j) = 0
        CONTRIB_4DAYDED_QT(pers_j) = 0
        CONTRIB_38HDED_QT(pers_j) = 0
        CONTRIB_UNSMEDED_QT(pers_j) = 0
        CONTRIB_DRTUGDED_QT(pers_j) = 0
        CONTRIB_SUBCDED_QT(pers_j) = 0
        CONTRIB_JOBSDED_QT(pers_j) = 0
        CONTRIB_JOBSMONT_QT(pers_j) = 0
        CONTRIB_SOCMARDE_QT(pers_j) = 0
        CONTRIB_SSPRIV_EMEE_QT(pers_j) = 0
        CONTRIB_SSPRIV_EMER_QT(pers_j) = 0
        !Contributions of wage earners on the public labour market               
        CONTRIB_REMBPUB_QT(pers_j) = 0
        CONTRIB_PUBNSCOM_EMEE_QT(pers_j,1:21) = 0
        CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:21) = 0
        CONTRIB_PUBNSHOL_EMER_QT(pers_j) = 0
        CONTRIB_PUBNSHOL_EMEE_QT(pers_j) = 0
        CONTRIB_PUBNSDED_EMER_QT(pers_j,1:21) = 0
        CONTRIB_PUBNSDED_EMEE_QT(pers_j) = 0
        CONTRIB_CSLOCFED_QT(pers_j) = 0
        CONTRIB_PUBSCOM_EMEE_QT(pers_j,1:6) = 0
        CONTRIB_PUBSCOM_EMER_QT(pers_j,1:6) = 0
        CONTRIB_PUBSHOL_EMEE_QT(pers_j) = 0
        CONTRIB_PUBSDED_EMER_QT(pers_j,1:7) = 0
        CONTRIB_PUBSDED_EMEE_QT(pers_j) = 0
        CONTRIB_SSPUB_EMEE_QT(pers_j) = 0
        CONTRIB_SSPUB_EMER_QT(pers_j) = 0
        !Contributions of self employed              
        CONTRIB_SELFYEAR_QT(pers_j) = 0
        CONTRIB_SELFCOM_EMER_QT(pers_j) = 0
        CONTRIB_SELFPCR_QT(pers_j) = 0
        CONTRIB_SELFFIRS_QT(pers_j) = 0
        CONTRIB_SELFDEDU_EMER_QT(pers_j) = 0
        CONTRIB_SSSELF_EMER_QT(pers_j) = 0
        !Contributions on non labour market income               
        CONTRIB_FAMCHARG_QT(pers_j) = 0
        CONTRIB_SSPEBASI_QT(pers_j) = 0
        CONTRIB_SSPESOLI_QT(pers_j) = 0
        CONTRIB_EARSTART_QT(pers_j) = 0
        CONTRIB_SSEARBEN_QT(pers_j) = 0
        CONTRIB_SSDISAB_QT(pers_j) = 0
        CONTRIB_SSINDOCC_QT(pers_j) = 0
        !Gross taxable income concepts               
        CONTRIB_GTIPRIV_QT(pers_j) = 0
        CONTRIB_GTIPUB_QT(pers_j) = 0
        CONTRIB_GTISELF_QT(pers_j) = 0
        CONTRIB_GTIPENS_QT(pers_j) = 0
        CONTRIB_GTIEARNE_QT(pers_j) = 0
        CONTRIB_GTIEAROL_QT(pers_j) = 0
        CONTRIB_GTIOLDUN_QT(pers_j) = 0
        CONTRIB_GTIOTHUN_QT(pers_j) = 0
        CONTRIB_GTIDISAB_QT(pers_j) = 0
        CONTRIB_GTINDOCC_QT(pers_j) = 0
        CONTRIB_GTISICK_QT(pers_j) = 0
        CONTRIB_GTIFAMAL_QT(pers_j) = 0
        !Other exchange concepts             
        CONTRIB_REVPRIV_QT(pers_j) = 0
        CONTRIB_REVPUB_QT(pers_j) = 0
        CONTRIB_GREMPLOY_QT(pers_j) = 0
        CONTRIB_SSEMPLOY_QT(pers_j) = 0
        CONTRIB_SSREPLAC_QT(pers_j) = 0
        !Instruments
        CONTRIB_WORKHOUR(pers_j) = 0
        CONTRIB_FULLPART_QT(pers_j) = 0
    END DO
    N_HH_CONTRIB=0

END SUBROUTINE CONTRIB_INITENDO

!========== SUBROUTINE CONTRIB_RANKANDSORT ===============!
!RANK and SORT exogenous variables 
!   Step 1: rank and sort all variables according to their 
!           individual identification number which is read in as an integer
!   Step 2: rank and sort all variables according to the 
!           household identification number
!=========================================================!

SUBROUTINE CONTRIB_RANKANDSORT

!STEP1: ranking and sorting with the individuals identification number
    !Rank individual identification number from the INPUT data file
    CALL RANKING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    !CONTRIB_INPUT_Write.sas
    !/*Variabelen uit PopChar-bestand*/
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    !/*HistorInfo*/
    CALL SORTING(LABMSTAT_PRIV(1:n_rec),NEW_RANK)
    CALL SORTING(LABMSTAT_PUB(1:n_rec),NEW_RANK)
    !/*Variabelen uit RSZ-bestand*/
    CALL SORTING(tauxpt_20084(1:n_rec),NEW_RANK)
    CALL SORTING(t_prest_20084(1:n_rec),NEW_RANK)
    CALL SORTING(secemp_20084(1:n_rec),NEW_RANK)
    CALL SORTING(codtra_20084(1:n_rec),NEW_RANK)
    CALL SORTING(clatra_20084(1:n_rec),NEW_RANK)
    CALL SORTING(clatr2_20084(1:n_rec),NEW_RANK)
    CALL SORTING(codasm_20084(1:n_rec),NEW_RANK)
    CALL SORTING(CATTRA_20084(1:n_rec),NEW_RANK)
    CALL SORTING(GROEP_VG(1:n_rec),NEW_RANK)
    CALL SORTING(sal100_agg(1:n_rec),NEW_RANK)
    CALL SORTING(primes_agg(1:n_rec),NEW_RANK)
    CALL SORTING(preavi_agg(1:n_rec),NEW_RANK)
    CALL SORTING(salatt_agg(1:n_rec),NEW_RANK)
    CALL SORTING(salfor_agg(1:n_rec),NEW_RANK)
    CALL SORTING(DeductionCode_Pers_1(1:n_rec),NEW_RANK)
    CALL SORTING(DeductionCode_Pat_1(1:n_rec),NEW_RANK)
    CALL SORTING(DeductionCode_Pat_2(1:n_rec),NEW_RANK)
    CALL SORTING(DeductionCode_Pat_3(1:n_rec),NEW_RANK)
    CALL SORTING(SumSal_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RSZ_LABMSTAT_PRIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RSZ_LABMSTAT_PUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(InRSZ(1:n_rec),NEW_RANK)
    CALL SORTING(RATE_FICTIF_RSZ(1:n_rec),NEW_RANK)
    !/*Variabelen uit RSZPPO-bestand*/
    CALL SORTING(apltauxpt_20084(1:n_rec),NEW_RANK)
    CALL SORTING(APL_t_prest_20084(1:n_rec),NEW_RANK)
    CALL SORTING(APL_cattra_20084(1:n_rec),NEW_RANK)
    CALL SORTING(APL_codtra_20084(1:n_rec),NEW_RANK)
    CALL SORTING(clatraApl_20084(1:n_rec),NEW_RANK)
    CALL SORTING(clatr2Apl_20084(1:n_rec),NEW_RANK)
    CALL SORTING(APL_GROEP_EV(1:n_rec),NEW_RANK)
    CALL SORTING(APL_GROEP_DV(1:n_rec),NEW_RANK)
    CALL SORTING(APL_sal100_agg(1:n_rec),NEW_RANK)
    CALL SORTING(APL_salcot_agg(1:n_rec),NEW_RANK)
    CALL SORTING(APL_primes_agg(1:n_rec),NEW_RANK)
    CALL SORTING(APL_DeductionCode_Pers_1(1:n_rec),NEW_RANK)
    CALL SORTING(APL_SumSal_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RSZPPO_LABMSTAT_PUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(InRSZPPO(1:n_rec),NEW_RANK)
    CALL SORTING(RATE_FICTIF_RSZPPO(1:n_rec),NEW_RANK)
    !/*Variabelen uit RSVZ-bestand*/
    CALL SORTING(HOEDRSVZ_2008(1:n_rec),NEW_RANK)
    CALL SORTING(BEGAANSC_2008(1:n_rec),NEW_RANK)
    CALL SORTING(BIJDCAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(inkomens_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SelfStat_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NoIncomeLine_RSVZ(1:n_rec),NEW_RANK)
    CALL SORTING(Bron_Aanv(1:n_rec),NEW_RANK)
    !Rank individual identification number from file CONTRIB_CONSTRUCT.txt                                             
    CALL RANKING(GECODEERD_INSZ_CONSTRUCT(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_CONSTRUCT.txt                                             
    !CONTRIB_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CONSTRUCT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_FLEXWEAR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_NUMHHMEM(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_SIMNUMMT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_SIMSAMMT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_HOURPRED(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WAGEPRED(1:n_rec),NEW_RANK)
    !Rank individual identification number from file CONTRIB_PENSWELF.txt                                             
    CALL RANKING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_PENSWELF.txt                                             
    CALL SORTING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_RETPENS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_SURPENS_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file CONTRIB_UNEM.txt                                             
    CALL RANKING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_UNEM.txt                                             
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_BENUNS1_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_ADDUNS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_BENEMP1_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_ADDEMP_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_OTHERUN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_OLDUNSS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_STATUS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_NUMMINU_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file CONTRIB_FAMAL.txt                                            
    CALL RANKING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_FAMAL.txt                                            
    CALL SORTING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMAL_AMOUNTF_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file CONTRIB_SICK.txt                                             
    CALL RANKING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_SICK.txt                                             
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_IND_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_OCC_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_ALOTHBEN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_DISABWEA_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_EMPLBPRI_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_EMPLBPUB_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file CONTRIB_PIT.txt                                              
    CALL RANKING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_PIT.txt                                              
    CALL SORTING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_DEPCHILD_ANN(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_FAMREL.txt                                           
    CALL RANKING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !Rank individual identification number from file WAGE_CONSTRUCT.txt                                           
    CALL RANKING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_WEIGHT.txt
    CALL RANKING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL CONTRIB_CONSISTENCY_CHECK
   
!STEP2: ranking and sorting with the household identification number
    !Rank household number
    CALL RANKING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    !Sort variables on rank of household number
    !CONTRIB_INPUT_Write.sas
    !/*Variabelen uit PopChar-bestand*/
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    !/*HistorInfo*/
    CALL SORTING(LABMSTAT_PRIV(1:n_rec),NEW_RANK)
    CALL SORTING(LABMSTAT_PUB(1:n_rec),NEW_RANK)
    !/*Variabelen uit RSZ-bestand*/
    CALL SORTING(tauxpt_20084(1:n_rec),NEW_RANK)
    CALL SORTING(t_prest_20084(1:n_rec),NEW_RANK)
    CALL SORTING(secemp_20084(1:n_rec),NEW_RANK)
    CALL SORTING(codtra_20084(1:n_rec),NEW_RANK)
    CALL SORTING(clatra_20084(1:n_rec),NEW_RANK)
    CALL SORTING(clatr2_20084(1:n_rec),NEW_RANK)
    CALL SORTING(codasm_20084(1:n_rec),NEW_RANK)
    CALL SORTING(CATTRA_20084(1:n_rec),NEW_RANK)
    CALL SORTING(GROEP_VG(1:n_rec),NEW_RANK)
    CALL SORTING(sal100_agg(1:n_rec),NEW_RANK)
    CALL SORTING(primes_agg(1:n_rec),NEW_RANK)
    CALL SORTING(preavi_agg(1:n_rec),NEW_RANK)
    CALL SORTING(salatt_agg(1:n_rec),NEW_RANK)
    CALL SORTING(salfor_agg(1:n_rec),NEW_RANK)
    CALL SORTING(DeductionCode_Pers_1(1:n_rec),NEW_RANK)
    CALL SORTING(DeductionCode_Pat_1(1:n_rec),NEW_RANK)
    CALL SORTING(DeductionCode_Pat_2(1:n_rec),NEW_RANK)
    CALL SORTING(DeductionCode_Pat_3(1:n_rec),NEW_RANK)
    CALL SORTING(SumSal_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RSZ_LABMSTAT_PRIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RSZ_LABMSTAT_PUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(InRSZ(1:n_rec),NEW_RANK)
    CALL SORTING(RATE_FICTIF_RSZ(1:n_rec),NEW_RANK)
    !/*Variabelen uit RSZPPO-bestand*/
    CALL SORTING(apltauxpt_20084(1:n_rec),NEW_RANK)
    CALL SORTING(APL_t_prest_20084(1:n_rec),NEW_RANK)
    CALL SORTING(APL_cattra_20084(1:n_rec),NEW_RANK)
    CALL SORTING(APL_codtra_20084(1:n_rec),NEW_RANK)
    CALL SORTING(clatraApl_20084(1:n_rec),NEW_RANK)
    CALL SORTING(clatr2Apl_20084(1:n_rec),NEW_RANK)
    CALL SORTING(APL_GROEP_EV(1:n_rec),NEW_RANK)
    CALL SORTING(APL_GROEP_DV(1:n_rec),NEW_RANK)
    CALL SORTING(APL_sal100_agg(1:n_rec),NEW_RANK)
    CALL SORTING(APL_salcot_agg(1:n_rec),NEW_RANK)
    CALL SORTING(APL_primes_agg(1:n_rec),NEW_RANK)
    CALL SORTING(APL_DeductionCode_Pers_1(1:n_rec),NEW_RANK)
    CALL SORTING(APL_SumSal_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RSZPPO_LABMSTAT_PUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(InRSZPPO(1:n_rec),NEW_RANK)
    CALL SORTING(RATE_FICTIF_RSZPPO(1:n_rec),NEW_RANK)
    !/*Variabelen uit RSVZ-bestand*/
    CALL SORTING(HOEDRSVZ_2008(1:n_rec),NEW_RANK)
    CALL SORTING(BEGAANSC_2008(1:n_rec),NEW_RANK)
    CALL SORTING(BIJDCAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(inkomens_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SelfStat_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NoIncomeLine_RSVZ(1:n_rec),NEW_RANK)
    CALL SORTING(Bron_Aanv(1:n_rec),NEW_RANK)
    !CONTRIB_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CONSTRUCT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_FLEXWEAR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_NUMHHMEM(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_SIMNUMMT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_SIMSAMMT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_HOURPRED(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WAGEPRED(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_PENSWELF.txt                                             
    CALL SORTING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_RETPENS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_SURPENS_QT(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_UNEM.txt                                             
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_BENUNS1_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_ADDUNS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_BENEMP1_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_ADDEMP_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_OTHERUN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_OLDUNSS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_STATUS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_NUMMINU_QT(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_FAMAL.txt                                            
    CALL SORTING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMAL_AMOUNTF_QT(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_SICK.txt                                             
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_IND_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_OCC_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_ALOTHBEN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_DISABWEA_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_EMPLBPRI_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_EMPLBPUB_QT(1:n_rec),NEW_RANK)
    !Sort variables from file CONTRIB_PIT.txt                                              
    CALL SORTING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_DEPCHILD_ANN(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL CONTRIB_CONSISTENCY_CHECK

END SUBROUTINE CONTRIB_RANKANDSORT

!========== SUBROUTINE PARAMETER_FILEREAD ===============!
!This subroutines reads in all parameters from         !
!the parameter file                                    !
!======================================================!
SUBROUTINE CONTRIB_PARAMETER_FILEREAD

    !Read in parameters for path and name of input files
    DO i=1,n_loclines/2
        !Read default path names
        !Start reading the normal file names
        IF (CONTROL_IO(2*i-1)=='CONTRIB_INPUT_PATH') THEN
            CONTRIB_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_INPUT_FILE') THEN
            CONTRIB_INPUT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_CONSTRUCT_FILE') THEN
            CONTRIB_CONSTRUCT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_PENSWELF_PRERUN_FILE') THEN
            CONTRIB_PENSWELF_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_UNEM_FILE') THEN
            CONTRIB_UNEM_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_FAMAL_FILE') THEN
            CONTRIB_FAMAL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_SICK_FILE') THEN
            CONTRIB_SICK_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_PIT_FILE') THEN
            CONTRIB_PIT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_INPUT_PATH') THEN
            FAMREL_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_FAMREL_FILE') THEN
            MIMOSIS_FAMREL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='WAGE_CONSTRUCT_FILE') THEN
            WAGE_CONSTRUCT_FILE = CONTROL_IO(2*i)
        !Read material to read sample weights                                                    
        ELSE IF (CONTROL_IO(2*i-1)=='CONSTANT_INPUT_PATH') THEN
            CONSTANT_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_SAMPWEIG_FILE') THEN
            MIMOSIS_SAMPWEIG_FILE = CONTROL_IO(2*i)
        !Read the file names of the intermediate output files
        ELSE IF (CONTROL_IO(2*i-1)=='PIT_CONTRIB_FILE') THEN
            PIT_CONTRIB_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_CONTRIB_FILE') THEN
            SOCBEN_CONTRIB_FILE = CONTROL_IO(2*i)
        !Read information necessary to construct file for evaluation module
        ELSE IF (CONTROL_IO(2*i-1)=='OUTPUT_PATH')THEN
            OUTPUT_PATH = CONTROL_IO(2*i)   
        ELSE IF (CONTROL_IO(2*i-1)=='EVAL_CONTRIB_FILE') THEN
            EVAL_CONTRIB_FILE = CONTROL_IO(2*i)
        END IF    
    END DO

    !check for length of default path and file name parameters
    CONTRIB_PAR_BREAK_POINT(1)  = INDEX(CONTRIB_INPUT_PATH,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(2)  = INDEX(CONTRIB_INPUT_FILE,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(3)  = INDEX(CONTRIB_PENSWELF_FILE,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(4)  = INDEX(CONTRIB_UNEM_FILE,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(5)  = INDEX(CONTRIB_FAMAL_FILE,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(6)  = INDEX(CONTRIB_SICK_FILE,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(7)  = INDEX(CONTRIB_PIT_FILE,CONTRIB_PAR_BREAK)    
    CONTRIB_PAR_BREAK_POINT(8)  = INDEX(FAMREL_INPUT_PATH,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(9)  = INDEX(MIMOSIS_FAMREL_FILE,CONTRIB_PAR_BREAK)    
    CONTRIB_PAR_BREAK_POINT(10) = INDEX(WAGE_CONSTRUCT_FILE,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(11) = INDEX(CONSTANT_INPUT_PATH,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(12) = INDEX(MIMOSIS_SAMPWEIG_FILE,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(13) = INDEX(PIT_CONTRIB_FILE,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(14) = INDEX(SOCBEN_CONTRIB_FILE,CONTRIB_PAR_BREAK)
    CONTRIB_PAR_BREAK_POINT(15) = INDEX(OUTPUT_PATH,CONTRIB_PAR_BREAK)    
    CONTRIB_PAR_BREAK_POINT(16) = INDEX(EVAL_CONTRIB_FILE,CONTRIB_PAR_BREAK)    
    CONTRIB_PAR_BREAK_POINT(17) = INDEX(CONTRIB_CONSTRUCT_FILE,CONTRIB_PAR_BREAK)    

    !merge information (path+name) to create default input file names
    CONTRIB_INPUT = CONTRIB_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(1)-1)//&
    &CONTRIB_INPUT_FILE(1:CONTRIB_PAR_BREAK_POINT(2)-1)
    CONTRIB_CONSTRUCT = CONTRIB_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(1)-1)//&
    &CONTRIB_CONSTRUCT_FILE(1:CONTRIB_PAR_BREAK_POINT(17)-1)
    CONTRIB_PENSWELF = CONTRIB_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(1)-1)//&
    &CONTRIB_PENSWELF_FILE(1:CONTRIB_PAR_BREAK_POINT(3)-1)
    CONTRIB_UNEM = CONTRIB_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(1)-1)//&
    &CONTRIB_UNEM_FILE(1:CONTRIB_PAR_BREAK_POINT(4)-1)
    CONTRIB_FAMAL = CONTRIB_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(1)-1)//&
    &CONTRIB_FAMAL_FILE(1:CONTRIB_PAR_BREAK_POINT(5)-1)
    CONTRIB_SICK = CONTRIB_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(1)-1)//&
    &CONTRIB_SICK_FILE(1:CONTRIB_PAR_BREAK_POINT(6)-1)
    CONTRIB_PIT = CONTRIB_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(1)-1)//&
    &CONTRIB_PIT_FILE(1:CONTRIB_PAR_BREAK_POINT(7)-1)
    MIMOSIS_FAMREL = FAMREL_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(8)-1)//&
    &MIMOSIS_FAMREL_FILE(1:CONTRIB_PAR_BREAK_POINT(9)-1)    
    WAGE_CONSTRUCT = CONTRIB_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(1)-1)//&
    &WAGE_CONSTRUCT_FILE(1:CONTRIB_PAR_BREAK_POINT(10)-1)
    MIMOSIS_SAMPWEIG = CONSTANT_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(11)-1)//&
    &MIMOSIS_SAMPWEIG_FILE(1:CONTRIB_PAR_BREAK_POINT(12)-1)    
    !construct names of output files
    PIT_CONTRIB = CONTRIB_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(1)-1)//&
    &PIT_CONTRIB_FILE(1:CONTRIB_PAR_BREAK_POINT(13)-1)
    SOCBEN_CONTRIB = CONTRIB_INPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_CONTRIB_FILE(1:CONTRIB_PAR_BREAK_POINT(14)-1)
    EVAL_CONTRIB = OUTPUT_PATH(1:CONTRIB_PAR_BREAK_POINT(15)-1)//&
    &EVAL_CONTRIB_FILE(1:CONTRIB_PAR_BREAK_POINT(16)-1)

    !Trim the result
    !of input files
    CALL TRIM@(CONTRIB_INPUT)   
    CALL TRIM@(CONTRIB_CONSTRUCT)   
    CALL TRIM@(CONTRIB_PENSWELF)    
    CALL TRIM@(CONTRIB_UNEM)    
    CALL TRIM@(CONTRIB_FAMAL)   
    CALL TRIM@(CONTRIB_SICK)    
    CALL TRIM@(CONTRIB_PIT)     
    CALL TRIM@(MIMOSIS_FAMREL)    
    CALL TRIM@(WAGE_CONSTRUCT)  
    CALL TRIM@(MIMOSIS_SAMPWEIG)
    !of output files
    CALL TRIM@(PIT_CONTRIB)     
    CALL TRIM@(SOCBEN_CONTRIB) 
    CALL TRIM@(EVAL_CONTRIB)    
    
    !write names to log file
    WRITE(15,'(''Print of reconstructed input and output file names'')')
    CALL DRAWLINE(15,100,'=')    
    WRITE(15,*) 'CONTRIB_INPUT= ',      CONTRIB_INPUT
    WRITE(15,*) 'CONTRIB_CONSTRUCT= ',  CONTRIB_CONSTRUCT
    WRITE(15,*) 'CONTRIB_PENSWELF= ',   CONTRIB_PENSWELF
    WRITE(15,*) 'CONTRIB_UNEM= ',       CONTRIB_UNEM
    WRITE(15,*) 'CONTRIB_FAMAL= ',      CONTRIB_FAMAL
    WRITE(15,*) 'CONTRIB_SICK= ',       CONTRIB_SICK
    WRITE(15,*) 'CONTRIB_PIT= ',        CONTRIB_PIT
    WRITE(15,*) 'MIMOSIS_FAMREL= ',     MIMOSIS_FAMREL
    WRITE(15,*) 'WAGE_CONSTRUCT= ',     WAGE_CONSTRUCT
    WRITE(15,*) 'MIMOSIS_SAMPWEIG= ',   MIMOSIS_SAMPWEIG
    WRITE(15,*) 'PIT_CONTRIB= ',        PIT_CONTRIB
    WRITE(15,*) 'SOCBEN_CONTRIB= ',     SOCBEN_CONTRIB
    WRITE(15,*) 'EVAL_CONTRIB= ',       EVAL_CONTRIB

    !open parameter file        
    OPEN(UNIT=3, FILE=CONTRIB_PARLOC, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_PARLOC
        WRITE(15,'("Could not find ", a100)') CONTRIB_PARLOC
        PAUSE
        STOP
    END IF        

    ALLOCATE(CONTRIB_PAR_NAMES(n_par))
    !read in all parameters by looping over the lines in the parameter file
    WRITE(15,'(/,''Print of read in operation of parameter file'')')
    CALL DRAWLINE(15,100,'=')    
    i=1
    DO
        READ(3,'(a40,2i10,a10)') CONTRIB_PAR_NAMES(i), CONTRIB_PAR_L, &
        &CONTRIB_PAR_C, CONTRIB_PAR_CURR
        IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_PERSALB_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_PERSALB_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_PERSALB_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SHBLUE_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_SHBLUE_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_SHBLUE_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_DHBLUE_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_DHBLUE_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_DHBLUE_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_DHWHITE_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_DHWHITE_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_DHWHITE_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_DHCSFIX_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_DHCSFIX_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_DHCSFIX_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_DHCSVAR_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_DHCSVAR_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_DHCSVAR_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_WEARCOM_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_WEARCOM_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_WEARCOM_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_WEARCOM_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_WEARHOLA_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_WEARHOLA_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_WEARHOLA_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='MIMOSIS_PRIINDEX_ANN') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) MIMOSIS_PRIINDEX_ANN
            WRITE(15,CONTRIB_PAR_FORMAT) MIMOSIS_PRIINDEX_ANN
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_WEARHOLD_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_WEARHOLD_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_WEARHOLD_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SD1_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SD1_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_SD1_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_SD1_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SD2_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SD2_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_SD2_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_SD2_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SD3_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SD3_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_SD3_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_SD3_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SD4_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SD4_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_SD4_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_SD4_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SOCMAR_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_SOCMAR_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_SOCMAR_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_YWDED_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_YWDED_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_YWDED_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_PLUSDEDU_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_PLUSDEDU_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_PLUSDEDU_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_PLUSDEDU_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_UNPROFDP_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_UNPROFDP_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_UNPROFDP_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_UNPROFDP_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_UNPROFDF_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_UNPROFDF_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_UNPROFDF_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_UNPROFDF_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_UNPRERDP_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_UNPRERDP_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_UNPRERDP_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_UNPRERDP_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_UNPRERDF_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_UNPRERDF_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_UNPRERDF_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_UNPRERDF_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_4DAYDEDA_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_4DAYDEDA_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_4DAYDEDA_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_4DAYDEDA_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_4DAYDEDB_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_4DAYDEDB_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_4DAYDEDB_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_38HDEDA_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_38HDEDA_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_38HDEDA_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_JOBSDEDA_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_JOBSDEDA_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_JOBSDEDA_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_JOBSDEDA_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_PDEDBL_MONTH') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_PDEDBL_MONTH(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_PDEDBL_MONTH(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_PDEDBL_MONTH(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_PDEDWI_MONTH') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_PDEDWI_MONTH(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_PDEDWI_MONTH(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_PDEDWI_MONTH(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_CSCOMLOC_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_CSCOMLOC_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_CSCOMLOC_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_CSCOMLOC_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_CSCOMFED_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_CSCOMFED_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_CSCOMFED_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_CSCOMFED_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_CSDHCON_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_CSDHCON_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_CSDHCON_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_LSALLSE_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_LSALLSE_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_LSALLSE_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_LSALLSE_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFPAA_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SELFPAA_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_SELFPAA_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_SELFPAA_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFPAB_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SELFPAB_QT(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_SELFPAB_QT(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_SELFPAB_QT(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFPAC_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SELFPAC_QT(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_SELFPAC_QT(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_SELFPAC_QT(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFCAA_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SELFCAA_QT(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_SELFCAA_QT(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_SELFCAA_QT(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFARA_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SELFARA_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_SELFARA_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_SELFARA_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFARB_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SELFARB_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_SELFARB_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_SELFARB_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFARC_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SELFARC_QT(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_SELFARC_QT(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_SELFARC_QT(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFASA_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SELFASA_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_SELFASA_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_SELFASA_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFASB_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_SELFASB_QT(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_SELFASB_QT(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_SELFASB_QT(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFDEDA_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_SELFDEDA_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_SELFDEDA_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SELFDEDB_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_SELFDEDB_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_SELFDEDB_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='MIMOSIS_REFYEAR_ANN') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) MIMOSIS_REFYEAR_ANN
            WRITE(15,CONTRIB_PAR_FORMAT) MIMOSIS_REFYEAR_ANN
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_PELIMBA_MONTH') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_PELIMBA_MONTH(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_PELIMBA_MONTH(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_PELIMBA_MONTH(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_PENSPERC_MONTH') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_PENSPERC_MONTH
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_PENSPERC_MONTH
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_PELIMNOC_MONTH') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_PELIMNOC_MONTH(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_PELIMNOC_MONTH(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_PELIMNOC_MONTH(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_PELIMCHI_MONTH') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_PELIMCHI_MONTH(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_PELIMCHI_MONTH(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_PELIMCHI_MONTH(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_EARPERC_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_EARPERC_QT(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_EARPERC_QT(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_EARPERC_QT(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_EALIMFT_MONTH') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_EALIMFT_MONTH(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_EALIMFT_MONTH(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_EALIMFT_MONTH(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_EALIMPT_MONTH') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_EALIMPT_MONTH(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_EALIMPT_MONTH(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_EALIMPT_MONTH(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_DISABIM_DAY') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_DISABIM_DAY(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_DISABIM_DAY(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_DISABIM_DAY(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_DISAPERC_DAY') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_DISAPERC_DAY
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_DISAPERC_DAY
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_INDOCCPC_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_INDOCCPC_QT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_INDOCCPC_QT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_HOURSIM') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_HOURSIM
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_HOURSIM
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_WAGEINC') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_WAGEINC
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_WAGEINC
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SEXINCR') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_SEXINCR
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_SEXINCR
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_DUURP') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_DUURP
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_DUURP
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SIMUNEM') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_SIMUNEM
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_SIMUNEM
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_HOURUNEM') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_HOURUNEM
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_HOURUNEM
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_SIMNRMT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) CONTRIB_SIMNRMT
            WRITE(15,CONTRIB_PAR_FORMAT) CONTRIB_SIMNRMT
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_STRU2008_QT') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_STRU2008_QT(CONTRIB_PAR_L,CONTRIB_PAR_C))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) ((CONTRIB_STRU2008_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) ((CONTRIB_STRU2008_QT(j,k),k=1,CONTRIB_PAR_C),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_CEPWEARN_YEAR') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_CEPWEARN_YEAR(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_CEPWEARN_YEAR(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_CEPWEARN_YEAR(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='CONTRIB_RAPWEARN_YEAR') THEN
            WRITE(15,'(a40)') CONTRIB_PAR_NAMES(i)
            ALLOCATE(CONTRIB_RAPWEARN_YEAR(CONTRIB_PAR_L))
            READ(3,'(a20)') CONTRIB_PAR_FORMAT
            READ(3,CONTRIB_PAR_FORMAT) (CONTRIB_RAPWEARN_YEAR(j),j=1,CONTRIB_PAR_L)
            WRITE(15,CONTRIB_PAR_FORMAT) (CONTRIB_RAPWEARN_YEAR(j),j=1,CONTRIB_PAR_L)
        ELSE IF (CONTRIB_PAR_NAMES(i)=='end_parameters') THEN
            EXIT
        END IF
        i=i+1
    END DO
    DEALLOCATE(CONTRIB_PAR_NAMES)
    
    !close the parameter file
    CLOSE(UNIT=3)
    WRITE(*,*) 'Subroutine CONTRIB_PARAMETER_FILEREAD of NORMAL RUN...runned correctly'  

END SUBROUTINE CONTRIB_PARAMETER_FILEREAD

!============= SUBROUTINE INPUT_FILEREAD ================!
!This subroutines reads in data from INPUT file          !
!========================================================!

SUBROUTINE CONTRIB_INPUT_FILEREAD

    OPEN(UNIT=5, FILE=CONTRIB_INPUT, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_INPUT
        WRITE(15,'("Could not find ", a100)') CONTRIB_INPUT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(A14,I10,A7,I10,I10,I10,I10,A1,A1,I10,I10,A2,I10,I10,I10,F15.2,&
            &F15.2,F15.2,F15.2,F15.2,I10,I10,I10,I10,F15.2,I10,I10,I10,F15.10,I10,A1,I10,&
            &I10,I10,A2,I10,I10,F15.2,F15.2,F15.2,I10,F15.2,I10,I10,F15.10,A1,A7,A1,&
            &F15.2,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
            &NAREGNIS_SEX(pers_j),LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j),&
            &TAUXPT_20084(pers_j),T_PREST_20084(pers_j),SECEMP_20084(pers_j),&
            &CODTRA_20084(pers_j),CLATRA_20084(pers_j),CLATR2_20084(pers_j),&
            &CODASM_20084(pers_j),CATTRA_20084(pers_j),GROEP_VG(pers_j),&
            &SAL100_AGG(pers_j),PRIMES_AGG(pers_j),PREAVI_AGG(pers_j),&
            &SALATT_AGG(pers_j),SALFOR_AGG(pers_j),DEDUCTIONCODE_PERS_1(pers_j),&
            &DEDUCTIONCODE_PAT_1(pers_j),DEDUCTIONCODE_PAT_2(pers_j),DEDUCTIONCODE_PAT_3(pers_j),&
            &SUMSAL_2008(pers_j),RSZ_LABMSTAT_PRIV_2008(pers_j),RSZ_LABMSTAT_PUB_2008(pers_j),&
            &INRSZ(pers_j),RATE_FICTIF_RSZ(pers_j),APLTAUXPT_20084(pers_j),&
            &APL_T_PREST_20084(pers_j),APL_CATTRA_20084(pers_j),APL_CODTRA_20084(pers_j),&
            &CLATRAAPL_20084(pers_j),CLATR2APL_20084(pers_j),APL_GROEP_EV(pers_j),&
            &APL_GROEP_DV(pers_j),APL_SAL100_AGG(pers_j),APL_SALCOT_AGG(pers_j),&
            &APL_PRIMES_AGG(pers_j),APL_DEDUCTIONCODE_PERS_1(pers_j),APL_SUMSAL_2008(pers_j),&
            &RSZPPO_LABMSTAT_PUB_2008(pers_j),INRSZPPO(pers_j),RATE_FICTIF_RSZPPO(pers_j),&
            &HOEDRSVZ_2008(pers_j),BEGAANSC_2008(pers_j),BIJDCAT_2008(pers_j),&
            &INKOMENS_2008(pers_j),SELFSTAT_2008(pers_j),NOINCOMELINE_RSVZ(pers_j),&
            &BRON_AANV(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,CONTRIB_INPUT
            END IF
            !Read in numeric part of the individual indentification character variable
            READ (GECODEERD_INSZ_CHAR(pers_j)(8:), '(I10)') GECODEERD_INSZ_INPUT(pers_j)
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &NAREGNIS_SEX(pers_j),LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j),&
                &TAUXPT_20084(pers_j),T_PREST_20084(pers_j),SECEMP_20084(pers_j),&
                &CODTRA_20084(pers_j),CLATRA_20084(pers_j),CLATR2_20084(pers_j),&
                &CODASM_20084(pers_j),CATTRA_20084(pers_j),GROEP_VG(pers_j),&
                &SAL100_AGG(pers_j),PRIMES_AGG(pers_j),PREAVI_AGG(pers_j),&
                &SALATT_AGG(pers_j),SALFOR_AGG(pers_j),DEDUCTIONCODE_PERS_1(pers_j),&
                &DEDUCTIONCODE_PAT_1(pers_j),DEDUCTIONCODE_PAT_2(pers_j),DEDUCTIONCODE_PAT_3(pers_j),&
                &SUMSAL_2008(pers_j),RSZ_LABMSTAT_PRIV_2008(pers_j),RSZ_LABMSTAT_PUB_2008(pers_j),&
                &INRSZ(pers_j),RATE_FICTIF_RSZ(pers_j),APLTAUXPT_20084(pers_j),&
                &APL_T_PREST_20084(pers_j),APL_CATTRA_20084(pers_j),APL_CODTRA_20084(pers_j),&
                &CLATRAAPL_20084(pers_j),CLATR2APL_20084(pers_j),APL_GROEP_EV(pers_j),&
                &APL_GROEP_DV(pers_j),APL_SAL100_AGG(pers_j),APL_SALCOT_AGG(pers_j),&
                &APL_PRIMES_AGG(pers_j),APL_DEDUCTIONCODE_PERS_1(pers_j),APL_SUMSAL_2008(pers_j),&
                &RSZPPO_LABMSTAT_PUB_2008(pers_j),INRSZPPO(pers_j),RATE_FICTIF_RSZPPO(pers_j),&
                &HOEDRSVZ_2008(pers_j),BEGAANSC_2008(pers_j),BIJDCAT_2008(pers_j),&
                &INKOMENS_2008(pers_j),SELFSTAT_2008(pers_j),NOINCOMELINE_RSVZ(pers_j),&
                &BRON_AANV(pers_j)'')')
                WRITE(15,'(A14,I10,A7,I10,I10,I10,I10,A1,A1,I10,I10,A2,I10,I10,I10,F15.2,&
                &F15.2,F15.2,F15.2,F15.2,I10,I10,I10,I10,F15.2,I10,I10,I10,F15.10,I10,A1,I10,&
                &I10,I10,A2,I10,I10,F15.2,F15.2,F15.2,I10,F15.2,I10,I10,F15.10,A1,A7,A1,&
                &F15.2,I10,I10,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &NAREGNIS_SEX(pers_j),LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j),&
                &TAUXPT_20084(pers_j),T_PREST_20084(pers_j),SECEMP_20084(pers_j),&
                &CODTRA_20084(pers_j),CLATRA_20084(pers_j),CLATR2_20084(pers_j),&
                &CODASM_20084(pers_j),CATTRA_20084(pers_j),GROEP_VG(pers_j),&
                &SAL100_AGG(pers_j),PRIMES_AGG(pers_j),PREAVI_AGG(pers_j),&
                &SALATT_AGG(pers_j),SALFOR_AGG(pers_j),DEDUCTIONCODE_PERS_1(pers_j),&
                &DEDUCTIONCODE_PAT_1(pers_j),DEDUCTIONCODE_PAT_2(pers_j),DEDUCTIONCODE_PAT_3(pers_j),&
                &SUMSAL_2008(pers_j),RSZ_LABMSTAT_PRIV_2008(pers_j),RSZ_LABMSTAT_PUB_2008(pers_j),&
                &INRSZ(pers_j),RATE_FICTIF_RSZ(pers_j),APLTAUXPT_20084(pers_j),&
                &APL_T_PREST_20084(pers_j),APL_CATTRA_20084(pers_j),APL_CODTRA_20084(pers_j),&
                &CLATRAAPL_20084(pers_j),CLATR2APL_20084(pers_j),APL_GROEP_EV(pers_j),&
                &APL_GROEP_DV(pers_j),APL_SAL100_AGG(pers_j),APL_SALCOT_AGG(pers_j),&
                &APL_PRIMES_AGG(pers_j),APL_DEDUCTIONCODE_PERS_1(pers_j),APL_SUMSAL_2008(pers_j),&
                &RSZPPO_LABMSTAT_PUB_2008(pers_j),INRSZPPO(pers_j),RATE_FICTIF_RSZPPO(pers_j),&
                &HOEDRSVZ_2008(pers_j),BEGAANSC_2008(pers_j),BIJDCAT_2008(pers_j),&
                &INKOMENS_2008(pers_j),SELFSTAT_2008(pers_j),NOINCOMELINE_RSVZ(pers_j),&
                &BRON_AANV(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(A14,I10,A7,I10,I10,I10,I10,A1,A1,I10,I10,A2,I10,I10,I10,F15.2,&
                &F15.2,F15.2,F15.2,F15.2,I10,I10,I10,I10,F15.2,I10,I10,I10,F15.10,I10,A1,I10,&
                &I10,I10,A2,I10,I10,F15.2,F15.2,F15.2,I10,F15.2,I10,I10,F15.10,A1,A7,A1,&
                &F15.2,I10,I10,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &NAREGNIS_SEX(pers_j),LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j),&
                &TAUXPT_20084(pers_j),T_PREST_20084(pers_j),SECEMP_20084(pers_j),&
                &CODTRA_20084(pers_j),CLATRA_20084(pers_j),CLATR2_20084(pers_j),&
                &CODASM_20084(pers_j),CATTRA_20084(pers_j),GROEP_VG(pers_j),&
                &SAL100_AGG(pers_j),PRIMES_AGG(pers_j),PREAVI_AGG(pers_j),&
                &SALATT_AGG(pers_j),SALFOR_AGG(pers_j),DEDUCTIONCODE_PERS_1(pers_j),&
                &DEDUCTIONCODE_PAT_1(pers_j),DEDUCTIONCODE_PAT_2(pers_j),DEDUCTIONCODE_PAT_3(pers_j),&
                &SUMSAL_2008(pers_j),RSZ_LABMSTAT_PRIV_2008(pers_j),RSZ_LABMSTAT_PUB_2008(pers_j),&
                &INRSZ(pers_j),RATE_FICTIF_RSZ(pers_j),APLTAUXPT_20084(pers_j),&
                &APL_T_PREST_20084(pers_j),APL_CATTRA_20084(pers_j),APL_CODTRA_20084(pers_j),&
                &CLATRAAPL_20084(pers_j),CLATR2APL_20084(pers_j),APL_GROEP_EV(pers_j),&
                &APL_GROEP_DV(pers_j),APL_SAL100_AGG(pers_j),APL_SALCOT_AGG(pers_j),&
                &APL_PRIMES_AGG(pers_j),APL_DEDUCTIONCODE_PERS_1(pers_j),APL_SUMSAL_2008(pers_j),&
                &RSZPPO_LABMSTAT_PUB_2008(pers_j),INRSZPPO(pers_j),RATE_FICTIF_RSZPPO(pers_j),&
                &HOEDRSVZ_2008(pers_j),BEGAANSC_2008(pers_j),BIJDCAT_2008(pers_j),&
                &INKOMENS_2008(pers_j),SELFSTAT_2008(pers_j),NOINCOMELINE_RSVZ(pers_j),&
                &BRON_AANV(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine CONTRIB_INPUT_FILEREAD of NORMAL RUN.......runned correctly'
    WRITE(15,'(I7, " records uploaded from INPUT_FILE")') pers_j-1

END SUBROUTINE CONTRIB_INPUT_FILEREAD

!============= SUBROUTINE CONSTRUCT_FILEREAD ================!
!This subroutines reads in data from the CONSTRUCT file      !
!============================================================!
SUBROUTINE CONTRIB_CONSTRUCT_FILEREAD

    OPEN(UNIT=5, FILE=CONTRIB_CONSTRUCT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_CONSTRUCT
        WRITE(15,'("Could not find ", a100)') CONTRIB_CONSTRUCT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,I10,F15.10,I10,I10,I10,F15.10,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CONSTRUCT(pers_j),MIMOSIS_FLEXWEAR(pers_j),MIMOSIS_HOUR(pers_j),&
            &MIMOSIS_NUMHHMEM(pers_j),MIMOSIS_SIMNUMMT(pers_j),MIMOSIS_SIMSAMMT(pers_j),&
            &MIMOSIS_HOURPRED(pers_j),MIMOSIS_WAGEPRED(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,CONTRIB_CONSTRUCT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CONSTRUCT(pers_j),MIMOSIS_FLEXWEAR(pers_j),MIMOSIS_HOUR(pers_j),&
                &MIMOSIS_NUMHHMEM(pers_j),MIMOSIS_SIMNUMMT(pers_j),MIMOSIS_SIMSAMMT(pers_j),&
                &MIMOSIS_HOURPRED(pers_j),MIMOSIS_WAGEPRED(pers_j)'')')
                WRITE(15,'(7x,I7,I10,F15.10,I10,I10,I10,F15.10,F15.2)') &
                &GECODEERD_INSZ_CONSTRUCT(pers_j),MIMOSIS_FLEXWEAR(pers_j),MIMOSIS_HOUR(pers_j),&
                &MIMOSIS_NUMHHMEM(pers_j),MIMOSIS_SIMNUMMT(pers_j),MIMOSIS_SIMSAMMT(pers_j),&
                &MIMOSIS_HOURPRED(pers_j),MIMOSIS_WAGEPRED(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,F15.10,I10,I10,I10,F15.10,F15.2)') &
                &GECODEERD_INSZ_CONSTRUCT(pers_j),MIMOSIS_FLEXWEAR(pers_j),MIMOSIS_HOUR(pers_j),&
                &MIMOSIS_NUMHHMEM(pers_j),MIMOSIS_SIMNUMMT(pers_j),MIMOSIS_SIMSAMMT(pers_j),&
                &MIMOSIS_HOURPRED(pers_j),MIMOSIS_WAGEPRED(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine CONTRIB_CONSTRUCT_FILEREAD of NORMAL RUN...runned correctly'
    WRITE(15,'(I7, " records uploaded from CONSTRUCT_FILE")') pers_j-1

END SUBROUTINE CONTRIB_CONSTRUCT_FILEREAD

!============= SUBROUTINE PENSWELF_FILEREAD ================!
!This subroutines reads in data from the PENSWELF file      !
!===========================================================!
SUBROUTINE CONTRIB_PENSWELF_FILEREAD

    OPEN(UNIT=5, FILE=CONTRIB_PENSWELF, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_PENSWELF
        WRITE(15,'("Could not find ", a100)') CONTRIB_PENSWELF
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_RETPENS_QT(pers_j),PENSWELF_SURPENS_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,CONTRIB_PENSWELF
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_RETPENS_QT(pers_j),PENSWELF_SURPENS_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2)') &
                &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_RETPENS_QT(pers_j),PENSWELF_SURPENS_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2)') &
                &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_RETPENS_QT(pers_j),PENSWELF_SURPENS_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine CONTRIB_PENSWELF_FILEREAD of NORMAL RUN....runned correctly'
    WRITE(15,'(I7, " records uploaded from PENSWELF_FILE")') pers_j-1

END SUBROUTINE CONTRIB_PENSWELF_FILEREAD

!============= SUBROUTINE UNEM_FILEREAD ================!
!This subroutines reads in data from the UNEM file      !
!=======================================================!
SUBROUTINE CONTRIB_UNEM_FILEREAD

    OPEN(UNIT=5, FILE=CONTRIB_UNEM, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_UNEM
        WRITE(15,'("Could not find ", a100)') CONTRIB_UNEM
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUNS1_QT(pers_j),UNEM_ADDUNS_QT(pers_j),&
            &UNEM_BENEMP1_QT(pers_j),UNEM_ADDEMP_QT(pers_j),UNEM_OTHERUN_QT(pers_j),UNEM_OLDUNSS_QT(pers_j),&
            &UNEM_STATUS_QT(pers_j),UNEM_NUMMINU_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,CONTRIB_UNEM
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUNS1_QT(pers_j),UNEM_ADDUNS_QT(pers_j),&
                &UNEM_BENEMP1_QT(pers_j),UNEM_ADDEMP_QT(pers_j),UNEM_OTHERUN_QT(pers_j),UNEM_OLDUNSS_QT(pers_j),&
                &UNEM_STATUS_QT(pers_j),UNEM_NUMMINU_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,I10,I10)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUNS1_QT(pers_j),UNEM_ADDUNS_QT(pers_j),&
                &UNEM_BENEMP1_QT(pers_j),UNEM_ADDEMP_QT(pers_j),UNEM_OTHERUN_QT(pers_j),UNEM_OLDUNSS_QT(pers_j),&
                &UNEM_STATUS_QT(pers_j),UNEM_NUMMINU_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,I10,I10)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUNS1_QT(pers_j),UNEM_ADDUNS_QT(pers_j),&
                &UNEM_BENEMP1_QT(pers_j),UNEM_ADDEMP_QT(pers_j),UNEM_OTHERUN_QT(pers_j),UNEM_OLDUNSS_QT(pers_j),&
                &UNEM_STATUS_QT(pers_j),UNEM_NUMMINU_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine CONTRIB_UNEM_FILEREAD of NORMAL RUN........runned correctly'
    WRITE(15,'(I7, " records uploaded from UNEM_FILE")') pers_j-1

END SUBROUTINE CONTRIB_UNEM_FILEREAD

!============= SUBROUTINE FAMAL_FILEREAD ================!
!This subroutines reads in data from the FAMAL file      !
!========================================================!
SUBROUTINE CONTRIB_FAMAL_FILEREAD

    OPEN(UNIT=5, FILE=CONTRIB_FAMAL, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_FAMAL
        WRITE(15,'("Could not find ", a100)') CONTRIB_FAMAL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMAL(pers_j),FAMAL_AMOUNTF_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,CONTRIB_FAMAL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMAL(pers_j),FAMAL_AMOUNTF_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_FAMAL(pers_j),FAMAL_AMOUNTF_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_FAMAL(pers_j),FAMAL_AMOUNTF_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine CONTRIB_FAMAL_FILEREAD of NORMAL RUN.......runned correctly'
    WRITE(15,'(I7, " records uploaded from FAMAL_FILE")') pers_j-1

END SUBROUTINE CONTRIB_FAMAL_FILEREAD

!============= SUBROUTINE SICK_FILEREAD ================!
!This subroutines reads in data from the SICK file      !
!=======================================================!
SUBROUTINE CONTRIB_SICK_FILEREAD

    OPEN(UNIT=5, FILE=CONTRIB_SICK, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_SICK
        WRITE(15,'("Could not find ", a100)') CONTRIB_SICK
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,,F15.2,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_SICK(pers_j),SICK_IND_QT(pers_j),SICK_OCC_QT(pers_j),&
            &SICK_ALOTHBEN_QT(pers_j),SICK_DISABWEA_QT(pers_j),SICK_EMPLBPRI_QT(pers_j),&
            &SICK_EMPLBPUB_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,CONTRIB_SICK
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_SICK(pers_j),SICK_IND_QT(pers_j),SICK_OCC_QT(pers_j),&
                &SICK_ALOTHBEN_QT(pers_j),SICK_DISABWEA_QT(pers_j),SICK_EMPLBPRI_QT(pers_j),&
                &SICK_EMPLBPUB_QT(pers_j))'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,,F15.2,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_IND_QT(pers_j),SICK_OCC_QT(pers_j),&
                &SICK_ALOTHBEN_QT(pers_j),SICK_DISABWEA_QT(pers_j),SICK_EMPLBPRI_QT(pers_j),&
                &SICK_EMPLBPUB_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,,F15.2,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_IND_QT(pers_j),SICK_OCC_QT(pers_j),&
                &SICK_ALOTHBEN_QT(pers_j),SICK_DISABWEA_QT(pers_j),SICK_EMPLBPRI_QT(pers_j),&
                &SICK_EMPLBPUB_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine CONTRIB_SICK_FILEREAD of NORMAL RUN........runned correctly'
    WRITE(15,'(I7, " records uploaded from SICK_FILE")') pers_j-1

END SUBROUTINE CONTRIB_SICK_FILEREAD

!==================================!
!Reads in data from the PIT file   !
!==================================!

SUBROUTINE CONTRIB_PIT_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=CONTRIB_PIT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') CONTRIB_PIT
        WRITE (15,'("Could not find or open ", a100)') CONTRIB_PIT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,CONTRIB_PIT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)'')')
                WRITE(15,'(7x,I7,I10)') &
                &GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10)') &
                &GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine CONTRIB_PIT_FILEREAD of NORMAL RUN.........runned correctly'
    WRITE (15,'(I7, " records uploaded from PIT_FILE")') pers_j-1

END SUBROUTINE CONTRIB_PIT_FILEREAD

!=====================================!
!Reads in data from the FAMREL file   !
!=====================================!

SUBROUTINE CONTRIB_FAMREL_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=MIMOSIS_FAMREL, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') MIMOSIS_FAMREL
        WRITE (15,'("Could not find or open ", a100)') MIMOSIS_FAMREL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
            &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_FAMREL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine CONTRIB_FAMREL_FILEREAD of NORMAL RUN......runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_FAMREL")') pers_j-1

END SUBROUTINE CONTRIB_FAMREL_FILEREAD

!=======================================!
!Reads in data from the WAGE INPUT file !
!=======================================!

SUBROUTINE CONTRIB_WAGE_FILEREAD

    OPEN (UNIT=5, FILE=WAGE_CONSTRUCT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') WAGE_CONSTRUCT
        WRITE (15,'("Could not find or open ", a100)') WAGE_CONSTRUCT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
            &MIMOSIS_GRINC_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,WAGE_CONSTRUCT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine CONTRIB_WAGE_FILEREAD of NORMAL RUN........runned correctly'
    WRITE (15,'(I7, " records uploaded from WAGE_CONSTRUCT")') pers_j-1 

END SUBROUTINE CONTRIB_WAGE_FILEREAD

!===========================================!
!Reads in data from the MIMOSIS_WEIGHT file !
!===========================================!

SUBROUTINE CONTRIB_WEIGHT_FILEREAD

    OPEN (UNIT=5, FILE=MIMOSIS_SAMPWEIG, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') MIMOSIS_SAMPWEIG
        WRITE (15,'("Could not find or open ", a100)') MIMOSIS_SAMPWEIG
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_SAMPWEIG
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine CONTRIB_WEIGHT_FILEREAD of NORMAL RUN......runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_SAMPWEIG")') pers_j-1 

END SUBROUTINE CONTRIB_WEIGHT_FILEREAD

!=======================================================================!
!Check whether individual identification numbers are sorted consistently!
!=======================================================================!

SUBROUTINE CONTRIB_CONSISTENCY_CHECK

    INTEGER:: pers_j
    
    DO pers_j=1, n_rec
        IF (&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_CONSTRUCT(pers_j))     .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))          .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))         .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))          .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))           .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))        .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))          .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))        .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))  .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))      .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))     .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))      .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))       .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))      .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))    .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))       .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))      .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))       .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))        .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))     .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))       .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))     .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))          .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))           .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))            .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))         .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))           .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))         .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))          .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))           .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))        .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))          .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))        .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))            .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))         .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))           .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))         .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))          .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))            .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))          .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))         .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))       .OR.&
            &(GECODEERD_INSZ_WAGE(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))         &
            &) THEN
                WRITE (*,*) "Error: Ranking of external files does not match!"
                PAUSE
                STOP
        END IF
    END DO   
   
END SUBROUTINE CONTRIB_CONSISTENCY_CHECK 

!============= SUBROUTINE CONTRIB_RECODE=====================!
!This subroutines treat missing value                          !
!==============================================================!
SUBROUTINE CONTRIB_RECODE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        !Exogenous variables
        !CONTRIB_INPUT_WRITE.SAS
        !/*VARIABELEN UIT POPCHAR-BESTAND*/
        IF (INT(NAREGNIS_HH_ID(pers_j)).EQ.9999999)                 NAREGNIS_HH_ID(pers_j)=0
        CALL TRIM@(NAREGNIS_DATBIRTH(pers_j))
        IF (INT(NAREGNIS_SEX(pers_j)).EQ.9999999)                   NAREGNIS_SEX(pers_j)=0
        !/*HISTORINFO*/
        IF (INT(LABMSTAT_PRIV(pers_j)).EQ.9999999)                  LABMSTAT_PRIV(pers_j)=0
        IF (INT(LABMSTAT_PUB(pers_j)).EQ.9999999)                   LABMSTAT_PUB(pers_j)=0
        !/*VARIABELEN UIT RSZ-BESTAND*/
        IF (INT(TAUXPT_20084(pers_j)).EQ.9999999)                   TAUXPT_20084(pers_j)=0
        CALL TRIM@(T_PREST_20084(pers_j))
        CALL TRIM@(SECEMP_20084(pers_j))
        IF (INT(CODTRA_20084(pers_j)).EQ.9999999)                   CODTRA_20084(pers_j)=0
        IF (INT(CLATRA_20084(pers_j)).EQ.9999999)                   CLATRA_20084(pers_j)=0
        CALL TRIM@(CLATR2_20084(pers_j))
        IF (INT(CODASM_20084(pers_j)).EQ.9999999)                   CODASM_20084(pers_j)=0
        IF (INT(CATTRA_20084(pers_j)).EQ.9999999)                   CATTRA_20084(pers_j)=0
        IF (INT(GROEP_VG(pers_j)).EQ.9999999)                       GROEP_VG(pers_j)=0
        IF (INT(SAL100_AGG(pers_j)).EQ.9999999)                     SAL100_AGG(pers_j)=0
        IF (INT(PRIMES_AGG(pers_j)).EQ.9999999)                     PRIMES_AGG(pers_j)=0
        IF (INT(PREAVI_AGG(pers_j)).EQ.9999999)                     PREAVI_AGG(pers_j)=0
        IF (INT(SALATT_AGG(pers_j)).EQ.9999999)                     SALATT_AGG(pers_j)=0
        IF (INT(SALFOR_AGG(pers_j)).EQ.9999999)                     SALFOR_AGG(pers_j)=0
        IF (INT(DEDUCTIONCODE_PERS_1(pers_j)).EQ.9999999)           DEDUCTIONCODE_PERS_1(pers_j)=0
        IF (INT(DEDUCTIONCODE_PAT_1(pers_j)).EQ.9999999)            DEDUCTIONCODE_PAT_1(pers_j)=0
        IF (INT(DEDUCTIONCODE_PAT_2(pers_j)).EQ.9999999)            DEDUCTIONCODE_PAT_2(pers_j)=0
        IF (INT(DEDUCTIONCODE_PAT_3(pers_j)).EQ.9999999)            DEDUCTIONCODE_PAT_3(pers_j)=0
        IF (INT(SUMSAL_2008(pers_j)).EQ.9999999)                    SUMSAL_2008(pers_j)=0
        IF (INT(RSZ_LABMSTAT_PRIV_2008(pers_j)).EQ.9999999)         RSZ_LABMSTAT_PRIV_2008(pers_j)=0
        IF (INT(RSZ_LABMSTAT_PUB_2008(pers_j)).EQ.9999999)          RSZ_LABMSTAT_PUB_2008(pers_j)=0
        IF (INT(INRSZ(pers_j)).EQ.9999999)                          INRSZ(pers_j)=0
        IF (INT(RATE_FICTIF_RSZ(pers_j)).EQ.9999999)                RATE_FICTIF_RSZ(pers_j)=0
        !/*VARIABELEN UIT RSZPPO-BESTAND*/
        IF (INT(APLTAUXPT_20084(pers_j)).EQ.9999999)                APLTAUXPT_20084(pers_j)=0
        CALL TRIM@(APL_T_PREST_20084(pers_j))
        IF (INT(APL_CATTRA_20084(pers_j)).EQ.9999999)               APL_CATTRA_20084(pers_j)=0
        IF (INT(APL_CODTRA_20084(pers_j)).EQ.9999999)               APL_CODTRA_20084(pers_j)=0
        IF (INT(CLATRAAPL_20084(pers_j)).EQ.9999999)                CLATRAAPL_20084(pers_j)=0
        CALL TRIM@(CLATR2APL_20084(pers_j))
        IF (INT(APL_GROEP_EV(pers_j)).EQ.9999999)                   APL_GROEP_EV(pers_j)=0
        IF (INT(APL_GROEP_DV(pers_j)).EQ.9999999)                   APL_GROEP_DV(pers_j)=0
        IF (INT(APL_SAL100_AGG(pers_j)).EQ.9999999)                 APL_SAL100_AGG(pers_j)=0
        IF (INT(APL_SALCOT_AGG(pers_j)).EQ.9999999)                 APL_SALCOT_AGG(pers_j)=0
        IF (INT(APL_PRIMES_AGG(pers_j)).EQ.9999999)                 APL_PRIMES_AGG(pers_j)=0
        IF (INT(APL_DEDUCTIONCODE_PERS_1(pers_j)).EQ.9999999)       APL_DEDUCTIONCODE_PERS_1(pers_j)=0
        IF (INT(APL_SUMSAL_2008(pers_j)).EQ.9999999)                APL_SUMSAL_2008(pers_j)=0
        IF (INT(RSZPPO_LABMSTAT_PUB_2008(pers_j)).EQ.9999999)       RSZPPO_LABMSTAT_PUB_2008(pers_j)=0
        IF (INT(INRSZPPO(pers_j)).EQ.9999999)                       INRSZPPO(pers_j)=0
        IF (INT(RATE_FICTIF_RSZPPO(pers_j)).EQ.9999999)             RATE_FICTIF_RSZPPO(pers_j)=0
        !/*VARIABELEN UIT RSVZ-BESTAND*/
        CALL TRIM@(HOEDRSVZ_2008(pers_j))
        CALL TRIM@(BEGAANSC_2008(pers_j))
        CALL TRIM@(BIJDCAT_2008(pers_j))
        IF (INT(INKOMENS_2008(pers_j)).EQ.9999999)                  INKOMENS_2008(pers_j)=0
        IF (INT(SELFSTAT_2008(pers_j)).EQ.9999999)                  SELFSTAT_2008(pers_j)=0
        IF (INT(NOINCOMELINE_RSVZ(pers_j)).EQ.9999999)              NOINCOMELINE_RSVZ(pers_j)=0
        IF (INT(BRON_AANV(pers_j)).EQ.9999999)                      BRON_AANV(pers_j)=0
        !CONTRIB_CONSTRUCT_WRITE.SAS
        IF (INT(MIMOSIS_FLEXWEAR(pers_j)).EQ.9999999)               MIMOSIS_FLEXWEAR(pers_j)=0
        IF (INT(MIMOSIS_HOUR(pers_j)).EQ.9999999)                   MIMOSIS_HOUR(pers_j)=0
        IF (INT(MIMOSIS_NUMHHMEM(pers_j)).EQ.9999999)               MIMOSIS_NUMHHMEM(pers_j)=0
        IF (INT(MIMOSIS_SIMNUMMT(pers_j)).EQ.9999999)               MIMOSIS_SIMNUMMT(pers_j)=0
        IF (INT(MIMOSIS_SIMSAMMT(pers_j)).EQ.9999999)               MIMOSIS_SIMSAMMT(pers_j)=0
        IF (INT(MIMOSIS_HOURPRED(pers_j)).EQ.9999999)               MIMOSIS_HOURPRED(pers_j)=0
        IF (INT(MIMOSIS_WAGEPRED(pers_j)).EQ.9999999)               MIMOSIS_WAGEPRED(pers_j)=0
        !Variables read from file CONTRIB_PENSWELF.txt                                                       
        IF (INT(PENSWELF_RETPENS_QT(pers_j)).EQ.9999999)            PENSWELF_RETPENS_QT(pers_j)=0
        IF (INT(PENSWELF_SURPENS_QT(pers_j)).EQ.9999999)            PENSWELF_SURPENS_QT(pers_j)=0
        !Variables read from file CONTRIB_UNEM.txt                                                       
        IF (INT(UNEM_BENUNS1_QT(pers_j)).EQ.9999999)                UNEM_BENUNS1_QT(pers_j)=0
        IF (INT(UNEM_ADDUNS_QT(pers_j)).EQ.9999999)                 UNEM_ADDUNS_QT(pers_j)=0
        IF (INT(UNEM_BENEMP1_QT(pers_j)).EQ.9999999)                UNEM_BENEMP1_QT(pers_j)=0
        IF (INT(UNEM_ADDEMP_QT(pers_j)).EQ.9999999)                 UNEM_ADDEMP_QT(pers_j)=0
        IF (INT(UNEM_OTHERUN_QT(pers_j)).EQ.9999999)                UNEM_OTHERUN_QT(pers_j)=0
        IF (INT(UNEM_OLDUNSS_QT(pers_j)).EQ.9999999)                UNEM_OLDUNSS_QT(pers_j)=0
        IF (INT(UNEM_STATUS_QT(pers_j)).EQ.9999999)                 UNEM_STATUS_QT(pers_j)=0
        IF (INT(UNEM_NUMMINU_QT(pers_j)).EQ.9999999)                UNEM_NUMMINU_QT(pers_j)=0
        !Variables read from file CONTRIB_FAMAL.txt                                                      
        IF (INT(FAMAL_AMOUNTF_QT(pers_j)).EQ.9999999)               FAMAL_AMOUNTF_QT(pers_j)=0
        !Variables read from file CONTRIB_SICK.txt                                                       
        IF (INT(SICK_IND_QT(pers_j)).EQ.9999999)                    SICK_IND_QT(pers_j)=0
        IF (INT(SICK_OCC_QT(pers_j)).EQ.9999999)                    SICK_OCC_QT(pers_j)=0
        IF (INT(SICK_ALOTHBEN_QT(pers_j)).EQ.9999999)               SICK_ALOTHBEN_QT(pers_j)=0
        IF (INT(SICK_DISABWEA_QT(pers_j)).EQ.9999999)               SICK_DISABWEA_QT(pers_j)=0
        !Variables read from file CONTRIB_PIT.txt                                                        
        !Variables read from file MIMOSIS_FAMREL.txt                                           
        !WAGE_CONSTRUCT_WRITE.SAS
        IF (INT(SALJRS_LAST_CORR(pers_j)).EQ.9999999)               SALJRS_LAST_CORR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_HOUR(pers_j)).EQ.9999999)             MIMOSIS_GRINC_HOUR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_QT(pers_j)).EQ.9999999)               MIMOSIS_GRINC_QT(pers_j)=0
        !MIMOSIS_SAMPWEIG_WRITE.SAS
        IF (INT(MIMOSIS_WEIGHT(pers_j)).EQ.9999999)                 MIMOSIS_WEIGHT(pers_j)=0
    END DO   

END SUBROUTINE CONTRIB_RECODE

!============= SUBROUTINE CONTRIB_INFLATE===============================!
!This subroutines inflates nominal values to an appropriate price level !
!=======================================================================!
SUBROUTINE CONTRIB_INFLATE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        !Exogenous variables
        !Variables read from file CONTRIB_INPUT.txt                                            
        sal100_agg(pers_j)                 = INFLATE_NOMVAL(1)*sal100_agg(pers_j)                            ! loon 100% (aggregaat op kwartaalbasis)
        primes_agg(pers_j)                 = INFLATE_NOMVAL(1)*primes_agg(pers_j)                            ! premies (aggregaat op kwartaalbasis)
        preavi_agg(pers_j)                 = INFLATE_NOMVAL(1)*preavi_agg(pers_j)                            ! vooropzeg (aggregaat op kwartaalbasis)
        salatt_agg(pers_j)                 = INFLATE_NOMVAL(1)*salatt_agg(pers_j)                            ! wachtloon (aggregaat op kwartaalbasis)
        salfor_agg(pers_j)                 = INFLATE_NOMVAL(1)*salfor_agg(pers_j)                            ! forfaitair loon (aggregaat op kwartaalbasis)
        APL_sal100_agg(pers_j)             = INFLATE_NOMVAL(1)*APL_sal100_agg(pers_j)                        ! Bedrag van de gewone bezoldiging in het kwartaal. (aggregaat op kwartaalbasis)
        APL_salcot_agg(pers_j)             = INFLATE_NOMVAL(1)*APL_salcot_agg(pers_j)                        ! Bedrag van de bijdrageplichtige loonmassa inzake sociale zekerheid in het kwartaal (aggregaat op kwartaalbasis)
        APL_primes_agg(pers_j)             = INFLATE_NOMVAL(1)*APL_primes_agg(pers_j)                        ! Het bedrag van de bezoldigingen in het kwartaal die niet rechtstreeks verbonden zijn met de prestaties gedurende een bepaald kwartaal. (aggregaat op kwartaalbasis)
        inkomens_2008(pers_j)              = INFLATE_NOMVAL(1)*inkomens_2008(pers_j)                         ! Inkomen op jaarbasis na aanvulling van ontbrekende waarden en uitgedrukt in prijzen van 2008
        !Variables read from file CONTRIB_CONSTRUCT.txt                                                       
        MIMOSIS_WAGEPRED(pers_j)           = INFLATE_NOMVAL(1)*MIMOSIS_WAGEPRED(pers_j)                       !Income earned as self employed in the quarter of analysis 
        !Variables read from file CONTRIB_PENSWELF.txt                                                       
!        PENSWELF_RETPENS_QT(pers_j)        = INFLATE_NOMVAL(2)*PENSWELF_RETPENS_QT(pers_j)                   !Retirement pension received in a given quarter
!        PENSWELF_SURPENS_QT(pers_j)        = INFLATE_NOMVAL(2)*PENSWELF_SURPENS_QT(pers_j)                   !Survival pension received in a given quarter
        !Variables read from file CONTRIB_UNEM.txt                                                       
!        UNEM_BENUNS1_QT(pers_j)            = INFLATE_NOMVAL(3)*UNEM_BENUNS1_QT(pers_j)                       !Early retirement benefit paid by the RVA to unemployed not in search of work received in a given quarter
!        UNEM_ADDUNS_QT(pers_j)             = INFLATE_NOMVAL(3)*UNEM_ADDUNS_QT(pers_j)                        !Early retirement benefit paid by the employer to unemployed not in search of work received in a given quarter
!        UNEM_BENEMP1_QT(pers_j)            = INFLATE_NOMVAL(3)*UNEM_BENEMP1_QT(pers_j)                       !Early retirement benefit paid by the RVA to a part time employee received in a given quarter
!        UNEM_ADDEMP_QT(pers_j)             = INFLATE_NOMVAL(3)*UNEM_ADDEMP_QT(pers_j)                        !Early retirement benefit paid by the employer to a part time employee received in a given quarter
!        UNEM_OTHERUN_QT(pers_j)            = INFLATE_NOMVAL(3)*UNEM_OTHERUN_QT(pers_j)                       !Unemployment benefits, other than early retirement benefits and benefits for older unemployed with seniority supplement received in a given quarter
!        UNEM_OLDUNSS_QT(pers_j)            = INFLATE_NOMVAL(3)*UNEM_OLDUNSS_QT(pers_j)                       !Unemployment benefits for older unemployed with seniority supplement paid in a given quarter
        !Variables read from file CONTRIB_FAMAL.txt                                                      
!        FAMAL_AMOUNTF_QT(pers_j)           = INFLATE_NOMVAL(3)*FAMAL_AMOUNTF_QT(pers_j)                      !Total amount of family allowances received in a given quarter by the recipient
        !Variables read from file CONTRIB_SICK.txt                                                       
!        SICK_IND_QT(pers_j)                = INFLATE_NOMVAL(3)*SICK_IND_QT(pers_j)                           !Disability benefits received by wage earners because of industrial accidents (benefits for help of a third person excluded)
!        SICK_OCC_QT(pers_j)                = INFLATE_NOMVAL(3)*SICK_OCC_QT(pers_j)                           !Disability benefits received by wage earners because of occupational diseases (benefits for help of a third person excluded)
!        SICK_DISABWEA_QT(pers_j)           = INFLATE_NOMVAL(3)*SICK_DISABWEA_QT(pers_j)                      !Disability benefits received by wage earners on the private or public labour market
!        SICK_ALOTHBEN_QT(pers_j)           = INFLATE_NOMVAL(3)*SICK_ALOTHBEN_QT(pers_j)                      !Benefits in case of sickness or disability other than those covered by SICK_IND_QT, SICK_OCC_QT and SICK_DISABWEA_QT
        !Variables read from file CONTRIB_PIT.txt                                                        
        !Variables read from file MIMOSIS_FAMREL.txt                                           
        !Variables read from file WAGE_CONSTRUCT.txt                                                     
        SALJRS_LAST_CORR(pers_j)           = INFLATE_NOMVAL(1)*SALJRS_LAST_CORR(pers_j)                       !Estimate of last observed income per day
        MIMOSIS_GRINC_HOUR(pers_j)         = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_HOUR(pers_j)                     !Estimate of last observed income per hour (Based on CIMIRe, DWH and minimum-wage)
        MIMOSIS_GRINC_QT(pers_j)           = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_QT(pers_j)                       !Estimate of last observed income per quarter (Based on CIMIRe, DWH and minimum-wage)
        !Variables read from file MIMOSIS_WEIGHT.txt                                                    
    END DO   

END SUBROUTINE CONTRIB_INFLATE

!===============================================!
! HOUSEHOLD_SIZE computes household size        !
! and identifies starting and ending record of  !
! sociological household and stores it in a     !
! vector with household information             !
!Constructs variables                           !
!   CONTRIB_HH_SIZE                             !
!   CONTRIB_START_HH                            !
!   CONTRIB_END_HH                              !
!   N_HH_CONTRIB                                !
!===============================================!

SUBROUTINE CONTRIB_HOUSEHOLD_SIZE

    INTEGER::  I
    INTEGER::  J
    INTEGER::  K
    INTEGER::  N

    I=1
    J=1
    K=0
    N=0

    DO
        J=J+N
        N=0
        K=K+1
        DO WHILE (NAREGNIS_HH_ID(i)==NAREGNIS_HH_ID(j))
            N=N+1
            I=I+1
            IF (i>n_rec) EXIT    
        END DO
        CONTRIB_HH_SIZE(k)=n
        CONTRIB_START_HH(k)=j
        CONTRIB_END_HH(k)=j+n-1
        IF (i>n_rec.OR.j>n_rec)EXIT
    END DO
    N_HH_CONTRIB=K

END SUBROUTINE CONTRIB_HOUSEHOLD_SIZE

!=================================================================
!Computes the age of each individual in terms of years and months!
!=================================================================

!============================================!
! Subroutine main input : NAREGNIS_DATBIRTH  ! 
! Subroutine main output: CONTRIB_YEAR_AGE   !
!                         CONTRIB_MONTH_AGE  !
!                         CONTRIB_YEAR_BIRTH !
!                         CONTRIB_MONTH_BIRTH!   
!============================================!

SUBROUTINE CONTRIB_AGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        CONTRIB_PAR_BREAK_POINT(1)=INDEX(NAREGNIS_DATBIRTH(pers_j),'-')
        READ (NAREGNIS_DATBIRTH(pers_j)(1:CONTRIB_PAR_BREAK_POINT(1)-1),  '(I10)') CONTRIB_MONTH_BIRTH(pers_j)
        READ (NAREGNIS_DATBIRTH(pers_j)(CONTRIB_PAR_BREAK_POINT(1)+1:),   '(I10)') CONTRIB_YEAR_BIRTH(pers_j)          
        !determine age as number of years at the end of the year for which data are registered
        CONTRIB_YEAR_AGE(pers_j)  =MIMOSIS_YEAR-CONTRIB_YEAR_BIRTH(pers_j)
        !determine number of months as year age plus number of months passed until the end of the year
        CONTRIB_MONTH_AGE(pers_j) =CONTRIB_YEAR_AGE(pers_j)*12+(12-CONTRIB_MONTH_BIRTH(pers_j))
    END DO

END SUBROUTINE CONTRIB_AGE

!===================================================================================!
!Draw a line of a give number of characters
!===================================================================================!

SUBROUTINE DRAWLINE(NUNIT,N,WRITECHAR)

    ! DRAWS A LINE OF N CHARACTERS OF TYPE WRITECHAR ON UNIT NUNIT
    INTEGER                                                 :: N                                    !Number of characters to be drawn
    INTEGER                                                 :: NUNIT                                    !Number of characters to be drawn
    CHARACTER(len=1)                                        :: WRITECHAR                            !Type of character that has to be reproduced N times
    CHARACTER(len=1),DIMENSION(N)                           :: A                                    !Vector of dimension N containing WRITECHAR in the N positions
    CHARACTER(len=15)                                       :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=7)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK

    A(1:N)=WRITECHAR
    !convert the integer value N into a character value NCHAR with the Fortran internal Write function                                                        
    WRITE (NCHAR, '(I7)') N
    !Construct the output format by aid of the internal Fortran function NONBLK which identifies the first non blank character
    !and read the remainder of the string, i.e. :
    OUTPUTFMT = '(' // NCHAR(NONBLK(NCHAR):) // 'A1)'
    WRITE(NUNIT,OUTPUTFMT)(A(I),I=1,N)

END SUBROUTINE DRAWLINE

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as a nominal amount that has to be selected from a 3 column matrix     !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The amount is in column 3 of this matrix.            !
!In this module to be applied with parameters                           !
!   CONTRIB_LSALLSE_QT                                                  !
!=======================================================================!

FUNCTION CONTRIB_LSAMOUNT_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,3)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the last but one bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !Set the lump sum amount
            OUTPUT_AMOUNT=MAT_BRACRATE(i,3)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 
    !in the last bracket: set the lump sum amount
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=MAT_BRACRATE(NLINES_BRACRATE,3)                                                       
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function CONTRIB_LSAMOUNT_3COLUMNS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION CONTRIB_LSAMOUNT_3COLUMNS

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as a result of different rates applied on an input amount.             !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The rate is in column 3 of this matrix.              !
!An example of this procedure is the social security contribution of    ! 
!self employed in principal activity before retirement age              !
!In this module to be applied with parameters                           !
!   CONTRIB_SELFPAA_QT                                                  !
!   CONTRIB_SELFARA_QT                                                  !
!   CONTRIB_SELFARB_QT                                                  !
!   CONTRIB_SELFASA_QT                                                  !
!=======================================================================!

FUNCTION CONTRIB_BRACRATE_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,3)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !apply the rate to the difference between income and lower limit (and add it up to the deductions computed in the previous brackets)
            OUTPUT_AMOUNT=OUTPUT_AMOUNT+(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        !otherwise the amount is higher than the upper bracket
        ELSE IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                                                                     
            !apply rate to the difference between upper and lower limit (and add it up to the amount computed with previous brackets) 
            OUTPUT_AMOUNT=OUTPUT_AMOUNT+(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)
            FUNCSELE = 1
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=OUTPUT_AMOUNT+(INPUT_AMOUNT-MAT_BRACRATE(NLINES_BRACRATE,1))*(MAT_BRACRATE(NLINES_BRACRATE,3)/100.D0)                                                       
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function CONTRIB_BRACRATE_3COLUMNS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION CONTRIB_BRACRATE_3COLUMNS

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as                                                                     !
!a) a fixed nominal amount MINUS                                        !
!b) the result of a rate applied on part of the input amount            !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The nominal amounts to which the rate part is to be  !
!added are in column 3 of the matrix, the rate is in column 4 of this   !
!matrix.                                                                !
!An example of this procedure is the computation of structural          !
!employer deduction in 2001                                             !
!In this module to be applied with parameters                           !
!   CONTRIB_SD1_QT                                                      !
!   CONTRIB_SD2_QT                                                      !
!   CONTRIB_SD3_QT                                                      !
!   CONTRIB_SD4_QT                                                      !
!   CONTRIB_PDEDBL_MONTH                                                !
!   CONTRIB_PDEDWI_MONTH                                                !
!=======================================================================!

FUNCTION CONTRIB_BRACRATE_4COLUMNS_MIN(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,4)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !apply the rate to the difference between income and lower limit (and add it up to the deductions computed in the previous brackets)
            OUTPUT_AMOUNT=MAT_BRACRATE(i,3)-(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,4)/100.D0)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=MAT_BRACRATE(NLINES_BRACRATE,3)-(INPUT_AMOUNT-MAT_BRACRATE(NLINES_BRACRATE,1))*&
        &(MAT_BRACRATE(NLINES_BRACRATE,4)/100.D0)
        FUNCSELE = 1
    END IF
    
!    WRITE(15,*) 'FUNCWB',INDIV,INPUT_AMOUNT,OUTPUT_AMOUNT 
    
    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function CONTRIB_BRACRATE_4COLUMNS_MIN has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION CONTRIB_BRACRATE_4COLUMNS_MIN

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as                                                                     !
!a) a fixed nominal amount MINUS                                        !
!b) the result of a rate applied on part of the input amount            !
!the input amount is computed as a value minus a fixed value            !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The nominal amounts to which the rate part is to be  !
!added are in column 3 of the matrix, the rate is in column 4 of this   !
!matrix.                                                                !
!An example of this procedure is the computation of the workbonus from  !
!2009 on                                                                !
!In this module to be applied with parameters                           !
!   CONTRIB_PDEDBL_MONTH                                                !
!   CONTRIB_PDEDWI_MONTH                                                !
!=======================================================================!

FUNCTION CONTRIB_BRACRATE_4COLUMNS_WB2009(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,4)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !apply the rate to the difference between income and lower limit (and add it up to the deductions computed in the previous brackets)
            OUTPUT_AMOUNT=MAT_BRACRATE(i,3)-(INPUT_AMOUNT-MAT_BRACRATE(1,2))*(MAT_BRACRATE(i,4)/100.D0)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=MAT_BRACRATE(NLINES_BRACRATE,3)-(INPUT_AMOUNT-MAT_BRACRATE(1,2))*&
        &(MAT_BRACRATE(NLINES_BRACRATE,4)/100.D0)
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function CONTRIB_BRACRATE_4COLUMNS_MIN has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION CONTRIB_BRACRATE_4COLUMNS_WB2009

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as                                                                     !
!a) a fixed nominal amount PLUS                                         !
!b) the result of a rate applied on part of the input amount            !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The nominal amounts to which the rate part is to be  !
!added are in column 3 of the matrix, the lower ceiling necessary to    !
!determine the amount on which the rate should be applied is in column 4!
!and the rate is in column 5 of this matrix.                            !                                   !
!An example of this procedure is the computation of the solidarity      !
!contribution on pensions in 2001                                       !
!In this module to be applied with parameters                           !
!   CONTRIB_PELIMNOC_MONTH                                              !
!   CONTRIB_PELIMCHI_MONTH                                              !
!=======================================================================!

FUNCTION CONTRIB_BRACRATE_5COLUMNS_PLUS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,5)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !apply the rate to the difference between income and lower limit (and add it up to the deductions computed in the previous brackets)
            OUTPUT_AMOUNT=MAT_BRACRATE(i,3)+(INPUT_AMOUNT-MAT_BRACRATE(i,4))*(MAT_BRACRATE(i,5)/100.D0)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=MAT_BRACRATE(NLINES_BRACRATE,3)+(INPUT_AMOUNT-MAT_BRACRATE(NLINES_BRACRATE,4))*&
        &(MAT_BRACRATE(NLINES_BRACRATE,5)/100.D0)
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function CONTRIB_BRACRATE_5COLUMNS_PLUS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION CONTRIB_BRACRATE_5COLUMNS_PLUS

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as                                                                     !
!a) a fixed nominal amount PLUS                                         !
!b) the result of a rate applied on part of the input amount            !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!An example of this procedure is the computation of the structural      !
!deduction as applied from 2005 on                                      !
!In this module to be applied with parameters                           !
!   CONTRIB_STRU2008_QT                                                 !
!=======================================================================!

FUNCTION CONTRIB_BRACRATE_5COLUMNS_MIN(INPUT_AMOUNTA,INPUT_AMOUNTB,NLINES_BRACRATE,MAT_BRACRATE,LINENUM,INDIV) &
&RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNTA                            !Amount on which the rates will be applied (full time equivalent)
    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNTB                            !Amount on which the rates will be applied (basic observation)
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,5)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: LINENUM                                  !Line that has to be used for the following computation
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !Set the fixed value
    OUTPUT_AMOUNT=MAT_BRACRATE(LINENUM,1)
    !Add the structural deduction for low income
    IF (INT(100.D0*INPUT_AMOUNTA).LT.INT(100.D0*MAT_BRACRATE(LINENUM,3)).AND.&
       &INT(100.D0*INPUT_AMOUNTA).GT.0.D0) THEN
        OUTPUT_AMOUNT=OUTPUT_AMOUNT+(MAT_BRACRATE(LINENUM,2)/100.D0)*(MAT_BRACRATE(LINENUM,3)-INPUT_AMOUNTA)
    END IF
    !Add the structural deduction for high income
    IF (INT(100.D0*INPUT_AMOUNTB).GT.INT(100.D0*MAT_BRACRATE(LINENUM,5))) THEN
        OUTPUT_AMOUNT=OUTPUT_AMOUNT+(MAT_BRACRATE(LINENUM,4)/100.D0)*(INPUT_AMOUNTB-MAT_BRACRATE(LINENUM,5))
    END IF

END FUNCTION CONTRIB_BRACRATE_5COLUMNS_MIN

!===================================================================================!
!Define a function that selects a percentage conditional on two status variables    !
!a) the first variable is a fixed status variable (value in column 1 of the matrix) !
!b) the value of the second variable should be greater or equal to the value in     !
!column 2 of the matrix and smaller or equal to the value in column 3 of the matrix !
!The percentage to be selected is in column 4 of the matrix.                        !
!This function is used f.e. to select the percentage to compute the                 !
!deduction in case of the plusplan                                                  !
!In this module to be applied with parameters                                       !
!   CONTRIB_PLUSDEDU_QT                                                             !
!===================================================================================!

FUNCTION CONTRIB_SELPERC_4COLUMNS_2STATINM(STATUS1,STATUS2,NLINES_PERCENT,MAT_PERCENT,INDIV) RESULT (OUTPUT_PERCENT)

    INTEGER                                             :: STATUS1                             !Value of status variable 1
    INTEGER                                             :: STATUS2                             !Value of status variable 2 (f.e. number of months in status)
    REAL(KIND=KIND_REAL)                                :: OUTPUT_PERCENT                      !Percentage produced as output of this function 
    INTEGER                                             :: NLINES_PERCENT                      !Number of lines in matrix
    INTEGER                                             :: i                                   !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_PERCENT,4)   :: MAT_PERCENT                         !Matrix with values of status variables and percentage to be selected
    INTEGER                                             :: INDIV                               !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                            !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_PERCENT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the last bracket
    DO i=1,NLINES_PERCENT                                                                        
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (STATUS1.EQ.INT(MAT_PERCENT(i,1)).AND.STATUS2.GE.INT(MAT_PERCENT(i,2)).AND.&
        &STATUS2.LE.INT(MAT_PERCENT(i,3))) THEN
            OUTPUT_PERCENT=MAT_PERCENT(i,4)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function CONTRIB_SELPERC_4COLUMNS_2STATINM has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION CONTRIB_SELPERC_4COLUMNS_2STATINM

!===================================================================================!
!Define a function that selects a percentage conditional on two status variables    !
!a) the first variable is a fixed status variable that is NOT in the matrix         !
!b) the value of the second variable should be greater or equal to the value in     !
!column 1 of the matrix and smaller or equal to the value in column 2 of the matrix !
!The percentages to be selected are in column 3 and 4 of the matrix.                !
!This function is used f.e. to select the percentage to compute the                 !
!deductions for professional career interruption                                    !            !
!In this module to be applied with parameters                                       !
!   CONTRIB_UNPROFDP_QT                                                             !
!   CONTRIB_UNPROFDF_QT                                                             !
!   CONTRIB_UNPRERDP_QT                                                             !
!   CONTRIB_UNPRERDF_QT                                                             !
!===================================================================================!

FUNCTION CONTRIB_SELPERC_4COLUMNS_1STATINM(STATUS1,STATUS2,NLINES_PERCENT,MAT_PERCENT,INDIV) RESULT (OUTPUT_PERCENT)

    INTEGER                                             :: STATUS1                             !Value of status variable 1
    INTEGER                                             :: STATUS2                             !Value of status variable 2 (f.e. number of months in status)
    REAL(KIND=KIND_REAL)                                :: OUTPUT_PERCENT                      !Percentage produced as output of this function 
    INTEGER                                             :: NLINES_PERCENT                      !Number of lines in matrix
    INTEGER                                             :: i                                   !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_PERCENT,4)   :: MAT_PERCENT                         !Matrix with values of status variables and percentage to be selected
    INTEGER                                             :: INDIV                               !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                            !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_PERCENT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the last bracket
    DO i=1,NLINES_PERCENT                                                                        
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (STATUS1.EQ.1.AND.STATUS2.GE.INT(MAT_PERCENT(i,1)).AND.&
        &STATUS2.LE.INT(MAT_PERCENT(i,2))) THEN
            OUTPUT_PERCENT=MAT_PERCENT(i,3)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        ELSE IF (STATUS1.EQ.2.AND.STATUS2.GE.INT(MAT_PERCENT(i,1)).AND.&
        &STATUS2.LE.INT(MAT_PERCENT(i,2))) THEN
            OUTPUT_PERCENT=MAT_PERCENT(i,4)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function CONTRIB_SELPERC_4COLUMNS_1STATINM has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION CONTRIB_SELPERC_4COLUMNS_1STATINM

!===================================================================================!
!Define a function that selects a percentage conditional on one status variables    !
!the value of the status variable should be greater or equal to the value in        !
!column 1 of the matrix and smaller or equal to the value in column 2 of the matrix !
!The percentage to be selected is in column 3 of the matrix.                        !
!This function is used f.e. to select the percentage to compute the                 !
!deduction in case of a four week deduction                                         !
!In this module to be applied with parameters                                       !
!   CONTRIB_4DAYDEDA_QT                                                             !
!===================================================================================!

FUNCTION CONTRIB_SELPERC_3COLUMNS_1STATINM(STATUS1,NLINES_PERCENT,MAT_PERCENT,INDIV) RESULT (OUTPUT_PERCENT)

    INTEGER                                             :: STATUS1                             !Value of status variable 1
    REAL(KIND=KIND_REAL)                                :: OUTPUT_PERCENT                      !Percentage produced as output of this function 
    INTEGER                                             :: NLINES_PERCENT                      !Number of lines in matrix
    INTEGER                                             :: i                                   !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_PERCENT,3)   :: MAT_PERCENT                         !Matrix with values of status variables and percentage to be selected
    INTEGER                                             :: INDIV                               !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                            !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_PERCENT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the last bracket
    DO i=1,NLINES_PERCENT                                                                        
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (STATUS1.GE.INT(MAT_PERCENT(i,1)).AND.&
        &STATUS1.LE.INT(MAT_PERCENT(i,2))) THEN
            OUTPUT_PERCENT=MAT_PERCENT(i,3)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function CONTRIB_SELPERC_3COLUMNS_1STATINM has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION CONTRIB_SELPERC_3COLUMNS_1STATINM

!===================================================================================!
!Define a function that selects a percentage conditional on three status variables  !
!a) the first variable is a fixed status variable (value in column 1 of the matrix) !
!b) the second variable is a fixed status variable (value in column 2 of the matrix)!
!c) the value of the second variable should be greater or equal to the value in     !
!column 3 of the matrix and smaller or equal to the value in column 4 of the matrix !
!The percentage to be selected is in column 5 of the matrix.                        !
!This function is used f.e. to select the percentage to compute the                 !
!deduction for job seekers                                                          !
!In this module to be applied with parameters                                       !
!   CONTRIB_JOBSDEDA_QT                                                             !
!===================================================================================!

FUNCTION CONTRIB_SELPERC_5COLUMNS_3STATINM(STATUS1,STATUS2,STATUS3,NLINES_PERCENT,MAT_PERCENT,INDIV) RESULT (OUTPUT_PERCENT)

    INTEGER                                             :: STATUS1                             !Value of status variable 1
    INTEGER                                             :: STATUS2                             !Value of status variable 2 
    INTEGER                                             :: STATUS3                             !Value of status variable 3 (f.e. number of months in status)
    REAL(KIND=KIND_REAL)                                :: OUTPUT_PERCENT                      !Percentage produced as output of this function 
    INTEGER                                             :: NLINES_PERCENT                      !Number of lines in matrix
    INTEGER                                             :: i                                   !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_PERCENT,5)   :: MAT_PERCENT                         !Matrix with values of status variables and percentage to be selected
    INTEGER                                             :: INDIV                               !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                            !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_PERCENT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the last bracket
    DO i=1,NLINES_PERCENT                                                                        
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (STATUS1.EQ.INT(MAT_PERCENT(i,1)).AND.STATUS2.EQ.INT(MAT_PERCENT(i,2)).AND.&
        &STATUS3.GE.INT(MAT_PERCENT(i,3)).AND.STATUS3.LE.INT(MAT_PERCENT(i,4))) THEN
            OUTPUT_PERCENT=MAT_PERCENT(i,5)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function CONTRIB_SELPERC_5COLUMNS_3STATINM has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION CONTRIB_SELPERC_5COLUMNS_3STATINM

!===================================================================================!
!With subroutine CONTRIB_LABMSTAT different labour market status variables are      !
!reconstructed                                                                      !
!Constructed variables                                                              !
!   CONTRIB_LABMSTAT_PRIV_QT                                                        !
!   CONTRIB_LABMSTAT_PUB_QT                                                         !
!   CONTRIB_LABMSTAT_SELF_QT                                                        !
!===================================================================================!

SUBROUTINE CONTRIB_LABMSTAT

    INTEGER:: pers_j
    INTEGER:: HOEDRSVZ_INT !Variable to store integer value of character variable HOEDRSVZ

    !CONTRIB_LABMSTAT_PRIV_QT Labour market status of a wage earner on the private labour market          
    !1   Blue collar worker
    !2   White collar worker
    DO pers_j=1,n_rec
        IF (InRSZ(pers_j).GE.1) THEN
            IF (secemp_20084(pers_j).EQ.'1') THEN
                IF (clatra_20084(pers_j).EQ.1) THEN 
                    CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 1
                ELSE IF (clatra_20084(pers_j).EQ.2) THEN 
                    CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 2
                ELSE IF (clatra_20084(pers_j).EQ.4) THEN 
                    CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 5
                END IF
            END IF
        END IF
    END DO

    !CONTRIB_LABMSTAT_PUB_QT  Labour market status of a wage earner on the public labour market           
    !1   Blue collar worker without statutory service
    !2   White collar worker without statutory service
    !3   Blue collar worker with statutory service
    !4   White collar worker with statutory service
    DO pers_j=1,n_rec
        !Start with RSZ data
        IF (InRSZ(pers_j).GE.1) THEN
            IF (secemp_20084(pers_j).EQ.'2') THEN
                !Select value based on Clatra
                IF (clatra_20084(pers_j).EQ.1) THEN 
                    CONTRIB_LABMSTAT_PUB_QT(pers_j) = 1
                ELSE IF (clatra_20084(pers_j).EQ.2) THEN 
                    CONTRIB_LABMSTAT_PUB_QT(pers_j) = 2
                ELSE IF (clatra_20084(pers_j).EQ.3) THEN 
                    CONTRIB_LABMSTAT_PUB_QT(pers_j) = 4
                ELSE IF (clatra_20084(pers_j).EQ.4) THEN 
                    CONTRIB_LABMSTAT_PUB_QT(pers_j) = 5
                END IF
                !If value still zero, select value based on Codtra
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0) THEN
                    IF (ANY(codtra_20084(pers_j).EQ.(/10, 11, 12, 13, 14, 15, 16, 22, 24, 25, 27, 35, 41, 45/))) THEN 
                        CONTRIB_LABMSTAT_PUB_QT(pers_j) = 1
                    ELSE IF (ANY(codtra_20084(pers_j).EQ.(/439, 484, 485, 487, 492, 493, 494, 495, 673, 840, 876, 877, 879/))) THEN
                        CONTRIB_LABMSTAT_PUB_QT(pers_j) = 2
                    ELSE IF (ANY(codtra_20084(pers_j).EQ.(/671, 675/))) THEN 
                        CONTRIB_LABMSTAT_PUB_QT(pers_j) = 4
                    END IF
                END IF
            END IF
        END IF    
        !Add definition based on RSZPPO variables
        IF (InRSZPPO(pers_j).GE.1) THEN
            IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0.OR.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5) THEN
                IF      (ANY(APL_codtra_20084(pers_j).EQ.&
                       &   (/102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113,&
                       &     114, 115, 116, 117, 118, 119, 120, 121, 122, 126, 127, 128,& 
                       &     129, 132, 133, 134, 135, 136, 138, 140, 142, 143, 144, 145,& 
                       &     146, 147, 148, 149, 150, 152, 153, 154, 155, 156, 157, 158,& 
                       &     159, 162, 163, 164, 165, 166, 167, 168, 181, 182, 183, 184,& 
                       &     191, 192, 193, 194, 195, 196, 197, 198/))) THEN                     
                            CONTRIB_LABMSTAT_PUB_QT(pers_j) = 1
                ELSE IF (ANY(APL_codtra_20084(pers_j).EQ.&
                       &   (/202, 203, 204, 205, 206, 207, 208, 209, 210, 212, 213, 214,&
                       &     215, 217, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228,& 
                       &     229, 232, 233, 234, 235, 236, 237, 238, 240, 242, 243, 244,& 
                       &     245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255, 256,& 
                       &     257, 258, 259, 260, 261, 262, 263, 264, 265, 266, 267, 270,& 
                       &     271, 273, 274, 275, 276, 277, 278, 279, 281, 282, 283, 284,& 
                       &     285, 286, 287, 288, 291, 292, 293, 294, 295, 296, 297, 298,&
                       &     761, 762, 763/))) THEN                                              
                            CONTRIB_LABMSTAT_PUB_QT(pers_j) = 2
                ELSE IF (ANY(APL_codtra_20084(pers_j).EQ.&
                       &   (/601, 602, 605, 606, 607, 608, 609, 610, 611, 615, 616, 617,& 
                       &     618, 773, 774, 775, 776, 777, 778, 790, 795/))) THEN                
                            CONTRIB_LABMSTAT_PUB_QT(pers_j) = 3
                ELSE IF (ANY(APL_codtra_20084(pers_j).EQ.&
                       &   (/620, 621, 622, 623, 625, 626, 630, 631, 632, 633, 635, 636,& 
                       &     649, 650, 651, 652, 659, 660, 671, 672, 673, 674, 675, 681,& 
                       &     682, 683, 684, 685, 700, 710, 720, 731, 732, 733, 734, 735,& 
                       &     736, 737, 738, 740, 771, 772, 781, 782, 783, 784, 785, 786,& 
                       &     787, 788/))) THEN                                                   
                            CONTRIB_LABMSTAT_PUB_QT(pers_j) = 4
                END IF
            END IF
        END IF
    END DO

    !Add historical information in cases of lacking status
    DO pers_j=1,n_rec
        !Use annual info RSZ
        IF (InRSZ(pers_j).GE.1.AND.(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0)) THEN
            CONTRIB_LABMSTAT_PRIV_QT(pers_j) = RSZ_LABMSTAT_PRIV_2008(pers_j)
            CONTRIB_LABMSTAT_PUB_QT(pers_j)  = RSZ_LABMSTAT_PUB_2008(pers_j)
        END IF            
        !Use annual info RSZPPO
        IF (InRSZPPO(pers_j).GE.1.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0) THEN
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = RSZPPO_LABMSTAT_PUB_2008(pers_j)
        END IF            
        !Use historic info RSZ and RSZPPO
        IF (InRSZ(pers_j).GE.1.AND.(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0)) THEN
            CONTRIB_LABMSTAT_PRIV_QT(pers_j) = LABMSTAT_PRIV(pers_j)
            CONTRIB_LABMSTAT_PUB_QT(pers_j)  = LABMSTAT_PUB(pers_j)
        END IF            
        !Use annual info RSZPPO
        IF (InRSZPPO(pers_j).GE.1.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0) THEN
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = LABMSTAT_PUB(pers_j)
        END IF
        !Correct values 5 with historic info
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.5.AND.ANY(RSZ_LABMSTAT_PRIV_2008(pers_j).EQ.(/1, 2/))) THEN
            CONTRIB_LABMSTAT_PRIV_QT(pers_j) = RSZ_LABMSTAT_PRIV_2008(pers_j)
        END IF
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.5.AND.ANY(LABMSTAT_PRIV(pers_j).EQ.(/1, 2/))) THEN
            CONTRIB_LABMSTAT_PRIV_QT(pers_j) = LABMSTAT_PRIV(pers_j)
        END IF
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5.AND.ANY(RSZ_LABMSTAT_PUB_2008(pers_j).EQ.(/1, 2, 3, 4/))) THEN
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = RSZ_LABMSTAT_PUB_2008(pers_j)
        END IF
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5.AND.ANY(RSZPPO_LABMSTAT_PUB_2008(pers_j).EQ.(/1, 2, 3, 4/))) THEN
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = RSZPPO_LABMSTAT_PUB_2008(pers_j)
        END IF
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5.AND.ANY(LABMSTAT_PUB(pers_j).EQ.(/1, 2, 3, 4/))) THEN
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = LABMSTAT_PUB(pers_j)
        END IF
        !If Priv or Pub is 5 and not the inverse then copy the other status
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.5.AND.ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 3/))) THEN 
            CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 1
        END IF
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.5.AND.ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/2, 4/))) THEN 
            CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 2
        END IF
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5.AND.ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1, 3/))) THEN 
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = 1
        END IF
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5.AND.ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/2, 4/))) THEN 
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = 2
        END IF
        !Gebruik vakantiegeld-regime om code 5 uit te schakelen
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.5.AND.GROEP_VG(pers_j).EQ.1) THEN 
            CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 1
        END IF
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.5.AND.ANY(GROEP_VG(pers_j).EQ.(/2, 3/))) THEN 
            CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 2
        END IF
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5.AND.(GROEP_VG(pers_j).EQ.1.OR.APL_GROEP_EV(pers_j).EQ.1)) THEN 
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = 1
        END IF
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5.AND.(ANY(GROEP_VG(pers_j).EQ.(/2, 3/)).OR.&
        &ANY(APL_GROEP_EV(pers_j).EQ.(/2, 3, 4, 5/)))) THEN 
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = 2
        END IF
        !Schakel resterende codes 5 uit
        IF (InRSZ(pers_j).GE.1.AND.CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.5) THEN 
            CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 2
        END IF
        IF (InRSZ(pers_j).GE.1.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5) THEN 
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = 4
        END IF
        IF (InRSZPPO(pers_j).GE.1.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5) THEN 
            CONTRIB_LABMSTAT_PUB_QT(pers_j) = 4
        END IF
!        IF (InRSZ(pers_j).EQ.2) THEN
!            WRITE(15,*) '5a',pers_j,clatra_20084(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),&
!            &CONTRIB_LABMSTAT_PUB_QT(pers_j),GROEP_VG(pers_j)
!        END IF
!        IF (InRSZPPO(pers_j).EQ.2) THEN
!            WRITE(15,*) '5a',pers_j,clatra_20084(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),GROEP_VG(pers_j)
!        END IF
!        !Test cases
!        IF (InRSZ(pers_j).GE.1.AND.CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.5) THEN
!            WRITE(15,*) '5a',pers_j,clatra_20084(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),&
!            &CONTRIB_LABMSTAT_PUB_QT(pers_j),GROEP_VG(pers_j)
!        END IF
!        IF ((InRSZ(pers_j).GE.1.OR.InRSZPPO(pers_j).EQ.1).AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.5) THEN
!            WRITE(15,*) '5b',pers_j,clatra_20084(pers_j),GROEP_VG(pers_j)
!        END IF
!        IF ((InRSZ(pers_j).GE.1.OR.InRSZPPO(pers_j).EQ.1).AND.CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.0.&
!        &AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0) THEN
!            WRITE(15,*) '5c',pers_j,clatra_20084(pers_j)
!        END IF
    END DO
        
    !CONTRIB_LABMSTAT_SELF_QT Labour market status of a self employed         
    !1   Individual has a self employed activity
    DO pers_j=1,n_rec
        READ (HOEDRSVZ_2008(pers_j)(1:1),  '(I10)') HOEDRSVZ_INT
        IF (ANY(HOEDRSVZ_INT.EQ.(/1,2,3,4/))) THEN
            CONTRIB_LABMSTAT_SELF_QT(pers_j) = 1
        END IF
    END DO
    
    !Adapt groups to compute holiday earnings
    DO pers_j=1,n_rec
        !Adapt groep vakantiegeld RSZ
        IF (GROEP_VG(pers_j).EQ.0) THEN
            IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1.OR.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                GROEP_VG(pers_j) = 1
            ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2.OR.ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/2, 3, 4/))) THEN
                GROEP_VG(pers_j) = 2
            END IF
        END IF            
        !Adapt groep enkel vakantiegeld RSZPPO
        IF (APL_GROEP_EV(pers_j).EQ.0) THEN
            IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1.OR.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                APL_GROEP_EV(pers_j) = 1
            ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2.OR.ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/2, 3, 4/))) THEN
                APL_GROEP_EV(pers_j) = 2
            END IF
        END IF            
        !Adapt groep dubbel vakantiegeld RSZPPO
        IF (APL_GROEP_DV(pers_j).EQ.0) THEN
            IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1.OR.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                APL_GROEP_DV(pers_j) = 1
            ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2.OR.ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/2, 3, 4/))) THEN
                APL_GROEP_DV(pers_j) = 2
            END IF
        END IF            
    END DO

END SUBROUTINE CONTRIB_LABMSTAT

!==========================================================================================================!
! The subroutine CONTRIB_WORKHOUR_CONSTRUC creates a vector with predefined hours worked by men and women
! It sets the hours worked conditional on the sex of the individual
! and determines the fraction of hours worked as a fraction of a full time worker
!==========================================================================================================!

SUBROUTINE CONTRIB_WORKHOUR_CONSTRUC

    INTEGER :: pers_j,pers_k

    !loop over all households
    DO hh_i=1,N_HH_CONTRIB
        !loop over all members of the sociological household
        DO pers_j=CONTRIB_START_HH(hh_i),CONTRIB_END_HH(hh_i) 
            !If a person is available for the labour market as wage earner and the hour has to be identified from an external file
            IF (MIMOSIS_FLEXWEAR(pers_j).EQ.1.AND.CONTRIB_HOURSIM.EQ.1) THEN
                !Check whether the individual is head of the household 
                IF (FAMREL_RELATION(pers_j).EQ.1) THEN
                    !the head always gets the number of hours of spouse 1
                    CONTRIB_WORKHOUR(pers_j) = DBLE(WORKHOUR_SPOUSE1)
                    !Check whether the head has a partner 
                    DO pers_k=CONTRIB_START_HH(hh_i),CONTRIB_END_HH(hh_i)
                        !look for married partner of the head of the family
                        IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                        &FAMREL_RELATION(pers_k).EQ.2) THEN
                            CONTRIB_WORKHOUR(pers_k) = DBLE(WORKHOUR_SPOUSE2)
                        END IF
                    END DO
                !look up household members that are not partner    
                ELSE IF (FAMREL_RELATION(pers_j).NE.2) THEN
                    !other household members that are not the partner of the head always work hours of spouse 1
                    CONTRIB_WORKHOUR(pers_j) = DBLE(WORKHOUR_SPOUSE1)
                END IF
                !Introduce an additional scheme for those that enter the labour market because of simulation
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0) THEN
                    !new individuals on the labour market are defined as white collar workers in the private labour market
                    CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 2
                END IF
                !Avoid double labour market status for this type of simulation
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).NE.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).NE.0) THEN
                    CONTRIB_LABMSTAT_PUB_QT(pers_j) = 0
                END IF
            END IF
            !If the hour has to be identified as the actual hour worked
            IF (CONTRIB_HOURSIM.EQ.2) THEN
                CONTRIB_WORKHOUR(pers_j) = MIMOSIS_HOURPRED(pers_j)
                !Introduce an additional scheme for those that enter the labour market because of simulation
                IF (INT(100.D0*CONTRIB_WORKHOUR(pers_j)).NE.0.AND.CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.0.AND.&
                &CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0) THEN
                    !new individuals on the labour market are defined as white collar workers in the private labour market
                    CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 2
                END IF        
            END IF
            !If the hour has to be identified as the hour for computation of marginal tax rates
            IF (ANY(CONTRIB_HOURSIM.EQ.(/3, 4/))) THEN
                IF (MIMOSIS_SIMNUMMT(pers_j).EQ.CONTRIB_SIMNRMT) THEN
                    CONTRIB_WORKHOUR(pers_j) = DBLE(WORKHOUR_SPOUSE1)
                ELSE
                    CONTRIB_WORKHOUR(pers_j) = MIMOSIS_HOURPRED(pers_j)
                END IF
                !Introduce an additional scheme for those that enter the labour market because of simulation
                IF (INT(100.D0*CONTRIB_WORKHOUR(pers_j)).NE.0.AND.CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.0.AND.&
                &CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0) THEN
                    !new individuals on the labour market are defined as white collar workers in the private labour market
                    CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 2
                END IF        
                !Avoid double labour market status for this type of simulation
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).NE.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).NE.0) THEN
                    CONTRIB_LABMSTAT_PUB_QT(pers_j) = 0
                END IF
            END IF
        END DO
    END DO
    
END SUBROUTINE CONTRIB_WORKHOUR_CONSTRUC

!===================================================================================!
!With subroutine CONTRIB_LABMINC we reconstruct labour market income for            !
!the different labour market statuses                                               !
!Constructed variables:                                                             !
!   CONTRIB_INCPRIV_QT                                                              !
!   CONTRIB_SHPRIV_QT                                                               !
!   CONTRIB_DHPRIV_QT                                                               !
!   CONTRIB_INCSPRIV_QT                                                             !
!   CONTRIB_INCPUB_QT                                                               !
!   CONTRIB_SHPUB_QT                                                                !
!   CONTRIB_DHPUB_QT                                                                !
!   CONTRIB_INCSPUB_QT                                                              !
!   CONTRIB_INCSELF_QT                                                              !
!   CONTRIB_SELFEXTR_QT                                                              !
!   PENSWELF_RETPENS_QT: adaptation in case of simulated labour activity            !
!   PENSWELF_SURPENS_QT: adaptation in case of simulated labour activity            !
!===================================================================================!

SUBROUTINE CONTRIB_LABMINC

    INTEGER                 :: pers_j
    INTEGER                 :: LOWRTAGE                                         !Indicator that says that individual is below retirement age
    REAL(KIND=KIND_REAL)    :: INCCONC                                          !Income concept used to compare with the ceiling of allowed labour income for pensioners
    REAL(KIND=KIND_REAL)    :: EXCESSR                                          !Rate with which labour income exceede the first floor
    REAL(KIND=KIND_REAL)    :: EV_RSZ_QT,DV_RSZ_QT,EV_RSZPPO_QT,DV_RSZPPO_QT    !Values of single and double holiday earnings depending on source
    REAL(KIND=KIND_REAL)    :: INCCONC1,INCCONC2                                !Variabelen om mogelijke inkomensconcepten op te slagen
    REAL(KIND=KIND_REAL)    :: EXT_FORF                                         !Extra forfait voor bepaling dubbel vakantiegeld
    REAL(KIND=KIND_REAL)    :: SUMINC                                           !Extra forfait voor bepaling dubbel vakantiegeld

    !construct labour market income if the user requests non simulated labour market incomes
    !Bij opvragen met geobserveerde gegevens worden voor alle jaren de regels van 2008 toegepast op de berekening
    !van het vakantiegeld
    IF (CONTRIB_HOURSIM.EQ.0) THEN

        DO pers_j=1,n_rec

            !Eerst constructie vakantiegeld
            !Als basis worden kwartaalconcepten gebruikt omdat die best aansluiten bij de
            !bedoelde vakantiegeld-groep
            EV_RSZ_QT = 0.D0
            DV_RSZ_QT = 0.D0
            EV_RSZPPO_QT = 0.D0
            DV_RSZPPO_QT = 0.D0 

            INCCONC1 = 0.D0 
            INCCONC2 = 0.D0 
            !Verwerk gegevens RSZ
            !Enkel voor groep 1 moet het enkel vakantiegeld gereconstrueerd worden
            IF (GROEP_VG(pers_j).EQ.1) THEN
                INCCONC1 = SAL100_agg(pers_j)+PRIMES_agg(pers_j)+SALATT_agg(pers_j)
                INCCONC2 = SumSal_2008(pers_j)/4.D0
                EV_RSZ_QT = (CONTRIB_SHBLUE_QT/100.D0)*((CONTRIB_PERSALB_QT/100.D0)*INCCONC1)
                IF (EV_RSZ_QT.EQ.0.D0.AND.INCCONC2.NE.0.D0) THEN
                    EV_RSZ_QT = (CONTRIB_SHBLUE_QT/100.D0)*((CONTRIB_PERSALB_QT/100.D0)*INCCONC2)
                END IF
            END IF
            !Voor alle groepen dubbel vakantiegeld bepalen
            !Bepaal extra forfaitair deel: regel is dezelfde voor alle groepen
            EXT_FORF = 0.D0
            IF (ANY(GROEP_VG(pers_j).EQ.(/1, 2, 3/))) THEN
                INCCONC1 = SAL100_agg(pers_j)+SALATT_agg(pers_j)
                INCCONC2 = SumSal_2008(pers_j)/4.D0
                EXT_FORF = INCCONC1*RATE_FICTIF_RSZ(pers_j)
                IF (EXT_FORF.EQ.0.D0.AND.INCCONC2.NE.0.D0) THEN
                    EXT_FORF = INCCONC2*RATE_FICTIF_RSZ(pers_j)
                END IF
            END IF
            !Bepaal dubbel vankantiegeld
            !Merk op: extra forfait is aan 100%
            !Voor ambtenaren niet langer stuk forfait en procent maar direct 92%
            IF (GROEP_VG(pers_j).EQ.1) THEN
                INCCONC1 = SAL100_agg(pers_j)+PRIMES_agg(pers_j)+SALATT_agg(pers_j)
                INCCONC2 = SumSal_2008(pers_j)/4.D0
                DV_RSZ_QT = (CONTRIB_DHBLUE_QT/100.D0)*((CONTRIB_PERSALB_QT/100.D0)*INCCONC1+EXT_FORF)
                IF (DV_RSZ_QT.EQ.0.D0.AND.INCCONC2.NE.0.D0) THEN
                    DV_RSZ_QT = (CONTRIB_DHBLUE_QT/100.D0)*((CONTRIB_PERSALB_QT/100.D0)*INCCONC2+EXT_FORF)
                END IF
            ELSE IF (ANY(GROEP_VG(pers_j).EQ.(/2, 3/))) THEN
                INCCONC1 = SAL100_agg(pers_j)+SALATT_agg(pers_j)
                INCCONC2 = SumSal_2008(pers_j)/4.D0
                DV_RSZ_QT = (CONTRIB_DHWHITE_QT/100.D0)*(3.D0/12.D0)*((INCCONC1+EXT_FORF)/3.D0)
                IF (DV_RSZ_QT.EQ.0.D0.AND.INCCONC2.NE.0.D0) THEN
                    DV_RSZ_QT = (CONTRIB_DHWHITE_QT/100.D0)*(3.D0/12.D0)*((INCCONC2+EXT_FORF)/3.D0)
                END IF
            END IF

            !Verwerk gegevens RSZPPO
            !Enkel voor groep 1 moet het enkel vakantiegeld gereconstrueerd worden
            EXT_FORF = 0.D0
            IF (APL_GROEP_EV(pers_j).EQ.1) THEN
                INCCONC1 = APL_sal100_agg(pers_j)+APL_primes_agg(pers_j)
                INCCONC2 = APL_SumSal_2008(pers_j)/4.D0
                EXT_FORF = INCCONC1*RATE_FICTIF_RSZPPO(pers_j)
                IF (EXT_FORF.EQ.0.D0.AND.INCCONC2.NE.0.D0) THEN
                    EXT_FORF = INCCONC2*RATE_FICTIF_RSZPPO(pers_j)
                END IF
                EV_RSZPPO_QT = (CONTRIB_SHBLUE_QT/100.D0)*((CONTRIB_PERSALB_QT/100.D0)*INCCONC1+EXT_FORF)
                IF (EV_RSZPPO_QT.EQ.0.D0.AND.INCCONC2.NE.0.D0) THEN
                    EV_RSZPPO_QT = (CONTRIB_SHBLUE_QT/100.D0)*((CONTRIB_PERSALB_QT/100.D0)*INCCONC2+EXT_FORF)
                END IF
            END IF
            !Dubbel vakantiegeld
            IF (ANY(APL_GROEP_DV(pers_j).EQ.(/1, 2/))) THEN
                INCCONC1 = APL_sal100_agg(pers_j)
                INCCONC2 = APL_SumSal_2008(pers_j)/4.D0
                EXT_FORF = INCCONC1*RATE_FICTIF_RSZPPO(pers_j)
                IF (EXT_FORF.EQ.0.D0.AND.INCCONC2.NE.0.D0) THEN
                    EXT_FORF = INCCONC2*RATE_FICTIF_RSZPPO(pers_j)
                END IF
                DV_RSZPPO_QT = (CONTRIB_DHWHITE_QT/100.D0)*(3.D0/12.D0)*((INCCONC1+EXT_FORF)/3.D0)
                IF (DV_RSZPPO_QT.EQ.0.D0.AND.INCCONC2.NE.0.D0) THEN
                    DV_RSZPPO_QT = (CONTRIB_DHWHITE_QT/100.D0)*(3.D0/12.D0)*((INCCONC2+EXT_FORF)/3.D0)
                END IF
            END IF

            !Limiteer vakantiegeld tot maximaal bedrag met behulp van jaarinkomen
            !voor gevallen waar niet met een fictief loon gewerkt werd
            IF (RATE_FICTIF_RSZ(pers_j).EQ.0.D0) THEN
                IF (InRSZ(pers_j).GE.1.AND.EV_RSZ_QT.GT.(SumSal_2008(pers_j)/4.D0)) THEN
!                    WRITE(15,*) 'EV-A',pers_j,EV_RSZ_QT,SumSal_2008(pers_j)
                    EV_RSZ_QT = (SumSal_2008(pers_j)/4.D0)
                END IF
                IF (InRSZ(pers_j).GE.1.AND.DV_RSZ_QT.GT.(SumSal_2008(pers_j)/4.D0)) THEN
!                    WRITE(15,*) 'DV-A',pers_j,DV_RSZ_QT,SumSal_2008(pers_j)
                    DV_RSZ_QT = (SumSal_2008(pers_j)/4.D0)
                END IF
            END IF
            IF (RATE_FICTIF_RSZPPO(pers_j).EQ.0.D0) THEN
                IF (InRSZPPO(pers_j).GE.1.AND.EV_RSZPPO_QT.GT.(APL_SumSal_2008(pers_j)/4.D0)) THEN
!                    WRITE(15,*) 'EV-B',pers_j,EV_RSZPPO_QT,APL_SumSal_2008(pers_j)
                    EV_RSZPPO_QT = (APL_SumSal_2008(pers_j)/4.D0)
                END IF
                IF (InRSZPPO(pers_j).GE.1.AND.DV_RSZPPO_QT.GT.(APL_SumSal_2008(pers_j)/4.D0)) THEN
!                    WRITE(15,*) 'DV-B',pers_j,DV_RSZPPO_QT,APL_SumSal_2008(pers_j)
                    DV_RSZPPO_QT = (APL_SumSal_2008(pers_j)/4.D0)
                END IF
            END IF

            !Dan bepaling verschillende concepten
            !Construct labour market income of wage earner on the private labour market
            !CONTRIB_INCPRIV_QT       Labour income, earned as wage earner on the private labour market           
            !CONTRIB_SHPRIV_QT        Single holiday earnings earned on the private labour market         
            !CONTRIB_DHPRIV_QT        Single holiday earnings earned on the private labour market         
            !CONTRIB_INCSPRIV_QT      Supplement to real earned labour income, other than holiday earnings, obtained as wage earner on the private labour market
            !Construct labour market income of wage earner on the public labour market
            !CONTRIB_INCPUB_QT        Labour income, earned as wage earner on the public labour market            
            !CONTRIB_SHPUB_QT         Single holiday earnings earned on the public labour market          
            !CONTRIB_DHPUB_QT         Single holiday earnings earned on the public labour market          
            !CONTRIB_INCSPUB_QT       Supplement to real earned labour income, other than holiday earnings, obtained as wage earner on the public labour market
            IF (InRSZ(pers_j).GE.1.AND.InRSZPPO(pers_j).EQ.0) THEN
                IF (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
                    CONTRIB_INCPRIV_QT(pers_j)  = (SumSal_2008(pers_j)/4.D0)
                    CONTRIB_SHPRIV_QT(pers_j)   = EV_RSZ_QT
                    CONTRIB_DHPRIV_QT(pers_j)   = DV_RSZ_QT
                    CONTRIB_INCSPRIV_QT(pers_j) = SICK_EMPLBPRI_QT(pers_j)                           
                ELSE IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2, 3, 4/))) THEN
                    CONTRIB_INCPUB_QT(pers_j)   = (SumSal_2008(pers_j)/4.D0)
                    CONTRIB_SHPUB_QT(pers_j)    = EV_RSZ_QT
                    CONTRIB_DHPUB_QT(pers_j)    = DV_RSZ_QT
                    CONTRIB_INCSPUB_QT(pers_j)  = SICK_EMPLBPRI_QT(pers_j)                           
                END IF
            ELSE IF (InRSZ(pers_j).EQ.0.AND.InRSZPPO(pers_j).GE.1) THEN
                IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2, 3, 4/))) THEN
                    CONTRIB_INCPUB_QT(pers_j)   = (APL_SumSal_2008(pers_j)/4.D0)
                    CONTRIB_SHPUB_QT(pers_j)    = EV_RSZPPO_QT
                    CONTRIB_DHPUB_QT(pers_j)    = DV_RSZPPO_QT
                    CONTRIB_INCSPUB_QT(pers_j)  = SICK_EMPLBPRI_QT(pers_j)                           
                END IF
            ELSE IF (InRSZ(pers_j).GE.1.AND.InRSZPPO(pers_j).GE.1) THEN
                !indien er privaat statuut is is het zeker dat rsz privaat is en rszppo publiek
                IF (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
                    CONTRIB_INCPRIV_QT(pers_j)  = (SumSal_2008(pers_j)/4.D0)
                    CONTRIB_SHPRIV_QT(pers_j)   = EV_RSZ_QT
                    CONTRIB_DHPRIV_QT(pers_j)   = DV_RSZ_QT
                    CONTRIB_INCSPRIV_QT(pers_j) = SICK_EMPLBPRI_QT(pers_j)                           
                    CONTRIB_INCPUB_QT(pers_j)   = (APL_SumSal_2008(pers_j)/4.D0)
                    CONTRIB_SHPUB_QT(pers_j)    = EV_RSZPPO_QT
                    CONTRIB_DHPUB_QT(pers_j)    = DV_RSZPPO_QT
                    CONTRIB_INCSPUB_QT(pers_j)  = 0.D0
                ELSE IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2, 3, 4/))) THEN
                    CONTRIB_INCPUB_QT(pers_j)   = ((SumSal_2008(pers_j)+APL_SumSal_2008(pers_j))/4.D0)
                    CONTRIB_SHPUB_QT(pers_j)    = EV_RSZ_QT + EV_RSZPPO_QT
                    CONTRIB_DHPUB_QT(pers_j)    = DV_RSZ_QT + DV_RSZPPO_QT
                    CONTRIB_INCSPUB_QT(pers_j)  = SICK_EMPLBPRI_QT(pers_j)                           
                END IF
            END IF
            
!            !Test of er gevallen op de arbeidsmarkt zijn waarvoor we geen inkomen of dubbel vakantiegeld observeren
!            IF (InRSZ(pers_j).GE.1.OR.InRSZPPO(pers_j).GE.1) THEN
!                IF (CONTRIB_INCPRIV_QT(pers_j).EQ.0.D0.AND.CONTRIB_INCPUB_QT(pers_j).EQ.0.D0) THEN
!                    WRITE(15,*) 'INC',pers_j,SumSal_2008(pers_j),APL_SumSal_2008(pers_j)
!                END IF
!                IF (CONTRIB_DHPRIV_QT(pers_j).EQ.0.D0.AND.CONTRIB_DHPUB_QT(pers_j).EQ.0.D0) THEN
!                    WRITE(15,*) 'DH',pers_j,SumSal_2008(pers_j),APL_SumSal_2008(pers_j)
!                END IF
!            END IF

!            !Test of er bij RSZ gevallen zijn met een dubbel statuut: door reductie data zou dat niet mogen
!            IF (InRSZ(pers_j).GE.1.AND.InRSZPPO(pers_j).GE.0) THEN
!                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).NE.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).NE.0) THEN
!                    WRITE(15,*) 'DUB',pers_j,SumSal_2008(pers_j),APL_SumSal_2008(pers_j)
!                END IF
!            END IF

        END DO

        !Construct labour market income of self employed
        !CONTRIB_INCSELF_QT       Labour income, earned as self employed  
        !distinguish "taxable" and "non taxable" income
!        SUMINC = 0.D0
        DO pers_j=1,n_rec
            IF (CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.1) THEN
                IF (Bron_Aanv(pers_j).EQ.1) THEN 
                    CONTRIB_INCSELF_QT(pers_j) = inkomens_2008(pers_j)/4.D0
!                    SUMINC = SUMINC + inkomens_2008(pers_j)*MIMOSIS_WEIGHT(pers_j)
                ELSE
                    CONTRIB_SELFEXTR_QT(pers_j) = inkomens_2008(pers_j)/4.D0
                END IF
            END IF    
        END DO
!        WRITE(15,*) 'IncSelf',SUMINC

    !construct labour market income if the user requests simulated labour market incomes
    ELSE
        DO pers_j=1,n_rec
            !Correct hourly wage if below the minimum wage
            IF (MIMOSIS_GRINC_HOUR(pers_j).LT.7.63D0) THEN
                MIMOSIS_GRINC_HOUR(pers_j)=7.63D0
            END IF
            !Increase the wage with a percentage if necessary
            IF (CONTRIB_SEXINCR.NE.0) THEN
                !Increase wage conditional on the parameter sex-increase
                !If a man
                IF (CONTRIB_SEXINCR.EQ.1.AND.NAREGNIS_SEX(pers_j).EQ.1) THEN
                    MIMOSIS_GRINC_HOUR(pers_j) = CONTRIB_WAGEINC*MIMOSIS_GRINC_HOUR(pers_j)
                !If a woman
                ELSE IF (CONTRIB_SEXINCR.EQ.2.AND.NAREGNIS_SEX(pers_j).EQ.2) THEN
                    MIMOSIS_GRINC_HOUR(pers_j) = CONTRIB_WAGEINC*MIMOSIS_GRINC_HOUR(pers_j)
                END IF
            END IF            
        END DO
        
        !Construct labour market income of wage earner on the private labour market
        !CONTRIB_INCPRIV_QT       Labour income, earned as wage earner on the private labour market           
        !CONTRIB_SHPRIV_QT        Single holiday earnings earned on the private labour market         
        !CONTRIB_DHPRIV_QT        Single holiday earnings earned on the private labour market         
        !CONTRIB_INCSPRIV_QT      Supplement to real earned labour income, other than holiday earnings, obtained as wage earner on the private labour market
        DO pers_j=1,n_rec
            !for blue collar workers
            IF  (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                CONTRIB_INCPRIV_QT(pers_j) = 13.D0*(CONTRIB_WORKHOUR(pers_j)*MIMOSIS_GRINC_HOUR(pers_j))
                CONTRIB_SHPRIV_QT(pers_j) = (CONTRIB_SHBLUE_QT/100.D0)*((CONTRIB_PERSALB_QT/100.D0)*&
                                            &CONTRIB_INCPRIV_QT(pers_j))
                CONTRIB_DHPRIV_QT(pers_j) = (CONTRIB_DHBLUE_QT/100.D0)*((CONTRIB_PERSALB_QT/100.D0)*&
                                            &CONTRIB_INCPRIV_QT(pers_j))
                CONTRIB_INCSPRIV_QT(pers_j) = SICK_EMPLBPRI_QT(pers_j)                           
            END IF
            !for white collar workers
            IF  (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                CONTRIB_INCPRIV_QT(pers_j) = 13.D0*(CONTRIB_WORKHOUR(pers_j)*MIMOSIS_GRINC_HOUR(pers_j))
                CONTRIB_SHPRIV_QT(pers_j) = 0
                CONTRIB_DHPRIV_QT(pers_j) = (CONTRIB_DHWHITE_QT/100.D0)*(3.D0/12.D0)*(CONTRIB_INCPRIV_QT(pers_j)/3.D0)
                CONTRIB_INCSPRIV_QT(pers_j) = SICK_EMPLBPRI_QT(pers_j)                            
            END IF
        END DO
    
        !Construct labour market income of wage earner on the public labour market
        !CONTRIB_INCPUB_QT        Labour income, earned as wage earner on the public labour market            
        !CONTRIB_SHPUB_QT         Single holiday earnings earned on the public labour market          
        !CONTRIB_DHPUB_QT         Single holiday earnings earned on the public labour market          
        !CONTRIB_INCSPUB_QT      Supplement to real earned labour income, other than holiday earnings, obtained as wage earner on the public labour market
        DO pers_j=1,n_rec
            !for blue collar workers without statutory service
            IF  (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                CONTRIB_INCPUB_QT(pers_j) = 13.D0*(CONTRIB_WORKHOUR(pers_j)*MIMOSIS_GRINC_HOUR(pers_j))
                CONTRIB_SHPUB_QT(pers_j) = (CONTRIB_SHBLUE_QT/100.D0)*((CONTRIB_PERSALB_QT/100.D0)*&
                                            &CONTRIB_INCPUB_QT(pers_j))
                CONTRIB_DHPUB_QT(pers_j) = (CONTRIB_DHBLUE_QT/100.D0)*((CONTRIB_PERSALB_QT/100.D0)*&
                                            &CONTRIB_INCPUB_QT(pers_j))
                CONTRIB_INCSPUB_QT(pers_j) = SICK_EMPLBPUB_QT(pers_j)                            
            END IF
            !for white collar workers without statutory service
            IF  (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                CONTRIB_INCPUB_QT(pers_j) = 13.D0*(CONTRIB_WORKHOUR(pers_j)*MIMOSIS_GRINC_HOUR(pers_j))
                CONTRIB_SHPUB_QT(pers_j) = 0
                CONTRIB_DHPUB_QT(pers_j) = (CONTRIB_DHWHITE_QT/100.D0)*(3.D0/12.D0)*(CONTRIB_INCPUB_QT(pers_j)/3.D0)
                CONTRIB_INCSPUB_QT(pers_j) = SICK_EMPLBPUB_QT(pers_j)                            
            END IF
            !for blue collar workers with statutory service
            IF  (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.3) THEN
                CONTRIB_INCPUB_QT(pers_j) = 13.D0*(CONTRIB_WORKHOUR(pers_j)*MIMOSIS_GRINC_HOUR(pers_j))
                CONTRIB_SHPUB_QT(pers_j) = 0
                IF (CONTRIB_WORKHOUR(pers_j).GT.0.D0) THEN
                    CONTRIB_DHPUB_QT(pers_j) = CONTRIB_DHCSFIX_QT + ((CONTRIB_DHCSVAR_QT/100.D0)*&
                                               &CONTRIB_INCPUB_QT(pers_j))
                END IF
                CONTRIB_INCSPUB_QT(pers_j) = SICK_EMPLBPUB_QT(pers_j)                            
            END IF
            !for white collar workers with statutory service
            IF  (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.4) THEN
                CONTRIB_INCPUB_QT(pers_j) = 13.D0*(CONTRIB_WORKHOUR(pers_j)*MIMOSIS_GRINC_HOUR(pers_j))
                CONTRIB_SHPUB_QT(pers_j) = 0
                IF (CONTRIB_WORKHOUR(pers_j).GT.0.D0) THEN
                    CONTRIB_DHPUB_QT(pers_j) = CONTRIB_DHCSFIX_QT + ((CONTRIB_DHCSVAR_QT/100.D0)*&
                                               &CONTRIB_INCPUB_QT(pers_j))
                END IF
                CONTRIB_INCSPUB_QT(pers_j) = SICK_EMPLBPUB_QT(pers_j)                            
            END IF
        END DO
    
        !Construct labour market income of self employed
        !CONTRIB_INCSELF_QT       Labour income, earned as self employed          
        DO pers_j=1,n_rec
            IF (CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.1) THEN
                IF (Bron_Aanv(pers_j).EQ.1) THEN 
                    CONTRIB_INCSELF_QT(pers_j) = inkomens_2008(pers_j)/4.D0
                ELSE
                    CONTRIB_SELFEXTR_QT(pers_j) = inkomens_2008(pers_j)/4.D0
                END IF
            END IF    
        END DO
        
        !Adapt pensions when labour market income is simulated 
        DO pers_j=1,n_rec
            IF (ANY(CONTRIB_HOURSIM.EQ.(/1, 3, 4/))) THEN
                !Determine the official retirement age
                LOWRTAGE = 0
                IF (SIMYEAR.LT.2003) THEN
                    IF (NAREGNIS_SEX(pers_j).EQ.1.AND.CONTRIB_YEAR_AGE(pers_j).GE.65) THEN
                        LOWRTAGE = 1
                    ELSE IF (NAREGNIS_SEX(pers_j).EQ.2.AND.CONTRIB_YEAR_AGE(pers_j).GE.62) THEN
                        LOWRTAGE = 1
                    END IF
                ELSE IF (SIMYEAR.GE.2003.AND.SIMYEAR.LT.2006) THEN
                    IF (NAREGNIS_SEX(pers_j).EQ.1.AND.CONTRIB_YEAR_AGE(pers_j).GE.65) THEN
                        LOWRTAGE = 1
                    ELSE IF (NAREGNIS_SEX(pers_j).EQ.2.AND.CONTRIB_YEAR_AGE(pers_j).GE.63) THEN
                        LOWRTAGE = 1
                    END IF
                ELSE IF (SIMYEAR.GE.2006.AND.SIMYEAR.LT.2009) THEN
                    IF (NAREGNIS_SEX(pers_j).EQ.1.AND.CONTRIB_YEAR_AGE(pers_j).GE.65) THEN
                        LOWRTAGE = 1
                    ELSE IF (NAREGNIS_SEX(pers_j).EQ.2.AND.CONTRIB_YEAR_AGE(pers_j).GE.64) THEN
                        LOWRTAGE = 1
                    END IF
                ELSE
                    IF (NAREGNIS_SEX(pers_j).EQ.1.AND.CONTRIB_YEAR_AGE(pers_j).GE.65) THEN
                        LOWRTAGE = 1
                    ELSE IF (NAREGNIS_SEX(pers_j).EQ.2.AND.CONTRIB_YEAR_AGE(pers_j).GE.65) THEN
                        LOWRTAGE = 1
                    END IF
                END IF
                !Determine income concept needed to compare with ceiling
                INCCONC = CONTRIB_INCPRIV_QT(pers_j) + CONTRIB_SHPRIV_QT(pers_j) + &
                          &CONTRIB_DHPRIV_QT(pers_j) + CONTRIB_INCSPRIV_QT(pers_j) + & 
                          &CONTRIB_INCPUB_QT(pers_j) + CONTRIB_SHPUB_QT(pers_j) + &
                          &CONTRIB_DHPUB_QT(pers_j) + CONTRIB_INCSPUB_QT(pers_j) 
                !Distinguish different cases
                IF (PENSWELF_RETPENS_QT(pers_j).GT.0.D0) THEN
                    !RP positive    and     OP 0        and individual below official retirement age
                    !RP positive    and     OP positive and individual below official retirement age
                    IF (LOWRTAGE.EQ.0) THEN
                        !Individual has no children in charge
                        IF (FAMAL_AMOUNTF_QT(pers_j).EQ.0.D0) THEN
                            !Check first ceiling
                            IF (INCCONC.GT.(CONTRIB_CEPWEARN_YEAR(1)/4.D0)) THEN
                                !proportional reduction of pension if income is below the increased ceiling
                                IF ((CONTRIB_CEPWEARN_YEAR(1)/4.D0).GT.0.D0) THEN
                                    EXCESSR = (INCCONC - (CONTRIB_CEPWEARN_YEAR(1)/4.D0))/(CONTRIB_CEPWEARN_YEAR(1)/4.D0)
                                ELSE                                    
                                    EXCESSR = 1.D0
                                END IF                                    
                                !proportional reduction of pension if income is below the increased ceiling
                                IF (EXCESSR.LE.(CONTRIB_RAPWEARN_YEAR(1)/100.D0)) THEN
                                    PENSWELF_RETPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_RETPENS_QT(pers_j)
                                    PENSWELF_SURPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_SURPENS_QT(pers_j) 
                                ELSE
                                    PENSWELF_RETPENS_QT(pers_j) =   0.D0
                                    PENSWELF_SURPENS_QT(pers_j) =   0.D0
                                END IF
                            END IF
                        !Individual has children in charge
                        ELSE IF (FAMAL_AMOUNTF_QT(pers_j).GT.0.D0) THEN
                            !Check first ceiling
                            IF (INCCONC.GT.(CONTRIB_CEPWEARN_YEAR(2)/4.D0)) THEN
                                !proportional reduction of pension if income is below the increased ceiling
                                IF ((CONTRIB_CEPWEARN_YEAR(2)/4.D0).GT.0.D0) THEN
                                    EXCESSR = (INCCONC - (CONTRIB_CEPWEARN_YEAR(2)/4.D0))/(CONTRIB_CEPWEARN_YEAR(2)/4.D0)
                                ELSE                                    
                                    EXCESSR = 1.D0
                                END IF                                    
                                !proportional reduction of pension if income is below the increased ceiling
                                IF (EXCESSR.LE.(CONTRIB_RAPWEARN_YEAR(2)/100.D0)) THEN
                                    PENSWELF_RETPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_RETPENS_QT(pers_j)
                                    PENSWELF_SURPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_SURPENS_QT(pers_j) 
                                ELSE
                                    PENSWELF_RETPENS_QT(pers_j) =   0.D0
                                    PENSWELF_SURPENS_QT(pers_j) =   0.D0
                                END IF
                            END IF
                        END IF
                    !RP positive    and     OP 0        and individual above official retirement age
                    !RP positive    and     OP positive and individual above official retirement age
                    ELSE IF (LOWRTAGE.EQ.1) THEN
                        !Individual has no children in charge
                        IF (FAMAL_AMOUNTF_QT(pers_j).EQ.0.D0) THEN
                            !Check first ceiling
                            IF (INCCONC.GT.(CONTRIB_CEPWEARN_YEAR(3)/4.D0)) THEN
                                !proportional reduction of pension if income is below the increased ceiling
                                IF ((CONTRIB_CEPWEARN_YEAR(3)/4.D0).GT.0.D0) THEN
                                    EXCESSR = (INCCONC - (CONTRIB_CEPWEARN_YEAR(3)/4.D0))/(CONTRIB_CEPWEARN_YEAR(3)/4.D0)
                                ELSE                                    
                                    EXCESSR = 1.D0
                                END IF                                    
                                !proportional reduction of pension if income is below the increased ceiling
                                IF (EXCESSR.LE.(CONTRIB_RAPWEARN_YEAR(3)/100.D0)) THEN
                                    PENSWELF_RETPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_RETPENS_QT(pers_j)
                                    PENSWELF_SURPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_SURPENS_QT(pers_j) 
                                ELSE
                                    PENSWELF_RETPENS_QT(pers_j) =   0.D0
                                    PENSWELF_SURPENS_QT(pers_j) =   0.D0
                                END IF
                            END IF
                        !Individual has children in charge
                        ELSE IF (FAMAL_AMOUNTF_QT(pers_j).GT.0.D0) THEN
                            !Check first ceiling
                            IF (INCCONC.GT.(CONTRIB_CEPWEARN_YEAR(4)/4.D0)) THEN
                                !proportional reduction of pension if income is below the increased ceiling
                                IF ((CONTRIB_CEPWEARN_YEAR(4)/4.D0).GT.0.D0) THEN
                                    EXCESSR = (INCCONC - (CONTRIB_CEPWEARN_YEAR(4)/4.D0))/(CONTRIB_CEPWEARN_YEAR(4)/4.D0)
                                ELSE                                    
                                    EXCESSR = 1.D0
                                END IF                                    
                                !proportional reduction of pension if income is below the increased ceiling
                                IF (EXCESSR.LE.(CONTRIB_RAPWEARN_YEAR(4)/100.D0)) THEN
                                    PENSWELF_RETPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_RETPENS_QT(pers_j)
                                    PENSWELF_SURPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_SURPENS_QT(pers_j) 
                                ELSE
                                    PENSWELF_RETPENS_QT(pers_j) =   0.D0
                                    PENSWELF_SURPENS_QT(pers_j) =   0.D0
                                END IF
                            END IF
                        END IF
                    END IF
                ELSE IF (PENSWELF_RETPENS_QT(pers_j).EQ.0.D0) THEN
                    IF (PENSWELF_SURPENS_QT(pers_j).GT.0.D0) THEN
                        !RP 0           and     OP positive and indvidual older than 65
                        IF (CONTRIB_YEAR_AGE(pers_j).GE.65) THEN
                            !Individual has no children in charge
                            IF (FAMAL_AMOUNTF_QT(pers_j).EQ.0.D0) THEN
                                !Check first ceiling
                                IF (INCCONC.GT.(CONTRIB_CEPWEARN_YEAR(3)/4.D0)) THEN
                                    !proportional reduction of pension if income is below the increased ceiling
                                    IF ((CONTRIB_CEPWEARN_YEAR(3)/4.D0).GT.0.D0) THEN
                                        EXCESSR = (INCCONC - (CONTRIB_CEPWEARN_YEAR(3)/4.D0))/(CONTRIB_CEPWEARN_YEAR(3)/4.D0)
                                    ELSE                                    
                                        EXCESSR = 1.D0
                                    END IF                                    
                                    !proportional reduction of pension if income is below the increased ceiling
                                    IF (EXCESSR.LE.(CONTRIB_RAPWEARN_YEAR(3)/100.D0)) THEN
                                        PENSWELF_RETPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_RETPENS_QT(pers_j)
                                        PENSWELF_SURPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_SURPENS_QT(pers_j) 
                                    ELSE
                                        PENSWELF_RETPENS_QT(pers_j) =   0.D0
                                        PENSWELF_SURPENS_QT(pers_j) =   0.D0
                                    END IF
                                END IF
                            !Individual has children in charge
                            ELSE IF (FAMAL_AMOUNTF_QT(pers_j).GT.0.D0) THEN
                                !Check first ceiling
                                IF (INCCONC.GT.(CONTRIB_CEPWEARN_YEAR(4)/4.D0)) THEN
                                    !proportional reduction of pension if income is below the increased ceiling
                                    IF ((CONTRIB_CEPWEARN_YEAR(4)/4.D0).GT.0.D0) THEN
                                        EXCESSR = (INCCONC - (CONTRIB_CEPWEARN_YEAR(4)/4.D0))/(CONTRIB_CEPWEARN_YEAR(4)/4.D0)
                                    ELSE                                    
                                        EXCESSR = 1.D0
                                    END IF                                    
                                    !proportional reduction of pension if income is below the increased ceiling
                                    IF (EXCESSR.LE.(CONTRIB_RAPWEARN_YEAR(4)/100.D0)) THEN
                                        PENSWELF_RETPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_RETPENS_QT(pers_j)
                                        PENSWELF_SURPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_SURPENS_QT(pers_j) 
                                    ELSE
                                        PENSWELF_RETPENS_QT(pers_j) =   0.D0
                                        PENSWELF_SURPENS_QT(pers_j) =   0.D0
                                    END IF
                                END IF
                            END IF
                        !RP 0           and     OP positive and indvidual younger than 65
                        ELSE IF (CONTRIB_YEAR_AGE(pers_j).LT.65) THEN
                            !Individual has no children in charge
                            IF (FAMAL_AMOUNTF_QT(pers_j).EQ.0.D0) THEN
                                !Check first ceiling
                                IF (INCCONC.GT.(CONTRIB_CEPWEARN_YEAR(5)/4.D0)) THEN
                                    !proportional reduction of pension if income is below the increased ceiling
                                    IF ((CONTRIB_CEPWEARN_YEAR(5)/4.D0).GT.0.D0) THEN
                                        EXCESSR = (INCCONC - (CONTRIB_CEPWEARN_YEAR(5)/4.D0))/(CONTRIB_CEPWEARN_YEAR(5)/4.D0)
                                    ELSE                                    
                                        EXCESSR = 1.D0
                                    END IF                                    
                                    !proportional reduction of pension if income is below the increased ceiling
                                    IF (EXCESSR.LE.(CONTRIB_RAPWEARN_YEAR(5)/100.D0)) THEN
                                        PENSWELF_RETPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_RETPENS_QT(pers_j)
                                        PENSWELF_SURPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_SURPENS_QT(pers_j) 
                                    ELSE
                                        PENSWELF_RETPENS_QT(pers_j) =   0.D0
                                        PENSWELF_SURPENS_QT(pers_j) =   0.D0
                                    END IF
                                END IF
                            !Individual has children in charge
                            ELSE IF (FAMAL_AMOUNTF_QT(pers_j).GT.0.D0) THEN
                                !Check first ceiling
                                IF (INCCONC.GT.(CONTRIB_CEPWEARN_YEAR(6)/4.D0)) THEN
                                    !proportional reduction of pension if income is below the increased ceiling
                                    IF ((CONTRIB_CEPWEARN_YEAR(6)/4.D0).GT.0.D0) THEN
                                        EXCESSR = (INCCONC - (CONTRIB_CEPWEARN_YEAR(6)/4.D0))/(CONTRIB_CEPWEARN_YEAR(6)/4.D0)
                                    ELSE                                    
                                        EXCESSR = 1.D0
                                    END IF                                    
                                    !proportional reduction of pension if income is below the increased ceiling
                                    IF (EXCESSR.LE.(CONTRIB_RAPWEARN_YEAR(6)/100.D0)) THEN
                                        PENSWELF_RETPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_RETPENS_QT(pers_j)
                                        PENSWELF_SURPENS_QT(pers_j) =   (1.D0-EXCESSR)*PENSWELF_SURPENS_QT(pers_j) 
                                    ELSE
                                        PENSWELF_RETPENS_QT(pers_j) =   0.D0
                                        PENSWELF_SURPENS_QT(pers_j) =   0.D0
                                    END IF
                                END IF
                            END IF
                        END IF
                    END IF
                END IF    
                !Check whether pensions did not became negative because of manipulations
                IF (PENSWELF_RETPENS_QT(pers_j).LT.0.D0) THEN
                    PENSWELF_RETPENS_QT(pers_j) = 0.D0
                END IF
                IF (PENSWELF_SURPENS_QT(pers_j).LT.0.D0) THEN
                    PENSWELF_SURPENS_QT(pers_j) = 0.D0
                END IF
            END IF
        END DO    
    END IF
    
END SUBROUTINE CONTRIB_LABMINC

!===================================================================================!
!With subroutine CONTRIB_UNEMINC we reconstruct labour market income for            !
!unemployed who are to drop out the system because of time restriction              !
!Constructed variables:                                                             !
!   CONTRIB_INCPRIV_QT                                                              !
!   CONTRIB_SHPRIV_QT                                                               !
!   CONTRIB_DHPRIV_QT                                                               !
!   CONTRIB_INCSPRIV_QT                                                             !
!===================================================================================!

SUBROUTINE CONTRIB_UNEMINC

    INTEGER:: pers_j
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: CONTRIB_RESULT_TEST                  !matrix with test results
    CHARACTER(len=75),ALLOCATABLE,DIMENSION(:)              :: CONTRIB_LAB_TEST                     !Labels of test variables

    ALLOCATE(CONTRIB_RESULT_TEST(3,3))
    ALLOCATE(CONTRIB_LAB_TEST(3))
    CONTRIB_RESULT_TEST(1:3,1:3)=0.D0    

    !Construct labour market income of unemployed who become wage earner
    !assume they become wage earner on the private labour market as white collar worker
    !CONTRIB_INCPRIV_QT       Labour income, earned as wage earner on the private labour market           
    !CONTRIB_SHPRIV_QT        Single holiday earnings earned on the private labour market         
    !CONTRIB_DHPRIV_QT        Single holiday earnings earned on the private labour market         
    !CONTRIB_INCSPRIV_QT      Supplement to real earned labour income, other than holiday earnings, obtained as wage earner on the private labour market
    DO pers_j=1,n_rec
        IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/1, 2/)).AND.UNEM_NUMMINU_QT(pers_j).GT.CONTRIB_DUURP) THEN
            !Correct hourly wage if below the minimum wage
            IF (MIMOSIS_GRINC_HOUR(pers_j).LT.7.63D0) THEN
                MIMOSIS_GRINC_HOUR(pers_j) = 7.63D0
            END IF
            CONTRIB_INCPRIV_QT(pers_j) = 13.D0*(CONTRIB_HOURUNEM*MIMOSIS_GRINC_HOUR(pers_j))
            CONTRIB_SHPRIV_QT(pers_j) = 0
            CONTRIB_DHPRIV_QT(pers_j) = (CONTRIB_DHWHITE_QT/100.D0)*(3.D0/12.D0)*(CONTRIB_INCPRIV_QT(pers_j)/3.D0)
            CONTRIB_INCSPRIV_QT(pers_j) = 0                            
            CONTRIB_LABMSTAT_PRIV_QT(pers_j) = 2
            !create test statistics
            IF (UNEM_STATUS_QT(pers_j).EQ.1) THEN
                CONTRIB_RESULT_TEST(1,1) = CONTRIB_RESULT_TEST(1,1) + 1
                CONTRIB_RESULT_TEST(1,2) = CONTRIB_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
                CONTRIB_RESULT_TEST(1,3) = CONTRIB_RESULT_TEST(1,3) + MIMOSIS_WEIGHT(pers_j)*CONTRIB_HOURUNEM
            ELSE IF (UNEM_STATUS_QT(pers_j).EQ.2) THEN
                CONTRIB_RESULT_TEST(2,1) = CONTRIB_RESULT_TEST(2,1) + 1
                CONTRIB_RESULT_TEST(2,2) = CONTRIB_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
                CONTRIB_RESULT_TEST(2,3) = CONTRIB_RESULT_TEST(2,3) + MIMOSIS_WEIGHT(pers_j)*CONTRIB_HOURUNEM
            ELSE IF (UNEM_STATUS_QT(pers_j).NE.1.AND.UNEM_STATUS_QT(pers_j).NE.2) THEN
                CONTRIB_RESULT_TEST(3,1) = CONTRIB_RESULT_TEST(3,1) + 1
                CONTRIB_RESULT_TEST(3,2) = CONTRIB_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
                CONTRIB_RESULT_TEST(3,3) = CONTRIB_RESULT_TEST(3,3) + MIMOSIS_WEIGHT(pers_j)*CONTRIB_HOURUNEM
            END IF            
        END IF
    END DO

    !Define labels
    CONTRIB_LAB_TEST(1) = "Unemployed after leaving school                              "
    CONTRIB_LAB_TEST(2) = "Unemployed after employment                                  "
    CONTRIB_LAB_TEST(3) = "Unknown status of unemployed                                 "
    
    !Write results
    Write(15,'(/,''                                                                        Aantal in steekproef       &
    &TotSampW  Aantal extra uren PW'')')
    Write(15,'(''Counts of unemployed who loose their benefit because of time restriction'')')
    DO i=1,3
        Write(15,'(a75,f10.0,2f15.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,3)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)
    DEALLOCATE(CONTRIB_LAB_TEST)
    
END SUBROUTINE CONTRIB_UNEMINC

!===================================================================================!
!With subroutine CONTRIB_NONLAINC we reconstruct non labour market income for       !
!the different labour market statuses                                               !
!Constructed variables:                                                             !
!   CONTRIB_PENS_QT                                                                 !
!   CONTRIB_UNEMFT_QT                                                               !
!   CONTRIB_UNEMPT_QT                                                               !
!   CONTRIB_DISABWEA_QT                                                             !
!   CONTRIB_INDOCC_QT                                                               !
!===================================================================================!

SUBROUTINE CONTRIB_NONLAINC

    INTEGER:: pers_j

    !Construct non labour market income
    !CONTRIB_PENS_QT          Pension benefits on which social security contributions have to be paid         
    !CONTRIB_UNEMFT_QT        Early retirement benefits of a full time unemployed on which social security contributions have to be paid          
    !CONTRIB_UNEMPT_QT        Early retirement benefits of a part time unemployed on which social security contributions have to be paid          
    !CONTRIB_DISABWEA_QT      Disability benefits received by wage earners on the private or public labour market
    !CONTRIB_INDOCC_QT        Disability benefits received by wage earners because of industrial accidents or occupational diseases (benefits for help of a third person excluded)
    DO pers_j=1,n_rec
        CONTRIB_PENS_QT(pers_j)     = PENSWELF_RETPENS_QT(pers_j) + PENSWELF_SURPENS_QT(pers_j) 
        CONTRIB_UNEMFT_QT(pers_j)   = UNEM_BENUNS1_QT(pers_j) + UNEM_ADDUNS_QT(pers_j) 
        CONTRIB_UNEMPT_QT(pers_j)   = UNEM_BENEMP1_QT(pers_j) + UNEM_ADDEMP_QT(pers_j) 
        CONTRIB_DISABWEA_QT(pers_j) = SICK_DISABWEA_QT(pers_j)
        CONTRIB_INDOCC_QT(pers_j)   = SICK_IND_QT(pers_j) + SICK_OCC_QT(pers_j)
    END DO

END SUBROUTINE CONTRIB_NONLAINC

!=====================================================================================!
!With subroutine CONTRIB_CHARAC we reconstruct other than the basic income variables  !
!used troughout the different subroutines for constructing the social security        !
!contributions                                                                        !
!Constructed variables                                                                !
!For private labour market wage earners and civil servants without statututory sevice !
!    CONTRIB_SIZECOMP_QT                                                              !
!    CONTRIB_REMBPRIV_QT                                                              !
!    CONTRIB_REMBPUB_QT                                                               !
!    CONTRIB_STRUCDED_QT                                                              !
!    CONTRIB_TAUXPT_QT                                                                !
!    CONTRIB_STRUCCAT_QT                                                              !
!    CONTRIB_PLUSPLAN_QT                                                              !
!    CONTRIB_WORKTIME_QT                                                              !
!    CONTRIB_JOBSDED_QT                                                               !
!    CONTRIB_JOBSMONT_QT                                                              !
!    CONTRIB_YOUNGJTR_QT                                                              !
!    CONTRIB_SUBCDED_QT                                                               !
!    CONTRIB_PUBSEDED_QT                                                              !
!    CONTRIB_DOMESDED_QT                                                              !
!    CONTRIB_UNPROFC_QT                                                               !
!    CONTRIB_UNPRERET_QT                                                              !
!    CONTRIB_SOCMARDE_QT                                                              !
!    CONTRIB_SCREADED_QT                                                              !
!    CONTRIB_DRTUGDED_QT                                                              !
!    CONTRIB_UNACTDED_QT                                                              !
!    CONTRIB_4DAYDED_QT                                                               !
!    CONTRIB_UNSMEDED_QT                                                              !
!    CONTRIB_YOUNGW_QT                                                                !
!    CONTRIB_38HDED_QT                                                                !
!For all public labour market wage earners                                            !
!    CONTRIB_CSLOCFED_QT                                                              !
!For self employed                                                                    !
!    CONTRIB_SELFYEAR_QT                                                              !
!    CONTRIB_SELFPCR_QT                                                               !
!    CONTRIB_SELFFIRS_QT                                                              !
!For people with non labour market income                                             !
!    CONTRIB_FAMCHARG_QT                                                              !
!    CONTRIB_EARSTART_QT                                                              !
!======================================================================================

SUBROUTINE CONTRIB_CHARAC

    INTEGER:: pers_j,count_zero
    INTEGER:: YEAR_SELF !Year in which the self employed was first registered as self employed

!    !CONTRIB_SIZECOMP_QT      Company size of the company the employee is working         
!    DO pers_j=1,n_rec
!        CONTRIB_SIZECOMP_QT(pers_j) = CODIRSZ_QT(pers_j) 
!    END DO

    !CONTRIB_REMBPRIV_QT      Basic remuneration concept used to determine social security contributions of wage earners on the private labour market         
    DO pers_j=1,n_rec
        IF  (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
            CONTRIB_REMBPRIV_QT(pers_j) = (CONTRIB_PERSALB_QT/100.D0)*(CONTRIB_INCPRIV_QT(pers_j)+CONTRIB_INCSPRIV_QT(pers_j))
        ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
            CONTRIB_REMBPRIV_QT(pers_j) = CONTRIB_INCPRIV_QT(pers_j)+CONTRIB_INCSPRIV_QT(pers_j)
        END IF
    END DO

    !CONTRIB_REMBPUB_QT       Basic remuneration concept used to determine social security contributions of people on the public labour market          
    DO pers_j=1,n_rec
        IF  (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
            CONTRIB_REMBPUB_QT(pers_j) = (CONTRIB_PERSALB_QT/100.D0)*(CONTRIB_INCPUB_QT(pers_j)+CONTRIB_INCSPUB_QT(pers_j))
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
            CONTRIB_REMBPUB_QT(pers_j) = CONTRIB_INCPUB_QT(pers_j)+CONTRIB_INCSPUB_QT(pers_j)
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.3) THEN
            CONTRIB_REMBPUB_QT(pers_j) = CONTRIB_INCPUB_QT(pers_j) + CONTRIB_SHPUB_QT(pers_j)+CONTRIB_INCSPUB_QT(pers_j)
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.4) THEN
            CONTRIB_REMBPUB_QT(pers_j) = CONTRIB_INCPUB_QT(pers_j) + CONTRIB_SHPUB_QT(pers_j)+CONTRIB_INCSPUB_QT(pers_j)
        END IF
    END DO
    
!Contributions of wage earners on the private labour market              

!    !CONTRIB_STRUCDED_QT      Identifier whether the employer can claim a structural deduction            
!    DO pers_j=1,n_rec
!        !Identification in case of RSZ employee (wage earner on the private labour market or federal civil servant)
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'08'.OR.CODRED_QT(pers_j)(1:2).EQ.'49'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'50'.OR.CODRED_QT(pers_j)(1:2).EQ.'51'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'52'.OR.CODRED_QT(pers_j)(1:2).EQ.'53'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'55'.OR.CODRED_QT(pers_j)(1:2).EQ.'63'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'64'.OR.CODRED_QT(pers_j)(1:2).EQ.'70'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'74'.OR.CODRED_QT(pers_j)(1:2).EQ.'93'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'S0') THEN
!                    CONTRIB_STRUCDED_QT(pers_j) = 1
!        END IF
!        !Identification in case of RSZPPO employee (local civil servant)
!        IF      (STFNCD_QT(pers_j)(1:1).EQ.'M'.OR.STFNCD_QT(pers_j)(1:1).EQ.'N'.OR.&
!        &        STFNCD_QT(pers_j)(1:1).EQ.'P') THEN
!                    CONTRIB_STRUCDED_QT(pers_j) = 1
!        END IF
!    END DO

    !CONTRIB_TAUXPT_QT        Fraction of time performed as a part time worker, compared to a full time worker            
    !value between 0 and 1
!    COUNT_ZERO = 0
    DO pers_j=1,n_rec
!Procdure die werkt maar niet zeer rechtlijnig is
!Werken = bedrag werkbonus ok en VTE's ok, maar in principe zouden we met onze regels een overschatting
!van de VTE's moeten hebben
!        !Compute value that is consitent with observed income
!        IF (MIMOSIS_GRINC_HOUR(pers_j).GT.0.D0) THEN
!            IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).GT.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0) THEN
!                CONTRIB_TAUXPT_QT(pers_j) = (CONTRIB_INCPRIV_QT(pers_j)+CONTRIB_INCSPRIV_QT(pers_j))/&
!                &(38.D0*13.D0*MIMOSIS_GRINC_HOUR(pers_j))
!            ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).GT.0) THEN
!                CONTRIB_TAUXPT_QT(pers_j) = (CONTRIB_INCPUB_QT(pers_j)+CONTRIB_INCSPUB_QT(pers_j))/&
!                &(38.D0*13.D0*MIMOSIS_GRINC_HOUR(pers_j))
!            ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).GT.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).GT.0) THEN
!                CONTRIB_TAUXPT_QT(pers_j) = (CONTRIB_INCPRIV_QT(pers_j)+CONTRIB_INCSPRIV_QT(pers_j)+CONTRIB_INCPUB_QT(pers_j)+&
!                &CONTRIB_INCSPUB_QT(pers_j))/(38.D0*13.D0*MIMOSIS_GRINC_HOUR(pers_j))
!            END IF
!        END IF
!        !If after the above operation the part time indicator is still 0 then use other sources
!        IF (CONTRIB_TAUXPT_QT(pers_j).LT.1.D0) THEN
!            !Value based on observations
!            IF (InRSZ(pers_j).GE.1.AND.InRSZPPO(pers_j).EQ.0) THEN
!                CONTRIB_TAUXPT_QT(pers_j) = DBLE(tauxpt_20084(pers_j))/100.D0
!                IF (t_prest_20084(pers_j).EQ.'F'.AND.tauxpt_20084(pers_j).EQ.0) THEN
!                    CONTRIB_TAUXPT_QT(pers_j) = 1.D0
!                END IF
!            ELSE IF (InRSZ(pers_j).EQ.0.AND.InRSZPPO(pers_j).GE.1) THEN
!                CONTRIB_TAUXPT_QT(pers_j) = DBLE(apltauxpt_20084(pers_j))/100.D0
!                IF (APL_t_prest_20084(pers_j).EQ.'F'.AND.apltauxpt_20084(pers_j).EQ.0) THEN
!                    CONTRIB_TAUXPT_QT(pers_j) = 1.D0
!                END IF
!            ELSE IF (InRSZ(pers_j).GE.1.AND.InRSZPPO(pers_j).GE.1) THEN
!                !If not part time in one of the two then full time
!                IF (t_prest_20084(pers_j).EQ.'F'.OR.APL_t_prest_20084(pers_j).EQ.'F') THEN
!                    CONTRIB_TAUXPT_QT(pers_j) = 1.D0
!                ELSE        
!                    IF (INT(SumSal_2008(pers_j)+APL_SumSal_2008(pers_j)).NE.0) THEN
!                        CONTRIB_TAUXPT_QT(pers_j) = &
!                        &(SumSal_2008(pers_j)/(SumSal_2008(pers_j)+APL_SumSal_2008(pers_j)))*(DBLE(tauxpt_20084(pers_j))/100.D0)+&
!                        &(APL_SumSal_2008(pers_j)/(SumSal_2008(pers_j)+APL_SumSal_2008(pers_j)))&
!                        &*(DBLE(apltauxpt_20084(pers_j))/100.D0)
!                    ELSE
!                        CONTRIB_TAUXPT_QT(pers_j) = &
!                        &(1.D0/2.D0)*(DBLE(tauxpt_20084(pers_j))/100.D0) + &
!                        &(1.D0/2.D0)*(DBLE(apltauxpt_20084(pers_j))/100.D0)
!                    END IF
!                END IF    
!            END IF
!        END IF
!        IF (CONTRIB_TAUXPT_QT(pers_j).GT.0.D0) THEN
!            CONTRIB_TAUXPT_QT(pers_j) = FLOOR(100.D0*(CONTRIB_TAUXPT_QT(pers_j)+0.01D0))/100.D0
!        END IF
!        !Correct for extreme values and round work values
!        IF (CONTRIB_TAUXPT_QT(pers_j).LT.0.D0) THEN
!            CONTRIB_TAUXPT_QT(pers_j) = 0.D0
!        ELSE IF (CONTRIB_TAUXPT_QT(pers_j).GT.0.8D0) THEN            
!            CONTRIB_TAUXPT_QT(pers_j) = 1.D0
!        END IF
!TestProcedure
        !Compute value that is consitent with observed income
        IF (MIMOSIS_GRINC_HOUR(pers_j).GT.0.D0) THEN
            IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).GT.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0) THEN
                CONTRIB_TAUXPT_QT(pers_j) = (CONTRIB_INCPRIV_QT(pers_j)+CONTRIB_INCSPRIV_QT(pers_j))/&
                &(38.D0*13.D0*MIMOSIS_GRINC_HOUR(pers_j))
            ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).GT.0) THEN
                CONTRIB_TAUXPT_QT(pers_j) = (CONTRIB_INCPUB_QT(pers_j)+CONTRIB_INCSPUB_QT(pers_j))/&
                &(38.D0*13.D0*MIMOSIS_GRINC_HOUR(pers_j))
            ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).GT.0.AND.CONTRIB_LABMSTAT_PUB_QT(pers_j).GT.0) THEN
                CONTRIB_TAUXPT_QT(pers_j) = (CONTRIB_INCPRIV_QT(pers_j)+CONTRIB_INCSPRIV_QT(pers_j)+CONTRIB_INCPUB_QT(pers_j)+&
                &CONTRIB_INCSPUB_QT(pers_j))/(38.D0*13.D0*MIMOSIS_GRINC_HOUR(pers_j))
            END IF
        END IF
        !If after the above operation the part time indicator is still 0 then use other sources
        IF (CONTRIB_TAUXPT_QT(pers_j).LT.1.D0) THEN
            !Value based on observations
            IF (InRSZ(pers_j).GE.1.AND.InRSZPPO(pers_j).EQ.0) THEN
                IF (t_prest_20084(pers_j).EQ.'F') THEN
                    CONTRIB_TAUXPT_QT(pers_j) = 1.D0
                END IF
            ELSE IF (InRSZ(pers_j).EQ.0.AND.InRSZPPO(pers_j).GE.1) THEN
                IF (APL_t_prest_20084(pers_j).EQ.'F') THEN
                    CONTRIB_TAUXPT_QT(pers_j) = 1.D0
                END IF
            ELSE IF (InRSZ(pers_j).GE.1.AND.InRSZPPO(pers_j).GE.1) THEN
                !If not part time in one of the two then full time
                IF (t_prest_20084(pers_j).EQ.'F'.OR.APL_t_prest_20084(pers_j).EQ.'F') THEN
                    CONTRIB_TAUXPT_QT(pers_j) = 1.D0
                END IF    
            END IF
        END IF
        IF (CONTRIB_TAUXPT_QT(pers_j).GT.0.D0) THEN
            CONTRIB_TAUXPT_QT(pers_j) = FLOOR(100.D0*(CONTRIB_TAUXPT_QT(pers_j)+0.01D0))/100.D0
        END IF
        !Correct for extreme values and round work values
        IF (CONTRIB_TAUXPT_QT(pers_j).LT.0.D0) THEN
            CONTRIB_TAUXPT_QT(pers_j) = 0.D0
        ELSE IF (CONTRIB_TAUXPT_QT(pers_j).GT.0.8D0) THEN            
            CONTRIB_TAUXPT_QT(pers_j) = 1.D0
        END IF
!        IF (CONTRIB_TAUXPT_QT(pers_j).GT.0.D0) THEN
!            WRITE(15,*) 'TestTaux',CONTRIB_TAUXPT_QT(pers_j),tauxpt_20084(pers_j),apltauxpt_20084(pers_j)
!        END IF
!Einde Test
!        IF ((InRSZ(pers_j).GE.1.OR.InRSZPPO(pers_j).GE.1).AND.INT(100.D0*CONTRIB_TAUXPT_QT(pers_j)).EQ.0) THEN
!            COUNT_ZERO = COUNT_ZERO + 1
!            WRITE(15,*) 'A',tauxpt_20084(pers_j),apltauxpt_20084(pers_j),CONTRIB_TAUXPT_QT(pers_j)
!        END IF
    END DO
!    WRITE(15,*) 'A',COUNT_ZERO
    
!    !CONTRIB_STRUCCAT_QT      Type of structural deduction that employer is allowed to apply for the employee         
!    !1   Blue-collar workers which are employed by employers who pay social security contributions for company closure
!    !2   Workers which are employed by employers of adapted work company
!    !3   Workers which are employed by employers in the non-market sector
!    !4   Other wage earners
!    DO pers_j=1,n_rec
!        IF      (CODFFE_QT(pers_j)(1:1).EQ.'1'.AND.&
!        &      ((NR_CP_QT(pers_j)(1:3).NE.'117'.AND.NR_CP_QT(pers_j)(1:3).NE.'305'.AND.&
!        &        NR_CP_QT(pers_j)(1:3).NE.'306'.AND.NR_CP_QT(pers_j)(1:3).NE.'307'.AND.&
!        &        NR_CP_QT(pers_j)(1:3).NE.'308'.AND.NR_CP_QT(pers_j)(1:3).NE.'309'.AND.&
!        &        NR_CP_QT(pers_j)(1:3).NE.'310'.AND.NR_CP_QT(pers_j)(1:3).NE.'326').OR.&
!        &        NR_CP_QT(pers_j)(1:3).EQ.'318')&
!        &       ) THEN
!                    CONTRIB_STRUCCAT_QT(pers_j) = 1
!        ELSE IF (NR_CP_QT(pers_j)(1:3).EQ.'327') THEN
!                    CONTRIB_STRUCCAT_QT(pers_j) = 2
!        ELSE IF (&  
!        &       (CODRED_QT(pers_j)(1:2).EQ.'22'.OR.CODRED_QT(pers_j)(1:2).EQ.'23'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'24'.OR.CODRED_QT(pers_j)(1:2).EQ.'25'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'26'.OR.CODRED_QT(pers_j)(1:2).EQ.'27'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'28'.OR.CODRED_QT(pers_j)(1:2).EQ.'29'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'30'.OR.CODRED_QT(pers_j)(1:2).EQ.'31'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'33'.OR.CODRED_QT(pers_j)(1:2).EQ.'43'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'44'.OR.CODRED_QT(pers_j)(1:2).EQ.'46'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'49'.OR.CODRED_QT(pers_j)(1:2).EQ.'50'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'54'.OR.CODRED_QT(pers_j)(1:2).EQ.'MS').AND.&
!        &        NR_CP_QT(pers_j)(1:3).NE.'318'.AND.NR_CP_QT(pers_j)(1:3).NE.'327'&
!                &) THEN
!                    CONTRIB_STRUCCAT_QT(pers_j) = 3
!        ELSE IF (ANY(CONTRIB_STRUCCAT_QT(pers_j).NE.(/1,2,3/))) THEN
!                    CONTRIB_STRUCCAT_QT(pers_j) = 4
!        END IF
!    END DO

!    !CONTRIB_PLUSPLAN_QT      Identification that the employee is a plus-plan         
!    !1 Employee employed in Plan+1
!    !2 Employee employed in Plan+2 
!    !3 Employee employed in Plan+3
!    DO pers_j=1,n_rec
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'P1'.OR.CODRED_QT(pers_j)(1:2).EQ.'P4') THEN
!                    CONTRIB_PLUSPLAN_QT(pers_j) = 1
!        ELSE IF (CODRED_QT(pers_j)(1:2).EQ.'P2'.OR.CODRED_QT(pers_j)(1:2).EQ.'P5') THEN
!                    CONTRIB_PLUSPLAN_QT(pers_j) = 2
!        ELSE IF (CODRED_QT(pers_j)(1:2).EQ.'01'.OR.CODRED_QT(pers_j)(1:2).EQ.'78'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'88'.OR.CODRED_QT(pers_j)(1:2).EQ.'P3'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'P6') THEN
!                    CONTRIB_PLUSPLAN_QT(pers_j) = 3
!        END IF
!    END DO

    !CONTRIB_WORKTIME_QT      Number of quarters the employee is employed         
    DO pers_j=1,n_rec
        CONTRIB_WORKTIME_QT(pers_j) = 1
    END DO

!    !CONTRIB_JOBSDED_QT       Employee is employed as job seeker yes or no            
!    DO pers_j=1,n_rec
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'79'.OR.CODRED_QT(pers_j)(1:2).EQ.'89'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'B1'.OR.CODRED_QT(pers_j)(1:2).EQ.'B2'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'B3'.OR.CODRED_QT(pers_j)(1:2).EQ.'B4') THEN
!                    CONTRIB_JOBSDED_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_JOBSMONT_QT      Number of months the job seeker was in a given status before he became employed         
!    DO pers_j=1,n_rec
!        CONTRIB_JOBSMONT_QT(pers_j) = 12
!    END DO

!    !CONTRIB_YOUNGJTR_QT      The individual is young worker in a job training program yes or no          
!    DO pers_j=1,n_rec
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'15'.OR.CODRED_QT(pers_j)(1:2).EQ.'36'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'U1'.OR.CODRED_QT(pers_j)(1:2).EQ.'U2'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'U3') THEN
!                    CONTRIB_YOUNGJTR_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_SUBCDED_QT       Employee is subsidised contract worker          
!    DO pers_j=1,n_rec
!        IF      (CLATR2_QT(pers_j)(1:2).EQ.'ES'.OR.CLATR2_QT(pers_j)(1:2).EQ.'OS') THEN 
!                    CONTRIB_SUBCDED_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_PUBSEDED_QT      The employee is civil servant that replaces other civil servants who reduce their work time         
!    DO pers_j=1,n_rec
!        !Identification in case of RSZ employee (wage earner on the private labour market or federal civil servant)
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'A1') THEN 
!                    CONTRIB_PUBSEDED_QT(pers_j) = 1
!        END IF
!        !Identification in case of RSZPPO employee (local civil servant)
!        IF      (STFNCD_QT(pers_j)(1:1).EQ.'6') THEN
!                    CONTRIB_PUBSEDED_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_DOMESDED_QT      Individual is a domestic worker yes or no           
!    DO pers_j=1,n_rec
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'X0') THEN 
!                    CONTRIB_DOMESDED_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_UNPROFC_QT       The employee was unemployed and is employed part time or full time          
!    !1   Individual is part time employed by other than small company
!    !2   Individual is full time employed by other than small company
!    !3   Individual is full time employed by a small company
!    DO pers_j=1,n_rec
!        !Identification in case of RSZ employee (wage earner on the private labour market or federal civil servant)
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'D1') THEN 
!                    CONTRIB_UNPROFC_QT(pers_j) = 1
!        ELSE IF (CODRED_QT(pers_j)(1:2).EQ.'D2'.OR.CODRED_QT(pers_j)(1:2).EQ.'80'.OR.CODRED_QT(pers_j)(1:2).EQ.'51') THEN 
!                    CONTRIB_UNPROFC_QT(pers_j) = 2
!        ELSE IF (CODRED_QT(pers_j)(1:2).EQ.'D3') THEN 
!                    CONTRIB_UNPROFC_QT(pers_j) = 3
!        END IF
!        !Identification in case of RSZPPO employee (local civil servant)
!        IF      (STFNCD_QT(pers_j)(1:1).EQ.'V') THEN
!                    CONTRIB_UNPROFC_QT(pers_j) = 2
!        END IF
!    END DO

!    !CONTRIB_UNPRERET_QT      The employee was unemployed and is employed part time or full time and replaces a pre retired person            
!    !1   Individual is part time employed by other than small company
!    !2   Individual is full time employed by other than small company
!    !3   Individual is full time employed by a small company
!    DO pers_j=1,n_rec
!        !Identification in case of RSZ employee (wage earner on the private labour market or federal civil servant)
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'G1') THEN 
!                    CONTRIB_UNPRERET_QT(pers_j) = 1
!        ELSE IF (CODRED_QT(pers_j)(1:2).EQ.'G2'.OR.CODRED_QT(pers_j)(1:2).EQ.'52') THEN 
!                    CONTRIB_UNPRERET_QT(pers_j) = 2
!        ELSE IF (CODRED_QT(pers_j)(1:2).EQ.'G3') THEN 
!                    CONTRIB_UNPRERET_QT(pers_j) = 3
!        END IF
!        !Identification in case of RSZPPO employee (local civil servant)
!        IF      (STFNCD_QT(pers_j)(1:1).EQ.'4'.OR.STFNCD_QT(pers_j)(1:1).EQ.'5'.OR.&
!        &        STFNCD_QT(pers_j)(1:1).EQ.'N') THEN
!                    CONTRIB_UNPRERET_QT(pers_j) = 2
!        END IF
!    END DO

!    !CONTRIB_SOCMARDE_QT      Employee generates the right on the social Maribel yes or no            
!    DO pers_j=1,n_rec
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'22'.OR.CODRED_QT(pers_j)(1:2).EQ.'23'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'24'.OR.CODRED_QT(pers_j)(1:2).EQ.'25'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'26'.OR.CODRED_QT(pers_j)(1:2).EQ.'27'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'28'.OR.CODRED_QT(pers_j)(1:2).EQ.'29'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'30'.OR.CODRED_QT(pers_j)(1:2).EQ.'31'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'33'.OR.CODRED_QT(pers_j)(1:2).EQ.'43'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'44'.OR.CODRED_QT(pers_j)(1:2).EQ.'46'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'MS') THEN
!                    CONTRIB_SOCMARDE_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_SCREADED_QT      The employer is allowed to reduce social security contributions because of hiring additional scientific workers         
!    DO pers_j=1,n_rec
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'WO') THEN 
!                    CONTRIB_SCREADED_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_DRTUGDED_QT      Employee generates deduction for activity in the dragging and tugging sector            
!    DO pers_j=1,n_rec
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'42'.OR.CODRED_QT(pers_j)(1:2).EQ.'70'.OR.CODRED_QT(pers_j)(1:2).EQ.'BA') THEN 
!                    CONTRIB_DRTUGDED_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_UNACTDED_QT      The employee was an unemployed that generates a deduction of employer social security contributions         
!    DO pers_j=1,n_rec
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'SB') THEN 
!                    CONTRIB_UNACTDED_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_4DAYDED_QT       The employee enters a 4 day week regime, which entitles the employer a deduction because of job creation            
!    DO pers_j=1,n_rec
!        !Identification in case of RSZ employee (wage earner on the private labour market or federal civil servant)
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'64'.OR.CODRED_QT(pers_j)(1:2).EQ.'S4') THEN 
!                    CONTRIB_4DAYDED_QT(pers_j) = 1
!        END IF
!        !Identification in case of RSZPPO employee (local civil servant)
!        IF      (STFNCD_QT(pers_j)(1:1).EQ.'3'.OR.STFNCD_QT(pers_j)(1:1).EQ.'P') THEN
!                    CONTRIB_4DAYDED_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_UNSMEDED_QT      The employee was a long term unemployed that was difficult to place         
!    DO pers_j=1,n_rec
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'BC') THEN 
!                    CONTRIB_UNSMEDED_QT(pers_j) = 1
!        END IF
!    END DO

!    !CONTRIB_YOUNGW_QT        Identification whether an individual is a young worker after his first job agreement yes or no          
!    DO pers_j=1,n_rec
!        !Identification in case of RSZ employee (wage earner on the private labour market or federal civil servant)
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'10'.OR.CODRED_QT(pers_j)(1:2).EQ.'55'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'74'.OR.CODRED_QT(pers_j)(1:2).EQ.'86'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'F0'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'F1'.OR.CODRED_QT(pers_j)(1:2).EQ.'F2'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'K0'.OR.CODRED_QT(pers_j)(1:2).EQ.'K1'.OR.&
!        &        CODRED_QT(pers_j)(1:2).EQ.'K2'.OR.CODRED_QT(pers_j)(1:2).EQ.'Y0') THEN
!                    CONTRIB_YOUNGW_QT(pers_j) = 1
!        END IF
!        !Identification in case of RSZPPO employee (local civil servant)
!        IF      (STFNCD_QT(pers_j)(1:1).EQ.'7') THEN
!                    CONTRIB_YOUNGW_QT(pers_j) = 1
!        END IF
!    END DO
  
!    !CONTRIB_38HDED_QT        The employee generates a deduction of social security contributions because he switches from a 39 hours week to a 38 hours week         
!    DO pers_j=1,n_rec
!        IF      (CODRED_QT(pers_j)(1:2).EQ.'R1') THEN
!                    CONTRIB_38HDED_QT(pers_j) = 1
!        END IF
!    END DO

!!Contributions of wage earners on the public labour market               

!    !CONTRIB_CSLOCFED_QT      Indicator whether civil servant works for a local authority or for the federal government           
!    !1   Civil servant working for local authority
!    !2   Civil servant working at the federal level
!    DO pers_j=1,n_rec
!        IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1,2,3,4/))) THEN
!            IF (TB2_QT(pers_j).EQ.1) THEN
!                CONTRIB_CSLOCFED_QT(pers_j) = 1
!            ELSE
!                CONTRIB_CSLOCFED_QT(pers_j) = 2
!            END IF                
!        END IF
!    END DO

!!Contributions of self employed              

    !CONTRIB_SELFYEAR_QT      Number of years the self employed is self employed          
    DO pers_j=1,n_rec
        IF (CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.1) THEN
            !identify the slash that identifies the end of the year registration
            CONTRIB_PAR_BREAK_POINT(1)=INDEX(BEGAANSC_2008(pers_j),'/')
            READ (BEGAANSC_2008(pers_j)(1:CONTRIB_PAR_BREAK_POINT(1)-1),  '(I10)') YEAR_SELF
            CONTRIB_SELFYEAR_QT(pers_j) = MIMOSIS_YEAR - YEAR_SELF
            IF (CONTRIB_SELFYEAR_QT(pers_j).LE.0) THEN
                CONTRIB_SELFYEAR_QT(pers_j) = 1
            END IF
        END IF            
    END DO
    
    !CONTRIB_SELFPCR_QT       Status of the self employed         
    !1   Self employed before the age of retirement  in principal activity
    !2   Self employed before the age of retirement in complementary activity
    !3   Self employed after the age of retirement
    !4   Self employed assistant before the age of retirement  in principal activity
    !5   Self employed assistant  before the age of retirement in complementary activity
    !6   Self employed assistant after the age of retirement
    DO pers_j=1,n_rec
        !identify self employed
        IF (HOEDRSVZ_2008(pers_j)(1:1).EQ.'1'.OR.HOEDRSVZ_2008(pers_j)(1:1).EQ.'3') THEN
            IF      (BIJDCAT_2008(pers_j)(1:1).EQ.'A'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'H'.OR.&
            &        BIJDCAT_2008(pers_j)(1:1).EQ.'I'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'J'.OR.&
            &        BIJDCAT_2008(pers_j)(1:1).EQ.'K'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'O'.OR.&
            &        BIJDCAT_2008(pers_j)(1:1).EQ.'Q'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'R'.OR.&
            &        BIJDCAT_2008(pers_j)(1:1).EQ.'S'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'U') THEN
                        CONTRIB_SELFPCR_QT(pers_j) = 1
            ELSE IF (BIJDCAT_2008(pers_j)(1:1).EQ.'D') THEN
                        CONTRIB_SELFPCR_QT(pers_j) = 2
            ELSE IF (BIJDCAT_2008(pers_j)(1:1).EQ.'E'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'F'.OR.&
            &        BIJDCAT_2008(pers_j)(1:1).EQ.'Y') THEN
                        CONTRIB_SELFPCR_QT(pers_j) = 3
            END IF
        !identify self employed assistants
        ELSE IF (HOEDRSVZ_2008(pers_j)(1:1).EQ.'2'.OR.HOEDRSVZ_2008(pers_j)(1:1).EQ.'4') THEN
            IF      (BIJDCAT_2008(pers_j)(1:1).EQ.'A'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'H'.OR.&
            &        BIJDCAT_2008(pers_j)(1:1).EQ.'I'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'J'.OR.&
            &        BIJDCAT_2008(pers_j)(1:1).EQ.'K'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'O'.OR.&
            &        BIJDCAT_2008(pers_j)(1:1).EQ.'Q'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'R'.OR.&
            &        BIJDCAT_2008(pers_j)(1:1).EQ.'S'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'U') THEN
                        CONTRIB_SELFPCR_QT(pers_j) = 4
            ELSE IF (BIJDCAT_2008(pers_j)(1:1).EQ.'D') THEN
                        CONTRIB_SELFPCR_QT(pers_j) = 5
            ELSE IF (BIJDCAT_2008(pers_j)(1:1).EQ.'E'.OR.BIJDCAT_2008(pers_j)(1:1).EQ.'F'.OR.&
            &        BIJDCAT_2008(pers_j)(1:1).EQ.'Y') THEN
                        CONTRIB_SELFPCR_QT(pers_j) = 6
            END IF
        END IF
    END DO

    !CONTRIB_SELFFIRS_QT      Self employed establishes his activity as self employed for the first time          
    DO pers_j=1,n_rec
        IF  (CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.1) THEN
            CONTRIB_SELFFIRS_QT(pers_j) = 1
        END IF
    END DO

!!Contributions on non labour market income               
    
    !CONTRIB_FAMCHARG_QT A pensioner is identified to have family charge yes or no           
    DO pers_j=1,n_rec
        IF (PIT_DEPCHILD_ANN(pers_j).GT.0) THEN
            CONTRIB_FAMCHARG_QT(pers_j) = 1
        END IF
    END DO

    !CONTRIB_EARSTART_QT      Identification whether early retirement  is of the old or the new type          
    !1 if the individual became early retired before 1997 
    !2 if the individual became early retired in 1997 or afterwards
    DO pers_j=1,n_rec
        IF (CONTRIB_UNEMFT_QT(pers_j).GT.0.OR.CONTRIB_UNEMPT_QT(pers_j).GT.0) THEN
            CONTRIB_EARSTART_QT(pers_j) = 2
        END IF
    END DO
    
    !CONTRIB_FULLPART_QT: variable that contains the value that expresses hours worked as a fraction of the time performed by a full timer
    !loop over all households
    DO hh_i=1,N_HH_CONTRIB
        !loop over all members of the sociological household
        DO pers_j=CONTRIB_START_HH(hh_i),CONTRIB_END_HH(hh_i) 
            IF (CONTRIB_HOURSIM.EQ.0) THEN
                CONTRIB_FULLPART_QT(pers_j) = CONTRIB_TAUXPT_QT(pers_j)
                !Vul aantal uren werk in om later door te geven aan unem
                CONTRIB_WORKHOUR(pers_j) = CONTRIB_TAUXPT_QT(pers_j)*38.D0
            ELSE
                CONTRIB_FULLPART_QT(pers_j) = CONTRIB_WORKHOUR(pers_j)/38.D0
            END IF
        END DO
    END DO
    
END SUBROUTINE CONTRIB_CHARAC

!===================================================================================!
!With subroutine CONTRIB_SSBEFDED_PRIVATE we reconstruct the social security        !
!contributions of private labour market wage earners before any of the possible     !
!deductions has been deducted                                                       !
!Variables constructed in this subroutine                                           !
!   CONTRIB_WEARCOM_EMEE_QT                                                         !
!   CONTRIB_WEARCOM_EMER_QT                                                         !
!   CONTRIB_WEARHOLA_EMER_QT                                                        !
!   CONTRIB_WEARHOLD_EMEE_QT                                                        !
!===================================================================================!

SUBROUTINE CONTRIB_SSBEFDED_PRIVATE(pers_j)

    INTEGER:: pers_j,i
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_PERCENT_DH                  !Variable to store percentage to compute contribution on double holiday earnings 
    
    !Compute common social security contributions paid for wage earners on the private labour market
    !either by the employee or the employer
    !CONTRIB_WEARCOM_EMEE_QT  Common social security contributions paid by the employee working as wage earner on the private labour market           
    !CONTRIB_WEARCOM_EMER_QT  Common social security contributions paid by the employer for an employee working as wage earner on the private labour market           
    DO i=1,SIZE(CONTRIB_WEARCOM_QT,1)
        !compute for blue collar workers
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
            !If not a percentage that depends on company size
            IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13,20,21/))) THEN
                CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,1)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,2)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
            !If company size should be taken into account
            ELSE IF (ANY(i.EQ.(/14,15,16,17,18,19/))) THEN
                !If 10 or more workers in the company
                IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,1)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,2)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,1)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,2)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                !If less than 20 workers                    
                ELSE IF (i.EQ.16.AND.CONTRIB_SIZECOMP_QT(pers_j).LE.3) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,1)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,2)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                ELSE IF (i.EQ.17.AND.CONTRIB_SIZECOMP_QT(pers_j).LE.3) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,1)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,2)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                !If 20 workers or more                                      
                ELSE IF (i.EQ.18.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.3) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,1)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,2)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                ELSE IF (i.EQ.19.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.3) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,1)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,2)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                END IF    
            END IF 
        !compute for white collar workers
        ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
            !If not a percentage that depends on company size
            IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13,20,21/))) THEN
                CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,4)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,5)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
            !If company size should be taken into account
            ELSE IF (ANY(i.EQ.(/14,15,16,17,18,19/))) THEN
                !If 10 or more workers in the company
                IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,4)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,5)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,4)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,5)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                !If less than 20 workers                    
                ELSE IF (i.EQ.16.AND.CONTRIB_SIZECOMP_QT(pers_j).LE.3) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,4)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,5)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                ELSE IF (i.EQ.17.AND.CONTRIB_SIZECOMP_QT(pers_j).LE.3) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,4)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,5)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                !If 20 workers or more                                      
                ELSE IF (i.EQ.18.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.3) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,4)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,5)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                ELSE IF (i.EQ.19.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.3) THEN
                    CONTRIB_WEARCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,4)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    CONTRIB_WEARCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,5)/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                END IF    
            END IF 
        END IF
    END DO 

    !Compute social security contributions for holiday earnings of blue collar workers
    !CONTRIB_WEARHOLA_EMER_QT Social security contribution paid for holiday earnings of blue collar workers on the private labour market          
    IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
        IF (INT(1000.D0*MIMOSIS_PRIINDEX_ANN).NE.0) THEN
            CONTRIB_WEARHOLA_EMER_QT(pers_j) = (CONTRIB_WEARHOLA_QT/100.D0)*(CONTRIB_REMBPRIV_QT(pers_j)/MIMOSIS_PRIINDEX_ANN)
        END IF            
    END IF
    
    !Compute social security contributions paid on double holiday earnings
    !CONTRIB_WEARHOLD_EMEE_QT Social security contributions paid on double holiday earnings of wage earners on the private labour market          
    IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
        CONTRIB_PERCENT_DH = SUM(CONTRIB_WEARCOM_QT(1:7,1))
        CONTRIB_WEARHOLD_EMEE_QT(pers_j) = (CONTRIB_PERCENT_DH/100.D0)*(CONTRIB_WEARHOLD_QT/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)  
    ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
        CONTRIB_PERCENT_DH = SUM(CONTRIB_WEARCOM_QT(1:7,4))
        CONTRIB_WEARHOLD_EMEE_QT(pers_j) = (CONTRIB_PERCENT_DH/100.D0)*CONTRIB_DHPRIV_QT(pers_j)        
    END IF
    
END SUBROUTINE CONTRIB_SSBEFDED_PRIVATE

!====================================================================================!
!With subroutine CONTRIB_SSDEDU_PRIVATE_2001 we reconstruct the deductions applied on!
!social security contributions of private labour market wage earners                 !
!Variables constructed in this subroutine                                            !
!   CONTRIB_WEARDEDU_EMER_QT                                                         !
!   CONTRIB_WEARDEDU_EMEE_QT                                                         !
!====================================================================================!

SUBROUTINE CONTRIB_SSDEDU_PRIVATE_2001(pers_j)

    INTEGER:: pers_j
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_PERCENT_DEDU                  !Variable to store percentage that is used to compute the reduction as a fraction of another amount
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SUM_DEDU                      !Variable to store the sum of all possible deductions in case a limit should be applied on this sum
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_INCCONC_LWDEDU                !Variable to store the income concept on which the low wage employee contribution deduction is applied

    !Contstruct deductions on employer social security contributions
    !CONTRIB_WEARDEDU_EMER_QT Deductions of employer social security contributions permitted for wage earners on the private labour market            

    !employer dedu 1:  THE STRUCTURAL DEDUCTION OF EMPLOYERS CONTRIBUTIONS: GENERAL CASE
        !first compute the deduction as if the employee is a full time employee
        IF (CONTRIB_STRUCDED_QT(pers_j).EQ.1) THEN
            !Determine the relevant income concept (i.e. the full time equivalent)
            IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).EQ.0) THEN
                CONTRIB_INCCONC_LWDEDU = CONTRIB_REMBPRIV_QT(pers_j) 
            ELSE
                CONTRIB_INCCONC_LWDEDU = (CONTRIB_REMBPRIV_QT(pers_j))/CONTRIB_FULLPART_QT(pers_j)
            END IF
            !Use the function CONTRIB_BRACRATE_4COLUMNS_MIN and apply it on the
            !appropriate matrix that is read in as parameter in the module
            IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.1) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_SD1_QT,1),CONTRIB_SD1_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.2) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_SD2_QT,1),CONTRIB_SD2_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.3) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_SD3_QT,1),CONTRIB_SD3_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.4) THEN         
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_SD4_QT,1),CONTRIB_SD4_QT,GECODEERD_INSZ_INPUT(pers_j))
            END IF 
        END IF 
        !correct the structural deduction if the employee is part time employee
        IF (CONTRIB_STRUCDED_QT(pers_j).EQ.1) THEN
            IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.33) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = 0
            ELSE IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).GE.33.AND.&
            &INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.81) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = CONTRIB_FULLPART_QT(pers_j)*CONTRIB_WEARDEDU_EMER_QT(pers_j,1)
            END IF
        END IF            
    !employer dedu 2:  THE DEDUCTION OF EMPLOYERS CONTRIBUTIONS TO STIMULATE EMPLOYMENT FOR NON MARKET ACTIVITIES
        !this deduction is not reconstructed
        CONTRIB_WEARDEDU_EMER_QT(pers_j,2) = 0
    !employer dedu 3:  TARGET-GROUP DEDUCTION: PLANS +1, +2 AND +3 DEDUCTIONS
        IF (ANY(CONTRIB_PLUSPLAN_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_2STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_2STATINM(CONTRIB_PLUSPLAN_QT(pers_j),CONTRIB_WORKTIME_QT(pers_j),&
            &SIZE(CONTRIB_PLUSDEDU_QT,1),CONTRIB_PLUSDEDU_QT,GECODEERD_INSZ_INPUT(pers_j))
            !Then compute the total deduction as the sum of contributions due on a fraction of the remunerations
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,10,11,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) + &
                        &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,10,11,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3)+ &
                        &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 4:  TARGET-GROUP DEDUCTION: PLAN FOR JOB-SEEKERS
        IF (CONTRIB_JOBSDED_QT(pers_j).EQ.1) THEN
            !Use the function CONTRIB_SELPERC_5COLUMNS_3STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_5COLUMNS_3STATINM(CONTRIB_JOBSDED_QT(pers_j),&
            &CONTRIB_JOBSMONT_QT(pers_j),CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_JOBSDEDA_QT,1),CONTRIB_JOBSDEDA_QT&
            &,GECODEERD_INSZ_INPUT(pers_j))
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 5:  TARGET-GROUP DEDUCTION: JOB-TRAINING DEDUCTIONS (AR. 495)
        IF (CONTRIB_YOUNGJTR_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 6:  TARGET-GROUP DEDUCTION: SUBSIDIZED CONTRACT WORKER
        IF (CONTRIB_SUBCDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,14
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,6) = CONTRIB_WEARDEDU_EMER_QT(pers_j,6) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,6) = CONTRIB_WEARDEDU_EMER_QT(pers_j,6) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,6) = CONTRIB_WEARDEDU_EMER_QT(pers_j,6)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,6) = CONTRIB_WEARDEDU_EMER_QT(pers_j,6) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 7:  TARGET-GROUP DEDUCTION: DEDUCTION FOR JOB REDISTRIBUTION IN THE PUBLIC SECTOR
        IF (CONTRIB_PUBSEDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,14
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,7) = CONTRIB_WEARDEDU_EMER_QT(pers_j,7) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,7) = CONTRIB_WEARDEDU_EMER_QT(pers_j,7) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,7) = CONTRIB_WEARDEDU_EMER_QT(pers_j,7)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,7) = CONTRIB_WEARDEDU_EMER_QT(pers_j,7) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 8:  TARGET-GROUP DEDUCTION: (AR. 230)
        !this deduction is not reconstructed
        CONTRIB_WEARDEDU_EMER_QT(pers_j,8) = 0
    !employer dedu 9:  TARGET-GROUP DEDUCTION: THE DEDUCTION FOR HIRING DISADVANTAGED YOUNG PEOPLE (AR. 499)
        !this deduction is not reconstructed
        CONTRIB_WEARDEDU_EMER_QT(pers_j,9) = 0
    !employer dedu 10: TARGET-GROUP DEDUCTION: THE DOMESTIC PERSON DEDUCTION (AR. 483)
        IF (CONTRIB_DOMESDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 11: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR PROFESSIONAL CAREER INTERRUPTION
        IF (ANY(CONTRIB_UNPROFC_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            IF (CONTRIB_UNPROFC_QT(pers_j).EQ.1) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPROFC_QT(pers_j),&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPROFDP_QT,1),CONTRIB_UNPROFDP_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (ANY(CONTRIB_UNPROFC_QT(pers_j).EQ.(/2,3/))) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPROFC_QT(pers_j)-1,&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPROFDF_QT,1),CONTRIB_UNPROFDF_QT,GECODEERD_INSZ_INPUT(pers_j))
            END IF
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 12: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR HALF-TIME PRE-RETIREMENT
        IF (ANY(CONTRIB_UNPRERET_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            IF (CONTRIB_UNPRERET_QT(pers_j).EQ.1) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPRERET_QT(pers_j),&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPRERDP_QT,1),CONTRIB_UNPRERDP_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (ANY(CONTRIB_UNPRERET_QT(pers_j).EQ.(/2,3/))) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPRERET_QT(pers_j)-1,&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPRERDF_QT,1),CONTRIB_UNPRERDF_QT,GECODEERD_INSZ_INPUT(pers_j))
            END IF
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 13: THE STRUCTURAL DEDUCTION OF EMPLOYERS CONTRIBUTIONS: STIMULATION OF EMPLOYMENT FOR NON MARKET ACTIVITIES (SOCIAL MARIBEL)
        IF (CONTRIB_SOCMARDE_QT(pers_j).EQ.1) THEN
            CONTRIB_WEARDEDU_EMER_QT(pers_j,13) = CONTRIB_SOCMAR_QT
        END IF
    !employer dedu 14: TARGET-GROUP DEDUCTION: THE SCIENTIFIC RESEARCH DEDUCTION
        IF (CONTRIB_SCREADED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 15: TARGET-GROUP DEDUCTION: LABOUR TIME REDUCTIONS BY COLLECTIVE AGREEMENT GRANTED BY THE MINISTER OF EMPLOYMENT AND LABOUR
        !this deduction is not reconstructed
        CONTRIB_WEARDEDU_EMER_QT(pers_j,15) = 0
    !employer dedu 16: TARGET-GROUP DEDUCTION: DRAGGING AND TUGGING SECTOR DEDUCTION
        IF (CONTRIB_DRTUGDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 17: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR ACTIVATION OF UNEMPLOYMENT ALLOWANCES
        IF (CONTRIB_UNACTDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 18: TARGET-GROUP DEDUCTION: THE 4-DAYS WEEK DEDUCTION
        IF (CONTRIB_4DAYDED_QT(pers_j).EQ.1) THEN
            !Use the function CONTRIB_SELPERC_3COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_3COLUMNS_1STATINM(CONTRIB_WORKTIME_QT(pers_j),&
            &SIZE(CONTRIB_4DAYDEDA_QT,1),CONTRIB_4DAYDEDA_QT,GECODEERD_INSZ_INPUT(pers_j))
            CONTRIB_SUM_DEDU = 0
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO
            IF (CONTRIB_SUM_DEDU.GT.CONTRIB_4DAYDEDB_QT) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,18) = CONTRIB_4DAYDEDB_QT
            ELSE
                CONTRIB_WEARDEDU_EMER_QT(pers_j,18) = CONTRIB_SUM_DEDU
            END IF
        END IF            
    !employer dedu 19: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR UNEMPLOYED PERSONS DIFFICULT TO PLACE(SMET JOB)
        IF (CONTRIB_UNSMEDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 20: TARGET-GROUP DEDUCTION: JOB STARTERS (ROSETTA PLAN)
        IF (CONTRIB_YOUNGW_QT(pers_j).EQ.1) THEN
            CONTRIB_WEARDEDU_EMER_QT(pers_j,20) = (CONTRIB_YWDED_QT/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
        END IF
    !employer dedu 21: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR TIME WORKING REDUCTION TO 38 HOURS PER WEEK
        IF (CONTRIB_38HDED_QT(pers_j).EQ.1) THEN
            IF (SUM(CONTRIB_WEARCOM_EMER_QT(pers_j,1:21)).LT.CONTRIB_38HDEDA_QT) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,21) = SUM(CONTRIB_WEARCOM_EMER_QT(pers_j,1:21))
            ELSE
                CONTRIB_WEARDEDU_EMER_QT(pers_j,21) = CONTRIB_38HDEDA_QT
            END IF
        END IF

    !Contstruct deductions on employee social security contributions
    !CONTRIB_WEARDEDU_EMEE_QT Deductions of employee social security contributions permitted for wage earners on the private labour market            
        CONTRIB_INCCONC_LWDEDU = MIMOSIS_GRINC_HOUR(pers_j)*38.D0*4.3333D0
!        !First contsturct the appropriate income concept        
!        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
!            IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).EQ.0) THEN
!                CONTRIB_INCCONC_LWDEDU = CONTRIB_INCPRIV_QT(pers_j) + CONTRIB_SHPRIV_QT(pers_j) 
!            ELSE
!                CONTRIB_INCCONC_LWDEDU = (CONTRIB_INCPRIV_QT(pers_j) + CONTRIB_SHPRIV_QT(pers_j))/CONTRIB_FULLPART_QT(pers_j)
!            END IF
!        ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
!            IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).EQ.0) THEN
!                CONTRIB_INCCONC_LWDEDU = CONTRIB_INCPRIV_QT(pers_j) 
!            ELSE
!                CONTRIB_INCCONC_LWDEDU = (CONTRIB_INCPRIV_QT(pers_j))/CONTRIB_FULLPART_QT(pers_j)
!            END IF
!        END IF
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
            CONTRIB_WEARDEDU_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
            &SIZE(CONTRIB_PDEDBL_MONTH,1),CONTRIB_PDEDBL_MONTH,GECODEERD_INSZ_INPUT(pers_j))
        ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
            CONTRIB_WEARDEDU_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
            &SIZE(CONTRIB_PDEDWI_MONTH,1),CONTRIB_PDEDWI_MONTH,GECODEERD_INSZ_INPUT(pers_j))
        END IF
        !Correct the deduction in case of part time work
        IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.100) THEN
            CONTRIB_WEARDEDU_EMEE_QT(pers_j) = CONTRIB_FULLPART_QT(pers_j)*CONTRIB_WEARDEDU_EMEE_QT(pers_j)
        END IF

END SUBROUTINE CONTRIB_SSDEDU_PRIVATE_2001

!=====================================================================================!
!With subroutine CONTRIB_SSDEDU_PRIVATE_2004 we reconstruct the deductions applied on !
!social security contributions of private labour market wage earners                  !
!Variables constructed in this subroutine                                             !
!   CONTRIB_WEARDEDU_EMER_QT                                                          !
!   CONTRIB_WEARDEDU_EMEE_QT                                                          !
!=====================================================================================!

SUBROUTINE CONTRIB_SSDEDU_PRIVATE_2004(pers_j)

    INTEGER:: pers_j
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_PERCENT_DEDU                  !Variable to store percentage that is used to compute the reduction as a fraction of another amount
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SUM_DEDU                      !Variable to store the sum of all possible deductions in case a limit should be applied on this sum
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_INCCONC_LWDEDU                !Variable to store the income concept on which the low wage employee contribution deduction is applied

    !Contstruct deductions on employer social security contributions
    !CONTRIB_WEARDEDU_EMER_QT Deductions of employer social security contributions permitted for wage earners on the private labour market            

    !employer dedu 1:  THE STRUCTURAL DEDUCTION OF EMPLOYERS CONTRIBUTIONS: GENERAL CASE
        !first compute the deduction as if the employee is a full time employee
        IF (CONTRIB_STRUCDED_QT(pers_j).EQ.1) THEN
            !Determine the relevant income concept (i.e. the full time equivalent)
            IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).EQ.0) THEN
                CONTRIB_INCCONC_LWDEDU = CONTRIB_REMBPRIV_QT(pers_j) 
            ELSE
                CONTRIB_INCCONC_LWDEDU = (CONTRIB_REMBPRIV_QT(pers_j))/CONTRIB_FULLPART_QT(pers_j)
            END IF
            !Use the function CONTRIB_BRACRATE_5COLUMNS_MIN and apply it on the
            !appropriate line of the matrix that is read in as parameter in the module
            IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.1) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_5COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &CONTRIB_REMBPRIV_QT(pers_j),SIZE(CONTRIB_STRU2008_QT,1),CONTRIB_STRU2008_QT,1,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.2) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_5COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &CONTRIB_REMBPRIV_QT(pers_j),SIZE(CONTRIB_STRU2008_QT,1),CONTRIB_STRU2008_QT,2,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.3) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_5COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &CONTRIB_REMBPRIV_QT(pers_j),SIZE(CONTRIB_STRU2008_QT,1),CONTRIB_STRU2008_QT,3,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.4) THEN         
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_5COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &CONTRIB_REMBPRIV_QT(pers_j),SIZE(CONTRIB_STRU2008_QT,1),CONTRIB_STRU2008_QT,4,GECODEERD_INSZ_INPUT(pers_j))
            END IF 
        END IF 
        !correct the structural deduction if the employee is part time employee
        IF (CONTRIB_STRUCDED_QT(pers_j).EQ.1) THEN
            IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.33) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = 0
            ELSE IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).GE.33.AND.&
            &INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.81) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,1) = CONTRIB_FULLPART_QT(pers_j)*CONTRIB_WEARDEDU_EMER_QT(pers_j,1)
            END IF
        END IF            
    !employer dedu 2:  THE DEDUCTION OF EMPLOYERS CONTRIBUTIONS TO STIMULATE EMPLOYMENT FOR NON MARKET ACTIVITIES
        !this deduction is not reconstructed
        CONTRIB_WEARDEDU_EMER_QT(pers_j,2) = 0
    !employer dedu 3:  TARGET-GROUP DEDUCTION: PLANS +1, +2 AND +3 DEDUCTIONS
        IF (ANY(CONTRIB_PLUSPLAN_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_2STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_2STATINM(CONTRIB_PLUSPLAN_QT(pers_j),CONTRIB_WORKTIME_QT(pers_j),&
            &SIZE(CONTRIB_PLUSDEDU_QT,1),CONTRIB_PLUSDEDU_QT,GECODEERD_INSZ_INPUT(pers_j))
            !Then compute the total deduction as the sum of contributions due on a fraction of the remunerations
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,10,11,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) + &
                        &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,10,11,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3)+ &
                        &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,3) = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 4:  TARGET-GROUP DEDUCTION: PLAN FOR JOB-SEEKERS
        IF (CONTRIB_JOBSDED_QT(pers_j).EQ.1) THEN
            !Use the function CONTRIB_SELPERC_5COLUMNS_3STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_5COLUMNS_3STATINM(CONTRIB_JOBSDED_QT(pers_j),&
            &CONTRIB_JOBSMONT_QT(pers_j),CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_JOBSDEDA_QT,1),CONTRIB_JOBSDEDA_QT&
            &,GECODEERD_INSZ_INPUT(pers_j))
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,4) = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 5:  TARGET-GROUP DEDUCTION: JOB-TRAINING DEDUCTIONS (AR. 495)
        IF (CONTRIB_YOUNGJTR_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,5) = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 6:  TARGET-GROUP DEDUCTION: SUBSIDIZED CONTRACT WORKER
        IF (CONTRIB_SUBCDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,14
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,6) = CONTRIB_WEARDEDU_EMER_QT(pers_j,6) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,6) = CONTRIB_WEARDEDU_EMER_QT(pers_j,6) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,6) = CONTRIB_WEARDEDU_EMER_QT(pers_j,6)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,6) = CONTRIB_WEARDEDU_EMER_QT(pers_j,6) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 7:  TARGET-GROUP DEDUCTION: DEDUCTION FOR JOB REDISTRIBUTION IN THE PUBLIC SECTOR
        IF (CONTRIB_PUBSEDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,14
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,7) = CONTRIB_WEARDEDU_EMER_QT(pers_j,7) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,7) = CONTRIB_WEARDEDU_EMER_QT(pers_j,7) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,7) = CONTRIB_WEARDEDU_EMER_QT(pers_j,7)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,7) = CONTRIB_WEARDEDU_EMER_QT(pers_j,7) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 8:  TARGET-GROUP DEDUCTION: (AR. 230)
        !this deduction is not reconstructed
        CONTRIB_WEARDEDU_EMER_QT(pers_j,8) = 0
    !employer dedu 9:  TARGET-GROUP DEDUCTION: THE DEDUCTION FOR HIRING DISADVANTAGED YOUNG PEOPLE (AR. 499)
        !this deduction is not reconstructed
        CONTRIB_WEARDEDU_EMER_QT(pers_j,9) = 0
    !employer dedu 10: TARGET-GROUP DEDUCTION: THE DOMESTIC PERSON DEDUCTION (AR. 483)
        IF (CONTRIB_DOMESDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 11: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR PROFESSIONAL CAREER INTERRUPTION
        IF (ANY(CONTRIB_UNPROFC_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            IF (CONTRIB_UNPROFC_QT(pers_j).EQ.1) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPROFC_QT(pers_j),&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPROFDP_QT,1),CONTRIB_UNPROFDP_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (ANY(CONTRIB_UNPROFC_QT(pers_j).EQ.(/2,3/))) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPROFC_QT(pers_j)-1,&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPROFDF_QT,1),CONTRIB_UNPROFDF_QT,GECODEERD_INSZ_INPUT(pers_j))
            END IF
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 12: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR HALF-TIME PRE-RETIREMENT
        IF (ANY(CONTRIB_UNPRERET_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            IF (CONTRIB_UNPRERET_QT(pers_j).EQ.1) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPRERET_QT(pers_j),&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPRERDP_QT,1),CONTRIB_UNPRERDP_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (ANY(CONTRIB_UNPRERET_QT(pers_j).EQ.(/2,3/))) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPRERET_QT(pers_j)-1,&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPRERDF_QT,1),CONTRIB_UNPRERDF_QT,GECODEERD_INSZ_INPUT(pers_j))
            END IF
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 13: THE STRUCTURAL DEDUCTION OF EMPLOYERS CONTRIBUTIONS: STIMULATION OF EMPLOYMENT FOR NON MARKET ACTIVITIES (SOCIAL MARIBEL)
        IF (CONTRIB_SOCMARDE_QT(pers_j).EQ.1) THEN
            CONTRIB_WEARDEDU_EMER_QT(pers_j,13) = CONTRIB_SOCMAR_QT
        END IF
    !employer dedu 14: TARGET-GROUP DEDUCTION: THE SCIENTIFIC RESEARCH DEDUCTION
        IF (CONTRIB_SCREADED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 15: TARGET-GROUP DEDUCTION: LABOUR TIME REDUCTIONS BY COLLECTIVE AGREEMENT GRANTED BY THE MINISTER OF EMPLOYMENT AND LABOUR
        !this deduction is not reconstructed
        CONTRIB_WEARDEDU_EMER_QT(pers_j,15) = 0
    !employer dedu 16: TARGET-GROUP DEDUCTION: DRAGGING AND TUGGING SECTOR DEDUCTION
        IF (CONTRIB_DRTUGDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 17: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR ACTIVATION OF UNEMPLOYMENT ALLOWANCES
        IF (CONTRIB_UNACTDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 18: TARGET-GROUP DEDUCTION: THE 4-DAYS WEEK DEDUCTION
        IF (CONTRIB_4DAYDED_QT(pers_j).EQ.1) THEN
            !Use the function CONTRIB_SELPERC_3COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_3COLUMNS_1STATINM(CONTRIB_WORKTIME_QT(pers_j),&
            &SIZE(CONTRIB_4DAYDEDA_QT,1),CONTRIB_4DAYDEDA_QT,GECODEERD_INSZ_INPUT(pers_j))
            CONTRIB_SUM_DEDU = 0
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO
            IF (CONTRIB_SUM_DEDU.GT.CONTRIB_4DAYDEDB_QT) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,18) = CONTRIB_4DAYDEDB_QT
            ELSE
                CONTRIB_WEARDEDU_EMER_QT(pers_j,18) = CONTRIB_SUM_DEDU
            END IF
        END IF            
    !employer dedu 19: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR UNEMPLOYED PERSONS DIFFICULT TO PLACE(SMET JOB)
        IF (CONTRIB_UNSMEDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19) + &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19)+ &
                        &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_WEARDEDU_EMER_QT(pers_j,19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19) + &
                            &CONTRIB_WEARCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 20: TARGET-GROUP DEDUCTION: JOB STARTERS (ROSETTA PLAN)
        IF (CONTRIB_YOUNGW_QT(pers_j).EQ.1) THEN
            CONTRIB_WEARDEDU_EMER_QT(pers_j,20) = (CONTRIB_YWDED_QT/100.D0)*CONTRIB_REMBPRIV_QT(pers_j)
        END IF
    !employer dedu 21: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR TIME WORKING REDUCTION TO 38 HOURS PER WEEK
        IF (CONTRIB_38HDED_QT(pers_j).EQ.1) THEN
            IF (SUM(CONTRIB_WEARCOM_EMER_QT(pers_j,1:21)).LT.CONTRIB_38HDEDA_QT) THEN
                CONTRIB_WEARDEDU_EMER_QT(pers_j,21) = SUM(CONTRIB_WEARCOM_EMER_QT(pers_j,1:21))
            ELSE
                CONTRIB_WEARDEDU_EMER_QT(pers_j,21) = CONTRIB_38HDEDA_QT
            END IF
        END IF

    !Contstruct deductions on employee social security contributions
    !CONTRIB_WEARDEDU_EMEE_QT Deductions of employee social security contributions permitted for wage earners on the private labour market            
        CONTRIB_INCCONC_LWDEDU = MIMOSIS_GRINC_HOUR(pers_j)*38.D0*4.3333D0
        !Then use the function CONTRIB_BRACRATE_4COLUMNS_MIN or CONTRIB_BRACRATE_4COLUMNS_WB2009
        !to select the appropriate deduction amount
        IF (SIMYEAR.LT.2009) THEN
            IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                CONTRIB_WEARDEDU_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_PDEDBL_MONTH,1),CONTRIB_PDEDBL_MONTH,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                CONTRIB_WEARDEDU_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_PDEDWI_MONTH,1),CONTRIB_PDEDWI_MONTH,GECODEERD_INSZ_INPUT(pers_j))
            END IF
        ELSE
            IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                CONTRIB_WEARDEDU_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_WB2009(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_PDEDBL_MONTH,1),CONTRIB_PDEDBL_MONTH,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                CONTRIB_WEARDEDU_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_WB2009(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_PDEDWI_MONTH,1),CONTRIB_PDEDWI_MONTH,GECODEERD_INSZ_INPUT(pers_j))
            END IF
        END IF
        !Correct the deduction in case of part time work
        IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.100) THEN
            CONTRIB_WEARDEDU_EMEE_QT(pers_j) = CONTRIB_FULLPART_QT(pers_j)*CONTRIB_WEARDEDU_EMEE_QT(pers_j)
        END IF

END SUBROUTINE CONTRIB_SSDEDU_PRIVATE_2004

!===================================================================================!
!With subroutine CONTRIB_SSNET_PRIVATE we reconstruct the social security           !
!contributions of private labour market wage earners net of deductions              !
!Variables constructed in this subroutine                                           !
!   CONTRIB_SSPRIV_EMER_QT                                                          !
!   CONTRIB_SSPRIV_EMEE_QT                                                          !
!===================================================================================!

SUBROUTINE CONTRIB_SSNET_PRIVATE(pers_j)

    INTEGER:: pers_j,i
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_WEARCOM_EMEE_QT,2)+1)    :: CONTRIB_WEARCOM_EMEE_NET         !Employee contributions of wage earners on the private labour market net of deductions
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_WEARCOM_EMER_QT,2)+1)    :: CONTRIB_WEARCOM_EMER_NET         !Employer contributions of wage earners on the private labour market net of deductions
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMEE_SUMCON      !Total sum of employee contributions of wage earners on the private labour market before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMER_SUMCON      !Total sum of employer contributions of wage earners on the private labour market before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMEE_SUMDED      !Total sum of employee deductions of wage earners on the private labour market before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMER_SUMDED      !Total sum of employer deductions of wage earners on the private labour market   

    !CONTRIBUTIONS OF THE EMPLOYEE
    !Make store variables empty
    CONTRIB_WEARCOM_EMEE_NET(1:SIZE(CONTRIB_WEARCOM_EMEE_NET,1)) = 0.D0
    CONTRIB_WEARCOM_EMEE_SUMCON = 0.D0
    CONTRIB_WEARCOM_EMEE_SUMDED = 0.D0
    !Make the sum of all employee contributions
    CONTRIB_WEARCOM_EMEE_SUMCON =   SUM(CONTRIB_WEARCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_EMEE_QT,2))) + &
                                    &CONTRIB_WEARHOLD_EMEE_QT(pers_j)
    !Make the sum of all employee deductions
    CONTRIB_WEARCOM_EMEE_SUMDED =   CONTRIB_WEARDEDU_EMEE_QT(pers_j)
    !Compare the personal contributions with the personal deductions
    IF (CONTRIB_WEARCOM_EMEE_SUMDED.GT.CONTRIB_WEARCOM_EMEE_SUMCON) THEN
        CONTRIB_WEARDEDU_EMEE_QT(pers_j) = CONTRIB_WEARCOM_EMEE_SUMCON
    END IF
    !recompute the sum of all employee deductions
    CONTRIB_WEARCOM_EMEE_SUMDED = CONTRIB_WEARDEDU_EMEE_QT(pers_j)
    !Compute the contributions net of deductions
    IF (INT(100.D0*CONTRIB_WEARCOM_EMEE_SUMCON).NE.0) THEN
        !Contributions in the matrix
        DO i=1,SIZE(CONTRIB_WEARCOM_EMEE_QT,2)
            CONTRIB_WEARCOM_EMEE_NET(i) = CONTRIB_WEARCOM_EMEE_QT(pers_j,i) - &
            &CONTRIB_WEARCOM_EMEE_SUMDED*(CONTRIB_WEARCOM_EMEE_QT(pers_j,i)/CONTRIB_WEARCOM_EMEE_SUMCON)        
        END DO
        !contributions on double holiday earnings
        CONTRIB_WEARCOM_EMEE_NET(SIZE(CONTRIB_WEARCOM_EMEE_NET,1)) = CONTRIB_WEARHOLD_EMEE_QT(pers_j) - &
        &CONTRIB_WEARCOM_EMEE_SUMDED*(CONTRIB_WEARHOLD_EMEE_QT(pers_j)/CONTRIB_WEARCOM_EMEE_SUMCON)        
    END IF
    !Make the sum of all employee contributions net of deductions
    CONTRIB_SSPRIV_EMEE_QT(pers_j) = SUM(CONTRIB_WEARCOM_EMEE_NET(1:SIZE(CONTRIB_WEARCOM_EMEE_NET,1))) 

    !CONTRIBUTIONS OF THE EMPLOYER
    !Make store variables empty
    CONTRIB_WEARCOM_EMER_NET(1:SIZE(CONTRIB_WEARCOM_EMER_NET,1)) = 0.D0
    CONTRIB_WEARCOM_EMER_SUMCON = 0.D0
    CONTRIB_WEARCOM_EMER_SUMDED = 0.D0
    !Make the sum of all employer contributions
    CONTRIB_WEARCOM_EMER_SUMCON =   SUM(CONTRIB_WEARCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_EMER_QT,2))) + &
                                    &CONTRIB_WEARHOLA_EMER_QT(pers_j)
    !Make the sum of all employer deductions
    CONTRIB_WEARCOM_EMER_SUMDED =   SUM(CONTRIB_WEARDEDU_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARDEDU_EMER_QT,2))) 
    !Compare the employer contributions with the employer deductions
    IF (CONTRIB_WEARCOM_EMER_SUMDED.GT.CONTRIB_WEARCOM_EMER_SUMCON.AND.INT(100.D0*CONTRIB_WEARCOM_EMER_SUMDED).NE.0) THEN
        DO i=1,SIZE(CONTRIB_WEARDEDU_EMER_QT,2)
            CONTRIB_WEARDEDU_EMER_QT(pers_j,i) = &
            &CONTRIB_WEARCOM_EMER_SUMCON*(CONTRIB_WEARDEDU_EMER_QT(pers_j,i)/CONTRIB_WEARCOM_EMER_SUMDED)        
        END DO
    END IF
    !recompute the sum of all employer deductions
    CONTRIB_WEARCOM_EMER_SUMDED = SUM(CONTRIB_WEARDEDU_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARDEDU_EMER_QT,2))) 
    !Compute the contributions net of deductions
    IF (INT(100.D0*CONTRIB_WEARCOM_EMER_SUMCON).NE.0) THEN
        !Contributions in the matrix
        DO i=1,SIZE(CONTRIB_WEARCOM_EMER_QT,2)
            CONTRIB_WEARCOM_EMER_NET(i) = CONTRIB_WEARCOM_EMER_QT(pers_j,i) - &
            &CONTRIB_WEARCOM_EMER_SUMDED*(CONTRIB_WEARCOM_EMER_QT(pers_j,i)/CONTRIB_WEARCOM_EMER_SUMCON)        
        END DO
        !contributions on holiday earnings for blue collar workers
        CONTRIB_WEARCOM_EMER_NET(SIZE(CONTRIB_WEARCOM_EMER_NET,1)) = CONTRIB_WEARHOLA_EMER_QT(pers_j) - &
        &CONTRIB_WEARCOM_EMER_SUMDED*(CONTRIB_WEARHOLA_EMER_QT(pers_j)/CONTRIB_WEARCOM_EMER_SUMCON)        
    END IF
    !Make the sum of all employer contributions net of deductions
    CONTRIB_SSPRIV_EMER_QT(pers_j) = SUM(CONTRIB_WEARCOM_EMER_NET(1:SIZE(CONTRIB_WEARCOM_EMER_NET,1))) 

END SUBROUTINE CONTRIB_SSNET_PRIVATE

!===================================================================================!
!With subroutine CONTRIB_SSBEFDED_PUBNSTAT we reconstruct the social security       !
!contributions of public labour market wage earners without statutory service       !
!before any of the possible deductions has been deducted                            !
!REMARK: applied parameters are those of the private labour market since civil      !
!servants without statutory service are treated in the same way as private labour   !
!market wage earners                                                                !
!Variables constructed in this subroutine                                           !
!   CONTRIB_PUBNSCOM_EMEE_QT                                                        !
!   CONTRIB_PUBNSCOM_EMER_QT                                                        !
!   CONTRIB_PUBNSHOL_EMER_QT                                                        !
!   CONTRIB_PUBNSHOL_EMEE_QT                                                        !
!===================================================================================!

SUBROUTINE CONTRIB_SSBEFDED_PUBNSTAT(pers_j)

    INTEGER:: pers_j,i
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_PERCENT_DH                  !Variable to store percentage to compute contribution on double holiday earnings 
    
    !Compute common social security contributions paid for wage earners on the public labour market without statutory service
    !either by the employee or the employer
    !CONTRIB_PUBNSCOM_EMEE_QT Common social security contributions paid by the employee working as wage earner on the public labour market but with non statutory service         
    !CONTRIB_PUBNSCOM_EMER_QT Common social security contributions paid by the employer for an employee working as wage earner on the public labour market but with non statutory service            
    DO i=1,SIZE(CONTRIB_WEARCOM_QT,1)
        !Remark: rates for company closure fund are not applied here since we compute contributions for civil servants here
        !compute for blue collar workers
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
            !If not a percentage that depends on company size
            IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13/))) THEN
                CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,1)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                CONTRIB_PUBNSCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,2)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
            !If company size should be taken into account
            ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                !If 10 or more workers in the company
                IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                    CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,1)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                    CONTRIB_PUBNSCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,2)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                    CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,1)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                    CONTRIB_PUBNSCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,2)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                END IF    
            END IF 
        !compute for white collar workers
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
            !If not a percentage that depends on company size
            IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13/))) THEN
                CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,4)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                CONTRIB_PUBNSCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,5)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
            !If company size should be taken into account
            ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                !If 10 or more workers in the company
                IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                    CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,4)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                    CONTRIB_PUBNSCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,5)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                    CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,4)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                    CONTRIB_PUBNSCOM_EMER_QT(pers_j,i) = (CONTRIB_WEARCOM_QT(i,5)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                END IF    
            END IF 
        END IF
    END DO 

    !Compute social security contributions for holiday earnings of blue collar workers
    !CONTRIB_PUBNSHOL_EMER_QT Social security contribution paid for holiday earnings of blue collar workers on the public labour market but with non statutory service           
    !CONTRIB_PUBNSHOL_EMEE_QT Social security contributions paid on double holiday earnings of wage earners on the public labour market but with non statutory service           
    IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
        IF (INT(1000.D0*MIMOSIS_PRIINDEX_ANN).NE.0) THEN
            CONTRIB_PUBNSHOL_EMER_QT(pers_j) = (CONTRIB_WEARHOLA_QT/100.D0)*(CONTRIB_REMBPUB_QT(pers_j)/MIMOSIS_PRIINDEX_ANN)
        END IF            
    END IF
    
    !Compute social security contributions paid on double holiday earnings
    IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
        CONTRIB_PERCENT_DH = SUM(CONTRIB_WEARCOM_QT(1:7,1))
        CONTRIB_PUBNSHOL_EMEE_QT(pers_j) = (CONTRIB_PERCENT_DH/100.D0)*(CONTRIB_WEARHOLD_QT/100.D0)*CONTRIB_REMBPUB_QT(pers_j)  
    ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
        CONTRIB_PERCENT_DH = SUM(CONTRIB_WEARCOM_QT(1:7,4))
        CONTRIB_PUBNSHOL_EMEE_QT(pers_j) = (CONTRIB_PERCENT_DH/100.D0)*CONTRIB_DHPUB_QT(pers_j)        
    END IF
    
END SUBROUTINE CONTRIB_SSBEFDED_PUBNSTAT

!====================================================================================!
!With subroutine CONTRIB_SSDEDU_PUBNSTAT_2001 we reconstruct the deductions applied on    !
!social security contributions of public labour market wage earners without statutory!
!service                                                                             !
!REMARK: applied parameters are those of the private labour market since civil       !
!servants without statutory service are treated in the same way as private labour    !
!market wage earners                                                                 !
!Variables constructed in this subroutine                                            !
!   CONTRIB_PUBNSDED_EMER_QT                                                         !
!   CONTRIB_PUBNSDED_EMEE_QT                                                         !
!====================================================================================!

SUBROUTINE CONTRIB_SSDEDU_PUBNSTAT_2001(pers_j)

    INTEGER:: pers_j
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_PERCENT_DEDU                  !Variable to store percentage that is used to compute the reduction as a fraction of another amount
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SUM_DEDU                      !Variable to store the sum of all possible deductions in case a limit should be applied on this sum
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_INCCONC_LWDEDU                !Variable to store the income concept on which the low wage employee contribution deduction is applied

    !Contstruct deductions on employer social security contributions
    !CONTRIB_PUBNSDED_EMER_QT Deductions of employer social security contributions permitted for wage earners on the public labour market but with non statutory service             

    !employer dedu 1:  THE STRUCTURAL DEDUCTION OF EMPLOYERS CONTRIBUTIONS: GENERAL CASE
        !first compute the deduction as if the employee is a full time employee
        IF (CONTRIB_STRUCDED_QT(pers_j).EQ.1) THEN
            !Determine the relevant income concept (i.e. the full time equivalent)
            IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).EQ.0) THEN
                CONTRIB_INCCONC_LWDEDU = CONTRIB_REMBPUB_QT(pers_j) 
            ELSE
                CONTRIB_INCCONC_LWDEDU = (CONTRIB_REMBPUB_QT(pers_j))/CONTRIB_FULLPART_QT(pers_j)
            END IF
            !Use the function CONTRIB_BRACRATE_4COLUMNS_MIN and apply it on the
            !appropriate matrix that is read in as parameter in the module
            IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.1) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_SD1_QT,1),CONTRIB_SD1_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.2) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_SD2_QT,1),CONTRIB_SD2_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.3) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_SD3_QT,1),CONTRIB_SD3_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.4) THEN         
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_SD4_QT,1),CONTRIB_SD4_QT,GECODEERD_INSZ_INPUT(pers_j))
            END IF 
        END IF 
        !correct the structural deduction if the employee is part time employee
        IF (CONTRIB_STRUCDED_QT(pers_j).EQ.1) THEN
            IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.33) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = 0
            ELSE IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).GE.33.AND.&
            &INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.81) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = CONTRIB_FULLPART_QT(pers_j)*CONTRIB_PUBNSDED_EMER_QT(pers_j,1)
            END IF
        END IF            
    !employer dedu 2:  THE DEDUCTION OF EMPLOYERS CONTRIBUTIONS TO STIMULATE EMPLOYMENT FOR NON MARKET ACTIVITIES
        !this deduction is not reconstructed
        CONTRIB_PUBNSDED_EMER_QT(pers_j,2) = 0
    !employer dedu 3:  TARGET-GROUP DEDUCTION: PLANS +1, +2 AND +3 DEDUCTIONS
        IF (ANY(CONTRIB_PLUSPLAN_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_2STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_2STATINM(CONTRIB_PLUSPLAN_QT(pers_j),CONTRIB_WORKTIME_QT(pers_j),&
            &SIZE(CONTRIB_PLUSDEDU_QT,1),CONTRIB_PLUSDEDU_QT,GECODEERD_INSZ_INPUT(pers_j))
            !Then compute the total deduction as the sum of contributions due on a fraction of the remunerations
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,10,11,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) + &
                        &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,10,11,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3)+ &
                        &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 4:  TARGET-GROUP DEDUCTION: PLAN FOR JOB-SEEKERS
        IF (CONTRIB_JOBSDED_QT(pers_j).EQ.1) THEN
            !Use the function CONTRIB_SELPERC_5COLUMNS_3STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_5COLUMNS_3STATINM(CONTRIB_JOBSDED_QT(pers_j),&
            &CONTRIB_JOBSMONT_QT(pers_j),CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_JOBSDEDA_QT,1),CONTRIB_JOBSDEDA_QT,&
            &GECODEERD_INSZ_INPUT(pers_j))
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 5:  TARGET-GROUP DEDUCTION: JOB-TRAINING DEDUCTIONS (AR. 495)
        IF (CONTRIB_YOUNGJTR_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 6:  TARGET-GROUP DEDUCTION: SUBSIDIZED CONTRACT WORKER
        IF (CONTRIB_SUBCDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,14
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,6) = CONTRIB_PUBNSDED_EMER_QT(pers_j,6) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,6) = CONTRIB_PUBNSDED_EMER_QT(pers_j,6) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,6) = CONTRIB_PUBNSDED_EMER_QT(pers_j,6)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,6) = CONTRIB_PUBNSDED_EMER_QT(pers_j,6) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 7:  TARGET-GROUP DEDUCTION: DEDUCTION FOR JOB REDISTRIBUTION IN THE PUBLIC SECTOR
        IF (CONTRIB_PUBSEDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,14
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,7) = CONTRIB_PUBNSDED_EMER_QT(pers_j,7) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,7) = CONTRIB_PUBNSDED_EMER_QT(pers_j,7) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,7) = CONTRIB_PUBNSDED_EMER_QT(pers_j,7)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,7) = CONTRIB_PUBNSDED_EMER_QT(pers_j,7) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 8:  TARGET-GROUP DEDUCTION: (AR. 230)
        !this deduction is not reconstructed
        CONTRIB_PUBNSDED_EMER_QT(pers_j,8) = 0
    !employer dedu 9:  TARGET-GROUP DEDUCTION: THE DEDUCTION FOR HIRING DISADVANTAGED YOUNG PEOPLE (AR. 499)
        !this deduction is not reconstructed
        CONTRIB_PUBNSDED_EMER_QT(pers_j,9) = 0
    !employer dedu 10: TARGET-GROUP DEDUCTION: THE DOMESTIC PERSON DEDUCTION (AR. 483)
        IF (CONTRIB_DOMESDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 11: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR PROFESSIONAL CAREER INTERRUPTION
        IF (ANY(CONTRIB_UNPROFC_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            IF (CONTRIB_UNPROFC_QT(pers_j).EQ.1) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPROFC_QT(pers_j),&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPROFDP_QT,1),CONTRIB_UNPROFDP_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (ANY(CONTRIB_UNPROFC_QT(pers_j).EQ.(/2,3/))) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPROFC_QT(pers_j)-1,&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPROFDF_QT,1),CONTRIB_UNPROFDF_QT,GECODEERD_INSZ_INPUT(pers_j))
            END IF
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 12: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR HALF-TIME PRE-RETIREMENT
        IF (ANY(CONTRIB_UNPRERET_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            IF (CONTRIB_UNPRERET_QT(pers_j).EQ.1) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPRERET_QT(pers_j),&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPRERDP_QT,1),CONTRIB_UNPRERDP_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (ANY(CONTRIB_UNPRERET_QT(pers_j).EQ.(/2,3/))) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPRERET_QT(pers_j)-1,&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPRERDF_QT,1),CONTRIB_UNPRERDF_QT,GECODEERD_INSZ_INPUT(pers_j))
            END IF
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 13: THE STRUCTURAL DEDUCTION OF EMPLOYERS CONTRIBUTIONS: STIMULATION OF EMPLOYMENT FOR NON MARKET ACTIVITIES (SOCIAL MARIBEL)
        IF (CONTRIB_SOCMARDE_QT(pers_j).EQ.1) THEN
            CONTRIB_PUBNSDED_EMER_QT(pers_j,13) = CONTRIB_SOCMAR_QT
        END IF
    !employer dedu 14: TARGET-GROUP DEDUCTION: THE SCIENTIFIC RESEARCH DEDUCTION
        IF (CONTRIB_SCREADED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 15: TARGET-GROUP DEDUCTION: LABOUR TIME REDUCTIONS BY COLLECTIVE AGREEMENT GRANTED BY THE MINISTER OF EMPLOYMENT AND LABOUR
        !this deduction is not reconstructed
        CONTRIB_PUBNSDED_EMER_QT(pers_j,15) = 0
    !employer dedu 16: TARGET-GROUP DEDUCTION: DRAGGING AND TUGGING SECTOR DEDUCTION
        IF (CONTRIB_DRTUGDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 17: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR ACTIVATION OF UNEMPLOYMENT ALLOWANCES
        IF (CONTRIB_UNACTDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 18: TARGET-GROUP DEDUCTION: THE 4-DAYS WEEK DEDUCTION
        IF (CONTRIB_4DAYDED_QT(pers_j).EQ.1) THEN
            !Use the function CONTRIB_SELPERC_3COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_3COLUMNS_1STATINM(CONTRIB_WORKTIME_QT(pers_j),&
            &SIZE(CONTRIB_4DAYDEDA_QT,1),CONTRIB_4DAYDEDA_QT,GECODEERD_INSZ_INPUT(pers_j))
            CONTRIB_SUM_DEDU = 0
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO
            IF (CONTRIB_SUM_DEDU.GT.CONTRIB_4DAYDEDB_QT) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,18) = CONTRIB_4DAYDEDB_QT
            ELSE
                CONTRIB_PUBNSDED_EMER_QT(pers_j,18) = CONTRIB_SUM_DEDU
            END IF
        END IF            
    !employer dedu 19: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR UNEMPLOYED PERSONS DIFFICULT TO PLACE(SMET JOB)
        IF (CONTRIB_UNSMEDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 20: TARGET-GROUP DEDUCTION: JOB STARTERS (ROSETTA PLAN)
        IF (CONTRIB_YOUNGW_QT(pers_j).EQ.1) THEN
            CONTRIB_PUBNSDED_EMER_QT(pers_j,20) = (CONTRIB_YWDED_QT/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
        END IF
    !employer dedu 21: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR TIME WORKING REDUCTION TO 38 HOURS PER WEEK
        IF (CONTRIB_38HDED_QT(pers_j).EQ.1) THEN
            IF (SUM(CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:21)).LT.CONTRIB_38HDEDA_QT) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,21) = SUM(CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:21))
            ELSE
                CONTRIB_PUBNSDED_EMER_QT(pers_j,21) = CONTRIB_38HDEDA_QT
            END IF
        END IF

    !Contstruct deductions on employee social security contributions
    !CONTRIB_PUBNSDED_EMEE_QT Deductions of employee social security contributions permitted for wage earners on the public labour market but with non statutory service             
        CONTRIB_INCCONC_LWDEDU = MIMOSIS_GRINC_HOUR(pers_j)*38.D0*4.3333D0
        !Then use the function CONTRIB_BRACRATE_4COLUMNS_MIN or CONTRIB_BRACRATE_4COLUMNS_WB2009
        !to select the appropriate deduction amount
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
            CONTRIB_PUBNSDED_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
            &SIZE(CONTRIB_PDEDBL_MONTH,1),CONTRIB_PDEDBL_MONTH,GECODEERD_INSZ_INPUT(pers_j))
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
            CONTRIB_PUBNSDED_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
            &SIZE(CONTRIB_PDEDWI_MONTH,1),CONTRIB_PDEDWI_MONTH,GECODEERD_INSZ_INPUT(pers_j))
        END IF
        !Correct the deduction in case of part time work
        IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.100) THEN
            CONTRIB_PUBNSDED_EMEE_QT(pers_j) = CONTRIB_FULLPART_QT(pers_j)*CONTRIB_PUBNSDED_EMEE_QT(pers_j)
        END IF

END SUBROUTINE CONTRIB_SSDEDU_PUBNSTAT_2001

!====================================================================================!
!With subroutine CONTRIB_SSDEDU_PUBNSTAT_2004 we reconstruct the deductions applied on    !
!social security contributions of public labour market wage earners without statutory!
!service                                                                             !
!REMARK: applied parameters are those of the private labour market since civil       !
!servants without statutory service are treated in the same way as private labour    !
!market wage earners                                                                 !
!Variables constructed in this subroutine                                            !
!   CONTRIB_PUBNSDED_EMER_QT                                                         !
!   CONTRIB_PUBNSDED_EMEE_QT                                                         !
!====================================================================================!

SUBROUTINE CONTRIB_SSDEDU_PUBNSTAT_2004(pers_j)

    INTEGER:: pers_j
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_PERCENT_DEDU                  !Variable to store percentage that is used to compute the reduction as a fraction of another amount
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SUM_DEDU                      !Variable to store the sum of all possible deductions in case a limit should be applied on this sum
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_INCCONC_LWDEDU                !Variable to store the income concept on which the low wage employee contribution deduction is applied

    !Contstruct deductions on employer social security contributions
    !CONTRIB_PUBNSDED_EMER_QT Deductions of employer social security contributions permitted for wage earners on the public labour market but with non statutory service             

    !employer dedu 1:  THE STRUCTURAL DEDUCTION OF EMPLOYERS CONTRIBUTIONS: GENERAL CASE
        !first compute the deduction as if the employee is a full time employee
        IF (CONTRIB_STRUCDED_QT(pers_j).EQ.1) THEN
            !Determine the relevant income concept (i.e. the full time equivalent)
            IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).EQ.0) THEN
                CONTRIB_INCCONC_LWDEDU = CONTRIB_REMBPUB_QT(pers_j) 
            ELSE
                CONTRIB_INCCONC_LWDEDU = (CONTRIB_REMBPUB_QT(pers_j))/CONTRIB_FULLPART_QT(pers_j)
            END IF
            !Use the function CONTRIB_BRACRATE_5COLUMNS_MIN and apply it on the
            !appropriate line of the matrix that is read in as parameter in the module
            IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.1) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_5COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &CONTRIB_REMBPUB_QT(pers_j),SIZE(CONTRIB_STRU2008_QT,1),CONTRIB_STRU2008_QT,1,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.2) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_5COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &CONTRIB_REMBPUB_QT(pers_j),SIZE(CONTRIB_STRU2008_QT,1),CONTRIB_STRU2008_QT,2,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.3) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_5COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &CONTRIB_REMBPUB_QT(pers_j),SIZE(CONTRIB_STRU2008_QT,1),CONTRIB_STRU2008_QT,3,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_STRUCCAT_QT(pers_j).EQ.4) THEN         
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = CONTRIB_BRACRATE_5COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &CONTRIB_REMBPUB_QT(pers_j),SIZE(CONTRIB_STRU2008_QT,1),CONTRIB_STRU2008_QT,4,GECODEERD_INSZ_INPUT(pers_j))
            END IF 
        END IF 
        !correct the structural deduction if the employee is part time employee
        IF (CONTRIB_STRUCDED_QT(pers_j).EQ.1) THEN
            IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.33) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = 0
            ELSE IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).GE.33.AND.&
            &INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.81) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,1) = CONTRIB_FULLPART_QT(pers_j)*CONTRIB_PUBNSDED_EMER_QT(pers_j,1)
            END IF
        END IF            
    !employer dedu 2:  THE DEDUCTION OF EMPLOYERS CONTRIBUTIONS TO STIMULATE EMPLOYMENT FOR NON MARKET ACTIVITIES
        !this deduction is not reconstructed
        CONTRIB_PUBNSDED_EMER_QT(pers_j,2) = 0
    !employer dedu 3:  TARGET-GROUP DEDUCTION: PLANS +1, +2 AND +3 DEDUCTIONS
        IF (ANY(CONTRIB_PLUSPLAN_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_2STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_2STATINM(CONTRIB_PLUSPLAN_QT(pers_j),CONTRIB_WORKTIME_QT(pers_j),&
            &SIZE(CONTRIB_PLUSDEDU_QT,1),CONTRIB_PLUSDEDU_QT,GECODEERD_INSZ_INPUT(pers_j))
            !Then compute the total deduction as the sum of contributions due on a fraction of the remunerations
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,10,11,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) + &
                        &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,2)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,10,11,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3)+ &
                        &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,3) = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) + &
                            &(CONTRIB_WEARCOM_QT(i,5)/100.D0)*(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 4:  TARGET-GROUP DEDUCTION: PLAN FOR JOB-SEEKERS
        IF (CONTRIB_JOBSDED_QT(pers_j).EQ.1) THEN
            !Use the function CONTRIB_SELPERC_5COLUMNS_3STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_5COLUMNS_3STATINM(CONTRIB_JOBSDED_QT(pers_j),&
            &CONTRIB_JOBSMONT_QT(pers_j),CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_JOBSDEDA_QT,1),CONTRIB_JOBSDEDA_QT,&
            &GECODEERD_INSZ_INPUT(pers_j))
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,4) = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 5:  TARGET-GROUP DEDUCTION: JOB-TRAINING DEDUCTIONS (AR. 495)
        IF (CONTRIB_YOUNGJTR_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,5) = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 6:  TARGET-GROUP DEDUCTION: SUBSIDIZED CONTRACT WORKER
        IF (CONTRIB_SUBCDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,14
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,6) = CONTRIB_PUBNSDED_EMER_QT(pers_j,6) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,6) = CONTRIB_PUBNSDED_EMER_QT(pers_j,6) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,6) = CONTRIB_PUBNSDED_EMER_QT(pers_j,6)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,6) = CONTRIB_PUBNSDED_EMER_QT(pers_j,6) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 7:  TARGET-GROUP DEDUCTION: DEDUCTION FOR JOB REDISTRIBUTION IN THE PUBLIC SECTOR
        IF (CONTRIB_PUBSEDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,14
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,7) = CONTRIB_PUBNSDED_EMER_QT(pers_j,7) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,7) = CONTRIB_PUBNSDED_EMER_QT(pers_j,7) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,7) = CONTRIB_PUBNSDED_EMER_QT(pers_j,7)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,7) = CONTRIB_PUBNSDED_EMER_QT(pers_j,7) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 8:  TARGET-GROUP DEDUCTION: (AR. 230)
        !this deduction is not reconstructed
        CONTRIB_PUBNSDED_EMER_QT(pers_j,8) = 0
    !employer dedu 9:  TARGET-GROUP DEDUCTION: THE DEDUCTION FOR HIRING DISADVANTAGED YOUNG PEOPLE (AR. 499)
        !this deduction is not reconstructed
        CONTRIB_PUBNSDED_EMER_QT(pers_j,9) = 0
    !employer dedu 10: TARGET-GROUP DEDUCTION: THE DOMESTIC PERSON DEDUCTION (AR. 483)
        IF (CONTRIB_DOMESDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 11: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR PROFESSIONAL CAREER INTERRUPTION
        IF (ANY(CONTRIB_UNPROFC_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            IF (CONTRIB_UNPROFC_QT(pers_j).EQ.1) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPROFC_QT(pers_j),&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPROFDP_QT,1),CONTRIB_UNPROFDP_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (ANY(CONTRIB_UNPROFC_QT(pers_j).EQ.(/2,3/))) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPROFC_QT(pers_j)-1,&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPROFDF_QT,1),CONTRIB_UNPROFDF_QT,GECODEERD_INSZ_INPUT(pers_j))
            END IF
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 12: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR HALF-TIME PRE-RETIREMENT
        IF (ANY(CONTRIB_UNPRERET_QT(pers_j).EQ.(/1,2,3/))) THEN
            !Use the function CONTRIB_SELPERC_4COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            IF (CONTRIB_UNPRERET_QT(pers_j).EQ.1) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPRERET_QT(pers_j),&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPRERDP_QT,1),CONTRIB_UNPRERDP_QT,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (ANY(CONTRIB_UNPRERET_QT(pers_j).EQ.(/2,3/))) THEN
                CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_4COLUMNS_1STATINM(CONTRIB_UNPRERET_QT(pers_j)-1,&
                &CONTRIB_WORKTIME_QT(pers_j),SIZE(CONTRIB_UNPRERDF_QT,1),CONTRIB_UNPRERDF_QT,GECODEERD_INSZ_INPUT(pers_j))
            END IF
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12)+ &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 13: THE STRUCTURAL DEDUCTION OF EMPLOYERS CONTRIBUTIONS: STIMULATION OF EMPLOYMENT FOR NON MARKET ACTIVITIES (SOCIAL MARIBEL)
        IF (CONTRIB_SOCMARDE_QT(pers_j).EQ.1) THEN
            CONTRIB_PUBNSDED_EMER_QT(pers_j,13) = CONTRIB_SOCMAR_QT
        END IF
    !employer dedu 14: TARGET-GROUP DEDUCTION: THE SCIENTIFIC RESEARCH DEDUCTION
        IF (CONTRIB_SCREADED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 15: TARGET-GROUP DEDUCTION: LABOUR TIME REDUCTIONS BY COLLECTIVE AGREEMENT GRANTED BY THE MINISTER OF EMPLOYMENT AND LABOUR
        !this deduction is not reconstructed
        CONTRIB_PUBNSDED_EMER_QT(pers_j,15) = 0
    !employer dedu 16: TARGET-GROUP DEDUCTION: DRAGGING AND TUGGING SECTOR DEDUCTION
        IF (CONTRIB_DRTUGDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 17: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR ACTIVATION OF UNEMPLOYMENT ALLOWANCES
        IF (CONTRIB_UNACTDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,8,9,10,11,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 18: TARGET-GROUP DEDUCTION: THE 4-DAYS WEEK DEDUCTION
        IF (CONTRIB_4DAYDED_QT(pers_j).EQ.1) THEN
            !Use the function CONTRIB_SELPERC_3COLUMNS_1STATINM to select the appropriate
            !percentage that is then applied on the remunerations for the computation of the deductions
            CONTRIB_PERCENT_DEDU = CONTRIB_SELPERC_3COLUMNS_1STATINM(CONTRIB_WORKTIME_QT(pers_j),&
            &SIZE(CONTRIB_4DAYDEDA_QT,1),CONTRIB_4DAYDEDA_QT,GECODEERD_INSZ_INPUT(pers_j))
            CONTRIB_SUM_DEDU = 0
            !Then compute the total deduction as the sum of a fraction of the obliged contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                        &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_SUM_DEDU = CONTRIB_SUM_DEDU + &
                            &(CONTRIB_PERCENT_DEDU/100.D0)*CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO
            IF (CONTRIB_SUM_DEDU.GT.CONTRIB_4DAYDEDB_QT) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,18) = CONTRIB_4DAYDEDB_QT
            ELSE
                CONTRIB_PUBNSDED_EMER_QT(pers_j,18) = CONTRIB_SUM_DEDU
            END IF
        END IF            
    !employer dedu 19: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR UNEMPLOYED PERSONS DIFFICULT TO PLACE(SMET JOB)
        IF (CONTRIB_UNSMEDED_QT(pers_j).EQ.1) THEN
            !Compute the total deduction as the sum of several different contributions
            DO i=1,15
                !compute for blue collar workers
                IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19) + &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                !compute for white collar workers
                ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                    !If not a percentage that depends on company size
                    IF (ANY(i.EQ.(/1,2,3,4,5,6,7,12,13/))) THEN
                        CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19)+ &
                        &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                    !If company size should be taken into account
                    ELSE IF (ANY(i.EQ.(/14,15/))) THEN
                        !If 10 or more workers in the company
                        IF (i.EQ.14.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        ELSE IF (i.EQ.15.AND.CONTRIB_SIZECOMP_QT(pers_j).GT.2) THEN
                            CONTRIB_PUBNSDED_EMER_QT(pers_j,19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19) + &
                            &CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)
                        END IF    
                    END IF 
                END IF
            END DO 
        END IF
    !employer dedu 20: TARGET-GROUP DEDUCTION: JOB STARTERS (ROSETTA PLAN)
        IF (CONTRIB_YOUNGW_QT(pers_j).EQ.1) THEN
            CONTRIB_PUBNSDED_EMER_QT(pers_j,20) = (CONTRIB_YWDED_QT/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
        END IF
    !employer dedu 21: TARGET-GROUP DEDUCTION: THE DEDUCTION FOR TIME WORKING REDUCTION TO 38 HOURS PER WEEK
        IF (CONTRIB_38HDED_QT(pers_j).EQ.1) THEN
            IF (SUM(CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:21)).LT.CONTRIB_38HDEDA_QT) THEN
                CONTRIB_PUBNSDED_EMER_QT(pers_j,21) = SUM(CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:21))
            ELSE
                CONTRIB_PUBNSDED_EMER_QT(pers_j,21) = CONTRIB_38HDEDA_QT
            END IF
        END IF

    !Contstruct deductions on employee social security contributions
    !CONTRIB_PUBNSDED_EMEE_QT Deductions of employee social security contributions permitted for wage earners on the public labour market but with non statutory service             
        CONTRIB_INCCONC_LWDEDU = MIMOSIS_GRINC_HOUR(pers_j)*38.D0*4.3333D0
        !Then use the function CONTRIB_BRACRATE_4COLUMNS_MIN or CONTRIB_BRACRATE_4COLUMNS_WB2009
        !to select the appropriate deduction amount
        IF (SIMYEAR.LT.2009) THEN
            IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                CONTRIB_PUBNSDED_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_PDEDBL_MONTH,1),CONTRIB_PDEDBL_MONTH,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                CONTRIB_PUBNSDED_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_MIN(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_PDEDWI_MONTH,1),CONTRIB_PDEDWI_MONTH,GECODEERD_INSZ_INPUT(pers_j))
            END IF
        ELSE
            IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                CONTRIB_PUBNSDED_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_WB2009(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_PDEDBL_MONTH,1),CONTRIB_PDEDBL_MONTH,GECODEERD_INSZ_INPUT(pers_j))
            ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                CONTRIB_PUBNSDED_EMEE_QT(pers_j) = 3.D0*CONTRIB_BRACRATE_4COLUMNS_WB2009(CONTRIB_INCCONC_LWDEDU,&
                &SIZE(CONTRIB_PDEDWI_MONTH,1),CONTRIB_PDEDWI_MONTH,GECODEERD_INSZ_INPUT(pers_j))
            END IF
        END IF
        !Correct the deduction in case of part time work
        IF (INT(100.D0*CONTRIB_FULLPART_QT(pers_j)).LT.100) THEN
            CONTRIB_PUBNSDED_EMEE_QT(pers_j) = CONTRIB_FULLPART_QT(pers_j)*CONTRIB_PUBNSDED_EMEE_QT(pers_j)
        END IF

END SUBROUTINE CONTRIB_SSDEDU_PUBNSTAT_2004

!===================================================================================!
!With subroutine CONTRIB_SSNET_PUBNSTAT we reconstruct the social security          !
!contributions of public labour market wage earners without statutory service       !
!net of deductions                                                                  !
!Variables constructed in this subroutine                                           !
!   CONTRIB_SSPUB_EMER_QT : start                                                   !
!   CONTRIB_SSPUB_EMEE_QT : start                                                   !
!===================================================================================!

SUBROUTINE CONTRIB_SSNET_PUBNSTAT(pers_j)

    INTEGER:: pers_j,i
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2)+1)       :: CONTRIB_PUBNSCOM_EMEE_NET         !Employee contributions of wage earners on the public labour market without statutory service net of deductions
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBNSCOM_EMER_QT,2)+1)       :: CONTRIB_PUBNSCOM_EMER_NET         !Employer contributions of wage earners on the public labour market without statutory service net of deductions
    REAL(KIND=DPRECI)                                                       :: CONTRIB_PUBNSCOM_EMEE_SUMCON      !Total sum of employee contributions of wage earners on the public labour market without statutory service before deductions  
    REAL(KIND=DPRECI)                                                       :: CONTRIB_PUBNSCOM_EMER_SUMCON      !Total sum of employer contributions of wage earners on the public labour market without statutory service before deductions  
    REAL(KIND=DPRECI)                                                       :: CONTRIB_PUBNSCOM_EMEE_SUMDED      !Total sum of employee deductions of wage earners on the public labour market without statutory service before deductions  
    REAL(KIND=DPRECI)                                                       :: CONTRIB_PUBNSCOM_EMER_SUMDED      !Total sum of employer deductions of wage earners on the public labour market without statutory service   

    !CONTRIBUTIONS OF THE EMPLOYEE
    !Make store variables empty
    CONTRIB_PUBNSCOM_EMEE_NET(1:SIZE(CONTRIB_PUBNSCOM_EMEE_NET,1)) = 0.D0
    CONTRIB_PUBNSCOM_EMEE_SUMCON = 0.D0
    CONTRIB_PUBNSCOM_EMEE_SUMDED = 0.D0
    !Make the sum of all employee contributions
    CONTRIB_PUBNSCOM_EMEE_SUMCON =   SUM(CONTRIB_PUBNSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2))) + &
                                    &CONTRIB_PUBNSHOL_EMEE_QT(pers_j)
    !Make the sum of all employee deductions
    CONTRIB_PUBNSCOM_EMEE_SUMDED =   CONTRIB_PUBNSDED_EMEE_QT(pers_j)
    !Compare the personal contributions with the personal deductions
    IF (CONTRIB_PUBNSCOM_EMEE_SUMDED.GT.CONTRIB_PUBNSCOM_EMEE_SUMCON) THEN
        CONTRIB_PUBNSDED_EMEE_QT(pers_j) = CONTRIB_PUBNSCOM_EMEE_SUMCON
    END IF
    !recompute the sum of all employee deductions
    CONTRIB_PUBNSCOM_EMEE_SUMDED = CONTRIB_PUBNSDED_EMEE_QT(pers_j)
    !Compute the contributions net of deductions
    IF (INT(100.D0*CONTRIB_PUBNSCOM_EMEE_SUMCON).NE.0) THEN
        !Contributions in the matrix
        DO i=1,SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2)
            CONTRIB_PUBNSCOM_EMEE_NET(i) = CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i) - &
            &CONTRIB_PUBNSCOM_EMEE_SUMDED*(CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i)/CONTRIB_PUBNSCOM_EMEE_SUMCON)        
        END DO
        !contributions on double holiday earnings
        CONTRIB_PUBNSCOM_EMEE_NET(SIZE(CONTRIB_PUBNSCOM_EMEE_NET,1)) = CONTRIB_PUBNSHOL_EMEE_QT(pers_j) - &
        &CONTRIB_PUBNSCOM_EMEE_SUMDED*(CONTRIB_PUBNSHOL_EMEE_QT(pers_j)/CONTRIB_PUBNSCOM_EMEE_SUMCON)        
    END IF
    !Make the sum of all employee contributions net of deductions
    CONTRIB_SSPUB_EMEE_QT(pers_j) = SUM(CONTRIB_PUBNSCOM_EMEE_NET(1:SIZE(CONTRIB_PUBNSCOM_EMEE_NET,1))) 

    !CONTRIBUTIONS OF THE EMPLOYER
    !Make store variables empty
    CONTRIB_PUBNSCOM_EMER_NET(1:SIZE(CONTRIB_PUBNSCOM_EMER_NET,1)) = 0.D0
    CONTRIB_PUBNSCOM_EMER_SUMCON = 0.D0
    CONTRIB_PUBNSCOM_EMER_SUMDED = 0.D0
    !Make the sum of all employer contributions
    CONTRIB_PUBNSCOM_EMER_SUMCON =   SUM(CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBNSCOM_EMER_QT,2))) + &
                                    &CONTRIB_PUBNSHOL_EMER_QT(pers_j)
    !Make the sum of all employer deductions
    CONTRIB_PUBNSCOM_EMER_SUMDED =   SUM(CONTRIB_PUBNSDED_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBNSDED_EMER_QT,2))) 
    !Compare the employer contributions with the employer deductions
    IF (CONTRIB_PUBNSCOM_EMER_SUMDED.GT.CONTRIB_PUBNSCOM_EMER_SUMCON.AND.INT(100.D0*CONTRIB_PUBNSCOM_EMER_SUMDED).NE.0) THEN
        DO i=1,SIZE(CONTRIB_PUBNSDED_EMER_QT,2)
            CONTRIB_PUBNSDED_EMER_QT(pers_j,i) = &
            &CONTRIB_PUBNSCOM_EMER_SUMCON*(CONTRIB_PUBNSDED_EMER_QT(pers_j,i)/CONTRIB_PUBNSCOM_EMER_SUMDED)        
        END DO
    END IF
    !recompute the sum of all employer deductions
    CONTRIB_PUBNSCOM_EMER_SUMDED =  SUM(CONTRIB_PUBNSDED_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBNSDED_EMER_QT,2)))
    !Compute the contributions net of deductions
    IF (INT(100.D0*CONTRIB_PUBNSCOM_EMER_SUMCON).NE.0) THEN
        !Contributions in the matrix
        DO i=1,SIZE(CONTRIB_PUBNSCOM_EMER_QT,2)
            CONTRIB_PUBNSCOM_EMER_NET(i) = CONTRIB_PUBNSCOM_EMER_QT(pers_j,i) - &
            &CONTRIB_PUBNSCOM_EMER_SUMDED*(CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)/CONTRIB_PUBNSCOM_EMER_SUMCON)        
        END DO
        !contributions on holiday earnings for blue collar workers
        CONTRIB_PUBNSCOM_EMER_NET(SIZE(CONTRIB_PUBNSCOM_EMER_NET,1)) = CONTRIB_PUBNSHOL_EMER_QT(pers_j) - &
        &CONTRIB_PUBNSCOM_EMER_SUMDED*(CONTRIB_PUBNSHOL_EMER_QT(pers_j)/CONTRIB_PUBNSCOM_EMER_SUMCON)        
    END IF
    !Make the sum of all employer contributions net of deductions
    CONTRIB_SSPUB_EMER_QT(pers_j) = SUM(CONTRIB_PUBNSCOM_EMER_NET(1:SIZE(CONTRIB_PUBNSCOM_EMER_NET,1))) 

END SUBROUTINE CONTRIB_SSNET_PUBNSTAT

!===================================================================================!
!With subroutine CONTRIB_SSBEFDED_PUBSTAT we reconstruct the social security        !
!contributions of public labour market wage earners with statutory service          !
!before any of the possible deductions has been deducted                            !
!Variables constructed in this subroutine                                           !
!   CONTRIB_PUBSCOM_EMEE_QT                                                         !
!   CONTRIB_PUBSCOM_EMER_QT                                                         !
!   CONTRIB_PUBSHOL_EMEE_QT                                                         !
!===================================================================================!

SUBROUTINE CONTRIB_SSBEFDED_PUBSTAT(pers_j)

    INTEGER:: pers_j,i

    !Compute common social security contributions paid for wage earners on the public labour market with statutory service
    !if they are working for a local authority
    !either paid by the employee or the employer
    !CONTRIB_PUBSCOM_EMEE_QT !Common social security contributions paid by the employee working as wage earner on the public labour market            
    !CONTRIB_PUBSCOM_EMER_QT !Common social security contributions paid by the employer for an employee working as wage earner on the public labour market            
    IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
        DO i=1,SIZE(CONTRIB_CSCOMLOC_QT,1)
            IF (ANY(i.EQ.(/1,2,3,4,5,6/))) THEN
                CONTRIB_PUBSCOM_EMEE_QT(pers_j,i) = (CONTRIB_CSCOMLOC_QT(i,1)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                CONTRIB_PUBSCOM_EMER_QT(pers_j,i) = (CONTRIB_CSCOMLOC_QT(i,2)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
            END IF
        END DO
    END IF        

    !Compute common social security contributions paid for wage earners on the public labour market with statutory service
    !if they are working for a federal authority
    !either paid by the employee or the employer
    IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
        DO i=1,SIZE(CONTRIB_CSCOMFED_QT,1)
            IF (ANY(i.EQ.(/1,2,3,4,5,6/))) THEN
                CONTRIB_PUBSCOM_EMEE_QT(pers_j,i) = (CONTRIB_CSCOMFED_QT(i,1)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
                CONTRIB_PUBSCOM_EMER_QT(pers_j,i) = (CONTRIB_CSCOMFED_QT(i,2)/100.D0)*CONTRIB_REMBPUB_QT(pers_j)
            END IF
        END DO
    END IF        

    !Compute social security contributions paid on double holiday earnings
    !CONTRIB_PUBSHOL_EMEE_QT  Social security contributions paid on double holiday earnings of wage earners on the public labour market           
    IF (ANY(CONTRIB_CSLOCFED_QT(pers_j).EQ.(/1,2/))) THEN
        CONTRIB_PUBSHOL_EMEE_QT(pers_j) = (CONTRIB_CSDHCON_QT/100.D0)*CONTRIB_DHPUB_QT(pers_j)
    END IF
    
END SUBROUTINE CONTRIB_SSBEFDED_PUBSTAT

!===================================================================================!
!With subroutine CONTRIB_SSDEDU_PUBSTAT we reconstruct the deductions applied on    !
!social security contributions of public labour market wage earners with statutory  !
!service                                                                            !
!Variables constructed in this subroutine                                           !
!   CONTRIB_PUBSDED_EMER_QT                                                         !
!   CONTRIB_PUBSDED_EMEE_QT                                                         !
!===================================================================================!

SUBROUTINE CONTRIB_SSDEDU_PUBSTAT(pers_j)

    INTEGER:: pers_j

    !CONTRIB_PUBSDED_EMER_QT  Deductions of employer social security contributions permitted for wage earners on the public labour market         

    !employer dedu 1   Target-group deduction: subsidized contract worker
        !this deduction is not reconstructed because of data problems
        CONTRIB_PUBSDED_EMER_QT(pers_j,1) = 0
    !employer dedu 2   Target-group deduction: employment incentive plan for the job seekers recruitment promotion
        !this deduction is not reconstructed because of data problems
        CONTRIB_PUBSDED_EMER_QT(pers_j,2) = 0
    !employer dedu 3   Target-group deduction: job redistribution in the public sector
        !this deduction is not reconstructed because of data problems
        CONTRIB_PUBSDED_EMER_QT(pers_j,3) = 0
    !employer dedu 4   Target-group deduction: occupation in accordance with art. 60  7 of the law of july 8th 1976 relative to cpas
        !this deduction is not reconstructed because of data problems
        CONTRIB_PUBSDED_EMER_QT(pers_j,4) = 0
    !employer dedu 5   Target-group deduction: social maribel
        !this deduction is not reconstructed because of data problems
        CONTRIB_PUBSDED_EMER_QT(pers_j,5) = 0
    !employer dedu 6   Target-group deduction: professional transition program
        !this deduction is not reconstructed because of data problems
        CONTRIB_PUBSDED_EMER_QT(pers_j,6) = 0
    !employer dedu 7   Target-group deduction: service jobs  smet jobs
        !this deduction is not reconstructed because of data problems
        CONTRIB_PUBSDED_EMER_QT(pers_j,7) = 0

    !Contstruct deductions on employee social security contributions
    !CONTRIB_PUBSDED_EMEE_QT  Social security contributions paid on double holiday earnings of wage earners on the public labour market           

        !no such deductions existed in 2001
        CONTRIB_PUBSDED_EMEE_QT(pers_j) = 0

END SUBROUTINE CONTRIB_SSDEDU_PUBSTAT

!===================================================================================!
!With subroutine CONTRIB_SSNET_PUBSTAT we reconstruct the social security           !
!contributions of public labour market wage earners with statutory service          !
!net of deductions                                                                  !
!Variables constructed in this subroutine                                           !
!   CONTRIB_SSPUB_EMER_QT : continued                                               !
!   CONTRIB_SSPUB_EMEE_QT : continued                                               !
!===================================================================================!

SUBROUTINE CONTRIB_SSNET_PUBSTAT(pers_j)

    INTEGER:: pers_j,i
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBSCOM_EMEE_QT,2)+1)        :: CONTRIB_PUBSCOM_EMEE_NET         !Employee contributions of wage earners on the public labour market with statutory service net of deductions
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBSCOM_EMER_QT,2))          :: CONTRIB_PUBSCOM_EMER_NET         !Employer contributions of wage earners on the public labour market with statutory service net of deductions
    REAL(KIND=DPRECI)                                                       :: CONTRIB_PUBSCOM_EMEE_SUMCON      !Total sum of employee contributions of wage earners on the public labour market with statutory service before deductions  
    REAL(KIND=DPRECI)                                                       :: CONTRIB_PUBSCOM_EMER_SUMCON      !Total sum of employer contributions of wage earners on the public labour market with statutory service before deductions  
    REAL(KIND=DPRECI)                                                       :: CONTRIB_PUBSCOM_EMEE_SUMDED      !Total sum of employee deductions of wage earners on the public labour market with statutory service before deductions  
    REAL(KIND=DPRECI)                                                       :: CONTRIB_PUBSCOM_EMER_SUMDED      !Total sum of employer deductions of wage earners on the public labour market with statutory service   

    !CONTRIBUTIONS OF THE EMPLOYEE
    !Make store variables empty
    CONTRIB_PUBSCOM_EMEE_NET(1:SIZE(CONTRIB_PUBSCOM_EMEE_NET,1)) = 0.D0
    CONTRIB_PUBSCOM_EMEE_SUMCON = 0.D0
    CONTRIB_PUBSCOM_EMEE_SUMDED = 0.D0
    !Make the sum of all employee contributions
    CONTRIB_PUBSCOM_EMEE_SUMCON =   SUM(CONTRIB_PUBSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_PUBSCOM_EMEE_QT,2))) + &
                                    &CONTRIB_PUBSHOL_EMEE_QT(pers_j)
    !Make the sum of all employee deductions
    CONTRIB_PUBSCOM_EMEE_SUMDED =   CONTRIB_PUBSDED_EMEE_QT(pers_j)
    !Compare the personal contributions with the personal deductions
    IF (CONTRIB_PUBSCOM_EMEE_SUMDED.GT.CONTRIB_PUBSCOM_EMEE_SUMCON) THEN
        CONTRIB_PUBSDED_EMEE_QT(pers_j) = CONTRIB_PUBSCOM_EMEE_SUMCON
    END IF
    !recompute the sum of all employee deductions
    CONTRIB_PUBSCOM_EMEE_SUMDED = CONTRIB_PUBSDED_EMEE_QT(pers_j)
    !Compute the contributions net of deductions
    IF (INT(100.D0*CONTRIB_PUBSCOM_EMEE_SUMCON).NE.0) THEN
        !Contributions in the matrix
        DO i=1,SIZE(CONTRIB_PUBSCOM_EMEE_QT,2)
            CONTRIB_PUBSCOM_EMEE_NET(i) = CONTRIB_PUBSCOM_EMEE_QT(pers_j,i) - &
            &CONTRIB_PUBSCOM_EMEE_SUMDED*(CONTRIB_PUBSCOM_EMEE_QT(pers_j,i)/CONTRIB_PUBSCOM_EMEE_SUMCON)        
        END DO
        !contributions on double holiday earnings
        CONTRIB_PUBSCOM_EMEE_NET(SIZE(CONTRIB_PUBSCOM_EMEE_NET,1)) = CONTRIB_PUBSHOL_EMEE_QT(pers_j) - &
        &CONTRIB_PUBSCOM_EMEE_SUMDED*(CONTRIB_PUBSHOL_EMEE_QT(pers_j)/CONTRIB_PUBSCOM_EMEE_SUMCON)        
    END IF
    !Make the sum of all employee contributions net of deductions
    CONTRIB_SSPUB_EMEE_QT(pers_j) = SUM(CONTRIB_PUBSCOM_EMEE_NET(1:SIZE(CONTRIB_PUBSCOM_EMEE_NET,1))) 

    !CONTRIBUTIONS OF THE EMPLOYER
    !Make store variables empty
    CONTRIB_PUBSCOM_EMER_NET(1:SIZE(CONTRIB_PUBSCOM_EMER_NET,1)) = 0.D0
    CONTRIB_PUBSCOM_EMER_SUMCON = 0.D0
    CONTRIB_PUBSCOM_EMER_SUMDED = 0.D0
    !Make the sum of all employer contributions
    CONTRIB_PUBSCOM_EMER_SUMCON =   SUM(CONTRIB_PUBSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBSCOM_EMER_QT,2))) 
    !Make the sum of all employer deductions
    CONTRIB_PUBSCOM_EMER_SUMDED =   SUM(CONTRIB_PUBSDED_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBSDED_EMER_QT,2))) 
    !Compare the employer contributions with the employer deductions
    IF (CONTRIB_PUBSCOM_EMER_SUMDED.GT.CONTRIB_PUBSCOM_EMER_SUMCON.AND.INT(100.D0*CONTRIB_PUBSCOM_EMER_SUMDED).NE.0) THEN
        DO i=1,SIZE(CONTRIB_PUBSDED_EMER_QT,2)
            CONTRIB_PUBSDED_EMER_QT(pers_j,i) = &
            &CONTRIB_PUBSCOM_EMER_SUMCON*(CONTRIB_PUBSDED_EMER_QT(pers_j,i)/CONTRIB_PUBSCOM_EMER_SUMDED)        
        END DO
    END IF
    !recompute the sum of all employer deductions
    CONTRIB_PUBSCOM_EMER_SUMDED = SUM(CONTRIB_PUBSDED_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBSDED_EMER_QT,2))) 
    !Compute the contributions net of deductions
    IF (INT(100.D0*CONTRIB_PUBSCOM_EMER_SUMCON).NE.0) THEN
        !Contributions in the matrix
        DO i=1,SIZE(CONTRIB_PUBSCOM_EMER_QT,2)
            CONTRIB_PUBSCOM_EMER_NET(i) = CONTRIB_PUBSCOM_EMER_QT(pers_j,i) - &
            &CONTRIB_PUBSCOM_EMER_SUMDED*(CONTRIB_PUBSCOM_EMER_QT(pers_j,i)/CONTRIB_PUBSCOM_EMER_SUMCON)        
        END DO
    END IF
    !Make the sum of all employer contributions net of deductions
    CONTRIB_SSPUB_EMER_QT(pers_j) = SUM(CONTRIB_PUBSCOM_EMER_NET(1:SIZE(CONTRIB_PUBSCOM_EMER_NET,1))) 

END SUBROUTINE CONTRIB_SSNET_PUBSTAT

!===================================================================================!
!With subroutine CONTRIB_SSBEFDED_SELF we reconstruct the social security           !
!contributions of self employed                                                     !
!before any of the possible deductions has been deducted                            !
!Variables constructed in this subroutine                                           !
!   CONTRIB_SELFCOM_EMER_QT                                                         !
!===================================================================================!
                                                                                    
SUBROUTINE CONTRIB_SSBEFDED_SELF(pers_j,hh_i)

    INTEGER:: pers_j,hh_i,pers_k
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_PERCENT_SS                  !Variable to store percentage that is used to compute the social security contribution of self employed
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SSCONT_LIM                  !Variable to store the social security contribution concept on which limits are applied
    INTEGER                                                 :: MARSASS                             !Indicator that says whether the self employed assistant is married with a self employed         

    !CONTRIB_SELFCOM_EMER_QT  Common social security contributions paid by the self employed          

    !CALCULATION OF COMMON SOCIAL SECURITY CONTRIBUTIONS FOR ALL SELF EMPLOYED

    !Use the function CONTRIB_LSAMOUNT_3COLUMNS to select the appropriate amount
    CONTRIB_SELFCOM_EMER_QT(pers_j) = CONTRIB_LSAMOUNT_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
    &SIZE(CONTRIB_LSALLSE_QT,1),CONTRIB_LSALLSE_QT,GECODEERD_INSZ_INPUT(pers_j))

    !CALCULATION OF OTHER SOCIAL SECURITY CONTRIBUTIONS FOR SELF EMPLOYED

        !SELF EMPLOYED IN PRINCIPAL ACTIVITY BEFORE RETIREMENT AGE
        IF (CONTRIB_SELFPCR_QT(pers_j).EQ.1) THEN
            CONTRIB_SSCONT_LIM = 0
            !Check whether the self employed is active for more than 3 years
            IF (CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                !Does the self employed receive a survival pension
                IF (PENSWELF_SURPENS_QT(pers_j).GT.0) THEN
                    !If the self employed has more income than a certain limit
                    IF (CONTRIB_INCSELF_QT(pers_j).GT.CONTRIB_SELFPAC_QT(3)) THEN
                        CONTRIB_SSCONT_LIM = (CONTRIB_SELFPAA_QT(1,3)/100.D0)*CONTRIB_SELFPAA_QT(1,1)
                    !If not compute the contribution as a percentage applied on income
                    ELSE
                        !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                        CONTRIB_SSCONT_LIM = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                        &SIZE(CONTRIB_SELFPAA_QT,1),CONTRIB_SELFPAA_QT,GECODEERD_INSZ_INPUT(pers_j))
                    END IF
                !if no survival pension
                ELSE
                    !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                    CONTRIB_SSCONT_LIM  = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                    &SIZE(CONTRIB_SELFPAA_QT,1),CONTRIB_SELFPAA_QT,GECODEERD_INSZ_INPUT(pers_j))
                END IF
                !Apply minimal and maximal limits on the computed amount
                IF (CONTRIB_SSCONT_LIM.LT.CONTRIB_SELFPAC_QT(1)) THEN
                    CONTRIB_SSCONT_LIM = CONTRIB_SELFPAC_QT(1)
                END IF
                IF (CONTRIB_SSCONT_LIM.GT.CONTRIB_SELFPAC_QT(2)) THEN
                    CONTRIB_SSCONT_LIM = CONTRIB_SELFPAC_QT(2)
                END IF
            !If the self employed is a starter
            ELSE
                !select the lump sum value conditional on CONTRIB_SELFYEAR_QT(pers_j)
                IF (CONTRIB_SELFYEAR_QT(pers_j).EQ.1) THEN
                    CONTRIB_SSCONT_LIM = CONTRIB_SELFPAB_QT(1)
                ELSE IF (CONTRIB_SELFYEAR_QT(pers_j).EQ.2) THEN
                    CONTRIB_SSCONT_LIM = CONTRIB_SELFPAB_QT(2)
                ELSE IF (CONTRIB_SELFYEAR_QT(pers_j).EQ.3) THEN
                    CONTRIB_SSCONT_LIM = CONTRIB_SELFPAB_QT(3)
                END IF
            END IF
            !add the computed value to the contributions already computed
            CONTRIB_SELFCOM_EMER_QT(pers_j) = CONTRIB_SELFCOM_EMER_QT(pers_j) + CONTRIB_SSCONT_LIM
        END IF

        !SELF EMPLOYED IN COMPLEMENTARY ACTIVITY BEFORE RETIREMENT AGE
        IF (CONTRIB_SELFPCR_QT(pers_j).EQ.2) THEN
            CONTRIB_SSCONT_LIM = 0
            !Check whether the self employed is active for more than 3 years
            IF (CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                !If the self employed has more income than a certain limit
                IF (CONTRIB_INCSELF_QT(pers_j).GT.CONTRIB_SELFCAA_QT(1)) THEN
                    !Does the self employed receive a survival pension
                    IF (PENSWELF_SURPENS_QT(pers_j).GT.0) THEN
                        CONTRIB_SSCONT_LIM = (CONTRIB_SELFPAA_QT(1,3)/100.D0)*CONTRIB_SELFPAA_QT(1,1)
                    ELSE
                        !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                        CONTRIB_SSCONT_LIM = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                        &SIZE(CONTRIB_SELFPAA_QT,1),CONTRIB_SELFPAA_QT,GECODEERD_INSZ_INPUT(pers_j))
                    END IF
                    !Apply minimal and maximal limits on the computed amount
                    IF (CONTRIB_SSCONT_LIM.LT.CONTRIB_SELFCAA_QT(2)) THEN
                        CONTRIB_SSCONT_LIM = CONTRIB_SELFCAA_QT(2)
                    END IF
                    IF (CONTRIB_SSCONT_LIM.GT.CONTRIB_SELFCAA_QT(3)) THEN
                        CONTRIB_SSCONT_LIM = CONTRIB_SELFCAA_QT(3)
                    END IF
                !if income is below the threshold
                ELSE
                    CONTRIB_SSCONT_LIM = 0
                END IF
            !If the self employed is a starter
            ELSE
                CONTRIB_SSCONT_LIM = CONTRIB_SELFCAA_QT(2)
            END IF
            !add the computed value to the contributions already computed
            CONTRIB_SELFCOM_EMER_QT(pers_j) = CONTRIB_SELFCOM_EMER_QT(pers_j) + CONTRIB_SSCONT_LIM
        END IF

        !SELF EMPLOYED AFTER RETIREMENT AGE
        IF (CONTRIB_SELFPCR_QT(pers_j).EQ.3) THEN
            CONTRIB_SSCONT_LIM = 0
            !Check whether the self employed receives a survival or retirement pension or a pre retirement pension
            IF ((CONTRIB_PENS_QT(pers_j)+CONTRIB_UNEMFT_QT(pers_j)+CONTRIB_UNEMPT_QT(pers_j)).GT.0) THEN
                !Check whether the self employed is active for more than 3 years
                IF (CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                    !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                    CONTRIB_SSCONT_LIM = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                    &SIZE(CONTRIB_SELFARA_QT,1),CONTRIB_SELFARA_QT,GECODEERD_INSZ_INPUT(pers_j))
                    !Apply minimal and maximal limits on the computed amount
                    IF (CONTRIB_SSCONT_LIM.LT.CONTRIB_SELFARC_QT(1)) THEN
                        CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(1)
                    END IF
                    IF (CONTRIB_SSCONT_LIM.GT.CONTRIB_SELFARC_QT(2)) THEN
                        CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(2)
                    END IF
                !If the self employed is a strater    
                ELSE
                    CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(1)
                END IF
            !If the self employed does not receive pension or pre retirement benefits
            ELSE
                !Check whether the self employed is active for more than 3 years
                IF (CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                    !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                    CONTRIB_SSCONT_LIM = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                    &SIZE(CONTRIB_SELFARB_QT,1),CONTRIB_SELFARB_QT,GECODEERD_INSZ_INPUT(pers_j))
                    !Apply minimal and maximal limits on the computed amount
                    IF (CONTRIB_SSCONT_LIM.LT.CONTRIB_SELFARC_QT(4)) THEN
                        CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(4)
                    END IF
                    IF (CONTRIB_SSCONT_LIM.GT.CONTRIB_SELFARC_QT(5)) THEN
                        CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(5)
                    END IF
                !If the self employed is a strater    
                ELSE
                    CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(4)
                END IF
            END IF
            !add the computed value to the contributions already computed
            CONTRIB_SELFCOM_EMER_QT(pers_j) = CONTRIB_SELFCOM_EMER_QT(pers_j) + CONTRIB_SSCONT_LIM
        END IF

    !CALCULATION OF OTHER SOCIAL SECURITY CONTRIBUTIONS FOR SELF EMPLOYED ASSISTANT

        IF (ANY(CONTRIB_SELFPCR_QT(pers_j).EQ.(/4,5,6/))) THEN
            !First check whether the assistant is the spouse of a self employed
            !We only check this condition in case of married couples
            MARSASS = 0
            IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
                !loop over all household member
                DO pers_k=CONTRIB_START_HH(hh_i),CONTRIB_END_HH(hh_i)
                    !look for member of the same family
                    IF (FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                        !j and k are a couple
                        IF ((FAMREL_RELATION(pers_j).EQ.1.AND.FAMREL_RELATION(pers_k).EQ.2).OR.&
                        &   (FAMREL_RELATION(pers_j).EQ.2.AND.FAMREL_RELATION(pers_k).EQ.1)) THEN
                            !if k is self employed
                            IF (CONTRIB_LABMSTAT_SELF_QT(pers_k).EQ.1) THEN
                                MARSASS = 1
                            END IF
                        END IF
                    END IF
                END DO                    
            END IF
            !if the self employed assistant is married to a self employed
            IF (MARSASS.EQ.1) THEN
                !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                CONTRIB_SSCONT_LIM = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                &SIZE(CONTRIB_SELFASA_QT,1),CONTRIB_SELFASA_QT,GECODEERD_INSZ_INPUT(pers_j))
                !Apply minimal and maximal limits on the computed amount
                IF (CONTRIB_SSCONT_LIM.LT.CONTRIB_SELFASB_QT(1)) THEN
                    CONTRIB_SSCONT_LIM = CONTRIB_SELFASB_QT(1)
                END IF
                IF (CONTRIB_SSCONT_LIM.GT.CONTRIB_SELFASB_QT(2)) THEN
                    CONTRIB_SSCONT_LIM = CONTRIB_SELFASB_QT(2)
                END IF
            !if the self employed assistant is not married to a self employed repeat the procedure of 
            !self employed that are not self employed assistant
            ELSE
                !SELF EMPLOYED ASSISTANT IN PRINCIPAL ACTIVITY BEFORE RETIREMENT AGE
                IF (CONTRIB_SELFPCR_QT(pers_j).EQ.4) THEN
                    CONTRIB_SSCONT_LIM = 0
                    !Check whether the self employed is active for more than 3 years
                    IF (CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                        !Does the self employed receive a survival pension
                        IF (PENSWELF_SURPENS_QT(pers_j).GT.0) THEN
                            !If the self employed has more income than a certain limit
                            IF (CONTRIB_INCSELF_QT(pers_j).GT.CONTRIB_SELFPAC_QT(3)) THEN
                                CONTRIB_SSCONT_LIM = (CONTRIB_SELFPAA_QT(1,3)/100.D0)*CONTRIB_SELFPAA_QT(1,1)
                            !If not compute the contribution as a percentage applied on income
                            ELSE
                                !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                                CONTRIB_SSCONT_LIM = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                                &SIZE(CONTRIB_SELFPAA_QT,1),CONTRIB_SELFPAA_QT,GECODEERD_INSZ_INPUT(pers_j))
                                !as a result for 2001 the value CONTRIB_SSCONT_LIM should be equal to 0
                            END IF
                        !if no survival pension
                        ELSE
                            !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                            CONTRIB_SSCONT_LIM = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                            &SIZE(CONTRIB_SELFPAA_QT,1),CONTRIB_SELFPAA_QT,GECODEERD_INSZ_INPUT(pers_j))
                        END IF
                        !Apply minimal and maximal limits on the computed amount
                        IF (CONTRIB_SSCONT_LIM.LT.CONTRIB_SELFPAC_QT(1)) THEN
                            CONTRIB_SSCONT_LIM = CONTRIB_SELFPAC_QT(1)
                        END IF
                        IF (CONTRIB_SSCONT_LIM.GT.CONTRIB_SELFPAC_QT(2)) THEN
                            CONTRIB_SSCONT_LIM = CONTRIB_SELFPAC_QT(2)
                        END IF
                    !If the self employed is a starter
                    ELSE
                        !select the lump sum value conditional on CONTRIB_SELFYEAR_QT(pers_j)
                        IF (CONTRIB_SELFYEAR_QT(pers_j).EQ.1) THEN
                            CONTRIB_SSCONT_LIM = CONTRIB_SELFPAB_QT(1)
                        ELSE IF (CONTRIB_SELFYEAR_QT(pers_j).EQ.2) THEN
                            CONTRIB_SSCONT_LIM = CONTRIB_SELFPAB_QT(2)
                        ELSE IF (CONTRIB_SELFYEAR_QT(pers_j).EQ.3) THEN
                            CONTRIB_SSCONT_LIM = CONTRIB_SELFPAB_QT(3)
                        END IF
                    END IF
                END IF

                !SELF EMPLOYED ASSISTANT IN COMPLEMENTARY ACTIVITY BEFORE RETIREMENT AGE
                IF (CONTRIB_SELFPCR_QT(pers_j).EQ.5) THEN
                    CONTRIB_SSCONT_LIM = 0
                    !Check whether the self employed is active for more than 3 years
                    IF (CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                        !If the self employed has more income than a certain limit
                        IF (CONTRIB_INCSELF_QT(pers_j).GT.CONTRIB_SELFCAA_QT(1)) THEN
                            !Does the self employed receive a survival pension
                            IF (PENSWELF_SURPENS_QT(pers_j).GT.0) THEN
                                CONTRIB_SSCONT_LIM = (CONTRIB_SELFPAA_QT(1,3)/100.D0)*CONTRIB_SELFPAA_QT(1,1)
                            ELSE
                                !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                                CONTRIB_SSCONT_LIM = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                                &SIZE(CONTRIB_SELFPAA_QT,1),CONTRIB_SELFPAA_QT,GECODEERD_INSZ_INPUT(pers_j))
                            END IF
                            !Apply minimal and maximal limits on the computed amount
                            IF (CONTRIB_SSCONT_LIM.LT.CONTRIB_SELFCAA_QT(2)) THEN
                                CONTRIB_SSCONT_LIM = CONTRIB_SELFCAA_QT(2)
                            END IF
                            IF (CONTRIB_SSCONT_LIM.GT.CONTRIB_SELFCAA_QT(3)) THEN
                                CONTRIB_SSCONT_LIM = CONTRIB_SELFCAA_QT(3)
                            END IF
                        !if income is below the threshold
                        ELSE
                            CONTRIB_SSCONT_LIM = 0
                        END IF
                    !If the self employed is a starter
                    ELSE
                        CONTRIB_SSCONT_LIM = CONTRIB_SELFCAA_QT(2)
                    END IF
                END IF

                !SELF EMPLOYED ASSISTANT AFTER RETIREMENT AGE
                IF (CONTRIB_SELFPCR_QT(pers_j).EQ.6) THEN
                    CONTRIB_SSCONT_LIM = 0
                    !Check whether the self employed receives a survival or retirement pension or a pre retirement pension
                    IF ((CONTRIB_PENS_QT(pers_j)+CONTRIB_UNEMFT_QT(pers_j)+CONTRIB_UNEMPT_QT(pers_j)).GT.0) THEN
                        !Check whether the self employed is active for more than 3 years
                        IF (CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                            !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                            CONTRIB_SSCONT_LIM = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                            &SIZE(CONTRIB_SELFARA_QT,1),CONTRIB_SELFARA_QT,GECODEERD_INSZ_INPUT(pers_j))
                            !Apply minimal and maximal limits on the computed amount
                            IF (CONTRIB_SSCONT_LIM.LT.CONTRIB_SELFARC_QT(1)) THEN
                                CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(1)
                            END IF
                            IF (CONTRIB_SSCONT_LIM.GT.CONTRIB_SELFARC_QT(2)) THEN
                                CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(2)
                            END IF
                        !If the self employed is a strater    
                        ELSE
                            CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(1)
                        END IF
                    !If the self employed does not receive pension or pre retirement benefits
                    ELSE
                        !Check whether the self employed is active for more than 3 years
                        IF (CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                            !Use the function CONTRIB_BRACRATE_3COLUMNS to select the appropriate amount
                            CONTRIB_SSCONT_LIM = CONTRIB_BRACRATE_3COLUMNS(CONTRIB_INCSELF_QT(pers_j),&
                            &SIZE(CONTRIB_SELFARB_QT,1),CONTRIB_SELFARB_QT,GECODEERD_INSZ_INPUT(pers_j))
                            !Apply minimal and maximal limits on the computed amount
                            IF (CONTRIB_SSCONT_LIM.LT.CONTRIB_SELFARC_QT(4)) THEN
                                CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(4)
                            END IF
                            IF (CONTRIB_SSCONT_LIM.GT.CONTRIB_SELFARC_QT(5)) THEN
                                CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(5)
                            END IF
                        !If the self employed is a strater    
                        ELSE
                            CONTRIB_SSCONT_LIM = CONTRIB_SELFARC_QT(4)
                        END IF
                    END IF
                END IF
            END IF
            !add the computed value to the contributions already computed
            CONTRIB_SELFCOM_EMER_QT(pers_j) = CONTRIB_SELFCOM_EMER_QT(pers_j) + CONTRIB_SSCONT_LIM
        END IF            

END SUBROUTINE CONTRIB_SSBEFDED_SELF

!===================================================================================!
!With subroutine CONTRIB_SSDEDU_SELF we reconstruct the deductions applied on       !
!social security contributions of self employed                                     !
!Variables constructed in this subroutine                                           !
!   CONTRIB_SELFDEDU_EMER_QT                                                        !
!===================================================================================!

SUBROUTINE CONTRIB_SSDEDU_SELF(pers_j)

    INTEGER:: pers_j
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SSDEDU_LIM                  !Variable to store the deduction of the social security contribution concept on which limits are applied

    !CONTRIB_SELFDEDU_EMER_QT Deduction of social security contributions of self employed         

    !if the self employed is in his first activity as self employed
    IF (CONTRIB_SELFFIRS_QT(pers_j).EQ.1) THEN
        IF (CONTRIB_SELFYEAR_QT(pers_j).EQ.4) THEN
            CONTRIB_SSDEDU_LIM = (CONTRIB_SELFDEDA_QT/100.D0)*CONTRIB_SELFCOM_EMER_QT(pers_j)
            IF (CONTRIB_SSDEDU_LIM.GT.CONTRIB_SELFDEDB_QT) THEN
                CONTRIB_SSDEDU_LIM = CONTRIB_SELFDEDB_QT
            END IF
            CONTRIB_SELFDEDU_EMER_QT(pers_j) = CONTRIB_SSDEDU_LIM
        END IF
    END IF        

END SUBROUTINE CONTRIB_SSDEDU_SELF

!===================================================================================!
!With subroutine CONTRIB_SSNET_SELF we reconstruct the social security              !
!contributions of self employed net of deductions                                   !
!Variables constructed in this subroutine                                           !
!   CONTRIB_SSSELF_EMER_QT                                                          !
!===================================================================================!

SUBROUTINE CONTRIB_SSNET_SELF(pers_j)

    INTEGER:: pers_j
    REAL(KIND=DPRECI)                                                   :: CONTRIB_SELF_EMER_NET            !Employer contributions of self employed net of deductions
    REAL(KIND=DPRECI)                                                   :: CONTRIB_SELF_EMER_SUMCON         !Total sum of employer contributions of self employed before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_SELF_EMER_SUMDED         !Total sum of employer deductions of self employed   

    !CONTRIBUTIONS OF THE SELF EMPLOYED
    !Make store variables empty
    CONTRIB_SELF_EMER_NET = 0.D0
    CONTRIB_SELF_EMER_SUMCON = 0.D0
    CONTRIB_SELF_EMER_SUMDED = 0.D0
    !Make the sum of all employer contributions
    CONTRIB_SELF_EMER_SUMCON = CONTRIB_SELFCOM_EMER_QT(pers_j)
    !Make the sum of all employer deductions
    CONTRIB_SELF_EMER_SUMDED = CONTRIB_SELFDEDU_EMER_QT(pers_j)
    !Compare the contributions with the deductions
    IF (CONTRIB_SELF_EMER_SUMDED.GT.CONTRIB_SELF_EMER_SUMCON) THEN
        CONTRIB_SELFDEDU_EMER_QT(pers_j) = CONTRIB_SELF_EMER_SUMCON
    END IF
    !recompute the sum of all employer deductions
    CONTRIB_SELF_EMER_SUMDED = CONTRIB_SELFDEDU_EMER_QT(pers_j)
    !Compute the contributions net of deductions
    CONTRIB_SELF_EMER_NET = CONTRIB_SELF_EMER_SUMCON - CONTRIB_SELF_EMER_SUMDED
    CONTRIB_SSSELF_EMER_QT(pers_j) = CONTRIB_SELF_EMER_NET 

END SUBROUTINE CONTRIB_SSNET_SELF

!===================================================================================!
!With subroutine CONTRIB_SSBEFDED_NONLAB we reconstruct the social security         !
!contributions of persons with non labour market income                             !
!before deductions have been deducted                                               !
!Variables constructed in this subroutine                                           !
!    CONTRIB_SSPEBASI_QT                                                            !
!    CONTRIB_SSPESOLI_QT                                                            !
!    CONTRIB_SSEARBEN_QT                                                            !
!    CONTRIB_SSDISAB_QT                                                             !
!    CONTRIB_SSINDOCC_QT                                                            !
!===================================================================================!

SUBROUTINE CONTRIB_SSBEFDED_NONLAB(pers_j)

    INTEGER:: pers_j
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_NETPENS_QT                   !Variable to store the pension net of the social security contribution
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_SSPENS_MONTH                 !Variable to store the solidarity contribution paid per month on pension benefits
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_NETEAR_QT                    !Variable to store the early retirement benefits net of social security contributions
    REAL(KIND=KIND_REAL),DIMENSION(2)                       :: CONTRIB_SSEAR_QT                     !Social security contributions paid on early retirement benefits
    REAL(KIND=KIND_REAL)                                    :: CONTRIB_NETDISA_QT                   !Variable to store the disability benefits net of social security contributions
    
    !SOCIAL SECURITY CONTRIBUTIONS PAID ON RETIREMENT AND SURVIVAL PENSIONS
        !CONTRIB_SSPEBASI_QT      The basic social security contributions paid on pension benefits            
        !Compute basic contribution paid on pensions
        IF (CONTRIB_PENS_QT(pers_j).GT.0) THEN
            !compute the amount of pensions net of contributions
            CONTRIB_NETPENS_QT = CONTRIB_PENS_QT(pers_j) - (CONTRIB_PENSPERC_MONTH/100.D0)*CONTRIB_PENS_QT(pers_j)
            !select the relevant threshold for pensions conditional on family charge
            IF (CONTRIB_FAMCHARG_QT(pers_j).GT.0) THEN
                IF (CONTRIB_NETPENS_QT.LT.CONTRIB_PELIMBA_MONTH(1)*3.D0) THEN
                    CONTRIB_SSPEBASI_QT(pers_j) = CONTRIB_PENS_QT(pers_j) - (CONTRIB_PELIMBA_MONTH(1)*3.D0)
                ELSE
                    CONTRIB_SSPEBASI_QT(pers_j) = (CONTRIB_PENSPERC_MONTH/100.D0)*CONTRIB_PENS_QT(pers_j)
                END IF
            ELSE
                IF (CONTRIB_NETPENS_QT.LT.CONTRIB_PELIMBA_MONTH(2)*3.D0) THEN
                    CONTRIB_SSPEBASI_QT(pers_j) = CONTRIB_PENS_QT(pers_j) - (CONTRIB_PELIMBA_MONTH(2)*3.D0)
                ELSE
                    CONTRIB_SSPEBASI_QT(pers_j) = (CONTRIB_PENSPERC_MONTH/100.D0)*CONTRIB_PENS_QT(pers_j)
                END IF
            END IF
            !Set contribution equal to 0 if it would become negative
            IF (CONTRIB_SSPEBASI_QT(pers_j).LT.0) THEN
                CONTRIB_SSPEBASI_QT(pers_j) = 0
            END IF            
        END IF

        !Compute solidarity contribution paid on pensions
        !CONTRIB_SSPESOLI_QT      Solidarity contributions paid on pension benefits           
        IF (CONTRIB_PENS_QT(pers_j).GT.0) THEN
            !select the relevant table for computations conditional on family charge
            !use the function CONTRIB_BRACRATE_5COLUMNS_PLUS to carry out the computation on monthly income
            IF (CONTRIB_FAMCHARG_QT(pers_j).GT.0) THEN
                CONTRIB_SSPENS_MONTH = CONTRIB_BRACRATE_5COLUMNS_PLUS((CONTRIB_PENS_QT(pers_j)/3.E0),&
                &SIZE(CONTRIB_PELIMCHI_MONTH,1),CONTRIB_PELIMCHI_MONTH,GECODEERD_INSZ_INPUT(pers_j)) 
            ELSE
                CONTRIB_SSPENS_MONTH = CONTRIB_BRACRATE_5COLUMNS_PLUS((CONTRIB_PENS_QT(pers_j)/3.E0),&
                &SIZE(CONTRIB_PELIMNOC_MONTH,1),CONTRIB_PELIMNOC_MONTH,GECODEERD_INSZ_INPUT(pers_j))
            END IF
            !convert the monthly amount to a quarterly amount
            CONTRIB_SSPESOLI_QT(pers_j) = CONTRIB_SSPENS_MONTH*3.D0
        END IF

    !SOCIAL SECURITY CONTRIBUTIONS PAID ON CONVENTIONAL EARLY RETIREMENT BENEFITS
        !CONTRIB_SSEARBEN_QT      Social security contributions paid on early retirement benefits         
        !Initialise vector to store results if there are early retirement benefits
        IF (CONTRIB_UNEMFT_QT(pers_j).GT.0.OR.CONTRIB_UNEMPT_QT(pers_j).GT.0) THEN
            CONTRIB_SSEAR_QT(1:2) = 0
        END IF
       
        !Computation for full time early retirement benefits
        IF (CONTRIB_UNEMFT_QT(pers_j).GT.0) THEN
            !compute early retirement benefit net of contributions
            CONTRIB_NETEAR_QT = 0
            IF (CONTRIB_EARSTART_QT(pers_j).EQ.1) THEN
                CONTRIB_NETEAR_QT = CONTRIB_UNEMFT_QT(pers_j) - (CONTRIB_EARPERC_QT(1)/100.D0)*CONTRIB_UNEMFT_QT(pers_j)
            ELSE IF (CONTRIB_EARSTART_QT(pers_j).EQ.2) THEN
                CONTRIB_NETEAR_QT = CONTRIB_UNEMFT_QT(pers_j) - (CONTRIB_EARPERC_QT(2)/100.D0)*CONTRIB_UNEMFT_QT(pers_j)
            END IF
            !select the relevant threshold for pensions conditional on family charge
            IF (CONTRIB_FAMCHARG_QT(pers_j).GT.0) THEN
                !check whether net early retirement benefits come below the threshold
                IF (CONTRIB_NETEAR_QT.LT.CONTRIB_EALIMFT_MONTH(1)*3.D0) THEN
                    CONTRIB_SSEAR_QT(1) = CONTRIB_UNEMFT_QT(pers_j) - (CONTRIB_EALIMFT_MONTH(1)*3.D0)
                ELSE
                    IF (CONTRIB_EARSTART_QT(pers_j).EQ.1) THEN
                        CONTRIB_SSEAR_QT(1)  = (CONTRIB_EARPERC_QT(1)/100.D0)*CONTRIB_UNEMFT_QT(pers_j)
                    ELSE IF (CONTRIB_EARSTART_QT(pers_j).EQ.2) THEN
                        CONTRIB_SSEAR_QT(1)  = (CONTRIB_EARPERC_QT(2)/100.D0)*CONTRIB_UNEMFT_QT(pers_j)
                    END IF
                END IF
            ELSE
                !check whether net early retirement benefits come below the threshold
                IF (CONTRIB_NETEAR_QT.LT.CONTRIB_EALIMFT_MONTH(2)*3.D0) THEN
                    CONTRIB_SSEAR_QT(1) = CONTRIB_UNEMFT_QT(pers_j) - (CONTRIB_EALIMFT_MONTH(2)*3.D0)
                ELSE
                    IF (CONTRIB_EARSTART_QT(pers_j).EQ.1) THEN
                        CONTRIB_SSEAR_QT(1)  = (CONTRIB_EARPERC_QT(1)/100.D0)*CONTRIB_UNEMFT_QT(pers_j)
                    ELSE IF (CONTRIB_EARSTART_QT(pers_j).EQ.2) THEN
                        CONTRIB_SSEAR_QT(1)  = (CONTRIB_EARPERC_QT(2)/100.D0)*CONTRIB_UNEMFT_QT(pers_j)
                    END IF
                END IF
            END IF
            !Set contribution equal to 0 if it would become negative
            IF (CONTRIB_SSEAR_QT(1).LT.0) THEN
                CONTRIB_SSEAR_QT(1) = 0
            END IF  
        END IF
        
        !Computation for part time early retirement benefits
        IF (CONTRIB_UNEMPT_QT(pers_j).GT.0) THEN
            !compute early retirement benefit net of contributions
            CONTRIB_NETEAR_QT = 0
            IF (CONTRIB_EARSTART_QT(pers_j).EQ.1) THEN
                CONTRIB_NETEAR_QT = CONTRIB_UNEMPT_QT(pers_j) - (CONTRIB_EARPERC_QT(1)/100.D0)*CONTRIB_UNEMPT_QT(pers_j)
            ELSE IF (CONTRIB_EARSTART_QT(pers_j).EQ.2) THEN
                CONTRIB_NETEAR_QT = CONTRIB_UNEMPT_QT(pers_j) - (CONTRIB_EARPERC_QT(2)/100.D0)*CONTRIB_UNEMPT_QT(pers_j)
            END IF
            !select the relevant threshold for pensions conditional on family charge
            IF (CONTRIB_FAMCHARG_QT(pers_j).GT.0) THEN
                !check whether net early retirement benefits come below the threshold
                IF (CONTRIB_NETEAR_QT.LT.CONTRIB_EALIMPT_MONTH(1)*3.D0) THEN
                    CONTRIB_SSEAR_QT(2) = CONTRIB_UNEMPT_QT(pers_j) - (CONTRIB_EALIMPT_MONTH(1)*3.D0)
                ELSE
                    IF (CONTRIB_EARSTART_QT(pers_j).EQ.1) THEN
                        CONTRIB_SSEAR_QT(2)  = (CONTRIB_EARPERC_QT(1)/100.D0)*CONTRIB_UNEMPT_QT(pers_j)
                    ELSE IF (CONTRIB_EARSTART_QT(pers_j).EQ.2) THEN
                        CONTRIB_SSEAR_QT(2)  = (CONTRIB_EARPERC_QT(2)/100.D0)*CONTRIB_UNEMPT_QT(pers_j)
                    END IF
                END IF
            ELSE
                !check whether net early retirement benefits come below the threshold
                IF (CONTRIB_NETEAR_QT.LT.CONTRIB_EALIMPT_MONTH(2)*3.D0) THEN
                    CONTRIB_SSEAR_QT(2) = CONTRIB_UNEMPT_QT(pers_j) - (CONTRIB_EALIMPT_MONTH(2)*3.D0)
                ELSE
                    IF (CONTRIB_EARSTART_QT(pers_j).EQ.1) THEN
                        CONTRIB_SSEAR_QT(2)  = (CONTRIB_EARPERC_QT(1)/100.D0)*CONTRIB_UNEMPT_QT(pers_j)
                    ELSE IF (CONTRIB_EARSTART_QT(pers_j).EQ.2) THEN
                        CONTRIB_SSEAR_QT(2)  = (CONTRIB_EARPERC_QT(2)/100.D0)*CONTRIB_UNEMPT_QT(pers_j)
                    END IF
                END IF
            END IF
            !Set contribution equal to 0 if it would become negative
            IF (CONTRIB_SSEAR_QT(2).LT.0) THEN
                CONTRIB_SSEAR_QT(2) = 0
            END IF            
        END IF

        !Add contributions if there are early retirement benefits
        IF (CONTRIB_UNEMFT_QT(pers_j).GT.0.OR.CONTRIB_UNEMPT_QT(pers_j).GT.0) THEN
            CONTRIB_SSEARBEN_QT(pers_j) = CONTRIB_SSEAR_QT(1) + CONTRIB_SSEAR_QT(2)
        END IF
         
    !SOCIAL SECURITY CONTRIBUTIONS PAID ON DISABILITY BENEFITS
        !CONTRIB_SSDISAB_QT       Social security contributions paid on disability benefits           
        IF (CONTRIB_DISABWEA_QT(pers_j).GT.0) THEN
            !compute the amount of disability benefits net of contributions
            CONTRIB_NETDISA_QT = CONTRIB_DISABWEA_QT(pers_j) - (CONTRIB_DISAPERC_DAY/100.D0)*CONTRIB_DISABWEA_QT(pers_j)
            !select the relevant threshold for disability benefits conditional on family charge
            IF (CONTRIB_FAMCHARG_QT(pers_j).GT.0) THEN
                !check whether net benefits come below the threshold
                IF (CONTRIB_NETDISA_QT.LT.CONTRIB_DISABIM_DAY(1)*78.D0) THEN
                    CONTRIB_SSDISAB_QT(pers_j) = CONTRIB_DISABWEA_QT(pers_j) - (CONTRIB_DISABIM_DAY(1)*78.D0)
                ELSE
                    CONTRIB_SSDISAB_QT(pers_j) = (CONTRIB_DISAPERC_DAY/100.D0)*CONTRIB_DISABWEA_QT(pers_j)
                END IF
            ELSE
                !check whether net benefits come below the threshold
                IF (CONTRIB_NETDISA_QT.LT.CONTRIB_DISABIM_DAY(2)*78.D0) THEN
                    CONTRIB_SSDISAB_QT(pers_j) = CONTRIB_DISABWEA_QT(pers_j) - (CONTRIB_DISABIM_DAY(2)*78.D0)
                ELSE
                    CONTRIB_SSDISAB_QT(pers_j) = (CONTRIB_DISAPERC_DAY/100.D0)*CONTRIB_DISABWEA_QT(pers_j)
                END IF
            END IF
            !Set contribution equal to 0 if it would become negative
            IF (CONTRIB_SSDISAB_QT(pers_j).LT.0) THEN
                CONTRIB_SSDISAB_QT(pers_j) = 0
            END IF            
        END IF

    !SOCIAL SECURITY CONTRIBUTIONS PAID ON BENEFITS IN CASE OF INDUSTRIAL ACCIDENTS OR OCCUPATIONAL DISEASES
        !CONTRIB_SSINDOCC_QT        Social security contributions paid on benefits for industrial accidents or occupational diseases
        CONTRIB_SSINDOCC_QT(pers_j) = (CONTRIB_INDOCCPC_QT/100.D0)*CONTRIB_INDOCC_QT(pers_j)
        
END SUBROUTINE CONTRIB_SSBEFDED_NONLAB

!===================================================================================!
!With subroutine CONTRIB_EXCHANGE we construct the variables                        !
!based on social security contribution concepts, that have to be exchanged to       !
!other modules                                                                      !
!Variables constructed in this subroutine                                           !
!Gross taxable income concepts                                                      !
!   CONTRIB_GTIPRIV_QT                                                              !
!   CONTRIB_GTIPUB_QT                                                               !
!   CONTRIB_GTISELF_QT                                                              !
!   CONTRIB_GTIPENS_QT                                                              !
!   CONTRIB_GTIEARNE_QT                                                             !
!   CONTRIB_GTIEAROL_QT                                                             !
!   CONTRIB_GTIOLDUN_QT                                                             !
!   CONTRIB_GTIOTHUN_QT                                                             !
!   CONTRIB_GTIDISAB_QT                                                             !
!   CONTRIB_GTINDOCC_QT                                                             !
!   CONTRIB_GTISICK_QT                                                              !
!   CONTRIB_GTIFAMAL_QT                                                             !
!Other exchange concepts                                                            !
!   CONTRIB_REVPRIV_QT                                                              !
!   CONTRIB_REVPUB_QT                                                               !
!   CONTRIB_GREMPLOY_QT                                                             !
!   CONTRIB_SSEMPLOY_QT                                                             !
!   CONTRIB_SSREPLAC_QT                                                             !
!===================================================================================!

SUBROUTINE CONTRIB_EXCHANGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec

    !VARIABLES CONSTRUCTED TO EXCHANGE TO THE PIT MODULE
        !GROSS TAXABLE LABOUR MARKET INCOME
            !CONTRIB_GTIPRIV_QT       Gross taxable labour income earned as wage earner on the private labour market          
            !CONTRIB_GTIPUB_QT        Gross taxable labour income earned as wage earner on the public labour market           
            !CONTRIB_GTISELF_QT       Gross taxable labour income earned as self employed         
            CONTRIB_GTIPRIV_QT(pers_j)      = CONTRIB_INCPRIV_QT(pers_j) + CONTRIB_SHPRIV_QT(pers_j) + &
                                              &CONTRIB_DHPRIV_QT(pers_j) + CONTRIB_INCSPRIV_QT(pers_j) - &
                                              &CONTRIB_SSPRIV_EMEE_QT(pers_j)
            CONTRIB_GTIPUB_QT(pers_j)       = CONTRIB_INCPUB_QT(pers_j) + CONTRIB_SHPUB_QT(pers_j) + &
                                              &CONTRIB_DHPUB_QT(pers_j) + CONTRIB_INCSPUB_QT(pers_j) - &
                                              &CONTRIB_SSPUB_EMEE_QT(pers_j)
            CONTRIB_GTISELF_QT(pers_j)      = CONTRIB_INCSELF_QT(pers_j) 
        !GROSS TAXABLE NON LABOUR MARKET INCOME
        !GROSS TAXABLE PENSIONS
            !CONTRIB_GTIPENS_QT       Gross taxable pension benefits          
            CONTRIB_GTIPENS_QT(pers_j)      = CONTRIB_PENS_QT(pers_j) - CONTRIB_SSPEBASI_QT(pers_j) - &
                                              &CONTRIB_SSPESOLI_QT(pers_j)
        !GROSS TAXABLE UNEMPLOYMENT BENEFITS
            !CONTRIB_GTIEARNE_QT      Gross taxable early retirement benefits of the new type         
            !CONTRIB_GTIEAROL_QT      Gross taxable early retirement benefits of the old type         
            !CONTRIB_GTIOTHUN_QT      Gross taxable unemployment benefits, other than early retirement benefits           
            IF (CONTRIB_EARSTART_QT(pers_j).EQ.1) THEN
                CONTRIB_GTIEARNE_QT(pers_j) = 0
                CONTRIB_GTIEAROL_QT(pers_j) = CONTRIB_UNEMFT_QT(pers_j) + CONTRIB_UNEMPT_QT(pers_j) - CONTRIB_SSEARBEN_QT(pers_j)
            ELSE IF (CONTRIB_EARSTART_QT(pers_j).EQ.2) THEN
                CONTRIB_GTIEARNE_QT(pers_j) = CONTRIB_UNEMFT_QT(pers_j) + CONTRIB_UNEMPT_QT(pers_j) - CONTRIB_SSEARBEN_QT(pers_j)
                CONTRIB_GTIEAROL_QT(pers_j) = 0
            END IF
            CONTRIB_GTIOLDUN_QT(pers_j)     = UNEM_OLDUNSS_QT(pers_j)            
            CONTRIB_GTIOTHUN_QT(pers_j)     = UNEM_OTHERUN_QT(pers_j)            
        !GROSS TAXABLE SICKNESS AND DISABILITY BENEFITS
            !CONTRIB_GTIDISAB_QT    Gross taxable disability benefits of wage earners
            !CONTRIB_GTINDOCC_QT    Gross taxable benefits in case of industrial accidents or occupational diseases (benefits for help of a third person excluded)
            !CONTRIB_GTISICK_QT     Gross taxable benefits in case of sickness and disability not covered by CONTRIB_GTIDISAB_QT and CONTRIB_GTINDOCC_QT          
            CONTRIB_GTIDISAB_QT(pers_j)     = CONTRIB_DISABWEA_QT(pers_j) - CONTRIB_SSDISAB_QT(pers_j)
            CONTRIB_GTINDOCC_QT(pers_j)     = CONTRIB_INDOCC_QT(pers_j) - CONTRIB_SSINDOCC_QT(pers_j)
            CONTRIB_GTISICK_QT(pers_j)      = SICK_ALOTHBEN_QT(pers_j)
        !GROSS TAXABLE FAMILY ALLOWANCES
            !CONTRIB_GTIFAMAL_QT      Gross taxable income from family allowances         
            CONTRIB_GTIFAMAL_QT(pers_j)     = FAMAL_AMOUNTF_QT(pers_j)

    !VARIABLES CONSTRUCTED TO EXCHANGE TO THE UNEM AND SICK MODULE
        !CONTRIB_REVPRIV_QT       Sum of gross labour income and holiday earnings, earned as wage earner on the private labour market         
        !CONTRIB_REVPUB_QT        Sum of gross labour income and holiday earnings, earned as wage earner on the public labour market          
        CONTRIB_REVPRIV_QT(pers_j)          = CONTRIB_INCPRIV_QT(pers_j) + CONTRIB_SHPRIV_QT(pers_j) + &
                                            &CONTRIB_DHPRIV_QT(pers_j) + CONTRIB_INCSPRIV_QT(pers_j)
        CONTRIB_REVPUB_QT(pers_j)           = CONTRIB_INCPUB_QT(pers_j) + CONTRIB_SHPUB_QT(pers_j) + &
                                            &CONTRIB_DHPUB_QT(pers_j) + CONTRIB_INCSPUB_QT(pers_j)

    !VARIABLES CONSTRUCTED TO EXCHANGE TO THE FAMAL MODULE
        !CONTRIB_GREMPLOY_QT: Gross income from employment        
        !CONTRIB_SSEMPLOY_QT: Social security contributions paid on income from employment        
        CONTRIB_GREMPLOY_QT(pers_j)         = CONTRIB_INCPRIV_QT(pers_j) + CONTRIB_SHPRIV_QT(pers_j) + &
                                            &CONTRIB_DHPRIV_QT(pers_j) + CONTRIB_INCSPRIV_QT(pers_j) + &
                                            &CONTRIB_INCPUB_QT(pers_j) + CONTRIB_SHPUB_QT(pers_j) + &
                                            &CONTRIB_DHPUB_QT(pers_j) + CONTRIB_INCSPUB_QT(pers_j) 
        CONTRIB_SSEMPLOY_QT(pers_j)         = CONTRIB_SSPRIV_EMEE_QT(pers_j) + &
                                            &CONTRIB_SSPUB_EMEE_QT(pers_j)

    !VARIABLES CONSTRUCTED TO EXCHANGE TO THE EVAL MODULE    
        !CONTRIB_SSREPLAC_QT: Social security contributions paid on all sorts of replacement income
        CONTRIB_SSREPLAC_QT(pers_j)         = CONTRIB_SSPEBASI_QT(pers_j) + CONTRIB_SSPESOLI_QT(pers_j) + &
                                            &CONTRIB_SSEARBEN_QT(pers_j) + &
                                            &CONTRIB_SSDISAB_QT(pers_j) + &
                                            &CONTRIB_SSINDOCC_QT(pers_j)
    END DO
    
END SUBROUTINE CONTRIB_EXCHANGE

!==========================================================!
!Produce some aggregate statistics to check the constuction
!==========================================================!

SUBROUTINE CONTRIB_TEST

    INTEGER                                                 :: pers_j
    INTEGER                                                 :: NumAnal                              !Number of summary statistics computed 
    REAL(KIND=DPRECI)                                       :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: CONTRIB_RESULT_TEST                  !matrix with test results
    CHARACTER(len=75),ALLOCATABLE,DIMENSION(:)              :: CONTRIB_LAB_TEST                     !Labels of test variables
    REAL(KIND=DPRECI),DIMENSION(22)                         :: SumGross                             !Vector with quarterly amounts on gross incomes

    WRITE(15,'(/,''Print of test statistics'')')
    CALL DRAWLINE(15,136,'=')    

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    
    !Compute aggregate statistics on labour market statuses

    !Private labour market status
    NumAnal = 3
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0    

    !Define labels
    CONTRIB_LAB_TEST(1) = "Not on the private labour market                   "
    CONTRIB_LAB_TEST(2) = "Blue collar worker                                 "
    CONTRIB_LAB_TEST(3) = "White collar worker                                "
    
    DO pers_j=1,n_rec
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.0) THEN
            CONTRIB_RESULT_TEST(1,1) = CONTRIB_RESULT_TEST(1,1) + 1
            CONTRIB_RESULT_TEST(1,2) = CONTRIB_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            CONTRIB_RESULT_TEST(1,4) = CONTRIB_RESULT_TEST(1,4) + MIMOSIS_WEIGHT(pers_j)*SUMSAL_2008(pers_j)
        ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1.AND.CONTRIB_INCPRIV_QT(pers_j).GT.0.D0) THEN
            CONTRIB_RESULT_TEST(2,1) = CONTRIB_RESULT_TEST(2,1) + 1
            CONTRIB_RESULT_TEST(2,2) = CONTRIB_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            CONTRIB_RESULT_TEST(2,4) = CONTRIB_RESULT_TEST(2,4) + MIMOSIS_WEIGHT(pers_j)*SUMSAL_2008(pers_j)
        ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2.AND.CONTRIB_INCPRIV_QT(pers_j).GT.0.D0) THEN
            CONTRIB_RESULT_TEST(3,1) = CONTRIB_RESULT_TEST(3,1) + 1
            CONTRIB_RESULT_TEST(3,2) = CONTRIB_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
            CONTRIB_RESULT_TEST(3,4) = CONTRIB_RESULT_TEST(3,4) + MIMOSIS_WEIGHT(pers_j)*SUMSAL_2008(pers_j)
        END IF
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                              TotSamp  TotSampW      %Pop&
    &             obs inc'')')
    Write(15,'(''Counts of variable CONTRIB_LABMSTAT_PRIV_QT: labour market status on the private labour market'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2,f20.0)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)
    DEALLOCATE(CONTRIB_LAB_TEST)
    
    !Public labour market status
    NumAnal = 5
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0    

    !Define labels
    CONTRIB_LAB_TEST(1) = "Not on the public labour market                    "
    CONTRIB_LAB_TEST(2) = "Blue collar worker without statutory service       "
    CONTRIB_LAB_TEST(3) = "White collar worker without statutory service      "
    CONTRIB_LAB_TEST(4) = "Blue collar worker with statutory service          "
    CONTRIB_LAB_TEST(5) = "White collar worker with statutory service         "

    DO pers_j=1,n_rec
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0) THEN
            CONTRIB_RESULT_TEST(1,1) = CONTRIB_RESULT_TEST(1,1) + 1
            CONTRIB_RESULT_TEST(1,2) = CONTRIB_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            CONTRIB_RESULT_TEST(1,4) = CONTRIB_RESULT_TEST(1,4) + MIMOSIS_WEIGHT(pers_j)&
            &*(SUMSAL_2008(pers_j)+APL_SUMSAL_2008(pers_j))
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1.AND.CONTRIB_INCPUB_QT(pers_j).GT.0.D0) THEN
            CONTRIB_RESULT_TEST(2,1) = CONTRIB_RESULT_TEST(2,1) + 1
            CONTRIB_RESULT_TEST(2,2) = CONTRIB_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            CONTRIB_RESULT_TEST(2,4) = CONTRIB_RESULT_TEST(2,4) + MIMOSIS_WEIGHT(pers_j)&
            &*(SUMSAL_2008(pers_j)+APL_SUMSAL_2008(pers_j))
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2.AND.CONTRIB_INCPUB_QT(pers_j).GT.0.D0) THEN
            CONTRIB_RESULT_TEST(3,1) = CONTRIB_RESULT_TEST(3,1) + 1
            CONTRIB_RESULT_TEST(3,2) = CONTRIB_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
            CONTRIB_RESULT_TEST(3,4) = CONTRIB_RESULT_TEST(3,4) + MIMOSIS_WEIGHT(pers_j)&
            &*(SUMSAL_2008(pers_j)+APL_SUMSAL_2008(pers_j))
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.3.AND.CONTRIB_INCPUB_QT(pers_j).GT.0.D0) THEN
            CONTRIB_RESULT_TEST(4,1) = CONTRIB_RESULT_TEST(4,1) + 1
            CONTRIB_RESULT_TEST(4,2) = CONTRIB_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
            CONTRIB_RESULT_TEST(4,4) = CONTRIB_RESULT_TEST(4,4) + MIMOSIS_WEIGHT(pers_j)&
            &*(SUMSAL_2008(pers_j)+APL_SUMSAL_2008(pers_j))
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.4.AND.CONTRIB_INCPUB_QT(pers_j).GT.0.D0) THEN
            CONTRIB_RESULT_TEST(5,1) = CONTRIB_RESULT_TEST(5,1) + 1
            CONTRIB_RESULT_TEST(5,2) = CONTRIB_RESULT_TEST(5,2) + MIMOSIS_WEIGHT(pers_j)
            CONTRIB_RESULT_TEST(5,4) = CONTRIB_RESULT_TEST(5,4) + MIMOSIS_WEIGHT(pers_j)&
            &*(SUMSAL_2008(pers_j)+APL_SUMSAL_2008(pers_j))
        END IF
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                              TotSamp  TotSampW      %Pop&
    &             obs inc'')')
    Write(15,'(''Counts of variable CONTRIB_LABMSTAT_PUB_QT: labour market status on the public labour market'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2,f20.0)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)
    DEALLOCATE(CONTRIB_LAB_TEST)

    !Self employed
    NumAnal = 2
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,3))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    CONTRIB_LAB_TEST(1) = "Not self employed                                  "
    CONTRIB_LAB_TEST(2) = "Self employed                                      "

    DO pers_j=1,n_rec
        IF (CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.0) THEN
            CONTRIB_RESULT_TEST(1,1) = CONTRIB_RESULT_TEST(1,1) + 1
            CONTRIB_RESULT_TEST(1,2) = CONTRIB_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
        ELSE IF (CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.1) THEN
            CONTRIB_RESULT_TEST(2,1) = CONTRIB_RESULT_TEST(2,1) + 1
            CONTRIB_RESULT_TEST(2,2) = CONTRIB_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                              TotSamp  TotSampW      %Pop'')')
    Write(15,'(''Counts of variable CONTRIB_LABMSTAT_SELF_QT: labour market status of self employed'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,3)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)
    DEALLOCATE(CONTRIB_LAB_TEST)

    !Social benefits with possibly contributions on it
    NumAnal = 3
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,3))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    CONTRIB_LAB_TEST(1) = "Pensions are <> 0 "
    CONTRIB_LAB_TEST(2) = "Early retirement benefits are <> 0 "
    CONTRIB_LAB_TEST(3) = "Disability benefits are <> 0"

    DO pers_j=1,n_rec
        IF (INT(100.D0*(CONTRIB_PENS_QT(pers_j))).NE.0) THEN
            CONTRIB_RESULT_TEST(1,1) = CONTRIB_RESULT_TEST(1,1) + 1
            CONTRIB_RESULT_TEST(1,2) = CONTRIB_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (INT(100.D0*(CONTRIB_UNEMFT_QT(pers_j)+CONTRIB_UNEMPT_QT(pers_j))).NE.0) THEN
            CONTRIB_RESULT_TEST(2,1) = CONTRIB_RESULT_TEST(2,1) + 1
            CONTRIB_RESULT_TEST(2,2) = CONTRIB_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (INT(100.D0*(CONTRIB_DISABWEA_QT(pers_j))).NE.0) THEN
            CONTRIB_RESULT_TEST(3,1) = CONTRIB_RESULT_TEST(3,1) + 1
            CONTRIB_RESULT_TEST(3,2) = CONTRIB_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                              TotSamp  TotSampW      %Pop'')')
    Write(15,'(''Tests to see whether people receive a social benefit on which ss contributions are due'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,3)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)
    DEALLOCATE(CONTRIB_LAB_TEST)

    !Tests to see whether people are NOT gifted with certain status but still pay contributions for this status
    NumAnal = 6
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,3))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    CONTRIB_LAB_TEST(1) = "Not a private labour market wage earner but contributions are <> 0"
    CONTRIB_LAB_TEST(2) = "Not a public labour market wage earner but contributions are <> 0"
    CONTRIB_LAB_TEST(3) = "Not a self employed but contributions are <> 0"
    CONTRIB_LAB_TEST(4) = "Pensions are 0 but contributions are <> 0"
    CONTRIB_LAB_TEST(5) = "Early retirement benefits are 0 but contributions are <> 0"
    CONTRIB_LAB_TEST(6) = "Disability benefits are 0 but contributions are <> 0"

    DO pers_j=1,n_rec
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.0.AND.INT(100.D0*(CONTRIB_SSPRIV_EMEE_QT(pers_j)+&
        &CONTRIB_SSPRIV_EMER_QT(pers_j))).NE.0) THEN
            CONTRIB_RESULT_TEST(1,1) = CONTRIB_RESULT_TEST(1,1) + 1
            CONTRIB_RESULT_TEST(1,2) = CONTRIB_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.0.AND.INT(100.D0*(CONTRIB_SSPUB_EMEE_QT(pers_j)+&
        &CONTRIB_SSPUB_EMER_QT(pers_j))).NE.0) THEN
            CONTRIB_RESULT_TEST(2,1) = CONTRIB_RESULT_TEST(2,1) + 1
            CONTRIB_RESULT_TEST(2,2) = CONTRIB_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.0.AND.INT(100.D0*(CONTRIB_SSSELF_EMER_QT(pers_j))).NE.0) THEN
            CONTRIB_RESULT_TEST(3,1) = CONTRIB_RESULT_TEST(3,1) + 1
            CONTRIB_RESULT_TEST(3,2) = CONTRIB_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (INT(100.D0*(CONTRIB_PENS_QT(pers_j))).EQ.0.AND.INT(100.D0*(CONTRIB_SSPEBASI_QT(pers_j)+&
        &CONTRIB_SSPESOLI_QT(pers_j))).NE.0) THEN
            CONTRIB_RESULT_TEST(4,1) = CONTRIB_RESULT_TEST(4,1) + 1
            CONTRIB_RESULT_TEST(4,2) = CONTRIB_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (INT(100.D0*(CONTRIB_UNEMFT_QT(pers_j)+CONTRIB_UNEMPT_QT(pers_j))).EQ.0.AND.&
        &INT(100.D0*(CONTRIB_SSEARBEN_QT(pers_j))).NE.0) THEN
            CONTRIB_RESULT_TEST(5,1) = CONTRIB_RESULT_TEST(5,1) + 1
            CONTRIB_RESULT_TEST(5,2) = CONTRIB_RESULT_TEST(5,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (INT(100.D0*(CONTRIB_DISABWEA_QT(pers_j))).EQ.0.AND.INT(100.D0*(CONTRIB_SSDISAB_QT(pers_j))).NE.0) THEN
            CONTRIB_RESULT_TEST(6,1) = CONTRIB_RESULT_TEST(6,1) + 1
            CONTRIB_RESULT_TEST(6,2) = CONTRIB_RESULT_TEST(6,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                              TotSamp  TotSampW      %Pop'')')
    Write(15,'(''Tests to see whether people are NOT gifted with certain status but still pay contributions for this status'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,3)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)
    DEALLOCATE(CONTRIB_LAB_TEST)

    !Tests to see whether people are gifted with a certain status but do not pay contributions for this status
    NumAnal = 6
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,3))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    CONTRIB_LAB_TEST(1) = "A private labour market wage earner but contributions are = 0"
    CONTRIB_LAB_TEST(2) = "A public labour market wage earner but contributions are = 0"
    CONTRIB_LAB_TEST(3) = "A self employed but contributions are = 0"
    CONTRIB_LAB_TEST(4) = "Pensions are <> 0 but contributions are = 0"
    CONTRIB_LAB_TEST(5) = "Early retirement benefits are <> 0 but contributions are = 0"
    CONTRIB_LAB_TEST(6) = "Disability benefits are <> 0 but contributions are = 0"

    DO pers_j=1,n_rec
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).NE.0.AND.INT(100.D0*(CONTRIB_SSPRIV_EMEE_QT(pers_j)+&
        &CONTRIB_SSPRIV_EMER_QT(pers_j))).EQ.0) THEN
            CONTRIB_RESULT_TEST(1,1) = CONTRIB_RESULT_TEST(1,1) + 1
            CONTRIB_RESULT_TEST(1,2) = CONTRIB_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).NE.0.AND.INT(100.D0*(CONTRIB_SSPUB_EMEE_QT(pers_j)+&
        &CONTRIB_SSPUB_EMER_QT(pers_j))).EQ.0) THEN
            CONTRIB_RESULT_TEST(2,1) = CONTRIB_RESULT_TEST(2,1) + 1
            CONTRIB_RESULT_TEST(2,2) = CONTRIB_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (CONTRIB_LABMSTAT_SELF_QT(pers_j).NE.0.AND.INT(100.D0*(CONTRIB_SSSELF_EMER_QT(pers_j))).EQ.0) THEN
            CONTRIB_RESULT_TEST(3,1) = CONTRIB_RESULT_TEST(3,1) + 1
            CONTRIB_RESULT_TEST(3,2) = CONTRIB_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (INT(100.D0*(CONTRIB_PENS_QT(pers_j))).NE.0.AND.INT(100.D0*(CONTRIB_SSPEBASI_QT(pers_j)+&
        &CONTRIB_SSPESOLI_QT(pers_j))).EQ.0) THEN
            CONTRIB_RESULT_TEST(4,1) = CONTRIB_RESULT_TEST(4,1) + 1
            CONTRIB_RESULT_TEST(4,2) = CONTRIB_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (INT(100.D0*(CONTRIB_UNEMFT_QT(pers_j)+CONTRIB_UNEMPT_QT(pers_j))).NE.0.AND.&
        &INT(100.D0*(CONTRIB_SSEARBEN_QT(pers_j))).EQ.0) THEN
            CONTRIB_RESULT_TEST(5,1) = CONTRIB_RESULT_TEST(5,1) + 1
            CONTRIB_RESULT_TEST(5,2) = CONTRIB_RESULT_TEST(5,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
        IF (INT(100.D0*(CONTRIB_DISABWEA_QT(pers_j))).NE.0.AND.INT(100.D0*(CONTRIB_SSDISAB_QT(pers_j))).EQ.0) THEN
            CONTRIB_RESULT_TEST(6,1) = CONTRIB_RESULT_TEST(6,1) + 1
            CONTRIB_RESULT_TEST(6,2) = CONTRIB_RESULT_TEST(6,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                              TotSamp  TotSampW      %Pop'')')
    Write(15,'(''Tests to see whether people are gifted with certain status but do not pay contributions for this status'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,3)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)
    DEALLOCATE(CONTRIB_LAB_TEST)

    !Compute aggregate income statistics
    NumAnal = 12

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Labour market income of wage earners on the private labour market          "
    CONTRIB_LAB_TEST(2) = "Single holiday earnings of wage earners on the private labour market       "
    CONTRIB_LAB_TEST(3) = "Double holiday earnings of wage earners on the private labour market       "
    CONTRIB_LAB_TEST(4) = "Supplement to real earned labour income, other than holiday earnings, obtained &
    &as wage earner on the private labour market"
    CONTRIB_LAB_TEST(5) = "Labour market income of wage earners on the public labour market           "
    CONTRIB_LAB_TEST(6) = "Single holiday earnings of wage earners on the public labour market        "
    CONTRIB_LAB_TEST(7) = "Double holiday earnings of wage earners on the public labour market        "
    CONTRIB_LAB_TEST(8) = "Supplement to real earned labour income, other than holiday earnings, obtained &
    &as wage earner on the public labour market"
    CONTRIB_LAB_TEST(9) = "Income of self employed                                                    "
    CONTRIB_LAB_TEST(10) = "Pensions  "
    CONTRIB_LAB_TEST(11) = "Early retirement benefits"
    CONTRIB_LAB_TEST(12) = "Disability benefits"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = CONTRIB_INCPRIV_QT(pers_j)
        SumGross(2) = CONTRIB_SHPRIV_QT(pers_j)
        SumGross(3) = CONTRIB_DHPRIV_QT(pers_j)
        SumGross(4) = CONTRIB_INCSPRIV_QT(pers_j)
        SumGross(5) = CONTRIB_INCPUB_QT(pers_j)
        SumGross(6) = CONTRIB_SHPUB_QT(pers_j)
        SumGross(7) = CONTRIB_DHPUB_QT(pers_j)
        SumGross(8) = CONTRIB_INCSPUB_QT(pers_j)
        SumGross(9) = CONTRIB_INCSELF_QT(pers_j)
        SumGross(10)= CONTRIB_PENS_QT(pers_j)
        SumGross(11)= CONTRIB_UNEMFT_QT(pers_j)+CONTRIB_UNEMPT_QT(pers_j)
        SumGross(12) = CONTRIB_DISABWEA_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Quarterly incomes that serve as basis to compute the different contributions'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)
    DEALLOCATE(CONTRIB_LAB_TEST)

    Write(15,'(/,''Some statistics on the contributions of wage earners on the private labour market'')')
    CALL DRAWLINE(15,136,'-')    

    !Compute aggregate statistics on contributions of private labour market wage earners before deductions
    NumAnal = 4

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Employee cont. for wage earners on the priv. labour market (bef. ded.)     "
    CONTRIB_LAB_TEST(2) = "Employer cont. for wage earners on the priv. labour market (bef. ded.)     "
    CONTRIB_LAB_TEST(3) = "SS for holiday earn of blue collar workers on the priv. lab (bef.ded.)     "
    CONTRIB_LAB_TEST(4) = "SS on double hol. earn. for wage earn on the priv. lab (bef.ded.)          "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = SUM(CONTRIB_WEARCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_QT,1)))
        SumGross(2) = SUM(CONTRIB_WEARCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_QT,1)))
        SumGross(3) = CONTRIB_WEARHOLA_EMER_QT(pers_j)
        SumGross(4) = CONTRIB_WEARHOLD_EMEE_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)    
    DEALLOCATE(CONTRIB_LAB_TEST)

    !Compute aggregate statistics on employer and employee ss deductions for private labour market wage earners
    NumAnal = 22

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "The structural deduction of employers contributions: general case         "
    CONTRIB_LAB_TEST(2) = "The deduction of employers contributions to stimulate employment for non m"
    CONTRIB_LAB_TEST(3) = "Target-group deduction: plans +1, +2 and +3 deductions                     "
    CONTRIB_LAB_TEST(4) = "Target-group deduction: plan for job-seekers                               "
    CONTRIB_LAB_TEST(5) = "Target-group deduction: job-training deductions (ar. 495)                  "
    CONTRIB_LAB_TEST(6) = "Target-group deduction: subsidized contract worker                         "
    CONTRIB_LAB_TEST(7) = "Target-group deduction: deduction for job redistribution in the public sect"
    CONTRIB_LAB_TEST(8) = "Target-group deduction: (ar. 230)                                          "
    CONTRIB_LAB_TEST(9) = "Target-group deduction: the deduction for hiring disadvantaged you ar. 499)"
    CONTRIB_LAB_TEST(10) = "Target-group deduction: the domestic person deduction (ar. 483)            "
    CONTRIB_LAB_TEST(11) = "Target-group deduction: the deduction for professional career interruption "
    CONTRIB_LAB_TEST(12) = "Target-group deduction: the deduction for half-time pre-retirement         "
    CONTRIB_LAB_TEST(13) = "The structural deduction of employers contributions: stimu(social maribel)"
    CONTRIB_LAB_TEST(14) = "Target-group deduction: the scientific research deduction                  "
    CONTRIB_LAB_TEST(15) = "Target-group deduction: labour time reductions by collective agreement gran"
    CONTRIB_LAB_TEST(16) = "Target-group deduction: dragging and tugging sector deduction              "
    CONTRIB_LAB_TEST(17) = "Target-group deduction: the deduction for activation of unemployment allowa"
    CONTRIB_LAB_TEST(18) = "Target-group deduction: the 4-days week deduction                          "
    CONTRIB_LAB_TEST(19) = "Target-group deduction: the deduction for unemployed persons diff(smet job)"
    CONTRIB_LAB_TEST(20) = "Target-group deduction: job starters (rosetta plan)                        "
    CONTRIB_LAB_TEST(21) = "Target-group deduction: the deduction for time working reduction to 38 hour"
    CONTRIB_LAB_TEST(22) = "Deduction on employee contributions for low wage employees                 "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1)  = CONTRIB_WEARDEDU_EMER_QT(pers_j,1) 
        SumGross(2)  = CONTRIB_WEARDEDU_EMER_QT(pers_j,2) 
        SumGross(3)  = CONTRIB_WEARDEDU_EMER_QT(pers_j,3) 
        SumGross(4)  = CONTRIB_WEARDEDU_EMER_QT(pers_j,4) 
        SumGross(5)  = CONTRIB_WEARDEDU_EMER_QT(pers_j,5) 
        SumGross(6)  = CONTRIB_WEARDEDU_EMER_QT(pers_j,6) 
        SumGross(7)  = CONTRIB_WEARDEDU_EMER_QT(pers_j,7) 
        SumGross(8)  = CONTRIB_WEARDEDU_EMER_QT(pers_j,8) 
        SumGross(9)  = CONTRIB_WEARDEDU_EMER_QT(pers_j,9) 
        SumGross(10) = CONTRIB_WEARDEDU_EMER_QT(pers_j,10)
        SumGross(11) = CONTRIB_WEARDEDU_EMER_QT(pers_j,11)
        SumGross(12) = CONTRIB_WEARDEDU_EMER_QT(pers_j,12)
        SumGross(13) = CONTRIB_WEARDEDU_EMER_QT(pers_j,13)
        SumGross(14) = CONTRIB_WEARDEDU_EMER_QT(pers_j,14)
        SumGross(15) = CONTRIB_WEARDEDU_EMER_QT(pers_j,15)
        SumGross(16) = CONTRIB_WEARDEDU_EMER_QT(pers_j,16)
        SumGross(17) = CONTRIB_WEARDEDU_EMER_QT(pers_j,17)
        SumGross(18) = CONTRIB_WEARDEDU_EMER_QT(pers_j,18)
        SumGross(19) = CONTRIB_WEARDEDU_EMER_QT(pers_j,19)
        SumGross(20) = CONTRIB_WEARDEDU_EMER_QT(pers_j,20)
        SumGross(21) = CONTRIB_WEARDEDU_EMER_QT(pers_j,21)
        SumGross(22) = CONTRIB_WEARDEDU_EMEE_QT(pers_j)  
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    !Write results
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST) 
    DEALLOCATE(CONTRIB_LAB_TEST)

    !Compute aggregate statistics on contributions and deductions of private labour market wage earners before deductions
    NumAnal = 3

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Sum of all employee and employer contributions before deductions           "
    CONTRIB_LAB_TEST(2) = "Sum of deductions on employee and employer contributions                   "
    CONTRIB_LAB_TEST(3) = "Contributions net of deductions                                            "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = SUM(CONTRIB_WEARCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_QT,1))) + &
                    & SUM(CONTRIB_WEARCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_QT,1))) + &
                    & CONTRIB_WEARHOLA_EMER_QT(pers_j)                                  + &
                    & CONTRIB_WEARHOLD_EMEE_QT(pers_j)
        SumGross(2) = SUM(CONTRIB_WEARDEDU_EMER_QT(pers_j,1:21)) + &
                    & CONTRIB_WEARDEDU_EMEE_QT(pers_j)
        SumGross(3) = SumGross(1) - SumGross(2) 
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST) 
    DEALLOCATE(CONTRIB_LAB_TEST)

    Write(15,'(/,''Some statistics on the contributions of wage earners on the public labour market with non statutory service'')')
    CALL DRAWLINE(15,100,'-')    

    !Compute aggregate statistics on contributions of public labour market wage earners without stat service before deductions
    NumAnal = 4

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Employee cont. for wage earners on the pub. labour market nosta (bef. ded.)"
    CONTRIB_LAB_TEST(2) = "Employer cont. for wage earners on the pub. labour market nosta (bef. ded.)"
    CONTRIB_LAB_TEST(3) = "SS for holiday earn of blue collar workers on the pub. lab nosta (bef.ded.)"
    CONTRIB_LAB_TEST(4) = "SS on double hol. earn. for wage earn on the pub. lab nosta (bef.ded.)     "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = SUM(CONTRIB_PUBNSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_QT,1)))
        SumGross(2) = SUM(CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_QT,1)))
        SumGross(3) = CONTRIB_PUBNSHOL_EMER_QT(pers_j)
        SumGross(4) = CONTRIB_PUBNSHOL_EMEE_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)    
    DEALLOCATE(CONTRIB_LAB_TEST)
   
    !Compute aggregate statistics on employer and employee ss deductions for public labour market wage earners without stat serv.
    NumAnal = 22

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "The structural deduction of employers contributions: general case         "
    CONTRIB_LAB_TEST(2) = "The deduction of employers contributions to stimulate employment for non m"
    CONTRIB_LAB_TEST(3) = "Target-group deduction: plans +1, +2 and +3 deductions                     "
    CONTRIB_LAB_TEST(4) = "Target-group deduction: plan for job-seekers                               "
    CONTRIB_LAB_TEST(5) = "Target-group deduction: job-training deductions (ar. 495)                  "
    CONTRIB_LAB_TEST(6) = "Target-group deduction: subsidized contract worker                         "
    CONTRIB_LAB_TEST(7) = "Target-group deduction: deduction for job redistribution in the public sect"
    CONTRIB_LAB_TEST(8) = "Target-group deduction: (ar. 230)                                          "
    CONTRIB_LAB_TEST(9) = "Target-group deduction: the deduction for hiring disadvantaged you ar. 499)"
    CONTRIB_LAB_TEST(10) = "Target-group deduction: the domestic person deduction (ar. 483)            "
    CONTRIB_LAB_TEST(11) = "Target-group deduction: the deduction for professional career interruption "
    CONTRIB_LAB_TEST(12) = "Target-group deduction: the deduction for half-time pre-retirement         "
    CONTRIB_LAB_TEST(13) = "The structural deduction of employers contributions: stimu(social maribel)"
    CONTRIB_LAB_TEST(14) = "Target-group deduction: the scientific research deduction                  "
    CONTRIB_LAB_TEST(15) = "Target-group deduction: labour time reductions by collective agreement gran"
    CONTRIB_LAB_TEST(16) = "Target-group deduction: dragging and tugging sector deduction              "
    CONTRIB_LAB_TEST(17) = "Target-group deduction: the deduction for activation of unemployment allowa"
    CONTRIB_LAB_TEST(18) = "Target-group deduction: the 4-days week deduction                          "
    CONTRIB_LAB_TEST(19) = "Target-group deduction: the deduction for unemployed persons diff(smet job)"
    CONTRIB_LAB_TEST(20) = "Target-group deduction: job starters (rosetta plan)                        "
    CONTRIB_LAB_TEST(21) = "Target-group deduction: the deduction for time working reduction to 38 hour"
    CONTRIB_LAB_TEST(22) = "Deduction on employee contributions for low wage employees                 "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1)  = CONTRIB_PUBNSDED_EMER_QT(pers_j,1) 
        SumGross(2)  = CONTRIB_PUBNSDED_EMER_QT(pers_j,2) 
        SumGross(3)  = CONTRIB_PUBNSDED_EMER_QT(pers_j,3) 
        SumGross(4)  = CONTRIB_PUBNSDED_EMER_QT(pers_j,4) 
        SumGross(5)  = CONTRIB_PUBNSDED_EMER_QT(pers_j,5) 
        SumGross(6)  = CONTRIB_PUBNSDED_EMER_QT(pers_j,6) 
        SumGross(7)  = CONTRIB_PUBNSDED_EMER_QT(pers_j,7) 
        SumGross(8)  = CONTRIB_PUBNSDED_EMER_QT(pers_j,8) 
        SumGross(9)  = CONTRIB_PUBNSDED_EMER_QT(pers_j,9) 
        SumGross(10) = CONTRIB_PUBNSDED_EMER_QT(pers_j,10)
        SumGross(11) = CONTRIB_PUBNSDED_EMER_QT(pers_j,11)
        SumGross(12) = CONTRIB_PUBNSDED_EMER_QT(pers_j,12)
        SumGross(13) = CONTRIB_PUBNSDED_EMER_QT(pers_j,13)
        SumGross(14) = CONTRIB_PUBNSDED_EMER_QT(pers_j,14)
        SumGross(15) = CONTRIB_PUBNSDED_EMER_QT(pers_j,15)
        SumGross(16) = CONTRIB_PUBNSDED_EMER_QT(pers_j,16)
        SumGross(17) = CONTRIB_PUBNSDED_EMER_QT(pers_j,17)
        SumGross(18) = CONTRIB_PUBNSDED_EMER_QT(pers_j,18)
        SumGross(19) = CONTRIB_PUBNSDED_EMER_QT(pers_j,19)
        SumGross(20) = CONTRIB_PUBNSDED_EMER_QT(pers_j,20)
        SumGross(21) = CONTRIB_PUBNSDED_EMER_QT(pers_j,21)
        SumGross(22) = CONTRIB_PUBNSDED_EMEE_QT(pers_j)  
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST) 
    DEALLOCATE(CONTRIB_LAB_TEST)

    !Compute aggregate statistics on contributions and deductions of public labour market wage earners without statutory service
    NumAnal = 3

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Sum of all employee and employer contributions before deductions           "
    CONTRIB_LAB_TEST(2) = "Sum of deductions on employee and employer contributions                   "
    CONTRIB_LAB_TEST(3) = "Contributions net of deductions                                            "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = SUM(CONTRIB_PUBNSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_QT,1))) + &
                    & SUM(CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_QT,1))) + &
                    & CONTRIB_PUBNSHOL_EMER_QT(pers_j)                                  + &
                    & CONTRIB_PUBNSHOL_EMEE_QT(pers_j)
        SumGross(2) = SUM(CONTRIB_PUBNSDED_EMER_QT(pers_j,1:21)) + &
                    & CONTRIB_PUBNSDED_EMEE_QT(pers_j)
        SumGross(3) = SumGross(1) - SumGross(2) 
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST) 
    DEALLOCATE(CONTRIB_LAB_TEST)

    Write(15,'(/,''Some statistics on the contributions of wage earners on the public labour market with statutory service'')')
    CALL DRAWLINE(15,100,'-')    

    !Compute aggregate statistics on contributions of public labour market wage earners with stat before deductions
    NumAnal = 3

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Employee cont. for wage earners on the pub. labour market sta (bef. ded.)  "
    CONTRIB_LAB_TEST(2) = "Employer cont. for wage earners on the pub. labour market sta (bef. ded.)  "
    CONTRIB_LAB_TEST(3) = "SS on double hol. earn. for wage earn on the pub. lab sta (bef.ded.)       "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = SUM(CONTRIB_PUBSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_CSCOMFED_QT,1)))
        SumGross(2) = SUM(CONTRIB_PUBSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_CSCOMFED_QT,1)))
        SumGross(3) = CONTRIB_PUBSHOL_EMEE_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)    
    DEALLOCATE(CONTRIB_LAB_TEST)
   
    !Compute aggregate statistics on employer and employee ss deductions for public labour market wage earners with statutory service
    NumAnal = 8

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Target-group deduction: subsidized contract worker                         "
    CONTRIB_LAB_TEST(2) = "Target-group deduction: employment incentive plan for the job seekers recru"
    CONTRIB_LAB_TEST(3) = "Target-group deduction: job redistribution in the public sector            "
    CONTRIB_LAB_TEST(4) = "Target-group deduction: occupation in accordance with art. 60  7 of the la"
    CONTRIB_LAB_TEST(5) = "Target-group deduction: social maribel                                     "
    CONTRIB_LAB_TEST(6) = "Target-group deduction: professional transition program                    "
    CONTRIB_LAB_TEST(7) = "Target-group deduction: service jobs  smet jobs                           "
    CONTRIB_LAB_TEST(8) = "Deduction on employee contributions for low wage employees                 "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1)  = CONTRIB_PUBSDED_EMER_QT(pers_j,1) 
        SumGross(2)  = CONTRIB_PUBSDED_EMER_QT(pers_j,2) 
        SumGross(3)  = CONTRIB_PUBSDED_EMER_QT(pers_j,3) 
        SumGross(4)  = CONTRIB_PUBSDED_EMER_QT(pers_j,4) 
        SumGross(5)  = CONTRIB_PUBSDED_EMER_QT(pers_j,5) 
        SumGross(6)  = CONTRIB_PUBSDED_EMER_QT(pers_j,6) 
        SumGross(7)  = CONTRIB_PUBSDED_EMER_QT(pers_j,7) 
        SumGross(8)  = CONTRIB_PUBSDED_EMEE_QT(pers_j) 
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST) 
    DEALLOCATE(CONTRIB_LAB_TEST)

    !Compute aggregate statistics on contributions and deductions of public labour market wage earners with statutory service
    NumAnal = 3

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Sum of all employee and employer contributions before deductions           "
    CONTRIB_LAB_TEST(2) = "Sum of deductions on employee and employer contributions                   "
    CONTRIB_LAB_TEST(3) = "Contributions net of deductions                                            "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = SUM(CONTRIB_PUBSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_CSCOMFED_QT,1))) + &
                    & SUM(CONTRIB_PUBSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_CSCOMFED_QT,1))) + &
                    & CONTRIB_PUBSHOL_EMEE_QT(pers_j)
        SumGross(2) = SUM(CONTRIB_PUBSDED_EMER_QT(pers_j,1:7)) + &
                    & CONTRIB_PUBSDED_EMEE_QT(pers_j)
        SumGross(3) = SumGross(1) - SumGross(2) 
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST) 
    DEALLOCATE(CONTRIB_LAB_TEST)

    Write(15,'(/,''Some statistics on the contributions of self employed'')')
    CALL DRAWLINE(15,100,'-')    

    !Compute aggregate statistics on contributions and deductions of self employed
    NumAnal = 3

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Sum of all employer contributions before deductions                        "
    CONTRIB_LAB_TEST(2) = "Sum of deductions on employer contributions                                "
    CONTRIB_LAB_TEST(3) = "Contributions net of deductions                                            "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = CONTRIB_SELFCOM_EMER_QT(pers_j)
        SumGross(2) = CONTRIB_SELFDEDU_EMER_QT(pers_j)
        SumGross(3) = SumGross(1) - SumGross(2) 
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST) 
    DEALLOCATE(CONTRIB_LAB_TEST)
    
    Write(15,'(/,''Some statistics on the contributions paid on social benefits'')')
    CALL DRAWLINE(15,100,'-')    

    !Compute aggregate statistics on contributions and deductions of social benefits
    NumAnal = 4

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Common social security contributions paid on pension benefits             "
    CONTRIB_LAB_TEST(2) = "Solidarity contributions paid on pension benefits                          "
    CONTRIB_LAB_TEST(3) = "Social seucurity contributions paid on early retirement benefits           "
    CONTRIB_LAB_TEST(4) = "Social seucurity contributions paid on disability benefits                 "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = CONTRIB_SSPEBASI_QT(pers_j)
        SumGross(2) = CONTRIB_SSPESOLI_QT(pers_j)
        SumGross(3) = CONTRIB_SSEARBEN_QT(pers_j)
        SumGross(4) = CONTRIB_SSDISAB_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST) 
    DEALLOCATE(CONTRIB_LAB_TEST)

    Write(15,'(/,''Some statistics on all gross income concepts and contributions paid on it'')')
    CALL DRAWLINE(15,100,'-')    

    !Compute aggregate statistics on contributions and deductions of all different income concepts
    NumAnal = 15

    !Allocate and initialise test result
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,4))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    CONTRIB_LAB_TEST(1) = "Gross income of wage earners on the private labour market                  "
    CONTRIB_LAB_TEST(2) = "Social security contributions paid on income earned by priv. mark.         "
    CONTRIB_LAB_TEST(3) = "Gross income of wage earners on the public labour market                   "
    CONTRIB_LAB_TEST(4) = "Social security contributions paid on income earned by pub. mark.          "
    CONTRIB_LAB_TEST(5) = "Gross income earned by self employed                                       "
    CONTRIB_LAB_TEST(6) = "Social security contributions paid by self employed                        "
    CONTRIB_LAB_TEST(7) = "Gross pension benefits                                                     "
    CONTRIB_LAB_TEST(8) = "Social security contributions paid on pension benefits                     "
    CONTRIB_LAB_TEST(9) = "Gross early retirement benefits                                            "
    CONTRIB_LAB_TEST(10) = "Social security contributions paid on early retirement benefits            "
    CONTRIB_LAB_TEST(11) = "Gross amount of other unemployment benefits                                "
    CONTRIB_LAB_TEST(12) = "Gross amount of disability benefits                                        "
    CONTRIB_LAB_TEST(13) = "Social security contributions paid on disability benefits                  "
    CONTRIB_LAB_TEST(14) = "Gross amount of sickness benefits                                          "
    CONTRIB_LAB_TEST(15) = "Total amount of family allowances                                          "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
            !Remark: single holiday earnings are not included in the contribution base of blue collar workers
            !Their contributions are instead computed on 108% of their income.
            SumGross(1) = CONTRIB_INCPRIV_QT(pers_j)+CONTRIB_DHPRIV_QT(pers_j)+CONTRIB_INCSPRIV_QT(pers_j)
        ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
            SumGross(1) = CONTRIB_INCPRIV_QT(pers_j)+CONTRIB_SHPRIV_QT(pers_j)+CONTRIB_DHPRIV_QT(pers_j)+&
            &CONTRIB_INCSPRIV_QT(pers_j)
        END IF        
        SumGross(2) = CONTRIB_SSPRIV_EMEE_QT(pers_j)
        !If on the public labour market
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
            SumGross(3) = CONTRIB_INCPUB_QT(pers_j)+CONTRIB_DHPUB_QT(pers_j)+CONTRIB_INCSPUB_QT(pers_j)
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
            SumGross(3) = CONTRIB_INCPUB_QT(pers_j)+CONTRIB_SHPUB_QT(pers_j)+CONTRIB_DHPUB_QT(pers_j)+&
            &CONTRIB_INCSPUB_QT(pers_j)
        ELSE IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            SumGross(3) = CONTRIB_INCPUB_QT(pers_j)+CONTRIB_SHPUB_QT(pers_j)+CONTRIB_DHPUB_QT(pers_j)+&
            &CONTRIB_INCSPUB_QT(pers_j)
        END IF        
        SumGross(4) = CONTRIB_SSPUB_EMEE_QT(pers_j)
        SumGross(5) = CONTRIB_INCSELF_QT(pers_j)
        SumGross(6) = CONTRIB_SSSELF_EMER_QT(pers_j)
        SumGross(7) = CONTRIB_PENS_QT(pers_j)
        SumGross(8) = CONTRIB_SSPEBASI_QT(pers_j) + CONTRIB_SSPESOLI_QT(pers_j)
        SumGross(9) = CONTRIB_UNEMFT_QT(pers_j) + CONTRIB_UNEMPT_QT(pers_j)
        SumGross(10) = CONTRIB_SSEARBEN_QT(pers_j)
        SumGross(11) = UNEM_OTHERUN_QT(pers_j)            
        SumGross(12) = CONTRIB_DISABWEA_QT(pers_j)
        SumGross(13) = CONTRIB_SSDISAB_QT(pers_j)
        SumGross(14) = SICK_ALOTHBEN_QT(pers_j)
        SumGross(15) = FAMAL_AMOUNTF_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                CONTRIB_RESULT_TEST(j,1)=CONTRIB_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                CONTRIB_RESULT_TEST(j,2)=CONTRIB_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(CONTRIB_RESULT_TEST(i,2)).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=CONTRIB_RESULT_TEST(i,1)/CONTRIB_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,4)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST) 
    DEALLOCATE(CONTRIB_LAB_TEST)

    WRITE(15,'(/,''Tests to see whether there are cases with more contributions than gross income'')')
    CALL DRAWLINE(15,136,'-')    

    NumAnal = 6
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,3))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:3)=0.D0    
    
    !Define labels
    CONTRIB_LAB_TEST(1) = "Contrib. of private lab are larger than gross inc. "
    CONTRIB_LAB_TEST(2) = "Contrib. of public lab are larger than gross inc.  "
    CONTRIB_LAB_TEST(3) = "Contrib. of self empl are larger than gross inc.   "
    CONTRIB_LAB_TEST(4) = "Contrib. of pensioners are larger than gross pens. "
    CONTRIB_LAB_TEST(5) = "Contrib. on early retirements are larger than benef"
    CONTRIB_LAB_TEST(6) = "Contrib. on disability benefits are larger than ben"

    DO pers_j=1,n_rec
        SumGross(1:NumAnal*2)=0.D0
        !Compute quarterly amounts
        SumGross(1) = CONTRIB_INCPRIV_QT(pers_j) + CONTRIB_SHPRIV_QT(pers_j) + CONTRIB_DHPRIV_QT(pers_j) + &
        &CONTRIB_INCSPRIV_QT(pers_j)
        SumGross(2) = CONTRIB_SSPRIV_EMEE_QT(pers_j)
        SumGross(3) = CONTRIB_INCPUB_QT(pers_j) + CONTRIB_SHPUB_QT(pers_j) + CONTRIB_DHPUB_QT(pers_j) + &
        &CONTRIB_INCSPUB_QT(pers_j)
        SumGross(4) = CONTRIB_SSPUB_EMEE_QT(pers_j)
        SumGross(5) = CONTRIB_INCSELF_QT(pers_j)
        SumGross(6) = CONTRIB_SSSELF_EMER_QT(pers_j)
        SumGross(7) = CONTRIB_PENS_QT(pers_j)
        SumGross(8) = CONTRIB_SSPEBASI_QT(pers_j) + CONTRIB_SSPESOLI_QT(pers_j)
        SumGross(9) = CONTRIB_UNEMFT_QT(pers_j) + CONTRIB_UNEMPT_QT(pers_j)
        SumGross(10) = CONTRIB_SSEARBEN_QT(pers_j)
        SumGross(11) = CONTRIB_DISABWEA_QT(pers_j)
        SumGross(12) = CONTRIB_SSDISAB_QT(pers_j)
        IF (SumGross(2).GT.SumGross(1)) THEN
            CONTRIB_RESULT_TEST(1,1) = CONTRIB_RESULT_TEST(1,1) + 1
            CONTRIB_RESULT_TEST(1,2) = CONTRIB_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
        ELSE IF (SumGross(4).GT.SumGross(3)) THEN
            CONTRIB_RESULT_TEST(2,1) = CONTRIB_RESULT_TEST(2,1) + 1
            CONTRIB_RESULT_TEST(2,2) = CONTRIB_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
        ELSE IF (SumGross(6).GT.SumGross(5)) THEN
            CONTRIB_RESULT_TEST(3,1) = CONTRIB_RESULT_TEST(3,1) + 1
            CONTRIB_RESULT_TEST(3,2) = CONTRIB_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
        ELSE IF (SumGross(8).GT.SumGross(7)) THEN
            CONTRIB_RESULT_TEST(4,1) = CONTRIB_RESULT_TEST(4,1) + 1
            CONTRIB_RESULT_TEST(4,2) = CONTRIB_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
        ELSE IF (SumGross(10).GT.SumGross(9)) THEN
            CONTRIB_RESULT_TEST(5,1) = CONTRIB_RESULT_TEST(5,1) + 1
            CONTRIB_RESULT_TEST(5,2) = CONTRIB_RESULT_TEST(5,2) + MIMOSIS_WEIGHT(pers_j)
        ELSE IF (SumGross(12).GT.SumGross(11)) THEN
            CONTRIB_RESULT_TEST(6,1) = CONTRIB_RESULT_TEST(6,1) + 1
            CONTRIB_RESULT_TEST(6,2) = CONTRIB_RESULT_TEST(6,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                            TotSamp  TotSampW      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,3)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)
    DEALLOCATE(CONTRIB_LAB_TEST)

    WRITE(15,'(/,''Tests to see whether contributions become negative'')')
    CALL DRAWLINE(15,136,'-')    

    NumAnal = 9
    ALLOCATE(CONTRIB_RESULT_TEST(NumAnal,3))
    ALLOCATE(CONTRIB_LAB_TEST(NumAnal))
    CONTRIB_RESULT_TEST(1:NumAnal,1:3)=0.D0    
    
    !Define labels
    CONTRIB_LAB_TEST(1) = "Contrib. of employee private lab are smaller than -1"
    CONTRIB_LAB_TEST(2) = "Contrib. of employer private lab are smaller than -1"
    CONTRIB_LAB_TEST(3) = "Contrib. of employee public lab are smaller than -1"
    CONTRIB_LAB_TEST(4) = "Contrib. of employer public lab are smaller than -1 "
    CONTRIB_LAB_TEST(5) = "Contrib. of self empl are smaller than -1"
    CONTRIB_LAB_TEST(6) = "Contrib. of pensioners are smaller than -1"
    CONTRIB_LAB_TEST(7) = "Contrib. on early retirements are smaller than -1"
    CONTRIB_LAB_TEST(8) = "Contrib. on disability benefits are smaller than -1"
    CONTRIB_LAB_TEST(9) = "Contrib. on industr acc and occup dis. smaller than -1"

    DO pers_j=1,n_rec
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = CONTRIB_SSPRIV_EMEE_QT(pers_j)
        SumGross(2) = CONTRIB_SSPRIV_EMER_QT(pers_j)
        SumGross(3) = CONTRIB_SSPUB_EMEE_QT(pers_j)
        SumGross(4) = CONTRIB_SSPUB_EMER_QT(pers_j)
        SumGross(5) = CONTRIB_SSSELF_EMER_QT(pers_j)
        SumGross(6) = CONTRIB_SSPEBASI_QT(pers_j) + CONTRIB_SSPESOLI_QT(pers_j)
        SumGross(7) = CONTRIB_SSEARBEN_QT(pers_j)
        SumGross(8) = CONTRIB_SSDISAB_QT(pers_j)
        SumGross(9) = CONTRIB_SSINDOCC_QT(pers_j)
        DO i=1,Numanal
            IF (SumGross(i).LT.-1.D0) THEN
                CONTRIB_RESULT_TEST(i,1) = CONTRIB_RESULT_TEST(i,1) + 1
                CONTRIB_RESULT_TEST(i,2) = CONTRIB_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO            
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_TEST(i,3)=100.D0*CONTRIB_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                            TotSamp  TotSampW      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')CONTRIB_LAB_TEST(i),(CONTRIB_RESULT_TEST(i,j),j=1,3)
    END DO

    DEALLOCATE(CONTRIB_RESULT_TEST)
    DEALLOCATE(CONTRIB_LAB_TEST)
    
END SUBROUTINE CONTRIB_TEST

!==========================================================!
!Produce aggregate statistics that can be compared with
!statistics in other sources
!==========================================================!

SUBROUTINE CONTRIB_COMPARE

    INTEGER                                                             :: pers_j,i,j
    INTEGER                                                             :: NumAnal                              !Number of summary statistics computed 
    !variables for computation of contribution of wage earners on the private labour market
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_WEARCOM_EMEE_QT,2)+1)    :: CONTRIB_WEARCOM_EMEE_NET         !Employee contributions of wage earners on the private labour market net of deductions
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_WEARCOM_EMER_QT,2)+1)    :: CONTRIB_WEARCOM_EMER_NET         !Employer contributions of wage earners on the private labour market net of deductions
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMEE_SUMCON      !Total sum of employee contributions of wage earners on the private labour market before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMER_SUMCON      !Total sum of employer contributions of wage earners on the private labour market before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMEE_SUMDED      !Total sum of employee deductions of wage earners on the private labour market before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMER_SUMDED      !Total sum of employer deductions of wage earners on the private labour market   
    !variables for computation of contribution of wage earners on the public labour market without statutory service
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2)+1)   :: CONTRIB_PUBNSCOM_EMEE_NET        !Employee contributions of wage earners on the public labour market without statutory service net of deductions
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBNSCOM_EMER_QT,2)+1)   :: CONTRIB_PUBNSCOM_EMER_NET        !Employer contributions of wage earners on the public labour market without statutory service net of deductions
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBNSCOM_EMEE_SUMCON     !Total sum of employee contributions of wage earners on the public labour market without statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBNSCOM_EMER_SUMCON     !Total sum of employer contributions of wage earners on the public labour market without statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBNSCOM_EMEE_SUMDED     !Total sum of employee deductions of wage earners on the public labour market without statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBNSCOM_EMER_SUMDED     !Total sum of employer deductions of wage earners on the public labour market without statutory service 
    !variables for computation of contribution of wage earners on the public labour market with statutory service
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBSCOM_EMEE_QT,2)+1)    :: CONTRIB_PUBSCOM_EMEE_NET         !Employee contributions of wage earners on the public labour market with statutory service net of deductions
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBSCOM_EMER_QT,2))      :: CONTRIB_PUBSCOM_EMER_NET         !Employer contributions of wage earners on the public labour market with statutory service net of deductions
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBSCOM_EMEE_SUMCON      !Total sum of employee contributions of wage earners on the public labour market with statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBSCOM_EMER_SUMCON      !Total sum of employer contributions of wage earners on the public labour market with statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBSCOM_EMEE_SUMDED      !Total sum of employee deductions of wage earners on the public labour market with statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBSCOM_EMER_SUMDED      !Total sum of employer deductions of wage earners on the public labour market with statutory service 
    !variables for computation of contribution of self employed
    REAL(KIND=DPRECI)                                                   :: CONTRIB_SELF_EMER_NET            !Employer contributions of self employed net of deductions
    REAL(KIND=DPRECI)                                                   :: CONTRIB_SELF_EMER_SUMCON         !Total sum of employer contributions of self employed before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_SELF_EMER_SUMDED         !Total sum of employer deductions of self employed   
    !variables for computation of aggregates
    REAL(KIND=DPRECI),DIMENSION(1:3)                                    :: CONTRIB_PRIV_FRACWMOD            !Vector with fractions of wage moderation contribution of private labour market wage earners
    REAL(KIND=DPRECI),DIMENSION(1:3)                                    :: CONTRIB_PUB_FRACWMOD             !Vector with fractions of wage moderation contribution of public labour market wage earners
    REAL(KIND=DPRECI)                                                   :: SumW                             !Variable containing sum of weights    
    REAL(KIND=DPRECI)                                                   :: CONTRIB_INCCONC                  !Variable to store income concept that forms the contribution base
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)                          :: CONTRIB_AGGREGA                  !Vector with aggregates of contributions comparable with other sources computed for 1 individual
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)                        :: CONTRIB_RESULT_COMPARE           !Vector with aggregates of contributions comparable with other sources computed over all individuals
    CHARACTER(len=100),ALLOCATABLE,DIMENSION(:)                         :: CONTRIB_LAB_COMPARE              !Vector with labels of the different reconstructed aggregates                       
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)                          :: CONTRIB_EXTERN                   !Vector with aggregates taken from external sources comparable with the reconstructed concepts
    !test
    REAL(KIND=DPRECI)                                                   :: EQUIV_L                          !Equivalent loon
 
    WRITE(15,'(/,''Print statistics comparable with other sources'')')
    CALL DRAWLINE(15,136,'=')    

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))

    !Contributions net of deductions by different items
    NumAnal = 28
    ALLOCATE(CONTRIB_AGGREGA(NumAnal))       
    ALLOCATE(CONTRIB_RESULT_COMPARE(NumAnal,6))
    ALLOCATE(CONTRIB_LAB_COMPARE(NumAnal))     
    ALLOCATE(CONTRIB_EXTERN(NumAnal))        

    !Initialise results
    CONTRIB_RESULT_COMPARE(1:SIZE(CONTRIB_RESULT_COMPARE,1),1:6) = 0.D0
    CONTRIB_EXTERN(1:SIZE(CONTRIB_EXTERN,1)) = 0.D0           
    
    !Define labels
    CONTRIB_LAB_COMPARE(1) = "Werknemers: Bijdragen: Gewone bijdragen: RSZ"
    CONTRIB_LAB_COMPARE(2) = "Werknemers: Bijdragen: Gewone bijdragen: RSZPPO"
    CONTRIB_LAB_COMPARE(3) = "Werknemers: Bijdragen: Gewone bijdragen: HVKZ"
    CONTRIB_LAB_COMPARE(4) = "Werknemers: Bijdragen: Loonmatiging"
    CONTRIB_LAB_COMPARE(5) = "Werknemers: Bijdragen: Specifieke bijdragen: inhouding dubbel vakantiegeld"
    CONTRIB_LAB_COMPARE(6) = "Werknemers: Bijdragen: Specifieke bijdragen: loonmatiging dubbel vakantiegeld"
    CONTRIB_LAB_COMPARE(7) = "Werknemers: Bijdragen: Specifieke bijdragen: bijdrage jobstudenten"
    CONTRIB_LAB_COMPARE(8) = "Werknemers: Bijdragen: Specifieke bijdragen: vermindering kinderbijslag"
    CONTRIB_LAB_COMPARE(9) = "Werknemers: Bijdragen: Specifieke bijdragen: bijdrage op premies groepsverz."
    CONTRIB_LAB_COMPARE(10) = "Werknemers: Bijdragen: Specifieke bijdragen: werkgeversbijdrage tijd. werkl + ancinnit."
    CONTRIB_LAB_COMPARE(11) = "Werknemers: Bijdragen: Specifieke bijdragen: werkgeversbijdrage brugpens. + oudere werkl."
    CONTRIB_LAB_COMPARE(12) = "Werknemers: Bijdragen: Specifieke bijdragen: compenserende bijdrage werkgevers brugpens."
    CONTRIB_LAB_COMPARE(13) = "Werknemers: Bijdragen: Specifieke bijdragen: heffing op priv gebruik bedrijfswagens"
    CONTRIB_LAB_COMPARE(14) = "Werknemers: Bijdragen: Specifieke bijdragen: bijdrage deeltijdse arbeid"
    CONTRIB_LAB_COMPARE(15) = "Werknemers: Bijdragen: Specifieke bijdragen: bijdrage deelname in winst"
    CONTRIB_LAB_COMPARE(16) = "Werknemers: Bijdragen: Bijzondere fondsen: andere dan jaarlijkse vakantie en sluiting ondernem"
    CONTRIB_LAB_COMPARE(17) = "Werknemers: Bijdragen: Bijzondere fondsen: jaarlijkse vakantie (bijdragen RSZ en RSZPPO)"
    CONTRIB_LAB_COMPARE(18) = "Werknemers: Bijdragen: Bijzondere fondsen: fonds voor sluiting van ondernemingen"
    CONTRIB_LAB_COMPARE(19) = "Zelfstandigen: Bijdragen: Gewone bijdragen"
    CONTRIB_LAB_COMPARE(20) = "Zelfstandigen: Bijdragen: Consolidatiebijdrage"
    CONTRIB_LAB_COMPARE(21) = "Zelfstandigen: Bijdragen: Vennootschapsbijdrage"
    CONTRIB_LAB_COMPARE(22) = "Zelfstandigen: Bijdragen: Specifieke bijdragen"
    CONTRIB_LAB_COMPARE(23) = "Inhoudingen op rust en overlevingspensioenen"
    CONTRIB_LAB_COMPARE(24) = "Inhoudingen op uitkeringen voor arbeidsongevallen en beroepsziekten"
    CONTRIB_LAB_COMPARE(25) = "Pensioenbijdragen statutaire federale ambtenaren (overlevingspensioen)"
    CONTRIB_LAB_COMPARE(26) = "Bijdragen statutaire federale ambtenaren buiten globaal beheer (andere dan pensioenbijdragen)"
    CONTRIB_LAB_COMPARE(27) = "Pensioenbijdragen statutaire lokale ambtenaren"
    CONTRIB_LAB_COMPARE(28) = "Bijdragen statutaire lokale ambtenaren buiten globaal beheer (andere dan pensioenbijdragen)"

    !Define values taken from external sources
    CONTRIB_EXTERN(1)   = 0
    CONTRIB_EXTERN(2)   = 0
    CONTRIB_EXTERN(3)   = 0
    CONTRIB_EXTERN(4)   = 0
    CONTRIB_EXTERN(5)   = 0
    CONTRIB_EXTERN(6)   = 0
    CONTRIB_EXTERN(7)   = 0
    CONTRIB_EXTERN(8)   = 0
    CONTRIB_EXTERN(9)   = 0
    CONTRIB_EXTERN(10)  = 0
    CONTRIB_EXTERN(11)  = 0
    CONTRIB_EXTERN(12)  = 0
    CONTRIB_EXTERN(13)  = 0
    CONTRIB_EXTERN(14)  = 0
    CONTRIB_EXTERN(15)  = 0
    CONTRIB_EXTERN(16)  = 0
    CONTRIB_EXTERN(17)  = 0
    CONTRIB_EXTERN(18)  = 0
    CONTRIB_EXTERN(19)  = 0
    CONTRIB_EXTERN(20)  = 0
    CONTRIB_EXTERN(21)  = 0
    CONTRIB_EXTERN(22)  = 0
    CONTRIB_EXTERN(23)  = 0
    CONTRIB_EXTERN(24)  = 0
    CONTRIB_EXTERN(25)  = 0
    CONTRIB_EXTERN(26)  = 0
    CONTRIB_EXTERN(27)  = 0
    CONTRIB_EXTERN(28)  = 0

    !Define fractions of wage moderation contribution
    CONTRIB_PRIV_FRACWMOD(1) = 5.67
    CONTRIB_PRIV_FRACWMOD(2) = 1.41
    CONTRIB_PRIV_FRACWMOD(3) = 0.4
    CONTRIB_PUB_FRACWMOD(1) = 5.67
    CONTRIB_PUB_FRACWMOD(2) = 1.41
    CONTRIB_PUB_FRACWMOD(3) = 0.4
    
    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !Make vector with local aggregates empty
        CONTRIB_AGGREGA(1:SIZE(CONTRIB_AGGREGA,1)) = 0.D0
        !Computations for wage earners on the private labour market
        IF (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            !Make store variables empty
            CONTRIB_WEARCOM_EMEE_NET(1:SIZE(CONTRIB_WEARCOM_EMEE_NET,1)) = 0.D0
            CONTRIB_WEARCOM_EMER_NET(1:SIZE(CONTRIB_WEARCOM_EMER_NET,1)) = 0.D0
            CONTRIB_WEARCOM_EMEE_SUMCON = 0.D0
            CONTRIB_WEARCOM_EMER_SUMCON = 0.D0
            CONTRIB_WEARCOM_EMEE_SUMDED = 0.D0
            CONTRIB_WEARCOM_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_WEARCOM_EMEE_SUMCON =   SUM(CONTRIB_WEARCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_EMEE_QT,2))) + &
                                            &CONTRIB_WEARHOLD_EMEE_QT(pers_j)
            CONTRIB_WEARCOM_EMER_SUMCON =   SUM(CONTRIB_WEARCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_EMER_QT,2))) + &
                                            &CONTRIB_WEARHOLA_EMER_QT(pers_j)
            CONTRIB_WEARCOM_EMEE_SUMDED =   CONTRIB_WEARDEDU_EMEE_QT(pers_j)
            CONTRIB_WEARCOM_EMER_SUMDED =   SUM(CONTRIB_WEARDEDU_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARDEDU_EMER_QT,2)))
            !Compute the employee contributions net of deductions
            IF (INT(100.D0*CONTRIB_WEARCOM_EMEE_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_WEARCOM_EMEE_QT,2)
                    CONTRIB_WEARCOM_EMEE_NET(i) = CONTRIB_WEARCOM_EMEE_QT(pers_j,i) - &
                    &CONTRIB_WEARCOM_EMEE_SUMDED*(CONTRIB_WEARCOM_EMEE_QT(pers_j,i)/CONTRIB_WEARCOM_EMEE_SUMCON)        
                END DO
                !contributions on double holiday earnings
                CONTRIB_WEARCOM_EMEE_NET(SIZE(CONTRIB_WEARCOM_EMEE_NET,1)) = CONTRIB_WEARHOLD_EMEE_QT(pers_j) - &
                &CONTRIB_WEARCOM_EMEE_SUMDED*(CONTRIB_WEARHOLD_EMEE_QT(pers_j)/CONTRIB_WEARCOM_EMEE_SUMCON)        
            END IF
            !Compute the employer contributions net of deductions
            IF (INT(100.D0*CONTRIB_WEARCOM_EMER_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_WEARCOM_EMER_QT,2)
                    CONTRIB_WEARCOM_EMER_NET(i) = CONTRIB_WEARCOM_EMER_QT(pers_j,i) - &
                    &CONTRIB_WEARCOM_EMER_SUMDED*(CONTRIB_WEARCOM_EMER_QT(pers_j,i)/CONTRIB_WEARCOM_EMER_SUMCON)        
                END DO
                !contributions on holiday earnings of blue collar workers
                CONTRIB_WEARCOM_EMER_NET(SIZE(CONTRIB_WEARCOM_EMER_NET,1)) = CONTRIB_WEARHOLA_EMER_QT(pers_j) - &
                &CONTRIB_WEARCOM_EMER_SUMDED*(CONTRIB_WEARHOLA_EMER_QT(pers_j)/CONTRIB_WEARCOM_EMER_SUMCON)        
            END IF
            !add the contributions to the correct vector element
            CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + SUM(CONTRIB_WEARCOM_EMEE_NET(1:7)) + SUM(CONTRIB_WEARCOM_EMER_NET(1:7))
            CONTRIB_AGGREGA(10) = CONTRIB_AGGREGA(10) + CONTRIB_WEARCOM_EMEE_NET(10) + CONTRIB_WEARCOM_EMER_NET(10)
            CONTRIB_AGGREGA(17) = CONTRIB_AGGREGA(17) + CONTRIB_WEARCOM_EMEE_NET(8) + CONTRIB_WEARCOM_EMER_NET(8)
            CONTRIB_AGGREGA(16) = CONTRIB_AGGREGA(16) + SUM(CONTRIB_WEARCOM_EMEE_NET(9:11)) + SUM(CONTRIB_WEARCOM_EMER_NET(9:11))
            !Remark: attribute fractions of the wage moderation contribution to the correct line
            CONTRIB_AGGREGA(4) =    CONTRIB_AGGREGA(4) + (CONTRIB_PRIV_FRACWMOD(1)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                    &(CONTRIB_WEARCOM_EMEE_NET(13) + CONTRIB_WEARCOM_EMER_NET(13))
            CONTRIB_AGGREGA(1) =    CONTRIB_AGGREGA(1) + (CONTRIB_PRIV_FRACWMOD(2)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                    &(CONTRIB_WEARCOM_EMEE_NET(13) + CONTRIB_WEARCOM_EMER_NET(13))
            CONTRIB_AGGREGA(6) =    CONTRIB_AGGREGA(6) + (CONTRIB_PRIV_FRACWMOD(3)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                    &(CONTRIB_WEARCOM_EMEE_NET(13) + CONTRIB_WEARCOM_EMER_NET(13))
            CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + CONTRIB_WEARCOM_EMEE_NET(14) + CONTRIB_WEARCOM_EMER_NET(14)
            CONTRIB_AGGREGA(4) = CONTRIB_AGGREGA(4) + CONTRIB_WEARCOM_EMEE_NET(15) + CONTRIB_WEARCOM_EMER_NET(15)
            CONTRIB_AGGREGA(18) = CONTRIB_AGGREGA(18) + SUM(CONTRIB_WEARCOM_EMEE_NET(16:21)) + SUM(CONTRIB_WEARCOM_EMER_NET(16:21))
            !annual vacation blue collars
            CONTRIB_AGGREGA(17) = CONTRIB_AGGREGA(17) + CONTRIB_WEARCOM_EMER_NET(22)
            !double holiday of employee
            CONTRIB_AGGREGA(5) = CONTRIB_AGGREGA(5) + CONTRIB_WEARCOM_EMEE_NET(22)
            !temporary unemployment and older unemployed
            CONTRIB_AGGREGA(11) = CONTRIB_AGGREGA(11) + CONTRIB_WEARCOM_EMEE_NET(12) + CONTRIB_WEARCOM_EMER_NET(12)
        END IF
        !Computations for wage earners on the public labour market without statutory service
        IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            !Make store variables empty
            CONTRIB_PUBNSCOM_EMEE_NET(1:SIZE(CONTRIB_PUBNSCOM_EMEE_NET,1)) = 0.D0
            CONTRIB_PUBNSCOM_EMER_NET(1:SIZE(CONTRIB_PUBNSCOM_EMER_NET,1)) = 0.D0
            CONTRIB_PUBNSCOM_EMEE_SUMCON = 0.D0
            CONTRIB_PUBNSCOM_EMER_SUMCON = 0.D0
            CONTRIB_PUBNSCOM_EMEE_SUMDED = 0.D0
            CONTRIB_PUBNSCOM_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_PUBNSCOM_EMEE_SUMCON =   SUM(CONTRIB_PUBNSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2))) + &
                                            &CONTRIB_PUBNSHOL_EMEE_QT(pers_j)
            CONTRIB_PUBNSCOM_EMER_SUMCON =   SUM(CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBNSCOM_EMER_QT,2))) + &
                                            &CONTRIB_PUBNSHOL_EMER_QT(pers_j)
            CONTRIB_PUBNSCOM_EMEE_SUMDED =   CONTRIB_PUBNSDED_EMEE_QT(pers_j)
            CONTRIB_PUBNSCOM_EMER_SUMDED =   SUM(CONTRIB_PUBNSDED_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBNSDED_EMER_QT,2)))
            !Compute the employee contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBNSCOM_EMEE_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2)
                    CONTRIB_PUBNSCOM_EMEE_NET(i) = CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i) - &
                    &CONTRIB_PUBNSCOM_EMEE_SUMDED*(CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i)/CONTRIB_PUBNSCOM_EMEE_SUMCON)        
                END DO
                !contributions on double holiday earnings
                CONTRIB_PUBNSCOM_EMEE_NET(SIZE(CONTRIB_PUBNSCOM_EMEE_NET,1)) = CONTRIB_PUBNSHOL_EMEE_QT(pers_j) - &
                &CONTRIB_PUBNSCOM_EMEE_SUMDED*(CONTRIB_PUBNSHOL_EMEE_QT(pers_j)/CONTRIB_PUBNSCOM_EMEE_SUMCON)        
            END IF
            !Compute the employer contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBNSCOM_EMER_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBNSCOM_EMER_QT,2)
                    CONTRIB_PUBNSCOM_EMER_NET(i) = CONTRIB_PUBNSCOM_EMER_QT(pers_j,i) - &
                    &CONTRIB_PUBNSCOM_EMER_SUMDED*(CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)/CONTRIB_PUBNSCOM_EMER_SUMCON)        
                END DO
                !contributions on holiday earnings of blue collar workers
                CONTRIB_PUBNSCOM_EMER_NET(SIZE(CONTRIB_PUBNSCOM_EMER_NET,1)) = CONTRIB_PUBNSHOL_EMER_QT(pers_j) - &
                &CONTRIB_PUBNSCOM_EMER_SUMDED*(CONTRIB_PUBNSHOL_EMER_QT(pers_j)/CONTRIB_PUBNSCOM_EMER_SUMCON)        
            END IF
            !add the contributions to the correct vector element
            !Distinguish local from federal civil servants
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + SUM(CONTRIB_PUBNSCOM_EMEE_NET(1:7)) + SUM(CONTRIB_PUBNSCOM_EMER_NET(1:7))
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + SUM(CONTRIB_PUBNSCOM_EMEE_NET(1:7)) + SUM(CONTRIB_PUBNSCOM_EMER_NET(1:7))
            END IF                
            CONTRIB_AGGREGA(10) = CONTRIB_AGGREGA(10) + CONTRIB_PUBNSCOM_EMEE_NET(10) + CONTRIB_PUBNSCOM_EMER_NET(10)
            CONTRIB_AGGREGA(17) = CONTRIB_AGGREGA(17) + CONTRIB_PUBNSCOM_EMEE_NET(8) + CONTRIB_PUBNSCOM_EMER_NET(8)
            CONTRIB_AGGREGA(16) = CONTRIB_AGGREGA(16) + SUM(CONTRIB_PUBNSCOM_EMEE_NET(9:11)) + SUM(CONTRIB_PUBNSCOM_EMER_NET(9:11))
            !Remark: attribute fractions of the wage moderation contribution to the correct line
            CONTRIB_AGGREGA(4) =    CONTRIB_AGGREGA(4) + (CONTRIB_PRIV_FRACWMOD(1)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                    &(CONTRIB_PUBNSCOM_EMEE_NET(13) + CONTRIB_PUBNSCOM_EMER_NET(13))
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) =    CONTRIB_AGGREGA(1) + (CONTRIB_PRIV_FRACWMOD(2)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                        &(CONTRIB_PUBNSCOM_EMEE_NET(13) + CONTRIB_PUBNSCOM_EMER_NET(13))
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                CONTRIB_AGGREGA(1) =    CONTRIB_AGGREGA(1) + (CONTRIB_PRIV_FRACWMOD(2)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                        &(CONTRIB_PUBNSCOM_EMEE_NET(13) + CONTRIB_PUBNSCOM_EMER_NET(13))
            END IF
            CONTRIB_AGGREGA(6) =    CONTRIB_AGGREGA(6) + (CONTRIB_PRIV_FRACWMOD(3)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                    &(CONTRIB_PUBNSCOM_EMEE_NET(13) + CONTRIB_PUBNSCOM_EMER_NET(13))
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + CONTRIB_PUBNSCOM_EMEE_NET(14) + CONTRIB_PUBNSCOM_EMER_NET(14)
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + CONTRIB_PUBNSCOM_EMEE_NET(14) + CONTRIB_PUBNSCOM_EMER_NET(14)
            END IF                
            CONTRIB_AGGREGA(4) = CONTRIB_AGGREGA(4) + CONTRIB_PUBNSCOM_EMEE_NET(15) + CONTRIB_PUBNSCOM_EMER_NET(15)
            CONTRIB_AGGREGA(18) = CONTRIB_AGGREGA(18) +  SUM(CONTRIB_PUBNSCOM_EMEE_NET(16:21)) + &
                                                        &SUM(CONTRIB_PUBNSCOM_EMER_NET(16:21))
            !annual vacation blue collars
            CONTRIB_AGGREGA(17) = CONTRIB_AGGREGA(17) + CONTRIB_PUBNSCOM_EMER_NET(22)
            !double holiday of employee
            CONTRIB_AGGREGA(5) = CONTRIB_AGGREGA(5) + CONTRIB_PUBNSCOM_EMEE_NET(22)
            !temporary unemployment and older unemployed
            CONTRIB_AGGREGA(11) = CONTRIB_AGGREGA(11) + CONTRIB_PUBNSCOM_EMEE_NET(12) + CONTRIB_PUBNSCOM_EMER_NET(12)
        END IF
        !Computations for wage earners on the public labour market with statutory service
        IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            !Make store variables empty
            CONTRIB_PUBSCOM_EMEE_NET(1:SIZE(CONTRIB_PUBSCOM_EMEE_NET,1)) = 0.D0
            CONTRIB_PUBSCOM_EMER_NET(1:SIZE(CONTRIB_PUBSCOM_EMER_NET,1)) = 0.D0
            CONTRIB_PUBSCOM_EMEE_SUMCON = 0.D0
            CONTRIB_PUBSCOM_EMER_SUMCON = 0.D0
            CONTRIB_PUBSCOM_EMEE_SUMDED = 0.D0
            CONTRIB_PUBSCOM_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_PUBSCOM_EMEE_SUMCON =   SUM(CONTRIB_PUBSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_PUBSCOM_EMEE_QT,2))) + &
                                            &CONTRIB_PUBSHOL_EMEE_QT(pers_j)
            CONTRIB_PUBSCOM_EMER_SUMCON =   SUM(CONTRIB_PUBSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBSCOM_EMER_QT,2))) 
            CONTRIB_PUBSCOM_EMEE_SUMDED =   CONTRIB_PUBSDED_EMEE_QT(pers_j)
            CONTRIB_PUBSCOM_EMER_SUMDED =   SUM(CONTRIB_PUBSDED_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBSDED_EMER_QT,2)))
            !Compute the employee contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBSCOM_EMEE_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBSCOM_EMEE_QT,2)
                    CONTRIB_PUBSCOM_EMEE_NET(i) = CONTRIB_PUBSCOM_EMEE_QT(pers_j,i) - &
                    &CONTRIB_PUBSCOM_EMEE_SUMDED*(CONTRIB_PUBSCOM_EMEE_QT(pers_j,i)/CONTRIB_PUBSCOM_EMEE_SUMCON)        
                END DO
                !contributions on double holiday earnings
                CONTRIB_PUBSCOM_EMEE_NET(SIZE(CONTRIB_PUBSCOM_EMEE_NET,1)) = CONTRIB_PUBSHOL_EMEE_QT(pers_j) - &
                &CONTRIB_PUBSCOM_EMEE_SUMDED*(CONTRIB_PUBSHOL_EMEE_QT(pers_j)/CONTRIB_PUBSCOM_EMEE_SUMCON)        
            END IF
            !Compute the employer contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBSCOM_EMER_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBSCOM_EMER_QT,2)
                    CONTRIB_PUBSCOM_EMER_NET(i) = CONTRIB_PUBSCOM_EMER_QT(pers_j,i) - &
                    &CONTRIB_PUBSCOM_EMER_SUMDED*(CONTRIB_PUBSCOM_EMER_QT(pers_j,i)/CONTRIB_PUBSCOM_EMER_SUMCON)        
                END DO
            END IF
            !add the contributions to the correct vector element
            !Distinguish local from federal civil servants
            !1 = local
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + CONTRIB_PUBSCOM_EMEE_NET(2) + CONTRIB_PUBSCOM_EMER_NET(2)
                CONTRIB_AGGREGA(25) = CONTRIB_AGGREGA(25) + CONTRIB_PUBSCOM_EMEE_NET(1) + CONTRIB_PUBSCOM_EMER_NET(1)
                CONTRIB_AGGREGA(26) = CONTRIB_AGGREGA(26) + SUM(CONTRIB_PUBSCOM_EMEE_NET(3:4)) + &
                &SUM(CONTRIB_PUBSCOM_EMER_NET(3:4)) + CONTRIB_PUBSCOM_EMEE_NET(6) + CONTRIB_PUBSCOM_EMER_NET(6)
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                CONTRIB_AGGREGA(2) = CONTRIB_AGGREGA(2) + CONTRIB_PUBSCOM_EMEE_NET(2) + CONTRIB_PUBSCOM_EMER_NET(2)
                CONTRIB_AGGREGA(27) = CONTRIB_AGGREGA(27) + CONTRIB_PUBSCOM_EMEE_NET(1) + CONTRIB_PUBSCOM_EMER_NET(1)
                CONTRIB_AGGREGA(28) = CONTRIB_AGGREGA(28) + SUM(CONTRIB_PUBSCOM_EMEE_NET(3:4)) + &
                &SUM(CONTRIB_PUBSCOM_EMER_NET(3:4)) + CONTRIB_PUBSCOM_EMEE_NET(6) + CONTRIB_PUBSCOM_EMER_NET(6)
            END IF  
            !Remark: attribute fractions of the wage moderation contribution to the correct line
            CONTRIB_AGGREGA(4) =    CONTRIB_AGGREGA(4) + (CONTRIB_PUB_FRACWMOD(1)/SUM(CONTRIB_PUB_FRACWMOD(1:3)))*&
                                    &(CONTRIB_PUBSCOM_EMEE_NET(5) + CONTRIB_PUBSCOM_EMER_NET(5))
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) =    CONTRIB_AGGREGA(1) + (CONTRIB_PUB_FRACWMOD(2)/SUM(CONTRIB_PUB_FRACWMOD(1:3)))*&
                                        &(CONTRIB_PUBSCOM_EMEE_NET(5) + CONTRIB_PUBSCOM_EMER_NET(5))
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                CONTRIB_AGGREGA(2) =    CONTRIB_AGGREGA(2) + (CONTRIB_PUB_FRACWMOD(2)/SUM(CONTRIB_PUB_FRACWMOD(1:3)))*&
                                        &(CONTRIB_PUBSCOM_EMEE_NET(5) + CONTRIB_PUBSCOM_EMER_NET(5))
            END IF
            CONTRIB_AGGREGA(6) =    CONTRIB_AGGREGA(6) + (CONTRIB_PUB_FRACWMOD(3)/SUM(CONTRIB_PUB_FRACWMOD(1:3)))*&
                                    &(CONTRIB_PUBSCOM_EMEE_NET(5) + CONTRIB_PUBSCOM_EMER_NET(5))
            !double holiday of employee
            CONTRIB_AGGREGA(5) = CONTRIB_AGGREGA(5) + CONTRIB_PUBSCOM_EMEE_NET(7)
        END IF
        !Computations for self employed
        IF (ANY(CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.(/1/))) THEN
            !Make store variables empty
            CONTRIB_SELF_EMER_NET = 0.D0
            CONTRIB_SELF_EMER_SUMCON = 0.D0
            CONTRIB_SELF_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_SELF_EMER_SUMCON = CONTRIB_SELFCOM_EMER_QT(pers_j)
            CONTRIB_SELF_EMER_SUMDED = CONTRIB_SELFDEDU_EMER_QT(pers_j)
            !Compute the employer contributions net of deductions
            CONTRIB_SELF_EMER_NET = CONTRIB_SELF_EMER_SUMCON - CONTRIB_SELF_EMER_SUMDED
            !add the contributions to the correct vector element
            CONTRIB_AGGREGA(19) = CONTRIB_AGGREGA(19) + CONTRIB_SELF_EMER_NET
        END IF
        !Add contributions paid on social benefits
        CONTRIB_AGGREGA(11) = CONTRIB_AGGREGA(11) + CONTRIB_SSDISAB_QT(pers_j) + CONTRIB_SSEARBEN_QT(pers_j)
        CONTRIB_AGGREGA(23) = CONTRIB_AGGREGA(23) + CONTRIB_SSPEBASI_QT(pers_j) + CONTRIB_SSPESOLI_QT(pers_j)
        CONTRIB_AGGREGA(24) = CONTRIB_AGGREGA(24) + CONTRIB_SSINDOCC_QT(pers_j)
        !Add all the individual results to a vector that aggregates results over all individuals
        DO j=1,SIZE(CONTRIB_AGGREGA,1)
            IF (INT(100.D0*CONTRIB_AGGREGA(j)).NE.0) THEN
                CONTRIB_RESULT_COMPARE(j,1)=CONTRIB_RESULT_COMPARE(j,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_AGGREGA(j)
                CONTRIB_RESULT_COMPARE(j,2)=CONTRIB_RESULT_COMPARE(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    !Inflate contributions to annual values
    !Before factor 3.60 based on SummaryOfAggregates.xls (salary mass RSZ and RSZPPO in fourth quarter as fraction of annual salary mass in sample)
    !Now 4.0
    !Use different factor for contributions paid on wages and other contributions
    CONTRIB_RESULT_COMPARE(1:18,1)=(CONTRIB_RESULT_COMPARE(1:18,1)*4.D0)/1000.D0
    CONTRIB_RESULT_COMPARE(19:28,1)=(CONTRIB_RESULT_COMPARE(19:28,1)*4.D0)/1000.D0

    !Compute columns 3 to 6 of the vector with comparable aggregates
    DO i=1,SIZE(CONTRIB_RESULT_COMPARE,1)
        !difference between real value and estimate
        CONTRIB_RESULT_COMPARE(i,3)= CONTRIB_RESULT_COMPARE(i,1) - CONTRIB_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(CONTRIB_EXTERN(i)).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,4)= 100.D0*(CONTRIB_RESULT_COMPARE(i,3)/CONTRIB_EXTERN(i))    
        END IF
        !Average amount
        IF (INT(CONTRIB_RESULT_COMPARE(i,2)).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,5)=CONTRIB_RESULT_COMPARE(i,1)/CONTRIB_RESULT_COMPARE(i,2)
        END IF
        !Fraction of population with non zero value
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,6)=100.D0*CONTRIB_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''Bijdragen na aftrek van bijdragenverminderingen op jaarbasis naar bijdragepost'')')
    Write(15,'(''Deze tabel bevat in kolom steekproef alle bijdragen die gesimuleerd werden met de CONTRIB module'')')
    Write(15,'(''Nominale bedragen per 1000 Euro in prijzen van 2008'')')
    Write(15,'(''                                                                                                         &
    &   Externe bron          Steekproef   Aantal individuen  Absoluut verschil  %Verschil     Gemid      %Pop'')')
    Write(15,'(''                                                                                                         &
    &             C1                  C2                                  C2-C1  100*C2/C1 per indiv'')')
    CALL DRAWLINE(15,210,'-')    
    DO i=1,SIZE(CONTRIB_RESULT_COMPARE,1)
        Write(15,'(a100,F20.2,F20.2,F20.0,F20.0,F10.2,F10.2,F10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),&
        &(CONTRIB_RESULT_COMPARE(i,j),j=1,6)
    END DO
    Write(15,'(100x,F20.2,F20.2)')SUM(CONTRIB_EXTERN(1:SIZE(CONTRIB_EXTERN,1))),&
    &SUM(CONTRIB_RESULT_COMPARE(1:SIZE(CONTRIB_RESULT_COMPARE,1),1))

    DEALLOCATE(CONTRIB_AGGREGA)      
    DEALLOCATE(CONTRIB_RESULT_COMPARE)
    DEALLOCATE(CONTRIB_LAB_COMPARE)     
    DEALLOCATE(CONTRIB_EXTERN)        

    !Contributions net of deductions as published by the RSZ
    NumAnal = 19
    ALLOCATE(CONTRIB_AGGREGA(NumAnal))       
    ALLOCATE(CONTRIB_RESULT_COMPARE(NumAnal,6))
    ALLOCATE(CONTRIB_LAB_COMPARE(NumAnal))     
    ALLOCATE(CONTRIB_EXTERN(NumAnal))        

    !Initialise results
    CONTRIB_RESULT_COMPARE(1:SIZE(CONTRIB_RESULT_COMPARE,1),1:6) = 0.D0
    CONTRIB_EXTERN(1:SIZE(CONTRIB_EXTERN,1)) = 0.D0           
    
    !Define labels
    CONTRIB_LAB_COMPARE(1) = "Bijdragen ten laste van de werknemer: SZ werknemersbijdragen"
    CONTRIB_LAB_COMPARE(2) = "Bijdragen ten laste van de werknemer: inhouding op het dubbel vakantiegeld"
    CONTRIB_LAB_COMPARE(3) = "Bijdragen ten laste van de werknemer: bijzondere bijdrage v/d sociale zekerheid"
    CONTRIB_LAB_COMPARE(4) = "Bijdragen ten laste van de werknemer: solidariteitsbijdrage voor niet verz. studenten"
    CONTRIB_LAB_COMPARE(5) = "Bijdragen ten laste van de werkgever: SZ kwartaalwerkgeversbijdrage"
    CONTRIB_LAB_COMPARE(6) = "Bijdragen ten laste van de werkgever: solidariteitsbijdrage voor niet verz. studenten"
    CONTRIB_LAB_COMPARE(7) = "Bijdragen ten laste van de werkgever: kwartaal en jaarlijkse bijdrage voor jaarlijkse vakantie"
    CONTRIB_LAB_COMPARE(8) = "Bijdragen ten laste van de werkgever: Fondsen voor bestaanszekerheid"
    CONTRIB_LAB_COMPARE(9) = "Bijdragen ten laste van de werkgever: Betaald educatief verlof"
    CONTRIB_LAB_COMPARE(10) = "Bijdragen ten laste van de werkgever: Fonds voor de sluiting van ondernemingen"
    CONTRIB_LAB_COMPARE(11) = "Bijdragen ten laste van de werkgever: Werknemers met brugpensioen"
    CONTRIB_LAB_COMPARE(12) = "Bijdragen ten laste van de werkgever: Extra legale pensioenen"
    CONTRIB_LAB_COMPARE(13) = "Bijdragen ten laste van de werkgever: Risicogroepen"
    CONTRIB_LAB_COMPARE(14) = "Bijdragen ten laste van de werkgever: Jongeren met een inschakenlingsparcours"
    CONTRIB_LAB_COMPARE(15) = "Bijdragen ten laste van de werkgever: Kinderopvang"
    CONTRIB_LAB_COMPARE(16) = "Bijdragen ten laste van de werkgever: Tijdelijke werkloosheid en oudere werklozen"
    CONTRIB_LAB_COMPARE(17) = "Bijdragen ten laste van de werkgever: Priv gebruik van bedrijfsvoertuigen"
    CONTRIB_LAB_COMPARE(18) = "Bijdragen ten laste van de werkgever: Economische werkloosheid"
    CONTRIB_LAB_COMPARE(19) = "Heverdeling van de sociale lasten (2de kwartaal)"

    !Define values taken from external sources
    CONTRIB_EXTERN(1)   = 0        
    CONTRIB_EXTERN(2)   = 0        
    CONTRIB_EXTERN(3)   = 0        
    CONTRIB_EXTERN(4)   = 0        
    CONTRIB_EXTERN(5)   = 0        
    CONTRIB_EXTERN(6)   = 0        
    CONTRIB_EXTERN(7)   = 0        
    CONTRIB_EXTERN(8)   = 0        
    CONTRIB_EXTERN(9)   = 0        
    CONTRIB_EXTERN(10)  = 0        
    CONTRIB_EXTERN(11)  = 0        
    CONTRIB_EXTERN(12)  = 0        
    CONTRIB_EXTERN(13)  = 0        
    CONTRIB_EXTERN(14)  = 0        
    CONTRIB_EXTERN(15)  = 0        
    CONTRIB_EXTERN(16)  = 0        
    CONTRIB_EXTERN(17)  = 0        
    CONTRIB_EXTERN(18)  = 0        
    CONTRIB_EXTERN(19)  = 0        

    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !Make vector with local aggregates empty
        CONTRIB_AGGREGA(1:SIZE(CONTRIB_AGGREGA,1)) = 0.D0
        !Computations for wage earners on the private labour market
        IF (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            !Make store variables empty
            CONTRIB_WEARCOM_EMEE_NET(1:SIZE(CONTRIB_WEARCOM_EMEE_NET,1)) = 0.D0
            CONTRIB_WEARCOM_EMER_NET(1:SIZE(CONTRIB_WEARCOM_EMER_NET,1)) = 0.D0
            CONTRIB_WEARCOM_EMEE_SUMCON = 0.D0
            CONTRIB_WEARCOM_EMER_SUMCON = 0.D0
            CONTRIB_WEARCOM_EMEE_SUMDED = 0.D0
            CONTRIB_WEARCOM_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_WEARCOM_EMEE_SUMCON =   SUM(CONTRIB_WEARCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_EMEE_QT,2))) + &
                                            &CONTRIB_WEARHOLD_EMEE_QT(pers_j)
            CONTRIB_WEARCOM_EMER_SUMCON =   SUM(CONTRIB_WEARCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_EMER_QT,2))) + &
                                            &CONTRIB_WEARHOLA_EMER_QT(pers_j)
            CONTRIB_WEARCOM_EMEE_SUMDED =   CONTRIB_WEARDEDU_EMEE_QT(pers_j)
            CONTRIB_WEARCOM_EMER_SUMDED =   SUM(CONTRIB_WEARDEDU_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARDEDU_EMER_QT,2)))
            !Compute the employee contributions net of deductions
            IF (INT(100.D0*CONTRIB_WEARCOM_EMEE_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_WEARCOM_EMEE_QT,2)
                    CONTRIB_WEARCOM_EMEE_NET(i) = CONTRIB_WEARCOM_EMEE_QT(pers_j,i) - &
                    &CONTRIB_WEARCOM_EMEE_SUMDED*(CONTRIB_WEARCOM_EMEE_QT(pers_j,i)/CONTRIB_WEARCOM_EMEE_SUMCON)        
                END DO
                !contributions on double holiday earnings
                CONTRIB_WEARCOM_EMEE_NET(SIZE(CONTRIB_WEARCOM_EMEE_NET,1)) = CONTRIB_WEARHOLD_EMEE_QT(pers_j) - &
                &CONTRIB_WEARCOM_EMEE_SUMDED*(CONTRIB_WEARHOLD_EMEE_QT(pers_j)/CONTRIB_WEARCOM_EMEE_SUMCON)        
            END IF
            !Compute the employer contributions net of deductions
            IF (INT(100.D0*CONTRIB_WEARCOM_EMER_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_WEARCOM_EMER_QT,2)
                    CONTRIB_WEARCOM_EMER_NET(i) = CONTRIB_WEARCOM_EMER_QT(pers_j,i) - &
                    &CONTRIB_WEARCOM_EMER_SUMDED*(CONTRIB_WEARCOM_EMER_QT(pers_j,i)/CONTRIB_WEARCOM_EMER_SUMCON)        
                END DO
                !contributions on holiday earnings of blue collar workers
                CONTRIB_WEARCOM_EMER_NET(SIZE(CONTRIB_WEARCOM_EMER_NET,1)) = CONTRIB_WEARHOLA_EMER_QT(pers_j) - &
                &CONTRIB_WEARCOM_EMER_SUMDED*(CONTRIB_WEARHOLA_EMER_QT(pers_j)/CONTRIB_WEARCOM_EMER_SUMCON)        
            END IF
            !add the contributions to the correct vector element
            CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + SUM(CONTRIB_WEARCOM_EMEE_NET(1:7)) 
            CONTRIB_AGGREGA(2) = CONTRIB_AGGREGA(2) + CONTRIB_WEARCOM_EMEE_NET(SIZE(CONTRIB_WEARCOM_EMEE_NET,1))
            CONTRIB_AGGREGA(3) = 0
            CONTRIB_AGGREGA(4) = 0
            CONTRIB_AGGREGA(5) = CONTRIB_AGGREGA(5) + SUM(CONTRIB_WEARCOM_EMER_NET(1:7)) + CONTRIB_WEARCOM_EMER_NET(13) &
                                                      &+ SUM(CONTRIB_WEARCOM_EMER_NET(14:15))
            CONTRIB_AGGREGA(6) = 0
            CONTRIB_AGGREGA(7) = CONTRIB_AGGREGA(7) +    CONTRIB_WEARCOM_EMER_NET(8) + &
                                                        &CONTRIB_WEARCOM_EMER_NET(SIZE(CONTRIB_WEARCOM_EMER_NET,1))
            CONTRIB_AGGREGA(8) = 0
            CONTRIB_AGGREGA(9) = CONTRIB_AGGREGA(9) + CONTRIB_WEARCOM_EMER_NET(9) 
            CONTRIB_AGGREGA(10) = CONTRIB_AGGREGA(10) + SUM(CONTRIB_WEARCOM_EMER_NET(16:21)) 
            CONTRIB_AGGREGA(12) = 0
            CONTRIB_AGGREGA(13) = 0
            CONTRIB_AGGREGA(14) = CONTRIB_AGGREGA(14) + CONTRIB_WEARCOM_EMER_NET(10) 
            CONTRIB_AGGREGA(15) = CONTRIB_AGGREGA(15) + CONTRIB_WEARCOM_EMER_NET(11) 
            CONTRIB_AGGREGA(16) = CONTRIB_AGGREGA(16) + CONTRIB_WEARCOM_EMER_NET(12) 
            CONTRIB_AGGREGA(17) = 0
            CONTRIB_AGGREGA(18) = 0
            CONTRIB_AGGREGA(19) = 0
        END IF
        !Computations for wage earners on the public labour market without statutory service
        IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            !Make store variables empty
            CONTRIB_PUBNSCOM_EMEE_NET(1:SIZE(CONTRIB_PUBNSCOM_EMEE_NET,1)) = 0.D0
            CONTRIB_PUBNSCOM_EMER_NET(1:SIZE(CONTRIB_PUBNSCOM_EMER_NET,1)) = 0.D0
            CONTRIB_PUBNSCOM_EMEE_SUMCON = 0.D0
            CONTRIB_PUBNSCOM_EMER_SUMCON = 0.D0
            CONTRIB_PUBNSCOM_EMEE_SUMDED = 0.D0
            CONTRIB_PUBNSCOM_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_PUBNSCOM_EMEE_SUMCON =   SUM(CONTRIB_PUBNSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2))) + &
                                            &CONTRIB_PUBNSHOL_EMEE_QT(pers_j)
            CONTRIB_PUBNSCOM_EMER_SUMCON =   SUM(CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBNSCOM_EMER_QT,2))) + &
                                            &CONTRIB_PUBNSHOL_EMER_QT(pers_j)
            CONTRIB_PUBNSCOM_EMEE_SUMDED =   CONTRIB_PUBNSDED_EMEE_QT(pers_j)
            CONTRIB_PUBNSCOM_EMER_SUMDED =   SUM(CONTRIB_PUBNSDED_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBNSDED_EMER_QT,2)))
            !Compute the employee contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBNSCOM_EMEE_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2)
                    CONTRIB_PUBNSCOM_EMEE_NET(i) = CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i) - &
                    &CONTRIB_PUBNSCOM_EMEE_SUMDED*(CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i)/CONTRIB_PUBNSCOM_EMEE_SUMCON)        
                END DO
                !contributions on double holiday earnings
                CONTRIB_PUBNSCOM_EMEE_NET(SIZE(CONTRIB_PUBNSCOM_EMEE_NET,1)) = CONTRIB_PUBNSHOL_EMEE_QT(pers_j) - &
                &CONTRIB_PUBNSCOM_EMEE_SUMDED*(CONTRIB_PUBNSHOL_EMEE_QT(pers_j)/CONTRIB_PUBNSCOM_EMEE_SUMCON)        
            END IF
            !Compute the employer contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBNSCOM_EMER_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBNSCOM_EMER_QT,2)
                    CONTRIB_PUBNSCOM_EMER_NET(i) = CONTRIB_PUBNSCOM_EMER_QT(pers_j,i) - &
                    &CONTRIB_PUBNSCOM_EMER_SUMDED*(CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)/CONTRIB_PUBNSCOM_EMER_SUMCON)        
                END DO
                !contributions on holiday earnings of blue collar workers
                CONTRIB_PUBNSCOM_EMER_NET(SIZE(CONTRIB_PUBNSCOM_EMER_NET,1)) = CONTRIB_PUBNSHOL_EMER_QT(pers_j) - &
                &CONTRIB_PUBNSCOM_EMER_SUMDED*(CONTRIB_PUBNSHOL_EMER_QT(pers_j)/CONTRIB_PUBNSCOM_EMER_SUMCON)        
            END IF
            !add the contributions to the correct vector element
            !Distinguish local from federal civil servants (see mail Larmuseau 24 november 2006)
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + SUM(CONTRIB_PUBNSCOM_EMEE_NET(1:7))
                CONTRIB_AGGREGA(2) = CONTRIB_AGGREGA(2) + CONTRIB_PUBNSCOM_EMEE_NET(SIZE(CONTRIB_PUBNSCOM_EMEE_NET,1))
                CONTRIB_AGGREGA(3) = 0
                CONTRIB_AGGREGA(4) = 0
                CONTRIB_AGGREGA(5) = CONTRIB_AGGREGA(5) + SUM(CONTRIB_PUBNSCOM_EMER_NET(1:7)) + CONTRIB_PUBNSCOM_EMER_NET(13) + &
                                                          &SUM(CONTRIB_PUBNSCOM_EMER_NET(14:15)) 
                CONTRIB_AGGREGA(6) = 0
                CONTRIB_AGGREGA(7) = CONTRIB_AGGREGA(7) +    CONTRIB_PUBNSCOM_EMER_NET(8) + &
                                                            &CONTRIB_PUBNSCOM_EMER_NET(SIZE(CONTRIB_PUBNSCOM_EMER_NET,1))
                CONTRIB_AGGREGA(8) = 0
                CONTRIB_AGGREGA(9) = CONTRIB_AGGREGA(9) + CONTRIB_PUBNSCOM_EMER_NET(9) 
                CONTRIB_AGGREGA(10) = CONTRIB_AGGREGA(10) + SUM(CONTRIB_PUBNSCOM_EMER_NET(16:21)) 
                CONTRIB_AGGREGA(12) = 0
                CONTRIB_AGGREGA(13) = 0
                CONTRIB_AGGREGA(14) = CONTRIB_AGGREGA(14) + CONTRIB_PUBNSCOM_EMER_NET(10) 
                CONTRIB_AGGREGA(15) = CONTRIB_AGGREGA(15) + CONTRIB_PUBNSCOM_EMER_NET(11) 
                CONTRIB_AGGREGA(16) = CONTRIB_AGGREGA(16) + CONTRIB_PUBNSCOM_EMER_NET(12) 
                CONTRIB_AGGREGA(17) = 0
                CONTRIB_AGGREGA(18) = 0
                CONTRIB_AGGREGA(19) = 0
            END IF
        END IF
        !Computations for wage earners on the public labour market with statutory service
        IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            !Make store variables empty
            CONTRIB_PUBSCOM_EMEE_NET(1:SIZE(CONTRIB_PUBSCOM_EMEE_NET,1)) = 0.D0
            CONTRIB_PUBSCOM_EMER_NET(1:SIZE(CONTRIB_PUBSCOM_EMER_NET,1)) = 0.D0
            CONTRIB_PUBSCOM_EMEE_SUMCON = 0.D0
            CONTRIB_PUBSCOM_EMER_SUMCON = 0.D0
            CONTRIB_PUBSCOM_EMEE_SUMDED = 0.D0
            CONTRIB_PUBSCOM_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_PUBSCOM_EMEE_SUMCON =   SUM(CONTRIB_PUBSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_PUBSCOM_EMEE_QT,2))) + &
                                            &CONTRIB_PUBSHOL_EMEE_QT(pers_j)
            CONTRIB_PUBSCOM_EMER_SUMCON =   SUM(CONTRIB_PUBSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBSCOM_EMER_QT,2))) 
            CONTRIB_PUBSCOM_EMEE_SUMDED =   CONTRIB_PUBSDED_EMEE_QT(pers_j)
            CONTRIB_PUBSCOM_EMER_SUMDED =   SUM(CONTRIB_PUBSDED_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBSDED_EMER_QT,2)))
            !Compute the employee contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBSCOM_EMEE_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBSCOM_EMEE_QT,2)
                    CONTRIB_PUBSCOM_EMEE_NET(i) = CONTRIB_PUBSCOM_EMEE_QT(pers_j,i) - &
                    &CONTRIB_PUBSCOM_EMEE_SUMDED*(CONTRIB_PUBSCOM_EMEE_QT(pers_j,i)/CONTRIB_PUBSCOM_EMEE_SUMCON)        
                END DO
                !contributions on double holiday earnings
                CONTRIB_PUBSCOM_EMEE_NET(SIZE(CONTRIB_PUBSCOM_EMEE_NET,1)) = CONTRIB_PUBSHOL_EMEE_QT(pers_j) - &
                &CONTRIB_PUBSCOM_EMEE_SUMDED*(CONTRIB_PUBSHOL_EMEE_QT(pers_j)/CONTRIB_PUBSCOM_EMEE_SUMCON)        
            END IF
            !Compute the employer contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBSCOM_EMER_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBSCOM_EMER_QT,2)
                    CONTRIB_PUBSCOM_EMER_NET(i) = CONTRIB_PUBSCOM_EMER_QT(pers_j,i) - &
                    &CONTRIB_PUBSCOM_EMER_SUMDED*(CONTRIB_PUBSCOM_EMER_QT(pers_j,i)/CONTRIB_PUBSCOM_EMER_SUMCON)        
                END DO
            END IF
            !add the contributions to the correct vector element
            !Distinguish local from federal civil servants
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + SUM(CONTRIB_PUBNSCOM_EMEE_NET(1:5))
                CONTRIB_AGGREGA(2) = CONTRIB_AGGREGA(2) + CONTRIB_PUBSCOM_EMEE_NET(SIZE(CONTRIB_PUBSCOM_EMEE_NET,1))
                CONTRIB_AGGREGA(5) = CONTRIB_AGGREGA(5) + SUM(CONTRIB_PUBNSCOM_EMER_NET(1:5))
                CONTRIB_AGGREGA(15) = CONTRIB_AGGREGA(15) + CONTRIB_PUBNSCOM_EMER_NET(6)
            END IF
        END IF
        CONTRIB_AGGREGA(11) = CONTRIB_AGGREGA(11) + CONTRIB_SSEARBEN_QT(pers_j) 
        !Add all the individual results to a vector that aggregates results over all individuals
        DO j=1,SIZE(CONTRIB_AGGREGA,1)
            IF (INT(100.D0*CONTRIB_AGGREGA(j)).NE.0) THEN
                CONTRIB_RESULT_COMPARE(j,1)=CONTRIB_RESULT_COMPARE(j,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_AGGREGA(j)
                CONTRIB_RESULT_COMPARE(j,2)=CONTRIB_RESULT_COMPARE(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    !Inflate contributions to annual values
    !Factor 3.60 based on SummaryOfAggregates.xls (salary mass RSZ and RSZPPO in fourth quarter as fraction of annual salary mass in sample)
    !Use different factor for contributions paid on wages and other contributions
    CONTRIB_RESULT_COMPARE(1:19,1)=(CONTRIB_RESULT_COMPARE(1:19,1)*4.D0)/1000.D0

    !Compute columns 3 to 6 of the vector with comparable aggregates
    DO i=1,SIZE(CONTRIB_RESULT_COMPARE,1)
        !difference between real value and estimate
        CONTRIB_RESULT_COMPARE(i,3)= CONTRIB_RESULT_COMPARE(i,1) - CONTRIB_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(CONTRIB_EXTERN(i)).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,4)= 100.D0*(CONTRIB_RESULT_COMPARE(i,3)/CONTRIB_EXTERN(i))    
        END IF
        !Average amount
        IF (INT(CONTRIB_RESULT_COMPARE(i,2)).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,5)=CONTRIB_RESULT_COMPARE(i,1)/CONTRIB_RESULT_COMPARE(i,2)
        END IF
        !Fraction of population with non zero value
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,6)=100.D0*CONTRIB_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''RSZ Bijdragen na aftrek van bijdragenverminderingen op jaarbasis naar bijdragepost van de RSZ'')')
    Write(15,'(''Deze tabel bevat in de kolom steekproef de gesimuleerde netto bijdragen van loon en weddetrekkenden in de'')')
    Write(15,'(''priv plus de netto bijdragen van ALLE federale ambtenaren (i.e. zowel statutair als contractueel)'')')
    Write(15,'(''Nominale bedragen per 1000 Euro in prijzen van 2008'')')
    Write(15,'(''                                                                                                         &
    &   Externe bron          Steekproef   Aantal individuen  Absoluut verschil  %Verschil     Gemid      %Pop'')')
    Write(15,'(''                                                                                                         &
    &             C1                  C2                                  C2-C1  100*C2/C1 per indiv'')')
    CALL DRAWLINE(15,210,'-')    
    DO i=1,SIZE(CONTRIB_RESULT_COMPARE,1)
        Write(15,'(a100,F20.2,F20.2,F20.0,F20.0,F10.2,F10.2,F10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),&
        &(CONTRIB_RESULT_COMPARE(i,j),j=1,6)
    END DO
    Write(15,'(100x,F20.2,F20.2)')SUM(CONTRIB_EXTERN(1:SIZE(CONTRIB_EXTERN,1))),&
    &SUM(CONTRIB_RESULT_COMPARE(1:SIZE(CONTRIB_RESULT_COMPARE,1),1))

    DEALLOCATE(CONTRIB_AGGREGA)      
    DEALLOCATE(CONTRIB_RESULT_COMPARE)
    DEALLOCATE(CONTRIB_LAB_COMPARE)     
    DEALLOCATE(CONTRIB_EXTERN)        

    !Number of workers
    NumAnal = 30
    ALLOCATE(CONTRIB_RESULT_COMPARE(NumAnal,5))
    ALLOCATE(CONTRIB_LAB_COMPARE(NumAnal))     
    ALLOCATE(CONTRIB_EXTERN(NumAnal))        

    !Initialise results
    CONTRIB_RESULT_COMPARE(1:SIZE(CONTRIB_RESULT_COMPARE,1),1:5) = 0.D0   
    CONTRIB_EXTERN(1:SIZE(CONTRIB_EXTERN,1)) = 0.D0           

    !Define labels
    CONTRIB_LAB_COMPARE(1)  = "RSZ: arbeiders, mannen, priv"
    CONTRIB_LAB_COMPARE(2)  = "RSZ: arbeiders, vrouwen, priv"
    CONTRIB_LAB_COMPARE(3)  = "RSZ: bedienden, mannen, priv"
    CONTRIB_LAB_COMPARE(4)  = "RSZ: bedienden, vrouwen, priv"
    CONTRIB_LAB_COMPARE(5)  = "RSZ: arbeiders, mannen, overheidssector"
    CONTRIB_LAB_COMPARE(6)  = "RSZ: arbeiders, vrouwen, overheidssector"
    CONTRIB_LAB_COMPARE(7)  = "RSZ: bedienden, mannen, overheidssector"
    CONTRIB_LAB_COMPARE(8)  = "RSZ: bedienden, vrouwen, overheidssector"
    CONTRIB_LAB_COMPARE(9)  = "RSZ: ambtenaren, mannen, overheidssector"
    CONTRIB_LAB_COMPARE(10) = "RSZ: ambtenaren, vrouwen, overheidssector"
    CONTRIB_LAB_COMPARE(11) = "RSZPPO: arbeiders, mannen, overheidssector"
    CONTRIB_LAB_COMPARE(12) = "RSZPPO: arbeiders, vrouwen, overheidssector"
    CONTRIB_LAB_COMPARE(13) = "RSZPPO: bedienden, mannen, overheidssector"
    CONTRIB_LAB_COMPARE(14) = "RSZPPO: bedienden, vrouwen, overheidssector"
    CONTRIB_LAB_COMPARE(15) = "RSZPPO: ambtenaren, mannen, overheidssector"
    CONTRIB_LAB_COMPARE(16) = "RSZPPO: ambtenaren, vrouwen, overheidssector"
    CONTRIB_LAB_COMPARE(17) = "HVKZ: zeelieden, mannen"
    CONTRIB_LAB_COMPARE(18) = "HVKZ: zeelieden, mannen"
    CONTRIB_LAB_COMPARE(19) = "RSVZ: zelfstandigen, hoofdbezigheid, mannen"
    CONTRIB_LAB_COMPARE(20) = "RSVZ: zelfstandigen, hoofdbezigheid, vrouwen"
    CONTRIB_LAB_COMPARE(21) = "RSVZ: zelfstandigen, nevenbezigheid, mannen"
    CONTRIB_LAB_COMPARE(22) = "RSVZ: zelfstandigen, nevenbezigheid, vrouwen"
    CONTRIB_LAB_COMPARE(23) = "RSVZ: zelfstandigen, activiteiten na pensioen, mannen"
    CONTRIB_LAB_COMPARE(24) = "RSVZ: zelfstandigen, activiteiten na pensioen, vrouwen"
    CONTRIB_LAB_COMPARE(25) = "RSVZ: helpers, hoofdbezigheid, mannen"
    CONTRIB_LAB_COMPARE(26) = "RSVZ: helpers, hoofdbezigheid, vrouwen"
    CONTRIB_LAB_COMPARE(27) = "RSVZ: helpers, nevenbezigheid, mannen"
    CONTRIB_LAB_COMPARE(28) = "RSVZ: helpers, nevenbezigheid, vrouwen"
    CONTRIB_LAB_COMPARE(29) = "RSVZ: helpers, activiteiten na pensioen, mannen"
    CONTRIB_LAB_COMPARE(30) = "RSVZ: helpers, activiteiten na pensioen, vrouwen"

    !Define values taken from external sources
    CONTRIB_EXTERN(1)   = 0
    CONTRIB_EXTERN(2)   = 0
    CONTRIB_EXTERN(3)   = 0
    CONTRIB_EXTERN(4)   = 0
    CONTRIB_EXTERN(5)   = 0
    CONTRIB_EXTERN(6)   = 0
    CONTRIB_EXTERN(7)   = 0
    CONTRIB_EXTERN(8)   = 0
    CONTRIB_EXTERN(9)   = 0
    CONTRIB_EXTERN(10)  = 0
    CONTRIB_EXTERN(11)  = 0
    CONTRIB_EXTERN(12)  = 0
    CONTRIB_EXTERN(13)  = 0
    CONTRIB_EXTERN(14)  = 0
    CONTRIB_EXTERN(15)  = 0
    CONTRIB_EXTERN(16)  = 0
    CONTRIB_EXTERN(17)  = 0
    CONTRIB_EXTERN(18)  = 0
    CONTRIB_EXTERN(19)  = 0
    CONTRIB_EXTERN(20)  = 0
    CONTRIB_EXTERN(21)  = 0
    CONTRIB_EXTERN(22)  = 0
    CONTRIB_EXTERN(23)  = 0
    CONTRIB_EXTERN(24)  = 0
    CONTRIB_EXTERN(25)  = 0
    CONTRIB_EXTERN(26)  = 0
    CONTRIB_EXTERN(27)  = 0
    CONTRIB_EXTERN(28)  = 0
    CONTRIB_EXTERN(29)  = 0
    CONTRIB_EXTERN(30)  = 0

    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !If on the private labour market
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
            IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                CONTRIB_RESULT_COMPARE(1,1) = CONTRIB_RESULT_COMPARE(1,1) + 1
                CONTRIB_RESULT_COMPARE(1,2) = CONTRIB_RESULT_COMPARE(1,2) + MIMOSIS_WEIGHT(pers_j)
            ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                CONTRIB_RESULT_COMPARE(2,1) = CONTRIB_RESULT_COMPARE(2,1) + 1
                CONTRIB_RESULT_COMPARE(2,2) = CONTRIB_RESULT_COMPARE(2,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
            IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                CONTRIB_RESULT_COMPARE(3,1) = CONTRIB_RESULT_COMPARE(3,1) + 1
                CONTRIB_RESULT_COMPARE(3,2) = CONTRIB_RESULT_COMPARE(3,2) + MIMOSIS_WEIGHT(pers_j)
            ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                CONTRIB_RESULT_COMPARE(4,1) = CONTRIB_RESULT_COMPARE(4,1) + 1
                CONTRIB_RESULT_COMPARE(4,2) = CONTRIB_RESULT_COMPARE(4,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF   
        !If on the public labour market
        !if working as blue collar without statutory service
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
            !If working at the federal level 
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(5,1) = CONTRIB_RESULT_COMPARE(5,1) + 1
                    CONTRIB_RESULT_COMPARE(5,2) = CONTRIB_RESULT_COMPARE(5,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(6,1) = CONTRIB_RESULT_COMPARE(6,1) + 1
                    CONTRIB_RESULT_COMPARE(6,2) = CONTRIB_RESULT_COMPARE(6,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(11,1) = CONTRIB_RESULT_COMPARE(11,1) + 1
                    CONTRIB_RESULT_COMPARE(11,2) = CONTRIB_RESULT_COMPARE(11,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(12,1) = CONTRIB_RESULT_COMPARE(12,1) + 1
                    CONTRIB_RESULT_COMPARE(12,2) = CONTRIB_RESULT_COMPARE(12,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            END IF
        !if working as white collar without statutory service
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
            !If working at the federal level 
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(7,1) = CONTRIB_RESULT_COMPARE(7,1) + 1
                    CONTRIB_RESULT_COMPARE(7,2) = CONTRIB_RESULT_COMPARE(7,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(8,1) = CONTRIB_RESULT_COMPARE(8,1) + 1
                    CONTRIB_RESULT_COMPARE(8,2) = CONTRIB_RESULT_COMPARE(8,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(13,1) = CONTRIB_RESULT_COMPARE(13,1) + 1
                    CONTRIB_RESULT_COMPARE(13,2) = CONTRIB_RESULT_COMPARE(13,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(14,1) = CONTRIB_RESULT_COMPARE(14,1) + 1
                    CONTRIB_RESULT_COMPARE(14,2) = CONTRIB_RESULT_COMPARE(14,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            END IF
        !if working as civil servant (=with statutory service)
        ELSE IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            !If working at the federal level 
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(9,1) = CONTRIB_RESULT_COMPARE(9,1) + 1
                    CONTRIB_RESULT_COMPARE(9,2) = CONTRIB_RESULT_COMPARE(9,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(10,1) = CONTRIB_RESULT_COMPARE(10,1) + 1
                    CONTRIB_RESULT_COMPARE(10,2) = CONTRIB_RESULT_COMPARE(10,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(15,1) = CONTRIB_RESULT_COMPARE(15,1) + 1
                    CONTRIB_RESULT_COMPARE(15,2) = CONTRIB_RESULT_COMPARE(15,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(16,1) = CONTRIB_RESULT_COMPARE(16,1) + 1
                    CONTRIB_RESULT_COMPARE(16,2) = CONTRIB_RESULT_COMPARE(16,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            END IF
        END IF            
        IF (CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.1) THEN
            IF (CONTRIB_SELFPCR_QT(pers_j).EQ.1) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(19,1) = CONTRIB_RESULT_COMPARE(19,1) + 1
                    CONTRIB_RESULT_COMPARE(19,2) = CONTRIB_RESULT_COMPARE(19,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(20,1) = CONTRIB_RESULT_COMPARE(20,1) + 1
                    CONTRIB_RESULT_COMPARE(20,2) = CONTRIB_RESULT_COMPARE(20,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            ELSE IF (CONTRIB_SELFPCR_QT(pers_j).EQ.2) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(21,1) = CONTRIB_RESULT_COMPARE(21,1) + 1
                    CONTRIB_RESULT_COMPARE(21,2) = CONTRIB_RESULT_COMPARE(21,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(22,1) = CONTRIB_RESULT_COMPARE(22,1) + 1
                    CONTRIB_RESULT_COMPARE(22,2) = CONTRIB_RESULT_COMPARE(22,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            ELSE IF (CONTRIB_SELFPCR_QT(pers_j).EQ.3) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(23,1) = CONTRIB_RESULT_COMPARE(23,1) + 1
                    CONTRIB_RESULT_COMPARE(23,2) = CONTRIB_RESULT_COMPARE(23,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(24,1) = CONTRIB_RESULT_COMPARE(24,1) + 1
                    CONTRIB_RESULT_COMPARE(24,2) = CONTRIB_RESULT_COMPARE(24,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            ELSE IF (CONTRIB_SELFPCR_QT(pers_j).EQ.4) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(25,1) = CONTRIB_RESULT_COMPARE(25,1) + 1
                    CONTRIB_RESULT_COMPARE(25,2) = CONTRIB_RESULT_COMPARE(25,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(26,1) = CONTRIB_RESULT_COMPARE(26,1) + 1
                    CONTRIB_RESULT_COMPARE(26,2) = CONTRIB_RESULT_COMPARE(26,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            ELSE IF (CONTRIB_SELFPCR_QT(pers_j).EQ.5) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(27,1) = CONTRIB_RESULT_COMPARE(27,1) + 1
                    CONTRIB_RESULT_COMPARE(27,2) = CONTRIB_RESULT_COMPARE(27,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(28,1) = CONTRIB_RESULT_COMPARE(28,1) + 1
                    CONTRIB_RESULT_COMPARE(28,2) = CONTRIB_RESULT_COMPARE(28,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            ELSE IF (CONTRIB_SELFPCR_QT(pers_j).EQ.6) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    CONTRIB_RESULT_COMPARE(29,1) = CONTRIB_RESULT_COMPARE(29,1) + 1
                    CONTRIB_RESULT_COMPARE(29,2) = CONTRIB_RESULT_COMPARE(29,2) + MIMOSIS_WEIGHT(pers_j)
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    CONTRIB_RESULT_COMPARE(30,1) = CONTRIB_RESULT_COMPARE(30,1) + 1
                    CONTRIB_RESULT_COMPARE(30,2) = CONTRIB_RESULT_COMPARE(30,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            END IF
        END IF
    END DO

    DO i=1,SIZE(CONTRIB_RESULT_COMPARE,1)
        !difference between real value and estimate
        CONTRIB_RESULT_COMPARE(i,3)= CONTRIB_RESULT_COMPARE(i,2) - CONTRIB_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(CONTRIB_EXTERN(i)).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,4)= 100.D0*(CONTRIB_RESULT_COMPARE(i,3)/CONTRIB_EXTERN(i))    
        END IF
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,5)=100.D0*CONTRIB_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''Aantal werkenden naar statuut en geslacht'')')
    Write(15,'(''Vergelijking met externe bron op basis van cijfers van 2008'')')
    Write(15,'(''                                                                                                        &
    &Extern   NietGew     Gewog Abs versc %Verschil      %Pop'')')
    Write(15,'(''                                                                                                         &
    &   C1                  C2     C2-C1 100*C2/C1'')')
    CALL DRAWLINE(15,160,'-')    
    DO i=1,4
        Write(15,'(a100,4f10.0,f10.2,f10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),(CONTRIB_RESULT_COMPARE(i,j),j=1,5)
    END DO
    CALL DRAWLINE(15,160,'-')    
    Write(15,'("Subtotaal RSZ priv",81x,3f10.0)')SUM(CONTRIB_EXTERN(1:4)),SUM(CONTRIB_RESULT_COMPARE(1:4,1)),&
    &SUM(CONTRIB_RESULT_COMPARE(1:4,2))
    CALL DRAWLINE(15,160,'-')    
    DO i=5,10
        Write(15,'(a100,4f10.0,f10.2,f10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),(CONTRIB_RESULT_COMPARE(i,j),j=1,5)
    END DO
    CALL DRAWLINE(15,160,'-')    
    Write(15,'("Subtotaal RSZ overheidssector",71x,3f10.0)')SUM(CONTRIB_EXTERN(5:10)),SUM(CONTRIB_RESULT_COMPARE(5:10,1)),&
    &SUM(CONTRIB_RESULT_COMPARE(5:10,2))
    CALL DRAWLINE(15,160,'-')    
    DO i=11,16
        Write(15,'(a100,4f10.0,f10.2,f10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),(CONTRIB_RESULT_COMPARE(i,j),j=1,5)
    END DO
    CALL DRAWLINE(15,160,'-')    
    Write(15,'("Subtotaal RSZPPO",84x,3f10.0)')SUM(CONTRIB_EXTERN(11:16)),SUM(CONTRIB_RESULT_COMPARE(11:16,1)),&
    &SUM(CONTRIB_RESULT_COMPARE(11:16,2))
    CALL DRAWLINE(15,160,'-')    
    DO i=17,18
        Write(15,'(a100,4f10.0,f10.2,f10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),(CONTRIB_RESULT_COMPARE(i,j),j=1,5)
    END DO
    CALL DRAWLINE(15,160,'-')    
    DO i=19,30
        Write(15,'(a100,4f10.0,f10.2,f10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),(CONTRIB_RESULT_COMPARE(i,j),j=1,5)
    END DO
    CALL DRAWLINE(15,160,'-')    
    Write(15,'("Subtotaal RSVZ",86x,3f10.0)')SUM(CONTRIB_EXTERN(19:30)),SUM(CONTRIB_RESULT_COMPARE(19:30,1)),&
    &SUM(CONTRIB_RESULT_COMPARE(19:30,2))
    CALL DRAWLINE(15,160,'-')    
    Write(15,'("Totaal",94x,3f10.0)')SUM(CONTRIB_EXTERN(1:SIZE(CONTRIB_RESULT_COMPARE,1))),&
    &SUM(CONTRIB_RESULT_COMPARE(1:SIZE(CONTRIB_RESULT_COMPARE,1),1)),&
    &SUM(CONTRIB_RESULT_COMPARE(1:SIZE(CONTRIB_RESULT_COMPARE,1),2))

    DEALLOCATE(CONTRIB_RESULT_COMPARE)
    DEALLOCATE(CONTRIB_LAB_COMPARE)     
    DEALLOCATE(CONTRIB_EXTERN)        

    !Contribution mass of private and public sector and self employed
    NumAnal = 8
    ALLOCATE(CONTRIB_RESULT_COMPARE(NumAnal,6))
    ALLOCATE(CONTRIB_LAB_COMPARE(NumAnal))     
    ALLOCATE(CONTRIB_EXTERN(NumAnal))        

    !Initialise results
    CONTRIB_RESULT_COMPARE(1:SIZE(CONTRIB_RESULT_COMPARE,1),1:6) = 0.D0   
    CONTRIB_EXTERN(1:SIZE(CONTRIB_EXTERN,1)) = 0.D0           

    !Define labels
    CONTRIB_LAB_COMPARE(1)  = "arbeiders, priv"
    CONTRIB_LAB_COMPARE(2)  = "bedienden, priv"
    CONTRIB_LAB_COMPARE(3)  = "arbeiders, overheidssector"
    CONTRIB_LAB_COMPARE(4)  = "bedienden, overheidssector"
    CONTRIB_LAB_COMPARE(5)  = "ambtenaren, overheidssector"
    CONTRIB_LAB_COMPARE(6)  = "zelfstandigen, hoofdbezigheid (inkomen van 1998 na herwaardering)"
    CONTRIB_LAB_COMPARE(7)  = "zelfstandigen, nevenbezigheid (inkomen van 1998 na herwaardering)"
    CONTRIB_LAB_COMPARE(8)  = "zelfstandigen, actief na pensioenleeftijd (inkomen van 1998 na herwaardering)"

    !Define values taken from external sources
    CONTRIB_EXTERN(1)   = 0
    CONTRIB_EXTERN(2)   = 0
    CONTRIB_EXTERN(3)   = 0
    CONTRIB_EXTERN(4)   = 0
    CONTRIB_EXTERN(5)   = 0
    CONTRIB_EXTERN(6)   = 0
    CONTRIB_EXTERN(7)   = 0
    CONTRIB_EXTERN(8)   = 0

    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !If on the private labour market
        IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
            !Remark: single holiday earnings are not included in the contribution base of blue collar workers
            !Their contributions are instead computed on 108% of their income.
            CONTRIB_INCCONC = CONTRIB_INCPRIV_QT(pers_j)+CONTRIB_DHPRIV_QT(pers_j)+CONTRIB_INCSPRIV_QT(pers_j)
            IF (INT(100.D0*CONTRIB_INCCONC).NE.0) THEN
                CONTRIB_RESULT_COMPARE(1,1)=CONTRIB_RESULT_COMPARE(1,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_INCCONC
                CONTRIB_RESULT_COMPARE(1,2)=CONTRIB_RESULT_COMPARE(1,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
            CONTRIB_INCCONC = CONTRIB_INCPRIV_QT(pers_j)+CONTRIB_SHPRIV_QT(pers_j)+CONTRIB_DHPRIV_QT(pers_j)+&
            &CONTRIB_INCSPRIV_QT(pers_j)
            IF (INT(100.D0*CONTRIB_INCCONC).NE.0) THEN
                CONTRIB_RESULT_COMPARE(2,1)=CONTRIB_RESULT_COMPARE(2,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_INCCONC
                CONTRIB_RESULT_COMPARE(2,2)=CONTRIB_RESULT_COMPARE(2,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF        
        !If on the public labour market
        IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
            CONTRIB_INCCONC = CONTRIB_INCPUB_QT(pers_j)+CONTRIB_DHPUB_QT(pers_j)+CONTRIB_INCSPUB_QT(pers_j)
            IF (INT(100.D0*CONTRIB_INCCONC).NE.0) THEN
                CONTRIB_RESULT_COMPARE(3,1)=CONTRIB_RESULT_COMPARE(3,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_INCCONC
                CONTRIB_RESULT_COMPARE(3,2)=CONTRIB_RESULT_COMPARE(3,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
            CONTRIB_INCCONC = CONTRIB_INCPUB_QT(pers_j)+CONTRIB_SHPUB_QT(pers_j)+CONTRIB_DHPUB_QT(pers_j)+&
            &CONTRIB_INCSPUB_QT(pers_j)
            IF (INT(100.D0*CONTRIB_INCCONC).NE.0) THEN
                CONTRIB_RESULT_COMPARE(4,1)=CONTRIB_RESULT_COMPARE(4,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_INCCONC
                CONTRIB_RESULT_COMPARE(4,2)=CONTRIB_RESULT_COMPARE(4,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        ELSE IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            CONTRIB_INCCONC = CONTRIB_INCPUB_QT(pers_j)+CONTRIB_SHPUB_QT(pers_j)+CONTRIB_DHPUB_QT(pers_j)+&
            &CONTRIB_INCSPUB_QT(pers_j)
            IF (INT(100.D0*CONTRIB_INCCONC).NE.0) THEN
                CONTRIB_RESULT_COMPARE(5,1)=CONTRIB_RESULT_COMPARE(5,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_INCCONC
                CONTRIB_RESULT_COMPARE(5,2)=CONTRIB_RESULT_COMPARE(5,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF        
        !If self employed
        IF (CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.1) THEN
            IF (ANY(CONTRIB_SELFPCR_QT(pers_j).EQ.(/1, 4/)).AND.CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                CONTRIB_INCCONC = CONTRIB_INCSELF_QT(pers_j)
                IF (INT(100.D0*CONTRIB_INCCONC).NE.0) THEN
                    CONTRIB_RESULT_COMPARE(6,1)=CONTRIB_RESULT_COMPARE(6,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_INCCONC
                    CONTRIB_RESULT_COMPARE(6,2)=CONTRIB_RESULT_COMPARE(6,2)+MIMOSIS_WEIGHT(pers_j)
                END IF
            ELSE IF (ANY(CONTRIB_SELFPCR_QT(pers_j).EQ.(/2, 5/)).AND.CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                CONTRIB_INCCONC = CONTRIB_INCSELF_QT(pers_j)
                IF (INT(100.D0*CONTRIB_INCCONC).NE.0) THEN
                    CONTRIB_RESULT_COMPARE(7,1)=CONTRIB_RESULT_COMPARE(7,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_INCCONC
                    CONTRIB_RESULT_COMPARE(7,2)=CONTRIB_RESULT_COMPARE(7,2)+MIMOSIS_WEIGHT(pers_j)
                END IF
            ELSE IF (ANY(CONTRIB_SELFPCR_QT(pers_j).EQ.(/3, 6/)).AND.CONTRIB_SELFYEAR_QT(pers_j).GT.3) THEN
                CONTRIB_INCCONC = CONTRIB_INCSELF_QT(pers_j)
                IF (INT(100.D0*CONTRIB_INCCONC).NE.0) THEN
                    CONTRIB_RESULT_COMPARE(8,1)=CONTRIB_RESULT_COMPARE(8,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_INCCONC
                    CONTRIB_RESULT_COMPARE(8,2)=CONTRIB_RESULT_COMPARE(8,2)+MIMOSIS_WEIGHT(pers_j)
                END IF
            END IF
        END IF        
    END DO

    !Divide results by 1000
    CONTRIB_RESULT_COMPARE(1:8,1) = CONTRIB_RESULT_COMPARE(1:8,1)/1000.D0

    !Compute columns 3 to 6 of the vector with comparable aggregates
    DO i=1,SIZE(CONTRIB_RESULT_COMPARE,1)
        !difference between real value and estimate
        CONTRIB_RESULT_COMPARE(i,3)= CONTRIB_RESULT_COMPARE(i,1) - CONTRIB_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(CONTRIB_EXTERN(i)).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,4)= 100.D0*(CONTRIB_RESULT_COMPARE(i,3)/CONTRIB_EXTERN(i))    
        END IF
        !Average amount
        IF (INT(CONTRIB_RESULT_COMPARE(i,2)).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,5)=CONTRIB_RESULT_COMPARE(i,1)/CONTRIB_RESULT_COMPARE(i,2)
        END IF
        !Fraction of population with non zero value
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,6)=100.D0*CONTRIB_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''Aan bijdragen onderworpen massa per kwartaal'')')
    Write(15,'(''Externe gegevens van het 4de kwartaal van 2008'')')
    Write(15,'(''Nominale bedragen per 1000 Euro in prijzen van 2008'')')
    Write(15,'(''                                                                                                         &
    &   Externe bron          Steekproef   Aantal individuen  Absoluut verschil  %Verschil     Gemid      %Pop'')')
    Write(15,'(''                                                                                                         &
    &             C1                  C2                                  C2-C1  100*C2/C1 per indiv'')')
    CALL DRAWLINE(15,210,'-')    
    DO i=1,2
        Write(15,'(a100,F20.2,F20.2,F20.0,F20.0,F10.2,F10.2,F10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),&
        &(CONTRIB_RESULT_COMPARE(i,j),j=1,6)
    END DO
    CALL DRAWLINE(15,210,'-')    
    Write(15,'("Subtotaal privsector",79x,F20.2,F20.2)')SUM(CONTRIB_EXTERN(1:2)),SUM(CONTRIB_RESULT_COMPARE(1:2,1))
    CALL DRAWLINE(15,210,'-')    
    DO i=3,5
        Write(15,'(a100,F20.2,F20.2,F20.0,F20.0,F10.2,F10.2,F10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),&
        &(CONTRIB_RESULT_COMPARE(i,j),j=1,6)
    END DO
    CALL DRAWLINE(15,210,'-')    
    Write(15,'("Subtotaal publieke sector",75x,F20.2,F20.2)')SUM(CONTRIB_EXTERN(3:5)),SUM(CONTRIB_RESULT_COMPARE(3:5,1))
    CALL DRAWLINE(15,210,'-')    
    DO i=6,8
        Write(15,'(a100,F20.2,F20.2,F20.0,F20.0,F10.2,F10.2,F10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),&
        &(CONTRIB_RESULT_COMPARE(i,j),j=1,6)
    END DO
    CALL DRAWLINE(15,210,'-')    
    Write(15,'("Subtotaal zelfstandigen",77x,F20.2,F20.2)')SUM(CONTRIB_EXTERN(6:8)),SUM(CONTRIB_RESULT_COMPARE(6:8,1))
    CALL DRAWLINE(15,210,'-')    
    Write(15,'("Totaal",94x,F20.2,F20.2)')SUM(CONTRIB_EXTERN(1:8)),SUM(CONTRIB_RESULT_COMPARE(1:8,1))
    CALL DRAWLINE(15,210,'-')    

    DEALLOCATE(CONTRIB_RESULT_COMPARE)
    DEALLOCATE(CONTRIB_LAB_COMPARE)     
    DEALLOCATE(CONTRIB_EXTERN)        

    !RSZ deductions
    NumAnal = 17
    ALLOCATE(CONTRIB_AGGREGA(NumAnal))       
    ALLOCATE(CONTRIB_RESULT_COMPARE(NumAnal,6))
    ALLOCATE(CONTRIB_LAB_COMPARE(NumAnal))     
    ALLOCATE(CONTRIB_EXTERN(NumAnal))        

    !Initialise results
    CONTRIB_RESULT_COMPARE(1:SIZE(CONTRIB_RESULT_COMPARE,1),1:6) = 0.D0   
    CONTRIB_EXTERN(1:SIZE(CONTRIB_EXTERN,1)) = 0.D0           

    !Define labels
    CONTRIB_LAB_COMPARE(1)   = "Structurele bijdragevermindering"
    CONTRIB_LAB_COMPARE(2)   = "Bedrijfsplannen en tewerkstellingsakkoorden"
    CONTRIB_LAB_COMPARE(3)   = "Plannen +1, +2, +3"
    CONTRIB_LAB_COMPARE(4)   = "Activa plan"
    CONTRIB_LAB_COMPARE(5)   = "Collectieve arbeidsduurvermindering"
    CONTRIB_LAB_COMPARE(6)   = "Startbanen"
    CONTRIB_LAB_COMPARE(7)   = "Activering v/d werkloosheidsuitkeringen"
    CONTRIB_LAB_COMPARE(8)   = "Gesubsidieerde contractuelen"
    CONTRIB_LAB_COMPARE(9)   = "Loopbaanonderbreking (+58 jaar)"
    CONTRIB_LAB_COMPARE(10)  = "Loonmatiging universitairen"
    CONTRIB_LAB_COMPARE(11)  = "Statuut kunstenaars"
    CONTRIB_LAB_COMPARE(12)  = "Onthaalmoeders"
    CONTRIB_LAB_COMPARE(13)  = "Diversen: KB's, deeltijdsbrugp., baggeraars, ..."
    CONTRIB_LAB_COMPARE(14)  = "Sociale Maribel"
    CONTRIB_LAB_COMPARE(15)  = "Vermindering persoonlijke bijdrage"
    CONTRIB_LAB_COMPARE(16)  = "Aantal FTE's werkbonus"
    CONTRIB_LAB_COMPARE(17)  = "Aantal FTE's totaal"

    !Define values taken from external sources
    !bedragen ingegeven in miljoen euro maar later vermenigvuldigd met 1000
    CONTRIB_EXTERN(1)   = 0
    CONTRIB_EXTERN(2)   = 0
    CONTRIB_EXTERN(3)   = 0
    CONTRIB_EXTERN(4)   = 0
    CONTRIB_EXTERN(5)   = 0
    CONTRIB_EXTERN(6)   = 0
    CONTRIB_EXTERN(7)   = 0
    CONTRIB_EXTERN(8)   = 0
    CONTRIB_EXTERN(9)   = 0
    CONTRIB_EXTERN(10)  = 0
    CONTRIB_EXTERN(11)  = 0
    CONTRIB_EXTERN(12)  = 0
    CONTRIB_EXTERN(13)  = 0
    CONTRIB_EXTERN(14)  = 0
    CONTRIB_EXTERN(15)  = 0

    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !Make vector with local aggregates empty
        CONTRIB_AGGREGA(1:SIZE(CONTRIB_AGGREGA,1)) = 0.D0
        !If on the private labour market
        IF (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            CONTRIB_AGGREGA(1)  = CONTRIB_AGGREGA(1) +   CONTRIB_WEARDEDU_EMER_QT(pers_j,1)
            CONTRIB_AGGREGA(2)  = CONTRIB_AGGREGA(2) +   CONTRIB_WEARDEDU_EMER_QT(pers_j,2) + &
                                                        &CONTRIB_WEARDEDU_EMER_QT(pers_j,7) + &
                                                        &CONTRIB_WEARDEDU_EMER_QT(pers_j,18)
            CONTRIB_AGGREGA(3)  = CONTRIB_AGGREGA(3)  +  CONTRIB_WEARDEDU_EMER_QT(pers_j,3)
            CONTRIB_AGGREGA(4)  = CONTRIB_AGGREGA(4)  +  CONTRIB_WEARDEDU_EMER_QT(pers_j,19)+ & 
                                                        &CONTRIB_WEARDEDU_EMER_QT(pers_j,4) + &
                                                        &CONTRIB_WEARDEDU_EMER_QT(pers_j,9) 
            CONTRIB_AGGREGA(5)  = CONTRIB_AGGREGA(5)  +  CONTRIB_WEARDEDU_EMER_QT(pers_j,15)
            CONTRIB_AGGREGA(6)  = CONTRIB_AGGREGA(6)  +  CONTRIB_WEARDEDU_EMER_QT(pers_j,20)
            CONTRIB_AGGREGA(7)  = CONTRIB_AGGREGA(7)  +  CONTRIB_WEARDEDU_EMER_QT(pers_j,17) 
            CONTRIB_AGGREGA(8)  = CONTRIB_AGGREGA(8)  +  CONTRIB_WEARDEDU_EMER_QT(pers_j,6)
            CONTRIB_AGGREGA(9)  = CONTRIB_AGGREGA(9)  +  CONTRIB_WEARDEDU_EMER_QT(pers_j,11)
            CONTRIB_AGGREGA(10) = CONTRIB_AGGREGA(10) +  CONTRIB_WEARDEDU_EMER_QT(pers_j,14)
            CONTRIB_AGGREGA(11) = 0
            CONTRIB_AGGREGA(12) = 0
            CONTRIB_AGGREGA(13) = CONTRIB_AGGREGA(13) +  CONTRIB_WEARDEDU_EMER_QT(pers_j,5) + &
                                                        &CONTRIB_WEARDEDU_EMER_QT(pers_j,8) + &
                                                        &CONTRIB_WEARDEDU_EMER_QT(pers_j,10) + &
                                                        &CONTRIB_WEARDEDU_EMER_QT(pers_j,12) + &
                                                        &CONTRIB_WEARDEDU_EMER_QT(pers_j,16) + &
                                                        &CONTRIB_WEARDEDU_EMER_QT(pers_j,21)
            CONTRIB_AGGREGA(14) = CONTRIB_AGGREGA(14) + CONTRIB_WEARDEDU_EMER_QT(pers_j,13)
            CONTRIB_AGGREGA(15) = CONTRIB_AGGREGA(15) + CONTRIB_WEARDEDU_EMEE_QT(pers_j)
            IF (CONTRIB_WEARDEDU_EMEE_QT(pers_j).GT.0.D0) THEN
                CONTRIB_AGGREGA(16) = CONTRIB_AGGREGA(16) + CONTRIB_FULLPART_QT(pers_j)
            END IF
            CONTRIB_AGGREGA(17) = CONTRIB_AGGREGA(17) + CONTRIB_FULLPART_QT(pers_j)
!            IF (DeductionCode_Pers_1(pers_j).EQ.0.AND.APL_DeductionCode_Pers_1(pers_j).EQ.0.AND.&
!            &CONTRIB_WEARDEDU_EMEE_QT(pers_j).GT.0.D0) THEN
!                WRITE(15,*) 'WerkBonus A',pers_j,CONTRIB_FULLPART_QT(pers_j),CONTRIB_INCPRIV_QT(pers_j),&
!                &CONTRIB_WEARDEDU_EMEE_QT(pers_j)
!            END IF
!            IF ((DeductionCode_Pers_1(pers_j).EQ.1.OR.APL_DeductionCode_Pers_1(pers_j).EQ.1).AND.&
!            &CONTRIB_WEARDEDU_EMEE_QT(pers_j).EQ.0.D0.AND.CONTRIB_PUBNSDED_EMEE_QT(pers_j).EQ.0.D0) THEN
!                EQUIV_L = MIMOSIS_GRINC_HOUR(pers_j)*38.D0*4.3333D0
!                IF (EQUIV_L.LT.2118.21) THEN
!                    !This case can appear but only if the individual pays no contributions in the end,
!                    !because of lacking income
!                    WRITE(15,'(a12,I10,F15.5,2F15.0,I10,2F15.0)') 'WerkBonus A0',GECODEERD_INSZ_INPUT(pers_j),&
!                    &CONTRIB_FULLPART_QT(pers_j),&
!                    &CONTRIB_INCPRIV_QT(pers_j),EQUIV_L,CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_WEARDEDU_EMEE_QT(pers_j),&
!                    &CONTRIB_PUBNSDED_EMEE_QT(pers_j)
!                END IF
!            END IF
        END IF
        !If on the public labour market
        IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            CONTRIB_AGGREGA(1)  = CONTRIB_AGGREGA(1) +   CONTRIB_PUBNSDED_EMER_QT(pers_j,1)
            CONTRIB_AGGREGA(2)  = CONTRIB_AGGREGA(2) +   CONTRIB_PUBNSDED_EMER_QT(pers_j,2) + &
                                                        &CONTRIB_PUBNSDED_EMER_QT(pers_j,7) + &
                                                        &CONTRIB_PUBNSDED_EMER_QT(pers_j,18)
            CONTRIB_AGGREGA(3)  = CONTRIB_AGGREGA(3)  +  CONTRIB_PUBNSDED_EMER_QT(pers_j,3)
            CONTRIB_AGGREGA(4)  = CONTRIB_AGGREGA(4)  +  CONTRIB_PUBNSDED_EMER_QT(pers_j,19)+ &
                                                        &CONTRIB_PUBNSDED_EMER_QT(pers_j,4) + &
                                                        &CONTRIB_PUBNSDED_EMER_QT(pers_j,9) 
            CONTRIB_AGGREGA(5)  = CONTRIB_AGGREGA(5)  +  CONTRIB_PUBNSDED_EMER_QT(pers_j,15)
            CONTRIB_AGGREGA(6)  = CONTRIB_AGGREGA(6)  +  CONTRIB_PUBNSDED_EMER_QT(pers_j,20)
            CONTRIB_AGGREGA(7)  = CONTRIB_AGGREGA(7)  +  CONTRIB_PUBNSDED_EMER_QT(pers_j,17) 
            CONTRIB_AGGREGA(8)  = CONTRIB_AGGREGA(8)  +  CONTRIB_PUBNSDED_EMER_QT(pers_j,6)
            CONTRIB_AGGREGA(9)  = CONTRIB_AGGREGA(9)  +  CONTRIB_PUBNSDED_EMER_QT(pers_j,11)
            CONTRIB_AGGREGA(10) = CONTRIB_AGGREGA(10) +  CONTRIB_PUBNSDED_EMER_QT(pers_j,14)
            CONTRIB_AGGREGA(11) = 0
            CONTRIB_AGGREGA(12) = 0
            CONTRIB_AGGREGA(13) = CONTRIB_AGGREGA(13) +  CONTRIB_PUBNSDED_EMER_QT(pers_j,5) + &
                                                        &CONTRIB_PUBNSDED_EMER_QT(pers_j,8) + &
                                                        &CONTRIB_PUBNSDED_EMER_QT(pers_j,10) + &
                                                        &CONTRIB_PUBNSDED_EMER_QT(pers_j,12) + &
                                                        &CONTRIB_PUBNSDED_EMER_QT(pers_j,16) + &
                                                        &CONTRIB_PUBNSDED_EMER_QT(pers_j,21)
            CONTRIB_AGGREGA(14) = CONTRIB_AGGREGA(14) +  CONTRIB_PUBNSDED_EMER_QT(pers_j,13)
            CONTRIB_AGGREGA(15) = CONTRIB_AGGREGA(15) + CONTRIB_PUBNSDED_EMEE_QT(pers_j)
            IF (CONTRIB_PUBNSDED_EMEE_QT(pers_j).GT.0.D0) THEN
                CONTRIB_AGGREGA(16) = CONTRIB_AGGREGA(16) + CONTRIB_FULLPART_QT(pers_j)
            END IF
            CONTRIB_AGGREGA(17) = CONTRIB_AGGREGA(17) + CONTRIB_FULLPART_QT(pers_j)
!            IF (DeductionCode_Pers_1(pers_j).EQ.0.AND.APL_DeductionCode_Pers_1(pers_j).EQ.0.AND.&
!            &CONTRIB_PUBNSDED_EMEE_QT(pers_j).GT.0.D0) THEN
!                WRITE(15,*) 'WerkBonus B',pers_j,CONTRIB_FULLPART_QT(pers_j),CONTRIB_INCPUB_QT(pers_j),&
!                &CONTRIB_PUBNSDED_EMEE_QT(pers_j)
!            END IF
!            IF ((DeductionCode_Pers_1(pers_j).EQ.1.OR.APL_DeductionCode_Pers_1(pers_j).EQ.1).AND.&
!            &CONTRIB_WEARDEDU_EMEE_QT(pers_j).EQ.0.D0.AND.CONTRIB_PUBNSDED_EMEE_QT(pers_j).EQ.0.D0) THEN
!                EQUIV_L = MIMOSIS_GRINC_HOUR(pers_j)*38.D0*4.3333D0
!                IF (EQUIV_L.LT.2118.21) THEN
!                    !This case can appear but only if the individual pays no contributions in the end,
!                    !because of lacking income
!                    WRITE(15,'(a12,I10,F15.5,2F15.0,I10,2F15.0)') 'WerkBonus B0',GECODEERD_INSZ_INPUT(pers_j),&
!                    &CONTRIB_FULLPART_QT(pers_j),&
!                    &CONTRIB_INCPUB_QT(pers_j),EQUIV_L,CONTRIB_LABMSTAT_PUB_QT(pers_j),CONTRIB_WEARDEDU_EMEE_QT(pers_j),&
!                    &CONTRIB_PUBNSDED_EMEE_QT(pers_j)
!                END IF
!            END IF
        END IF
        !Add all the individual results to a vector that aggregates results over all individuals
        DO j=1,15
            IF (INT(100.D0*CONTRIB_AGGREGA(j)).NE.0) THEN
                CONTRIB_RESULT_COMPARE(j,1)=CONTRIB_RESULT_COMPARE(j,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_AGGREGA(j)
                CONTRIB_RESULT_COMPARE(j,2)=CONTRIB_RESULT_COMPARE(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
        DO j=16,17
            IF (CONTRIB_AGGREGA(j).NE.0.D0) THEN
                CONTRIB_RESULT_COMPARE(j,1)=CONTRIB_RESULT_COMPARE(j,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_AGGREGA(j)
                CONTRIB_RESULT_COMPARE(j,2)=CONTRIB_RESULT_COMPARE(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO
    
    !Inflate contributions to annual values
    !Factor 3.60 based on SummaryOfAggregates.xls (salary mass RSZ and RSZPPO in fourth quarter as fraction of annual salary mass in sample)
    CONTRIB_RESULT_COMPARE(1:15,1)=(CONTRIB_RESULT_COMPARE(1:15,1)*4.D0)/1000.D0
    CONTRIB_EXTERN(1:15)=CONTRIB_EXTERN(1:15)*1000.D0

    !Compute columns 3 to 6 of the vector with comparable aggregates
    DO i=1,SIZE(CONTRIB_RESULT_COMPARE,1)
        !difference between real value and estimate
        CONTRIB_RESULT_COMPARE(i,3)= CONTRIB_RESULT_COMPARE(i,1) - CONTRIB_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(CONTRIB_EXTERN(i)).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,4)= 100.D0*(CONTRIB_RESULT_COMPARE(i,3)/CONTRIB_EXTERN(i))    
        END IF
        !Average amount
        IF (INT(CONTRIB_RESULT_COMPARE(i,2)).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,5)=CONTRIB_RESULT_COMPARE(i,1)/CONTRIB_RESULT_COMPARE(i,2)
        END IF
        !Fraction of population with non zero value
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,6)=100.D0*CONTRIB_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''RSZ bijdragenverminderingen op jaarbasis naar verminderingspost'')')
    Write(15,'(''Deze tabel bevat in de kolom steekproef de bijdrageverminderingen die gesimuleerd werden'')')
    Write(15,'(''voor loon en weddetrekkenden in de priv en alle niet statuaire ambtenaren'')')
    Write(15,'(''Nominale bedragen per 1.000 Euro in prijzen van 2008'')')
    Write(15,'(''                                                                                                         &
    &   Externe bron          Steekproef   Aantal individuen  Absoluut verschil  %Verschil     Gemid      %Pop'')')
    Write(15,'(''                                                                                                         &
    &             C1                  C2                                  C2-C1  100*C2/C1 per indiv'')')
    CALL DRAWLINE(15,210,'-')    
    DO i=1,SIZE(CONTRIB_RESULT_COMPARE,1)
        Write(15,'(a100,F20.2,F20.2,F20.0,F20.0,F10.2,F10.2,F10.2)')CONTRIB_LAB_COMPARE(i),CONTRIB_EXTERN(i),&
        &(CONTRIB_RESULT_COMPARE(i,j),j=1,6)
    END DO
    Write(15,'(100x,F20.2,F20.2)')SUM(CONTRIB_EXTERN(1:15)),&
    &SUM(CONTRIB_RESULT_COMPARE(1:15,1))

    DEALLOCATE(CONTRIB_AGGREGA)      
    DEALLOCATE(CONTRIB_RESULT_COMPARE)
    DEALLOCATE(CONTRIB_LAB_COMPARE)     
    DEALLOCATE(CONTRIB_EXTERN)        

END SUBROUTINE CONTRIB_COMPARE

!============= SUBROUTINE CONTRIB_GLOBBUD ======================================!
!This subroutine computes the global budget results and writes the results to
!the global budget file
!============================================================================!

SUBROUTINE CONTRIB_GLOBBUD

    INTEGER                                                             :: pers_j,i,j
    INTEGER                                                             :: NumAnal                              !Number of summary statistics computed 
    !variables for computation of contribution of wage earners on the private labour market
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_WEARCOM_EMEE_QT,2)+1)    :: CONTRIB_WEARCOM_EMEE_NET         !Employee contributions of wage earners on the private labour market net of deductions
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_WEARCOM_EMER_QT,2)+1)    :: CONTRIB_WEARCOM_EMER_NET         !Employer contributions of wage earners on the private labour market net of deductions
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMEE_SUMCON      !Total sum of employee contributions of wage earners on the private labour market before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMER_SUMCON      !Total sum of employer contributions of wage earners on the private labour market before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMEE_SUMDED      !Total sum of employee deductions of wage earners on the private labour market before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_WEARCOM_EMER_SUMDED      !Total sum of employer deductions of wage earners on the private labour market   
    !variables for computation of contribution of wage earners on the public labour market without statutory service
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2)+1)   :: CONTRIB_PUBNSCOM_EMEE_NET        !Employee contributions of wage earners on the public labour market without statutory service net of deductions
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBNSCOM_EMER_QT,2)+1)   :: CONTRIB_PUBNSCOM_EMER_NET        !Employer contributions of wage earners on the public labour market without statutory service net of deductions
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBNSCOM_EMEE_SUMCON     !Total sum of employee contributions of wage earners on the public labour market without statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBNSCOM_EMER_SUMCON     !Total sum of employer contributions of wage earners on the public labour market without statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBNSCOM_EMEE_SUMDED     !Total sum of employee deductions of wage earners on the public labour market without statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBNSCOM_EMER_SUMDED     !Total sum of employer deductions of wage earners on the public labour market without statutory service 
    !variables for computation of contribution of wage earners on the public labour market with statutory service
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBSCOM_EMEE_QT,2)+1)    :: CONTRIB_PUBSCOM_EMEE_NET         !Employee contributions of wage earners on the public labour market with statutory service net of deductions
    REAL(KIND=DPRECI),DIMENSION(1:SIZE(CONTRIB_PUBSCOM_EMER_QT,2))      :: CONTRIB_PUBSCOM_EMER_NET         !Employer contributions of wage earners on the public labour market with statutory service net of deductions
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBSCOM_EMEE_SUMCON      !Total sum of employee contributions of wage earners on the public labour market with statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBSCOM_EMER_SUMCON      !Total sum of employer contributions of wage earners on the public labour market with statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBSCOM_EMEE_SUMDED      !Total sum of employee deductions of wage earners on the public labour market with statutory service before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_PUBSCOM_EMER_SUMDED      !Total sum of employer deductions of wage earners on the public labour market with statutory service 
    !variables for computation of contribution of self employed
    REAL(KIND=DPRECI)                                                   :: CONTRIB_SELF_EMER_NET            !Employer contributions of self employed net of deductions
    REAL(KIND=DPRECI)                                                   :: CONTRIB_SELF_EMER_SUMCON         !Total sum of employer contributions of self employed before deductions  
    REAL(KIND=DPRECI)                                                   :: CONTRIB_SELF_EMER_SUMDED         !Total sum of employer deductions of self employed   
    !variables for computation of aggregates
    REAL(KIND=DPRECI),DIMENSION(1:3)                                    :: CONTRIB_PRIV_FRACWMOD            !Vector with fractions of wage moderation contribution of private labour market wage earners
    REAL(KIND=DPRECI),DIMENSION(1:3)                                    :: CONTRIB_PUB_FRACWMOD             !Vector with fractions of wage moderation contribution of public labour market wage earners
    REAL(KIND=DPRECI)                                                   :: SumW                             !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)                          :: CONTRIB_AGGREGA                  !Vector with aggregates of unem data comparable with other sources computed for 1 individual
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)                        :: CONTRIB_RESULT_COMPARE           !Vector with aggregates of unem data comparable with other sources computed over all individuals
    INTEGER,ALLOCATABLE,DIMENSION(:)                                    :: ClassifyGlob                     !Identification number saying in which line of the global budget table the amount must come

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))

    !Define fractions of wage moderation contribution
    CONTRIB_PRIV_FRACWMOD(1) = 5.67
    CONTRIB_PRIV_FRACWMOD(2) = 1.41
    CONTRIB_PRIV_FRACWMOD(3) = 0.4
    CONTRIB_PUB_FRACWMOD(1) = 5.67
    CONTRIB_PUB_FRACWMOD(2) = 1.41
    CONTRIB_PUB_FRACWMOD(3) = 0.4
    
    !compute unemployment benefits
    NumAnal = 4
    ALLOCATE(CONTRIB_AGGREGA(NumAnal))       
    ALLOCATE(CONTRIB_RESULT_COMPARE(NumAnal,4))
    ALLOCATE(ClassifyGlob(NumAnal))       

    !Initialise results
    CONTRIB_RESULT_COMPARE(1:SIZE(CONTRIB_RESULT_COMPARE,1),1:4) = 0.D0

    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !Make vector with local aggregates empty
        CONTRIB_AGGREGA(1:SIZE(CONTRIB_AGGREGA,1)) = 0.D0
        !Computations for wage earners on the private labour market
        IF (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            !Make store variables empty
            CONTRIB_WEARCOM_EMEE_NET(1:SIZE(CONTRIB_WEARCOM_EMEE_NET,1)) = 0.D0
            CONTRIB_WEARCOM_EMER_NET(1:SIZE(CONTRIB_WEARCOM_EMER_NET,1)) = 0.D0
            CONTRIB_WEARCOM_EMEE_SUMCON = 0.D0
            CONTRIB_WEARCOM_EMER_SUMCON = 0.D0
            CONTRIB_WEARCOM_EMEE_SUMDED = 0.D0
            CONTRIB_WEARCOM_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_WEARCOM_EMEE_SUMCON =   SUM(CONTRIB_WEARCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_EMEE_QT,2))) + &
                                            &CONTRIB_WEARHOLD_EMEE_QT(pers_j)
            CONTRIB_WEARCOM_EMER_SUMCON =   SUM(CONTRIB_WEARCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARCOM_EMER_QT,2))) + &
                                            &CONTRIB_WEARHOLA_EMER_QT(pers_j)
            CONTRIB_WEARCOM_EMEE_SUMDED =   CONTRIB_WEARDEDU_EMEE_QT(pers_j)
            CONTRIB_WEARCOM_EMER_SUMDED =   SUM(CONTRIB_WEARDEDU_EMER_QT(pers_j,1:SIZE(CONTRIB_WEARDEDU_EMER_QT,2)))
            !Compute the employee contributions net of deductions
            IF (INT(100.D0*CONTRIB_WEARCOM_EMEE_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_WEARCOM_EMEE_QT,2)
                    CONTRIB_WEARCOM_EMEE_NET(i) = CONTRIB_WEARCOM_EMEE_QT(pers_j,i) - &
                    &CONTRIB_WEARCOM_EMEE_SUMDED*(CONTRIB_WEARCOM_EMEE_QT(pers_j,i)/CONTRIB_WEARCOM_EMEE_SUMCON)        
                END DO
                !contributions on double holiday earnings
                CONTRIB_WEARCOM_EMEE_NET(SIZE(CONTRIB_WEARCOM_EMEE_NET,1)) = CONTRIB_WEARHOLD_EMEE_QT(pers_j) - &
                &CONTRIB_WEARCOM_EMEE_SUMDED*(CONTRIB_WEARHOLD_EMEE_QT(pers_j)/CONTRIB_WEARCOM_EMEE_SUMCON)        
            END IF
            !Compute the employer contributions net of deductions
            IF (INT(100.D0*CONTRIB_WEARCOM_EMER_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_WEARCOM_EMER_QT,2)
                    CONTRIB_WEARCOM_EMER_NET(i) = CONTRIB_WEARCOM_EMER_QT(pers_j,i) - &
                    &CONTRIB_WEARCOM_EMER_SUMDED*(CONTRIB_WEARCOM_EMER_QT(pers_j,i)/CONTRIB_WEARCOM_EMER_SUMCON)        
                END DO
                !contributions on holiday earnings of blue collar workers
                CONTRIB_WEARCOM_EMER_NET(SIZE(CONTRIB_WEARCOM_EMER_NET,1)) = CONTRIB_WEARHOLA_EMER_QT(pers_j) - &
                &CONTRIB_WEARCOM_EMER_SUMDED*(CONTRIB_WEARHOLA_EMER_QT(pers_j)/CONTRIB_WEARCOM_EMER_SUMCON)        
            END IF
            !add the contributions to the correct vector element
            CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + SUM(CONTRIB_WEARCOM_EMEE_NET(1:7)) + SUM(CONTRIB_WEARCOM_EMER_NET(1:7))
            CONTRIB_AGGREGA(3) = CONTRIB_AGGREGA(3) + CONTRIB_WEARCOM_EMEE_NET(10) + CONTRIB_WEARCOM_EMER_NET(10)
            !Remark: attribute fractions of the wage moderation contribution to the correct line
            CONTRIB_AGGREGA(2) =    CONTRIB_AGGREGA(2) + (CONTRIB_PRIV_FRACWMOD(1)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                    &(CONTRIB_WEARCOM_EMEE_NET(13) + CONTRIB_WEARCOM_EMER_NET(13))
            CONTRIB_AGGREGA(1) =    CONTRIB_AGGREGA(1) + (CONTRIB_PRIV_FRACWMOD(2)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                    &(CONTRIB_WEARCOM_EMEE_NET(13) + CONTRIB_WEARCOM_EMER_NET(13))
            CONTRIB_AGGREGA(3) =    CONTRIB_AGGREGA(3) + (CONTRIB_PRIV_FRACWMOD(3)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                    &(CONTRIB_WEARCOM_EMEE_NET(13) + CONTRIB_WEARCOM_EMER_NET(13))
            CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + CONTRIB_WEARCOM_EMEE_NET(14) + CONTRIB_WEARCOM_EMER_NET(14)
            CONTRIB_AGGREGA(2) = CONTRIB_AGGREGA(2) + CONTRIB_WEARCOM_EMEE_NET(15) + CONTRIB_WEARCOM_EMER_NET(15)
            !double holiday of employee
            CONTRIB_AGGREGA(3) = CONTRIB_AGGREGA(3) + CONTRIB_WEARCOM_EMEE_NET(22)
            !temporary unemployment and older unemployed
            CONTRIB_AGGREGA(3) = CONTRIB_AGGREGA(3) + CONTRIB_WEARCOM_EMEE_NET(12) + CONTRIB_WEARCOM_EMER_NET(12)
        END IF
        !Computations for wage earners on the public labour market without statutory service
        IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            !Make store variables empty
            CONTRIB_PUBNSCOM_EMEE_NET(1:SIZE(CONTRIB_PUBNSCOM_EMEE_NET,1)) = 0.D0
            CONTRIB_PUBNSCOM_EMER_NET(1:SIZE(CONTRIB_PUBNSCOM_EMER_NET,1)) = 0.D0
            CONTRIB_PUBNSCOM_EMEE_SUMCON = 0.D0
            CONTRIB_PUBNSCOM_EMER_SUMCON = 0.D0
            CONTRIB_PUBNSCOM_EMEE_SUMDED = 0.D0
            CONTRIB_PUBNSCOM_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_PUBNSCOM_EMEE_SUMCON =   SUM(CONTRIB_PUBNSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2))) + &
                                            &CONTRIB_PUBNSHOL_EMEE_QT(pers_j)
            CONTRIB_PUBNSCOM_EMER_SUMCON =   SUM(CONTRIB_PUBNSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBNSCOM_EMER_QT,2))) + &
                                            &CONTRIB_PUBNSHOL_EMER_QT(pers_j)
            CONTRIB_PUBNSCOM_EMEE_SUMDED =   CONTRIB_PUBNSDED_EMEE_QT(pers_j)
            CONTRIB_PUBNSCOM_EMER_SUMDED =   SUM(CONTRIB_PUBNSDED_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBNSDED_EMER_QT,2)))
            !Compute the employee contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBNSCOM_EMEE_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBNSCOM_EMEE_QT,2)
                    CONTRIB_PUBNSCOM_EMEE_NET(i) = CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i) - &
                    &CONTRIB_PUBNSCOM_EMEE_SUMDED*(CONTRIB_PUBNSCOM_EMEE_QT(pers_j,i)/CONTRIB_PUBNSCOM_EMEE_SUMCON)        
                END DO
                !contributions on double holiday earnings
                CONTRIB_PUBNSCOM_EMEE_NET(SIZE(CONTRIB_PUBNSCOM_EMEE_NET,1)) = CONTRIB_PUBNSHOL_EMEE_QT(pers_j) - &
                &CONTRIB_PUBNSCOM_EMEE_SUMDED*(CONTRIB_PUBNSHOL_EMEE_QT(pers_j)/CONTRIB_PUBNSCOM_EMEE_SUMCON)        
            END IF
            !Compute the employer contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBNSCOM_EMER_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBNSCOM_EMER_QT,2)
                    CONTRIB_PUBNSCOM_EMER_NET(i) = CONTRIB_PUBNSCOM_EMER_QT(pers_j,i) - &
                    &CONTRIB_PUBNSCOM_EMER_SUMDED*(CONTRIB_PUBNSCOM_EMER_QT(pers_j,i)/CONTRIB_PUBNSCOM_EMER_SUMCON)        
                END DO
                !contributions on holiday earnings of blue collar workers
                CONTRIB_PUBNSCOM_EMER_NET(SIZE(CONTRIB_PUBNSCOM_EMER_NET,1)) = CONTRIB_PUBNSHOL_EMER_QT(pers_j) - &
                &CONTRIB_PUBNSCOM_EMER_SUMDED*(CONTRIB_PUBNSHOL_EMER_QT(pers_j)/CONTRIB_PUBNSCOM_EMER_SUMCON)        
            END IF
            !add the contributions to the correct vector element
            !Distinguish local from federal civil servants
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + SUM(CONTRIB_PUBNSCOM_EMEE_NET(1:7)) + SUM(CONTRIB_PUBNSCOM_EMER_NET(1:7))
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + SUM(CONTRIB_PUBNSCOM_EMEE_NET(1:7)) + SUM(CONTRIB_PUBNSCOM_EMER_NET(1:7))
            END IF                
            CONTRIB_AGGREGA(3) = CONTRIB_AGGREGA(3) + CONTRIB_PUBNSCOM_EMEE_NET(10) + CONTRIB_PUBNSCOM_EMER_NET(10)
            !Remark: attribute fractions of the wage moderation contribution to the correct line
            CONTRIB_AGGREGA(2) =    CONTRIB_AGGREGA(2) + (CONTRIB_PRIV_FRACWMOD(1)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                    &(CONTRIB_PUBNSCOM_EMEE_NET(13) + CONTRIB_PUBNSCOM_EMER_NET(13))
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) =    CONTRIB_AGGREGA(1) + (CONTRIB_PRIV_FRACWMOD(2)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                        &(CONTRIB_PUBNSCOM_EMEE_NET(13) + CONTRIB_PUBNSCOM_EMER_NET(13))
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                CONTRIB_AGGREGA(1) =    CONTRIB_AGGREGA(1) + (CONTRIB_PRIV_FRACWMOD(2)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                        &(CONTRIB_PUBNSCOM_EMEE_NET(13) + CONTRIB_PUBNSCOM_EMER_NET(13))
            END IF
            CONTRIB_AGGREGA(3) =    CONTRIB_AGGREGA(3) + (CONTRIB_PRIV_FRACWMOD(3)/SUM(CONTRIB_PRIV_FRACWMOD(1:3)))*&
                                    &(CONTRIB_PUBNSCOM_EMEE_NET(13) + CONTRIB_PUBNSCOM_EMER_NET(13))
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + CONTRIB_PUBNSCOM_EMEE_NET(14) + CONTRIB_PUBNSCOM_EMER_NET(14)
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + CONTRIB_PUBNSCOM_EMEE_NET(14) + CONTRIB_PUBNSCOM_EMER_NET(14)
            END IF                
            CONTRIB_AGGREGA(2) = CONTRIB_AGGREGA(2) + CONTRIB_PUBNSCOM_EMEE_NET(15) + CONTRIB_PUBNSCOM_EMER_NET(15)
            !double holiday of employee
            CONTRIB_AGGREGA(3) = CONTRIB_AGGREGA(3) + CONTRIB_PUBNSCOM_EMEE_NET(22)
            !temporary unemployment and older unemployed
            CONTRIB_AGGREGA(3) = CONTRIB_AGGREGA(3) + CONTRIB_PUBNSCOM_EMEE_NET(12) + CONTRIB_PUBNSCOM_EMER_NET(12)
        END IF
        !Computations for wage earners on the public labour market with statutory service
        IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            !Make store variables empty
            CONTRIB_PUBSCOM_EMEE_NET(1:SIZE(CONTRIB_PUBSCOM_EMEE_NET,1)) = 0.D0
            CONTRIB_PUBSCOM_EMER_NET(1:SIZE(CONTRIB_PUBSCOM_EMER_NET,1)) = 0.D0
            CONTRIB_PUBSCOM_EMEE_SUMCON = 0.D0
            CONTRIB_PUBSCOM_EMER_SUMCON = 0.D0
            CONTRIB_PUBSCOM_EMEE_SUMDED = 0.D0
            CONTRIB_PUBSCOM_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_PUBSCOM_EMEE_SUMCON =   SUM(CONTRIB_PUBSCOM_EMEE_QT(pers_j,1:SIZE(CONTRIB_PUBSCOM_EMEE_QT,2))) + &
                                            &CONTRIB_PUBSHOL_EMEE_QT(pers_j)
            CONTRIB_PUBSCOM_EMER_SUMCON =   SUM(CONTRIB_PUBSCOM_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBSCOM_EMER_QT,2))) 
            CONTRIB_PUBSCOM_EMEE_SUMDED =   CONTRIB_PUBSDED_EMEE_QT(pers_j)
            CONTRIB_PUBSCOM_EMER_SUMDED =   SUM(CONTRIB_PUBSDED_EMER_QT(pers_j,1:SIZE(CONTRIB_PUBSDED_EMER_QT,2)))
            !Compute the employee contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBSCOM_EMEE_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBSCOM_EMEE_QT,2)
                    CONTRIB_PUBSCOM_EMEE_NET(i) = CONTRIB_PUBSCOM_EMEE_QT(pers_j,i) - &
                    &CONTRIB_PUBSCOM_EMEE_SUMDED*(CONTRIB_PUBSCOM_EMEE_QT(pers_j,i)/CONTRIB_PUBSCOM_EMEE_SUMCON)        
                END DO
                !contributions on double holiday earnings
                CONTRIB_PUBSCOM_EMEE_NET(SIZE(CONTRIB_PUBSCOM_EMEE_NET,1)) = CONTRIB_PUBSHOL_EMEE_QT(pers_j) - &
                &CONTRIB_PUBSCOM_EMEE_SUMDED*(CONTRIB_PUBSHOL_EMEE_QT(pers_j)/CONTRIB_PUBSCOM_EMEE_SUMCON)        
            END IF
            !Compute the employer contributions net of deductions
            IF (INT(100.D0*CONTRIB_PUBSCOM_EMER_SUMCON).NE.0) THEN
                !Contributions in the matrix
                DO i=1,SIZE(CONTRIB_PUBSCOM_EMER_QT,2)
                    CONTRIB_PUBSCOM_EMER_NET(i) = CONTRIB_PUBSCOM_EMER_QT(pers_j,i) - &
                    &CONTRIB_PUBSCOM_EMER_SUMDED*(CONTRIB_PUBSCOM_EMER_QT(pers_j,i)/CONTRIB_PUBSCOM_EMER_SUMCON)        
                END DO
            END IF
            !add the contributions to the correct vector element
            !Distinguish local from federal civil servants
            !1 = local
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + CONTRIB_PUBSCOM_EMEE_NET(2) + CONTRIB_PUBSCOM_EMER_NET(2)
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                CONTRIB_AGGREGA(1) = CONTRIB_AGGREGA(1) + CONTRIB_PUBSCOM_EMEE_NET(2) + CONTRIB_PUBSCOM_EMER_NET(2)
            END IF  
            !Remark: attribute fractions of the wage moderation contribution to the correct line
            CONTRIB_AGGREGA(2) =    CONTRIB_AGGREGA(2) + (CONTRIB_PUB_FRACWMOD(1)/SUM(CONTRIB_PUB_FRACWMOD(1:3)))*&
                                    &(CONTRIB_PUBSCOM_EMEE_NET(5) + CONTRIB_PUBSCOM_EMER_NET(5))
            IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.2) THEN
                CONTRIB_AGGREGA(1) =    CONTRIB_AGGREGA(1) + (CONTRIB_PUB_FRACWMOD(2)/SUM(CONTRIB_PUB_FRACWMOD(1:3)))*&
                                        &(CONTRIB_PUBSCOM_EMEE_NET(5) + CONTRIB_PUBSCOM_EMER_NET(5))
            ELSE IF (CONTRIB_CSLOCFED_QT(pers_j).EQ.1) THEN
                CONTRIB_AGGREGA(1) =    CONTRIB_AGGREGA(1) + (CONTRIB_PUB_FRACWMOD(2)/SUM(CONTRIB_PUB_FRACWMOD(1:3)))*&
                                        &(CONTRIB_PUBSCOM_EMEE_NET(5) + CONTRIB_PUBSCOM_EMER_NET(5))
            END IF
            CONTRIB_AGGREGA(3) =    CONTRIB_AGGREGA(3) + (CONTRIB_PUB_FRACWMOD(3)/SUM(CONTRIB_PUB_FRACWMOD(1:3)))*&
                                    &(CONTRIB_PUBSCOM_EMEE_NET(5) + CONTRIB_PUBSCOM_EMER_NET(5))
            !double holiday of employee
            CONTRIB_AGGREGA(3) = CONTRIB_AGGREGA(3) + CONTRIB_PUBSCOM_EMEE_NET(7)
        END IF
        !Computations for self employed
        IF (ANY(CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.(/1/))) THEN
            !Make store variables empty
            CONTRIB_SELF_EMER_NET = 0.D0
            CONTRIB_SELF_EMER_SUMCON = 0.D0
            CONTRIB_SELF_EMER_SUMDED = 0.D0
            !Compute the sum of all contributions and the sum of all deductions  
            CONTRIB_SELF_EMER_SUMCON = CONTRIB_SELFCOM_EMER_QT(pers_j)
            CONTRIB_SELF_EMER_SUMDED = CONTRIB_SELFDEDU_EMER_QT(pers_j)
            !Compute the employer contributions net of deductions
            CONTRIB_SELF_EMER_NET = CONTRIB_SELF_EMER_SUMCON - CONTRIB_SELF_EMER_SUMDED
            !add the contributions to the correct vector element
            CONTRIB_AGGREGA(4) = CONTRIB_AGGREGA(4) + CONTRIB_SELF_EMER_NET
        END IF
        !Add contributions paid on social benefits
        CONTRIB_AGGREGA(3) = CONTRIB_AGGREGA(3) + CONTRIB_SSDISAB_QT(pers_j) + CONTRIB_SSEARBEN_QT(pers_j)
        CONTRIB_AGGREGA(3) = CONTRIB_AGGREGA(3) + CONTRIB_SSPEBASI_QT(pers_j) + CONTRIB_SSPESOLI_QT(pers_j)
        CONTRIB_AGGREGA(3) = CONTRIB_AGGREGA(3) + CONTRIB_SSINDOCC_QT(pers_j)
        !Add all the individual results to a vector that aggregates results over all individuals
        DO j=1,SIZE(CONTRIB_AGGREGA,1)
            IF (INT(100.D0*CONTRIB_AGGREGA(j)).NE.0) THEN
                CONTRIB_RESULT_COMPARE(j,1)=CONTRIB_RESULT_COMPARE(j,1)+MIMOSIS_WEIGHT(pers_j)*CONTRIB_AGGREGA(j)
                CONTRIB_RESULT_COMPARE(j,2)=CONTRIB_RESULT_COMPARE(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    !Inflate contributions to annual values expressed in 1000 Euro's (=format of global budget)
    CONTRIB_RESULT_COMPARE(1:3,1)=(CONTRIB_RESULT_COMPARE(1:3,1)*4.D0)/&
    &1000.D0
    CONTRIB_RESULT_COMPARE(4,1)=(CONTRIB_RESULT_COMPARE(4,1)*4.D0)/&
    &1000.D0

    !Compute columns 3 to 4 of the vector with comparable aggregates
    DO i=1,SIZE(CONTRIB_RESULT_COMPARE,1)
        !Average amount
        IF (INT(CONTRIB_RESULT_COMPARE(i,2)).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,3)=CONTRIB_RESULT_COMPARE(i,1)/CONTRIB_RESULT_COMPARE(i,2)
        END IF
        !Fraction of population with non zero value
        IF (INT(SumW).NE.0) THEN
            CONTRIB_RESULT_COMPARE(i,4)=100.D0*CONTRIB_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Open output files
    OPEN(unit=16,FILE=GLOBBUD_EMPL,POSITION='APPEND',ACTION='WRITE')
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') GLOBBUD_EMPL
        WRITE(15,'("Could not find ", a100)') GLOBBUD_EMPL
        PAUSE
        STOP
    END IF

    OPEN(unit=17,FILE=GLOBBUD_SELFEMPL,POSITION='APPEND',ACTION='WRITE')    
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') GLOBBUD_SELFEMPL
        WRITE(15,'("Could not find ", a100)') GLOBBUD_SELFEMPL
        PAUSE
        STOP
    END IF

    !Determine in which line of the global budget the computed result must be stored
    ClassifyGlob(1) = 1
    ClassifyGlob(2) = 2
    ClassifyGlob(3) = 3
    ClassifyGlob(4) = 1

    write(16,'(''Annual sum of common social security contributions of employees                                     '',&
    &F20.2,F20.2,F10.2,F10.1,i10)')(CONTRIB_RESULT_COMPARE(1,j),j=1,4),ClassifyGlob(1)
    write(16,'(''Annual sum of wage moderation social security contributions                                         '',&
    &F20.2,F20.2,F10.2,F10.1,i10)')(CONTRIB_RESULT_COMPARE(2,j),j=1,4),ClassifyGlob(2)
    write(16,'(''Annual sum of special social security contributions of employees                                    '',&
    &F20.2,F20.2,F10.2,F10.1,i10)')(CONTRIB_RESULT_COMPARE(3,j),j=1,4),ClassifyGlob(3)
    write(17,'(''Annual sum of social security contributions of self employed                                        '',&
    &F20.2,F20.2,F10.2,F10.1,i10)')(CONTRIB_RESULT_COMPARE(4,j),j=1,4),ClassifyGlob(4)

    CLOSE(UNIT=16)
    CLOSE(UNIT=17)    

    DEALLOCATE(CONTRIB_AGGREGA)      
    DEALLOCATE(CONTRIB_RESULT_COMPARE)
    DEALLOCATE(ClassifyGlob)    

END SUBROUTINE CONTRIB_GLOBBUD

!============= SUBROUTINE OUTPUT_FILEWRIT ==============!
!This subroutine open output file and write output      !
!=======================================================!

SUBROUTINE CONTRIB_OUTPUT_FILEWRIT

    INTEGER :: pers_j
    
    OPEN(UNIT=12, FILE=PIT_CONTRIB,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PIT_CONTRIB
        WRITE(15,'("Could not find ", a100)') PIT_CONTRIB
        PAUSE
        STOP
    END IF        
    OPEN(UNIT=13, FILE=EVAL_CONTRIB,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_CONTRIB
        WRITE(15,'("Could not find ", a100)') EVAL_CONTRIB
        PAUSE
        STOP
    END IF        
    OPEN(UNIT=14, FILE=SOCBEN_CONTRIB,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_CONTRIB
        WRITE(15,'("Could not find ", a100)') SOCBEN_CONTRIB
        PAUSE
        STOP
    END IF        

    DO pers_j=1,n_rec
        WRITE(12,'(A14,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
        &F15.2,F15.2,F15.2,I10,I10,I10)') &
        &GECODEERD_INSZ_CHAR(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
        &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
        &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
        &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
        &CONTRIB_GTIFAMAL_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
        &CONTRIB_SELFPCR_QT(pers_j)
        WRITE(13,'(A14,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
        &F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2)') &
        &GECODEERD_INSZ_CHAR(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
        &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
        &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
        &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
        &CONTRIB_GTIFAMAL_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&
        &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_SSPRIV_EMER_QT(pers_j),CONTRIB_SSPUB_EMER_QT(pers_j),&
        &CONTRIB_SSREPLAC_QT(pers_j),CONTRIB_SELFEXTR_QT(pers_j)
        WRITE(14,'(A14,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
        &F15.2,F15.2,F15.2)') &
        &GECODEERD_INSZ_CHAR(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
        &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
        &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
        &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
        &CONTRIB_GTIFAMAL_QT(pers_j)
    END DO

    CLOSE(UNIT=12)
    CLOSE(UNIT=13)    
    CLOSE(UNIT=14)    

    WRITE(*,*) 'Subroutine CONTRIB_OUTPUT_FILEWRIT of NORMAL RUN......runned correctly'

END SUBROUTINE CONTRIB_OUTPUT_FILEWRIT

!=================================================================================!
!Routine to write CONTRIB variables for pre run
!=================================================================================!

SUBROUTINE WRITEPRE_CONTRIB

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\CONTRIB_PRERUN.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec                                                                                
        WRITE(12,'(A14,3I10,6F15.2,I10)') GECODEERD_INSZ_CHAR(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),&
        &CONTRIB_LABMSTAT_PUB_QT(pers_j),CONTRIB_LABMSTAT_SELF_QT(pers_j),CONTRIB_REVPRIV_QT(pers_j),&
        &CONTRIB_REVPUB_QT(pers_j),CONTRIB_INCSELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),&
        &CONTRIB_SSPUB_EMEE_QT(pers_j),CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_SELFPCR_QT(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEPRE_CONTRIB...........................runned correctly'

END SUBROUTINE WRITEPRE_CONTRIB

!=================================================================================!
!Routine to write CONTRIB variables for the labsupply data set
!=================================================================================!

SUBROUTINE WRITELAB_CONTRIB

    INTEGER :: pers_j
    REAL(KIND=DPRECI)                                                   :: INCPRIV,INCPUB  
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\CONTRIB_LABSUPPLY.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec                                                                                
        !Construct labour market income of wage earner on the private labour market that corresponds to real hours worked
        INCPRIV = 0
        IF  (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            INCPRIV = SAL100_QT(pers_j)+SALATT_QT(pers_j)+SALFOR_QT(pers_j)
        END IF

        !Construct labour market income of wage earner on the public labour market that corresponds to real hours worked
        INCPUB = 0
        IF  (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2, 3, 4/))) THEN
            IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).NE.1.AND.CONTRIB_LABMSTAT_PRIV_QT(pers_j).NE.2) THEN
                INCPUB = SAL100_QT(pers_j) + SALATT_QT(pers_j) + SALFOR_QT(pers_j)
            END IF
            INCPUB = INCPUB + LMASAMT_QT(pers_j)
        END IF

        WRITE(12,'(A14,3I10,5F15.2)') GECODEERD_INSZ_CHAR(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),&
        &CONTRIB_LABMSTAT_PUB_QT(pers_j),CONTRIB_LABMSTAT_SELF_QT(pers_j),INCPRIV,&
        &INCPUB,CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),CONTRIB_INCSELF_QT(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEPRE_CONTRIB...........................runned correctly'

END SUBROUTINE WRITELAB_CONTRIB

!===================================================================================!
!With subroutine WRITE_ESSPROS we write out the data required for part of the
!computation of the requested ESSPROS data for EUROSTAT
!===================================================================================!

SUBROUTINE WRITE_ESSPROS

!    !SOCIAL SECURITY CONTRIBUTIONS PAID ON RETIREMENT AND SURVIVAL PENSIONS
!    !CONTRIB_SSPEBASI_QT      The basic social security contributions paid on pension benefits  
!        CONTRIB_PENS_QT(pers_j)
!        CONTRIB_SSPEBASI_QT(pers_j) 
!    !CONTRIB_SSPESOLI_QT      Solidarity contributions paid on pension benefits 
!        CONTRIB_PENS_QT(pers_j)
!        CONTRIB_SSPESOLI_QT(pers_j)
!
!    !SOCIAL SECURITY CONTRIBUTIONS PAID ON CONVENTIONAL EARLY RETIREMENT BENEFITS
!    !CONTRIB_SSEARBEN_QT      Social security contributions paid on early retirement benefits         
!        CONTRIB_UNEMFT_QT(pers_j)+CONTRIB_UNEMPT_QT(pers_j)
!        CONTRIB_SSEARBEN_QT(pers_j) 
!         
!    !SOCIAL SECURITY CONTRIBUTIONS PAID ON DISABILITY BENEFITS
!    !CONTRIB_SSDISAB_QT       Social security contributions paid on disability benefits           
!        CONTRIB_DISABWEA_QT(pers_j)
!        CONTRIB_SSDISAB_QT(pers_j)
!
!    !SOCIAL SECURITY CONTRIBUTIONS PAID ON BENEFITS IN CASE OF INDUSTRIAL ACCIDENTS OR OCCUPATIONAL DISEASES
!    !CONTRIB_SSINDOCC_QT        Social security contributions paid on benefits for industrial accidents or occupational diseases
!        CONTRIB_INDOCC_QT(pers_j)
!        CONTRIB_SSINDOCC_QT(pers_j) 

    INTEGER :: pers_j
    REAL(KIND=DPRECI)                                                   :: INCEAR  
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\CONTRIB_ESSPROS.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec                                                                                
        !Construct early retirement benefits that serve as basis for the computation of contributions
        INCEAR = CONTRIB_UNEMFT_QT(pers_j)+CONTRIB_UNEMPT_QT(pers_j)

        WRITE(12,'(A14,10F15.2)') GECODEERD_INSZ_CHAR(pers_j),CONTRIB_PENS_QT(pers_j),CONTRIB_SSPEBASI_QT(pers_j),&
        &CONTRIB_PENS_QT(pers_j),CONTRIB_SSPESOLI_QT(pers_j),INCEAR,CONTRIB_SSEARBEN_QT(pers_j),CONTRIB_DISABWEA_QT(pers_j),&
        &CONTRIB_SSDISAB_QT(pers_j),CONTRIB_INDOCC_QT(pers_j),CONTRIB_SSINDOCC_QT(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITE_ESSPROS..............................runned correctly'

END SUBROUTINE WRITE_ESSPROS

!===================================================================================!
!Write employer deductions
!===================================================================================!

SUBROUTINE WRITE_EMERDEDU

    INTEGER :: pers_j
    REAL(KIND=DPRECI)                                                   :: INCEAR  
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\CONTRIB_EMERDEDU.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec                                                                                
        WRITE(12,'(A14,21F15.2,21F15.2,7F15.2,F15.2)') GECODEERD_INSZ_CHAR(pers_j),CONTRIB_WEARDEDU_EMER_QT(pers_j,1:21),&
        &CONTRIB_PUBNSDED_EMER_QT(pers_j,1:21),CONTRIB_PUBSDED_EMER_QT(pers_j,1:7),CONTRIB_SELFDEDU_EMER_QT(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITE_CONTRIB_EMERDEDU........................runned correctly'

END SUBROUTINE WRITE_EMERDEDU

!==========================================================!
!The subroutine CONTRIB_CALC is used to call the other     ! 
!subroutines and to carry out the necessary computations   ! 
!==========================================================!

SUBROUTINE CONTRIB_CALC

!Program heading

!=============================================================================
    WRITE (*,*) ' '
    WRITE (*,*) 'MIMOSIS CONTRIB IS RUNNING '
    WRITE (*,*) ' '
!=============================================================================

    !open log file
    OPEN(UNIT=15, FILE=CONTRIB_LOG, STATUS='REPLACE', ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_LOG
        WRITE(15,'("Could not find ", a100)') CONTRIB_LOG
        PAUSE
        STOP
    END IF

    !Allocate all exogenous vectors with dimension n_rec
    CALL CONTRIB_ALLOC_EXOG(n_rec)
    
    !Reads in PARAMETER file
    CALL CONTRIB_PARAMETER_FILEREAD
    !Read in files with input data
    CALL CONTRIB_INPUT_FILEREAD
    CALL CONTRIB_CONSTRUCT_FILEREAD
    CALL CONTRIB_PENSWELF_FILEREAD
    CALL CONTRIB_UNEM_FILEREAD
    CALL CONTRIB_FAMAL_FILEREAD
    CALL CONTRIB_SICK_FILEREAD
    CALL CONTRIB_PIT_FILEREAD
    CALL CONTRIB_FAMREL_FILEREAD
    CALL CONTRIB_WAGE_FILEREAD
    CALL CONTRIB_WEIGHT_FILEREAD

    WRITE(15,'(/,''Print of function calls for which no selection was carried out'')')
    CALL DRAWLINE(15,100,'=')                

    !Allocate all endogenous vectors with dimension n_rec
    CALL CONTRIB_ALLOC_ENDO(n_rec)
    !Initialise endogenous variables
    CALL CONTRIB_INITENDO(n_rec)
    
    !Rank and Sort all variables read in
    CALL CONTRIB_RANKANDSORT

    !Recode missing values
    CALL CONTRIB_RECODE
    !Inflate nominal values
    CALL CONTRIB_INFLATE
    !Determine household size  
    CALL CONTRIB_HOUSEHOLD_SIZE
    !Determine age of all individuals
    CALL CONTRIB_AGE

    !Construct variables necessary to run the different Contribution computation subroutines 
    CALL CONTRIB_LABMSTAT  !labour market status
    CALL CONTRIB_WORKHOUR_CONSTRUC !Construct variable with assumed hours worked during the week
    CALL CONTRIB_LABMINC   !labour market income
    IF (CONTRIB_SIMUNEM.EQ.1) THEN
        CALL CONTRIB_UNEMINC
    END IF        
    CALL CONTRIB_NONLAINC  !non labour market income
    CALL CONTRIB_CHARAC    !other variables
    
    !Loop over all households  
    DO hh_i=1,N_HH_CONTRIB
        !Loop over all persons within the household
        DO pers_j=CONTRIB_START_HH(hh_i),CONTRIB_END_HH(hh_i)
            !compute social security contributions of private labour market wage earners
            IF (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
                CALL CONTRIB_SSBEFDED_PRIVATE(pers_j) 
                IF (SIMYEAR.LT.2004) THEN
                    CALL CONTRIB_SSDEDU_PRIVATE_2001(pers_j)
                ELSE
                    CALL CONTRIB_SSDEDU_PRIVATE_2004(pers_j)
                END IF
                CALL CONTRIB_SSNET_PRIVATE(pers_j)
            END IF
            !compute social security contributions of public labour market wage earners without statutory service
            IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
                CALL CONTRIB_SSBEFDED_PUBNSTAT(pers_j) 
                IF (SIMYEAR.LT.2004) THEN
                    CALL CONTRIB_SSDEDU_PUBNSTAT_2001(pers_j)
                ELSE
                    CALL CONTRIB_SSDEDU_PUBNSTAT_2004(pers_j)
                END IF
                CALL CONTRIB_SSNET_PUBNSTAT(pers_j)
            END IF
            !compute social security contributions of public labour market wage earners with statutory service
            IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
                CALL CONTRIB_SSBEFDED_PUBSTAT(pers_j)
                CALL CONTRIB_SSDEDU_PUBSTAT(pers_j)
                CALL CONTRIB_SSNET_PUBSTAT(pers_j)
            END IF
            !compute social security contributions of self employed
            IF (ANY(CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.(/1/))) THEN
                CALL CONTRIB_SSBEFDED_SELF(pers_j,hh_i)
                CALL CONTRIB_SSDEDU_SELF(pers_j)
                CALL CONTRIB_SSNET_SELF(pers_j)
            END IF
            !Compute social security contributions on social benefits
            IF (CONTRIB_PENS_QT(pers_j).GT.0.OR.&
            &   CONTRIB_UNEMFT_QT(pers_j).GT.0.OR.&
            &   CONTRIB_UNEMPT_QT(pers_j).GT.0.OR.&
            &   CONTRIB_DISABWEA_QT(pers_j).GT.0.OR.&
            &   CONTRIB_INDOCC_QT(pers_j).GT.0) THEN
                CALL CONTRIB_SSBEFDED_NONLAB(pers_j)
            END IF                
        END DO        
    END DO  

    !Construct exchange concepts
    CALL CONTRIB_EXCHANGE

    !test construction of different contriubtion concepts
    CALL CONTRIB_TEST
    CALL CONTRIB_COMPARE

    !Call routine to create global budget concepts
    CALL CONTRIB_GLOBBUD

    !Write output to output file
    CALL CONTRIB_OUTPUT_FILEWRIT
    !Write file with CONTRIB variables for pre run
!    CALL WRITEPRE_CONTRIB
!    CALL WRITELAB_CONTRIB
!    CALL WRITE_ESSPROS
!    CALL WRITE_EMERDEDU
    
    !Deallocate allocated variables
    CALL CONTRIB_DEALLOC
    CLOSE(15)

END SUBROUTINE CONTRIB_CALC

END MODULE CONTRIB_SUBROUTINES
