!===================================================================!
!  Record of revisions:                                             !
!                                                                   !
!    Date      Programmer               Description of changes      !
!  ========    ====================     ======================      !
!  01/03/06    Consortium AG/01/086     Original code               !
!  01/10/11    FPS Social Security      Revision                    !
!===================================================================!

!==========================================================!
! This module declare all variables used in the program    !
!  - Parameters                                            !
!  - Exogenous Variables                                   !
!  - Endogeneous Variables                                 !
!==========================================================!

MODULE PENSWELF_PRERUN_DATA

    !========================================================================!
    ! Database specific parameters:                                          !
    ! number of lines in parameters file                                     !
    INTEGER, PARAMETER::n_par=67                                             !
    ! year of database                                                       !
    INTEGER, PARAMETER::MIMOSIS_YEAR=2008                                    !
    !(first quarter=1, second quarter=4, third quarter=9, fourth quarter=12) !
    INTEGER, PARAMETER::month=12                                             !
    !Define kind of Integer*4 in a portable way                              !                       
    INTEGER, PARAMETER :: INT4 = SELECTED_INT_KIND(18)                       !                       
    !Define Double precision, i.e. REAL*8 in a portable way                  !
    INTEGER, PARAMETER :: DPRECI = SELECTED_REAL_KIND(15,307)                !
    !Define Single precision, i.e. REAL*4 in a portable way                  !
    INTEGER, PARAMETER :: SPRECI = SELECTED_REAL_KIND(6,37)                  !
    !Define parameter that stores the precision value used for large vectors !
    INTEGER, PARAMETER :: KIND_REAL = SPRECI                                 !   
    !========================================================================!

    !==========================================================================
    !Define variables to read locations and input parameters
    !Variables for locating input 
    CHARACTER(len=150)                                      :: PENSWELF_INPUT_PATH                  !Default path for all input files
    CHARACTER(len=150)                                      :: PENSWELF_INPUT_FILE                  !Default name of input file (i.e. INPUT from NATIONAL REGISTER)
    CHARACTER(len=150)                                      :: PENSWELF_INPUT                       !Path+Name of INPUT file
    CHARACTER(len=150)                                      :: PENSWELF_CONSTRUCT_FILE              !Default name of SICK file (i.e. the OUTPUT FILE of the SICK module)
    CHARACTER(len=150)                                      :: PENSWELF_CONSTRUCT                   !Path+Name of SICK file
    CHARACTER(len=150)                                      :: PENSWELF_SICK_FILE                   !Default name of SICK file (i.e. the OUTPUT FILE of the SICK module)
    CHARACTER(len=150)                                      :: PENSWELF_SICK                        !Path+Name of SICK file
    CHARACTER(len=150)                                      :: PENSWELF_UNEM_FILE                   !Default name of UNEM file (i.e. the OUTPUT FILE of the UNEM module)
    CHARACTER(len=150)                                      :: PENSWELF_UNEM                        !Path+Name of UNEM file
    CHARACTER(len=150)                                      :: PENSWELF_CONTRIB_FILE                !Default name of CONTRIB file (i.e. the OUTPUT FILE of the CONTRIB module)
    CHARACTER(len=150)                                      :: PENSWELF_CONTRIB                     !Path+Name of CONTRIB file
    CHARACTER(len=150)                                      :: FAMREL_INPUT_PATH                    !Path name where file with FAMREL input data is stored
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_FILE                  !File name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL                       !Internal name to store Path+Name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: CONSTANT_INPUT_PATH                  !Path name where file with MIMOSIS Weights is stored
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG_FILE                !File name of file with MIMOSIS Weights                    
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG                     !Internal name to store Path+Name of file with MIMOSIS Weights
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT_FILE                  !File name of file with MIMOSIS gross wages
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT                       !Internal name to store Path+Name of file with MIMOSIS gross wages
    !Variables for locating output (local variables)
    CHARACTER(len=150)                                      :: CONTRIB_PENSWELF_FILE                !Name of the pre run file to be created for the CONTRIB module
    CHARACTER(len=150)                                      :: CONTRIB_PENSWELF                     !Path+Name of output for CONTRIB module   
    CHARACTER(len=150)                                      :: SICK_PENSWELF_FILE                   !Name of the pre run file to be created for the SICK module
    CHARACTER(len=150)                                      :: SICK_PENSWELF                        !Path+Name of output for SICK module
    CHARACTER(len=150)                                      :: UNEM_PENSWELF_FILE                   !Name of the pre run file to be created for the UNEM module 
    CHARACTER(len=150)                                      :: UNEM_PENSWELF                        !Path+Name of output for UNEM module 
    CHARACTER(len=150)                                      :: FAMAL_PENSWELF_FILE                  !Name of the pre run file to be created for the FAMAL module 
    CHARACTER(len=150)                                      :: FAMAL_PENSWELF                       !Path+Name of output for FAMAL module 
    !Tools to read input, output and parameters    
    CHARACTER(len=20)                                       :: PENSWELF_PAR_FORMAT                  !Format of parameter read in from parameter file
    CHARACTER(len=1)                                        :: PENSWELF_PAR_BREAK='@'               !Character used to identify end of string
    CHARACTER(len=10)                                       :: PENSWELF_PAR_CURR                    !Currency of the parameter read in from the parameter file
    CHARACTER(len=40),ALLOCATABLE,DIMENSION(:)              :: PENSWELF_PAR_NAMES                   !Vector of parameter names
    INTEGER                                                 :: PENSWELF_PAR_L                       !Number of lines (if parameter is a matrix)_
    INTEGER                                                 :: PENSWELF_PAR_C                       !Number of columnss (if parameter is a matrix)
    INTEGER,DIMENSION(15)                                   :: PENSWELF_PAR_BREAK_POINT             !number of break points to be determined when combining path names and file names
    !=======================================================================!
    !Parameters of the PENSWELF module
    !Identification variables necessary to compute welfare adaptations of pensions             
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: PENSWELF_FAMREL_QT                   !Matrix used to convert relationships of family members
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PENSWELF_MINPENS_MONTH               !Minimum pension amounts per month from the 1st June 2001
    INTEGER                                                 :: PENSWELF_TYPESIM                     !Type of welfare change to be applied on pension benefits (change in absolute amounts = 1, change in percentages = 2)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PENSWELF_ABSCHANG_MONTH              !Absolute amount with which pension benefits are to be adapted
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PENSWELF_PRCCHANG_MONTH              !Percentages with which pension benefits are to be adapted
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PENSWELF_WEHOLID_ANN                 !Holiday earnings paid  to pensioners with wage earner benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PENSWELF_CSHOLID_ANN                 !Holiday earnings paid  to pensioners with civil servant benefits
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_TIMESPAN                    !Time intervals expressed in months for welfare adaptations
    INTEGER                                                 :: PENSWELF_WELFADAP                    !Execute welfare adaptation with minimum income levels (see mail of Friday 16 March 2007)
    !=======================================================================!  
    !Exogenous variables
    !Exogenous variables
    !PENSWELF_INPUT_Write.sas
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: GECODEERD_INSZ_CHAR                  ! Geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_INPUT                 ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_RELATION                    ! Verband tussen het individu en het huishoudhoofd op 1 januari 2009
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_SEX                         ! Geslacht van het individu op 1 januari 2009
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_HH_ID                       ! Geanonimiseerd identificatienummer van het gezin waartoe het individu behoort
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NAREGNIS_DATBIRTH                    ! Geboortedatum van het individu in jaar en maand
    !PENSWELF_CONSTRUCT_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_CONSTRUCT             ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: MINPENS_NUMCYEAR                     ! aantal loopbaanjaren voor bepaling minimumrecht
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InPensCad                            ! Individu geselecteerd uit basisdata Pensioenkadaster voor verwerking steekproef
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: begindatum_pensioen_c_2008           ! begindatum_pensioen_c (geregistreerd in 2008)
    !/*Bedragen uit Pensioenkadaster*/
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RPWE_2008                            ! rustpensioen werknemer (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FIRPCAP_2008                         ! voordelen vrije verzekerden (rust) (kapitaal) (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FISPCAP_2008                         ! voordelen vrije verzekerden (overleving) (kapitaal) (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RPSE_2008                            ! rustpensioen zelfstandige (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RPCIV_2008                           ! rustpensioen overheid (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RPFOR_2008                           ! buitenlands rustpensioen (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RPDOSZ_2008                          ! rustpensioen DOSZ (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SPWE_2008                            ! overlevingspensioen werknemer (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SPSE_2008                            ! overlevingspensioen zelfstandige (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SPCIV_2008                           ! overlevingspensioen overheid (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SPFOR_2008                           ! buitenlands overlevingspensioen (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SPDOSZ_2008                          ! overlevingspensioen DOSZ (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CAPFIRST_2008                        ! kapitalen eerste pijler (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: IGO_2008                             ! GIB/IGO (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: DISAB_2008                           ! tegemoetkoming mindervaliden (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RPRENT_SPILPRIV_2008                 ! tweedepijler(rust)pensioen, rente-uitkering door private instelling (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RPRENT_SPILPUB_2008                  ! tweedepijler(rust)pensioen, rente-uitkering door publieke instelling (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SPRENT_SPILPRIV_2008                 ! tweedepijler(overlevings)pensioen, rente-uitkering door private instelling (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SPRENT_SPILPUB_2008                  ! tweedepijler(overlevings)pensioen, rente-uitkering door publieke instelling (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RPCAP_SPILPRIV_2008                  ! tweedepijler(rust)pensioen, kapitaaluitkering door private instelling (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RPCAP_SPILPUB_2008                   ! tweedepijler(rust)pensioen, kapitaaluitkering door publieke instelling (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SPCAP_SPILPRIV_2008                  ! tweedepijler(overlevings)pensioen, kapitaaluitkering door private instelling (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SPCAP_SPILPUB_2008                   ! tweedepijler(overlevings)pensioen, kapitaaluitkering door publieke instelling (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: DIRPENS_2008                         ! afgeleide pensioenen (betalingsjaar = 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: OTHPENS_2008                         ! overige pensioenbedragen (betalingsjaar = 2008)
    !/*Datums uit Pensioenkadaster*/
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RPWE_PDATE_2008                      ! startdatum betaling rustpensioen werknemer (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: FIRPCAP_PDATE_2008                   ! startdatum betaling voordelen vrije verzekerden (rust) (kapitaal) (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: FISPCAP_PDATE_2008                   ! startdatum betaling voordelen vrije verzekerden (overleving) (kapitaal) (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RPSE_PDATE_2008                      ! startdatum betaling rustpensioen zelfstandige (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RPCIV_PDATE_2008                     ! startdatum betaling rustpensioen overheid (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RPFOR_PDATE_2008                     ! startdatum betaling buitenlands rustpensioen (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RPDOSZ_PDATE_2008                    ! startdatum betaling rustpensioen DOSZ (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: SPWE_PDATE_2008                      ! startdatum betaling overlevingspensioen werknemer (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: SPSE_PDATE_2008                      ! startdatum betaling overlevingspensioen zelfstandige (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: SPCIV_PDATE_2008                     ! startdatum betaling overlevingspensioen overheid (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: SPFOR_PDATE_2008                     ! startdatum betaling buitenlands overlevingspensioen (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: SPDOSZ_PDATE_2008                    ! startdatum betaling overlevingspensioen DOSZ (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: CAPFIRST_PDATE_2008                  ! startdatum betaling kapitalen eerste pijler (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: IGO_PDATE_2008                       ! startdatum betaling GIB/IGO (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: DISAB_PDATE_2008                     ! startdatum betaling tegemoetkoming mindervaliden (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RPRENT_SPILPRIV_PDATE_2008           ! startdatum betaling tweedepijler(rust)pensioen, rente-uitkering door private instelling (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RPRENT_SPILPUB_PDATE_2008            ! startdatum betaling tweedepijler(rust)pensioen, rente-uitkering door publieke instelling (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: SPRENT_SPILPRIV_PDATE_2008           ! startdatum betaling tweedepijler(overlevings)pensioen, rente-uitkering door private instelling (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: SPRENT_SPILPUB_PDATE_2008            ! startdatum betaling tweedepijler(overlevings)pensioen, rente-uitkering door publieke instelling (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RPCAP_SPILPRIV_PDATE_2008            ! startdatum betaling tweedepijler(rust)pensioen, kapitaaluitkering door private instelling (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RPCAP_SPILPUB_PDATE_2008             ! startdatum betaling tweedepijler(rust)pensioen, kapitaaluitkering door publieke instelling (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: SPCAP_SPILPRIV_PDATE_2008            ! startdatum betaling tweedepijler(overlevings)pensioen, kapitaaluitkering door private instelling (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: SPCAP_SPILPUB_PDATE_2008             ! startdatum betaling tweedepijler(overlevings)pensioen, kapitaaluitkering door publieke instelling (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: DIRPENS_PDATE_2008                   ! startdatum betaling afgeleide pensioenen (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: OTHPENS_PDATE_2008                   ! startdatum betaling overige pensioenbedragen (betalingsjaar = 2008)
    !Variables read from file PENSWELF_CONTRIB.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_CONTRIB               !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REVPRIV_QT                   !Sum of gross labour income, holiday earnings and other supplements, earned as wage earner on the private labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REVPUB_QT                    !Sum of gross labour income, holiday earnings and other supplements, earned as wage earner on the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_INCSELF_QT                   !Labour income, earned as self employed          
    !Variables read from file PENSWELF_UNEM.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_UNEM                  !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_TOTBEN_QT                       !Gross amount of unemployment benefits           
    !Variables read from file PENSWELF_SICK.txt     
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_SICK                  !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_TOTBEN_QT                       !Sum of benefits for sickness, disability, industrial accidents and occupational diseases            
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMREL                !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMILY                        !Unique identifier identifying the family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_RELATION                      !Relation of the individual with respect to the head of the family
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMTYPE                       !Type of family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_COUPLE                        !Type of couple to which the individual belongs
    !WAGE_CONSTRUCT_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WAGE                  ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SALJRS_LAST_CORR                     ! Dagloon in laatste jaar met positieve waarde gecorrigeerd dagloon (in prijzen van 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_HOUR                   ! Uurloon gecorrigeerd voor minimum onder minimumgrens
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_QT                     ! Raming van het kwartaalinkomen in het laatst geobserveerde kwartaal
    !MIMOSIS_SAMPWEIG_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WEIGHT                ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_WEIGHT                       ! Gewicht dat corrigeert voor niet random selectie van de observatie-eenheden
    !=======================================================================!
    !Endogeneous variables
    !to rank variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NEW_RANK                             !New rank after sorting and re-ranking       
    !to construct age variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_YEAR_AGE                    !Age of the individual expressed in years
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_MONTH_AGE                   !Age of the individual expressed in moths
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_MONTH_BIRTH                 !Month of birth(mm)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_YEAR_BIRTH                  !Year of birth (yyyy)
    !to construct information at the level of the sociological household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_HH_SIZE                     !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_START_HH                    !Variable stores an identification of the first household member (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_END_HH                      !Variable stores an identification of the last household member (maximal size vector = number of individuals)   
    INTEGER                                                 :: N_HH_PENSWELF                        !number of households analysed
    !To construct variables listed in the module note
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: PENSWELF_NUMMONTH_ANN                !Number of months for which the pensioner already receives a pension benefit on January 1st 2002
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PENSWELF_BENBEFOR_MONTH              !Monthly pension benefit paid before welfare adaptation in benefit scheme 1 to 14
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_TOTPBEF_QT                  !Total sum of pension benefits received per quarter before welfare adaptations have been applied (sum of all retirement and survival benefits, but no guaranteed pension benefits)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_INDINC_QT                   !Total sum of labour market and replacement income received by an individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_REPRATE_QT                  !Replacement rate applied to compute pension benefit
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_WEMIPENS_ANN                !Wage earner benefits from a minimum pension (no=0, yes=1)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_SEMIPENS_ANN                !Self employed benefits from a minimum pension (no=0, yes=1)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_CSMIPENS_ANN                !Civil servant benefits from a minimum pension (no=0, yes=1)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PENSWELF_BENAFTER_MONTH              !Monthly pension benefit paid after welfare adaptation in benefit scheme 1 to 14
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_BENHOLID_ANN                !Holiday earnings paid to pensions (annual value)
    !Constructed exchange variables
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_RETPENS_QT                  !Quarterly amount of retirement pensions
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_SURPENS_QT                  !Quarterly amount of survival pensions
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_AMOUNTP_QT                  !Total sum of pension benefits received per quarter (sum of all retirement and survival benefits, but no guaranteed pension benefits)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_BENSCHEM_QT                 !Contribution scheme that generates the largest amount of pension benefits

    !Instruments
    INTEGER                                                 :: pers_j,hh_i,i,j,k
    INTEGER                                                 :: ierror
    
END MODULE PENSWELF_PRERUN_DATA

!==============================================!
! This module contains all the subroutines     !
! and functions used in the PENSWELF MODULE    !
!==============================================!
!List of subroutine names

MODULE PENSWELF_PRERUN_SUBROUTINES

    USE PENSWELF_PRERUN_DATA
    USE MIMOSIS_DATA    
    !use routine for sorting and ranking data
    USE RANKING_SORTING_ROUTINES

    IMPLICIT NONE
    CONTAINS

!=====================================================!
!Allocate all exogenous vectors of dimension DIM_ALLOC!
!=====================================================!

SUBROUTINE PENSWELF_ALLOC_EXOG(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !=======================================================================!  
    !Exogenous variables
    !PENSWELF_INPUT_Write.sas
    ALLOCATE(GECODEERD_INSZ_CHAR(DIM_ALLOC))
    ALLOCATE(GECODEERD_INSZ_INPUT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_RELATION(DIM_ALLOC))
    ALLOCATE(NAREGNIS_SEX(DIM_ALLOC))
    ALLOCATE(NAREGNIS_HH_ID(DIM_ALLOC))
    ALLOCATE(NAREGNIS_DATBIRTH(DIM_ALLOC))
    !PENSWELF_CONSTRUCT_Write.sas
    ALLOCATE(GECODEERD_INSZ_CONSTRUCT(DIM_ALLOC))
    ALLOCATE(MINPENS_NUMCYEAR(DIM_ALLOC))
    ALLOCATE(InPensCad(DIM_ALLOC))
    ALLOCATE(begindatum_pensioen_c_2008(DIM_ALLOC))
    !/*Bedragen uit Pensioenkadaster*/
    ALLOCATE(RPWE_2008(DIM_ALLOC))
    ALLOCATE(FIRPCAP_2008(DIM_ALLOC))
    ALLOCATE(FISPCAP_2008(DIM_ALLOC))
    ALLOCATE(RPSE_2008(DIM_ALLOC))
    ALLOCATE(RPCIV_2008(DIM_ALLOC))
    ALLOCATE(RPFOR_2008(DIM_ALLOC))
    ALLOCATE(RPDOSZ_2008(DIM_ALLOC))
    ALLOCATE(SPWE_2008(DIM_ALLOC))
    ALLOCATE(SPSE_2008(DIM_ALLOC))
    ALLOCATE(SPCIV_2008(DIM_ALLOC))
    ALLOCATE(SPFOR_2008(DIM_ALLOC))
    ALLOCATE(SPDOSZ_2008(DIM_ALLOC))
    ALLOCATE(CAPFIRST_2008(DIM_ALLOC))
    ALLOCATE(IGO_2008(DIM_ALLOC))
    ALLOCATE(DISAB_2008(DIM_ALLOC))
    ALLOCATE(RPRENT_SPILPRIV_2008(DIM_ALLOC))
    ALLOCATE(RPRENT_SPILPUB_2008(DIM_ALLOC))
    ALLOCATE(SPRENT_SPILPRIV_2008(DIM_ALLOC))
    ALLOCATE(SPRENT_SPILPUB_2008(DIM_ALLOC))
    ALLOCATE(RPCAP_SPILPRIV_2008(DIM_ALLOC))
    ALLOCATE(RPCAP_SPILPUB_2008(DIM_ALLOC))
    ALLOCATE(SPCAP_SPILPRIV_2008(DIM_ALLOC))
    ALLOCATE(SPCAP_SPILPUB_2008(DIM_ALLOC))
    ALLOCATE(DIRPENS_2008(DIM_ALLOC))
    ALLOCATE(OTHPENS_2008(DIM_ALLOC))
    !/*Datums uit Pensioenkadaster*/
    ALLOCATE(RPWE_PDATE_2008(DIM_ALLOC))
    ALLOCATE(FIRPCAP_PDATE_2008(DIM_ALLOC))
    ALLOCATE(FISPCAP_PDATE_2008(DIM_ALLOC))
    ALLOCATE(RPSE_PDATE_2008(DIM_ALLOC))
    ALLOCATE(RPCIV_PDATE_2008(DIM_ALLOC))
    ALLOCATE(RPFOR_PDATE_2008(DIM_ALLOC))
    ALLOCATE(RPDOSZ_PDATE_2008(DIM_ALLOC))
    ALLOCATE(SPWE_PDATE_2008(DIM_ALLOC))
    ALLOCATE(SPSE_PDATE_2008(DIM_ALLOC))
    ALLOCATE(SPCIV_PDATE_2008(DIM_ALLOC))
    ALLOCATE(SPFOR_PDATE_2008(DIM_ALLOC))
    ALLOCATE(SPDOSZ_PDATE_2008(DIM_ALLOC))
    ALLOCATE(CAPFIRST_PDATE_2008(DIM_ALLOC))
    ALLOCATE(IGO_PDATE_2008(DIM_ALLOC))
    ALLOCATE(DISAB_PDATE_2008(DIM_ALLOC))
    ALLOCATE(RPRENT_SPILPRIV_PDATE_2008(DIM_ALLOC))
    ALLOCATE(RPRENT_SPILPUB_PDATE_2008(DIM_ALLOC))
    ALLOCATE(SPRENT_SPILPRIV_PDATE_2008(DIM_ALLOC))
    ALLOCATE(SPRENT_SPILPUB_PDATE_2008(DIM_ALLOC))
    ALLOCATE(RPCAP_SPILPRIV_PDATE_2008(DIM_ALLOC))
    ALLOCATE(RPCAP_SPILPUB_PDATE_2008(DIM_ALLOC))
    ALLOCATE(SPCAP_SPILPRIV_PDATE_2008(DIM_ALLOC))
    ALLOCATE(SPCAP_SPILPUB_PDATE_2008(DIM_ALLOC))
    ALLOCATE(DIRPENS_PDATE_2008(DIM_ALLOC))
    ALLOCATE(OTHPENS_PDATE_2008(DIM_ALLOC))
    !Variables read from file PENSWELF_CONTRIB.txt
    ALLOCATE(GECODEERD_INSZ_CONTRIB(DIM_ALLOC))              
    ALLOCATE(CONTRIB_REVPRIV_QT(DIM_ALLOC))                  
    ALLOCATE(CONTRIB_REVPUB_QT(DIM_ALLOC))                   
    ALLOCATE(CONTRIB_INCSELF_QT(DIM_ALLOC))                  
    !Variables read from file PENSWELF_UNEM.txt
    ALLOCATE(GECODEERD_INSZ_UNEM(DIM_ALLOC))                 
    ALLOCATE(UNEM_TOTBEN_QT(DIM_ALLOC))                      
    !Variables read from file PENSWELF_SICK.txt     
    ALLOCATE(GECODEERD_INSZ_SICK(DIM_ALLOC))                 
    ALLOCATE(SICK_TOTBEN_QT(DIM_ALLOC))                      
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    ALLOCATE(GECODEERD_INSZ_FAMREL(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMILY(DIM_ALLOC))
    ALLOCATE(FAMREL_RELATION(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMTYPE(DIM_ALLOC))
    ALLOCATE(FAMREL_COUPLE(DIM_ALLOC))
    !WAGE_CONSTRUCT_Write.sas
    ALLOCATE(GECODEERD_INSZ_WAGE(DIM_ALLOC))
    ALLOCATE(SALJRS_LAST_CORR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_HOUR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_QT(DIM_ALLOC))
    !MIMOSIS_SAMPWEIG_Write.sas
    ALLOCATE(GECODEERD_INSZ_WEIGHT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WEIGHT(DIM_ALLOC))

END SUBROUTINE PENSWELF_ALLOC_EXOG

!======================================================!
!Allocate all endogenous vectors of dimension DIM_ALLOC!
!======================================================!

SUBROUTINE PENSWELF_ALLOC_ENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !Endogeneous variables
    !to rank variables
    ALLOCATE(NEW_RANK(DIM_ALLOC))
    !to construct age variables
    ALLOCATE(PENSWELF_YEAR_AGE(DIM_ALLOC))
    ALLOCATE(PENSWELF_MONTH_AGE(DIM_ALLOC))
    ALLOCATE(PENSWELF_MONTH_BIRTH(DIM_ALLOC))
    ALLOCATE(PENSWELF_YEAR_BIRTH(DIM_ALLOC))
    !to construct information at the level of the sociological household
    ALLOCATE(PENSWELF_HH_SIZE(DIM_ALLOC))
    ALLOCATE(PENSWELF_START_HH(DIM_ALLOC))
    ALLOCATE(PENSWELF_END_HH(DIM_ALLOC))
    !To construct variables listed in the module note
    ALLOCATE(PENSWELF_NUMMONTH_ANN(DIM_ALLOC,13))
    ALLOCATE(PENSWELF_BENBEFOR_MONTH(DIM_ALLOC,13))
    ALLOCATE(PENSWELF_TOTPBEF_QT(DIM_ALLOC))
    ALLOCATE(PENSWELF_INDINC_QT(DIM_ALLOC))
    ALLOCATE(PENSWELF_REPRATE_QT(DIM_ALLOC))
    ALLOCATE(PENSWELF_WEMIPENS_ANN(DIM_ALLOC))
    ALLOCATE(PENSWELF_SEMIPENS_ANN(DIM_ALLOC))
    ALLOCATE(PENSWELF_CSMIPENS_ANN(DIM_ALLOC))
    ALLOCATE(PENSWELF_BENAFTER_MONTH(DIM_ALLOC,13))
    ALLOCATE(PENSWELF_BENHOLID_ANN(DIM_ALLOC))
    !Constructed exchange variables
    ALLOCATE(PENSWELF_RETPENS_QT(DIM_ALLOC))
    ALLOCATE(PENSWELF_SURPENS_QT(DIM_ALLOC))
    ALLOCATE(PENSWELF_AMOUNTP_QT(DIM_ALLOC))
    ALLOCATE(PENSWELF_BENSCHEM_QT(DIM_ALLOC))
    
END SUBROUTINE PENSWELF_ALLOC_ENDO

SUBROUTINE PENSWELF_DEALLOC

    !=======================================================================!  
    !Exogenous variables
    !PENSWELF_INPUT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_CHAR)
    DEALLOCATE(GECODEERD_INSZ_INPUT)
    DEALLOCATE(NAREGNIS_RELATION)
    DEALLOCATE(NAREGNIS_SEX)
    DEALLOCATE(NAREGNIS_HH_ID)
    DEALLOCATE(NAREGNIS_DATBIRTH)
    !PENSWELF_CONSTRUCT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_CONSTRUCT)
    DEALLOCATE(MINPENS_NUMCYEAR)
    DEALLOCATE(InPensCad)
    DEALLOCATE(begindatum_pensioen_c_2008)
    !/*Bedragen uit Pensioenkadaster*/
    DEALLOCATE(RPWE_2008)
    DEALLOCATE(FIRPCAP_2008)
    DEALLOCATE(FISPCAP_2008)
    DEALLOCATE(RPSE_2008)
    DEALLOCATE(RPCIV_2008)
    DEALLOCATE(RPFOR_2008)
    DEALLOCATE(RPDOSZ_2008)
    DEALLOCATE(SPWE_2008)
    DEALLOCATE(SPSE_2008)
    DEALLOCATE(SPCIV_2008)
    DEALLOCATE(SPFOR_2008)
    DEALLOCATE(SPDOSZ_2008)
    DEALLOCATE(CAPFIRST_2008)
    DEALLOCATE(IGO_2008)
    DEALLOCATE(DISAB_2008)
    DEALLOCATE(RPRENT_SPILPRIV_2008)
    DEALLOCATE(RPRENT_SPILPUB_2008)
    DEALLOCATE(SPRENT_SPILPRIV_2008)
    DEALLOCATE(SPRENT_SPILPUB_2008)
    DEALLOCATE(RPCAP_SPILPRIV_2008)
    DEALLOCATE(RPCAP_SPILPUB_2008)
    DEALLOCATE(SPCAP_SPILPRIV_2008)
    DEALLOCATE(SPCAP_SPILPUB_2008)
    DEALLOCATE(DIRPENS_2008)
    DEALLOCATE(OTHPENS_2008)
    !/*Datums uit Pensioenkadaster*/
    DEALLOCATE(RPWE_PDATE_2008)
    DEALLOCATE(FIRPCAP_PDATE_2008)
    DEALLOCATE(FISPCAP_PDATE_2008)
    DEALLOCATE(RPSE_PDATE_2008)
    DEALLOCATE(RPCIV_PDATE_2008)
    DEALLOCATE(RPFOR_PDATE_2008)
    DEALLOCATE(RPDOSZ_PDATE_2008)
    DEALLOCATE(SPWE_PDATE_2008)
    DEALLOCATE(SPSE_PDATE_2008)
    DEALLOCATE(SPCIV_PDATE_2008)
    DEALLOCATE(SPFOR_PDATE_2008)
    DEALLOCATE(SPDOSZ_PDATE_2008)
    DEALLOCATE(CAPFIRST_PDATE_2008)
    DEALLOCATE(IGO_PDATE_2008)
    DEALLOCATE(DISAB_PDATE_2008)
    DEALLOCATE(RPRENT_SPILPRIV_PDATE_2008)
    DEALLOCATE(RPRENT_SPILPUB_PDATE_2008)
    DEALLOCATE(SPRENT_SPILPRIV_PDATE_2008)
    DEALLOCATE(SPRENT_SPILPUB_PDATE_2008)
    DEALLOCATE(RPCAP_SPILPRIV_PDATE_2008)
    DEALLOCATE(RPCAP_SPILPUB_PDATE_2008)
    DEALLOCATE(SPCAP_SPILPRIV_PDATE_2008)
    DEALLOCATE(SPCAP_SPILPUB_PDATE_2008)
    DEALLOCATE(DIRPENS_PDATE_2008)
    DEALLOCATE(OTHPENS_PDATE_2008)
    !Variables read from file PENSWELF_CONTRIB.txt
    DEALLOCATE(GECODEERD_INSZ_CONTRIB)
    DEALLOCATE(CONTRIB_REVPRIV_QT)
    DEALLOCATE(CONTRIB_REVPUB_QT)
    DEALLOCATE(CONTRIB_INCSELF_QT)
    !Variables read from file PENSWELF_UNEM.txt
    DEALLOCATE(GECODEERD_INSZ_UNEM)
    DEALLOCATE(UNEM_TOTBEN_QT)
    !Variables read from file PENSWELF_SICK.txt     
    DEALLOCATE(GECODEERD_INSZ_SICK)
    DEALLOCATE(SICK_TOTBEN_QT)
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    DEALLOCATE(GECODEERD_INSZ_FAMREL)
    DEALLOCATE(FAMREL_FAMILY)
    DEALLOCATE(FAMREL_RELATION)
    DEALLOCATE(FAMREL_FAMTYPE)
    DEALLOCATE(FAMREL_COUPLE)
    !WAGE_CONSTRUCT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WAGE)
    DEALLOCATE(SALJRS_LAST_CORR)
    DEALLOCATE(MIMOSIS_GRINC_HOUR)
    DEALLOCATE(MIMOSIS_GRINC_QT)
    !MIMOSIS_SAMPWEIG_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WEIGHT)
    DEALLOCATE(MIMOSIS_WEIGHT)

    !=======================================================================!  
    !Endogeneous variables
    !to rank variables
    DEALLOCATE(NEW_RANK)
    !to construct age variables
    DEALLOCATE(PENSWELF_YEAR_AGE)
    DEALLOCATE(PENSWELF_MONTH_AGE)
    DEALLOCATE(PENSWELF_MONTH_BIRTH)
    DEALLOCATE(PENSWELF_YEAR_BIRTH)
    !to construct information at the level of the sociological household
    DEALLOCATE(PENSWELF_HH_SIZE)
    DEALLOCATE(PENSWELF_START_HH)
    DEALLOCATE(PENSWELF_END_HH)
    !To construct variables listed in the module note
    DEALLOCATE(PENSWELF_NUMMONTH_ANN)
    DEALLOCATE(PENSWELF_BENBEFOR_MONTH)
    DEALLOCATE(PENSWELF_TOTPBEF_QT)
    DEALLOCATE(PENSWELF_INDINC_QT)
    DEALLOCATE(PENSWELF_REPRATE_QT)
    DEALLOCATE(PENSWELF_WEMIPENS_ANN)
    DEALLOCATE(PENSWELF_SEMIPENS_ANN)
    DEALLOCATE(PENSWELF_CSMIPENS_ANN)
    DEALLOCATE(PENSWELF_BENAFTER_MONTH)
    DEALLOCATE(PENSWELF_BENHOLID_ANN)
    !Constructed exchange variables
    DEALLOCATE(PENSWELF_RETPENS_QT)
    DEALLOCATE(PENSWELF_SURPENS_QT)
    DEALLOCATE(PENSWELF_AMOUNTP_QT)
    DEALLOCATE(PENSWELF_BENSCHEM_QT)

    !=======================================================================!
    !Deallocate parameters
    !Identification variables necessary to compute family allowances             
    DEALLOCATE(PENSWELF_FAMREL_QT)
    DEALLOCATE(PENSWELF_MINPENS_MONTH)
    DEALLOCATE(PENSWELF_ABSCHANG_MONTH)
    DEALLOCATE(PENSWELF_PRCCHANG_MONTH)
    DEALLOCATE(PENSWELF_WEHOLID_ANN)
    DEALLOCATE(PENSWELF_CSHOLID_ANN)
    DEALLOCATE(PENSWELF_TIMESPAN)
    
END SUBROUTINE PENSWELF_DEALLOC

!===============================================!
!Initialise endogenous variables                        
!===============================================!

SUBROUTINE PENSWELF_INITENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors
    INTEGER                                                 :: pers_j,i                             !index for do loop

    DO pers_j=1,DIM_ALLOC
        !to rank variables
        NEW_RANK(pers_j) = 0
        !to construct age variables
        PENSWELF_YEAR_AGE(pers_j) = 0
        PENSWELF_MONTH_AGE(pers_j) = 0
        PENSWELF_MONTH_BIRTH(pers_j) = 0
        PENSWELF_YEAR_BIRTH(pers_j) = 0
        !to construct information at the level of the sociological household
        PENSWELF_HH_SIZE(pers_j) = 0
        PENSWELF_START_HH(pers_j) = 0
        PENSWELF_END_HH(pers_j) = 0
        !To construct variables listed in the module note
        DO i=1,13
            PENSWELF_NUMMONTH_ANN(pers_j,i) = 0
            PENSWELF_BENBEFOR_MONTH(pers_j,i) = 0
        END DO    
        PENSWELF_TOTPBEF_QT(pers_j) = 0
        PENSWELF_INDINC_QT(pers_j) = 0
        PENSWELF_REPRATE_QT(pers_j) = 0
        PENSWELF_WEMIPENS_ANN(pers_j) = 0
        PENSWELF_SEMIPENS_ANN(pers_j) = 0
        PENSWELF_CSMIPENS_ANN(pers_j) = 0
        DO i=1,13
            PENSWELF_BENAFTER_MONTH(pers_j,i) = 0
        END DO    
        PENSWELF_BENHOLID_ANN(pers_j) = 0
        !Constructed exchange variables
        PENSWELF_RETPENS_QT(pers_j) = 0
        PENSWELF_SURPENS_QT(pers_j) = 0
        PENSWELF_AMOUNTP_QT(pers_j) = 0
        PENSWELF_BENSCHEM_QT(pers_j) = 0
    END DO
    N_HH_PENSWELF=0

END SUBROUTINE PENSWELF_INITENDO

!========== SUBROUTINE PENSWELF_RANKANDSORT =================================!
!RANK and SORT exogenous variables 
!   Step 1: rank and sort all variables according to their 
!           individual identification number which is read in as an integer
!   Step 2: rank and sort all variables according to the 
!           household identification number
!============================================================================!

SUBROUTINE PENSWELF_RANKANDSORT

!STEP1: ranking and sorting with the individuals identification number
    !Rank individual identification number from the INPUT data file
    CALL RANKING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    !PENSWELF_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    !Rank individual identification number from the PENSWELF_CONSTRUCT data file
    CALL RANKING(GECODEERD_INSZ_CONSTRUCT(1:n_rec),NEW_RANK)
    !PENSWELF_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CONSTRUCT(1:n_rec),NEW_RANK)
    CALL SORTING(MINPENS_NUMCYEAR(1:n_rec),NEW_RANK)
    CALL SORTING(InPensCad(1:n_rec),NEW_RANK)
    CALL SORTING(begindatum_pensioen_c_2008(1:n_rec),NEW_RANK)
    !/*Bedragen uit Pensioenkadaster*/
    CALL SORTING(RPWE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FIRPCAP_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FISPCAP_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPSE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPFOR_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPDOSZ_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPWE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPSE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPFOR_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPDOSZ_2008(1:n_rec),NEW_RANK)
    CALL SORTING(CAPFIRST_2008(1:n_rec),NEW_RANK)
    CALL SORTING(IGO_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DISAB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPRENT_SPILPRIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPRENT_SPILPUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPRENT_SPILPRIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPRENT_SPILPUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCAP_SPILPRIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCAP_SPILPUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCAP_SPILPRIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCAP_SPILPUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DIRPENS_2008(1:n_rec),NEW_RANK)
    CALL SORTING(OTHPENS_2008(1:n_rec),NEW_RANK)
    !/*Datums uit Pensioenkadaster*/
    CALL SORTING(RPWE_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FIRPCAP_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FISPCAP_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPSE_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPFOR_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPDOSZ_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPWE_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPSE_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPFOR_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPDOSZ_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(CAPFIRST_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(IGO_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DISAB_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPRENT_SPILPRIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPRENT_SPILPUB_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPRENT_SPILPRIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPRENT_SPILPUB_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCAP_SPILPRIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCAP_SPILPUB_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCAP_SPILPRIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCAP_SPILPUB_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DIRPENS_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(OTHPENS_PDATE_2008(1:n_rec),NEW_RANK)
    !Rank individual identification number from the PENSWELF_CONTRIB data file
    CALL RANKING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    !Sort variables read from file PENSWELF_CONTRIB.txt
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_INCSELF_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from the PENSWELF_UNEM data file
    CALL RANKING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    !Sort variables read from file PENSWELF_UNEM.txt
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_TOTBEN_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from the PENSWELF_SICK data file
    CALL RANKING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    !Sort variables read from file PENSWELF_SICK.txt     
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TOTBEN_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_FAMREL.txt                                           
    CALL RANKING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !Rank individual identification number from file WAGE_CONSTRUCT.txt                                           
    CALL RANKING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_WEIGHT.txt
    CALL RANKING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL PENSWELF_CONSISTENCY_CHECK
   
!STEP2: ranking and sorting with the household identification number
    !Rank household number
    CALL RANKING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    !Sort variables on rank of household number
    !PENSWELF_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    !PENSWELF_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CONSTRUCT(1:n_rec),NEW_RANK)
    CALL SORTING(MINPENS_NUMCYEAR(1:n_rec),NEW_RANK)
    CALL SORTING(InPensCad(1:n_rec),NEW_RANK)
    CALL SORTING(begindatum_pensioen_c_2008(1:n_rec),NEW_RANK)
    !/*Bedragen uit Pensioenkadaster*/
    CALL SORTING(RPWE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FIRPCAP_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FISPCAP_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPSE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPFOR_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPDOSZ_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPWE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPSE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPFOR_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPDOSZ_2008(1:n_rec),NEW_RANK)
    CALL SORTING(CAPFIRST_2008(1:n_rec),NEW_RANK)
    CALL SORTING(IGO_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DISAB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPRENT_SPILPRIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPRENT_SPILPUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPRENT_SPILPRIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPRENT_SPILPUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCAP_SPILPRIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCAP_SPILPUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCAP_SPILPRIV_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCAP_SPILPUB_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DIRPENS_2008(1:n_rec),NEW_RANK)
    CALL SORTING(OTHPENS_2008(1:n_rec),NEW_RANK)
    !/*Datums uit Pensioenkadaster*/
    CALL SORTING(RPWE_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FIRPCAP_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FISPCAP_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPSE_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPFOR_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPDOSZ_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPWE_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPSE_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPFOR_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPDOSZ_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(CAPFIRST_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(IGO_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DISAB_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPRENT_SPILPRIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPRENT_SPILPUB_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPRENT_SPILPRIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPRENT_SPILPUB_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCAP_SPILPRIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RPCAP_SPILPUB_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCAP_SPILPRIV_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(SPCAP_SPILPUB_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DIRPENS_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(OTHPENS_PDATE_2008(1:n_rec),NEW_RANK)
    !Sort variables read from file PENSWELF_CONTRIB.txt
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_INCSELF_QT(1:n_rec),NEW_RANK)
    !Sort variables read from file PENSWELF_UNEM.txt
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_TOTBEN_QT(1:n_rec),NEW_RANK)
    !Sort variables read from file PENSWELF_SICK.txt     
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TOTBEN_QT(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL PENSWELF_CONSISTENCY_CHECK

END SUBROUTINE PENSWELF_RANKANDSORT

!========== SUBROUTINE PARAMETER_FILEREAD ===============!
!This subroutines reads in all parameters from           !
!the parameter file                                      !
!========================================================!
SUBROUTINE PENSWELF_PARAMETER_FILEREAD

    !Read in parameters for path and name of input files
    DO i=1,n_loclines/2
        !Read default path names
        !Start reading the normal file names
        IF (CONTROL_IO(2*i-1)=='PENSWELF_INPUT_PATH') THEN
            PENSWELF_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='PENSWELF_INPUT_FILE') THEN
            PENSWELF_INPUT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='PENSWELF_SICK_PRERUN_FILE') THEN
            PENSWELF_SICK_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='PENSWELF_UNEM_PRERUN_FILE') THEN
            PENSWELF_UNEM_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='PENSWELF_CONTRIB_PRERUN_FILE') THEN
            PENSWELF_CONTRIB_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='PENSWELF_CONSTRUCT_FILE') THEN
            PENSWELF_CONSTRUCT_FILE = CONTROL_IO(2*i)
        !Read material of other input files                                                    
        ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_INPUT_PATH') THEN
            FAMREL_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_FAMREL_FILE') THEN
            MIMOSIS_FAMREL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='WAGE_CONSTRUCT_FILE') THEN
            WAGE_CONSTRUCT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='CONSTANT_INPUT_PATH') THEN
            CONSTANT_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_SAMPWEIG_FILE') THEN
            MIMOSIS_SAMPWEIG_FILE = CONTROL_IO(2*i)
        !Read in names of output files 
        !output files in this run are input files of other modules
        ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_PENSWELF_PRERUN_FILE') THEN
            CONTRIB_PENSWELF_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SICK_PENSWELF_PRERUN_FILE') THEN
            SICK_PENSWELF_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='UNEM_PENSWELF_PRERUN_FILE') THEN
            UNEM_PENSWELF_FILE = CONTROL_IO(2*i)            
        ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_PENSWELF_PRERUN_FILE') THEN
            FAMAL_PENSWELF_FILE = CONTROL_IO(2*i)            
        END IF
    END DO     
    !check for length of default path and file name parameters
    PENSWELF_PAR_BREAK_POINT(1)  = INDEX(PENSWELF_INPUT_PATH,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(2)  = INDEX(PENSWELF_INPUT_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(3)  = INDEX(PENSWELF_CONSTRUCT_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(4)  = INDEX(CONSTANT_INPUT_PATH,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(5)  = INDEX(MIMOSIS_SAMPWEIG_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(6)  = INDEX(FAMREL_INPUT_PATH,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(7)  = INDEX(MIMOSIS_FAMREL_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(8)  = INDEX(WAGE_CONSTRUCT_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(9)  = INDEX(CONTRIB_PENSWELF_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(10) = INDEX(SICK_PENSWELF_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(11) = INDEX(UNEM_PENSWELF_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(12) = INDEX(FAMAL_PENSWELF_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(13) = INDEX(PENSWELF_SICK_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(14) = INDEX(PENSWELF_UNEM_FILE,PENSWELF_PAR_BREAK)
    PENSWELF_PAR_BREAK_POINT(15) = INDEX(PENSWELF_CONTRIB_FILE,PENSWELF_PAR_BREAK)
    !merge information (path+name) to create default input file names
    !of input files
    PENSWELF_INPUT = PENSWELF_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(1)-1)//&
    &PENSWELF_INPUT_FILE(1:PENSWELF_PAR_BREAK_POINT(2)-1)
    PENSWELF_SICK = PENSWELF_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(1)-1)//&
    &PENSWELF_SICK_FILE(1:PENSWELF_PAR_BREAK_POINT(13)-1)
    PENSWELF_UNEM = PENSWELF_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(1)-1)//&
    &PENSWELF_UNEM_FILE(1:PENSWELF_PAR_BREAK_POINT(14)-1)
    PENSWELF_CONTRIB = PENSWELF_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(1)-1)//&
    &PENSWELF_CONTRIB_FILE(1:PENSWELF_PAR_BREAK_POINT(15)-1)
    PENSWELF_CONSTRUCT = PENSWELF_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(1)-1)//&
    &PENSWELF_CONSTRUCT_FILE(1:PENSWELF_PAR_BREAK_POINT(3)-1)
    MIMOSIS_FAMREL = FAMREL_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(6)-1)//&
    &MIMOSIS_FAMREL_FILE(1:PENSWELF_PAR_BREAK_POINT(7)-1)    
    WAGE_CONSTRUCT = PENSWELF_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(1)-1)//&
    &WAGE_CONSTRUCT_FILE(1:PENSWELF_PAR_BREAK_POINT(8)-1)
    MIMOSIS_SAMPWEIG = CONSTANT_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(4)-1)//&
    &MIMOSIS_SAMPWEIG_FILE(1:PENSWELF_PAR_BREAK_POINT(5)-1)
    !of output files
    CONTRIB_PENSWELF = PENSWELF_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(1)-1)//&
    &CONTRIB_PENSWELF_FILE(1:PENSWELF_PAR_BREAK_POINT(9)-1)        
    SICK_PENSWELF = PENSWELF_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(1)-1)//&
    &SICK_PENSWELF_FILE(1:PENSWELF_PAR_BREAK_POINT(10)-1)
    UNEM_PENSWELF = PENSWELF_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(1)-1)//&
    &UNEM_PENSWELF_FILE(1:PENSWELF_PAR_BREAK_POINT(11)-1)   
    FAMAL_PENSWELF = PENSWELF_INPUT_PATH(1:PENSWELF_PAR_BREAK_POINT(1)-1)//&
    &FAMAL_PENSWELF_FILE(1:PENSWELF_PAR_BREAK_POINT(12)-1)    

    !Trim the result
    !of input files
    CALL TRIM@(PENSWELF_INPUT)
    CALL TRIM@(PENSWELF_SICK)
    CALL TRIM@(PENSWELF_UNEM)
    CALL TRIM@(PENSWELF_CONTRIB)
    CALL TRIM@(PENSWELF_CONSTRUCT)
    CALL TRIM@(MIMOSIS_FAMREL)    
    CALL TRIM@(WAGE_CONSTRUCT)  
    CALL TRIM@(MIMOSIS_SAMPWEIG) 
    !of output files
    CALL TRIM@(CONTRIB_PENSWELF)
    CALL TRIM@(SICK_PENSWELF)    
    CALL TRIM@(UNEM_PENSWELF)  
    CALL TRIM@(FAMAL_PENSWELF)

    !write names to log file
    WRITE(15,'(''Print of reconstructed input and output file names'')')
    CALL DRAWLINE(15,100,'=')    
    !of input files
    WRITE(15,*)     'INPUT FILE=',      PENSWELF_INPUT
    WRITE(15,*)     'SICK FILE=',       PENSWELF_SICK
    WRITE(15,*)     'UNEM FILE=',       PENSWELF_UNEM
    WRITE(15,*)     'CONTRIB FILE=',    PENSWELF_CONTRIB
    WRITE(15,*)     'CONSTRUCT FILE=',  PENSWELF_CONSTRUCT
    WRITE(15,*)     'MIMOSIS_FAMREL= ', MIMOSIS_FAMREL
    WRITE(15,*)     'WAGE_CONSTRUCT= ', WAGE_CONSTRUCT
    WRITE(15,*)     'WEIGHT FILE= ',    MIMOSIS_SAMPWEIG    
    !of output files
    WRITE(15,*)     'CONTRIB_PENSWELF=',CONTRIB_PENSWELF
    WRITE(15,*)     'SICK_PENSWELF=',   SICK_PENSWELF  
    WRITE(15,*)     'UNEM_PENSWELF=',   UNEM_PENSWELF 
    WRITE(15,*)     'FAMAL_PENSWELF=',  FAMAL_PENSWELF 
        
    !open parameter file        
    OPEN(UNIT=3, FILE=PENSWELF_PARLOC, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PENSWELF_PARLOC
        WRITE(15,'("Could not find ", a100)') PENSWELF_PARLOC
        PAUSE
        STOP
    END IF        

    ALLOCATE(PENSWELF_PAR_NAMES(n_par))
    !read in all parameters by looping over the lines in the parameter file
    WRITE(15,'(/,''Print of read in operation of parameter file'')')
    CALL DRAWLINE(15,100,'=')    
    i=1
    DO
        READ(3,'(a40,2i10,a10)') PENSWELF_PAR_NAMES(i), PENSWELF_PAR_L, &
        &PENSWELF_PAR_C, PENSWELF_PAR_CURR
        IF (PENSWELF_PAR_NAMES(i)=='PENSWELF_FAMREL_QT') THEN
            WRITE(15,'(a40)') PENSWELF_PAR_NAMES(i)
            ALLOCATE(PENSWELF_FAMREL_QT(PENSWELF_PAR_L,PENSWELF_PAR_C))
            READ(3,'(a20)') PENSWELF_PAR_FORMAT
            READ(3,PENSWELF_PAR_FORMAT) ((PENSWELF_FAMREL_QT(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
            WRITE(15,PENSWELF_PAR_FORMAT) ((PENSWELF_FAMREL_QT(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
        ELSE IF (PENSWELF_PAR_NAMES(i)=='PENSWELF_MINPENS_MONTH') THEN
            WRITE(15,'(a40)') PENSWELF_PAR_NAMES(i)
            ALLOCATE(PENSWELF_MINPENS_MONTH(PENSWELF_PAR_L,PENSWELF_PAR_C))
            READ(3,'(a20)') PENSWELF_PAR_FORMAT
            READ(3,PENSWELF_PAR_FORMAT) ((PENSWELF_MINPENS_MONTH(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
            WRITE(15,PENSWELF_PAR_FORMAT) ((PENSWELF_MINPENS_MONTH(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
        ELSE IF (PENSWELF_PAR_NAMES(i)=='PENSWELF_TYPESIM') THEN
            WRITE(15,'(a40)') PENSWELF_PAR_NAMES(i)
            READ(3,'(a20)') PENSWELF_PAR_FORMAT
            READ(3,PENSWELF_PAR_FORMAT) PENSWELF_TYPESIM
            WRITE(15,PENSWELF_PAR_FORMAT) PENSWELF_TYPESIM
        ELSE IF (PENSWELF_PAR_NAMES(i)=='PENSWELF_ABSCHANG_MONTH') THEN
            WRITE(15,'(a40)') PENSWELF_PAR_NAMES(i)
            ALLOCATE(PENSWELF_ABSCHANG_MONTH(PENSWELF_PAR_L,PENSWELF_PAR_C))
            READ(3,'(a20)') PENSWELF_PAR_FORMAT
            READ(3,PENSWELF_PAR_FORMAT) ((PENSWELF_ABSCHANG_MONTH(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
            WRITE(15,PENSWELF_PAR_FORMAT) ((PENSWELF_ABSCHANG_MONTH(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
        ELSE IF (PENSWELF_PAR_NAMES(i)=='PENSWELF_PRCCHANG_MONTH') THEN
            WRITE(15,'(a40)') PENSWELF_PAR_NAMES(i)
            ALLOCATE(PENSWELF_PRCCHANG_MONTH(PENSWELF_PAR_L,PENSWELF_PAR_C))
            READ(3,'(a20)') PENSWELF_PAR_FORMAT
            READ(3,PENSWELF_PAR_FORMAT) ((PENSWELF_PRCCHANG_MONTH(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
            WRITE(15,PENSWELF_PAR_FORMAT) ((PENSWELF_PRCCHANG_MONTH(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
        ELSE IF (PENSWELF_PAR_NAMES(i)=='PENSWELF_WEHOLID_ANN') THEN
            WRITE(15,'(a40)') PENSWELF_PAR_NAMES(i)
            ALLOCATE(PENSWELF_WEHOLID_ANN(PENSWELF_PAR_L,PENSWELF_PAR_C))
            READ(3,'(a20)') PENSWELF_PAR_FORMAT
            READ(3,PENSWELF_PAR_FORMAT) ((PENSWELF_WEHOLID_ANN(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
            WRITE(15,PENSWELF_PAR_FORMAT) ((PENSWELF_WEHOLID_ANN(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
        ELSE IF (PENSWELF_PAR_NAMES(i)=='PENSWELF_CSHOLID_ANN') THEN
            WRITE(15,'(a40)') PENSWELF_PAR_NAMES(i)
            ALLOCATE(PENSWELF_CSHOLID_ANN(PENSWELF_PAR_L,PENSWELF_PAR_C))
            READ(3,'(a20)') PENSWELF_PAR_FORMAT
            READ(3,PENSWELF_PAR_FORMAT) ((PENSWELF_CSHOLID_ANN(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
            WRITE(15,PENSWELF_PAR_FORMAT) ((PENSWELF_CSHOLID_ANN(j,k),k=1,PENSWELF_PAR_C),j=1,PENSWELF_PAR_L)
        ELSE IF (PENSWELF_PAR_NAMES(i)=='PENSWELF_TIMESPAN') THEN
            WRITE(15,'(a40)') PENSWELF_PAR_NAMES(i)
            ALLOCATE(PENSWELF_TIMESPAN(PENSWELF_PAR_L))
            READ(3,'(a20)') PENSWELF_PAR_FORMAT
            READ(3,PENSWELF_PAR_FORMAT) (PENSWELF_TIMESPAN(j),j=1,PENSWELF_PAR_L)
            WRITE(15,PENSWELF_PAR_FORMAT) (PENSWELF_TIMESPAN(j),j=1,PENSWELF_PAR_L)
        ELSE IF (PENSWELF_PAR_NAMES(i)=='PENSWELF_WELFADAP') THEN
            WRITE(15,'(a40)') PENSWELF_PAR_NAMES(i)
            READ(3,'(a20)') PENSWELF_PAR_FORMAT
            READ(3,PENSWELF_PAR_FORMAT) PENSWELF_WELFADAP
            WRITE(15,PENSWELF_PAR_FORMAT) PENSWELF_WELFADAP
        ELSE IF (PENSWELF_PAR_NAMES(i)=='end_parameters') THEN
            EXIT
        END IF
        i=i+1
    END DO
    DEALLOCATE(PENSWELF_PAR_NAMES)

    !close the parameter file
    CLOSE(UNIT=3)
    WRITE(*,*) 'Subroutine PENSWELF_PARAMETER_FILEREAD of PRE RUN.....runned correctly'
    
END SUBROUTINE PENSWELF_PARAMETER_FILEREAD

!============= SUBROUTINE INPUT_FILEREAD ================!
!This subroutines reads in data from INPUT file          !
!========================================================!

SUBROUTINE PENSWELF_INPUT_FILEREAD

    OPEN(UNIT=5, FILE=PENSWELF_INPUT, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PENSWELF_INPUT
        WRITE(15,'("Could not find ", a100)') PENSWELF_INPUT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(A14,I10,I10,I10,A7)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
            &NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,PENSWELF_INPUT
            END IF
            !Read in numeric part of the individual indentification character variable
            READ (GECODEERD_INSZ_CHAR(pers_j)(8:), '(I10)') GECODEERD_INSZ_INPUT(pers_j)
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j)'')')
                WRITE(15,'(A14,I10,I10,I10,A7)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(A14,I10,I10,I10,A7)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine PENSWELF_INPUT_FILEREAD of PRE RUN.........runned correctly'
    WRITE(15,'(I7, " records uploaded from INPUT_FILE")') pers_j-1

END SUBROUTINE PENSWELF_INPUT_FILEREAD

!============= SUBROUTINE CONSTRUCT_FILEREAD ================!
!This subroutine  reads in data from the CONSTRUCT file      !
!============================================================!

SUBROUTINE PENSWELF_CONSTRUCT_FILEREAD

    OPEN(UNIT=5, FILE=PENSWELF_CONSTRUCT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PENSWELF_CONSTRUCT
        WRITE(15,'("Could not find ", a100)') PENSWELF_CONSTRUCT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,I10,I10,A7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
            &F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,A7,A7,A7,&
            &A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,&
            &A7,A7,A7,A7,A7,A7)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CONSTRUCT(pers_j),MINPENS_NUMCYEAR(pers_j),INPENSCAD(pers_j),&
            &BEGINDATUM_PENSIOEN_C_2008(pers_j),RPWE_2008(pers_j),FIRPCAP_2008(pers_j),&
            &FISPCAP_2008(pers_j),RPSE_2008(pers_j),RPCIV_2008(pers_j),&
            &RPFOR_2008(pers_j),RPDOSZ_2008(pers_j),SPWE_2008(pers_j),&
            &SPSE_2008(pers_j),SPCIV_2008(pers_j),SPFOR_2008(pers_j),&
            &SPDOSZ_2008(pers_j),CAPFIRST_2008(pers_j),IGO_2008(pers_j),&
            &DISAB_2008(pers_j),RPRENT_SPILPRIV_2008(pers_j),RPRENT_SPILPUB_2008(pers_j),&
            &SPRENT_SPILPRIV_2008(pers_j),SPRENT_SPILPUB_2008(pers_j),RPCAP_SPILPRIV_2008(pers_j),&
            &RPCAP_SPILPUB_2008(pers_j),SPCAP_SPILPRIV_2008(pers_j),SPCAP_SPILPUB_2008(pers_j),&
            &DIRPENS_2008(pers_j),OTHPENS_2008(pers_j),RPWE_PDATE_2008(pers_j),&
            &FIRPCAP_PDATE_2008(pers_j),FISPCAP_PDATE_2008(pers_j),RPSE_PDATE_2008(pers_j),&
            &RPCIV_PDATE_2008(pers_j),RPFOR_PDATE_2008(pers_j),RPDOSZ_PDATE_2008(pers_j),&
            &SPWE_PDATE_2008(pers_j),SPSE_PDATE_2008(pers_j),SPCIV_PDATE_2008(pers_j),&
            &SPFOR_PDATE_2008(pers_j),SPDOSZ_PDATE_2008(pers_j),CAPFIRST_PDATE_2008(pers_j),&
            &IGO_PDATE_2008(pers_j),DISAB_PDATE_2008(pers_j),RPRENT_SPILPRIV_PDATE_2008(pers_j),&
            &RPRENT_SPILPUB_PDATE_2008(pers_j),SPRENT_SPILPRIV_PDATE_2008(pers_j),SPRENT_SPILPUB_PDATE_2008(pers_j),&
            &RPCAP_SPILPRIV_PDATE_2008(pers_j),RPCAP_SPILPUB_PDATE_2008(pers_j),SPCAP_SPILPRIV_PDATE_2008(pers_j),&
            &SPCAP_SPILPUB_PDATE_2008(pers_j),DIRPENS_PDATE_2008(pers_j),OTHPENS_PDATE_2008(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,PENSWELF_CONSTRUCT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CONSTRUCT(pers_j),MINPENS_NUMCYEAR(pers_j),INPENSCAD(pers_j),&
                &BEGINDATUM_PENSIOEN_C_2008(pers_j),RPWE_2008(pers_j),FIRPCAP_2008(pers_j),&
                &FISPCAP_2008(pers_j),RPSE_2008(pers_j),RPCIV_2008(pers_j),&
                &RPFOR_2008(pers_j),RPDOSZ_2008(pers_j),SPWE_2008(pers_j),&
                &SPSE_2008(pers_j),SPCIV_2008(pers_j),SPFOR_2008(pers_j),&
                &SPDOSZ_2008(pers_j),CAPFIRST_2008(pers_j),IGO_2008(pers_j),&
                &DISAB_2008(pers_j),RPRENT_SPILPRIV_2008(pers_j),RPRENT_SPILPUB_2008(pers_j),&
                &SPRENT_SPILPRIV_2008(pers_j),SPRENT_SPILPUB_2008(pers_j),RPCAP_SPILPRIV_2008(pers_j),&
                &RPCAP_SPILPUB_2008(pers_j),SPCAP_SPILPRIV_2008(pers_j),SPCAP_SPILPUB_2008(pers_j),&
                &DIRPENS_2008(pers_j),OTHPENS_2008(pers_j),RPWE_PDATE_2008(pers_j),&
                &FIRPCAP_PDATE_2008(pers_j),FISPCAP_PDATE_2008(pers_j),RPSE_PDATE_2008(pers_j),&
                &RPCIV_PDATE_2008(pers_j),RPFOR_PDATE_2008(pers_j),RPDOSZ_PDATE_2008(pers_j),&
                &SPWE_PDATE_2008(pers_j),SPSE_PDATE_2008(pers_j),SPCIV_PDATE_2008(pers_j),&
                &SPFOR_PDATE_2008(pers_j),SPDOSZ_PDATE_2008(pers_j),CAPFIRST_PDATE_2008(pers_j),&
                &IGO_PDATE_2008(pers_j),DISAB_PDATE_2008(pers_j),RPRENT_SPILPRIV_PDATE_2008(pers_j),&
                &RPRENT_SPILPUB_PDATE_2008(pers_j),SPRENT_SPILPRIV_PDATE_2008(pers_j),SPRENT_SPILPUB_PDATE_2008(pers_j),&
                &RPCAP_SPILPRIV_PDATE_2008(pers_j),RPCAP_SPILPUB_PDATE_2008(pers_j),SPCAP_SPILPRIV_PDATE_2008(pers_j),&
                &SPCAP_SPILPUB_PDATE_2008(pers_j),DIRPENS_PDATE_2008(pers_j),OTHPENS_PDATE_2008(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,A7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
                &F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,A7,A7,A7,&
                &A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,&
                &A7,A7,A7,A7,A7,A7)') &
                &GECODEERD_INSZ_CONSTRUCT(pers_j),MINPENS_NUMCYEAR(pers_j),INPENSCAD(pers_j),&
                &BEGINDATUM_PENSIOEN_C_2008(pers_j),RPWE_2008(pers_j),FIRPCAP_2008(pers_j),&
                &FISPCAP_2008(pers_j),RPSE_2008(pers_j),RPCIV_2008(pers_j),&
                &RPFOR_2008(pers_j),RPDOSZ_2008(pers_j),SPWE_2008(pers_j),&
                &SPSE_2008(pers_j),SPCIV_2008(pers_j),SPFOR_2008(pers_j),&
                &SPDOSZ_2008(pers_j),CAPFIRST_2008(pers_j),IGO_2008(pers_j),&
                &DISAB_2008(pers_j),RPRENT_SPILPRIV_2008(pers_j),RPRENT_SPILPUB_2008(pers_j),&
                &SPRENT_SPILPRIV_2008(pers_j),SPRENT_SPILPUB_2008(pers_j),RPCAP_SPILPRIV_2008(pers_j),&
                &RPCAP_SPILPUB_2008(pers_j),SPCAP_SPILPRIV_2008(pers_j),SPCAP_SPILPUB_2008(pers_j),&
                &DIRPENS_2008(pers_j),OTHPENS_2008(pers_j),RPWE_PDATE_2008(pers_j),&
                &FIRPCAP_PDATE_2008(pers_j),FISPCAP_PDATE_2008(pers_j),RPSE_PDATE_2008(pers_j),&
                &RPCIV_PDATE_2008(pers_j),RPFOR_PDATE_2008(pers_j),RPDOSZ_PDATE_2008(pers_j),&
                &SPWE_PDATE_2008(pers_j),SPSE_PDATE_2008(pers_j),SPCIV_PDATE_2008(pers_j),&
                &SPFOR_PDATE_2008(pers_j),SPDOSZ_PDATE_2008(pers_j),CAPFIRST_PDATE_2008(pers_j),&
                &IGO_PDATE_2008(pers_j),DISAB_PDATE_2008(pers_j),RPRENT_SPILPRIV_PDATE_2008(pers_j),&
                &RPRENT_SPILPUB_PDATE_2008(pers_j),SPRENT_SPILPRIV_PDATE_2008(pers_j),SPRENT_SPILPUB_PDATE_2008(pers_j),&
                &RPCAP_SPILPRIV_PDATE_2008(pers_j),RPCAP_SPILPUB_PDATE_2008(pers_j),SPCAP_SPILPRIV_PDATE_2008(pers_j),&
                &SPCAP_SPILPUB_PDATE_2008(pers_j),DIRPENS_PDATE_2008(pers_j),OTHPENS_PDATE_2008(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,A7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
                &F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,A7,A7,A7,&
                &A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,A7,&
                &A7,A7,A7,A7,A7,A7)') &
                &GECODEERD_INSZ_CONSTRUCT(pers_j),MINPENS_NUMCYEAR(pers_j),INPENSCAD(pers_j),&
                &BEGINDATUM_PENSIOEN_C_2008(pers_j),RPWE_2008(pers_j),FIRPCAP_2008(pers_j),&
                &FISPCAP_2008(pers_j),RPSE_2008(pers_j),RPCIV_2008(pers_j),&
                &RPFOR_2008(pers_j),RPDOSZ_2008(pers_j),SPWE_2008(pers_j),&
                &SPSE_2008(pers_j),SPCIV_2008(pers_j),SPFOR_2008(pers_j),&
                &SPDOSZ_2008(pers_j),CAPFIRST_2008(pers_j),IGO_2008(pers_j),&
                &DISAB_2008(pers_j),RPRENT_SPILPRIV_2008(pers_j),RPRENT_SPILPUB_2008(pers_j),&
                &SPRENT_SPILPRIV_2008(pers_j),SPRENT_SPILPUB_2008(pers_j),RPCAP_SPILPRIV_2008(pers_j),&
                &RPCAP_SPILPUB_2008(pers_j),SPCAP_SPILPRIV_2008(pers_j),SPCAP_SPILPUB_2008(pers_j),&
                &DIRPENS_2008(pers_j),OTHPENS_2008(pers_j),RPWE_PDATE_2008(pers_j),&
                &FIRPCAP_PDATE_2008(pers_j),FISPCAP_PDATE_2008(pers_j),RPSE_PDATE_2008(pers_j),&
                &RPCIV_PDATE_2008(pers_j),RPFOR_PDATE_2008(pers_j),RPDOSZ_PDATE_2008(pers_j),&
                &SPWE_PDATE_2008(pers_j),SPSE_PDATE_2008(pers_j),SPCIV_PDATE_2008(pers_j),&
                &SPFOR_PDATE_2008(pers_j),SPDOSZ_PDATE_2008(pers_j),CAPFIRST_PDATE_2008(pers_j),&
                &IGO_PDATE_2008(pers_j),DISAB_PDATE_2008(pers_j),RPRENT_SPILPRIV_PDATE_2008(pers_j),&
                &RPRENT_SPILPUB_PDATE_2008(pers_j),SPRENT_SPILPRIV_PDATE_2008(pers_j),SPRENT_SPILPUB_PDATE_2008(pers_j),&
                &RPCAP_SPILPRIV_PDATE_2008(pers_j),RPCAP_SPILPUB_PDATE_2008(pers_j),SPCAP_SPILPRIV_PDATE_2008(pers_j),&
                &SPCAP_SPILPUB_PDATE_2008(pers_j),DIRPENS_PDATE_2008(pers_j),OTHPENS_PDATE_2008(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine PENSWELF_CONSTRUCT_FILEREAD of PRE RUN.....runned correctly'
    WRITE(15,'(I7, " records uploaded from PENSWELF_CONSTRUCT")') pers_j-1

END SUBROUTINE PENSWELF_CONSTRUCT_FILEREAD

!============= SUBROUTINE SICK_FILEREAD ================!
!This subroutine  reads in data from the SICK file      !
!=======================================================!
SUBROUTINE PENSWELF_SICK_FILEREAD

    OPEN(UNIT=5, FILE=PENSWELF_SICK, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PENSWELF_SICK
        WRITE(15,'("Could not find ", a100)') PENSWELF_SICK
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_SICK(pers_j),SICK_TOTBEN_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,PENSWELF_SICK
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_SICK(pers_j),SICK_TOTBEN_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_TOTBEN_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_TOTBEN_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine PENSWELF_SICK_FILEREAD of PRE RUN..........runned correctly'
    WRITE(15,'(I7, " records uploaded from SICK_FILE")') pers_j-1

END SUBROUTINE PENSWELF_SICK_FILEREAD

!============= SUBROUTINE UNEM_FILEREAD ================!
!This subroutine  reads in data from the UNEM file      !
!=======================================================!
SUBROUTINE PENSWELF_UNEM_FILEREAD

    OPEN(UNIT=5, FILE=PENSWELF_UNEM, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PENSWELF_UNEM
        WRITE(15,'("Could not find ", a100)') PENSWELF_UNEM
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_UNEM(pers_j),UNEM_TOTBEN_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,PENSWELF_UNEM
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_UNEM(pers_j),UNEM_TOTBEN_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_TOTBEN_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_TOTBEN_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine PENSWELF_UNEM_FILEREAD of PRE RUN..........runned correctly'
    WRITE(15,'(I7, " records uploaded from UNEM_FILE")') pers_j-1

END SUBROUTINE PENSWELF_UNEM_FILEREAD

!============= SUBROUTINE CONTRIB_FILEREAD ================!
!This subroutine reads in data from the CONTRIB file       !
!==========================================================!
SUBROUTINE PENSWELF_CONTRIB_FILEREAD

    OPEN(UNIT=5, FILE=PENSWELF_CONTRIB, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PENSWELF_CONTRIB
        WRITE(15,'("Could not find ", a100)') PENSWELF_CONTRIB
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),CONTRIB_INCSELF_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,PENSWELF_CONTRIB
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),&
                &CONTRIB_INCSELF_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),CONTRIB_INCSELF_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),CONTRIB_INCSELF_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine PENSWELF_CONTRIB_FILEREAD of PRE RUN.......runned correctly'
    WRITE(15,'(I7, " records uploaded from CONTRIB_FILE")') pers_j-1

END SUBROUTINE PENSWELF_CONTRIB_FILEREAD

!=====================================!
!Reads in data from the FAMREL file   !
!=====================================!

SUBROUTINE PENSWELF_FAMREL_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=MIMOSIS_FAMREL, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') MIMOSIS_FAMREL
        WRITE (15,'("Could not find or open ", a100)') MIMOSIS_FAMREL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
            &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_FAMREL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine PENSWELF_FAMREL_FILEREAD of PRE RUN........runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_FAMREL")') pers_j-1

END SUBROUTINE PENSWELF_FAMREL_FILEREAD

!=======================================!
!Reads in data from the WAGE INPUT file !
!=======================================!

SUBROUTINE PENSWELF_WAGE_FILEREAD

    OPEN (UNIT=5, FILE=WAGE_CONSTRUCT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') WAGE_CONSTRUCT
        WRITE (15,'("Could not find or open ", a100)') WAGE_CONSTRUCT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
            &MIMOSIS_GRINC_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,WAGE_CONSTRUCT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine PENSWELF_WAGE_FILEREAD of PRE RUN..........runned correctly'
    WRITE (15,'(I7, " records uploaded from WAGE_CONSTRUCT")') pers_j-1 

END SUBROUTINE PENSWELF_WAGE_FILEREAD

!===========================================!
!Reads in data from the MIMOSIS_WEIGHT file !
!===========================================!

SUBROUTINE PENSWELF_WEIGHT_FILEREAD

    OPEN (UNIT=5, FILE=MIMOSIS_SAMPWEIG, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') MIMOSIS_SAMPWEIG
        WRITE (15,'("Could not find or open ", a100)') MIMOSIS_SAMPWEIG
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_SAMPWEIG
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine PENSWELF_WEIGHT_FILEREAD of PRE RUN........runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_SAMPWEIG")') pers_j-1 

END SUBROUTINE PENSWELF_WEIGHT_FILEREAD

!=======================================================================!
!Check whether individual identification numbers are sorted consistently!
!=======================================================================!

SUBROUTINE PENSWELF_CONSISTENCY_CHECK

    INTEGER:: pers_j
    
    DO pers_j=1, n_rec
        IF (&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_CONSTRUCT(pers_j))     .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))          .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))          .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))       .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))        .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))          .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))        .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))      .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))      .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))   .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))      .OR.&
            &(GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))    .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))           .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))        .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))         .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))           .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))         .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))        .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))         .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))           .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))         .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))      .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))        .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))      .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))         .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))       .OR.&
            &(GECODEERD_INSZ_WAGE(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))         &
            &) THEN
                WRITE (*,*) "Error: Ranking of external files does not match!"
                PAUSE
                STOP
        END IF
    END DO   
   
END SUBROUTINE PENSWELF_CONSISTENCY_CHECK 

!============= SUBROUTINE PENSWELF_RECODE=========================!
!This subroutines treat missing value                          !
!==============================================================!
SUBROUTINE PENSWELF_RECODE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        !Exogenous variables
        !PENSWELF_INPUT_WRITE.SAS
        IF (INT(NAREGNIS_RELATION(pers_j)).EQ.9999999)          NAREGNIS_RELATION(pers_j)=0
        IF (INT(NAREGNIS_SEX(pers_j)).EQ.9999999)               NAREGNIS_SEX(pers_j)=0
        IF (INT(NAREGNIS_HH_ID(pers_j)).EQ.9999999)             NAREGNIS_HH_ID(pers_j)=0
        CALL TRIM@(NAREGNIS_DATBIRTH(pers_j))
        !PENSWELF_CONSTRUCT_WRITE.SAS
        IF (INT(MINPENS_NUMCYEAR(pers_j)).EQ.9999999)           MINPENS_NUMCYEAR(pers_j)=0
        IF (INT(INPENSCAD(pers_j)).EQ.9999999)                  INPENSCAD(pers_j)=0
        CALL TRIM@(BEGINDATUM_PENSIOEN_C_2008(pers_j))
        !/*BEDRAGEN UIT PENSIOENKADASTER*/
        IF (INT(RPWE_2008(pers_j)).EQ.9999999)                  RPWE_2008(pers_j)=0
        IF (INT(FIRPCAP_2008(pers_j)).EQ.9999999)               FIRPCAP_2008(pers_j)=0
        IF (INT(FISPCAP_2008(pers_j)).EQ.9999999)               FISPCAP_2008(pers_j)=0
        IF (INT(RPSE_2008(pers_j)).EQ.9999999)                  RPSE_2008(pers_j)=0
        IF (INT(RPCIV_2008(pers_j)).EQ.9999999)                 RPCIV_2008(pers_j)=0
        IF (INT(RPFOR_2008(pers_j)).EQ.9999999)                 RPFOR_2008(pers_j)=0
        IF (INT(RPDOSZ_2008(pers_j)).EQ.9999999)                RPDOSZ_2008(pers_j)=0
        IF (INT(SPWE_2008(pers_j)).EQ.9999999)                  SPWE_2008(pers_j)=0
        IF (INT(SPSE_2008(pers_j)).EQ.9999999)                  SPSE_2008(pers_j)=0
        IF (INT(SPCIV_2008(pers_j)).EQ.9999999)                 SPCIV_2008(pers_j)=0
        IF (INT(SPFOR_2008(pers_j)).EQ.9999999)                 SPFOR_2008(pers_j)=0
        IF (INT(SPDOSZ_2008(pers_j)).EQ.9999999)                SPDOSZ_2008(pers_j)=0
        IF (INT(CAPFIRST_2008(pers_j)).EQ.9999999)              CAPFIRST_2008(pers_j)=0
        IF (INT(IGO_2008(pers_j)).EQ.9999999)                   IGO_2008(pers_j)=0
        IF (INT(DISAB_2008(pers_j)).EQ.9999999)                 DISAB_2008(pers_j)=0
        IF (INT(RPRENT_SPILPRIV_2008(pers_j)).EQ.9999999)       RPRENT_SPILPRIV_2008(pers_j)=0
        IF (INT(RPRENT_SPILPUB_2008(pers_j)).EQ.9999999)        RPRENT_SPILPUB_2008(pers_j)=0
        IF (INT(SPRENT_SPILPRIV_2008(pers_j)).EQ.9999999)       SPRENT_SPILPRIV_2008(pers_j)=0
        IF (INT(SPRENT_SPILPUB_2008(pers_j)).EQ.9999999)        SPRENT_SPILPUB_2008(pers_j)=0
        IF (INT(RPCAP_SPILPRIV_2008(pers_j)).EQ.9999999)        RPCAP_SPILPRIV_2008(pers_j)=0
        IF (INT(RPCAP_SPILPUB_2008(pers_j)).EQ.9999999)         RPCAP_SPILPUB_2008(pers_j)=0
        IF (INT(SPCAP_SPILPRIV_2008(pers_j)).EQ.9999999)        SPCAP_SPILPRIV_2008(pers_j)=0
        IF (INT(SPCAP_SPILPUB_2008(pers_j)).EQ.9999999)         SPCAP_SPILPUB_2008(pers_j)=0
        IF (INT(DIRPENS_2008(pers_j)).EQ.9999999)               DIRPENS_2008(pers_j)=0
        IF (INT(OTHPENS_2008(pers_j)).EQ.9999999)               OTHPENS_2008(pers_j)=0
        !/*DATUMS UIT PENSIOENKADASTER*/
        CALL TRIM@(RPWE_PDATE_2008(pers_j))
        CALL TRIM@(FIRPCAP_PDATE_2008(pers_j))
        CALL TRIM@(FISPCAP_PDATE_2008(pers_j))
        CALL TRIM@(RPSE_PDATE_2008(pers_j))
        CALL TRIM@(RPCIV_PDATE_2008(pers_j))
        CALL TRIM@(RPFOR_PDATE_2008(pers_j))
        CALL TRIM@(RPDOSZ_PDATE_2008(pers_j))
        CALL TRIM@(SPWE_PDATE_2008(pers_j))
        CALL TRIM@(SPSE_PDATE_2008(pers_j))
        CALL TRIM@(SPCIV_PDATE_2008(pers_j))
        CALL TRIM@(SPFOR_PDATE_2008(pers_j))
        CALL TRIM@(SPDOSZ_PDATE_2008(pers_j))
        CALL TRIM@(CAPFIRST_PDATE_2008(pers_j))
        CALL TRIM@(IGO_PDATE_2008(pers_j))
        CALL TRIM@(DISAB_PDATE_2008(pers_j))
        CALL TRIM@(RPRENT_SPILPRIV_PDATE_2008(pers_j))
        CALL TRIM@(RPRENT_SPILPUB_PDATE_2008(pers_j))
        CALL TRIM@(SPRENT_SPILPRIV_PDATE_2008(pers_j))
        CALL TRIM@(SPRENT_SPILPUB_PDATE_2008(pers_j))
        CALL TRIM@(RPCAP_SPILPRIV_PDATE_2008(pers_j))
        CALL TRIM@(RPCAP_SPILPUB_PDATE_2008(pers_j))
        CALL TRIM@(SPCAP_SPILPRIV_PDATE_2008(pers_j))
        CALL TRIM@(SPCAP_SPILPUB_PDATE_2008(pers_j))
        CALL TRIM@(DIRPENS_PDATE_2008(pers_j))
        CALL TRIM@(OTHPENS_PDATE_2008(pers_j))
        !Variables read from file PENSWELF_CONTRIB.txt
        IF (INT(CONTRIB_REVPRIV_QT(pers_j)).EQ.9999999)         CONTRIB_REVPRIV_QT(pers_j)=0
        IF (INT(CONTRIB_REVPUB_QT(pers_j)).EQ.9999999)          CONTRIB_REVPUB_QT(pers_j)=0.
        IF (INT(CONTRIB_INCSELF_QT(pers_j)).EQ.9999999)         CONTRIB_INCSELF_QT(pers_j)=0
        !Variables read from file PENSWELF_UNEM.txt               
        IF (INT(UNEM_TOTBEN_QT(pers_j)).EQ.9999999)             UNEM_TOTBEN_QT(pers_j)=0
        !Variables read from file PENSWELF_SICK.txt               
        IF (INT(SICK_TOTBEN_QT(pers_j)).EQ.9999999)             SICK_TOTBEN_QT(pers_j)=0
        !Variables read from file MIMOSIS_FAMREL.txt                                         
        !WAGE_CONSTRUCT_WRITE.SAS
        IF (INT(SALJRS_LAST_CORR(pers_j)).EQ.9999999)           SALJRS_LAST_CORR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_HOUR(pers_j)).EQ.9999999)         MIMOSIS_GRINC_HOUR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_QT(pers_j)).EQ.9999999)           MIMOSIS_GRINC_QT(pers_j)=0
        !MIMOSIS_SAMPWEIG_WRITE.SAS
        IF (INT(MIMOSIS_WEIGHT(pers_j)).EQ.9999999)             MIMOSIS_WEIGHT(pers_j)=0
    END DO   

END SUBROUTINE PENSWELF_RECODE

!============= SUBROUTINE PENSWELF_INFLATE==============================!
!This subroutines inflates nominal values to an appropriate price level !
!=======================================================================!
SUBROUTINE PENSWELF_INFLATE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        !Exogenous variables
        !PENSWELF_INPUT_WRITE.SAS
        !PENSWELF_CONSTRUCT_WRITE.SAS
        !/*BEDRAGEN UIT PENSIOENKADASTER*/
        RPWE_2008(pers_j)           =INFLATE_NOMVAL(2)*RPWE_2008(pers_j)   
        RPSE_2008(pers_j)           =INFLATE_NOMVAL(2)*RPSE_2008(pers_j)   
        RPCIV_2008(pers_j)          =INFLATE_NOMVAL(2)*RPCIV_2008(pers_j)  
        RPFOR_2008(pers_j)          =INFLATE_NOMVAL(2)*RPFOR_2008(pers_j)  
        RPDOSZ_2008(pers_j)         =INFLATE_NOMVAL(2)*RPDOSZ_2008(pers_j) 
        SPWE_2008(pers_j)           =INFLATE_NOMVAL(2)*SPWE_2008(pers_j)   
        SPSE_2008(pers_j)           =INFLATE_NOMVAL(2)*SPSE_2008(pers_j)   
        SPCIV_2008(pers_j)          =INFLATE_NOMVAL(2)*SPCIV_2008(pers_j)  
        SPFOR_2008(pers_j)          =INFLATE_NOMVAL(2)*SPFOR_2008(pers_j)  
        SPDOSZ_2008(pers_j)         =INFLATE_NOMVAL(2)*SPDOSZ_2008(pers_j) 
        DISAB_2008(pers_j)          =INFLATE_NOMVAL(2)*DISAB_2008(pers_j)  
        DIRPENS_2008(pers_j)        =INFLATE_NOMVAL(2)*DIRPENS_2008(pers_j)
        OTHPENS_2008(pers_j)        =INFLATE_NOMVAL(2)*OTHPENS_2008(pers_j)
        !/*DATUMS UIT PENSIOENKADASTER*/
        !Variables read from file PENSWELF_CONTRIB.txt
        CONTRIB_REVPRIV_QT(pers_j)  =INFLATE_NOMVAL(1)*CONTRIB_REVPRIV_QT(pers_j) 
        CONTRIB_REVPUB_QT(pers_j)   =INFLATE_NOMVAL(1)*CONTRIB_REVPUB_QT(pers_j)
        CONTRIB_INCSELF_QT(pers_j)  =INFLATE_NOMVAL(1)*CONTRIB_INCSELF_QT(pers_j) 
        !Variables read from file PENSWELF_UNEM.txt               
        UNEM_TOTBEN_QT(pers_j)      =INFLATE_NOMVAL(3)*UNEM_TOTBEN_QT(pers_j)
        !Variables read from file PENSWELF_SICK.txt               
        SICK_TOTBEN_QT(pers_j)      =INFLATE_NOMVAL(3)*SICK_TOTBEN_QT(pers_j)
        !Variables read from file MIMOSIS_FAMREL.txt                                         
        !WAGE_CONSTRUCT_WRITE.SAS
        SALJRS_LAST_CORR(pers_j)    =INFLATE_NOMVAL(1)*SALJRS_LAST_CORR(pers_j)
        MIMOSIS_GRINC_HOUR(pers_j)  =INFLATE_NOMVAL(1)*MIMOSIS_GRINC_HOUR(pers_j)
        MIMOSIS_GRINC_QT(pers_j)    =INFLATE_NOMVAL(1)*MIMOSIS_GRINC_QT(pers_j)
        !MIMOSIS_SAMPWEIG_WRITE.SAS
    END DO   

END SUBROUTINE PENSWELF_INFLATE

!===============================================!
! HOUSEHOLD_SIZE computes household size        !
! and identifies starting and ending record of  !
! sociological household and stores it in a     !
! vector with household information             !
!Constructs variables                           !
!   PENSWELF_HH_SIZE                            !
!   PENSWELF_START_HH                           !
!   PENSWELF_END_HH                             !
!   N_HH_PENSWELF                               !
!===============================================!

SUBROUTINE PENSWELF_HOUSEHOLD_SIZE

    INTEGER::  I
    INTEGER::  J
    INTEGER::  K
    INTEGER::  N

    I=1
    J=1
    K=0
    N=0

    DO
        J=J+N
        N=0
        K=K+1
        DO WHILE (NAREGNIS_HH_ID(i)==NAREGNIS_HH_ID(j))
            N=N+1
            I=I+1
            IF (i>n_rec) EXIT    
        END DO
        PENSWELF_HH_SIZE(k)=n
        PENSWELF_START_HH(k)=j
        PENSWELF_END_HH(k)=j+n-1
        IF (i>n_rec.OR.j>n_rec)EXIT
    END DO
    N_HH_PENSWELF=K

END SUBROUTINE PENSWELF_HOUSEHOLD_SIZE

!=================================================================
!Computes the age of each individual in terms of years and months!
!=================================================================

!===============================================!
! Subroutine main input : NAREGNIS_DATBIRTH     ! 
! Subroutine main output: PENSWELF_YEAR_AGE     !
!                         PENSWELF_MONTH_AGE    !
!                         PENSWELF_YEAR_BIRTH   !
!                         PENSWELF_MONTH_BIRTH  !   
!===============================================!

SUBROUTINE PENSWELF_AGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        PENSWELF_PAR_BREAK_POINT(1)=INDEX(NAREGNIS_DATBIRTH(pers_j),'-')
        IF (PENSWELF_PAR_BREAK_POINT(1).GT.1) THEN
            READ (NAREGNIS_DATBIRTH(pers_j)(1:PENSWELF_PAR_BREAK_POINT(1)-1),  '(I10)') PENSWELF_MONTH_BIRTH(pers_j)
            READ (NAREGNIS_DATBIRTH(pers_j)(PENSWELF_PAR_BREAK_POINT(1)+1:),   '(I10)') PENSWELF_YEAR_BIRTH(pers_j)          
        END IF
        !determine age as number of years at the end of the year for which data are registered
        PENSWELF_YEAR_AGE(pers_j)  =MIMOSIS_YEAR-PENSWELF_YEAR_BIRTH(pers_j)
        !determine number of months as year age plus number of months passed until the end of the year
        PENSWELF_MONTH_AGE(pers_j) =PENSWELF_YEAR_AGE(pers_j)*12+(12-PENSWELF_MONTH_BIRTH(pers_j))
    END DO

END SUBROUTINE PENSWELF_AGE

!===================================================================================!
!Draw a line of a give number of characters
!===================================================================================!

SUBROUTINE DRAWLINE(NUNIT,N,WRITECHAR)

    ! DRAWS A LINE OF N CHARACTERS OF TYPE WRITECHAR ON UNIT NUNIT
    INTEGER                                                 :: N                                    !Number of characters to be drawn
    INTEGER                                                 :: NUNIT                                    !Number of characters to be drawn
    CHARACTER(len=1)                                        :: WRITECHAR                            !Type of character that has to be reproduced N times
    CHARACTER(len=1),DIMENSION(N)                           :: A                                    !Vector of dimension N containing WRITECHAR in the N positions
    CHARACTER(len=15)                                       :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=7)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK

    A(1:N)=WRITECHAR
    !convert the integer value N into a character value NCHAR with the Fortran internal Write function                                                        
    WRITE (NCHAR, '(I7)') N
    !Construct the output format by aid of the internal Fortran function NONBLK which identifies the first non blank character
    !and read the remainder of the string, i.e. :
    OUTPUTFMT = '(' // NCHAR(NONBLK(NCHAR):) // 'A1)'
    WRITE(NUNIT,OUTPUTFMT)(A(I),I=1,N)

END SUBROUTINE DRAWLINE

!======================================================!
!subroutine to skip n lines on file k (reads as blanks)
!======================================================!
SUBROUTINE RSKIP(k,n)

    INTEGER :: k,n

    DO i=1,n
        read(k,'(a)')
    END DO    

END SUBROUTINE RSKIP

!===================================================================================!
!With subroutine PENSWELF_CHARAC we reconstruct variables 
!used troughout the different subroutines for computing welfare adaptations of pension benefits
!Constructed variables
! PENSWELF_NUMMONTH_ANN                
! PENSWELF_BENBEFOR_MONTH              
! PENSWELF_TOTPBEF_QT                  
! PENSWELF_INDINC_QT                   
! PENSWELF_REPRATE_QT                  
! PENSWELF_WEMIPENS_ANN                
! PENSWELF_SEMIPENS_ANN                
! PENSWELF_CSMIPENS_ANN                
!===================================================================================!

SUBROUTINE PENSWELF_CHARAC

    INTEGER                             :: pers_j,pers_k,pers_l,i,lijn_j
    REAL(KIND=KIND_REAL)                :: INC_PARTNER                          !intermediate variable to store the income of the partner
    REAL(KIND=KIND_REAL)                :: REFVAL_REP                           !intermediate variable to store the reference value in case of minium pensions
    REAL(KIND=KIND_REAL)                :: REALVAL_REP                          !intermediate variable to store the real monthly benefit in case of minium pensions
    INTEGER,DIMENSION(5)                :: COUNT_REPTYPE
    REAL(KIND=KIND_REAL),DIMENSION(5)   :: POPU_REPTYPE

    !PENSWELF_TOTPBEF_QT: Total sum of pension benefits received per quarter before welfare adaptations have been applied (sum of all retirement and survival benefits, but no guaranteed pension benefits)
    DO pers_j=1,n_rec
        PENSWELF_TOTPBEF_QT(pers_j) =   &
        &(  RPWE_2008(pers_j)   +&                            ! rustpensioen werknemer (betalingsjaar = 2008)
            RPSE_2008(pers_j)   +&                            ! rustpensioen zelfstandige (betalingsjaar = 2008)
            RPCIV_2008(pers_j)  +&                            ! rustpensioen overheid (betalingsjaar = 2008)
            RPFOR_2008(pers_j)  +&                            ! buitenlands rustpensioen (betalingsjaar = 2008)
            RPDOSZ_2008(pers_j) +&                            ! rustpensioen DOSZ (betalingsjaar = 2008)
            SPWE_2008(pers_j)   +&                            ! overlevingspensioen werknemer (betalingsjaar = 2008)
            SPSE_2008(pers_j)   +&                            ! overlevingspensioen zelfstandige (betalingsjaar = 2008)
            SPCIV_2008(pers_j)  +&                            ! overlevingspensioen overheid (betalingsjaar = 2008)
            SPFOR_2008(pers_j)  +&                            ! buitenlands overlevingspensioen (betalingsjaar = 2008)
            SPDOSZ_2008(pers_j) +&                            ! overlevingspensioen DOSZ (betalingsjaar = 2008)
            DISAB_2008(pers_j)  +&                            ! tegemoetkoming mindervaliden (betalingsjaar = 2008)
            DIRPENS_2008(pers_j)+&                            ! afgeleide pensioenen (betalingsjaar = 2008)
            OTHPENS_2008(pers_j))/4.D0                        ! overige pensioenbedragen (betalingsjaar = 2008)
    END DO

    !PENSWELF_NUMMONTH_ANN: Number of months for which the pensioner already receives a pension benefit on January 1st 2009
    !PENSWELF_NUMMONTH_ANN is a matrix with 13 columns, one for each of the 13 reconstructed benefit types
    DO pers_j=1,n_rec
        !if the individual is a pensioner, i.e. receives pension benefits
        IF (InPensCad(pers_j).NE.0) THEN
            ! rustpensioen werknemer (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,1) = PENSWELF_DATE(RPWE_PDATE_2008(pers_j))
            ! rustpensioen zelfstandige (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,2) = PENSWELF_DATE(RPSE_PDATE_2008(pers_j))                           
            ! rustpensioen overheid (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,3) = PENSWELF_DATE(RPCIV_PDATE_2008(pers_j))                          
            ! buitenlands rustpensioen (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,4) = PENSWELF_DATE(RPFOR_PDATE_2008(pers_j))                          
            ! rustpensioen DOSZ (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,5) = PENSWELF_DATE(RPDOSZ_PDATE_2008(pers_j))                         
            ! overlevingspensioen werknemer (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,6) = PENSWELF_DATE(SPWE_PDATE_2008(pers_j))                           
            ! overlevingspensioen zelfstandige (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,7) = PENSWELF_DATE(SPSE_PDATE_2008(pers_j))                           
            ! overlevingspensioen overheid (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,8) = PENSWELF_DATE(SPCIV_PDATE_2008(pers_j))                          
            ! buitenlands overlevingspensioen (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,9) = PENSWELF_DATE(SPFOR_PDATE_2008(pers_j))                          
            ! overlevingspensioen DOSZ (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,10) = PENSWELF_DATE(SPDOSZ_PDATE_2008(pers_j))                         
            ! tegemoetkoming mindervaliden (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,11) = PENSWELF_DATE(DISAB_PDATE_2008(pers_j))                          
            ! afgeleide pensioenen (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,12) = PENSWELF_DATE(DIRPENS_PDATE_2008(pers_j))                        
            ! overige pensioenbedragen (betalingsjaar = 2008)
            PENSWELF_NUMMONTH_ANN(pers_j,13) = PENSWELF_DATE(OTHPENS_PDATE_2008(pers_j))                        
        END IF
    END DO    

    !PENSWELF_BENBEFOR_MONTH: Monthly pension benefit paid before welfare adaptation in benefit scheme 1 to 14
    !PENSWELF_BENBEFOR_MONTH is a matrix with 14 columns, one for each of the 14 reconstructed benefit types
    DO pers_j=1,n_rec
        !if the individual is a pensioner, i.e. receives pension benefits
        IF (InPensCad(pers_j).NE.0) THEN
            ! rustpensioen werknemer (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,1) = PENSWELF_MONTHBEN(RPWE_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,1))
            ! rustpensioen zelfstandige (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,2) = PENSWELF_MONTHBEN(RPSE_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,2))                           
            ! rustpensioen overheid (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,3) = PENSWELF_MONTHBEN(RPCIV_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,3))                          
            ! buitenlands rustpensioen (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,4) = PENSWELF_MONTHBEN(RPFOR_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,4))                          
            ! rustpensioen DOSZ (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,5) = PENSWELF_MONTHBEN(RPDOSZ_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,5))                         
            ! overlevingspensioen werknemer (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,6) = PENSWELF_MONTHBEN(SPWE_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,6))                           
            ! overlevingspensioen zelfstandige (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,7) = PENSWELF_MONTHBEN(SPSE_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,7))                           
            ! overlevingspensioen overheid (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,8) = PENSWELF_MONTHBEN(SPCIV_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,8))                          
            ! buitenlands overlevingspensioen (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,9) = PENSWELF_MONTHBEN(SPFOR_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,9))                          
            ! overlevingspensioen DOSZ (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,10) = PENSWELF_MONTHBEN(SPDOSZ_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,10))                         
            ! tegemoetkoming mindervaliden (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,11) = PENSWELF_MONTHBEN(DISAB_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,11))                          
            ! afgeleide pensioenen (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,12) = PENSWELF_MONTHBEN(DIRPENS_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,12))                        
            ! overige pensioenbedragen (betalingsjaar = 2008)
            PENSWELF_BENBEFOR_MONTH(pers_j,13) = PENSWELF_MONTHBEN(OTHPENS_2008(pers_j),PENSWELF_NUMMONTH_ANN(pers_j,13))                        
        END IF
    END DO    

    !Do check on the dates and amounts to be sure that each amount corresponds at leas with one monthly payment
    DO pers_j=1,n_rec
        !if the individual is a pensioner, i.e. receives pension benefits
        IF (InPensCad(pers_j).NE.0) THEN
            DO i=1,13
                IF (PENSWELF_BENBEFOR_MONTH(pers_j,i).NE.0.AND.PENSWELF_NUMMONTH_ANN(pers_j,i).EQ.0) THEN
                    PENSWELF_NUMMONTH_ANN(pers_j,i) = 1   
                END IF
            END DO
        END IF
    END DO
    
    !PENSWELF_INDINC_QT: Total sum of labour market and replacement income received by an individual
    DO pers_j=1,n_rec
        PENSWELF_INDINC_QT(pers_j) =        &
        &    PENSWELF_TOTPBEF_QT(pers_j)    +&
        &    CONTRIB_REVPRIV_QT(pers_j)     +&
        &    CONTRIB_REVPUB_QT(pers_j)      +&
        &    CONTRIB_INCSELF_QT(pers_j)     +&
        &    UNEM_TOTBEN_QT(pers_j)         +&
        &    SICK_TOTBEN_QT(pers_j)
    END DO

    !PENSWELF_REPRATE_QT: Replacement rate applied to compute pension benefit
    COUNT_REPTYPE(1:5) = 0
    POPU_REPTYPE(1:5) = 0
    DO hh_i=1,N_HH_PENSWELF
        !Loop over all persons within the household
        DO pers_j=PENSWELF_START_HH(hh_i),PENSWELF_END_HH(hh_i)
            !if an individual is a pensioner, i.e. receives pension benefits
            IF (InPensCad(pers_j).NE.0) THEN
                !if the replacement rate has not been determined yet
                IF (PENSWELF_REPRATE_QT(pers_j).EQ.0) THEN
                    !check whether this individual has a partner with which there is a legal marriage
                    !if not, give the individual the replacement rate of a single
                    IF (FAMREL_COUPLE(pers_j).NE.1) THEN
                        PENSWELF_REPRATE_QT(pers_j) = 60
                        COUNT_REPTYPE(1) = COUNT_REPTYPE(1) + 1
                        POPU_REPTYPE(1) = POPU_REPTYPE(1) + MIMOSIS_WEIGHT(pers_j)
                    ELSE
                    !if there is a partner, look up the partner and check its income sources
                        INC_PARTNER = 0
                        DO pers_k=PENSWELF_START_HH(hh_i),PENSWELF_END_HH(hh_i)
                            !either j is partner 1
                            IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                            &(FAMREL_RELATION(pers_j).EQ.1.AND.FAMREL_RELATION(pers_k).EQ.2)) THEN
                                INC_PARTNER = PENSWELF_INDINC_QT(pers_k)
                                !if the partner has an own income source then the pensioner does not get the household rate
                                IF (INC_PARTNER.NE.0) THEN
                                    PENSWELF_REPRATE_QT(pers_j) = 60
                                    COUNT_REPTYPE(2) = COUNT_REPTYPE(2) + 1
                                    POPU_REPTYPE(2) = POPU_REPTYPE(2) + MIMOSIS_WEIGHT(pers_j)
                                    IF (InPensCad(pers_j).NE.0) THEN
                                        PENSWELF_REPRATE_QT(pers_k) = 60
                                        COUNT_REPTYPE(3) = COUNT_REPTYPE(3) + 1
                                        POPU_REPTYPE(3) = POPU_REPTYPE(3) + MIMOSIS_WEIGHT(pers_k)
                                    END IF
                                !if the partner has an no income source then the pensioner gets the household rate
                                ELSE
                                    PENSWELF_REPRATE_QT(pers_j) = 75
                                    COUNT_REPTYPE(4) = COUNT_REPTYPE(4) + 1
                                    POPU_REPTYPE(4) = POPU_REPTYPE(4) + MIMOSIS_WEIGHT(pers_j)
                                    IF (InPensCad(pers_j).NE.0) THEN
                                        PENSWELF_REPRATE_QT(pers_k) = 60
                                        COUNT_REPTYPE(5) = COUNT_REPTYPE(5) + 1
                                        POPU_REPTYPE(5) = POPU_REPTYPE(5) + MIMOSIS_WEIGHT(pers_k)
                                    END IF
                                END IF
                                EXIT
                            !or k is partner 1    
                            ELSE IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                            &(FAMREL_RELATION(pers_j).EQ.2.AND.FAMREL_RELATION(pers_k).EQ.1)) THEN
                                INC_PARTNER = PENSWELF_INDINC_QT(pers_k)
                                !if the partner has an own income source then the pensioner does not get the household rate
                                IF (INC_PARTNER.NE.0) THEN
                                    PENSWELF_REPRATE_QT(pers_j) = 60
                                    COUNT_REPTYPE(2) = COUNT_REPTYPE(2) + 1
                                    POPU_REPTYPE(2) = POPU_REPTYPE(2) + MIMOSIS_WEIGHT(pers_j)
                                    IF (InPensCad(pers_j).NE.0) THEN
                                        PENSWELF_REPRATE_QT(pers_k) = 60
                                        COUNT_REPTYPE(3) = COUNT_REPTYPE(3) + 1
                                        POPU_REPTYPE(3) = POPU_REPTYPE(3) + MIMOSIS_WEIGHT(pers_k)
                                    END IF
                                !if the partner has an no income source then the pensioner gets the household rate
                                ELSE
                                    PENSWELF_REPRATE_QT(pers_j) = 75
                                    COUNT_REPTYPE(4) = COUNT_REPTYPE(4) + 1
                                    POPU_REPTYPE(4) = POPU_REPTYPE(4) + MIMOSIS_WEIGHT(pers_j)
                                    IF (InPensCad(pers_j).NE.0) THEN
                                        PENSWELF_REPRATE_QT(pers_k) = 60
                                        COUNT_REPTYPE(5) = COUNT_REPTYPE(5) + 1
                                        POPU_REPTYPE(5) = POPU_REPTYPE(5) + MIMOSIS_WEIGHT(pers_k)
                                    END IF
                                END IF
                                EXIT
                            END IF
                        END DO
                    END IF
                END IF        
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of beneficiaries and recipients
    Write(15,'('' '')')
    Write(15,'(''Pensioners according to replacement rate'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Counts'')')
    Write(15,'(''Singles with a replacement rate of 60                                          '', i10)') COUNT_REPTYPE(1)
    Write(15,'(''Spouse in couple with replacement rate of 60 because partner has income        '', i10)') COUNT_REPTYPE(2)
    Write(15,'(''Partner of a pensioner with a replacement rate of 60                           '', i10)') COUNT_REPTYPE(3)
    Write(15,'(''Spouse in couple with replacement rate of 75 because partner has no income     '', i10)') COUNT_REPTYPE(4)
    Write(15,'(''Partner of a pensioner with a replacement rate of 75                           '', i10)') COUNT_REPTYPE(5)
    Write(15,'(''Total                                                                          '', i10)') &
    &SUM(COUNT_REPTYPE(1:5))
    Write(15,'(''Weighted amounts'')')
    Write(15,'(''Singles with a replacement rate of 60                                          '', f10.0)') POPU_REPTYPE(1)
    Write(15,'(''Spouse in couple with replacement rate of 60 because partner has income        '', f10.0)') POPU_REPTYPE(2)
    Write(15,'(''Partner of a pensioner with a replacement rate of 60                           '', f10.0)') POPU_REPTYPE(3)
    Write(15,'(''Spouse in couple with replacement rate of 75 because partner has no income     '', f10.0)') POPU_REPTYPE(4)
    Write(15,'(''Partner of a pensioner with a replacement rate of 75                           '', f10.0)') POPU_REPTYPE(5)
    Write(15,'(''Total                                                                          '', f10.0)') &
    &SUM(POPU_REPTYPE(1:5))

    !PENSWELF_WEMIPENS_ANN: Wage earner benefits from a minimum pension (no=0, yes=1)
    DO pers_j=1,n_rec
        !if the individual is a pensioner, i.e. receives pension benefits
        IF (InPensCad(pers_j).NE.0) THEN
            !if the individual receives a retirement pension as wage earner
            IF (PENSWELF_BENBEFOR_MONTH(pers_j,1).NE.0) THEN
                REALVAL_REP = PENSWELF_BENBEFOR_MONTH(pers_j,1)
                REFVAL_REP = 0
                IF (PENSWELF_REPRATE_QT(pers_j).EQ.75) THEN
                    REFVAL_REP = PENSWELF_MINPENS_MONTH(1,1)
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.60) THEN
                    REFVAL_REP = PENSWELF_MINPENS_MONTH(2,1)
                END IF
                !adapt value conditional on number of career years
                IF (MINPENS_NUMCYEAR(pers_j).GT.0.AND.MINPENS_NUMCYEAR(pers_j).LT.45) THEN
                    REFVAL_REP = (MINPENS_NUMCYEAR(pers_j)/45.D0)*REFVAL_REP
                END IF
                !if the real and the reference value deviate only slightly then the indvidual benefits from a minimum pension
                IF (REALVAL_REP.GT.(0.99D0*REFVAL_REP).AND.REALVAL_REP.LT.(1.01D0*REFVAL_REP)) THEN
                    PENSWELF_WEMIPENS_ANN(pers_j) = 1
                END IF
            END IF
            !if the individual receives a survival pension as wage earner
            IF (PENSWELF_BENBEFOR_MONTH(pers_j,6).NE.0) THEN
                REALVAL_REP = PENSWELF_BENBEFOR_MONTH(pers_j,6)
                REFVAL_REP = PENSWELF_MINPENS_MONTH(3,1)
                !no adaptation done, conditional on the number of career years since the relevant number of career years are
                !those of the former spouse
                !if the real and the reference value deviate only slightly then the indvidual benefits from a minimum pension
                IF (REALVAL_REP.GT.(0.99D0*REFVAL_REP).AND.REALVAL_REP.LT.(1.01D0*REFVAL_REP)) THEN
                    PENSWELF_WEMIPENS_ANN(pers_j) = 1
                END IF
            END IF
        END IF
    END DO

    !PENSWELF_SEMIPENS_ANN: Self employed benefits from a minimum pension (no=0, yes=1)
    !This variable can not be identified given the available information
    DO pers_j=1,n_rec
        PENSWELF_SEMIPENS_ANN(pers_j) = 0
    END DO

    !PENSWELF_CSMIPENS_ANN: Civil servant benefits from a minimum pension (no=0, yes=1)
    !This variable can not be identified given the available information
    DO pers_j=1,n_rec
        PENSWELF_CSMIPENS_ANN(pers_j) = 0
    END DO

END SUBROUTINE PENSWELF_CHARAC

!=====================================================================================!
!Define a function that computes the number of months as result of a year and month   !
!value derived from an input variable                                                 !
!=====================================================================================!

FUNCTION PENSWELF_DATE(INPUT_DATE) RESULT (OUTPUT_MONTHS)

    CHARACTER(len=7)                                    :: INPUT_DATE                               !Date consisting of a year value (first four digits) and a month value (last two digits)
    INTEGER                                             :: MONTH,YEAR                               !Integer values of month and year
    INTEGER                                             :: OUTPUT_MONTHS                            !Number of months starting from the input date to January 1st 2009
    INTEGER                                             :: i                                        !Loop variable

    !initialize result to zero
    OUTPUT_MONTHS = 0
    YEAR = MIMOSIS_YEAR
    MONTH = 12
    PENSWELF_PAR_BREAK_POINT(1)=INDEX(INPUT_DATE,'-')
    IF (PENSWELF_PAR_BREAK_POINT(1).GT.1) THEN
        READ (INPUT_DATE(1:PENSWELF_PAR_BREAK_POINT(1)-1),  '(I10)') MONTH
        READ (INPUT_DATE(PENSWELF_PAR_BREAK_POINT(1)+1:),   '(I10)') YEAR         
    END IF
    !If year is known (i.e. not lager than year of analysis)
    IF (YEAR.LE.MIMOSIS_YEAR) THEN
        !if the month is known count the number of months for which a benefit is paid throughout the year
        IF (MONTH.GT.0.AND.MONTH.LE.12) THEN
            OUTPUT_MONTHS = 12*(MIMOSIS_YEAR-YEAR) + (12 - MONTH + 1)
        !if the month is not known, assume that the benefit is only paid in the last month of the year
        ELSE
            OUTPUT_MONTHS = 12*(MIMOSIS_YEAR-YEAR) + 1
        END IF
    !if year is not known, assume that the benefit is paid in the year of analysis
    ELSE
        !if the month is known count the number of months for which a benefit is paid throughout the year
        IF (MONTH.GT.0.AND.MONTH.LE.12) THEN
            OUTPUT_MONTHS = (12 - MONTH + 1)
        !if the month is not known, assume that no benefit is paid
        ELSE
            OUTPUT_MONTHS = 0
        END IF
    END IF

END FUNCTION PENSWELF_DATE

!=====================================================================================!
!Define a function that computes the monthly benefit using a benefit and the number of!
!months as input                                                                      !
!=====================================================================================!

FUNCTION PENSWELF_MONTHBEN(INPUT_BEN,NUMMONTH) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)         :: INPUT_BEN       !Input amount (=amount per year)
    INTEGER                      :: NUMMONTH        !number of months the pensioner already receives the benefit
    REAL(KIND=KIND_REAL)         :: OUTPUT_AMOUNT   !Output amount

    OUTPUT_AMOUNT = 0
    !If number of months not known, assume that it is a monthly benefit
    IF (NUMMONTH.EQ.0) THEN
        IF (INPUT_BEN.NE.0) THEN
            OUTPUT_AMOUNT = INPUT_BEN
        END IF
    !if number of months less than or equal to 1 year on January 1st 2009 then divide by the number of months in the year   
    ELSE IF (NUMMONTH.GT.0.AND.NUMMONTH.LE.12) THEN
        OUTPUT_AMOUNT = INPUT_BEN/NUMMONTH
    !if payments started in 2000 or before, divide the annual amount by 12
    ELSE IF (NUMMONTH.GT.12) THEN
        OUTPUT_AMOUNT = INPUT_BEN/12
    END IF

END FUNCTION PENSWELF_MONTHBEN

!===================================================================================!
!With subroutine PENSWELF_ADAP we reconstruct variables 
!used troughout the different subroutines for computing welfare adaptations of pension benefits
!Constructed variables
!   PENSWELF_BENAFTER_MONTH 
!   PENSWELF_BENHOLID_ANN 
!===================================================================================!

SUBROUTINE PENSWELF_ADAP

    INTEGER                             :: pers_j,pers_k,pers_l,i,lijn_j
    REAL(KIND=KIND_REAL)                :: SUM1,SUM2                     !intermediate constructs

    !PENSWELF_BENAFTER_MONTH: Monthly pension benefit paid after welfare adaptation in benefit scheme 1 to 13
    !compute new monthly benefits in case absolute changes are requested
    IF (PENSWELF_TYPESIM.EQ.1) THEN
        DO pers_j=1,n_rec
            !if the individual is a pensioner, i.e. receives pension benefits
            IF (InPensCad(pers_j).NE.0) THEN
                !Select the appropriate value of the change in retirement pension conditional on the replacement rate and the minimum income condition of a wage earner
                IF (PENSWELF_REPRATE_QT(pers_j).EQ.75.AND.PENSWELF_WEMIPENS_ANN(pers_j).EQ.0) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,1).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(1,1)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(1,2)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(1,3)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(1,4)
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.60.AND.PENSWELF_WEMIPENS_ANN(pers_j).EQ.0) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,1).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(2,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(2,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(2,3) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(2,4) 
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.75.AND.PENSWELF_WEMIPENS_ANN(pers_j).EQ.1) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,1).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(9,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(9,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(9,3) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(9,4) 
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.60.AND.PENSWELF_WEMIPENS_ANN(pers_j).EQ.1) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,1).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(10,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(10,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(10,3) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + PENSWELF_ABSCHANG_MONTH(10,4) 
                    END IF 
                END IF
                !Select the appropriate value of the change in survival pension conditional on the minimum income condition of a wage earner
                IF (PENSWELF_WEMIPENS_ANN(pers_j).EQ.0) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,6).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + PENSWELF_ABSCHANG_MONTH(3,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,6).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + PENSWELF_ABSCHANG_MONTH(3,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,6).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + PENSWELF_ABSCHANG_MONTH(3,3) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + PENSWELF_ABSCHANG_MONTH(3,4) 
                    END IF 
                ELSE IF (PENSWELF_WEMIPENS_ANN(pers_j).EQ.1) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,6).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + PENSWELF_ABSCHANG_MONTH(11,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,6).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + PENSWELF_ABSCHANG_MONTH(11,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,6).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + PENSWELF_ABSCHANG_MONTH(11,3) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + PENSWELF_ABSCHANG_MONTH(11,4) 
                    END IF 
                END IF
                !Select the appropriate value of the change in retirement pension conditional on the replacement rate and the minimum income condition of a self employed
                IF (PENSWELF_REPRATE_QT(pers_j).EQ.75.AND.PENSWELF_SEMIPENS_ANN(pers_j).EQ.0) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,2).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(4,1)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(4,2)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(4,3)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(4,4)
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.60.AND.PENSWELF_SEMIPENS_ANN(pers_j).EQ.0) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,2).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(5,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(5,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(5,3) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(5,4) 
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.75.AND.PENSWELF_SEMIPENS_ANN(pers_j).EQ.1) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,2).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(12,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(12,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(12,3) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(12,4) 
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.60.AND.PENSWELF_SEMIPENS_ANN(pers_j).EQ.1) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,2).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(13,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(13,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(13,3) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + PENSWELF_ABSCHANG_MONTH(13,4) 
                    END IF 
                END IF
                !Select the appropriate value of the change in survival pension conditional on the minimum income condition of a self employed
                IF (PENSWELF_SEMIPENS_ANN(pers_j).EQ.0) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,7).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + PENSWELF_ABSCHANG_MONTH(6,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,7).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + PENSWELF_ABSCHANG_MONTH(6,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,7).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + PENSWELF_ABSCHANG_MONTH(6,3) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + PENSWELF_ABSCHANG_MONTH(6,4) 
                    END IF 
                ELSE IF (PENSWELF_SEMIPENS_ANN(pers_j).EQ.1) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,7).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + PENSWELF_ABSCHANG_MONTH(14,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,7).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + PENSWELF_ABSCHANG_MONTH(14,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,7).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + PENSWELF_ABSCHANG_MONTH(14,3) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + PENSWELF_ABSCHANG_MONTH(14,4) 
                    END IF 
                END IF
                !Put components that are not manipulated equal to monthly amount before adaptation
                PENSWELF_BENAFTER_MONTH(pers_j,3) = PENSWELF_BENBEFOR_MONTH(pers_j,3) 
                PENSWELF_BENAFTER_MONTH(pers_j,4) = PENSWELF_BENBEFOR_MONTH(pers_j,4) 
                PENSWELF_BENAFTER_MONTH(pers_j,5) = PENSWELF_BENBEFOR_MONTH(pers_j,5) 
                PENSWELF_BENAFTER_MONTH(pers_j,8) = PENSWELF_BENBEFOR_MONTH(pers_j,8) 
                PENSWELF_BENAFTER_MONTH(pers_j,9) = PENSWELF_BENBEFOR_MONTH(pers_j,9) 
                PENSWELF_BENAFTER_MONTH(pers_j,10) = PENSWELF_BENBEFOR_MONTH(pers_j,10) 
                PENSWELF_BENAFTER_MONTH(pers_j,11) = PENSWELF_BENBEFOR_MONTH(pers_j,11) 
                PENSWELF_BENAFTER_MONTH(pers_j,12) = PENSWELF_BENBEFOR_MONTH(pers_j,12) 
                PENSWELF_BENAFTER_MONTH(pers_j,13) = PENSWELF_BENBEFOR_MONTH(pers_j,13) 
            END IF
        END DO
    !if a percentage increase is chosen            
    ELSE IF (PENSWELF_TYPESIM.EQ.2) THEN
        DO pers_j=1,n_rec
            !if the individual is a pensioner, i.e. receives pension benefits
            IF (InPensCad(pers_j).NE.0) THEN
                !Select the appropriate value of the change in retirement pension conditional on the replacement rate and the minimum income condition of a wage earner
                IF (PENSWELF_REPRATE_QT(pers_j).EQ.75.AND.PENSWELF_WEMIPENS_ANN(pers_j).EQ.0) THEN
                    !execute analysis for main retirement benefits
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,1).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(1,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(1,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(1,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(1,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1)
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.60.AND.PENSWELF_WEMIPENS_ANN(pers_j).EQ.0) THEN
                    !execute analysis for main retirement benefits
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,1).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(2,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(2,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(2,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(2,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.75.AND.PENSWELF_WEMIPENS_ANN(pers_j).EQ.1) THEN
                    !execute analysis for main retirement benefits
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,1).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(9,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(9,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(9,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(9,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.60.AND.PENSWELF_WEMIPENS_ANN(pers_j).EQ.1) THEN
                    !execute analysis for main retirement benefits
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,1).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(10,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(10,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,1).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(10,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,1).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,1) = PENSWELF_BENBEFOR_MONTH(pers_j,1) + &
                        &(PENSWELF_PRCCHANG_MONTH(10,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,1) 
                    END IF 
                END IF
                !Select the appropriate value of the change in survival pension conditional on the minimum income condition of a wage earner
                IF (PENSWELF_WEMIPENS_ANN(pers_j).EQ.0) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,6).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + &
                        &(PENSWELF_PRCCHANG_MONTH(3,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,6) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,6).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + &
                        &(PENSWELF_PRCCHANG_MONTH(3,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,6) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,6).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + &
                        &(PENSWELF_PRCCHANG_MONTH(3,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,6) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + &
                        &(PENSWELF_PRCCHANG_MONTH(3,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,6) 
                    END IF 
                ELSE IF (PENSWELF_WEMIPENS_ANN(pers_j).EQ.1) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,6).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + &
                        &(PENSWELF_PRCCHANG_MONTH(11,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,6) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,6).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + &
                        &(PENSWELF_PRCCHANG_MONTH(11,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,6) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,6).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + &
                        &(PENSWELF_PRCCHANG_MONTH(11,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,6) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,6).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,6) = PENSWELF_BENBEFOR_MONTH(pers_j,6) + &
                        &(PENSWELF_PRCCHANG_MONTH(11,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,6) 
                    END IF 
                END IF
                !Select the appropriate value of the change in retirement pension conditional on the replacement rate and the minimum income condition of a self employed
                IF (PENSWELF_REPRATE_QT(pers_j).EQ.75.AND.PENSWELF_SEMIPENS_ANN(pers_j).EQ.0) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,2).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(4,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(4,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(4,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2)
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(4,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2)
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.60.AND.PENSWELF_SEMIPENS_ANN(pers_j).EQ.0) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,2).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(5,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(5,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(5,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(5,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.75.AND.PENSWELF_SEMIPENS_ANN(pers_j).EQ.1) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,2).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(12,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(12,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(12,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(12,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    END IF 
                ELSE IF (PENSWELF_REPRATE_QT(pers_j).EQ.60.AND.PENSWELF_SEMIPENS_ANN(pers_j).EQ.1) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,2).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(13,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(13,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,2).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(13,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,2).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,2) = PENSWELF_BENBEFOR_MONTH(pers_j,2) + &
                        &(PENSWELF_PRCCHANG_MONTH(13,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,2) 
                    END IF 
                END IF
                !Select the appropriate value of the change in survival pension conditional on the minimum income condition of a self employed
                IF (PENSWELF_SEMIPENS_ANN(pers_j).EQ.0) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,7).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + &
                        &(PENSWELF_PRCCHANG_MONTH(6,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,7) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,7).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + &
                        &(PENSWELF_PRCCHANG_MONTH(6,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,7) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,7).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + &
                        &(PENSWELF_PRCCHANG_MONTH(6,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,7) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + &
                        &(PENSWELF_PRCCHANG_MONTH(6,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,7) 
                    END IF 
                ELSE IF (PENSWELF_SEMIPENS_ANN(pers_j).EQ.1) THEN
                    !check the number of years for which the benefit has been paid and select the appropriate column of the welfare adaptation matrix
                    IF (PENSWELF_NUMMONTH_ANN(pers_j,7).LT.PENSWELF_TIMESPAN(1)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + &
                        &(PENSWELF_PRCCHANG_MONTH(14,1)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,7) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(1).AND.PENSWELF_NUMMONTH_ANN(pers_j,7).LT.&
                    &PENSWELF_TIMESPAN(2)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + &
                        &(PENSWELF_PRCCHANG_MONTH(14,2)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,7) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(2).AND.PENSWELF_NUMMONTH_ANN(pers_j,7).LT.&
                    &PENSWELF_TIMESPAN(3)) THEN                        
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + &
                        &(PENSWELF_PRCCHANG_MONTH(14,3)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,7) 
                    ELSE IF (PENSWELF_NUMMONTH_ANN(pers_j,7).GE.PENSWELF_TIMESPAN(3)) THEN
                        PENSWELF_BENAFTER_MONTH(pers_j,7) = PENSWELF_BENBEFOR_MONTH(pers_j,7) + &
                        &(PENSWELF_PRCCHANG_MONTH(14,4)/100.D0)*PENSWELF_BENBEFOR_MONTH(pers_j,7) 
                    END IF 
                END IF
                !Put components that are not manipulated equal to monthly amount before adaptation
                PENSWELF_BENAFTER_MONTH(pers_j,3) = PENSWELF_BENBEFOR_MONTH(pers_j,3) 
                PENSWELF_BENAFTER_MONTH(pers_j,4) = PENSWELF_BENBEFOR_MONTH(pers_j,4) 
                PENSWELF_BENAFTER_MONTH(pers_j,5) = PENSWELF_BENBEFOR_MONTH(pers_j,5) 
                PENSWELF_BENAFTER_MONTH(pers_j,8) = PENSWELF_BENBEFOR_MONTH(pers_j,8) 
                PENSWELF_BENAFTER_MONTH(pers_j,9) = PENSWELF_BENBEFOR_MONTH(pers_j,9) 
                PENSWELF_BENAFTER_MONTH(pers_j,10) = PENSWELF_BENBEFOR_MONTH(pers_j,10) 
                PENSWELF_BENAFTER_MONTH(pers_j,11) = PENSWELF_BENBEFOR_MONTH(pers_j,11) 
                PENSWELF_BENAFTER_MONTH(pers_j,12) = PENSWELF_BENBEFOR_MONTH(pers_j,12) 
                PENSWELF_BENAFTER_MONTH(pers_j,13) = PENSWELF_BENBEFOR_MONTH(pers_j,13) 
            END IF
        END DO
    END IF

END SUBROUTINE PENSWELF_ADAP

!===================================================================================!
!With subroutine PENSWELF_EXCHANGE we construct the variables                       !
!PENSWELF_RETPENS_QT 
!PENSWELF_SURPENS_QT 
!PENSWELF_AMOUNTP_QT 
!PENSWELF_BENSCHEM_QT
!===================================================================================!

SUBROUTINE PENSWELF_EXCHANGE

    REAL(KIND=KIND_REAL)                :: SUM1,SUM2,SUM3,SUM4                     !intermediate constructs

    !PENSWELF_RETPENS_QT: Quarterly amount of retirement pensions
    !PENSWELF_SURPENS_QT: Quarterly amount of survival pensions
    !PENSWELF_AMOUNTP_QT: Total sum of pension benefits received per quarter (sum of all retirement and survival benefits, but no guaranteed pension benefits)    DO pers_j=1,n_rec
    DO pers_j=1,n_rec
        !if the individual is a pensioner, i.e. receives pension benefits
        IF (InPensCad(pers_j).NE.0) THEN
            !Compute annual retirement pension
            SUM1 = 0
            SUM1 = SUM1 + PENSWELF_BENAFTER_MONTH(pers_j,1)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,1))
            SUM1 = SUM1 + PENSWELF_BENAFTER_MONTH(pers_j,2)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,2))
            SUM1 = SUM1 + PENSWELF_BENAFTER_MONTH(pers_j,3)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,3))
            SUM1 = SUM1 + PENSWELF_BENAFTER_MONTH(pers_j,4)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,4))
            SUM1 = SUM1 + PENSWELF_BENAFTER_MONTH(pers_j,5)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,5))
            SUM1 = SUM1 + PENSWELF_BENAFTER_MONTH(pers_j,11)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,11))
            SUM1 = SUM1 + PENSWELF_BENAFTER_MONTH(pers_j,12)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,12))
            SUM1 = SUM1 + PENSWELF_BENAFTER_MONTH(pers_j,13)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,13))
            !Compute annual survival pension
            SUM2 = 0
            SUM2 = SUM2 + PENSWELF_BENAFTER_MONTH(pers_j,6)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,6))
            SUM2 = SUM2 + PENSWELF_BENAFTER_MONTH(pers_j,7)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,7))
            SUM2 = SUM2 + PENSWELF_BENAFTER_MONTH(pers_j,8)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,8))
            SUM2 = SUM2 + PENSWELF_BENAFTER_MONTH(pers_j,9)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,9))
            SUM2 = SUM2 + PENSWELF_BENAFTER_MONTH(pers_j,10)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,10))
            !Compute quarterly values
            PENSWELF_RETPENS_QT(pers_j) = SUM1/4.D0 
            PENSWELF_SURPENS_QT(pers_j) = SUM2/4.D0
            !Compute sum of both concepts
            PENSWELF_AMOUNTP_QT(pers_j) = PENSWELF_RETPENS_QT(pers_j) + PENSWELF_SURPENS_QT(pers_j)
        END IF
    END DO

    !PENSWELF_BENSCHEM_QT: Contribution scheme that generates the largest amount of pension benefits
    !1) benefits are paid in the wage earner scheme
    !2) benefits are paid in the civil servant scheme
    !3) benefits are paid in the self employed scheme 
    !4) benefits could not be classified.
    DO pers_j=1,n_rec
        !if the individual is a pensioner, i.e. receives pension benefits
        IF (InPensCad(pers_j).NE.0) THEN
            !Compute annual pension of wage earners
            SUM1 = 0
            SUM1 = SUM1 + PENSWELF_BENBEFOR_MONTH(pers_j,1)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,1))
            SUM1 = SUM1 + PENSWELF_BENBEFOR_MONTH(pers_j,6)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,6))
            !Compute annual pension of civil servant
            SUM2 = 0
            SUM2 = SUM2 + PENSWELF_BENBEFOR_MONTH(pers_j,3)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,3))
            SUM2 = SUM2 + PENSWELF_BENBEFOR_MONTH(pers_j,8)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,8))
            !Compute annual pension of self employed
            SUM3 = 0
            SUM3 = SUM3 + PENSWELF_BENBEFOR_MONTH(pers_j,2)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,2))
            SUM3 = SUM3 + PENSWELF_BENBEFOR_MONTH(pers_j,7)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,7))
            !Compute annual pensions that are foreign or from DOSZ
            SUM4 = 0
            SUM4 = SUM4 + PENSWELF_BENBEFOR_MONTH(pers_j,4)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,4))
            SUM4 = SUM4 + PENSWELF_BENBEFOR_MONTH(pers_j,9)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,9))
            SUM4 = SUM4 + PENSWELF_BENBEFOR_MONTH(pers_j,5)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,5))
            SUM4 = SUM4 + PENSWELF_BENBEFOR_MONTH(pers_j,10)*MIN(12,PENSWELF_NUMMONTH_ANN(pers_j,10))
            !Compute the contribution scheme as the scheme that generated most of the benefits
            IF (SUM1.GE.SUM2.AND.SUM1.GE.SUM3.AND.SUM1.GE.SUM4.AND.SUM1.NE.0) THEN
                PENSWELF_BENSCHEM_QT(pers_j) = 1
            ELSE IF (SUM2.GT.SUM1.AND.SUM2.GT.SUM3.AND.SUM2.GT.SUM4.AND.SUM2.NE.0) THEN
                PENSWELF_BENSCHEM_QT(pers_j) = 2
            ELSE IF (SUM3.GT.SUM1.AND.SUM3.GT.SUM2.AND.SUM3.GT.SUM4.AND.SUM3.NE.0) THEN
                PENSWELF_BENSCHEM_QT(pers_j) = 3
            ELSE IF (SUM4.GT.SUM1.AND.SUM4.GT.SUM2.AND.SUM4.GT.SUM3.AND.SUM4.NE.0) THEN
                PENSWELF_BENSCHEM_QT(pers_j) = 4
            END IF
        END IF
    END DO
    
END SUBROUTINE PENSWELF_EXCHANGE

!==========================================================!
!Produce some aggregate statistics to check the constuction
!==========================================================!

SUBROUTINE PENSWELF_TEST

    !Set local variables
    INTEGER                                                 :: pers_j,i
    INTEGER                                                 :: NumAnal                              !Number of summary statistics computed 
    REAL(KIND=DPRECI)                                       :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: PENSWELF_RESULT_TEST                 !lump sum costs related to employment individual
    CHARACTER(len=75),ALLOCATABLE,DIMENSION(:)              :: PENSWELF_LAB_TEST                    !Labels of test variables
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)              :: SumPENSWELF                          !Vector with quarterly amounts on adapted pensions

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))

    !Start analysis of basic input data: i.e. annual data
    NumAnal = 16
    !Allocate and initialise test result
    ALLOCATE(PENSWELF_RESULT_TEST(NumAnal,4))
    ALLOCATE(PENSWELF_LAB_TEST(NumAnal))
    ALLOCATE(SumPENSWELF(NumAnal))
    PENSWELF_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PENSWELF_LAB_TEST(1) =  "rustpensioen werknemer (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(2) =  "rustpensioen zelfstandige (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(3) =  "rustpensioen overheid (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(4) =  "buitenlands rustpensioen (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(5) =  "rustpensioen DOSZ (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(6) =  "overlevingspensioen werknemer (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(7) =  "overlevingspensioen zelfstandige (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(8) =  "overlevingspensioen overheid (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(9) =  "buitenlands overlevingspensioen (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(10) = "overlevingspensioen DOSZ (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(11) = "tegemoetkoming mindervaliden (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(12) = "afgeleide pensioenen (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(13) = "overige pensioenbedragen (betalingsjaar = 2008)"
    PENSWELF_LAB_TEST(14) = "Gross amount of all pension benefits in treated subgroups"
    PENSWELF_LAB_TEST(15) = "Sum of pension benefits in sum concept"
    PENSWELF_LAB_TEST(16) = "Sum of all individual incomes (=salary mass + replacement income)      "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                  
        SumPENSWELF(1:NumAnal)=0.D0
        !Construct quarterly concepts
        SumPENSWELF(1)  = RPWE_2008(pers_j)     
        SumPENSWELF(2)  = RPSE_2008(pers_j)     
        SumPENSWELF(3)  = RPCIV_2008(pers_j)    
        SumPENSWELF(4)  = RPFOR_2008(pers_j)    
        SumPENSWELF(5)  = RPDOSZ_2008(pers_j)   
        SumPENSWELF(6)  = SPWE_2008(pers_j)     
        SumPENSWELF(7)  = SPSE_2008(pers_j)     
        SumPENSWELF(8)  = SPCIV_2008(pers_j)    
        SumPENSWELF(9)  = SPFOR_2008(pers_j)    
        SumPENSWELF(10) = SPDOSZ_2008(pers_j)   
        SumPENSWELF(11) = DISAB_2008(pers_j)    
        SumPENSWELF(12) = DIRPENS_2008(pers_j)  
        SumPENSWELF(13) = OTHPENS_2008(pers_j)
        SumPENSWELF(14) = SUM(SumPENSWELF(1:13))
        SumPENSWELF(15) = PENSWELF_TOTPBEF_QT(pers_j)*4.D0
        SumPENSWELF(16) = PENSWELF_INDINC_QT(pers_j)*4.D0
        DO j=1,NumAnal
            IF (INT(SumPENSWELF(j)).NE.0) THEN
                PENSWELF_RESULT_TEST(j,1)=PENSWELF_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumPENSWELF(j)
                PENSWELF_RESULT_TEST(j,2)=PENSWELF_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(PENSWELF_RESULT_TEST(i,2)).NE.0) THEN
            PENSWELF_RESULT_TEST(i,3)=PENSWELF_RESULT_TEST(i,1)/PENSWELF_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PENSWELF_RESULT_TEST(i,4)=100.D0*PENSWELF_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Annual amounts of pension benefits by scheme'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PENSWELF_LAB_TEST(i),(PENSWELF_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PENSWELF_RESULT_TEST)
    DEALLOCATE(PENSWELF_LAB_TEST)
    DEALLOCATE(SumPENSWELF)

    !Start analysis of monthly benefits before adaptation
    NumAnal = 14
    !Allocate and initialise test result
    ALLOCATE(PENSWELF_RESULT_TEST(NumAnal,4))
    ALLOCATE(PENSWELF_LAB_TEST(NumAnal))
    ALLOCATE(SumPENSWELF(NumAnal))
    PENSWELF_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PENSWELF_LAB_TEST(1) =  "rustpensioen werknemer (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(2) =  "rustpensioen zelfstandige (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(3) =  "rustpensioen overheid (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(4) =  "buitenlands rustpensioen (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(5) =  "rustpensioen DOSZ (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(6) =  "overlevingspensioen werknemer (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(7) =  "overlevingspensioen zelfstandige (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(8) =  "overlevingspensioen overheid (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(9) =  "buitenlands overlevingspensioen (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(10) = "overlevingspensioen DOSZ (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(11) = "tegemoetkoming mindervaliden (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(12) = "afgeleide pensioenen (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(13) = "overige pensioenbedragen (betalingsjaar = 2008): bruto maandbedrag voor hervorming"
    PENSWELF_LAB_TEST(14) = "Gross amount of all monthly pension benefits                                   "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                  
        SumPENSWELF(1:NumAnal)=0.D0
        !Construct quarterly concepts
        DO i=1,13
            SumPENSWELF(i) = PENSWELF_BENBEFOR_MONTH(pers_j,i)
        END DO
        SumPENSWELF(14) = SUM(SumPENSWELF(1:13))
        DO j=1,NumAnal
            IF (INT(SumPENSWELF(j)).NE.0) THEN
                PENSWELF_RESULT_TEST(j,1)=PENSWELF_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumPENSWELF(j)
                PENSWELF_RESULT_TEST(j,2)=PENSWELF_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(PENSWELF_RESULT_TEST(i,2)).NE.0) THEN
            PENSWELF_RESULT_TEST(i,3)=PENSWELF_RESULT_TEST(i,1)/PENSWELF_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PENSWELF_RESULT_TEST(i,4)=100.D0*PENSWELF_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Monthly amounts of pension benefits by scheme before welfare adaptation'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PENSWELF_LAB_TEST(i),(PENSWELF_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PENSWELF_RESULT_TEST)
    DEALLOCATE(PENSWELF_LAB_TEST)
    DEALLOCATE(SumPENSWELF)

    !Start analysis of number of months during which benefit is paid
    NumAnal = 14
    !Allocate and initialise test result
    ALLOCATE(PENSWELF_RESULT_TEST(NumAnal,4))
    ALLOCATE(PENSWELF_LAB_TEST(NumAnal))
    ALLOCATE(SumPENSWELF(NumAnal))
    PENSWELF_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PENSWELF_LAB_TEST(1) =  "rustpensioen werknemer (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(2) =  "rustpensioen zelfstandige (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(3) =  "rustpensioen overheid (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(4) =  "buitenlands rustpensioen (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(5) =  "rustpensioen DOSZ (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(6) =  "overlevingspensioen werknemer (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(7) =  "overlevingspensioen zelfstandige (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(8) =  "overlevingspensioen overheid (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(9) =  "buitenlands overlevingspensioen (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(10) = "overlevingspensioen DOSZ (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(11) = "tegemoetkoming mindervaliden (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(12) = "afgeleide pensioenen (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(13) = "overige pensioenbedragen (betalingsjaar = 2008): aantal betalingsmaanden"
    PENSWELF_LAB_TEST(14) = "Total number of months during which a benefit is paid"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                  
        SumPENSWELF(1:NumAnal)=0.D0
        !Construct quarterly concepts
        DO i=1,13
            SumPENSWELF(i) = PENSWELF_NUMMONTH_ANN(pers_j,i)
        END DO
        SumPENSWELF(14) = SUM(SumPENSWELF(1:13))
        DO j=1,NumAnal
            IF (INT(SumPENSWELF(j)).NE.0) THEN
                PENSWELF_RESULT_TEST(j,1)=PENSWELF_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumPENSWELF(j)
                PENSWELF_RESULT_TEST(j,2)=PENSWELF_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(PENSWELF_RESULT_TEST(i,2)).NE.0) THEN
            PENSWELF_RESULT_TEST(i,3)=PENSWELF_RESULT_TEST(i,1)/PENSWELF_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PENSWELF_RESULT_TEST(i,4)=100.D0*PENSWELF_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Number of months during which a benefit is paid per scheme'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PENSWELF_LAB_TEST(i),(PENSWELF_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PENSWELF_RESULT_TEST)
    DEALLOCATE(PENSWELF_LAB_TEST)
    DEALLOCATE(SumPENSWELF)

    !Start analysis of monthly benefits after adaptation
    NumAnal = 14
    !Allocate and initialise test result
    ALLOCATE(PENSWELF_RESULT_TEST(NumAnal,4))
    ALLOCATE(PENSWELF_LAB_TEST(NumAnal))
    ALLOCATE(SumPENSWELF(NumAnal))
    PENSWELF_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PENSWELF_LAB_TEST(1) =  "rustpensioen werknemer (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(2) =  "rustpensioen zelfstandige (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(3) =  "rustpensioen overheid (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(4) =  "buitenlands rustpensioen (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(5) =  "rustpensioen DOSZ (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(6) =  "overlevingspensioen werknemer (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(7) =  "overlevingspensioen zelfstandige (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(8) =  "overlevingspensioen overheid (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(9) =  "buitenlands overlevingspensioen (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(10) = "overlevingspensioen DOSZ (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(11) = "tegemoetkoming mindervaliden (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(12) = "afgeleide pensioenen (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(13) = "overige pensioenbedragen (betalingsjaar = 2008): bruto maandbedrag na hervorming"
    PENSWELF_LAB_TEST(14) = "Gross amount of all monthly pension benefits                                   "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                  
        SumPENSWELF(1:NumAnal)=0.D0
        !Construct quarterly concepts
        DO i=1,13
            SumPENSWELF(i) = PENSWELF_BENAFTER_MONTH(pers_j,i)
        END DO
        SumPENSWELF(14) = SUM(SumPENSWELF(1:13))
        DO j=1,NumAnal
            IF (INT(SumPENSWELF(j)).NE.0) THEN
                PENSWELF_RESULT_TEST(j,1)=PENSWELF_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumPENSWELF(j)
                PENSWELF_RESULT_TEST(j,2)=PENSWELF_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(PENSWELF_RESULT_TEST(i,2)).NE.0) THEN
            PENSWELF_RESULT_TEST(i,3)=PENSWELF_RESULT_TEST(i,1)/PENSWELF_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PENSWELF_RESULT_TEST(i,4)=100.D0*PENSWELF_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Monthly amounts of pension benefits by scheme after welfare adaptation'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PENSWELF_LAB_TEST(i),(PENSWELF_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PENSWELF_RESULT_TEST)
    DEALLOCATE(PENSWELF_LAB_TEST)
    DEALLOCATE(SumPENSWELF)

    !Start analysis of pensioner characteristics
    NumAnal = 7
    !Allocate and initialise test result
    ALLOCATE(PENSWELF_RESULT_TEST(NumAnal,4))
    ALLOCATE(PENSWELF_LAB_TEST(NumAnal))
    ALLOCATE(SumPENSWELF(NumAnal))
    PENSWELF_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PENSWELF_LAB_TEST(1) =  "Number of pensioners"
    PENSWELF_LAB_TEST(2) =  "Number of pensioners for which no replacement rate was determined"
    PENSWELF_LAB_TEST(3) =  "Number of pensioners with a replacement rate of 75"
    PENSWELF_LAB_TEST(4) =  "Number of pensioners with a replacement rate of 60"
    PENSWELF_LAB_TEST(5) =  "Number of wage earners with minimum pensions"
    PENSWELF_LAB_TEST(6) =  "Number of self employed with minimum pensions"
    PENSWELF_LAB_TEST(7) =  "Number of civil servants with minimum pensions"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                  
        !number of pensioners
        IF (InPensCad(pers_j).NE.0) THEN
            PENSWELF_RESULT_TEST(1,1)=PENSWELF_RESULT_TEST(1,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
            PENSWELF_RESULT_TEST(1,2)=PENSWELF_RESULT_TEST(1,2)+MIMOSIS_WEIGHT(pers_j)
        END IF
        !number of pensioners for which no replacement rate was determined
        IF (InPensCad(pers_j).NE.0) THEN
            IF (PENSWELF_REPRATE_QT(pers_j).NE.75.AND.PENSWELF_REPRATE_QT(pers_j).NE.60) THEN
                PENSWELF_RESULT_TEST(2,1)=PENSWELF_RESULT_TEST(2,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
                PENSWELF_RESULT_TEST(2,2)=PENSWELF_RESULT_TEST(2,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
        !Number of pensioners with a replacement rate of 75        
        IF (InPensCad(pers_j).NE.0) THEN
            IF (PENSWELF_REPRATE_QT(pers_j).EQ.75) THEN
                PENSWELF_RESULT_TEST(3,1)=PENSWELF_RESULT_TEST(3,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
                PENSWELF_RESULT_TEST(3,2)=PENSWELF_RESULT_TEST(3,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
        !Number of pensioners with a replacement rate of 60        
        IF (InPensCad(pers_j).NE.0) THEN
            IF (PENSWELF_REPRATE_QT(pers_j).EQ.60) THEN
                PENSWELF_RESULT_TEST(4,1)=PENSWELF_RESULT_TEST(4,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
                PENSWELF_RESULT_TEST(4,2)=PENSWELF_RESULT_TEST(4,2)+MIMOSIS_WEIGHT(pers_j)
            END IF 
        END IF
        !Number of wage earners with minimum pensions        
        IF (PENSWELF_WEMIPENS_ANN(pers_j).NE.0) THEN
            PENSWELF_RESULT_TEST(5,1)=PENSWELF_RESULT_TEST(5,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
            PENSWELF_RESULT_TEST(5,2)=PENSWELF_RESULT_TEST(5,2)+MIMOSIS_WEIGHT(pers_j)
        END IF
        !Number of self employed with minimum pensions        
        IF (PENSWELF_SEMIPENS_ANN(pers_j).NE.0) THEN
            PENSWELF_RESULT_TEST(6,1)=PENSWELF_RESULT_TEST(6,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
            PENSWELF_RESULT_TEST(6,2)=PENSWELF_RESULT_TEST(6,2)+MIMOSIS_WEIGHT(pers_j)
        END IF
        !Number of civil servants with minimum pensions        
        IF (PENSWELF_CSMIPENS_ANN(pers_j).NE.0) THEN
            PENSWELF_RESULT_TEST(7,1)=PENSWELF_RESULT_TEST(7,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
            PENSWELF_RESULT_TEST(7,2)=PENSWELF_RESULT_TEST(7,2)+MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO

    DO i=1,NumAnal
        IF (INT(PENSWELF_RESULT_TEST(i,2)).NE.0) THEN
            PENSWELF_RESULT_TEST(i,3)=PENSWELF_RESULT_TEST(i,1)/PENSWELF_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PENSWELF_RESULT_TEST(i,4)=100.D0*PENSWELF_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Characteristics of pensioners'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PENSWELF_LAB_TEST(i),(PENSWELF_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PENSWELF_RESULT_TEST)
    DEALLOCATE(PENSWELF_LAB_TEST)
    DEALLOCATE(SumPENSWELF)

    !Start analysis of holiday earnings and exchange concepts
    NumAnal = 4
    !Allocate and initialise test result
    ALLOCATE(PENSWELF_RESULT_TEST(NumAnal,4))
    ALLOCATE(PENSWELF_LAB_TEST(NumAnal))
    ALLOCATE(SumPENSWELF(NumAnal))
    PENSWELF_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PENSWELF_LAB_TEST(1) =  "Gross amount per year of holiday earnings paid to wage earners or civil serv   "
    PENSWELF_LAB_TEST(2) =  "Gross amount of retirement pensions per year                                   "
    PENSWELF_LAB_TEST(3) =  "Gross amount of survival pensions per year                                     "
    PENSWELF_LAB_TEST(4) =  "Gross amount of pensions per year (guaranteed allowances excluded)             "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                  
        SumPENSWELF(1:NumAnal)=0.D0
        !Construct annual concepts
        SumPENSWELF(1) = PENSWELF_BENHOLID_ANN(pers_j) 
        SumPENSWELF(2) = PENSWELF_RETPENS_QT(pers_j)*4.D0  
        SumPENSWELF(3) = PENSWELF_SURPENS_QT(pers_j)*4.D0    
        SumPENSWELF(4) = PENSWELF_AMOUNTP_QT(pers_j)*4.D0    
        DO j=1,NumAnal
            IF (INT(SumPENSWELF(j)).NE.0) THEN
                PENSWELF_RESULT_TEST(j,1)=PENSWELF_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumPENSWELF(j)
                PENSWELF_RESULT_TEST(j,2)=PENSWELF_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(PENSWELF_RESULT_TEST(i,2)).NE.0) THEN
            PENSWELF_RESULT_TEST(i,3)=PENSWELF_RESULT_TEST(i,1)/PENSWELF_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PENSWELF_RESULT_TEST(i,4)=100.D0*PENSWELF_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Annual amounts of pension benefits to be exchanged'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PENSWELF_LAB_TEST(i),(PENSWELF_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PENSWELF_RESULT_TEST)
    DEALLOCATE(PENSWELF_LAB_TEST)
    DEALLOCATE(SumPENSWELF)

    !Start analysis of pensioner characteristics
    NumAnal = 4
    !Allocate and initialise test result
    ALLOCATE(PENSWELF_RESULT_TEST(NumAnal,4))
    ALLOCATE(PENSWELF_LAB_TEST(NumAnal))
    ALLOCATE(SumPENSWELF(NumAnal))
    PENSWELF_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PENSWELF_LAB_TEST(1) =  "Pensioners with mainly wage earner pension"
    PENSWELF_LAB_TEST(2) =  "Pensioners with mainly civil servant pension"
    PENSWELF_LAB_TEST(3) =  "Pensioners with mainly self employed pension"
    PENSWELF_LAB_TEST(4) =  "Pensioners with mainly foreign pensions or DOSZ pensions"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                  
        !Pensioners with mainly wage earner pension
        IF (PENSWELF_BENSCHEM_QT(pers_j).EQ.1) THEN
            PENSWELF_RESULT_TEST(1,1)=PENSWELF_RESULT_TEST(1,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
            PENSWELF_RESULT_TEST(1,2)=PENSWELF_RESULT_TEST(1,2)+MIMOSIS_WEIGHT(pers_j)
        END IF
        !Pensioners with mainly civil servant pension
        IF (PENSWELF_BENSCHEM_QT(pers_j).EQ.2) THEN
            PENSWELF_RESULT_TEST(2,1)=PENSWELF_RESULT_TEST(2,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
            PENSWELF_RESULT_TEST(2,2)=PENSWELF_RESULT_TEST(2,2)+MIMOSIS_WEIGHT(pers_j)
        END IF
        !Pensioners with mainly self employed pension
        IF (PENSWELF_BENSCHEM_QT(pers_j).EQ.3) THEN
            PENSWELF_RESULT_TEST(3,1)=PENSWELF_RESULT_TEST(3,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
            PENSWELF_RESULT_TEST(3,2)=PENSWELF_RESULT_TEST(3,2)+MIMOSIS_WEIGHT(pers_j)
        END IF
        !Pensioners with mainly non classified pension
        IF (PENSWELF_BENSCHEM_QT(pers_j).EQ.4) THEN
            PENSWELF_RESULT_TEST(4,1)=PENSWELF_RESULT_TEST(4,1)+MIMOSIS_WEIGHT(pers_j)*PENSWELF_TOTPBEF_QT(pers_j)*4.D0
            PENSWELF_RESULT_TEST(4,2)=PENSWELF_RESULT_TEST(4,2)+MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO

    DO i=1,NumAnal
        IF (INT(PENSWELF_RESULT_TEST(i,2)).NE.0) THEN
            PENSWELF_RESULT_TEST(i,3)=PENSWELF_RESULT_TEST(i,1)/PENSWELF_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PENSWELF_RESULT_TEST(i,4)=100.D0*PENSWELF_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Pensions according to beneficiary scheme'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PENSWELF_LAB_TEST(i),(PENSWELF_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PENSWELF_RESULT_TEST)
    DEALLOCATE(PENSWELF_LAB_TEST)
    DEALLOCATE(SumPENSWELF)

    !Analysis of observed guaranteed income for elderly
    NumAnal = 1
    !Allocate and initialise test result
    ALLOCATE(PENSWELF_RESULT_TEST(NumAnal,4))
    ALLOCATE(PENSWELF_LAB_TEST(NumAnal))
    ALLOCATE(SumPENSWELF(NumAnal))
    PENSWELF_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PENSWELF_LAB_TEST(1) =  "Guaranteed income for elderly"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                  
        IF (IGO_2008(pers_j).NE.0) THEN
            PENSWELF_RESULT_TEST(1,1)=PENSWELF_RESULT_TEST(1,1)+MIMOSIS_WEIGHT(pers_j)*IGO_2008(pers_j)
            PENSWELF_RESULT_TEST(1,2)=PENSWELF_RESULT_TEST(1,2)+MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO

    DO i=1,NumAnal
        IF (INT(PENSWELF_RESULT_TEST(i,2)).NE.0) THEN
            PENSWELF_RESULT_TEST(i,3)=PENSWELF_RESULT_TEST(i,1)/PENSWELF_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PENSWELF_RESULT_TEST(i,4)=100.D0*PENSWELF_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Guaranteed income for elderly'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PENSWELF_LAB_TEST(i),(PENSWELF_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PENSWELF_RESULT_TEST)
    DEALLOCATE(PENSWELF_LAB_TEST)
    DEALLOCATE(SumPENSWELF)

END SUBROUTINE PENSWELF_TEST

!==========================================================!
!Produce aggregate statistics that can be compared with
!statistics in other sources
!==========================================================!

SUBROUTINE PENSWELF_COMPARE

END SUBROUTINE PENSWELF_COMPARE

!============= SUBROUTINE OUTPUT_FILEWRIT ==============!
!This subroutine open output file and write output      !
!=======================================================!

SUBROUTINE PENSWELF_OUTPUT_FILEWRIT

    INTEGER :: pers_j
    
    OPEN(UNIT=12, FILE=CONTRIB_PENSWELF,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_PENSWELF
        WRITE(15,'("Could not find ", a100)') CONTRIB_PENSWELF
        PAUSE
        STOP
    END IF        
    OPEN(UNIT=13, FILE=SICK_PENSWELF,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_PENSWELF
        WRITE(15,'("Could not find ", a100)') SICK_PENSWELF
        PAUSE
        STOP
    END IF        
    OPEN(UNIT=14, FILE=UNEM_PENSWELF,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_PENSWELF
        WRITE(15,'("Could not find ", a100)') UNEM_PENSWELF
        PAUSE
        STOP
    END IF        
    OPEN(UNIT=16, FILE=FAMAL_PENSWELF,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMAL_PENSWELF
        WRITE(15,'("Could not find ", a100)') FAMAL_PENSWELF
        PAUSE
        STOP
    END IF        

    DO pers_j=1,n_rec
        WRITE(12,'(A14,f15.2,f15.2)') GECODEERD_INSZ_CHAR(pers_j),PENSWELF_RETPENS_QT(pers_j),PENSWELF_SURPENS_QT(pers_j)
        WRITE(13,'(A14,f15.2)') GECODEERD_INSZ_CHAR(pers_j),PENSWELF_AMOUNTP_QT(pers_j)
        WRITE(14,'(A14,f15.2)') GECODEERD_INSZ_CHAR(pers_j),PENSWELF_AMOUNTP_QT(pers_j) 
        WRITE(16,'(A14,i10,f15.2,f15.2)') GECODEERD_INSZ_CHAR(pers_j),PENSWELF_BENSCHEM_QT(pers_j),PENSWELF_AMOUNTP_QT(pers_j),&
        &PENSWELF_SURPENS_QT(pers_j)
    END DO

    CLOSE(UNIT=12)
    CLOSE(UNIT=13)    
    CLOSE(UNIT=14)    
    CLOSE(UNIT=16)    

    WRITE(*,*) 'Subroutine PENSWELF_OUTPUT_FILEWRIT of PRE RUN........runned correctly'

END SUBROUTINE PENSWELF_OUTPUT_FILEWRIT

!==========================================================!
!The subroutine PENSWELF_CALC is used to call the other    ! 
!subroutines and to carry out the necessary computations   ! 
!==========================================================!

SUBROUTINE PENSWELF_PRERUN_CALC

!Program heading

!=============================================================================
    WRITE (*,*) ' '
    WRITE (*,*) 'MIMOSIS PENSWELF IS RUNNING '
    WRITE (*,*) ' '
!=============================================================================

    !open log file
    OPEN(UNIT=15, FILE=PENSWELF_PRERUN_LOG, STATUS='REPLACE', ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PENSWELF_LOG
        WRITE(15,'("Could not find ", a100)') PENSWELF_LOG
        PAUSE
        STOP
    END IF

    !Allocate all exogenous vectors with dimension n_rec
    CALL PENSWELF_ALLOC_EXOG(n_rec)
    
    !Reads in PARAMETER file
    CALL PENSWELF_PARAMETER_FILEREAD
    !Read in files with input data
    CALL PENSWELF_INPUT_FILEREAD
    CALL PENSWELF_SICK_FILEREAD
    CALL PENSWELF_UNEM_FILEREAD
    CALL PENSWELF_CONTRIB_FILEREAD
    CALL PENSWELF_CONSTRUCT_FILEREAD
    CALL PENSWELF_FAMREL_FILEREAD
    CALL PENSWELF_WAGE_FILEREAD
    CALL PENSWELF_WEIGHT_FILEREAD

    WRITE(15,'(/,''Print of function calls for which no selection was carried out'')')
    CALL DRAWLINE(15,100,'=')                

    !Allocate all endogenous vectors with dimension n_rec
    CALL PENSWELF_ALLOC_ENDO(n_rec)
    !Initialise endogenous variables
    CALL PENSWELF_INITENDO(n_rec)
    
    !Rank and Sort all variables read in
    CALL PENSWELF_RANKANDSORT

    !Recode missing values
    CALL PENSWELF_RECODE
    !Inflate values to the appropriate price level
    CALL PENSWELF_INFLATE
    !Determine household size  
    CALL PENSWELF_HOUSEHOLD_SIZE
    !Determine age of all individuals
    CALL PENSWELF_AGE

    !Determine the variables necessary to compute welfare adaptations
    CALL PENSWELF_CHARAC
    !Determine the type of welfare adaptation
    IF (PENSWELF_WELFADAP.EQ.0) THEN
        CALL PENSWELF_ADAP
    END IF

    !Construct exchange concepts
    CALL PENSWELF_EXCHANGE

    !test construction of different contriubtion concepts
    CALL PENSWELF_TEST
    CALL PENSWELF_COMPARE

    !Write output to output file
    CALL PENSWELF_OUTPUT_FILEWRIT

    !Deallocate allocated variables
    CALL PENSWELF_DEALLOC
    CLOSE(15)

END SUBROUTINE PENSWELF_PRERUN_CALC

END MODULE PENSWELF_PRERUN_SUBROUTINES
