!===================================================================!
!  Record of revisions:                                             !
!                                                                   !
!    Date      Programmer               Description of changes      !
!  ========    ====================     ======================      !
!  01/03/06    Consortium AG/01/086     Original code               !
!  01/10/11    FPS Social Security      Revision                    !
!===================================================================!

!==========================================================!
! This module declare all variables used in the program    !
!  - Parameters                                            !
!  - Exogenous Variables                                   !
!  - Endogeneous Variables                                 !
!==========================================================!

MODULE SICK_PRERUN_DATA

    !========================================================================!
    ! Database specific parameters:                                          !
    ! number of lines in parameters file                                     !
    INTEGER, PARAMETER::n_par=157
    ! year of database                                                       !
    INTEGER, PARAMETER::MIMOSIS_YEAR=2008
    !(first quarter=1, second quarter=4, third quarter=9, fourth quarter=12) !
    INTEGER, PARAMETER::month=12                                             !
    !Define kind of Integer*4 in a portable way                              !                       
    INTEGER, PARAMETER :: INT4 = SELECTED_INT_KIND(18)                       !                       
    !Define Double precision, i.e. REAL*8 in a portable way                  !
    INTEGER, PARAMETER :: DPRECI = SELECTED_REAL_KIND(15,307)                !
    !Define Single precision, i.e. REAL*4 in a portable way                  !
    INTEGER, PARAMETER :: SPRECI = SELECTED_REAL_KIND(6,37)                  !
    !Define parameter that stores the precision value used for large vectors !
    INTEGER, PARAMETER :: KIND_REAL = SPRECI                                 !   
    !==========================================================================

    !==========================================================================
    !Define variables to read locations and input parameters
    !Variables for locating input 
    CHARACTER(len=150)                                      :: SICK_INPUT_PATH                      !Path name where file with SICK input data is stored
    CHARACTER(len=150)                                      :: SICK_INPUT_FILE                      !File name of file with SICK input data
    CHARACTER(len=150)                                      :: SICK_INPUT                           !Internal name to store Path+Name of file with SICK input data
    CHARACTER(len=150)                                      :: SICK_CONTRIB_FILE                    !File name of file with CONTRIB input data for SICK module
    CHARACTER(len=150)                                      :: SICK_CONTRIB                         !Internal name to store Path+Name of file with CONTRIB input data for SICK module
    CHARACTER(len=150)                                      :: SICK_UNEM_FILE                       !File name of file with UNEM input data for SICK module
    CHARACTER(len=150)                                      :: SICK_UNEM                            !Internal name to store Path+Name of file with UNEM input data for SICK module
    CHARACTER(len=150)                                      :: SICK_PIT_FILE                        !File name of file with PIT input data for SICK module    
    CHARACTER(len=150)                                      :: SICK_PIT                             !Internal name to store Path+Name of file with PIT input data for SICK module
    CHARACTER(len=150)                                      :: SICK_PENSWELF_FILE                   !File name of file with PENSWELF input data for SICK module
    CHARACTER(len=150)                                      :: SICK_PENSWELF                        !Internal name to store Path+Name of file with PENSWELF input data for SICK module
    CHARACTER(len=150)                                      :: SICK_FAMAL_FILE                      !File name of file with FAMAL input data for SICK module
    CHARACTER(len=150)                                      :: SICK_FAMAL                           !Internal name to store Path+Name of file with FAMAL input data for SICK module
    CHARACTER(len=150)                                      :: SICK_SICK_FILE                       !File name of file with SICK input data for SICK module
    CHARACTER(len=150)                                      :: SICK_SICK                            !Internal name to store Path+Name of file with SICK input data for SICK module
    CHARACTER(len=150)                                      :: FAMREL_INPUT_PATH                    !Path name where file with FAMREL input data is stored
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_FILE                  !File name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL                       !Internal name to store Path+Name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: CONSTANT_INPUT_PATH                  !Path name where file with MIMOSIS Weights is stored
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG_FILE                !File name of file with MIMOSIS Weights                    
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG                     !Internal name to store Path+Name of file with MIMOSIS Weights
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT_FILE                  !File name of file with MIMOSIS gross wages
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT                       !Internal name to store Path+Name of file with MIMOSIS gross wages
    !Variables for locating output (local variables)
    CHARACTER(len=150)                                      :: UNEM_SICK_FILE                       !File name of file with SICK output data for UNEM module
    CHARACTER(len=150)                                      :: UNEM_SICK                            !Internal name to store Path+Name of file with UNEM output data
    CHARACTER(len=150)                                      :: FAMAL_SICK_FILE                      !File name of file with SICK output data for FAMAL module
    CHARACTER(len=150)                                      :: FAMAL_SICK                           !Internal name to store Path+Name of file with FAMAL output data   
    CHARACTER(len=150)                                      :: PENSWELF_SICK_FILE                   !File name of file with SICK output data for PENSWELF module
    CHARACTER(len=150)                                      :: PENSWELF_SICK                        !Internal name to store Path+Name of file with PENSWELF output data   
    CHARACTER(len=150)                                      :: SICK_OUTP_FILE                       !File name of file with SICK output data for SICK module
    CHARACTER(len=150)                                      :: SICK_OUTP                            !Internal name to store Path+Name of file with SICK output data   
    CHARACTER(len=150)                                      :: SOCBEN_SICK_FILE                     !File name of file with SICK output data for SOCBEN module
    CHARACTER(len=150)                                      :: SOCBEN_SICK                          !Internal name to store Path+Name of file with SOCBEN output data   
    !Tools to read input, output and parameters    
    CHARACTER(len=20)                                       :: SICK_PAR_FORMAT                      !Format of parameter read in from parameter file
    CHARACTER(len=1)                                        :: SICK_PAR_BREAK='@'                   !Character used to identify end of string
    CHARACTER(len=10)                                       :: SICK_PAR_CURR                        !Currency of the parameter read in from the parameter file
    CHARACTER(len=40),ALLOCATABLE,DIMENSION(:)              :: SICK_PAR_NAMES                       !Vector of parameter names
    INTEGER                                                 :: SICK_PAR_L                           !Number of lines (if parameter is a matrix)_
    INTEGER                                                 :: SICK_PAR_C                           !Number of columnss (if parameter is a matrix)
    INTEGER,DIMENSION(18)                                   :: SICK_PAR_BREAK_POINT                 !number of break points to be determined when combining path names and file names
    !=======================================================================!
    !Parameters of the SICK module
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: SICK_FAMREL_QT                       !Matrix used to convert relationships of family members
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_WAGEDFQ1_QT                     !Income threshold used to determine whether household member is income dependent yes or no
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_LIMPRIV_DAY                     !Thresholds used to limit earnings per day in the private labour market system
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_PERCPRIV_DAY                    !Percentages and thresholds used to determine the daily benefit for sickness or disability in the private labour market system
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_SUPPRIV_DAY                     !Percentages used to determine the supplement of blue collar workers in case of sickness
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_THIRPRIV_DAY                    !Percentages and thresholds used to determine the daily benefit for help of a third person in case of disability for an individual insured as private labour market wage earner
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_MATPRIV_DAY                     !Percentages used to determine the daily maternity benefit in the private labour market system
    REAL(KIND=KIND_REAL)                                    :: SICK_INDACCLW_DAY                    !Threshold used to limit daily wages to determine the daily benefit for industrial accident in the private labour market system
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_INDACCPC_DAY                    !Percentages and thresholds used to determine the daily benefit for industrial accident in the private labour market system
    REAL(KIND=KIND_REAL)                                    :: SICK_OCCDISLW_DAY                    !Threshold used to limit daily wages to determine the daily benefit for occupational disease in the private labour market system
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_OCCDISPC_DAY                    !Percentages and thresholds used to determine the daily benefit for occupational disease in the private labour market system
    REAL(KIND=KIND_REAL)                                    :: SICK_PEPUBSIC_DAY                    !Percentage used to compute sickness or disability benefit for individual insured as statutory civil servant
    REAL(KIND=KIND_REAL)                                    :: SICK_PEPUBMAT_DAY                    !Percentage used to compute maternity benefits for individual insured as statutory civil servant
    REAL(KIND=KIND_REAL)                                    :: SICK_PEPUBIND_DAY                    !Percentage used to compute benefits for industrial accident for individual insured as statutory civil servant
    REAL(KIND=KIND_REAL)                                    :: SICK_PEPUBOCC_DAY                    !Percentage used to compute benefits for occupational disease for individual insured as statutory civil servant
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_LSSELSIC_DAY                    !Lump sum benefits for sickness or disability for individual insured as self employed
    REAL(KIND=KIND_REAL)                                    :: SICK_THIRSELF_DAY                    !Lump sum benefit for help of a third person for individual insured as self employed
    REAL(KIND=KIND_REAL)                                    :: SICK_MATSELF_WEEK                    !Lump sum maternity benefit paid per week to individual insured as self employed
    REAL(KIND=KIND_REAL)                                    :: SICK_SSCONT_QT                       !Percentage used to detemine social security contributions paid on gross income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_COSTSCAL_ANN                    !Percentages and thresholds used to determine professional costs in the prepayment computation
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_TAXRATES_ANN                    !Tax rates and thresholds used to determine annual taxes in the prepayment computation
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_REDCHILD_ANN                    !Tax credit for children in prepayment computation
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_LABINCRA_DAY                    !Thresholds and rates used to determine daily amount that has to be subtracted from daily benefit (system from 2003)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_LABCUMUL_DAY                    !Rates used to determine daily amount that has to be subtracted from daily benefit (system before 2003)
    !=======================================================================!  
    !Exogenous variables
    !SICK_INPUT_Write.sas
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: GECODEERD_INSZ_CHAR                  ! Geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_INPUT                 ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_HH_ID                       ! Geanonimiseerd identificatienummer van het gezin waartoe het individu behoort
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NAREGNIS_DATBIRTH                    ! Geboortedatum van het individu in jaar en maand
    !/*Variabelen uit FAO-bestand*/
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: TAUXIT_2008                          ! Graad van tijdelijke arbeidsongeschiktheid (geregistreerd in 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: DDEBUT_2008                          ! Begin van de periode van ongeschiktheid (geregistreerd in 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: DFIN_2008                            ! Einde van de periode van ongeschiktheid (geregistreerd in 2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: TAUXIPRES_2008                       ! Graad van permanente arbeidsongeschiktheid (geregistreerd in 2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: TAUXATRES_2008                       ! Graad voor hulp van derden (geregistreerd in 2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FATFAO_BENEF_2008                    ! Het indvidiu ontvangt een uitkering voor een arbeidsongeval gedurende het gegeven jaar
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MONTITT_2008                         ! Bedrag voor tijdelijke arbeidsongeschiktheid: volledige afwezigheid (geregistreerd in 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MONTITP_2008                         ! Bedrag voor tijdelijke arbeidsongeschiktheid: gedeeltelijke afwezigheid (geregistreerd in 2008)
    !/*Variabelen uit FBZ-bestand*/
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: FBZ_BEGDBET_2008                     ! Begindatum betaling beroepsziekte (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: FBZ_ENDDBET_2008                     ! Einddatum betaling beroepsziekte (2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FBZ_BEDRAG_2008                      ! Betaald bedrag beroepsziekte (2008)
    CHARACTER(len=5),ALLOCATABLE,DIMENSION(:)               :: FBZ_PERCENT_2008                     ! Percentage arbeidsongeschiktheid als gevolg van beroepsziekte (2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FBZFMB_BENEF_2008                    ! Het indvidiu ontvangt een uitkering voor een beroepsziekte gedurende het gegeven jaar
    CHARACTER(len=2),ALLOCATABLE,DIMENSION(:)               :: FBZ_SRTUIT_2008                      ! Soort uitkering beroepsziekte (2008)
    !/*Variabelen uit RIZIV-bestand*/
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RIZIVINA_STELSEL_INVAL_2008          ! Code die type verzekering weergeeft (persoon in invaliditeit) (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RIZIVINA_BETALING_INVAL_2008         ! Betalingscode (persoon in invaliditeit) (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RIZIVINA_AANVANGS_INVAL_2008         ! Aanvangsdatum van de betalingsperiode (persoon in invaliditeit) (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RIZIVINA_EINDBETA_INVAL_2008         ! Einddatum van de betalingsperiode (persoon in invaliditeit) (2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RIZIVINA_BEDRAG_INVAL_2008           ! Uitgekeerd bedrag (persoon in invaliditeit) (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RIZIVINA_STELSEL_MAT_2008            ! Code die type verzekering weergeeft (zwangerschapsverlof gedurende invaliditeit) (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RIZIVINA_BETALING_MAT_2008           ! Betalingscode (zwangerschapsverlof gedurende invaliditeit) (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RIZIVINA_AANVANGS_MAT_2008           ! Aanvangsdatum van de betalingsperiode (zwangerschapsverlof gedurende invaliditeit) (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: RIZIVINA_EINDBETA_MAT_2008           ! Einddatum van de betalingsperiode (zwangerschapsverlof gedurende invaliditeit) (2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: RIZIVINA_BEDRAG_MAT_2008             ! Uitgekeerd bedrag (zwangerschapsverlof gedurende invaliditeit) (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: B_ZIEKTE_2008                        ! Begindatum primaire arbeidsongeschiktheid (2008)
    CHARACTER(len=15),ALLOCATABLE,DIMENSION(:)              :: AANWIJZ_2008                         ! Gegevens met betrekking tot de gezinslast en de arbeidsmarktpositie van het individu op het moment van de aanvraag tot erkenning invaliditeit (2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InRIZIV                              ! Individu geselecteerd uit basisdata RIZIV voor verwerking steekproef
    !/*Variabelen uit NIC-bestand*/
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NIC_STELSEL_2008                     ! Stelsel waarin het recht geklasseerd wordt (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NIC_SRTDAGEN_2008                    ! Soort dagen (PAO, zwangerschap, ...) (2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NIC_DAGENAO_2008                     ! Aantal dagen arbeidsongeschiktheid (2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: NIC_BEDRAG_2008                      ! Betaald bedrag aan uitkeringen (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NIC_BEGDATUM_2008_CORR               ! Begindatum ziekte (2008): gecorrigeerd op basis van aantal ziektedagen per lijn
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InNIC                                ! Individu geselecteerd uit basisdata NIC voor verwerking steekproef
    !/*HistorInfo*/
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: LABMSTAT_PRIV                        ! Meest voorkomende arbeidsmarktstatus op de private arbeidsmarkt in de periode 1997 tot en met eindjaar 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: LABMSTAT_PUB                         ! Meest voorkomende arbeidsmarktstatus op de publieke arbeidsmarkt in de periode 1997 tot en met eindjaar
    !Variables read from file SICK_CONTRIB.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_CONTRIB               !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REVPRIV_QT                   !Sum of gross labour income and holiday earnings, earned as wage earner on the private labour market in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REVPUB_QT                    !Sum of gross labour income and holiday earnings, earned as wage earner on the public labour market in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_INCSELF_QT                   !Gross income earned as self employed in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPRIV_EMEE_QT               !Social security contributions paid on CONTRIB_REVPRIV_QT in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPUB_EMEE_QT                !Social security contributions paid on CONTRIB_REVPUB_QT in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSSELF_EMER_QT               !Social security contributions paid on CONTRIB_INCSELF_QT in a given quater
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_PRIV_QT             !Labour market status of a wage earner on the private labour market in a given quarter
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_PUB_QT              !Labour market status of a wage earner on the public labour market in a given quarter
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_SELF_QT             !Labour market status of a self employed in a given quarter
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: MIMOSIS_SIMNUMMT                     !Unique identification number identifying the position of the individual in the household for the computations of the marginal taxes
    !Variables read from file SICK_UNEM.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_UNEM                  !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENUN_QT                        !Gross amount of real unemployment benefits in a given quarter (no benefits paid by RVA as supplement to wage)
    !Variables read from file SICK_PIT.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_PIT                   !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_DEPCHILD_ANN                     !Number of dependent children in the tax unit of the individual in a given year
    !Variables read from file SICK_PENSWELF.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_PENSWELF              !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_AMOUNTP_QT                  !Gross amount of pensions received in a given quater
    !Variables read from file SICK_FAMAL.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMAL                 !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_AMOUNTF_QT                     !Gross amount of family allowances received in a given quater by the recipient
    !Variables read from file SICK_SICK.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_SICK                  !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_TOTBENPR_QT                     !Gross amount of sickness and disability benefits estimated before sickness and disability benefits are computed
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMREL                !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMILY                        !Unique identifier identifying the family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_RELATION                      !Relation of the individual with respect to the head of the family
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMTYPE                       !Type of family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_COUPLE                        !Type of couple to which the individual belongs
    !WAGE_CONSTRUCT_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WAGE                  ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SALJRS_LAST_CORR                     ! Dagloon in laatste jaar met positieve waarde gecorrigeerd dagloon (in prijzen van 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_HOUR                   ! Uurloon gecorrigeerd voor minimum onder minimumgrens
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_QT                     ! Raming van het kwartaalinkomen in het laatst geobserveerde kwartaal
    !MIMOSIS_SAMPWEIG_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WEIGHT                ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_WEIGHT                       ! Gewicht dat corrigeert voor niet random selectie van de observatie-eenheden
    !=======================================================================!
    !Endogeneous variables
    !to rank variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NEW_RANK                             !New rank after sorting and re-ranking       
    !to construct age variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_YEAR_AGE                        !Age of the individual expressed in years
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_MONTH_AGE                       !Age of the individual expressed in moths
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_MONTH_BIRTH                     !Month of birth(mm)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_YEAR_BIRTH                      !Year of birth (yyyy)
    !to construct information at the level of the sociological household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_HH_SIZE                         !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_START_HH                        !Variable stores an identification of the first household member (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_END_HH                          !Variable stores an identification of the last household member (maximal size vector = number of individuals)   
    INTEGER                                                 :: N_HH_SICK                            !number of households analysed
    !To construct variables listed in the module note
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_RISKGLOB_QT                     !Indicator whether an individual is sick or disabled         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_REL_QT                          !Relationship of the individual with respect to other family members         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DEPINC_QT                       !Variable containing an indication whether an individual is income dependent yes or no           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_FAMSIT_QT                       !Family charge position of the sick or disabled          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_TYPEINSU_PRIV_QT                !Insurance type of the sick or disabled in the wage earner regime on the private labour market           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_TYPEINSU_PUB_QT                 !Insurance type of the sick or disabled in the wage earner regime on the public labour market            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_TYPEINSU_SELF_QT                !Insurance type of the sick or disabled as self employed         
    !Sickness and disability private             
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELIGPRI1_QT                     !Individual receives a benefit for sickness or disability in the wage earner regime of the private labour market         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_WOWEPRIV_QT                     !Type of working week the sick or disabled is working in         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DAYSPRI1_QT                     !Number of days the individual receives a benefit for sickness or disability in the given quarter (individual is insured as wage earner on the private labour market)            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_WEEKPRI1_QT                     !Number of weeks the individual already receives a benefit for sickness or disability (individual is insured as wage earner on the private labour market)            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_SOLEPRIV_QT                     !Sick or disabled lost his sole source of income when he became sick or disabled         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_REGWPRIV_QT                     !Individual was a regular worker before he became sick or disabled           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPRI1_DAY                     !Daily benefit for sickness or disability for individual in the wage earner regime of the private labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPRI1_QT                      !Quarterly benefit for sickness or disability for individual in the wage earner regime of the private labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPBPRI1_QT                     !Part of the quarterly benefit for sickness or disability paid by the employer for individual insured as wage earner on the private labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_LDAYWAGE_DAY                    !Daily wage lost when the individual became sick or disabled         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_SUPPRI1_DAY                     !Additional daily supplement for blue collar workers insured as wage earner on the private labour market         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_SUPPRI1_QT                      !Additional quarterly supplement for blue collar worker who is insured as wage earner on the private labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPSPRI1_QT                     !Part of the additional quarterly supplement paid by the employer for blue collar worker who is insured as wage earner on the private labour market          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELTHPRIV_QT                     !Individual receives a supplementary benefit for help of a third person and is insured as wage earner on the private labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BETHPRIV_DAY                    !Daily supplement for help of a third person (individual is insured as wage earner on the private labour market)         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BETHPRIV_QT                     !Quarterly supplement for help of a third person (individual is insured as wage earner on the private labour market)         
    !Maternity benefits private              
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELIGPRI2_QT                     !Individual receives a maternity benefit and is insured as wage earner on the private labour market          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_MATLPRI2_QT                     !Number of days the individual is on maternity leave at the beginning of the quarter (individual is insured as wage earner on the private labour market)         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DAYSPRI2_QT                     !Number of days the individual receives a maternity benefit in the given quarter (individual is insured as wage earner on the private labour market)         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPRI2_DAY                     !Maternity benefit per day for individual insured as wage earner on the private labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPRI2_QT                      !Maternity benefit per quarter for individual insured as wage earner on the private labour market            
    !Industrial accidents private                
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELIGPRI3_QT                     !Individual receives a benefit for industrial accident and is insured as wage earner on the private labour market            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_TYDBPRI3_QT                     !Type of benefit for an industrial accident the individual, insured as wage earner on the private labour market, is entitled to          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DAYSPRI3_QT                     !Number of days for which the individual receives a benefit for an industrial accident in the given quarter (individual is insured as wage earner on the private labour market)          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_WEEKPRI3_QT                     !Number of weeks the individual already receives a benefit for an industrial accident (individual is insured as wage earner on the private labour market)            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_DEGRPRI3_QT                     !Degree of disability because of industrial accident for individual insured as wage earner on the private labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPRI3_DAY                     !Benefit per day received in case of an industrial accident (individual is insured as wage earner on the private labour market)          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPRI3_QT                      !Benefit per quarter received in case of an industrial accident (individual is insured as wage earner on the private labour market)          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPBPRI3_QT                     !Part of the benefit per quarter received in case of an industrial accident and paid by the employer (individual is insured as wage earner on the private labour market)         
    !Occupational diseases private               
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELIGPRI4_QT                     !Individual receives a benefit for occupational disease and is insured as wage earner on the private labour market           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_TYDBPRI4_QT                     !Type of benefit for an occupational disease the individual, insured as wage earner on the private labour market, is entitled to         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DAYSPRI4_QT                     !Number of days for which the individual receives a benefit for an occupational disease in the given quarter (individual is insured as wage earner on the private labour market)         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_WEEKPRI4_QT                     !Number of weeks the individual already receives a benefit for an occupational disease (individual is insured as wage earner on the private labour market)           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_DEGRPRI4_QT                     !Degree of disability because of industrial accident for individual insured as wage earner on the private labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPRI4_DAY                     !Benefit per day received in case of an occupational disease (individual is insured as wage earner on the private labour market)         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPRI4_QT                      !Benefit per quarter received in case of an occupational disease (individual is insured as wage earner on the private labour market)         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPBPRI4_QT                     !Part of the benefit per quarter received in case of an occupational disease and paid by the employer (individual is insured as wage earner on the private labour market)            
    !Sickness and disability public              
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELIGPUB1_QT                     !Individual receives a benefit for sickness or disability in the wage earner regime of the public labour market          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_WOWEPUB_QT                      !Type of working week the sick or disabled is working in         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DAYSPUB1_QT                     !Number of days the individual receives a benefit for sickness or disability in the given quarter (individual is insured as wage earner on the public labour market)         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_WEEKPUB1_QT                     !Number of weeks the individual already receives a benefit for sickness or disability (individual is insured as wage earner on the public labour market)         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_SOLEPUB_QT                      !Sick or disabled lost his sole source of income when he became sick or disabled         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_REGWPUB_QT                      !Individual was a regular worker before he became sick or disabled           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPUB1_DAY                     !Daily benefit for sickness or disability for individual in the wage earner regime of the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPUB1_QT                      !Quarterly benefit for sickness or disability for individual in the wage earner regime of the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPBPUB1_QT                     !Part of the quarterly benefit for sickness or disability paid by the employer for individual insured as wage earner on the public labour market         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_SUPPUB1_DAY                     !Additional daily supplement for blue collar workers insured as wage earner on the public labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_SUPPUB1_QT                      !Additional quarterly supplement for blue collar worker who is insured as wage earner on the public labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPSPUB1_QT                     !Part of the additional quarterly supplement paid by the employer for blue collar worker who is insured as wage earner on the public labour market           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELTHPUB_QT                      !Individual receives a supplementary benefit for help of a third person and is insured as wage earner on the public labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BETHPUB_DAY                     !Daily supplement for help of a third person (individual is insured as wage earner on the public labour market)          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BETHPUB_QT                      !Quarterly supplement for help of a third person (individual is insured as wage earner on the public labour market)          
    !Maternity benefits public               
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELIGPUB2_QT                     !Individual receives a maternity benefit and is insured as wage earner on the public labour market           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DAYSPUB2_QT                     !Number of days the individual receives a maternity benefit in the given quarter (individual is insured as wage earner on the public labour market)          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_MATLPUB2_QT                     !Number of days the individual is on maternity leave at the beginning of the quarter (individual is insured as wage earner on the public labour market)          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPUB2_DAY                     !Maternity benefit per day for individual insured as wage earner on the public labour market         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPUB2_QT                      !Maternity benefit per quarter for individual insured as wage earner on the public labour market         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPBPUB2_QT                     !Individual receives a maternity benefit and is insured as wage earner on the public labour market           
    !Industrial accidents public             
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELIGPUB3_QT                     !Individual receives a benefit for industrial accident and is insured as wage earner on the public labour market         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_TYDBPUB3_QT                     !Type of benefit for an industrial accident the individual, insured as wage earner on the public labour market, is entitled to           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DAYSPUB3_QT                     !Number of days for which the individual receives a benefit for an industrial accident in the given quarter (individual is insured as wage earner on the public labour market)           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_WEEKPUB3_QT                     !Number of weeks the individual already receives a benefit for an industrial accident (individual is insured as wage earner on the public labour market)         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_DEGRPUB3_QT                     !Degree of disability because of industrial accident for individual insured as wage earner on the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPUB3_DAY                     !Benefit per day received in case of an industrial accident (individual is insured as wage earner on the public labour market)           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPUB3_QT                      !Benefit per quarter received in case of an industrial accident (individual is insured as wage earner on the public labour market)           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPBPUB3_QT                     !Part of the benefit per quarter received in case of an industrial accident and paid by the employer (individual is insured as wage earner on the public labour market)          
    !Occupational diseases public                
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELIGPUB4_QT                     !Individual receives a benefit for occupational disease and is insured as wage earner on the public labour market            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_TYDBPUB4_QT                     !Type of benefit for an occupational disease the individual, insured as wage earner on the public labour market, is entitled to          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DAYSPUB4_QT                     !Number of days for which the individual receives a benefit for an occupational disease in the given quarter (individual is insured as wage earner on the public labour market)          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_WEEKPUB4_QT                     !Number of weeks the individual already receives a benefit for an occupational disease (individual is insured as wage earner on the public labour market)            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_DEGRPUB4_QT                     !Degree of disability because of industrial accident for individual insured as wage earner on the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPUB4_DAY                     !Benefit per day received in case of an occupational disease (individual is insured as wage earner on the public labour market)          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENPUB4_QT                      !Benefit per quarter received in case of an occupational disease (individual is insured as wage earner on the public labour market)          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPBPUB4_QT                     !Part of the benefit per quarter received in case of an occupational disease and paid by the employer (individual is insured as wage earner on the public labour market)         
    !Sickness and disability self employed               
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELIGSEL1_QT                     !Individual receives a benefit for sickness or disability in the self employed regime            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENSEL1_DAY                     !Daily benefit for sickness or disability for individual insured as self employed            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENSEL1_QT                      !Quarterly benefit for sickness or disability for individual insured as self employed            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_WEEKSEL1_QT                     !Number of weeks the individual already receives a benefit for sickness or invalidity (individual is insured as self employed)           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DAYSSEL1_QT                     !Number of days the individual already receives a benefit for sickness or invalidity in the given quarter (individual is insured as self employed)           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_CLOSESEL_QT                     !The self employed closed his business after the incident that induced the disability status         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BETHSELF_QT                     !Quarterly supplement for help of a third person (individual is insured as self employed)            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELTHSELF_QT                     !Individual receives a supplementary benefit for help of a third person and is insured as self employed          
    !Maternity benefits self employed                
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_ELIGSEL2_QT                     !Individual receives a maternity benefit and is insured as self employed         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_WEEKSEL2_QT                     !Number of weeks the individual receives a maternity benefit in the given quarter for individual insured as a self employed          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENSEL2_QT                      !Quarterly maternity benefit for individual insured as self employed         
    !Prepayment variables                
    !Reconstructed exchange concepts             
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_TOTBEN_QT                       !Sum of benefits for sickness, disability, industrial accidents and occupational diseases            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_IND_QT                          !Disability benefits received by wage earners because of industrial accidents (benefits for help of a third person excluded)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_OCC_QT                          !Disability benefits received by wage earners because of occupational diseases (benefits for help of a third person excluded)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_DISABWEA_QT                     !Disability benefits received by wage earners on the private or public labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_ALOTHBEN_QT                     !Benefits in case of sickness or disability other than those covered by SICK_IND_QT, SICK_OCC_QT and SICK_DISABWEA_QT
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPLBPRI_QT                     !Benefits in case of sickness or disability paid by the employer in the private sector
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_EMPLBPUB_QT                     !Benefits in case of sickness or disability paid by the employer in the public sector
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENINOC_QT                      !Sum of benefits for industrial accidents and occupational diseases          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_NONEINOC_QT                     !Sum of benefits for sickness and disability         
    !Variables for counting reconstructed concepts
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_SICEMPNM_QT                     !Sum of all benefits, paid by social security budget, received by employees in sickness but not on maternity leave
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_DISEMPNM_QT                     !Sum of all benefits, paid by social security budget, received by employees in disability but not on maternity leave
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_SICSELNM_QT                     !Sum of all benefits, paid by social security budget, received by self employed in sickness but not on maternity leave
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SICK_DISSELNM_QT                     !Sum of all benefits, paid by social security budget, received by self employed in disability but not on maternity leave
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DISABBEN_QT                     !Identification of the fact that the beneficiary receives disability benefits
    !Variables to determine benefit in combination with labour activity
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_LABDEDUC_DAY                    !Amount to be deducted from daily benefit (system from 2003 on)

    !Instruments
    INTEGER                                                 :: pers_j,hh_i,i,j,k
    INTEGER                                                 :: ierror
    INTEGER                                                 :: START_DAYQT                          !First day of the quarter being analysed
    INTEGER                                                 :: STOP_DAYQT                           !Last day of the quarter being analysed    
    
END MODULE SICK_PRERUN_DATA

!==========================================!
! This module contains all the subroutines !
! and functions used in the SICK PRERUN MODULE    !
!==========================================!
!List of subroutine names
!SUBROUTINE SICK_ALLOC_EXOG(DIM_ALLOC)
!SUBROUTINE SICK_ALLOC_ENDO(DIM_ALLOC)
!SUBROUTINE SICK_DEALLOC
!SUBROUTINE SICK_INITENDO(DIM_ALLOC)
!SUBROUTINE SICK_RANKANDSORT
!SUBROUTINE SICK_PARAMETER_FILEREAD
!SUBROUTINE SICK_INPUT_FILEREAD
!SUBROUTINE SICK_CONTRIB_FILEREAD
!SUBROUTINE SICK_UNEM_FILEREAD
!SUBROUTINE SICK_PIT_FILEREAD
!SUBROUTINE SICK_PENSWELF_FILEREAD
!SUBROUTINE SICK_FAMAL_FILEREAD
!SUBROUTINE SICK_FAMREL_FILEREAD
!SUBROUTINE SICK_WAGE_FILEREAD
!SUBROUTINE SICK_WEIGHT_FILEREAD
!SUBROUTINE SICK_CONSISTENCY_CHECK
!SUBROUTINE SICK_RECODE
!SUBROUTINE SICK_HOUSEHOLD_SIZE             !Determine household size  
!SUBROUTINE SICK_AGE                        !Determine age of all individuals
!SUBROUTINE SICK_CHARAC                     !construction of different characteristics of the sick or disabled (mainly eligibility variables)
!SUBROUTINE SICK_FAMSIT_CONSTRUC            !determine the household charge of each sick or disabled
!SUBROUTINE SICK_BENPRIV_CONSTRUC(pers_j)   !Compute sickness and disability benefits for those insured as wage earner on the private labour market
!SUBROUTINE SICK_BENPUBNS_CONSTRUC(pers_j)  !Compute sickness and disability benefits for those insured as wage earner on the public labour market WITHOUT statutory service
!SUBROUTINE SICK_BENPUBST_CONSTRUC(pers_j)  !Compute sickness and disability benefits for those insured as wage earner on the public labour market WITH statutory service
!SUBROUTINE SICK_BENSELF_CONSTRUC(pers_j)   !Compute sickness and disability benefits for those insured as self employed
!SUBROUTINE SICK_OUTPUT_FILEWRIT            !Write output to output file
!SUBROUTINE SICK_TEST                       !test construction of sickness and disability constructions
!SUBROUTINE SICK_DEALLOC                    !Deallocate allocated variables
!List of functions
!FUNCTION SICK_BRACRATE_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV)   RESULT (OUTPUT_AMOUNT)
!FUNCTION SICK_SUMAMOUNT_1COLUMN(STATUS1,NLINES_AMOUNT,MAT_AMOUNT,INDIV)            RESULT (OUTPUT_AMOUNT)
!FUNCTION SICK_PREPAY_COMPUTE(INPUT_AMOUNT,CHILDREN,INDIV)                          RESULT (OUTPUT_AMOUNT)
!FUNCTION DAY_FUNC(DATE, DATE_KIND, DATE_DEF)                                       RESULT (OUTPUT_DAY)
!FUNCTION DAYS_COMPUTE(START_DAY, STOP_DAY, DIVIDE, MULTIPLY, MAX_DAY)              RESULT (OUTPUT_DAYS)
!FUNCTION WEEKS_COMPUTE(START_DAY, STOP_DAY, DIVIDE, MULTIPLY, MIN_WEEK,MAX_WEEK)   RESULT (OUTPUT_WEEKS)

MODULE SICK_PRERUN_SUBROUTINES

    USE SICK_PRERUN_DATA
    USE MIMOSIS_DATA    
    !use routine for sorting and ranking data
    USE RANKING_SORTING_ROUTINES

    IMPLICIT NONE
    CONTAINS

!=====================================================!
!Allocate all exogenous vectors of dimension DIM_ALLOC
!=====================================================!

SUBROUTINE SICK_ALLOC_EXOG(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !=======================================================================!  
    !Exogenous variables
    !SICK_INPUT_Write.sas
    ALLOCATE(GECODEERD_INSZ_CHAR(DIM_ALLOC))
    ALLOCATE(GECODEERD_INSZ_INPUT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_HH_ID(DIM_ALLOC))
    ALLOCATE(NAREGNIS_DATBIRTH(DIM_ALLOC))
    !/*Variabelen uit FAO-bestand*/
    ALLOCATE(TAUXIT_2008(DIM_ALLOC))
    ALLOCATE(DDEBUT_2008(DIM_ALLOC))
    ALLOCATE(DFIN_2008(DIM_ALLOC))
    ALLOCATE(TAUXIPRES_2008(DIM_ALLOC))
    ALLOCATE(TAUXATRES_2008(DIM_ALLOC))
    ALLOCATE(FATFAO_BENEF_2008(DIM_ALLOC))
    ALLOCATE(MONTITT_2008(DIM_ALLOC))
    ALLOCATE(MONTITP_2008(DIM_ALLOC))
    !/*Variabelen uit FBZ-bestand*/
    ALLOCATE(FBZ_BEGDBET_2008(DIM_ALLOC))
    ALLOCATE(FBZ_ENDDBET_2008(DIM_ALLOC))
    ALLOCATE(FBZ_BEDRAG_2008(DIM_ALLOC))
    ALLOCATE(FBZ_PERCENT_2008(DIM_ALLOC))
    ALLOCATE(FBZFMB_BENEF_2008(DIM_ALLOC))
    ALLOCATE(FBZ_SRTUIT_2008(DIM_ALLOC))
    !/*Variabelen uit RIZIV-bestand*/
    ALLOCATE(RIZIVINA_STELSEL_INVAL_2008(DIM_ALLOC))
    ALLOCATE(RIZIVINA_BETALING_INVAL_2008(DIM_ALLOC))
    ALLOCATE(RIZIVINA_AANVANGS_INVAL_2008(DIM_ALLOC))
    ALLOCATE(RIZIVINA_EINDBETA_INVAL_2008(DIM_ALLOC))
    ALLOCATE(RIZIVINA_BEDRAG_INVAL_2008(DIM_ALLOC))
    ALLOCATE(RIZIVINA_STELSEL_MAT_2008(DIM_ALLOC))
    ALLOCATE(RIZIVINA_BETALING_MAT_2008(DIM_ALLOC))
    ALLOCATE(RIZIVINA_AANVANGS_MAT_2008(DIM_ALLOC))
    ALLOCATE(RIZIVINA_EINDBETA_MAT_2008(DIM_ALLOC))
    ALLOCATE(RIZIVINA_BEDRAG_MAT_2008(DIM_ALLOC))
    ALLOCATE(B_ZIEKTE_2008(DIM_ALLOC))
    ALLOCATE(AANWIJZ_2008(DIM_ALLOC))
    ALLOCATE(InRIZIV(DIM_ALLOC))
    !/*Variabelen uit NIC-bestand*/
    ALLOCATE(NIC_STELSEL_2008(DIM_ALLOC))
    ALLOCATE(NIC_SRTDAGEN_2008(DIM_ALLOC))
    ALLOCATE(NIC_DAGENAO_2008(DIM_ALLOC))
    ALLOCATE(NIC_BEDRAG_2008(DIM_ALLOC))
    ALLOCATE(NIC_BEGDATUM_2008_CORR(DIM_ALLOC))
    ALLOCATE(InNIC(DIM_ALLOC))
    !/*HistorInfo*/
    ALLOCATE(LABMSTAT_PRIV(DIM_ALLOC))
    ALLOCATE(LABMSTAT_PUB(DIM_ALLOC))
    !Variables read from file SICK_CONTRIB.txt
    ALLOCATE(GECODEERD_INSZ_CONTRIB(DIM_ALLOC))
    ALLOCATE(CONTRIB_REVPRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_REVPUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_INCSELF_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPRIV_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPUB_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSSELF_EMER_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_LABMSTAT_PRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_LABMSTAT_PUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_LABMSTAT_SELF_QT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_SIMNUMMT(DIM_ALLOC))
    !Variables read from file SICK_UNEM.txt
    ALLOCATE(GECODEERD_INSZ_UNEM(DIM_ALLOC))
    ALLOCATE(UNEM_BENUN_QT(DIM_ALLOC))
    !Variables read from file SICK_PIT.txt
    ALLOCATE(GECODEERD_INSZ_PIT(DIM_ALLOC))
    ALLOCATE(PIT_DEPCHILD_ANN(DIM_ALLOC))
    !Variables read from file SICK_PENSWELF.txt
    ALLOCATE(GECODEERD_INSZ_PENSWELF(DIM_ALLOC))
    ALLOCATE(PENSWELF_AMOUNTP_QT(DIM_ALLOC))
    !Variables read from file SICK_FAMAL.txt
    ALLOCATE(GECODEERD_INSZ_FAMAL(DIM_ALLOC))
    ALLOCATE(FAMAL_AMOUNTF_QT(DIM_ALLOC))
    !Variables read from file SICK_SICK.txt
    ALLOCATE(GECODEERD_INSZ_SICK(DIM_ALLOC))
    ALLOCATE(SICK_TOTBENPR_QT(DIM_ALLOC))
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    ALLOCATE(GECODEERD_INSZ_FAMREL(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMILY(DIM_ALLOC))
    ALLOCATE(FAMREL_RELATION(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMTYPE(DIM_ALLOC))
    ALLOCATE(FAMREL_COUPLE(DIM_ALLOC))
    !WAGE_CONSTRUCT_Write.sas
    ALLOCATE(GECODEERD_INSZ_WAGE(DIM_ALLOC))
    ALLOCATE(SALJRS_LAST_CORR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_HOUR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_QT(DIM_ALLOC))
    !MIMOSIS_SAMPWEIG_Write.sas
    ALLOCATE(GECODEERD_INSZ_WEIGHT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WEIGHT(DIM_ALLOC))

END SUBROUTINE SICK_ALLOC_EXOG

!======================================================!
!Allocate all endogenous vectors of dimension DIM_ALLOC
!======================================================!

SUBROUTINE SICK_ALLOC_ENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !Endogeneous variables
    !to rank variables
    ALLOCATE(NEW_RANK(DIM_ALLOC))
    !to construct age variables
    ALLOCATE(SICK_YEAR_AGE(DIM_ALLOC))
    ALLOCATE(SICK_MONTH_AGE(DIM_ALLOC))
    ALLOCATE(SICK_MONTH_BIRTH(DIM_ALLOC))
    ALLOCATE(SICK_YEAR_BIRTH(DIM_ALLOC))
    !to construct information at the level of the sociological household
    ALLOCATE(SICK_HH_SIZE(DIM_ALLOC))
    ALLOCATE(SICK_START_HH(DIM_ALLOC))
    ALLOCATE(SICK_END_HH(DIM_ALLOC))
    !To construct variables listed in the module note
    ALLOCATE(SICK_RISKGLOB_QT(DIM_ALLOC))
    ALLOCATE(SICK_REL_QT(DIM_ALLOC))
    ALLOCATE(SICK_DEPINC_QT(DIM_ALLOC))
    ALLOCATE(SICK_FAMSIT_QT(DIM_ALLOC))
    ALLOCATE(SICK_TYPEINSU_PRIV_QT(DIM_ALLOC))
    ALLOCATE(SICK_TYPEINSU_PUB_QT(DIM_ALLOC))
    ALLOCATE(SICK_TYPEINSU_SELF_QT(DIM_ALLOC))
    !Sickness and disability private             
    ALLOCATE(SICK_ELIGPRI1_QT(DIM_ALLOC))
    ALLOCATE(SICK_WOWEPRIV_QT(DIM_ALLOC))
    ALLOCATE(SICK_DAYSPRI1_QT(DIM_ALLOC))
    ALLOCATE(SICK_WEEKPRI1_QT(DIM_ALLOC))
    ALLOCATE(SICK_SOLEPRIV_QT(DIM_ALLOC))
    ALLOCATE(SICK_REGWPRIV_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENPRI1_DAY(DIM_ALLOC))
    ALLOCATE(SICK_BENPRI1_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPBPRI1_QT(DIM_ALLOC))
    ALLOCATE(SICK_LDAYWAGE_DAY(DIM_ALLOC))
    ALLOCATE(SICK_SUPPRI1_DAY(DIM_ALLOC))
    ALLOCATE(SICK_SUPPRI1_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPSPRI1_QT(DIM_ALLOC))
    ALLOCATE(SICK_ELTHPRIV_QT(DIM_ALLOC))
    ALLOCATE(SICK_BETHPRIV_QT(DIM_ALLOC))
    ALLOCATE(SICK_BETHPRIV_DAY(DIM_ALLOC))
    !Maternity benefits private              
    ALLOCATE(SICK_ELIGPRI2_QT(DIM_ALLOC))
    ALLOCATE(SICK_MATLPRI2_QT(DIM_ALLOC))
    ALLOCATE(SICK_DAYSPRI2_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENPRI2_DAY(DIM_ALLOC))
    ALLOCATE(SICK_BENPRI2_QT(DIM_ALLOC))
    !Industrial accidents private                
    ALLOCATE(SICK_ELIGPRI3_QT(DIM_ALLOC))
    ALLOCATE(SICK_TYDBPRI3_QT(DIM_ALLOC))
    ALLOCATE(SICK_DAYSPRI3_QT(DIM_ALLOC))
    ALLOCATE(SICK_WEEKPRI3_QT(DIM_ALLOC))
    ALLOCATE(SICK_DEGRPRI3_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENPRI3_DAY(DIM_ALLOC))
    ALLOCATE(SICK_BENPRI3_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPBPRI3_QT(DIM_ALLOC))
    !Occupational diseases private               
    ALLOCATE(SICK_ELIGPRI4_QT(DIM_ALLOC))
    ALLOCATE(SICK_TYDBPRI4_QT(DIM_ALLOC))
    ALLOCATE(SICK_DAYSPRI4_QT(DIM_ALLOC))
    ALLOCATE(SICK_WEEKPRI4_QT(DIM_ALLOC))
    ALLOCATE(SICK_DEGRPRI4_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENPRI4_DAY(DIM_ALLOC))
    ALLOCATE(SICK_BENPRI4_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPBPRI4_QT(DIM_ALLOC))
    !Sickness and disability public              
    ALLOCATE(SICK_ELIGPUB1_QT(DIM_ALLOC))
    ALLOCATE(SICK_WOWEPUB_QT(DIM_ALLOC))
    ALLOCATE(SICK_DAYSPUB1_QT(DIM_ALLOC))
    ALLOCATE(SICK_WEEKPUB1_QT(DIM_ALLOC))
    ALLOCATE(SICK_SOLEPUB_QT(DIM_ALLOC))
    ALLOCATE(SICK_REGWPUB_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENPUB1_DAY(DIM_ALLOC))
    ALLOCATE(SICK_BENPUB1_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPBPUB1_QT(DIM_ALLOC))
    ALLOCATE(SICK_SUPPUB1_DAY(DIM_ALLOC))
    ALLOCATE(SICK_SUPPUB1_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPSPUB1_QT(DIM_ALLOC))
    ALLOCATE(SICK_ELTHPUB_QT(DIM_ALLOC))
    ALLOCATE(SICK_BETHPUB_DAY(DIM_ALLOC))
    ALLOCATE(SICK_BETHPUB_QT(DIM_ALLOC))
    !Maternity benefits public               
    ALLOCATE(SICK_ELIGPUB2_QT(DIM_ALLOC))
    ALLOCATE(SICK_DAYSPUB2_QT(DIM_ALLOC))
    ALLOCATE(SICK_MATLPUB2_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENPUB2_DAY(DIM_ALLOC))
    ALLOCATE(SICK_BENPUB2_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPBPUB2_QT(DIM_ALLOC))
    !Industrial accidents public             
    ALLOCATE(SICK_ELIGPUB3_QT(DIM_ALLOC))
    ALLOCATE(SICK_TYDBPUB3_QT(DIM_ALLOC))
    ALLOCATE(SICK_DAYSPUB3_QT(DIM_ALLOC))
    ALLOCATE(SICK_WEEKPUB3_QT(DIM_ALLOC))
    ALLOCATE(SICK_DEGRPUB3_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENPUB3_DAY(DIM_ALLOC))
    ALLOCATE(SICK_BENPUB3_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPBPUB3_QT(DIM_ALLOC))
    !Occupational diseases public                
    ALLOCATE(SICK_ELIGPUB4_QT(DIM_ALLOC))
    ALLOCATE(SICK_TYDBPUB4_QT(DIM_ALLOC))
    ALLOCATE(SICK_DAYSPUB4_QT(DIM_ALLOC))
    ALLOCATE(SICK_WEEKPUB4_QT(DIM_ALLOC))
    ALLOCATE(SICK_DEGRPUB4_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENPUB4_DAY(DIM_ALLOC))
    ALLOCATE(SICK_BENPUB4_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPBPUB4_QT(DIM_ALLOC))
    !Sickness and disability self employed               
    ALLOCATE(SICK_ELIGSEL1_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENSEL1_DAY(DIM_ALLOC))
    ALLOCATE(SICK_BENSEL1_QT(DIM_ALLOC))
    ALLOCATE(SICK_WEEKSEL1_QT(DIM_ALLOC))
    ALLOCATE(SICK_DAYSSEL1_QT(DIM_ALLOC))
    ALLOCATE(SICK_CLOSESEL_QT(DIM_ALLOC))
    ALLOCATE(SICK_BETHSELF_QT(DIM_ALLOC))
    ALLOCATE(SICK_ELTHSELF_QT(DIM_ALLOC))
    !Maternity benefits self employed                
    ALLOCATE(SICK_ELIGSEL2_QT(DIM_ALLOC))
    ALLOCATE(SICK_WEEKSEL2_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENSEL2_QT(DIM_ALLOC))
    !Prepayment variables                
    !Reconstructed exchange concepts             
    ALLOCATE(SICK_TOTBEN_QT(DIM_ALLOC))
    ALLOCATE(SICK_IND_QT(DIM_ALLOC))     
    ALLOCATE(SICK_OCC_QT(DIM_ALLOC))    
    ALLOCATE(SICK_DISABWEA_QT(DIM_ALLOC))
    ALLOCATE(SICK_ALOTHBEN_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPLBPRI_QT(DIM_ALLOC))
    ALLOCATE(SICK_EMPLBPUB_QT(DIM_ALLOC))
    ALLOCATE(SICK_BENINOC_QT(DIM_ALLOC))
    ALLOCATE(SICK_NONEINOC_QT(DIM_ALLOC))
    !Variables for counting reconstructed concepts
    ALLOCATE(SICK_SICEMPNM_QT(DIM_ALLOC,2))
    ALLOCATE(SICK_DISEMPNM_QT(DIM_ALLOC,2))
    ALLOCATE(SICK_SICSELNM_QT(DIM_ALLOC,2))
    ALLOCATE(SICK_DISSELNM_QT(DIM_ALLOC,2))
    ALLOCATE(SICK_DISABBEN_QT(DIM_ALLOC))
    !Variables to determine benefit in combination with labour activity
    ALLOCATE(SICK_LABDEDUC_DAY(DIM_ALLOC))

END SUBROUTINE SICK_ALLOC_ENDO

SUBROUTINE SICK_DEALLOC

    !=======================================================================!  
    !Exogenous variables
    !SICK_INPUT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_CHAR)
    DEALLOCATE(GECODEERD_INSZ_INPUT)
    DEALLOCATE(NAREGNIS_HH_ID)
    DEALLOCATE(NAREGNIS_DATBIRTH)
    !/*Variabelen uit FAO-bestand*/
    DEALLOCATE(TAUXIT_2008)
    DEALLOCATE(DDEBUT_2008)
    DEALLOCATE(DFIN_2008)
    DEALLOCATE(TAUXIPRES_2008)
    DEALLOCATE(TAUXATRES_2008)
    DEALLOCATE(FATFAO_BENEF_2008)
    DEALLOCATE(MONTITT_2008)
    DEALLOCATE(MONTITP_2008)
    !/*Variabelen uit FBZ-bestand*/
    DEALLOCATE(FBZ_BEGDBET_2008)
    DEALLOCATE(FBZ_ENDDBET_2008)
    DEALLOCATE(FBZ_BEDRAG_2008)
    DEALLOCATE(FBZ_PERCENT_2008)
    DEALLOCATE(FBZFMB_BENEF_2008)
    DEALLOCATE(FBZ_SRTUIT_2008)
    !/*Variabelen uit RIZIV-bestand*/
    DEALLOCATE(RIZIVINA_STELSEL_INVAL_2008)
    DEALLOCATE(RIZIVINA_BETALING_INVAL_2008)
    DEALLOCATE(RIZIVINA_AANVANGS_INVAL_2008)
    DEALLOCATE(RIZIVINA_EINDBETA_INVAL_2008)
    DEALLOCATE(RIZIVINA_BEDRAG_INVAL_2008)
    DEALLOCATE(RIZIVINA_STELSEL_MAT_2008)
    DEALLOCATE(RIZIVINA_BETALING_MAT_2008)
    DEALLOCATE(RIZIVINA_AANVANGS_MAT_2008)
    DEALLOCATE(RIZIVINA_EINDBETA_MAT_2008)
    DEALLOCATE(RIZIVINA_BEDRAG_MAT_2008)
    DEALLOCATE(B_ZIEKTE_2008)
    DEALLOCATE(AANWIJZ_2008)
    DEALLOCATE(InRIZIV)
    !/*Variabelen uit NIC-bestand*/
    DEALLOCATE(NIC_STELSEL_2008)
    DEALLOCATE(NIC_SRTDAGEN_2008)
    DEALLOCATE(NIC_DAGENAO_2008)
    DEALLOCATE(NIC_BEDRAG_2008)
    DEALLOCATE(NIC_BEGDATUM_2008_CORR)
    DEALLOCATE(InNIC)
    !/*HistorInfo*/
    DEALLOCATE(LABMSTAT_PRIV)
    DEALLOCATE(LABMSTAT_PUB)
    !Variables read from file SICK_CONTRIB.txt
    DEALLOCATE(GECODEERD_INSZ_CONTRIB)
    DEALLOCATE(CONTRIB_REVPRIV_QT)
    DEALLOCATE(CONTRIB_REVPUB_QT)
    DEALLOCATE(CONTRIB_INCSELF_QT)
    DEALLOCATE(CONTRIB_SSPRIV_EMEE_QT)
    DEALLOCATE(CONTRIB_SSPUB_EMEE_QT)
    DEALLOCATE(CONTRIB_SSSELF_EMER_QT)
    DEALLOCATE(CONTRIB_LABMSTAT_PRIV_QT)
    DEALLOCATE(CONTRIB_LABMSTAT_PUB_QT)
    DEALLOCATE(CONTRIB_LABMSTAT_SELF_QT)
    DEALLOCATE(MIMOSIS_SIMNUMMT)
    !Variables read from file SICK_UNEM.txt
    DEALLOCATE(GECODEERD_INSZ_UNEM)
    DEALLOCATE(UNEM_BENUN_QT)
    !Variables read from file SICK_PIT.txt
    DEALLOCATE(GECODEERD_INSZ_PIT)
    DEALLOCATE(PIT_DEPCHILD_ANN)
    !Variables read from file SICK_PENSWELF.txt
    DEALLOCATE(GECODEERD_INSZ_PENSWELF)
    DEALLOCATE(PENSWELF_AMOUNTP_QT)
    !Variables read from file SICK_FAMAL.txt
    DEALLOCATE(GECODEERD_INSZ_FAMAL)
    DEALLOCATE(FAMAL_AMOUNTF_QT)
    !Variables read from file SICK_SICK.txt
    DEALLOCATE(GECODEERD_INSZ_SICK)
    DEALLOCATE(SICK_TOTBENPR_QT)
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    DEALLOCATE(GECODEERD_INSZ_FAMREL)
    DEALLOCATE(FAMREL_FAMILY)
    DEALLOCATE(FAMREL_RELATION)
    DEALLOCATE(FAMREL_FAMTYPE)
    DEALLOCATE(FAMREL_COUPLE)
    !WAGE_CONSTRUCT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WAGE)
    DEALLOCATE(SALJRS_LAST_CORR)
    DEALLOCATE(MIMOSIS_GRINC_HOUR)
    DEALLOCATE(MIMOSIS_GRINC_QT)
    !MIMOSIS_SAMPWEIG_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WEIGHT)
    DEALLOCATE(MIMOSIS_WEIGHT)

    !=======================================================================!  
    !Endogeneous variables
    !to rank variables
    DEALLOCATE(NEW_RANK)
    !to construct age variables
    DEALLOCATE(SICK_YEAR_AGE)
    DEALLOCATE(SICK_MONTH_AGE)
    DEALLOCATE(SICK_MONTH_BIRTH)
    DEALLOCATE(SICK_YEAR_BIRTH)
    !to construct information at the level of the sociological household
    DEALLOCATE(SICK_HH_SIZE)
    DEALLOCATE(SICK_START_HH)
    DEALLOCATE(SICK_END_HH)
    !To construct variables listed in the module note
    DEALLOCATE(SICK_RISKGLOB_QT)
    DEALLOCATE(SICK_REL_QT)
    DEALLOCATE(SICK_DEPINC_QT)
    DEALLOCATE(SICK_FAMSIT_QT)
    DEALLOCATE(SICK_TYPEINSU_PRIV_QT)
    DEALLOCATE(SICK_TYPEINSU_PUB_QT)
    DEALLOCATE(SICK_TYPEINSU_SELF_QT)
    !Sickness and disability private             
    DEALLOCATE(SICK_ELIGPRI1_QT)
    DEALLOCATE(SICK_WOWEPRIV_QT)
    DEALLOCATE(SICK_DAYSPRI1_QT)
    DEALLOCATE(SICK_WEEKPRI1_QT)
    DEALLOCATE(SICK_SOLEPRIV_QT)
    DEALLOCATE(SICK_REGWPRIV_QT)
    DEALLOCATE(SICK_BENPRI1_DAY)
    DEALLOCATE(SICK_BENPRI1_QT)
    DEALLOCATE(SICK_EMPBPRI1_QT)
    DEALLOCATE(SICK_LDAYWAGE_DAY)
    DEALLOCATE(SICK_SUPPRI1_DAY)
    DEALLOCATE(SICK_SUPPRI1_QT)
    DEALLOCATE(SICK_EMPSPRI1_QT)
    DEALLOCATE(SICK_ELTHPRIV_QT)
    DEALLOCATE(SICK_BETHPRIV_QT)
    DEALLOCATE(SICK_BETHPRIV_DAY)
    !Maternity benefits private              
    DEALLOCATE(SICK_ELIGPRI2_QT)
    DEALLOCATE(SICK_MATLPRI2_QT)
    DEALLOCATE(SICK_DAYSPRI2_QT)
    DEALLOCATE(SICK_BENPRI2_DAY)
    DEALLOCATE(SICK_BENPRI2_QT)
    !Industrial accidents private                
    DEALLOCATE(SICK_ELIGPRI3_QT)
    DEALLOCATE(SICK_TYDBPRI3_QT)
    DEALLOCATE(SICK_DAYSPRI3_QT)
    DEALLOCATE(SICK_WEEKPRI3_QT)
    DEALLOCATE(SICK_DEGRPRI3_QT)
    DEALLOCATE(SICK_BENPRI3_DAY)
    DEALLOCATE(SICK_BENPRI3_QT)
    DEALLOCATE(SICK_EMPBPRI3_QT)
    !Occupational diseases private               
    DEALLOCATE(SICK_ELIGPRI4_QT)
    DEALLOCATE(SICK_TYDBPRI4_QT)
    DEALLOCATE(SICK_DAYSPRI4_QT)
    DEALLOCATE(SICK_WEEKPRI4_QT)
    DEALLOCATE(SICK_DEGRPRI4_QT)
    DEALLOCATE(SICK_BENPRI4_DAY)
    DEALLOCATE(SICK_BENPRI4_QT)
    DEALLOCATE(SICK_EMPBPRI4_QT)
    !Sickness and disability public              
    DEALLOCATE(SICK_ELIGPUB1_QT)
    DEALLOCATE(SICK_WOWEPUB_QT)
    DEALLOCATE(SICK_DAYSPUB1_QT)
    DEALLOCATE(SICK_WEEKPUB1_QT)
    DEALLOCATE(SICK_SOLEPUB_QT)
    DEALLOCATE(SICK_REGWPUB_QT)
    DEALLOCATE(SICK_BENPUB1_DAY)
    DEALLOCATE(SICK_BENPUB1_QT)
    DEALLOCATE(SICK_EMPBPUB1_QT)
    DEALLOCATE(SICK_SUPPUB1_DAY)
    DEALLOCATE(SICK_SUPPUB1_QT)
    DEALLOCATE(SICK_EMPSPUB1_QT)
    DEALLOCATE(SICK_ELTHPUB_QT)
    DEALLOCATE(SICK_BETHPUB_DAY)
    DEALLOCATE(SICK_BETHPUB_QT)
    !Maternity benefits public               
    DEALLOCATE(SICK_ELIGPUB2_QT)
    DEALLOCATE(SICK_DAYSPUB2_QT)
    DEALLOCATE(SICK_MATLPUB2_QT)
    DEALLOCATE(SICK_BENPUB2_DAY)
    DEALLOCATE(SICK_BENPUB2_QT)
    DEALLOCATE(SICK_EMPBPUB2_QT)
    !Industrial accidents public             
    DEALLOCATE(SICK_ELIGPUB3_QT)
    DEALLOCATE(SICK_TYDBPUB3_QT)
    DEALLOCATE(SICK_DAYSPUB3_QT)
    DEALLOCATE(SICK_WEEKPUB3_QT)
    DEALLOCATE(SICK_DEGRPUB3_QT)
    DEALLOCATE(SICK_BENPUB3_DAY)
    DEALLOCATE(SICK_BENPUB3_QT)
    DEALLOCATE(SICK_EMPBPUB3_QT)
    !Occupational diseases public                
    DEALLOCATE(SICK_ELIGPUB4_QT)
    DEALLOCATE(SICK_TYDBPUB4_QT)
    DEALLOCATE(SICK_DAYSPUB4_QT)
    DEALLOCATE(SICK_WEEKPUB4_QT)
    DEALLOCATE(SICK_DEGRPUB4_QT)
    DEALLOCATE(SICK_BENPUB4_DAY)
    DEALLOCATE(SICK_BENPUB4_QT)
    DEALLOCATE(SICK_EMPBPUB4_QT)
    !Sickness and disability self employed               
    DEALLOCATE(SICK_ELIGSEL1_QT)
    DEALLOCATE(SICK_BENSEL1_DAY)
    DEALLOCATE(SICK_BENSEL1_QT)
    DEALLOCATE(SICK_WEEKSEL1_QT)
    DEALLOCATE(SICK_DAYSSEL1_QT)
    DEALLOCATE(SICK_CLOSESEL_QT)
    DEALLOCATE(SICK_BETHSELF_QT)
    DEALLOCATE(SICK_ELTHSELF_QT)
    !Maternity benefits self employed                
    DEALLOCATE(SICK_ELIGSEL2_QT)
    DEALLOCATE(SICK_WEEKSEL2_QT)
    DEALLOCATE(SICK_BENSEL2_QT)
    !Prepayment variables                
    !Reconstructed exchange concepts             
    DEALLOCATE(SICK_TOTBEN_QT)
    DEALLOCATE(SICK_IND_QT)    
    DEALLOCATE(SICK_OCC_QT)   
    DEALLOCATE(SICK_DISABWEA_QT)
    DEALLOCATE(SICK_ALOTHBEN_QT)
    DEALLOCATE(SICK_EMPLBPRI_QT)
    DEALLOCATE(SICK_EMPLBPUB_QT)
    DEALLOCATE(SICK_BENINOC_QT)
    DEALLOCATE(SICK_NONEINOC_QT)
    !Variables for counting reconstructed concepts
    DEALLOCATE(SICK_SICEMPNM_QT)
    DEALLOCATE(SICK_DISEMPNM_QT)
    DEALLOCATE(SICK_SICSELNM_QT)
    DEALLOCATE(SICK_DISSELNM_QT)
    DEALLOCATE(SICK_DISABBEN_QT)
    !Variables to determine benefit in combination with labour activity
    DEALLOCATE(SICK_LABDEDUC_DAY)

    !=======================================================================!
    !Deallocate parameters
    DEALLOCATE(SICK_FAMREL_QT)                      
    DEALLOCATE(SICK_WAGEDFQ1_QT)     
    DEALLOCATE(SICK_LIMPRIV_DAY)                     
    DEALLOCATE(SICK_PERCPRIV_DAY)                    
    DEALLOCATE(SICK_SUPPRIV_DAY)                     
    DEALLOCATE(SICK_THIRPRIV_DAY)                    
    DEALLOCATE(SICK_MATPRIV_DAY)                     
    DEALLOCATE(SICK_INDACCPC_DAY)                    
    DEALLOCATE(SICK_OCCDISPC_DAY)                    
    DEALLOCATE(SICK_LSSELSIC_DAY)                    
    DEALLOCATE(SICK_COSTSCAL_ANN)                    
    DEALLOCATE(SICK_TAXRATES_ANN)                    
    DEALLOCATE(SICK_REDCHILD_ANN)                    
    DEALLOCATE(SICK_LABINCRA_DAY)                    
    DEALLOCATE(SICK_LABCUMUL_DAY)
    
END SUBROUTINE SICK_DEALLOC

!===============================================!
!Initialise endogenous variables                        
!===============================================!

SUBROUTINE SICK_INITENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors
    INTEGER                                                 :: pers_j                                    !index for do loop

    DO pers_j=1,DIM_ALLOC
        !to rank variables
        NEW_RANK(pers_j) = 0
        !to construct age variables
        SICK_YEAR_AGE(pers_j) = 0
        SICK_MONTH_AGE(pers_j) = 0
        SICK_MONTH_BIRTH(pers_j) = 0
        SICK_YEAR_BIRTH(pers_j) = 0
        !to construct information at the level of the sociological household
        SICK_HH_SIZE(pers_j) = 0
        SICK_START_HH(pers_j) = 0
        SICK_END_HH(pers_j) = 0
        !To construct variables listed in the module note
        SICK_RISKGLOB_QT(pers_j) = 0
        SICK_REL_QT(pers_j) = 0
        SICK_DEPINC_QT(pers_j) = 0
        SICK_FAMSIT_QT(pers_j) = 0
        SICK_TYPEINSU_PRIV_QT(pers_j) = 0
        SICK_TYPEINSU_PUB_QT(pers_j) = 0
        SICK_TYPEINSU_SELF_QT(pers_j) = 0
        !Sickness and disability private             
        SICK_ELIGPRI1_QT(pers_j) = 0
        SICK_WOWEPRIV_QT(pers_j) = 0
        SICK_DAYSPRI1_QT(pers_j) = 0
        SICK_WEEKPRI1_QT(pers_j) = 0
        SICK_SOLEPRIV_QT(pers_j) = 0
        SICK_REGWPRIV_QT(pers_j) = 0
        SICK_BENPRI1_DAY(pers_j) = 0
        SICK_BENPRI1_QT(pers_j) = 0
        SICK_EMPBPRI1_QT(pers_j) = 0
        SICK_LDAYWAGE_DAY(pers_j) = 0
        SICK_SUPPRI1_DAY(pers_j) = 0
        SICK_SUPPRI1_QT(pers_j) = 0
        SICK_EMPSPRI1_QT(pers_j) = 0
        SICK_ELTHPRIV_QT(pers_j) = 0
        SICK_BETHPRIV_QT(pers_j) = 0
        SICK_BETHPRIV_DAY(pers_j) = 0
        !Maternity benefits private              
        SICK_ELIGPRI2_QT(pers_j) = 0
        SICK_MATLPRI2_QT(pers_j) = 0
        SICK_DAYSPRI2_QT(pers_j) = 0
        SICK_BENPRI2_DAY(pers_j) = 0
        SICK_BENPRI2_QT(pers_j) = 0
        !Industrial accidents private                
        SICK_ELIGPRI3_QT(pers_j) = 0
        SICK_TYDBPRI3_QT(pers_j) = 0
        SICK_DAYSPRI3_QT(pers_j) = 0
        SICK_WEEKPRI3_QT(pers_j) = 0
        SICK_DEGRPRI3_QT(pers_j) = 0
        SICK_BENPRI3_DAY(pers_j) = 0
        SICK_BENPRI3_QT(pers_j) = 0
        SICK_EMPBPRI3_QT(pers_j) = 0
        !Occupational diseases private               
        SICK_ELIGPRI4_QT(pers_j) = 0
        SICK_TYDBPRI4_QT(pers_j) = 0
        SICK_DAYSPRI4_QT(pers_j) = 0
        SICK_WEEKPRI4_QT(pers_j) = 0
        SICK_DEGRPRI4_QT(pers_j) = 0
        SICK_BENPRI4_DAY(pers_j) = 0
        SICK_BENPRI4_QT(pers_j) = 0
        SICK_EMPBPRI4_QT(pers_j) = 0
        !Sickness and disability public              
        SICK_ELIGPUB1_QT(pers_j) = 0
        SICK_WOWEPUB_QT(pers_j) = 0
        SICK_DAYSPUB1_QT(pers_j) = 0
        SICK_WEEKPUB1_QT(pers_j) = 0
        SICK_SOLEPUB_QT(pers_j) = 0
        SICK_REGWPUB_QT(pers_j) = 0
        SICK_BENPUB1_DAY(pers_j) = 0
        SICK_BENPUB1_QT(pers_j) = 0
        SICK_EMPBPUB1_QT(pers_j) = 0
        SICK_SUPPUB1_DAY(pers_j) = 0
        SICK_SUPPUB1_QT(pers_j) = 0
        SICK_EMPSPUB1_QT(pers_j) = 0
        SICK_ELTHPUB_QT(pers_j) = 0
        SICK_BETHPUB_DAY(pers_j) = 0
        SICK_BETHPUB_QT(pers_j) = 0
        !Maternity benefits public               
        SICK_ELIGPUB2_QT(pers_j) = 0
        SICK_DAYSPUB2_QT(pers_j) = 0
        SICK_MATLPUB2_QT(pers_j) = 0
        SICK_BENPUB2_DAY(pers_j) = 0
        SICK_BENPUB2_QT(pers_j) = 0
        SICK_EMPBPUB2_QT(pers_j) = 0
        !Industrial accidents public             
        SICK_ELIGPUB3_QT(pers_j) = 0
        SICK_TYDBPUB3_QT(pers_j) = 0
        SICK_DAYSPUB3_QT(pers_j) = 0
        SICK_WEEKPUB3_QT(pers_j) = 0
        SICK_DEGRPUB3_QT(pers_j) = 0
        SICK_BENPUB3_DAY(pers_j) = 0
        SICK_BENPUB3_QT(pers_j) = 0
        SICK_EMPBPUB3_QT(pers_j) = 0
        !Occupational diseases public                
        SICK_ELIGPUB4_QT(pers_j) = 0
        SICK_TYDBPUB4_QT(pers_j) = 0
        SICK_DAYSPUB4_QT(pers_j) = 0
        SICK_WEEKPUB4_QT(pers_j) = 0
        SICK_DEGRPUB4_QT(pers_j) = 0
        SICK_BENPUB4_DAY(pers_j) = 0
        SICK_BENPUB4_QT(pers_j) = 0
        SICK_EMPBPUB4_QT(pers_j) = 0
        !Sickness and disability self employed               
        SICK_ELIGSEL1_QT(pers_j) = 0
        SICK_BENSEL1_DAY(pers_j) = 0
        SICK_BENSEL1_QT(pers_j) = 0
        SICK_WEEKSEL1_QT(pers_j) = 0
        SICK_DAYSSEL1_QT(pers_j) = 0
        SICK_CLOSESEL_QT(pers_j) = 0
        SICK_BETHSELF_QT(pers_j) = 0
        SICK_ELTHSELF_QT(pers_j) = 0
        !Maternity benefits self employed                
        SICK_ELIGSEL2_QT(pers_j) = 0
        SICK_WEEKSEL2_QT(pers_j) = 0
        SICK_BENSEL2_QT(pers_j) = 0
        !Prepayment variables                
        !Reconstructed exchange concepts             
        SICK_TOTBEN_QT(pers_j) = 0
        SICK_IND_QT(pers_j) = 0     
        SICK_OCC_QT(pers_j) = 0   
        SICK_DISABWEA_QT(pers_j) = 0
        SICK_ALOTHBEN_QT(pers_j) = 0
        SICK_EMPLBPRI_QT(pers_j) = 0
        SICK_EMPLBPUB_QT(pers_j) = 0
        SICK_BENINOC_QT(pers_j) = 0
        SICK_NONEINOC_QT(pers_j) = 0
        !Variables for counting reconstructed concepts
        SICK_SICEMPNM_QT(pers_j,1:2) = 0
        SICK_DISEMPNM_QT(pers_j,1:2) = 0
        SICK_SICSELNM_QT(pers_j,1:2) = 0
        SICK_DISSELNM_QT(pers_j,1:2) = 0
        SICK_DISABBEN_QT(pers_j) = 0
        !Variables to determine benefit in combination with labour activity
        SICK_LABDEDUC_DAY(pers_j) = 0
    END DO
    N_HH_SICK=0

END SUBROUTINE SICK_INITENDO

!========== SUBROUTINE SICK_RANKANDSORT ======================================!
!RANK and SORT exogenous variables                                            !  
!   Step 1: rank and sort all variables according to their                    !
!           individual identification number which is read in as an integer   !
!   Step 2: rank and sort all variables according to the                      !
!           household identification number                                   !
!=============================================================================!

SUBROUTINE SICK_RANKANDSORT

!STEP1: ranking and sorting with the individuals identification number
    !Rank individual identification number from the INPUT data file
    CALL RANKING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    !SICK_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    !/*Variabelen uit FAO-bestand*/
    CALL SORTING(TAUXIT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DDEBUT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DFIN_2008(1:n_rec),NEW_RANK)
    CALL SORTING(TAUXIPRES_2008(1:n_rec),NEW_RANK)
    CALL SORTING(TAUXATRES_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FATFAO_BENEF_2008(1:n_rec),NEW_RANK)
    CALL SORTING(MONTITT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(MONTITP_2008(1:n_rec),NEW_RANK)
    !/*Variabelen uit FBZ-bestand*/
    CALL SORTING(FBZ_BEGDBET_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FBZ_ENDDBET_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FBZ_BEDRAG_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FBZ_PERCENT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FBZFMB_BENEF_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FBZ_SRTUIT_2008(1:n_rec),NEW_RANK)
    !/*Variabelen uit RIZIV-bestand*/
    CALL SORTING(RIZIVINA_STELSEL_INVAL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_BETALING_INVAL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_AANVANGS_INVAL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_EINDBETA_INVAL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_BEDRAG_INVAL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_STELSEL_MAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_BETALING_MAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_AANVANGS_MAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_EINDBETA_MAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_BEDRAG_MAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(B_ZIEKTE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(AANWIJZ_2008(1:n_rec),NEW_RANK)
    CALL SORTING(InRIZIV(1:n_rec),NEW_RANK)
    !/*Variabelen uit NIC-bestand*/
    CALL SORTING(NIC_STELSEL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NIC_SRTDAGEN_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NIC_DAGENAO_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NIC_BEDRAG_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NIC_BEGDATUM_2008_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(InNIC(1:n_rec),NEW_RANK)
    !/*HistorInfo*/
    CALL SORTING(LABMSTAT_PRIV(1:n_rec),NEW_RANK)
    CALL SORTING(LABMSTAT_PUB(1:n_rec),NEW_RANK)
    !Rank individual identification number from SICK_CONTRIB.txt
    CALL RANKING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_CONTRIB.txt
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_INCSELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_SELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_SIMNUMMT(1:n_rec),NEW_RANK)
    !Rank individual identification number from SICK_UNEM.txt
    CALL RANKING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_UNEM.txt
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_BENUN_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from SICK_PIT.txt
    CALL RANKING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_PIT.txt
    CALL SORTING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_DEPCHILD_ANN(1:n_rec),NEW_RANK)
    !Rank individual identification number from SICK_PENSWELF.txt
    CALL RANKING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_PENSWELF.txt
    CALL SORTING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_AMOUNTP_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from SICK_FAMAL.txt
    CALL RANKING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_FAMAL.txt
    CALL SORTING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMAL_AMOUNTF_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from SICK_SICK.txt
    CALL RANKING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_SICK.txt
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TOTBENPR_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_FAMREL.txt                                           
    CALL RANKING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !Rank individual identification number from file WAGE_CONSTRUCT.txt                                           
    CALL RANKING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_WEIGHT.txt
    CALL RANKING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL SICK_CONSISTENCY_CHECK
   
!STEP2: ranking and sorting with the household identification number
    !Rank household number
    CALL RANKING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    !Sort variables on rank of household number
    !SICK_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    !/*Variabelen uit FAO-bestand*/
    CALL SORTING(TAUXIT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DDEBUT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DFIN_2008(1:n_rec),NEW_RANK)
    CALL SORTING(TAUXIPRES_2008(1:n_rec),NEW_RANK)
    CALL SORTING(TAUXATRES_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FATFAO_BENEF_2008(1:n_rec),NEW_RANK)
    CALL SORTING(MONTITT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(MONTITP_2008(1:n_rec),NEW_RANK)
    !/*Variabelen uit FBZ-bestand*/
    CALL SORTING(FBZ_BEGDBET_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FBZ_ENDDBET_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FBZ_BEDRAG_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FBZ_PERCENT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FBZFMB_BENEF_2008(1:n_rec),NEW_RANK)
    CALL SORTING(FBZ_SRTUIT_2008(1:n_rec),NEW_RANK)
    !/*Variabelen uit RIZIV-bestand*/
    CALL SORTING(RIZIVINA_STELSEL_INVAL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_BETALING_INVAL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_AANVANGS_INVAL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_EINDBETA_INVAL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_BEDRAG_INVAL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_STELSEL_MAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_BETALING_MAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_AANVANGS_MAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_EINDBETA_MAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RIZIVINA_BEDRAG_MAT_2008(1:n_rec),NEW_RANK)
    CALL SORTING(B_ZIEKTE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(AANWIJZ_2008(1:n_rec),NEW_RANK)
    CALL SORTING(InRIZIV(1:n_rec),NEW_RANK)
    !/*Variabelen uit NIC-bestand*/
    CALL SORTING(NIC_STELSEL_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NIC_SRTDAGEN_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NIC_DAGENAO_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NIC_BEDRAG_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NIC_BEGDATUM_2008_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(InNIC(1:n_rec),NEW_RANK)
    !/*HistorInfo*/
    CALL SORTING(LABMSTAT_PRIV(1:n_rec),NEW_RANK)
    CALL SORTING(LABMSTAT_PUB(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_CONTRIB.txt
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_INCSELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_SELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_SIMNUMMT(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_UNEM.txt
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_BENUN_QT(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_PIT.txt
    CALL SORTING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_DEPCHILD_ANN(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_PENSWELF.txt
    CALL SORTING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_AMOUNTP_QT(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_FAMAL.txt
    CALL SORTING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMAL_AMOUNTF_QT(1:n_rec),NEW_RANK)
    !Sort variables from file SICK_SICK.txt
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TOTBENPR_QT(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL SICK_CONSISTENCY_CHECK

END SUBROUTINE SICK_RANKANDSORT

!========== SUBROUTINE PARAMETER_FILEREAD ===============!
!This subroutines reads in all parameters from           !
!the parameter file                                      !
!========================================================!
SUBROUTINE SICK_PARAMETER_FILEREAD

    !Read in parameters for path and name of input files
    DO i=1,n_loclines/2
        !Read default path names
        !Start reading the normal file names
        IF (CONTROL_IO(2*i-1)=='SICK_INPUT_PATH') THEN
            SICK_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SICK_INPUT_FILE') THEN
            SICK_INPUT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SICK_CONTRIB_PRERUN_FILE') THEN
            SICK_CONTRIB_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SICK_UNEM_PRERUN_FILE') THEN
            SICK_UNEM_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SICK_PIT_PRERUN_FILE') THEN
            SICK_PIT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SICK_PENSWELF_PRERUN_FILE') THEN
            SICK_PENSWELF_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SICK_FAMAL_PRERUN_FILE') THEN
            SICK_FAMAL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SICK_SICK_PRERUN_FILE') THEN
            SICK_SICK_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_INPUT_PATH') THEN
            FAMREL_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_FAMREL_FILE') THEN
            MIMOSIS_FAMREL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='WAGE_CONSTRUCT_FILE') THEN
            WAGE_CONSTRUCT_FILE = CONTROL_IO(2*i)
        !Read material to read sample weights                                                    
        ELSE IF (CONTROL_IO(2*i-1)=='CONSTANT_INPUT_PATH') THEN
            CONSTANT_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_SAMPWEIG_FILE') THEN
            MIMOSIS_SAMPWEIG_FILE = CONTROL_IO(2*i)
        !Read the file names of the intermediate output files
        ELSE IF (CONTROL_IO(2*i-1)=='UNEM_SICK_PRERUN_FILE') THEN
            UNEM_SICK_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='PENSWELF_SICK_FILE') THEN
            PENSWELF_SICK_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_SICK_FILE') THEN
            FAMAL_SICK_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SICK_SICK_FILE') THEN
            SICK_OUTP_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_SICK_PRERUN_FILE') THEN
            SOCBEN_SICK_FILE = CONTROL_IO(2*i)
        !Read information necessary to construct file for evaluation module
        END IF    
    END DO

    !check for length of default path and file name parameters
    !Input file specific for SICK
    SICK_PAR_BREAK_POINT(1) = INDEX(SICK_INPUT_PATH,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(2) = INDEX(SICK_INPUT_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(3) = INDEX(SICK_CONTRIB_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(4) = INDEX(SICK_UNEM_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(5) = INDEX(SICK_PIT_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(6) = INDEX(SICK_PENSWELF_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(7) = INDEX(SICK_FAMAL_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(15)= INDEX(SICK_SICK_FILE,SICK_PAR_BREAK)
    !Input files with MIMOSIS info
    SICK_PAR_BREAK_POINT(8)  = INDEX(FAMREL_INPUT_PATH,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(9)  = INDEX(MIMOSIS_FAMREL_FILE,SICK_PAR_BREAK)    
    SICK_PAR_BREAK_POINT(10) = INDEX(WAGE_CONSTRUCT_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(11) = INDEX(CONSTANT_INPUT_PATH,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(12) = INDEX(MIMOSIS_SAMPWEIG_FILE,SICK_PAR_BREAK)
    !Output files 
    SICK_PAR_BREAK_POINT(13) = INDEX(UNEM_SICK_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(14) = INDEX(FAMAL_SICK_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(16) = INDEX(SICK_OUTP_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(17) = INDEX(PENSWELF_SICK_FILE,SICK_PAR_BREAK)
    SICK_PAR_BREAK_POINT(18) = INDEX(SOCBEN_SICK_FILE,SICK_PAR_BREAK)

    !merge information (path+name) to create default input file names
    !Input file specific for SICK
    SICK_INPUT = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &SICK_INPUT_FILE(1:SICK_PAR_BREAK_POINT(2)-1)
    SICK_CONTRIB = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &SICK_CONTRIB_FILE(1:SICK_PAR_BREAK_POINT(3)-1)
    SICK_UNEM = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &SICK_UNEM_FILE(1:SICK_PAR_BREAK_POINT(4)-1)
    SICK_PIT = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &SICK_PIT_FILE(1:SICK_PAR_BREAK_POINT(5)-1)
    SICK_PENSWELF = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &SICK_PENSWELF_FILE(1:SICK_PAR_BREAK_POINT(6)-1)
    SICK_FAMAL = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &SICK_FAMAL_FILE(1:SICK_PAR_BREAK_POINT(7)-1)
    SICK_SICK = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &SICK_SICK_FILE(1:SICK_PAR_BREAK_POINT(15)-1)
    !Input files with MIMOSIS info
    MIMOSIS_FAMREL = FAMREL_INPUT_PATH(1:SICK_PAR_BREAK_POINT(8)-1)//&
    &MIMOSIS_FAMREL_FILE(1:SICK_PAR_BREAK_POINT(9)-1)    
    WAGE_CONSTRUCT = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &WAGE_CONSTRUCT_FILE(1:SICK_PAR_BREAK_POINT(10)-1)
    MIMOSIS_SAMPWEIG = CONSTANT_INPUT_PATH(1:SICK_PAR_BREAK_POINT(11)-1)//&
    &MIMOSIS_SAMPWEIG_FILE(1:SICK_PAR_BREAK_POINT(12)-1)    
    !Output files 
    UNEM_SICK = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &UNEM_SICK_FILE(1:SICK_PAR_BREAK_POINT(13)-1)
    FAMAL_SICK = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &FAMAL_SICK_FILE(1:SICK_PAR_BREAK_POINT(14)-1)
    SICK_OUTP = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &SICK_OUTP_FILE(1:SICK_PAR_BREAK_POINT(16)-1)
    PENSWELF_SICK = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &PENSWELF_SICK_FILE(1:SICK_PAR_BREAK_POINT(17)-1)
    SOCBEN_SICK = SICK_INPUT_PATH(1:SICK_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_SICK_FILE(1:SICK_PAR_BREAK_POINT(18)-1)

    !Trim the result
    !Input file specific for SICK
    CALL TRIM@(SICK_INPUT) 
    CALL TRIM@(SICK_CONTRIB) 
    CALL TRIM@(SICK_UNEM) 
    CALL TRIM@(SICK_PIT) 
    CALL TRIM@(SICK_PENSWELF) 
    CALL TRIM@(SICK_FAMAL) 
    CALL TRIM@(SICK_SICK) 
    !Input files with MIMOSIS info
    CALL TRIM@(MIMOSIS_FAMREL)    
    CALL TRIM@(WAGE_CONSTRUCT)  
    CALL TRIM@(MIMOSIS_SAMPWEIG)
    !Output files 
    CALL TRIM@(UNEM_SICK)
    CALL TRIM@(FAMAL_SICK)
    CALL TRIM@(SICK_OUTP)
    CALL TRIM@(PENSWELF_SICK)
    CALL TRIM@(SOCBEN_SICK)
    
    !write names to log file
    !Input file specific for SICK
    WRITE(15,'(''Print of reconstructed input and output file names'')')
    CALL DRAWLINE(15,100,'=')    
    WRITE(15,*) 'SICK_INPUT= ',        SICK_INPUT 
    WRITE(15,*) 'SICK_CONTRIB= ',      SICK_CONTRIB 
    WRITE(15,*) 'SICK_UNEM= ',         SICK_UNEM 
    WRITE(15,*) 'SICK_PIT= ',          SICK_PIT 
    WRITE(15,*) 'SICK_PENSWELF= ',     SICK_PENSWELF 
    WRITE(15,*) 'SICK_FAMAL= ',        SICK_FAMAL 
    WRITE(15,*) 'SICK_SICK= ',         SICK_SICK
    !Input files with MIMOSIS info
    WRITE(15,*) 'MIMOSIS_FAMREL= ',    MIMOSIS_FAMREL   
    WRITE(15,*) 'WAGE_CONSTRUCT= ',    WAGE_CONSTRUCT  
    WRITE(15,*) 'MIMOSIS_SAMPWEIG= ',  MIMOSIS_SAMPWEIG
    !Output files 
    WRITE(15,*) 'UNEM_SICK= ',         UNEM_SICK
    WRITE(15,*) 'FAMAL_SICK= ',        FAMAL_SICK
    WRITE(15,*) 'SICK_OUTP= ',         SICK_OUTP
    WRITE(15,*) 'PENSWELF_SICK= ',     PENSWELF_SICK
    WRITE(15,*) 'SOCBEN_SICK= ',       SOCBEN_SICK

    !open parameter file        
    OPEN(UNIT=3, FILE=SICK_PARLOC, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_PARLOC
        WRITE(15,'("Could not find ", a100)') SICK_PARLOC
        PAUSE
        STOP
    END IF

    ALLOCATE(SICK_PAR_NAMES(n_par))
    !read in all parameters by looping over the lines in the parameter file
    WRITE(15,'(/,''Print of read in operation of parameter file'')')
    CALL DRAWLINE(15,100,'=')    
    i=1
    DO
        READ(3,'(a40,2i10,a10)') SICK_PAR_NAMES(i),SICK_PAR_L, &
        &SICK_PAR_C,SICK_PAR_CURR
        IF (SICK_PAR_NAMES(i)=='SICK_FAMREL_QT') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_FAMREL_QT(SICK_PAR_L,SICK_PAR_C))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) ((SICK_FAMREL_QT(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) ((SICK_FAMREL_QT(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_WAGEDFQ1_QT') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_WAGEDFQ1_QT(SICK_PAR_L))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) (SICK_WAGEDFQ1_QT(j),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) (SICK_WAGEDFQ1_QT(j),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_LIMPRIV_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_LIMPRIV_DAY(SICK_PAR_L))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) (SICK_LIMPRIV_DAY(j),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) (SICK_LIMPRIV_DAY(j),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_PERCPRIV_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_PERCPRIV_DAY(SICK_PAR_L,SICK_PAR_C))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) ((SICK_PERCPRIV_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) ((SICK_PERCPRIV_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_SUPPRIV_DAY ') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_SUPPRIV_DAY (SICK_PAR_L))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) (SICK_SUPPRIV_DAY (j),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) (SICK_SUPPRIV_DAY (j),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_THIRPRIV_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_THIRPRIV_DAY(SICK_PAR_L,SICK_PAR_C))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) ((SICK_THIRPRIV_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) ((SICK_THIRPRIV_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_MATPRIV_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_MATPRIV_DAY(SICK_PAR_L))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) (SICK_MATPRIV_DAY(j),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) (SICK_MATPRIV_DAY(j),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_INDACCLW_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) SICK_INDACCLW_DAY
            WRITE(15,SICK_PAR_FORMAT) SICK_INDACCLW_DAY
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_INDACCPC_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_INDACCPC_DAY(SICK_PAR_L,SICK_PAR_C))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) ((SICK_INDACCPC_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) ((SICK_INDACCPC_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_OCCDISLW_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) SICK_OCCDISLW_DAY
            WRITE(15,SICK_PAR_FORMAT) SICK_OCCDISLW_DAY
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_OCCDISPC_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_OCCDISPC_DAY(SICK_PAR_L,SICK_PAR_C))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) ((SICK_OCCDISPC_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) ((SICK_OCCDISPC_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_PEPUBSIC_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) SICK_PEPUBSIC_DAY
            WRITE(15,SICK_PAR_FORMAT) SICK_PEPUBSIC_DAY
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_PEPUBMAT_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) SICK_PEPUBMAT_DAY
            WRITE(15,SICK_PAR_FORMAT) SICK_PEPUBMAT_DAY
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_PEPUBIND_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) SICK_PEPUBIND_DAY
            WRITE(15,SICK_PAR_FORMAT) SICK_PEPUBIND_DAY
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_PEPUBOCC_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) SICK_PEPUBOCC_DAY
            WRITE(15,SICK_PAR_FORMAT) SICK_PEPUBOCC_DAY
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_LSSELSIC_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_LSSELSIC_DAY(SICK_PAR_L,SICK_PAR_C))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) ((SICK_LSSELSIC_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) ((SICK_LSSELSIC_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_THIRSELF_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) SICK_THIRSELF_DAY
            WRITE(15,SICK_PAR_FORMAT) SICK_THIRSELF_DAY
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_MATSELF_WEEK') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) SICK_MATSELF_WEEK
            WRITE(15,SICK_PAR_FORMAT) SICK_MATSELF_WEEK
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_SSCONT_QT') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) SICK_SSCONT_QT
            WRITE(15,SICK_PAR_FORMAT) SICK_SSCONT_QT
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_COSTSCAL_ANN') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_COSTSCAL_ANN(SICK_PAR_L,SICK_PAR_C))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) ((SICK_COSTSCAL_ANN(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) ((SICK_COSTSCAL_ANN(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_TAXRATES_ANN') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_TAXRATES_ANN(SICK_PAR_L,SICK_PAR_C))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) ((SICK_TAXRATES_ANN(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) ((SICK_TAXRATES_ANN(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_REDCHILD_ANN') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_REDCHILD_ANN(SICK_PAR_L))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) (SICK_REDCHILD_ANN(j),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) (SICK_REDCHILD_ANN(j),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_LABINCRA_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_LABINCRA_DAY(SICK_PAR_L,SICK_PAR_C))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) ((SICK_LABINCRA_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) ((SICK_LABINCRA_DAY(j,k),k=1,SICK_PAR_C),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='SICK_LABCUMUL_DAY') THEN
            WRITE(15,'(a40)') SICK_PAR_NAMES(i)
            ALLOCATE(SICK_LABCUMUL_DAY(SICK_PAR_L))
            READ(3,'(a20)') SICK_PAR_FORMAT
            READ(3,SICK_PAR_FORMAT) (SICK_LABCUMUL_DAY(j),j=1,SICK_PAR_L)
            WRITE(15,SICK_PAR_FORMAT) (SICK_LABCUMUL_DAY(j),j=1,SICK_PAR_L)
        ELSE IF (SICK_PAR_NAMES(i)=='end_parameters') THEN
            EXIT
        END IF
        i=i+1
    END DO
    DEALLOCATE(SICK_PAR_NAMES)
    
    !close the parameter file
    CLOSE(UNIT=3)
    WRITE(*,*) 'Subroutine SICK_PARAMETER_FILEREAD of PRE RUN.........runned correctly'  

END SUBROUTINE SICK_PARAMETER_FILEREAD

!============= SUBROUTINE INPUT_FILEREAD ================!
!This subroutines reads in data from INPUT file          !
!========================================================!

SUBROUTINE SICK_INPUT_FILEREAD

    OPEN(UNIT=5, FILE=SICK_INPUT, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_INPUT
        WRITE(15,'("Could not find ", a100)') SICK_INPUT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(A14,I10,A7,I10,A7,A7,I10,I10,I10,F15.2,F15.2,A7,A7,F15.2,A5,I10,&
            &A2,A7,A7,A7,A7,F15.2,A7,A7,A7,A7,F15.2,A7,A15,I10,A7,A7,&
            &I10,F15.2,A7,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
            &TAUXIT_2008(pers_j),DDEBUT_2008(pers_j),DFIN_2008(pers_j),&
            &TAUXIPRES_2008(pers_j),TAUXATRES_2008(pers_j),FATFAO_BENEF_2008(pers_j),&
            &MONTITT_2008(pers_j),MONTITP_2008(pers_j),FBZ_BEGDBET_2008(pers_j),&
            &FBZ_ENDDBET_2008(pers_j),FBZ_BEDRAG_2008(pers_j),FBZ_PERCENT_2008(pers_j),&
            &FBZFMB_BENEF_2008(pers_j),FBZ_SRTUIT_2008(pers_j),RIZIVINA_STELSEL_INVAL_2008(pers_j),&
            &RIZIVINA_BETALING_INVAL_2008(pers_j),RIZIVINA_AANVANGS_INVAL_2008(pers_j),RIZIVINA_EINDBETA_INVAL_2008(pers_j),&
            &RIZIVINA_BEDRAG_INVAL_2008(pers_j),RIZIVINA_STELSEL_MAT_2008(pers_j),RIZIVINA_BETALING_MAT_2008(pers_j),&
            &RIZIVINA_AANVANGS_MAT_2008(pers_j),RIZIVINA_EINDBETA_MAT_2008(pers_j),RIZIVINA_BEDRAG_MAT_2008(pers_j),&
            &B_ZIEKTE_2008(pers_j),AANWIJZ_2008(pers_j),INRIZIV(pers_j),&
            &NIC_STELSEL_2008(pers_j),NIC_SRTDAGEN_2008(pers_j),NIC_DAGENAO_2008(pers_j),&
            &NIC_BEDRAG_2008(pers_j),NIC_BEGDATUM_2008_CORR(pers_j),INNIC(pers_j),&
            &LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SICK_INPUT
            END IF
            !Read in numeric part of the individual indentification character variable
            READ (GECODEERD_INSZ_CHAR(pers_j)(8:), '(I10)') GECODEERD_INSZ_INPUT(pers_j)
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &TAUXIT_2008(pers_j),DDEBUT_2008(pers_j),DFIN_2008(pers_j),&
                &TAUXIPRES_2008(pers_j),TAUXATRES_2008(pers_j),FATFAO_BENEF_2008(pers_j),&
                &MONTITT_2008(pers_j),MONTITP_2008(pers_j),FBZ_BEGDBET_2008(pers_j),&
                &FBZ_ENDDBET_2008(pers_j),FBZ_BEDRAG_2008(pers_j),FBZ_PERCENT_2008(pers_j),&
                &FBZFMB_BENEF_2008(pers_j),FBZ_SRTUIT_2008(pers_j),RIZIVINA_STELSEL_INVAL_2008(pers_j),&
                &RIZIVINA_BETALING_INVAL_2008(pers_j),RIZIVINA_AANVANGS_INVAL_2008(pers_j),RIZIVINA_EINDBETA_INVAL_2008(pers_j),&
                &RIZIVINA_BEDRAG_INVAL_2008(pers_j),RIZIVINA_STELSEL_MAT_2008(pers_j),RIZIVINA_BETALING_MAT_2008(pers_j),&
                &RIZIVINA_AANVANGS_MAT_2008(pers_j),RIZIVINA_EINDBETA_MAT_2008(pers_j),RIZIVINA_BEDRAG_MAT_2008(pers_j),&
                &B_ZIEKTE_2008(pers_j),AANWIJZ_2008(pers_j),INRIZIV(pers_j),&
                &NIC_STELSEL_2008(pers_j),NIC_SRTDAGEN_2008(pers_j),NIC_DAGENAO_2008(pers_j),&
                &NIC_BEDRAG_2008(pers_j),NIC_BEGDATUM_2008_CORR(pers_j),INNIC(pers_j),&
                &LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j)'')')
                WRITE(15,'(A14,I10,A7,I10,A7,A7,I10,I10,I10,F15.2,F15.2,A7,A7,F15.2,A5,I10,&
                &A2,A7,A7,A7,A7,F15.2,A7,A7,A7,A7,F15.2,A7,A15,I10,A7,A7,&
                &I10,F15.2,A7,I10,I10,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &TAUXIT_2008(pers_j),DDEBUT_2008(pers_j),DFIN_2008(pers_j),&
                &TAUXIPRES_2008(pers_j),TAUXATRES_2008(pers_j),FATFAO_BENEF_2008(pers_j),&
                &MONTITT_2008(pers_j),MONTITP_2008(pers_j),FBZ_BEGDBET_2008(pers_j),&
                &FBZ_ENDDBET_2008(pers_j),FBZ_BEDRAG_2008(pers_j),FBZ_PERCENT_2008(pers_j),&
                &FBZFMB_BENEF_2008(pers_j),FBZ_SRTUIT_2008(pers_j),RIZIVINA_STELSEL_INVAL_2008(pers_j),&
                &RIZIVINA_BETALING_INVAL_2008(pers_j),RIZIVINA_AANVANGS_INVAL_2008(pers_j),RIZIVINA_EINDBETA_INVAL_2008(pers_j),&
                &RIZIVINA_BEDRAG_INVAL_2008(pers_j),RIZIVINA_STELSEL_MAT_2008(pers_j),RIZIVINA_BETALING_MAT_2008(pers_j),&
                &RIZIVINA_AANVANGS_MAT_2008(pers_j),RIZIVINA_EINDBETA_MAT_2008(pers_j),RIZIVINA_BEDRAG_MAT_2008(pers_j),&
                &B_ZIEKTE_2008(pers_j),AANWIJZ_2008(pers_j),INRIZIV(pers_j),&
                &NIC_STELSEL_2008(pers_j),NIC_SRTDAGEN_2008(pers_j),NIC_DAGENAO_2008(pers_j),&
                &NIC_BEDRAG_2008(pers_j),NIC_BEGDATUM_2008_CORR(pers_j),INNIC(pers_j),&
                &LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(A14,I10,A7,I10,A7,A7,I10,I10,I10,F15.2,F15.2,A7,A7,F15.2,A5,I10,&
                &A2,A7,A7,A7,A7,F15.2,A7,A7,A7,A7,F15.2,A7,A15,I10,A7,A7,&
                &I10,F15.2,A7,I10,I10,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &TAUXIT_2008(pers_j),DDEBUT_2008(pers_j),DFIN_2008(pers_j),&
                &TAUXIPRES_2008(pers_j),TAUXATRES_2008(pers_j),FATFAO_BENEF_2008(pers_j),&
                &MONTITT_2008(pers_j),MONTITP_2008(pers_j),FBZ_BEGDBET_2008(pers_j),&
                &FBZ_ENDDBET_2008(pers_j),FBZ_BEDRAG_2008(pers_j),FBZ_PERCENT_2008(pers_j),&
                &FBZFMB_BENEF_2008(pers_j),FBZ_SRTUIT_2008(pers_j),RIZIVINA_STELSEL_INVAL_2008(pers_j),&
                &RIZIVINA_BETALING_INVAL_2008(pers_j),RIZIVINA_AANVANGS_INVAL_2008(pers_j),RIZIVINA_EINDBETA_INVAL_2008(pers_j),&
                &RIZIVINA_BEDRAG_INVAL_2008(pers_j),RIZIVINA_STELSEL_MAT_2008(pers_j),RIZIVINA_BETALING_MAT_2008(pers_j),&
                &RIZIVINA_AANVANGS_MAT_2008(pers_j),RIZIVINA_EINDBETA_MAT_2008(pers_j),RIZIVINA_BEDRAG_MAT_2008(pers_j),&
                &B_ZIEKTE_2008(pers_j),AANWIJZ_2008(pers_j),INRIZIV(pers_j),&
                &NIC_STELSEL_2008(pers_j),NIC_SRTDAGEN_2008(pers_j),NIC_DAGENAO_2008(pers_j),&
                &NIC_BEDRAG_2008(pers_j),NIC_BEGDATUM_2008_CORR(pers_j),INNIC(pers_j),&
                &LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SICK_INPUT_FILEREAD of PRE RUN.............runned correctly'
    WRITE(15,'(I7, " records uploaded from INPUT_FILE")') pers_j-1

END SUBROUTINE SICK_INPUT_FILEREAD

!============= SUBROUTINE CONTRIB_FILEREAD ================!
!This subroutines reads in data from the SICK file         !
!==========================================================!
SUBROUTINE SICK_CONTRIB_FILEREAD

    OPEN(UNIT=5, FILE=SICK_CONTRIB, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_CONTRIB
        WRITE(15,'("Could not find ", a100)') SICK_CONTRIB
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),&                  
            &CONTRIB_INCSELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&              
            &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&            
            &CONTRIB_LABMSTAT_SELF_QT(pers_j),MIMOSIS_SIMNUMMT(pers_j)      
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SICK_CONTRIB
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),&                  
                &CONTRIB_INCSELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&              
                &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&            
                &CONTRIB_LABMSTAT_SELF_QT(pers_j),MIMOSIS_SIMNUMMT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),&                  
                &CONTRIB_INCSELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&              
                &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&            
                &CONTRIB_LABMSTAT_SELF_QT(pers_j),MIMOSIS_SIMNUMMT(pers_j)      
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),&                  
                &CONTRIB_INCSELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&              
                &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&            
                &CONTRIB_LABMSTAT_SELF_QT(pers_j),MIMOSIS_SIMNUMMT(pers_j)      
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SICK_CONTRIB_FILEREAD of PRE RUN...........runned correctly'
    WRITE(15,'(I7, " records uploaded from CONTRIB_FILE")') pers_j-1

END SUBROUTINE SICK_CONTRIB_FILEREAD

!============= SUBROUTINE UNEM_FILEREAD ================!
!This subroutines reads in data from the UNEM file      !
!=======================================================!
SUBROUTINE SICK_UNEM_FILEREAD

    OPEN(UNIT=5, FILE=SICK_UNEM, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_UNEM
        WRITE(15,'("Could not find ", a100)') SICK_UNEM
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUN_QT(pers_j)                     
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SICK_UNEM
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUN_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUN_QT(pers_j)                     
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUN_QT(pers_j)                     
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SICK_UNEM_FILEREAD of PRE RUN..............runned correctly'
    WRITE(15,'(I7, " records uploaded from UNEM_FILE")') pers_j-1

END SUBROUTINE SICK_UNEM_FILEREAD

!==================================!
!Reads in data from the PIT file   !
!==================================!

SUBROUTINE SICK_PIT_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=SICK_PIT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_PIT
        WRITE(15,'("Could not find ", a100)') SICK_PIT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SICK_PIT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)'')')
                WRITE(15,'(7x,I7,I10)') &
                &GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)                     
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10)') &
                &GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)                     
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine SICK_PIT_FILEREAD of PRE RUN...............runned correctly'
    WRITE (15,'(I7, " records uploaded from PIT_FILE")') pers_j-1

END SUBROUTINE SICK_PIT_FILEREAD

!============= SUBROUTINE PENSWELF_FILEREAD ================!
!This subroutines reads in data from the PENSWELF file      !
!===========================================================!
SUBROUTINE SICK_PENSWELF_FILEREAD

    OPEN(UNIT=5, FILE=SICK_PENSWELF, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_PENSWELF
        WRITE(15,'("Could not find ", a100)') SICK_PENSWELF
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_AMOUNTP_QT(pers_j) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SICK_PENSWELF
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_AMOUNTP_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_AMOUNTP_QT(pers_j)                     
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_AMOUNTP_QT(pers_j)                     
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SICK_PENSWELF_FILEREAD of PRE RUN..........runned correctly'
    WRITE(15,'(I7, " records uploaded from PENSWELF_FILE")') pers_j-1

END SUBROUTINE SICK_PENSWELF_FILEREAD

!============= SUBROUTINE FAMAL_FILEREAD ================!
!This subroutines reads in data from the FAMAL file      !
!========================================================!
SUBROUTINE SICK_FAMAL_FILEREAD

    OPEN(UNIT=5, FILE=SICK_FAMAL, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_FAMAL
        WRITE(15,'("Could not find ", a100)') SICK_FAMAL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMAL(pers_j),FAMAL_AMOUNTF_QT(pers_j) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SICK_FAMAL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_PENSWELF(pers_j),FAMAL_AMOUNTF_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_PENSWELF(pers_j),FAMAL_AMOUNTF_QT(pers_j)                     
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_PENSWELF(pers_j),FAMAL_AMOUNTF_QT(pers_j)                     
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SICK_FAMAL_FILEREAD of PRE RUN.............runned correctly'
    WRITE(15,'(I7, " records uploaded from FAMAL_FILE")') pers_j-1

END SUBROUTINE SICK_FAMAL_FILEREAD

!============= SUBROUTINE SICK_FILEREAD =================!
!This subroutines reads in data from the SICK file       !
!========================================================!
SUBROUTINE SICK_SICK_FILEREAD

    OPEN(UNIT=5, FILE=SICK_SICK, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_SICK
        WRITE(15,'("Could not find ", a100)') SICK_SICK
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_SICK(pers_j),SICK_TOTBENPR_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SICK_SICK
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_SICK(pers_j),SICK_TOTBENPR_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_TOTBENPR_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_TOTBENPR_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SICK_SICK_FILEREAD of PRE RUN..............runned correctly'
    WRITE(15,'(I7, " records uploaded from SICK_FILE")') pers_j-1

END SUBROUTINE SICK_SICK_FILEREAD

!=====================================!
!Reads in data from the FAMREL file   !
!=====================================!

SUBROUTINE SICK_FAMREL_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=MIMOSIS_FAMREL, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') MIMOSIS_FAMREL
        WRITE(15,'("Could not find ", a100)') MIMOSIS_FAMREL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
            &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_FAMREL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)                     
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)                     
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine SICK_FAMREL_FILEREAD of PRE RUN............runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_FAMREL")') pers_j-1

END SUBROUTINE SICK_FAMREL_FILEREAD

!=======================================!
!Reads in data from the WAGE INPUT file !
!=======================================!

SUBROUTINE SICK_WAGE_FILEREAD

    OPEN (UNIT=5, FILE=WAGE_CONSTRUCT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror .NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') WAGE_CONSTRUCT
        WRITE (15,'("Could not find or open ", a100)') WAGE_CONSTRUCT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
            &MIMOSIS_GRINC_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,WAGE_CONSTRUCT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine SICK_WAGE_FILEREAD of PRE RUN..............runned correctly'
    WRITE (15,'(I7, " records uploaded from WAGE_CONSTRUCT")') pers_j-1 

END SUBROUTINE SICK_WAGE_FILEREAD

!===========================================!
!Reads in data from the MIMOSIS_WEIGHT file !
!===========================================!

SUBROUTINE SICK_WEIGHT_FILEREAD

    OPEN (UNIT=5, FILE=MIMOSIS_SAMPWEIG, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror .NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') MIMOSIS_SAMPWEIG
        WRITE (15,'("Could not find or open ", a100)') MIMOSIS_SAMPWEIG
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_SAMPWEIG
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine SICK_WEIGHT_FILEREAD of PRE RUN............runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_SAMPWEIG")') pers_j-1 

END SUBROUTINE SICK_WEIGHT_FILEREAD

!=======================================================================!
!Check whether individual identification numbers are sorted consistently!
!=======================================================================!

SUBROUTINE SICK_CONSISTENCY_CHECK

    INTEGER:: pers_j
    
    DO pers_j=1, n_rec
        IF (&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))   .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))       .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))  .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))     .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))    .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))     .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j)).OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))   .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))  .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))  .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))        .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))   .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))      .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))       .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))     .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))       .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))     .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))    .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))       .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))        .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))      .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))        .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))      .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))  .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))   .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j)) .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))   .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j)) .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))      .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))    .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))      .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))    .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))     .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))       .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))     .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))     .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))   .OR.&
            &(GECODEERD_INSZ_WAGE(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))         &
            &) THEN
                WRITE (*,*) "Error: Ranking of external files does not match!"
                PAUSE
                STOP
        END IF
    END DO   

END SUBROUTINE SICK_CONSISTENCY_CHECK 

!============= SUBROUTINE SICK_RECODE==========================!
!This subroutines treat missing value                          !
!==============================================================!
SUBROUTINE SICK_RECODE

    DO pers_j=1,n_rec
        !Exogenous variables
        !Variables read from file SICK_INPUT.txt
        !SICK_INPUT_WRITE.SAS
        IF (INT(NAREGNIS_HH_ID(pers_j)).EQ.9999999)                 NAREGNIS_HH_ID(pers_j)=0
        CALL TRIM@(NAREGNIS_DATBIRTH(pers_j))
        !/*VARIABELEN UIT FAO-BESTAND*/
        IF (INT(TAUXIT_2008(pers_j)).EQ.9999999)                    TAUXIT_2008(pers_j)=0
        CALL TRIM@(DDEBUT_2008(pers_j))
        CALL TRIM@(DFIN_2008(pers_j))
        IF (INT(TAUXIPRES_2008(pers_j)).EQ.9999999)                 TAUXIPRES_2008(pers_j)=0
        IF (INT(TAUXATRES_2008(pers_j)).EQ.9999999)                 TAUXATRES_2008(pers_j)=0
        IF (INT(FATFAO_BENEF_2008(pers_j)).EQ.9999999)              FATFAO_BENEF_2008(pers_j)=0
        IF (INT(MONTITT_2008(pers_j)).EQ.9999999)                   MONTITT_2008(pers_j)=0
        IF (INT(MONTITP_2008(pers_j)).EQ.9999999)                   MONTITP_2008(pers_j)=0
        !/*VARIABELEN UIT FBZ-BESTAND*/
        CALL TRIM@(FBZ_BEGDBET_2008(pers_j))
        CALL TRIM@(FBZ_ENDDBET_2008(pers_j))
        IF (INT(FBZ_BEDRAG_2008(pers_j)).EQ.9999999)                FBZ_BEDRAG_2008(pers_j)=0
        CALL TRIM@(FBZ_PERCENT_2008(pers_j))
        IF (INT(FBZFMB_BENEF_2008(pers_j)).EQ.9999999)              FBZFMB_BENEF_2008(pers_j)=0
        CALL TRIM@(FBZ_SRTUIT_2008(pers_j))
        !/*VARIABELEN UIT RIZIV-BESTAND*/
        CALL TRIM@(RIZIVINA_STELSEL_INVAL_2008(pers_j))
        CALL TRIM@(RIZIVINA_BETALING_INVAL_2008(pers_j))
        CALL TRIM@(RIZIVINA_AANVANGS_INVAL_2008(pers_j))
        CALL TRIM@(RIZIVINA_EINDBETA_INVAL_2008(pers_j))
        IF (INT(RIZIVINA_BEDRAG_INVAL_2008(pers_j)).EQ.9999999)     RIZIVINA_BEDRAG_INVAL_2008(pers_j)=0
        CALL TRIM@(RIZIVINA_STELSEL_MAT_2008(pers_j))
        CALL TRIM@(RIZIVINA_BETALING_MAT_2008(pers_j))
        CALL TRIM@(RIZIVINA_AANVANGS_MAT_2008(pers_j))
        CALL TRIM@(RIZIVINA_EINDBETA_MAT_2008(pers_j))
        IF (INT(RIZIVINA_BEDRAG_MAT_2008(pers_j)).EQ.9999999)       RIZIVINA_BEDRAG_MAT_2008(pers_j)=0
        CALL TRIM@(B_ZIEKTE_2008(pers_j))
        CALL TRIM@(AANWIJZ_2008(pers_j))
        IF (INT(INRIZIV(pers_j)).EQ.9999999)                        INRIZIV(pers_j)=0
        !/*VARIABELEN UIT NIC-BESTAND*/
        CALL TRIM@(NIC_STELSEL_2008(pers_j))
        CALL TRIM@(NIC_SRTDAGEN_2008(pers_j))
        IF (INT(NIC_DAGENAO_2008(pers_j)).EQ.9999999)               NIC_DAGENAO_2008(pers_j)=0
        IF (INT(NIC_BEDRAG_2008(pers_j)).EQ.9999999)                NIC_BEDRAG_2008(pers_j)=0
        CALL TRIM@(NIC_BEGDATUM_2008_CORR(pers_j))
        IF (INT(INNIC(pers_j)).EQ.9999999)                          INNIC(pers_j)=0
        !/*HISTORINFO*/
        IF (INT(LABMSTAT_PRIV(pers_j)).EQ.9999999)                  LABMSTAT_PRIV(pers_j)=0
        IF (INT(LABMSTAT_PUB(pers_j)).EQ.9999999)                   LABMSTAT_PUB(pers_j)=0
        !Variables read from file SICK_CONTRIB.txt
        IF (INT(CONTRIB_REVPRIV_QT(pers_j)).EQ.9999999)             CONTRIB_REVPRIV_QT(pers_j)=0
        IF (INT(CONTRIB_REVPUB_QT(pers_j)).EQ.9999999)              CONTRIB_REVPUB_QT(pers_j)=0
        IF (INT(CONTRIB_INCSELF_QT(pers_j)).EQ.9999999)             CONTRIB_INCSELF_QT(pers_j)=0
        IF (INT(CONTRIB_SSPRIV_EMEE_QT(pers_j)).EQ.9999999)         CONTRIB_SSPRIV_EMEE_QT(pers_j)=0
        IF (INT(CONTRIB_SSPUB_EMEE_QT(pers_j)).EQ.9999999)          CONTRIB_SSPUB_EMEE_QT(pers_j)=0
        IF (INT(CONTRIB_SSSELF_EMER_QT(pers_j)).EQ.9999999)         CONTRIB_SSSELF_EMER_QT(pers_j)=0
        !Variables read from file SICK_UNEM.txt
        IF (INT(UNEM_BENUN_QT(pers_j)).EQ.9999999)                  UNEM_BENUN_QT(pers_j)=0
        !Variables read from file SICK_PIT.txt
        !Variables read from file SICK_PENSWELF.txt
        IF (INT(PENSWELF_AMOUNTP_QT(pers_j)).EQ.9999999)            PENSWELF_AMOUNTP_QT(pers_j)=0
        !Variables read from file SICK_FAMAL.txt
        IF (INT(FAMAL_AMOUNTF_QT(pers_j)).EQ.9999999)               FAMAL_AMOUNTF_QT(pers_j)=0
        !Variables read from file SICK_SICK.txt
        IF (INT(SICK_TOTBENPR_QT(pers_j)).EQ.9999999)               SICK_TOTBENPR_QT(pers_j)=0
        !Variables read from file MIMOSIS_FAMREL.txt                                           
        !WAGE_CONSTRUCT_WRITE.SAS
        IF (INT(SALJRS_LAST_CORR(pers_j)).EQ.9999999)               SALJRS_LAST_CORR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_HOUR(pers_j)).EQ.9999999)             MIMOSIS_GRINC_HOUR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_QT(pers_j)).EQ.9999999)               MIMOSIS_GRINC_QT(pers_j)=0
        !MIMOSIS_SAMPWEIG_WRITE.SAS
        IF (INT(MIMOSIS_WEIGHT(pers_j)).EQ.9999999)                 MIMOSIS_WEIGHT(pers_j)=0
    END DO   

END SUBROUTINE SICK_RECODE

!============= SUBROUTINE SICK_INFLATE==================================!
!This subroutines inflates nominal values to an appropriate price level !
!=======================================================================!
SUBROUTINE SICK_INFLATE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        !Exogenous variables
        !Variables read from file SICK_INPUT.txt
        !/*Variabelen uit FAO-bestand*/
        MONTITT_2008(pers_j)                = INFLATE_NOMVAL(3)*MONTITT_2008(pers_j)                         ! Bedrag voor tijdelijke arbeidsongeschiktheid: volledige afwezigheid (geregistreerd in 2008)
        MONTITP_2008(pers_j)                = INFLATE_NOMVAL(3)*MONTITP_2008(pers_j)                         ! Bedrag voor tijdelijke arbeidsongeschiktheid: gedeeltelijke afwezigheid (geregistreerd in 2008)
        !/*Variabelen uit FBZ-bestand*/
        FBZ_BEDRAG_2008(pers_j)             = INFLATE_NOMVAL(3)*FBZ_BEDRAG_2008(pers_j)                      ! Betaald bedrag beroepsziekte (2008)
        !/*Variabelen uit RIZIV-bestand*/
        RIZIVINA_BEDRAG_INVAL_2008(pers_j)  = INFLATE_NOMVAL(3)*RIZIVINA_BEDRAG_INVAL_2008(pers_j)           ! Uitgekeerd bedrag (persoon in invaliditeit) (2008)
        RIZIVINA_BEDRAG_MAT_2008(pers_j)    = INFLATE_NOMVAL(3)*RIZIVINA_BEDRAG_MAT_2008(pers_j)             ! Uitgekeerd bedrag (zwangerschapsverlof gedurende invaliditeit) (2008)
        !/*Variabelen uit NIC-bestand*/
        NIC_BEDRAG_2008(pers_j)             = INFLATE_NOMVAL(3)*NIC_BEDRAG_2008(pers_j)                      ! Betaald bedrag aan uitkeringen (2008)
        !/*HistorInfo*/
        !Variables read from file SICK_CONTRIB.txt
!        CONTRIB_REVPRIV_QT(pers_j)      = INFLATE_NOMVAL(1)*CONTRIB_REVPRIV_QT(pers_j)                   !Sum of gross labour income and holiday earnings, earned as wage earner on the private labour market in a given quater
!        CONTRIB_REVPUB_QT(pers_j)       = INFLATE_NOMVAL(1)*CONTRIB_REVPUB_QT(pers_j)                    !Sum of gross labour income and holiday earnings, earned as wage earner on the public labour market in a given quater
!        CONTRIB_INCSELF_QT(pers_j)      = INFLATE_NOMVAL(1)*CONTRIB_INCSELF_QT(pers_j)                   !Gross income earned as self employed in a given quater
!        CONTRIB_SSPRIV_EMEE_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_SSPRIV_EMEE_QT(pers_j)               !Social security contributions paid on CONTRIB_REVPRIV_QT in a given quater
!        CONTRIB_SSPUB_EMEE_QT(pers_j)   = INFLATE_NOMVAL(1)*CONTRIB_SSPUB_EMEE_QT(pers_j)                !Social security contributions paid on CONTRIB_REVPUB_QT in a given quater
!        CONTRIB_SSSELF_EMER_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_SSSELF_EMER_QT(pers_j)               !Social security contributions paid on CONTRIB_INCSELF_QT in a given quater
        !Variables read from file SICK_UNEM.txt
        UNEM_BENUN_QT(pers_j)           = INFLATE_NOMVAL(3)*UNEM_BENUN_QT(pers_j)                        !Gross amount of real unemployment benefits in a given quarter (no benefits paid by RVA as supplement to wage)
        !Variables read from file SICK_PIT.txt
        !Variables read from file SICK_PENSWELF.txt
!        PENSWELF_AMOUNTP_QT(pers_j)     = INFLATE_NOMVAL(2)*PENSWELF_AMOUNTP_QT(pers_j)                  !Gross amount of pensions received in a given quater
        !Variables read from file SICK_FAMAL.txt
!        FAMAL_AMOUNTF_QT(pers_j)        = INFLATE_NOMVAL(3)*FAMAL_AMOUNTF_QT(pers_j)                     !Gross amount of family allowances received in a given quater by the recipient
        !Variables read from file SICK_SICK.txt
        SICK_TOTBENPR_QT(pers_j)        = INFLATE_NOMVAL(3)*SICK_TOTBENPR_QT(pers_j)                     !Gross amount of sickness and disability benefits estimated before sickness and disability benefits are computed
        !Variables read from file MIMOSIS_FAMREL.txt                                           
        !Variables read from file WAGE_CONSTRUCT.txt                                           
        SALJRS_LAST_CORR(pers_j)        = INFLATE_NOMVAL(1)*SALJRS_LAST_CORR(pers_j)                     !Estimate of last observed income per day
        MIMOSIS_GRINC_HOUR(pers_j)      = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_HOUR(pers_j)                   !Estimate of last observed income per hour (Based on CIMIRe, DWH and minimum-wage)
        MIMOSIS_GRINC_QT(pers_j)        = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_QT(pers_j)                     !Estimate of last observed income per quarter (Based on CIMIRe, DWH and minimum-wage)
        !Variables read from file MIMOSIS_WEIGHT.txt
    END DO   

END SUBROUTINE SICK_INFLATE

!==============================================!
!HOUSEHOLD_SIZE computes household size        !
!and identifies starting and ending record of  !
!sociological household and stores it in a     !
!vector with household information             !
!Constructs variables                          !
!   SICK_HH_SIZE                               !
!   SICK_START_HH                              !
!   SICK_END_HH                                !
!   N_HH_SICK                                  !
!==============================================!

SUBROUTINE SICK_HOUSEHOLD_SIZE

    INTEGER::  I
    INTEGER::  J
    INTEGER::  K
    INTEGER::  N

    I=1
    J=1
    K=0
    N=0

    DO
        J=J+N
        N=0
        K=K+1
        DO WHILE (NAREGNIS_HH_ID(i)==NAREGNIS_HH_ID(j))
            N=N+1
            I=I+1
            IF (i>n_rec) EXIT    
        END DO
        SICK_HH_SIZE(k)=n
        SICK_START_HH(k)=j
        SICK_END_HH(k)=j+n-1
        IF (i>n_rec.OR.j>n_rec)EXIT
    END DO
    N_HH_SICK=K

END SUBROUTINE SICK_HOUSEHOLD_SIZE

!=================================================================
!Computes the age of each individual in terms of years and months!
!=================================================================

!============================================!
! Subroutine main input : NAREGNIS_DATBIRTH  ! 
! Subroutine main output: SICK_YEAR_AGE      !
!                         SICK_MONTH_AGE     !
!                         SICK_YEAR_BIRTH    !
!                         SICK_MONTH_BIRTH   !   
!============================================!

SUBROUTINE SICK_AGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        SICK_PAR_BREAK_POINT(1)=INDEX(NAREGNIS_DATBIRTH(pers_j),'-')
        READ (NAREGNIS_DATBIRTH(pers_j)(1:SICK_PAR_BREAK_POINT(1)-1),  '(I10)') SICK_MONTH_BIRTH(pers_j)
        READ (NAREGNIS_DATBIRTH(pers_j)(SICK_PAR_BREAK_POINT(1)+1:),   '(I10)') SICK_YEAR_BIRTH(pers_j)          
        !determine age as number of years at the end of the year for which data are registered
        SICK_YEAR_AGE(pers_j)  =MIMOSIS_YEAR-SICK_YEAR_BIRTH(pers_j)
        !determine number of months as year age plus number of months passed until the end of the year
        SICK_MONTH_AGE(pers_j) =SICK_YEAR_AGE(pers_j)*12+(12-SICK_MONTH_BIRTH(pers_j))
    END DO

END SUBROUTINE SICK_AGE

!==========================================!
!Draw a line of a give number of characters
!==========================================!

SUBROUTINE DRAWLINE(NUNIT,N,WRITECHAR)

    ! DRAWS A LINE OF N CHARACTERS OF TYPE WRITECHAR ON UNIT NUNIT
    INTEGER                                                 :: N                                    !Number of characters to be drawn
    INTEGER                                                 :: NUNIT                                    !Number of characters to be drawn
    CHARACTER(len=1)                                        :: WRITECHAR                            !Type of character that has to be reproduced N times
    CHARACTER(len=1),DIMENSION(N)                           :: A                                    !Vector of dimension N containing WRITECHAR in the N positions
    CHARACTER(len=15)                                       :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=7)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK

    A(1:N)=WRITECHAR
    !convert the integer value N into a character value NCHAR with the Fortran internal Write function                                                        
    WRITE (NCHAR, '(I7)') N
    !Construct the output format by aid of the internal Fortran function NONBLK which identifies the first non blank character
    !and read the remainder of the string, i.e. :
    OUTPUTFMT = '(' // NCHAR(NONBLK(NCHAR):) // 'A1)'
    WRITE(NUNIT,OUTPUTFMT)(A(I),I=1,N)

END SUBROUTINE DRAWLINE

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as a result of different rates applied on an input amount.             !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The rate is in column 3 of this matrix.              !
!An example of this procedure is the social security contribution of    ! 
!self employed in principal activity before retirement age              !
!In this module to be applied with parameters                           !
!   SICK_COSTSCAL_ANN                                                   !
!   SICK_TAXRATES_ANN                                                   !
!   SICK_LABINCRA_DAY                                                   !
!=======================================================================!

FUNCTION SICK_BRACRATE_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,3)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !apply the rate to the difference between income and lower limit (and add it up to the deductions computed in the previous brackets)
            OUTPUT_AMOUNT=OUTPUT_AMOUNT+(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        !otherwise the amount is higher than the upper bracket
        ELSE IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                                                                     
            !apply rate to the difference between upper and lower limit (and add it up to the amount computed with previous brackets) 
            OUTPUT_AMOUNT=OUTPUT_AMOUNT+(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)
            FUNCSELE = 1
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=OUTPUT_AMOUNT+(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)                                                       
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function SICK_BRACRATE_3COLUMNS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION SICK_BRACRATE_3COLUMNS

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as the sum of different amounts conditional on the value of a status   !
!variable                                                               !
!The amounts are stored in the vector MAT_AMOUNT                        !
!In this module to be applied with parameters                           !
!   SICK_REDCHILD_ANN                                                   !
!=======================================================================!

FUNCTION SICK_SUMAMOUNT_1COLUMN(STATUS1,NLINES_AMOUNT,MAT_AMOUNT,INDIV) RESULT (OUTPUT_AMOUNT)

    INTEGER                                             :: STATUS1                                  !Status variable used to compute the sum of nominal amounts 
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_AMOUNT                            !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_AMOUNT)      :: MAT_AMOUNT                               !Matrix with status variable and nominal amounts
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first value to the second last value
    DO i=1,NLINES_AMOUNT-1                                                                          
        !if the status is larger than the value in column 1 add the amount
        IF (STATUS1.GE.i) THEN
            OUTPUT_AMOUNT = OUTPUT_AMOUNT + MAT_AMOUNT(i)
            FUNCSELE = 1
        END IF
        !if no selection is necessary still keep track of running procedure
        IF (STATUS1.EQ.0) THEN
            FUNCSELE = 1
        END IF
    END DO 
    !in case of the last line a special manipulation is required
    IF (STATUS1.GE.NLINES_AMOUNT) THEN
        OUTPUT_AMOUNT = OUTPUT_AMOUNT+(STATUS1 + 1 - NLINES_AMOUNT)*MAT_AMOUNT(NLINES_AMOUNT)
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function SICK_SUMAMOUNT_1COLUMN has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION SICK_SUMAMOUNT_1COLUMN

!========================================================================!
!Define a function that computes the prepayments                         !
!We assume the the INPUT_AMOUNT is gross taxable income per year on which!
!prepayments have to be paid                                             !
!========================================================================!

FUNCTION SICK_PREPAY_COMPUTE(INPUT_AMOUNT,CHILDREN,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                 !Amount on which the rates will be applied
    INTEGER                                             :: CHILDREN                     !Number of children in charge
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                !Amount that is produced as output 
    REAL(KIND=KIND_REAL)                                :: SICK_NTINC_ANN               !Net taxable income per year
    REAL(KIND=KIND_REAL)                                :: SICK_PREPAY_BASE_ANN         !Basic amount of prepayments paid per year
    REAL(KIND=KIND_REAL)                                :: SICK_AMOUNT_REDCHILD_ANN     !Annual amount of income, exempted from the bottom up, for dependent children
    REAL(KIND=KIND_REAL)                                :: SICK_PREPAY_REDCHILD_ANN     !Annual tax reduction because of dependent children
    REAL(KIND=KIND_REAL)                                :: SICK_PREPAY_ANN              !Annual amount of prepayments
    REAL(KIND=KIND_REAL)                                :: SICK_PREPAY_DAY              !daily amount of prepayments
    INTEGER                                             :: INDIV                        !Individual identification number for which function is selected

    !initialize computation variables
    OUTPUT_AMOUNT = 0              
    SICK_NTINC_ANN = 0             
    SICK_PREPAY_BASE_ANN = 0       
    SICK_AMOUNT_REDCHILD_ANN = 0   
    SICK_PREPAY_REDCHILD_ANN = 0   
    SICK_PREPAY_ANN = 0             
    SICK_PREPAY_DAY = 0             
    
    !Determine net taxable wages per year
    SICK_NTINC_ANN = INPUT_AMOUNT - SICK_BRACRATE_3COLUMNS(INPUT_AMOUNT,SIZE(SICK_COSTSCAL_ANN,1),SICK_COSTSCAL_ANN,INDIV)
    !Determine the basic amount of prepayments before deductions are applied    
    SICK_PREPAY_BASE_ANN = SICK_BRACRATE_3COLUMNS(SICK_NTINC_ANN,SIZE(SICK_TAXRATES_ANN,1),SICK_TAXRATES_ANN,INDIV)
    !Determine the amount of income that is exempted from the bottom up for dependent children
    SICK_AMOUNT_REDCHILD_ANN = SICK_SUMAMOUNT_1COLUMN(CHILDREN,SIZE(SICK_REDCHILD_ANN),SICK_REDCHILD_ANN,INDIV)
    !Determine the tax reduction because of dependent children
    SICK_PREPAY_REDCHILD_ANN = SICK_BRACRATE_3COLUMNS(SICK_AMOUNT_REDCHILD_ANN,SIZE(SICK_TAXRATES_ANN,1),SICK_TAXRATES_ANN,INDIV)
    !Compute the final amount of prepayments
    SICK_PREPAY_ANN = SICK_PREPAY_BASE_ANN - SICK_PREPAY_REDCHILD_ANN
    IF (SICK_PREPAY_ANN.LT.0) THEN
        SICK_PREPAY_ANN = 0
    END IF
    SICK_PREPAY_DAY = SICK_PREPAY_ANN/260.D0
    
    OUTPUT_AMOUNT = SICK_PREPAY_DAY
    
END FUNCTION SICK_PREPAY_COMPUTE 

!=========================================================================!
!With the subroutine SSDATE we compute the value of the first and the     !
!last of the quarter under analysis                                       !
!This subroutine uses the predefined parameters                           !
!   month                                                                 !
!   MIMOSIS_YEAR as input                                                 !
!for reference of the basic conversion rule see the definition of DAY_FUNC!       
!Constructed variables                                                    !
!   START_DAYQT                                                           !
!   STOP_DAYQT                                                            !
!=========================================================================!

SUBROUTINE SICK_SSDATE

    INTEGER      :: DD, MM, YYYY                                !Variables to store parts of the date 

    !Initialise variables
    DD = 0
    MM = 0
    YYYY = 0
    START_DAYQT = 0
    STOP_DAYQT = 0
    
    !Compute the first day of the quarter that is under analysis in the module
    IF (ANY(month.EQ.(/1, 2, 3/))) THEN
       DD = 1
       MM = 1
    ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
       DD = 1
       MM = 4
    ELSE IF (ANY(month.EQ.(/7, 8, 9/))) THEN
       DD = 1
       MM = 7
    ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
       DD = 1
       MM = 10
    END IF
    YYYY = MIMOSIS_YEAR
    START_DAYQT = DD - 32075 + 1461*(YYYY+4800+(MM-14)/12)/4 + 367*(MM-2-(MM-14)/12*12)/12 - 3*((YYYY+4900+(MM-14)/12)/100)/4

    !Compute the last day of the quarter that is under analysis in the module
    IF (ANY(month.EQ.(/1, 2, 3/))) THEN
       DD = 31
       MM = 3
    ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
       DD = 30
       MM = 6
    ELSE IF (ANY(month.EQ.(/7, 8, 9/))) THEN
       DD = 30
       MM = 9
    ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
       DD = 31
       MM = 12
    END IF
    YYYY = MIMOSIS_YEAR
    STOP_DAYQT = DD - 32075 + 1461*(YYYY+4800+(MM-14)/12)/4 + 367*(MM-2-(MM-14)/12*12)/12 - 3*((YYYY+4900+(MM-14)/12)/100)/4

END SUBROUTINE SICK_SSDATE

!========================================================================!
!Define a function that converts a day into a unique integer value       !
!given a user defined format                                             !
!Function goes back to an example in                                     !
!http://groups.google.com/group/comp.lang.pl1/browse_frm/thread/9a41f0d192b11933/7ab15b6f9c62c8cf?lnk=st&q=&rnum=2#7ab15b6f9c62c8cf
!No simple uppercase function found                                      !
!========================================================================!

FUNCTION DAY_FUNC(DATE, DATE_KIND, DATE_DEF) RESULT (OUTPUT_DAY)

    CHARACTER(*) :: DATE                                            !A date expressed in the format covered by the varialbe DATE_KIND
    CHARACTER(*) :: DATE_KIND                                       !Format of the character variable in DATE
    INTEGER      :: DD, MM, YYYY, YY                                !Variables to store parts of the date 
    INTEGER      :: OUTPUT_DAY, REGIST_DAY                          !Variables to store converted date values
    INTEGER      :: DATE_DEF                                        !Default date, to be used if it is impossible the date value from the observation

    !Initialise variables
    DD = 0
    MM = 0
    YYYY = 0
    YY = 0
    OUTPUT_DAY = 0
    REGIST_DAY = 0 
    
    !Compute the day that corresponds with the day in the variable DATE
    IF (DATE(1:1).NE.'.'.AND.DATE(1:1).NE.' ') THEN
        !Read day, month and year values
        IF (DATE_KIND.EQ.'DDMMYYYY') THEN
            READ (DATE,'(2i2,i4)') DD, MM, YYYY
        ELSE IF (DATE_KIND.EQ.'DDMMMYY') THEN
            READ (DATE,'(i2,3x,i2)') DD, YY
            YYYY = 2000 + YY
            !Convert month
            IF (DATE(3:5).EQ.'JAN') THEN
                MM = 1
            ELSE IF (DATE(3:5).EQ.'FEB') THEN
                MM = 2
            ELSE IF (DATE(3:5).EQ.'MAR') THEN
                MM = 3
            ELSE IF (DATE(3:5).EQ.'APR') THEN
                MM = 4
            ELSE IF (DATE(3:5).EQ.'MAY') THEN
                MM = 5
            ELSE IF (DATE(3:5).EQ.'JUN') THEN
                MM = 6
            ELSE IF (DATE(3:5).EQ.'JUL') THEN
                MM = 7
            ELSE IF (DATE(3:5).EQ.'AUG') THEN
                MM = 8
            ELSE IF (DATE(3:5).EQ.'SEP') THEN
                MM = 9 
            ELSE IF (DATE(3:5).EQ.'OCT') THEN
                MM = 10
            ELSE IF (DATE(3:5).EQ.'NOV') THEN
                MM = 11
            ELSE IF (DATE(3:5).EQ.'DEC') THEN
                MM = 12
            END IF            
        ELSE IF (DATE_KIND.EQ.'YYYY/MM') THEN
            READ (DATE,'(i4,1x,i2)') YYYY, MM
            DD = 1
        ELSE IF (DATE_KIND.EQ.'MM-YYYY') THEN
            READ (DATE,'(i2,1x,i4)') MM, YYYY
            DD = 1
        ELSE IF (DATE_KIND.EQ.'DD/MM/YYYY') THEN
            READ (DATE,'(i2,1x,i2,1x,i4)') DD, MM, YYYY
        ELSE
            WRITE(*,*) DATE_KIND, ' format used for date is unrecognised'
        END IF
        !Compute the registered day
        REGIST_DAY = DD - 32075 + 1461*(YYYY+4800+(MM-14)/12)/4 + 367*(MM-2-(MM-14)/12*12)/12 - 3*((YYYY+4900+(MM-14)/12)/100)/4
    ELSE
        REGIST_DAY = DATE_DEF
    END IF

    OUTPUT_DAY = REGIST_DAY 

    !Remark:    not correct for value 9999 since this is covered by STOP_DAY correction
    !           if date unknown i.e. . of blank the output date is equal to 0
    !           The number of days per quarter as a comparison between two computed results, is limited with
    !           the function DAYS_COMPUTE
    
END FUNCTION DAY_FUNC

!========================================================================!
!Define a function that converts a span of days into a derived number of !
!days within this time span                                              !
!========================================================================!

FUNCTION DAYS_COMPUTE(START_DAY, STOP_DAY, DIVIDE, MULTIPLY, MAX_DAY) RESULT (OUTPUT_DAYS)

    INTEGER      :: START_DAY, STOP_DAY     !Variables in which date values are stores
    INTEGER      :: DIVIDE, MULTIPLY        !Variable values used to divide and multiply the time span
    INTEGER      :: MAX_DAY                 !If necessary limit the computed number of days to a maximum
    INTEGER      :: OUTPUT_DAYS             !Number of days within the time span that is analysed 

    !initialise output
    OUTPUT_DAYS = 0
    !compute result conditional on comparison with the predefined values of 
    !starting and ending date of the quarter that is being analysed
    IF (START_DAY.LT.START_DAYQT.AND.STOP_DAY.LT.START_DAYQT) THEN
        OUTPUT_DAYS = 0
    ELSE IF (START_DAY.GT.STOP_DAYQT.AND.STOP_DAY.GT.STOP_DAYQT) THEN
        OUTPUT_DAYS = 0
    ELSE IF (START_DAY.LT.START_DAYQT.AND.STOP_DAY.LE.STOP_DAYQT) THEN
        IF (DIVIDE.NE.0) THEN
            OUTPUT_DAYS = CEILING(((DBLE(STOP_DAY) - DBLE(START_DAYQT)+1.D0)/DBLE(DIVIDE))*DBLE(MULTIPLY))
        END IF    
        IF (OUTPUT_DAYS.GT.MAX_DAY) THEN
            OUTPUT_DAYS = MAX_DAY
        END IF
    ELSE IF (START_DAY.GE.START_DAYQT.AND.STOP_DAY.GT.STOP_DAYQT) THEN
        IF (DIVIDE.NE.0) THEN
            OUTPUT_DAYS = CEILING(((DBLE(STOP_DAYQT) - DBLE(START_DAY)+1.D0)/DBLE(DIVIDE))*DBLE(MULTIPLY))
        END IF    
        IF (OUTPUT_DAYS.GT.MAX_DAY) THEN
            OUTPUT_DAYS = MAX_DAY
        END IF
    ELSE IF (START_DAY.GE.START_DAYQT.AND.STOP_DAY.LE.STOP_DAYQT) THEN
        IF (DIVIDE.NE.0) THEN
            OUTPUT_DAYS = CEILING(((DBLE(STOP_DAY) - DBLE(START_DAY)+1.D0)/DBLE(DIVIDE))*DBLE(MULTIPLY))
        END IF    
        IF (OUTPUT_DAYS.GT.MAX_DAY) THEN
            OUTPUT_DAYS = MAX_DAY
        END IF
    END IF
    
END FUNCTION DAYS_COMPUTE

!========================================================================!
!Define a function that converts a span of days into a derived number of !
!weeks within this time span                                             !
!========================================================================!

FUNCTION WEEKS_COMPUTE(START_DAY, STOP_DAY, DIVIDE, MULTIPLY, MIN_WEEK, MAX_WEEK) RESULT (OUTPUT_WEEKS)

    INTEGER      :: START_DAY, STOP_DAY     !Variables in which date values are stores
    INTEGER      :: DIVIDE, MULTIPLY        !Variable values used to divide and multiply the time span
    INTEGER      :: MIN_WEEK                !If necessary limit the computed number of weeks to a lower limit
    INTEGER      :: MAX_WEEK                !If necessary limit the computed number of weeks to a maximum
    INTEGER      :: OUTPUT_WEEKS            !Number of weeks within the time span that is analysed 

    !initialise output
    OUTPUT_WEEKS = 0
    !compute result  
    IF (START_DAY.GT.STOP_DAY) THEN
        OUTPUT_WEEKS = 0
    ELSE IF (START_DAY.LE.STOP_DAY) THEN
        IF (DIVIDE.NE.0) THEN
            OUTPUT_WEEKS = CEILING(((DBLE(STOP_DAY) - DBLE(START_DAY)+1.D0)/DBLE(DIVIDE))*DBLE(MULTIPLY))
        END IF    
        IF (OUTPUT_WEEKS.LT.MIN_WEEK) THEN
            OUTPUT_WEEKS = MIN_WEEK
        END IF
        IF (OUTPUT_WEEKS.GT.MAX_WEEK) THEN
            OUTPUT_WEEKS = MAX_WEEK
        END IF
    END IF
    
END FUNCTION WEEKS_COMPUTE

!==========================================================================================!
!With subroutine SICK_CHARAC we reconstruct variables                                      !
!used troughout the different subroutines for constructing the benefits                    !
!Constructed variables                                                                     !
!   SICK_RISKGLOB_QT                                                                       !
!   SICK_TYPEINSU_PRIV_QT                                                                  !
!   SICK_TYPEINSU_PUB_QT                                                                   !
!   SICK_TYPEINSU_SELF_QT                                                                  !
!Variables needed for sick or disabiled insured as wage earner on the private labour market!
!   Sickness and disability on private labour market                                       !
!   SICK_ELIGPRI1_QT                                                                       !
!   SICK_WOWEPRIV_QT                                                                       !
!   SICK_DAYSPRI1_QT                                                                       !
!   SICK_WEEKPRI1_QT                                                                       !
!   SICK_SOLEPRIV_QT                                                                       !
!   SICK_REGWPRIV_QT                                                                       !
!   SICK_LDAYWAGE_DAY                                                                      !
!   SICK_ELTHPRIV_QT                                                                       !
!   Maternity benefits private                                                             !
!   SICK_ELIGPRI2_QT                                                                       !
!   SICK_DAYSPRI2_QT                                                                       !
!   SICK_MATLPRI2_QT                                                                       !
!   Industrial accidents private                                                           !
!   SICK_ELIGPRI3_QT                                                                       !
!   SICK_TYDBPRI3_QT                                                                       !
!   SICK_DAYSPRI3_QT                                                                       !
!   SICK_WEEKPRI3_QT                                                                       !
!   SICK_DEGRPRI3_QT                                                                       !
!   Occupational diseases private                                                          !
!   SICK_ELIGPRI4_QT                                                                       !
!   SICK_TYDBPRI4_QT                                                                       !
!   SICK_DAYSPRI4_QT                                                                       !
!   SICK_WEEKPRI4_QT                                                                       !
!   SICK_DEGRPRI4_QT                                                                       !
!Variables needed for sick or disabiled insured as wage earner on the public labour market !
!   Sickness and disability public                                                         !
!   SICK_ELIGPUB1_QT                                                                       !
!   SICK_WOWEPUB_QT                                                                        !
!   SICK_DAYSPUB1_QT                                                                       !
!   SICK_WEEKPUB1_QT                                                                       !
!   SICK_SOLEPUB_QT                                                                        !
!   SICK_REGWPUB_QT                                                                        !
!   SICK_ELTHPUB_QT                                                                        !
!   Maternity benefits public                                                              !
!   SICK_ELIGPUB2_QT                                                                       !
!   SICK_DAYSPUB2_QT                                                                       !
!   SICK_MATLPUB2_QT                                                                       !
!   Industrial accidents public                                                            !
!   SICK_ELIGPUB3_QT                                                                       !
!   SICK_TYDBPUB3_QT                                                                       !
!   SICK_DAYSPUB3_QT                                                                       !
!   SICK_WEEKPUB3_QT                                                                       !
!   SICK_DEGRPUB3_QT                                                                       !
!   Occupational diseases public                                                           !
!   SICK_ELIGPUB4_QT                                                                       !
!   SICK_TYDBPUB4_QT                                                                       !
!   SICK_DAYSPUB4_QT                                                                       !
!   SICK_WEEKPUB4_QT                                                                       !
!   SICK_DEGRPUB4_QT                                                                       !
!Variables needed for sick or disabiled insured as self employed                           !
!   Sickness and disability self employed                                                  !
!   SICK_ELIGSEL1_QT                                                                       !
!   SICK_WEEKSEL1_QT                                                                       !
!   SICK_DAYSSEL1_QT                                                                       !
!   SICK_CLOSESEL_QT                                                                       !
!   SICK_ELTHSELF_QT                                                                       !
!   Maternity benefits self employed                                                       !
!   SICK_ELIGSEL2_QT                                                                       !
!   SICK_WEEKSEL2_QT                                                                       !
!==========================================================================================!

SUBROUTINE SICK_CHARAC

    INTEGER                 :: pers_j
    REAL(KIND=KIND_REAL)    :: WEEK_DIV                             !Number of weeks obtained by dividing the number of days per quarter by the number of days worked per week
    INTEGER                 :: FBZ_PERCENT                          !Read numeric value from character variable that contains percentage
    INTEGER                 :: DAY_START                            !Value of the day on which registration starts (either a real date in the quarter or the first day of the quarter)
    INTEGER                 :: DAY_STOP                             !Value of the day on which registration stops (either a real date in the quarter or the last day of the quarter)
    REAL(KIND=KIND_REAL)    :: SUM_TEST

    !SICK_RISKGLOB_QT: an indicator that identifies whether the individual is sick or disabled
    DO pers_j=1,n_rec
        IF  (InNIC(pers_j).EQ.1.OR.&
        &   (RIZIVINA_STELSEL_INVAL_2008(pers_j)(1:1).EQ.'1'.OR.RIZIVINA_STELSEL_INVAL_2008(pers_j)(1:1).EQ.'2'.OR.&
        &    RIZIVINA_STELSEL_MAT_2008(pers_j)(1:1).EQ.'3'.OR.RIZIVINA_STELSEL_MAT_2008(pers_j)(1:1).EQ.'4').OR.&
        &    FATFAO_BENEF_2008(pers_j).EQ.1.OR.&
        &    FBZFMB_BENEF_2008(pers_j).EQ.1) THEN
                SICK_RISKGLOB_QT(pers_j) = 1
        END IF
    END DO

    !SICK_TYPEINSU_PRIV_QT: Type of insurance of sick or disabled insured as wage earner on the private labour market
    !1 Individual is insured as a blue collar worker
    !2 Individual is insured as a white collar worker
    DO pers_j=1,n_rec
        IF (SICK_RISKGLOB_QT(pers_j).EQ.1) THEN
            !If no status info from sickness or disability source
            !On the basis of the labour market status of the sick or disabled
            IF (InNIC(pers_j).EQ.0.AND.InRIZIV(pers_j).EQ.0) THEN
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    SICK_TYPEINSU_PRIV_QT(pers_j) = 1
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    SICK_TYPEINSU_PRIV_QT(pers_j) = 2
                END IF
            END IF
!Aangepast september 2011
            !On the basis of NIC info
            !assume blue collar worker
            IF (NIC_STELSEL_2008(pers_j).EQ.'1'.AND.SICK_TYPEINSU_PRIV_QT(pers_j).EQ.0) THEN
                !First use current stattus if available
                IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.1) THEN
                    SICK_TYPEINSU_PRIV_QT(pers_j) = 1
                ELSE IF (CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.2) THEN
                    SICK_TYPEINSU_PRIV_QT(pers_j) = 2
                END IF
                !If status not identified yet, use information from the past and if not available
                !assume blue collar worker
                !Most likely status in case of primary disablement
                IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.0) THEN
                    IF (LABMSTAT_PRIV(pers_j).EQ.1) THEN
                        SICK_TYPEINSU_PRIV_QT(pers_j) = 1
                    ELSE IF (LABMSTAT_PRIV(pers_j).EQ.2) THEN
                        SICK_TYPEINSU_PRIV_QT(pers_j) = 2
                    ELSE
                        SICK_TYPEINSU_PRIV_QT(pers_j) = 1
                    END IF
                END IF
            END IF            
            !Use extra RIZIV information on the family charge and the position of the sick or disabled
            IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.0) THEN
                IF (&
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'1111'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'1112'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'1121'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'1122'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'1311'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'1312'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'1321'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'1322'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'8111'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'8112'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'8121'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'8122'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'8311'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'8312'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'8321'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'8322'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'9111'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'9112'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'9121'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'9122'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'9311'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'9312'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'9321'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'9322') THEN
                        SICK_TYPEINSU_PRIV_QT(pers_j) = 1
                ELSE IF (&
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'1211'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'1212'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'1221'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'1222'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'8211'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'8212'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'8221'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'8222'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'9211'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'9212'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'9221'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'9222') THEN
                        SICK_TYPEINSU_PRIV_QT(pers_j) = 2
                END IF
            END IF
!Later eventueel vervangen door CATPROFVICT_2008
!            !Use extra FAO information the sector and professional acitivity of the insured            IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.0) THEN
!            IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.0) THEN
!                IF (FATFAO_CSECACC_ANN(pers_j)(1:1).EQ.'R') THEN
!                    IF (&
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'1'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'A'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'H') THEN
!                            SICK_TYPEINSU_PRIV_QT(pers_j) = 1
!                    ELSE IF (&
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'2'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'3'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'4'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'B'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'C'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'D'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'E'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'F'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'G'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'I'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'J'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'L'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'M'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'N') THEN
!                            SICK_TYPEINSU_PRIV_QT(pers_j) = 2
!                    END IF
!                END IF
!            END IF
        END IF
    END DO
    
    !SICK_TYPEINSU_PUB_QT: Type of insurance of sick or disabled insured as wage earner on the public labour market
    !1 Individual is insured as a blue collar worker without statutory service
    !2 Individual is insured as a white collar worker without statutory service
    !3 Individual is insured as a blue collar worker with statutory service
    !4 Individual is insured as a white collar worker with statutory service
    DO pers_j=1,n_rec
        IF (SICK_RISKGLOB_QT(pers_j).EQ.1) THEN
            !On the basis of the labour market status of the sick or disabled
            IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.1) THEN
                SICK_TYPEINSU_PUB_QT(pers_j) = 1
            ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.2) THEN
                SICK_TYPEINSU_PUB_QT(pers_j) = 2
            ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.3) THEN
                SICK_TYPEINSU_PUB_QT(pers_j) = 3
            ELSE IF (CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.4) THEN
                SICK_TYPEINSU_PUB_QT(pers_j) = 4
            END IF
!Later eventueel vervangen door CATPROFVICT_2008
!            !Use extra FAO information the sector and professional acitivity of the insured
!            IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.0) THEN
!                IF (FATFAO_CSECACC_ANN(pers_j)(1:1).EQ.'U') THEN
!                    IF (&
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'1'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'A'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'H') THEN
!                            SICK_TYPEINSU_PUB_QT(pers_j) = 1
!                    ELSE IF (&
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'2'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'3'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'4'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'B'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'C'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'D'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'E'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'F'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'G'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'I'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'J'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'L'.OR.& 
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'M'.OR.FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'N') THEN
!                            SICK_TYPEINSU_PUB_QT(pers_j) = 2
!                    ELSE IF (&
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.' ') THEN
!                            SICK_TYPEINSU_PUB_QT(pers_j) = 3
!                    ELSE IF (&
!                        &FATFAO_BERCAT_ANN(pers_j)(1:1).EQ.'K') THEN
!                            SICK_TYPEINSU_PUB_QT(pers_j) = 4
!                    END IF
!                END IF
!            END IF
        END IF
    END DO

    !SICK_TYPEINSU_SELF_QT: Type of insurance of sick or disabled insured as self employed
    !1 Individual is insured as a self employed
    DO pers_j=1,n_rec
        IF (SICK_RISKGLOB_QT(pers_j).EQ.1) THEN
            !On the basis of the labour market status of the sick or disabled
            IF (InNIC(pers_j).EQ.0.AND.InRIZIV(pers_j).EQ.0) THEN
                IF (CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.1) THEN
                    SICK_TYPEINSU_SELF_QT(pers_j) = 1
                END IF
            END IF
!Aangepast september 2011
            !On the basis of NIC info
            IF (NIC_STELSEL_2008(pers_j).EQ.'2') THEN
                SICK_TYPEINSU_SELF_QT(pers_j) = 1
            END IF            
            !Use extra RIZIV information on the family charge and the position of the sick or disabled
            IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.0) THEN
                IF (&
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'1411'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'1412'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'1421'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'1422'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'1511'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'1512'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'1521'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'1522'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'8411'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'8412'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'8421'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'8422'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'8511'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'8512'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'8521'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'8522'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'9411'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'9412'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'9421'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'9422'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'9511'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'9512'.OR.& 
                    &AANWIJZ_2008(pers_j)(1:4).EQ.'9521'.OR.AANWIJZ_2008(pers_j)(1:4).EQ.'9522') THEN
                        SICK_TYPEINSU_SELF_QT(pers_j) = 1
                END IF
            END IF    
        END IF
    END DO
    
    !Correct multiple identifications of the insurance type and those not identified yet
    DO pers_j=1,n_rec
        IF (SICK_RISKGLOB_QT(pers_j).EQ.1) THEN
!            IF (InNIC(pers_j).EQ.1.AND.SICK_TYPEINSU_PRIV_QT(pers_j).EQ.0.AND.SICK_TYPEINSU_PUB_QT(pers_j).EQ.0.AND.&
!            &SICK_TYPEINSU_PRIV_QT(pers_j).EQ.0) THEN
!               WRITE(15,*) 'TestNic',pers_j,NIC_STELSEL_2008(pers_j),NIC_SRTDAGEN_2008(pers_j)
!            END IF
            !if private labour market worker keep this value
            IF  (SICK_TYPEINSU_PRIV_QT(pers_j).NE.0.AND.&
               &(SICK_TYPEINSU_PUB_QT(pers_j).NE.0.OR.SICK_TYPEINSU_SELF_QT(pers_j).NE.0)) THEN
                    SICK_TYPEINSU_PUB_QT(pers_j) = 0
                    SICK_TYPEINSU_SELF_QT(pers_j) = 0    
            END IF
            !if not on private labour market, keep public labour market
            IF  (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.0.AND.&
               & SICK_TYPEINSU_PUB_QT(pers_j).NE.0.AND.&
               & SICK_TYPEINSU_SELF_QT(pers_j).NE.0) THEN
                    SICK_TYPEINSU_SELF_QT(pers_j) = 0    
            END IF
            !if not identified yet, set equal to white collar worker on the private labour market
            IF  (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.0.AND.&
               & SICK_TYPEINSU_PUB_QT(pers_j).EQ.0.AND.&
               & SICK_TYPEINSU_SELF_QT(pers_j).EQ.0) THEN
                !If status not identified yet, use information from the past and if not available
                !assume white collar worker
                !Most likely status in all cases
                IF (LABMSTAT_PRIV(pers_j).EQ.1) THEN
                    SICK_TYPEINSU_PRIV_QT(pers_j) = 1
                ELSE IF (LABMSTAT_PRIV(pers_j).EQ.2) THEN
                    SICK_TYPEINSU_PRIV_QT(pers_j) = 2
                ELSE
                    SICK_TYPEINSU_PRIV_QT(pers_j) = 2
                END IF
            END IF
        END IF
    END DO        

!Variables needed for sick or disabiled insured as wage earner on the private labour market
    !Sickness and disability on private labour market             

    !SICK_ELIGPRI1_QT: Individual receives a benefit for sickness or disability in the wage earner regime of the private labour market         
    !1 the individual is in the first year of sickness 
    !2 the individual is in disability.
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (InNIC(pers_j).EQ.1.AND.NIC_STELSEL_2008(pers_j).EQ.'1'.AND.NIC_SRTDAGEN_2008(pers_j).EQ.'01') THEN
                SICK_ELIGPRI1_QT(pers_j) = 1
!                WRITE(15,*) 'TestA',InNIC(pers_j),NIC_STELSEL_2008(pers_j),NIC_SRTDAGEN_2008(pers_j)
            END IF
            !if registered as disabled by the RIZIV we consider this registration to be more reliable
            IF (RIZIVINA_STELSEL_INVAL_2008(pers_j)(1:1).EQ.'1') THEN
                IF (AANWIJZ_2008(pers_j)(1:2).EQ.'11'.OR.&
                   &AANWIJZ_2008(pers_j)(1:2).EQ.'12'.OR.&
                   &AANWIJZ_2008(pers_j)(1:2).EQ.'13') THEN
                    SICK_ELIGPRI1_QT(pers_j) = 2
!                    WRITE(15,*) 'TestA',AANWIJZ_2008(pers_j)
                END IF
            END IF
        END IF
    END DO        

    !SICK_WOWEPRIV_QT: Type of working week the sick or disabled is working in         
    !1 the individual is working in a 5 day week regime
    !2 the individual is working in a 6 day week regime
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            SICK_WOWEPRIV_QT(pers_j) = 1    
        END IF
    END DO        

    !SICK_DAYSPRI1_QT: Number of days the individual receives a benefit for sickness or disability in the given quarter (individual is insured as wage earner on the private labour market)
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            !if sick
            IF (SICK_ELIGPRI1_QT(pers_j).EQ.1) THEN
                WEEK_DIV = 1.D0           
                IF (SICK_WOWEPRIV_QT(pers_j).EQ.1) THEN
                    WEEK_DIV = DBLE(NIC_DAGENAO_2008(pers_j))/5.D0
                ELSE IF (SICK_WOWEPRIV_QT(pers_j).EQ.2) THEN
                    WEEK_DIV = DBLE(NIC_DAGENAO_2008(pers_j))/6.D0
                END IF
                SICK_DAYSPRI1_QT(pers_j) = CEILING(WEEK_DIV*6.D0)
                IF (SICK_DAYSPRI1_QT(pers_j).GT.78) THEN
                    SICK_DAYSPRI1_QT(pers_j) = 78
                END IF
            !if disabled                
            ELSE IF (SICK_ELIGPRI1_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for which a benefit was received with DAY_FUNC
                DAY_START = DAY_FUNC(RIZIVINA_AANVANGS_INVAL_2008(pers_j),'MM-YYYY', START_DAYQT)
!We use the last day of the quarter instead of RIZIVINA_INVAL_EINDBETA_QT(pers_j) since the use of this variable performs better
!                DAY_STOP = DAY_FUNC(RIZIVINA_EINDBETA_INVAL_QT(pers_j),'DDMMMYY', STOP_DAYQT)
                DAY_STOP = STOP_DAYQT
                !Compute the number of days for which benefit is received with the function DAYS_COMPUTE
                SICK_DAYSPRI1_QT(pers_j) = DAYS_COMPUTE(DAY_START, DAY_STOP, 7, 6, 78)
            END IF
        END IF
    END DO        

    !SICK_WEEKPRI1_QT: Number of weeks the individual already receives a benefit for sickness or disability (individual is insured as wage earner on the private labour market)            
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            !if sick
            IF (SICK_ELIGPRI1_QT(pers_j).EQ.1) THEN
                SICK_WEEKPRI1_QT(pers_j) = CEILING(DBLE(SICK_DAYSPRI1_QT(pers_j))/6.D0)
            !if disabled                
            ELSE IF (SICK_ELIGPRI1_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for the disability period, use function DAY_FUNC if necessary
                DAY_START = DAY_FUNC(B_ZIEKTE_2008(pers_j),'MM-YYYY', START_DAYQT)
                DAY_STOP = STOP_DAYQT !use last day of the quarter being analysed
                !Compute the number of weeks with the function WEEKS_COMPUTE
                SICK_WEEKPRI1_QT(pers_j) = WEEKS_COMPUTE(DAY_START, DAY_STOP, 7, 1, 53, 9999999)
                !If result is still 0 after this operation, we assume that the individual is already a whole quarter in disability
                IF (SICK_WEEKPRI1_QT(pers_j).EQ.0) THEN
                    SICK_WEEKPRI1_QT(pers_j) = 66
                END IF
            END IF
        END IF
    END DO        

    !SICK_SOLEPRIV_QT: Sick or disabled lost his sole source of income when he became sick or disabled         
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            SICK_SOLEPRIV_QT(pers_j) = 1
        END IF
    END DO        

    !SICK_REGWPRIV_QT: Individual was a regular worker before he became sick or disabled           
    !1 Individual is regular worker
    !2 Individual is an irregular worker
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            SICK_REGWPRIV_QT(pers_j) = 1
        END IF
    END DO        

    !SICK_LDAYWAGE_DAY: Daily wage lost when the individual became sick or disabled         
    DO pers_j=1,n_rec
        SICK_LDAYWAGE_DAY(pers_j) = MIMOSIS_GRINC_HOUR(pers_j)*7.6D0
    END DO        

    !SICK_ELTHPRIV_QT: Individual receives a supplementary benefit for help of a third person and is insured as wage earner on the private labour market           
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (RIZIVINA_BETALING_INVAL_2008(pers_j)(2:2).EQ.'1') THEN
                SICK_ELTHPRIV_QT(pers_j) = 1
            END IF 
        END IF
    END DO        

    !Maternity benefits on private labour market             
    !SICK_ELIGPRI2_QT: Individual receives a maternity benefit and is insured as wage earner on the private labour market          
    !1 the individual is not in disability and entitled to maternity benefits 
    !2 the individual is in disability and entitled to maternity benefits
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (InNIC(pers_j).EQ.1.AND.NIC_STELSEL_2008(pers_j).EQ.'1') THEN
!Merk op: procedure ANY werkt hier niet met karakters terwijl ze in FAMAL by Handamount wel werkt            
                IF (&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'03'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'05'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'06'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'07'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'08'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'09'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'10') THEN
                    SICK_ELIGPRI2_QT(pers_j) = 1
!                    WRITE(15,*) 'TestB',InNIC(pers_j),NIC_STELSEL_2008(pers_j),NIC_SRTDAGEN_2008(pers_j)
                END IF                    
            END IF
            !if registered as disabled by the RIZIV we consider this registration to be more reliable
            IF (RIZIVINA_STELSEL_MAT_2008(pers_j)(1:1).EQ.'3') THEN
                IF (AANWIJZ_2008(pers_j)(1:2).EQ.'11'.OR.&
                   &AANWIJZ_2008(pers_j)(1:2).EQ.'12'.OR.&
                   &AANWIJZ_2008(pers_j)(1:2).EQ.'13') THEN
                    SICK_ELIGPRI2_QT(pers_j) = 2
                END IF                    
            END IF
        END IF
    END DO        
   
    !SICK_DAYSPRI2_QT: Number of days the individual receives a maternity benefit in the given quarter (individual is insured as wage earner on the private labour market)         
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            !if sick
            IF (SICK_ELIGPRI2_QT(pers_j).EQ.1) THEN
                SICK_DAYSPRI2_QT(pers_j) = NIC_DAGENAO_2008(pers_j)
                IF (SICK_DAYSPRI2_QT(pers_j).GT.75) THEN
                    SICK_DAYSPRI2_QT(pers_j) = 75
                END IF
            !if disabled                
            ELSE IF (SICK_ELIGPRI2_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for which a benefit was received with DAY_FUNC
                DAY_START = DAY_FUNC(RIZIVINA_AANVANGS_MAT_2008(pers_j),'MM-YYYY', START_DAYQT)
!We use the last day of the quarter instead of RIZIVINA_EINDBETA_MAT_QT(pers_j) since the use of this variable performs better
!                DAY_STOP = DAY_FUNC(RIZIVINA_EINDBETA_MAT_QT(pers_j),'DDMMMYY', STOP_DAYQT)
                DAY_STOP = STOP_DAYQT
                !Compute the number of days for which benefit is received with the function DAYS_COMPUTE
                SICK_DAYSPRI2_QT(pers_j) = DAYS_COMPUTE(DAY_START, DAY_STOP, 1, 1, 75)
            END IF
        END IF
    END DO        

    !SICK_MATLPRI2_QT: Number of days the individual is on maternity leave at the beginning of the quarter (individual is insured as wage earner on the private labour market)         
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            !if sick
            IF (SICK_ELIGPRI2_QT(pers_j).EQ.1) THEN
                !no info to identify number of consumed days
            !if disabled                
            ELSE IF (SICK_ELIGPRI2_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for which a benefit was received with DAY_FUNC
                DAY_START = DAY_FUNC(RIZIVINA_AANVANGS_MAT_2008(pers_j),'MM-YYYY', START_DAYQT)
                DAY_STOP = START_DAYQT
                !Compute the number of days for which the individual already received a benefit for maternity leave
                SICK_MATLPRI2_QT(pers_j) = CEILING(((DBLE(DAY_STOP) - DBLE(DAY_START)+1.D0)/7.D0)*5.D0) 
                IF (SICK_MATLPRI2_QT(pers_j).LT.0) THEN
                    SICK_MATLPRI2_QT(pers_j) = 0
                END IF
                IF (SICK_MATLPRI2_QT(pers_j).GT.75) THEN
                    SICK_MATLPRI2_QT(pers_j) = 75
                END IF
            END IF
        END IF
    END DO        

    !Benefits in case of industrial accidents on private labour market             
    !SICK_ELIGPRI3_QT: Individual receives a benefit for industrial accident and is insured as wage earner on the private labour market            
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (TAUXIT_2008(pers_j).NE.0.OR.TAUXIPRES_2008(pers_j).NE.0) THEN
                SICK_ELIGPRI3_QT(pers_j) = 1
            END IF
        END IF
    END DO        

    !SICK_TYDBPRI3_QT: Type of benefit for an industrial accident the individual, insured as wage earner on the private labour market, is entitled to          
    !1 the employee is in temporary full disablement
    !2 the employee is in temporary partial disablement
    !3 the employee is in permanent disablement
    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI3_QT(pers_j).EQ.1) THEN
            !If percentage of permanent disablement is observed, we consider the indvidual as permanent disabled
            IF (TAUXIPRES_2008(pers_j).NE.0) THEN
                SICK_TYDBPRI3_QT(pers_j) = 3
            END IF
            !If there is an idication of full disablement, consider the individual as temporary full disabled
            IF (SICK_TYDBPRI3_QT(pers_j).EQ.0.AND.MONTITT_2008(pers_j).NE.0) THEN
                SICK_TYDBPRI3_QT(pers_j) = 1
            END IF
            !If there is an idication of partial disablement, consider the individual as temporary partially disabled
            IF (SICK_TYDBPRI3_QT(pers_j).EQ.0.AND.MONTITP_2008(pers_j).NE.0) THEN
                SICK_TYDBPRI3_QT(pers_j) = 2
            END IF
        END IF
    END DO        

    !SICK_DAYSPRI3_QT: Number of days for which the individual receives a benefit for an industrial accident in the given quarter (individual is insured as wage earner on the private labour market)          
    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI3_QT(pers_j).EQ.1) THEN
            !If in permanent disablement
            IF (ANY(SICK_TYDBPRI3_QT(pers_j).EQ.(/3/))) THEN
                !Compute the value of the date of the industrial accident with the function DAY_FUNC
                DAY_START = DAY_FUNC(DDEBUT_2008(pers_j),'MM-YYYY', START_DAYQT)
                DAY_STOP = STOP_DAYQT   !use last day of the quarter being analysed as stop date
                SICK_DAYSPRI3_QT(pers_j) = DAY_STOP - DAY_START
                IF (SICK_DAYSPRI3_QT(pers_j).LT.0) THEN
                    SICK_DAYSPRI3_QT(pers_j) = 0
                END IF
            END IF
            !If in temporary disablement
            IF (ANY(SICK_TYDBPRI3_QT(pers_j).EQ.(/1, 2/))) THEN
                !Compute the value of the date of the industrial accident with the function DAY_FUNC
                DAY_START = DAY_FUNC(DDEBUT_2008(pers_j),'MM-YYYY', START_DAYQT)
                DAY_STOP = STOP_DAYQT   !use last day of the quarter being analysed as stop date
                SICK_DAYSPRI3_QT(pers_j) = DAY_STOP - DAY_START
                IF (SICK_DAYSPRI3_QT(pers_j).LT.0) THEN
                    SICK_DAYSPRI3_QT(pers_j) = 0
                END IF
            END IF
            !correct if maximum number of days is exceeded
            IF (SICK_DAYSPRI3_QT(pers_j).GT.91) THEN
                SICK_DAYSPRI3_QT(pers_j) = 91
            END IF
        END IF
    END DO        

    !SICK_WEEKPRI3_QT: Number of weeks the individual already receives a benefit for an industrial accident (individual is insured as wage earner on the private labour market)            
    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI3_QT(pers_j).EQ.1) THEN
            !Compute the value of the date of the industrial accident with the function DAY_FUNC
            DAY_START = DAY_FUNC(DDEBUT_2008(pers_j),'MM-YYYY', START_DAYQT)
            DAY_STOP = STOP_DAYQT   !use last day of the quarter being analysed as stop date
            !Compute the number of weeks disabled at the end of the quarter with the function WEEKS_COMPUTE
            SICK_WEEKPRI3_QT(pers_j) = WEEKS_COMPUTE(DAY_START, DAY_STOP, 7, 1, 0, 9999999)
            !If result is still 0 after this operation, we assume that the individual was disabled throughout the whole quarter
            IF (SICK_WEEKPRI3_QT(pers_j).EQ.0) THEN
                SICK_WEEKPRI3_QT(pers_j) = 13
            END IF
        END IF
    END DO        

    !SICK_DEGRPRI3_QT: Degree of disability because of industrial accident for individual insured as wage earner on the private labour market          
    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI3_QT(pers_j).EQ.1) THEN
            SICK_DEGRPRI3_QT(pers_j) = DBLE(TAUXIT_2008(pers_j))/100.D0
            IF (SICK_DEGRPRI3_QT(pers_j).EQ.0.D0.AND.TAUXIPRES_2008(pers_j).NE.0) THEN
                SICK_DEGRPRI3_QT(pers_j) = DBLE(TAUXIPRES_2008(pers_j))/100.D0
            END IF
            IF (INT(SICK_DEGRPRI3_QT(pers_j)).GT.1) THEN
                SICK_DEGRPRI3_QT(pers_j) = 1.D0
            END IF
        END IF
    END DO        

    !Benefits in case of occupational diseases on private labour market             
    !SICK_ELIGPRI4_QT: Individual receives a benefit for occupational disease and is insured as wage earner on the private labour market            
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (FBZ_SRTUIT_2008(pers_j)(1:1).EQ.'1'.OR.FBZ_SRTUIT_2008(pers_j)(1:1).EQ.'2') THEN
                SICK_ELIGPRI4_QT(pers_j) = 1
            END IF
        END IF
    END DO        

    !SICK_TYDBPRI4_QT: Type of benefit for an occupational disease the individual, insured as wage earner on the private labour market, is entitled to          
    !1 the employee is in temporary full disablement
    !2 the employee is in temporary partial disablement
    !3 the employee is in permanent disablement
    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI4_QT(pers_j).EQ.1) THEN
            !Read percentage
            READ (FBZ_PERCENT_2008(pers_j)(1:),  '(I10)') FBZ_PERCENT
            !If percentage of permanent disablement is observed, we consider the indvidual as permanent disabled
            IF (FBZ_SRTUIT_2008(pers_j)(1:1).EQ.'2') THEN
                SICK_TYDBPRI4_QT(pers_j) = 3
            END IF
            !If there is an idication of temporary disablement
            IF (FBZ_SRTUIT_2008(pers_j)(1:1).EQ.'1') THEN
                !If the percentage of disablement is 100%, consider the individual as temporary full disabled
                IF (SICK_TYDBPRI4_QT(pers_j).EQ.0.AND.FBZ_PERCENT.GE.100) THEN
                    SICK_TYDBPRI4_QT(pers_j) = 1
                !If type of disablement not identified yet, individual is temporary partial disabled
                ELSE IF (SICK_TYDBPRI4_QT(pers_j).EQ.0.AND.FBZ_PERCENT.LT.100) THEN
                    SICK_TYDBPRI4_QT(pers_j) = 2
                END IF
            END IF
        END IF
    END DO        

    !SICK_DAYSPRI4_QT: Number of days for which the individual receives a benefit for an occupational disease in the given quarter (individual is insured as wage earner on the private labour market)          
    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI4_QT(pers_j).EQ.1) THEN
            !Compute the value of the first day of disability and the foreseen ending day with the function DAY_FUNC
            DAY_START = DAY_FUNC(FBZ_BEGDBET_2008(pers_j),'MM-YYYY', START_DAYQT)
            DAY_STOP = DAY_FUNC(FBZ_ENDDBET_2008(pers_j),'MM-YYYY', STOP_DAYQT)
            !Compute the number of days for which benefit is received with the function DAYS_COMPUTE
            SICK_DAYSPRI4_QT(pers_j) = DAYS_COMPUTE(DAY_START, DAY_STOP, 1, 1, 91)
        END IF
    END DO        

    !SICK_WEEKPRI4_QT: Number of weeks the individual already receives a benefit for an occupational disease (individual is insured as wage earner on the private labour market)            
    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI4_QT(pers_j).EQ.1) THEN
            !Compute the value of the first day of disability with the function DAY_FUNC
            DAY_START = DAY_FUNC(FBZ_BEGDBET_2008(pers_j),'MM-YYYY', START_DAYQT)
            DAY_STOP = STOP_DAYQT   !use last day of the quarter being analysed as stop date
            !Compute the number of weeks disabled at the end of the quarter with the function WEEKS_COMPUTE
            SICK_WEEKPRI4_QT(pers_j) = WEEKS_COMPUTE(DAY_START, DAY_STOP, 7, 1, 0, 9999999)
            !If result is still 0 after this operation, we assume that the individual was disabled throughout the whole quarter
            IF (SICK_WEEKPRI4_QT(pers_j).EQ.0) THEN
                SICK_WEEKPRI4_QT(pers_j) = 13
            END IF
        END IF
    END DO        

    !SICK_DEGRPRI4_QT: Degree of disability because of the occupational disease for individual insured as wage earner on the private labour market          
    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI4_QT(pers_j).EQ.1) THEN
            !Read percentage
            READ (FBZ_PERCENT_2008(pers_j)(1:),  '(I10)') FBZ_PERCENT
            SICK_DEGRPRI4_QT(pers_j) = DBLE(FBZ_PERCENT)/100.D0
            IF (INT(SICK_DEGRPRI4_QT(pers_j)).GT.1) THEN
                SICK_DEGRPRI4_QT(pers_j) = 1.D0
            END IF
        END IF
    END DO        

!Variables needed for sick or disabiled insured as wage earner on the public labour market
!START WITH CONSRUCTION OF VALUES FOR THOSE WITHOUT STATUTORY SERVICE
    !Sickness and disability on public labour market             

    !SICK_ELIGPUB1_QT: Individual receives a benefit for sickness or disability in the wage earner regime of the public labour market         
    !1 the individual is in the first year of sickness 
    !2 the individual is in disability.
    SUM_TEST = 0.D0
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (InNIC(pers_j).EQ.1.AND.NIC_STELSEL_2008(pers_j).EQ.'1'.AND.NIC_SRTDAGEN_2008(pers_j).EQ.'01') THEN
                SICK_ELIGPUB1_QT(pers_j) = 1
            END IF
            !if registered as disabled by the RIZIV we consider this registration to be more reliable
            IF (RIZIVINA_STELSEL_INVAL_2008(pers_j)(1:1).EQ.'1') THEN
                IF (AANWIJZ_2008(pers_j)(1:2).EQ.'11'.OR.&
                   &AANWIJZ_2008(pers_j)(1:2).EQ.'12'.OR.&
                   &AANWIJZ_2008(pers_j)(1:2).EQ.'13') THEN
                    SICK_ELIGPUB1_QT(pers_j) = 2
                END IF                    
            END IF
        END IF
!        IF (InNIC(pers_j).EQ.1.AND.NIC_STELSEL_2008(pers_j).EQ.'2'.AND.NIC_SRTDAGEN_2008(pers_j).EQ.'01') THEN
!            SUM_TEST = SUM_TEST + NIC_BEDRAG_2008(pers_j)*MIMOSIS_WEIGHT(pers_j)
!        END IF
    END DO        
!    WRITE(15,'(F15.0)') SUM_TEST

    !SICK_WOWEPUB_QT: Type of working week the sick or disabled is working in         
    !1 the individual is working in a 5 day week regime
    !2 the individual is working in a 6 day week regime
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            SICK_WOWEPUB_QT(pers_j) = 1    
        END IF
    END DO        

    !SICK_DAYSPUB1_QT: Number of days the individual receives a benefit for sickness or disability in the given quarter (individual is insured as wage earner on the public labour market)
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            !if sick
            IF (SICK_ELIGPUB1_QT(pers_j).EQ.1) THEN
                WEEK_DIV = 1.D0           
                IF (SICK_WOWEPRIV_QT(pers_j).EQ.1) THEN
                    WEEK_DIV = DBLE(NIC_DAGENAO_2008(pers_j))/5.D0
                ELSE IF (SICK_WOWEPRIV_QT(pers_j).EQ.2) THEN
                    WEEK_DIV = DBLE(NIC_DAGENAO_2008(pers_j))/6.D0
                END IF
                SICK_DAYSPUB1_QT(pers_j) = CEILING(WEEK_DIV*6.D0)
                IF (SICK_DAYSPUB1_QT(pers_j).GT.78) THEN
                    SICK_DAYSPUB1_QT(pers_j) = 78
                END IF
            !if disabled                
            ELSE IF (SICK_ELIGPUB1_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for which a benefit was received with DAY_FUNC
                DAY_START = DAY_FUNC(RIZIVINA_AANVANGS_INVAL_2008(pers_j),'MM-YYYY', START_DAYQT)
!We use the last day of the quarter instead of RIZIVINA_EINDBETA_INVAL_QT(pers_j) since the use of this variable performs better
!                DAY_STOP = DAY_FUNC(RIZIVINA_EINDBETA_INVAL_QT(pers_j),'DDMMMYY', STOP_DAYQT)
                DAY_STOP = STOP_DAYQT
                !Compute the number of days for which benefit is received with the function DAYS_COMPUTE
                SICK_DAYSPUB1_QT(pers_j) = DAYS_COMPUTE(DAY_START, DAY_STOP, 7, 6, 78)
            END IF
        END IF
    END DO        

    !SICK_WEEKPUB1_QT: Number of weeks the individual already receives a benefit for sickness or disability (individual is insured as wage earner on the public labour market)            
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            !if sick
            IF (SICK_ELIGPUB1_QT(pers_j).EQ.1) THEN
                SICK_WEEKPUB1_QT(pers_j) = CEILING(DBLE(SICK_DAYSPUB1_QT(pers_j))/6.D0)
            !if disabled                
            ELSE IF (SICK_ELIGPUB1_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for the disability period, use function DAY_FUNC if necessary
                DAY_START = DAY_FUNC(B_ZIEKTE_2008(pers_j),'MM-YYYY', START_DAYQT)
                DAY_STOP = STOP_DAYQT !use last day of the quarter being analysed
                !Compute the number of weeks with the function WEEKS_COMPUTE
                SICK_WEEKPUB1_QT(pers_j) = WEEKS_COMPUTE(DAY_START, DAY_STOP, 7, 1, 53, 9999999)
                !If result is still 0 after this operation, we assume that the individual is already a whole quarter in disability
                IF (SICK_WEEKPUB1_QT(pers_j).EQ.0) THEN
                    SICK_WEEKPUB1_QT(pers_j) = 66
                END IF
            END IF
        END IF
    END DO        

    !SICK_SOLEPUB_QT: Sick or disabled lost his sole source of income when he became sick or disabled         
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            SICK_SOLEPUB_QT(pers_j) = 1
        END IF
    END DO        

    !SICK_REGWPUB_QT: Individual was a regular worker before he became sick or disabled           
    !1 Individual is regular worker
    !2 Individual is an irregular worker
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            SICK_REGWPUB_QT(pers_j) = 1
        END IF
    END DO        

    !SICK_ELTHPUB_QT: Individual receives a supplementary benefit for help of a third person and is insured as wage earner on the public labour market           
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (RIZIVINA_BETALING_INVAL_2008(pers_j)(2:2).EQ.'1') THEN
                SICK_ELTHPUB_QT(pers_j) = 1
            END IF 
        END IF
    END DO        

    !Maternity benefits on public labour market             
    !SICK_ELIGPUB2_QT: Individual receives a maternity benefit and is insured as wage earner on the public labour market          
    !1 the individual is not in disability and entitled to maternity benefits 
    !2 the individual is in disability and entitled to maternity benefits
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (InNIC(pers_j).EQ.1.AND.NIC_STELSEL_2008(pers_j).EQ.'1') THEN
!Merk op: procedure ANY werkt hier niet met karakters terwijl ze in FAMAL by Handamount wel werkt            
                IF (&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'03'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'05'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'06'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'07'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'08'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'09'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'10') THEN
                    SICK_ELIGPUB2_QT(pers_j) = 1
                END IF
            END IF
            !if registered as disabled by the RIZIV we consider this registration to be more reliable
            IF (RIZIVINA_STELSEL_MAT_2008(pers_j)(1:1).EQ.'3') THEN
                IF (AANWIJZ_2008(pers_j)(1:2).EQ.'11'.OR.&
                   &AANWIJZ_2008(pers_j)(1:2).EQ.'12'.OR.&
                   &AANWIJZ_2008(pers_j)(1:2).EQ.'13') THEN
                    SICK_ELIGPUB2_QT(pers_j) = 2
                END IF                    
            END IF
        END IF
    END DO        
   
    !SICK_DAYSPUB2_QT: Number of days the individual receives a maternity benefit in the given quarter (individual is insured as wage earner on the public labour market)         
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            !if sick
            IF (SICK_ELIGPUB2_QT(pers_j).EQ.1) THEN
                SICK_DAYSPUB2_QT(pers_j) = NIC_DAGENAO_2008(pers_j)
                IF (SICK_DAYSPUB2_QT(pers_j).GT.75) THEN
                    SICK_DAYSPUB2_QT(pers_j) = 75
                END IF
            !if disabled                
            ELSE IF (SICK_ELIGPUB2_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for which a benefit was received with DAY_FUNC
                DAY_START = DAY_FUNC(RIZIVINA_AANVANGS_MAT_2008(pers_j),'MM-YYYY', START_DAYQT)
!We use the last day of the quarter instead of RIZIVINA_EINDBETA_MAT_QT(pers_j) since the use of this variable performs better
!                DAY_STOP = DAY_FUNC(RIZIVINA_EINDBETA_MAT_QT(pers_j),'DDMMMYY', STOP_DAYQT)
                DAY_STOP = STOP_DAYQT
                !Compute the number of days for which benefit is received with the function DAYS_COMPUTE
                SICK_DAYSPUB2_QT(pers_j) = DAYS_COMPUTE(DAY_START, DAY_STOP, 1, 1, 75)
            END IF
        END IF
    END DO        

    !SICK_MATLPUB2_QT: Number of days the individual is on maternity leave at the beginning of the quarter (individual is insured as wage earner on the public labour market)         
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            !if sick
            IF (SICK_ELIGPUB2_QT(pers_j).EQ.1) THEN
                !no info to identify number of consumed days
            !if disabled                
            ELSE IF (SICK_ELIGPUB2_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for which a benefit was received with DAY_FUNC
                DAY_START = DAY_FUNC(RIZIVINA_AANVANGS_MAT_2008(pers_j),'MM-YYYY', START_DAYQT)
                DAY_STOP = START_DAYQT
                !Compute the number of days for which the individual already received a benefit for maternity leave
                SICK_MATLPUB2_QT(pers_j) = CEILING(((DBLE(DAY_STOP) - DBLE(DAY_START)+1.D0)/7.D0)*5.D0) 
                IF (SICK_MATLPUB2_QT(pers_j).LT.0) THEN
                    SICK_MATLPUB2_QT(pers_j) = 0
                END IF
                IF (SICK_MATLPUB2_QT(pers_j).GT.75) THEN
                    SICK_MATLPUB2_QT(pers_j) = 75
                END IF
            END IF
        END IF
    END DO        

    !Benefits in case of industrial accidents on public labour market             
    !SICK_ELIGPUB3_QT: Individual receives a benefit for industrial accident and is insured as wage earner on the public labour market            
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (TAUXIT_2008(pers_j).NE.0.OR.TAUXIPRES_2008(pers_j).NE.0) THEN
                SICK_ELIGPUB3_QT(pers_j) = 1
            END IF
        END IF
    END DO        

    !SICK_TYDBPUB3_QT: Type of benefit for an industrial accident the individual, insured as wage earner on the public labour market, is entitled to          
    !1 the employee is in temporary full disablement
    !2 the employee is in temporary partial disablement
    !3 the employee is in permanent disablement
    DO pers_j=1,n_rec
        IF (SICK_ELIGPUB3_QT(pers_j).EQ.1) THEN
            !If percentage of permanent disablement is observed, we consider the indvidual as permanent disabled
            IF (TAUXIPRES_2008(pers_j).NE.0) THEN
                SICK_TYDBPUB3_QT(pers_j) = 3
            END IF
            !If there is an idication of full disablement, consider the individual as temporary full disabled
            IF (SICK_TYDBPUB3_QT(pers_j).EQ.0.AND.MONTITT_2008(pers_j).NE.0) THEN
                SICK_TYDBPUB3_QT(pers_j) = 1
            END IF
            !If type of disablement not identified yet, individual is temporary partial disabled
            IF (SICK_TYDBPUB3_QT(pers_j).EQ.0.AND.MONTITP_2008(pers_j).NE.0) THEN
                SICK_TYDBPUB3_QT(pers_j) = 2
            END IF
        END IF
    END DO        

    !SICK_DAYSPUB3_QT: Number of days for which the individual receives a benefit for an industrial accident in the given quarter (individual is insured as wage earner on the public labour market)          
    DO pers_j=1,n_rec
        IF (SICK_ELIGPUB3_QT(pers_j).EQ.1) THEN
            !If in permanent disablement
            IF (ANY(SICK_TYDBPUB3_QT(pers_j).EQ.(/3/))) THEN
                !Compute the value of the date of the industrial accident with the function DAY_FUNC
                DAY_START = DAY_FUNC(DDEBUT_2008(pers_j),'MM-YYYY', START_DAYQT)
                DAY_STOP = STOP_DAYQT   !use last day of the quarter being analysed as stop date
                SICK_DAYSPUB3_QT(pers_j) = DAY_STOP - DAY_START
                IF (SICK_DAYSPUB3_QT(pers_j).LT.0) THEN
                    SICK_DAYSPUB3_QT(pers_j) = 0
                END IF
            END IF
            !If in temporary disablement
            IF (ANY(SICK_TYDBPUB3_QT(pers_j).EQ.(/1, 2/))) THEN
                !Compute the value of the date of the industrial accident with the function DAY_FUNC
                DAY_START = DAY_FUNC(DDEBUT_2008(pers_j),'MM-YYYY', START_DAYQT)
                DAY_STOP = STOP_DAYQT   !use last day of the quarter being analysed as stop date
                SICK_DAYSPUB3_QT(pers_j) = DAY_STOP - DAY_START
                IF (SICK_DAYSPUB3_QT(pers_j).LT.0) THEN
                    SICK_DAYSPUB3_QT(pers_j) = 0
                END IF
            END IF
            !correct if maximum number of days is exceeded
            IF (SICK_DAYSPUB3_QT(pers_j).GT.91) THEN
                SICK_DAYSPUB3_QT(pers_j) = 91
            END IF
        END IF
    END DO        

    !SICK_WEEKPUB3_QT: Number of weeks the individual already receives a benefit for an industrial accident (individual is insured as wage earner on the public labour market)            
    DO pers_j=1,n_rec
        IF (SICK_ELIGPUB3_QT(pers_j).EQ.1) THEN
            !Compute the value of the date of the industrial accident with the function DAY_FUNC
            DAY_START = DAY_FUNC(DDEBUT_2008(pers_j),'MM-YYYY', START_DAYQT)
            DAY_STOP = STOP_DAYQT   !use last day of the quarter being analysed as stop date
            !Compute the number of weeks disabled at the end of the quarter with the function WEEKS_COMPUTE
            SICK_WEEKPUB3_QT(pers_j) = WEEKS_COMPUTE(DAY_START, DAY_STOP, 7, 1, 0, 9999999)
            !If result is still 0 or lager than 13 after this operation, we assume that the individual was disabled throughout the whole quarter
            IF (SICK_WEEKPUB3_QT(pers_j).EQ.0) THEN
                SICK_WEEKPUB3_QT(pers_j) = 13
            END IF
        END IF
    END DO        

    !SICK_DEGRPUB3_QT: Degree of disability because of industrial accident for individual insured as wage earner on the public labour market          
    DO pers_j=1,n_rec
        IF (SICK_ELIGPUB3_QT(pers_j).EQ.1) THEN
            SICK_DEGRPUB3_QT(pers_j) = DBLE(TAUXIT_2008(pers_j))/100.D0
            IF (SICK_DEGRPUB3_QT(pers_j).EQ.0.D0.AND.TAUXIPRES_2008(pers_j).NE.0) THEN
                SICK_DEGRPUB3_QT(pers_j) = DBLE(TAUXIPRES_2008(pers_j))/100.D0
            END IF
            IF (INT(SICK_DEGRPUB3_QT(pers_j)).GT.1) THEN
                SICK_DEGRPUB3_QT(pers_j) = 1.D0
            END IF
        END IF
    END DO        

    !Benefits in case of occupational diseases on public labour market             
    !SICK_ELIGPUB4_QT: Individual receives a benefit for occupational disease and is insured as wage earner on the public labour market            
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (FBZ_SRTUIT_2008(pers_j)(1:1).EQ.'1'.OR.FBZ_SRTUIT_2008(pers_j)(1:1).EQ.'2') THEN
                SICK_ELIGPUB4_QT(pers_j) = 1
            END IF
        END IF
    END DO        

    !SICK_TYDBPUB4_QT: Type of benefit for an occupational disease the individual, insured as wage earner on the public labour market, is entitled to          
    !1 the employee is in temporary full disablement
    !2 the employee is in temporary partial disablement
    !3 the employee is in permanent disablement
    DO pers_j=1,n_rec
        IF (SICK_ELIGPUB4_QT(pers_j).EQ.1) THEN
            !If percentage of permanent disablement is observed, we consider the indvidual as permanent disabled
            IF (FBZ_SRTUIT_2008(pers_j)(1:1).EQ.'2') THEN
                SICK_TYDBPUB4_QT(pers_j) = 3
            END IF
            !If there is an idication of temporary disablement
            IF (FBZ_SRTUIT_2008(pers_j)(1:1).EQ.'1') THEN
                !Read percentage
                READ (FBZ_PERCENT_2008(pers_j)(1:),  '(I10)') FBZ_PERCENT
                !If the percentage of disablement is 100%, consider the individual as temporary full disabled
                IF (SICK_TYDBPUB4_QT(pers_j).EQ.0.AND.FBZ_PERCENT.GE.100) THEN
                    SICK_TYDBPUB4_QT(pers_j) = 1
                !If type of disablement not identified yet, individual is temporary partial disabled
                ELSE IF (SICK_TYDBPUB4_QT(pers_j).EQ.0.AND.FBZ_PERCENT.LT.100) THEN
                    SICK_TYDBPUB4_QT(pers_j) = 2
                END IF
            END IF
        END IF
    END DO        

    !SICK_DAYSPUB4_QT: Number of days for which the individual receives a benefit for an occupational disease in the given quarter (individual is insured as wage earner on the public labour market)          
    DO pers_j=1,n_rec
        IF (SICK_ELIGPUB4_QT(pers_j).EQ.1) THEN
            !Compute the value of the first day of disability and the foreseen ending day with the function DAY_FUNC
            DAY_START = DAY_FUNC(FBZ_BEGDBET_2008(pers_j),'MM-YYYY', START_DAYQT)
            DAY_STOP = DAY_FUNC(FBZ_ENDDBET_2008(pers_j),'MM-YYYY', STOP_DAYQT)
            !Compute the number of days for which benefit is received with the function DAYS_COMPUTE
            SICK_DAYSPUB4_QT(pers_j) = DAYS_COMPUTE(DAY_START, DAY_STOP, 1, 1, 91)
        END IF
    END DO        

    !SICK_WEEKPUB4_QT: Number of weeks the individual already receives a benefit for an occupational disease (individual is insured as wage earner on the public labour market)            
    DO pers_j=1,n_rec
        IF (SICK_ELIGPUB4_QT(pers_j).EQ.1) THEN
            !Compute the value of the first day of disability with the function DAY_FUNC
            DAY_START = DAY_FUNC(FBZ_BEGDBET_2008(pers_j),'MM-YYYY', START_DAYQT)
            DAY_STOP = STOP_DAYQT   !use last day of the quarter being analysed as stop date
            !Compute the number of weeks disabled at the end of the quarter with the function WEEKS_COMPUTE
            SICK_WEEKPUB4_QT(pers_j) = WEEKS_COMPUTE(DAY_START, DAY_STOP, 7, 1, 0, 9999999)
            !If result is still 0 after this operation, we assume that the individual was disabled throughout the whole quarter
            IF (SICK_WEEKPUB4_QT(pers_j).EQ.0) THEN
                SICK_WEEKPUB4_QT(pers_j) = 13
            END IF
        END IF
    END DO        

    !SICK_DEGRPUB4_QT: Degree of disability because of the occupational disease for individual insured as wage earner on the public labour market          
    DO pers_j=1,n_rec
        IF (SICK_ELIGPUB4_QT(pers_j).EQ.1) THEN
            !Read percentage
            READ (FBZ_PERCENT_2008(pers_j)(1:),  '(I10)') FBZ_PERCENT
            SICK_DEGRPUB4_QT(pers_j) = DBLE(FBZ_PERCENT)/100.D0
            IF (INT(SICK_DEGRPUB4_QT(pers_j)).GT.1) THEN
                SICK_DEGRPUB4_QT(pers_j) = 1.D0
            END IF
        END IF
    END DO        

!CONTINUE WITH CONSRUCTION OF VALUES FOR CERTAIN VARIABLES FOR THOSE WITH STATUTORY SERVICE

    !SICK_ELIGPUB1_QT: Individual receives a benefit for sickness or disability in the wage earner regime of the public labour market         
    !1 the individual is in sickness 
!We tellen deze gevallen niet meer voor RIZIV budget, in de veronderstelling dat hun kosten gedragen worden door de instelling
!waarvoor ze werken. Vraag is dan wel waarom ze zouden kunnen voorkomen in de NIC bestanden
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            IF (InNIC(pers_j).EQ.1.AND.NIC_STELSEL_2008(pers_j).EQ.'1'.AND.NIC_SRTDAGEN_2008(pers_j).EQ.'01') THEN
                SICK_ELIGPUB1_QT(pers_j) = 1
!                WRITE(15,*) NIC_DAGENAO_2008(pers_j),NIC_BEDRAGEN_2008(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
!        IF (SICK_ELIGPUB1_QT(pers_j).GT.0.AND.SICK_ELIGPRI1_QT(pers_j).GT.0) THEN
!            WRITE(15,*) 'Test dub stat',pers_j,AANWIJZ_2008(pers_j),SICK_TYPEINSU_PUB_QT(pers_j),SICK_TYPEINSU_PRIV_QT(pers_j)
!        END IF
    END DO        

    !SICK_DAYSPUB1_QT: Number of days the individual receives a benefit for sickness or disability in the given quarter (individual is insured as wage earner on the public labour market)
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            !if sick
            IF (SICK_ELIGPUB1_QT(pers_j).EQ.1) THEN
                SICK_DAYSPUB1_QT(pers_j) = NIC_DAGENAO_2008(pers_j)
                IF (SICK_DAYSPUB1_QT(pers_j).GT.91) THEN
                    SICK_DAYSPUB1_QT(pers_j) = 91
                END IF
            END IF
        END IF
    END DO        

    !Maternity benefits on public labour market             
    !SICK_ELIGPUB2_QT: Individual receives a maternity benefit and is insured as wage earner on the public labour market          
    !1 the individual is not in disability and entitled to maternity benefits 
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            IF (InNIC(pers_j).EQ.1.AND.NIC_STELSEL_2008(pers_j).EQ.'1') THEN
!Merk op: procedure ANY werkt hier niet met karakters terwijl ze in FAMAL by Handamount wel werkt            
                IF (&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'03'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'05'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'06'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'07'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'08'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'09'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'10') THEN
                    SICK_ELIGPUB2_QT(pers_j) = 1
                END IF
            END IF
        END IF
    END DO        
   
    !SICK_DAYSPUB2_QT: Number of days the individual receives a maternity benefit in the given quarter (individual is insured as wage earner on the public labour market)         
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            !if sick
            IF (SICK_ELIGPUB2_QT(pers_j).EQ.1) THEN
                SICK_DAYSPUB2_QT(pers_j) = NIC_DAGENAO_2008(pers_j)
                IF (SICK_DAYSPUB2_QT(pers_j).GT.75) THEN
                    SICK_DAYSPUB2_QT(pers_j) = 75
                END IF
            END IF
        END IF
    END DO        

    !Benefits in case of industrial accidents on public labour market             
    !SICK_ELIGPUB3_QT: Individual receives a benefit for industrial accident and is insured as wage earner on the public labour market            
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            IF (TAUXIT_2008(pers_j).NE.0.OR.TAUXIPRES_2008(pers_j).NE.0) THEN
                SICK_ELIGPUB3_QT(pers_j) = 1
            END IF
        END IF
    END DO        

    !SICK_DAYSPUB3_QT: Number of days for which the individual receives a benefit for an industrial accident in the given quarter (individual is insured as wage earner on the public labour market)          
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            !Compute the value of the date of the industrial accident with the function DAY_FUNC
            DAY_START = DAY_FUNC(DDEBUT_2008(pers_j),'MM-YYYY', START_DAYQT)
            DAY_STOP = STOP_DAYQT   !use last day of the quarter being analysed as stop date
            SICK_DAYSPUB3_QT(pers_j) = DAY_STOP - DAY_START
            IF (SICK_DAYSPUB3_QT(pers_j).LT.0) THEN
                SICK_DAYSPUB3_QT(pers_j) = 0
            END IF
            IF (SICK_DAYSPUB3_QT(pers_j).GT.75) THEN
                SICK_DAYSPUB3_QT(pers_j) = 75
            END IF
        END IF
    END DO   
    
    !Benefits in case of occupational diseases on public labour market             
    !SICK_ELIGPUB4_QT: Individual receives a benefit for occupational diseases and is insured as wage earner on the public labour market            
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            IF (FBZ_SRTUIT_2008(pers_j)(1:1).EQ.'1'.OR.FBZ_SRTUIT_2008(pers_j)(1:1).EQ.'2') THEN
                SICK_ELIGPUB4_QT(pers_j) = 1
            END IF
        END IF
    END DO        

    !SICK_DAYSPUB4_QT: Number of days for which the individual receives a benefit for an occupational diseases in the given quarter (individual is insured as wage earner on the public labour market)          
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            !Compute the value of the first day of disability and the foreseen ending day with the function DAY_FUNC
            DAY_START = DAY_FUNC(FBZ_BEGDBET_2008(pers_j),'MM-YYYY', START_DAYQT)
            DAY_STOP = DAY_FUNC(FBZ_ENDDBET_2008(pers_j),'MM-YYYY', STOP_DAYQT)
            !Compute the number of days for which benefit is received with the function DAYS_COMPUTE
            SICK_DAYSPUB4_QT(pers_j) = DAYS_COMPUTE(DAY_START, DAY_STOP, 1, 1, 75)
        END IF
    END DO        

!Variables needed for sick or disabiled insured as self employed                           !
    !Sickness and disability self employed                                                  !
    
    !SICK_ELIGSEL1_QT: Individual receives a benefit for sickness or disability in the self employed regime            
    !1 the individual is in the first year of sickness 
    !2 the individual is in disability.
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            IF (InNIC(pers_j).EQ.1.AND.NIC_STELSEL_2008(pers_j).EQ.'2'.AND.NIC_SRTDAGEN_2008(pers_j).EQ.'01') THEN
                SICK_ELIGSEL1_QT(pers_j) = 1
            END IF
            !if registered as disabled by the RIZIV we consider the individual as disabled
            IF (RIZIVINA_STELSEL_INVAL_2008(pers_j)(1:1).EQ.'2') THEN
                IF (AANWIJZ_2008(pers_j)(1:2).EQ.'14'.OR.&
                   &AANWIJZ_2008(pers_j)(1:2).EQ.'15') THEN
                    SICK_ELIGSEL1_QT(pers_j) = 2
                END IF
            END IF
        END IF
    END DO        

    !SICK_DAYSSEL1_QT: Number of days the individual already receives a benefit for sickness or invalidity in the given quarter (individual is insured as self employed)           
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            !if sick
            IF (SICK_ELIGSEL1_QT(pers_j).EQ.1) THEN
                !No further identification information
            !if disabled                
            ELSE IF (SICK_ELIGSEL1_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for which a benefit was received with DAY_FUNC
                DAY_START = DAY_FUNC(RIZIVINA_AANVANGS_INVAL_2008(pers_j),'MM-YYYY', START_DAYQT)
!We use the last day of the quarter instead of RIZIVINA_EINDBETA_INVAL_QT(pers_j) since the use of this variable performs better
!                DAY_STOP = DAY_FUNC(RIZIVINA_EINDBETA_INVAL_QT(pers_j),'DDMMMYY', STOP_DAYQT)
                DAY_STOP = STOP_DAYQT
                !Compute the number of days for which benefit is received with the function DAYS_COMPUTE
                SICK_DAYSSEL1_QT(pers_j) = DAYS_COMPUTE(DAY_START, DAY_STOP, 7, 6, 78)
            END IF
        END IF
    END DO        

    !SICK_WEEKSEL1_QT: Number of weeks the individual already receives a benefit for sickness or invalidity (individual is insured as self employed)           
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            !if sick
            IF (SICK_ELIGSEL1_QT(pers_j).EQ.1) THEN
                !No further identification information
            !if disabled                
            ELSE IF (SICK_ELIGSEL1_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for the disability period, use function DAY_FUNC if necessary
                DAY_START = DAY_FUNC(B_ZIEKTE_2008(pers_j),'MM-YYYY', START_DAYQT)
                DAY_STOP = STOP_DAYQT !use last day of the quarter being analysed
                !Compute the number of weeks with the function WEEKS_COMPUTE
                SICK_WEEKSEL1_QT(pers_j) = WEEKS_COMPUTE(DAY_START, DAY_STOP, 7, 1,53, 9999999)
                !If result is still 0 after this operation, we assume that the individual is already a whole quarter in disability
                IF (SICK_WEEKSEL1_QT(pers_j).EQ.0) THEN
                    SICK_WEEKSEL1_QT(pers_j) = 66
                END IF
            END IF
        END IF
    END DO        

    !SICK_CLOSESEL_QT: The self employed closed his business after the incident that induced the disability status         
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            IF ( RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'10A'.OR.RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'20A'.OR.&
                &RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'30A'.OR.RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'41A'.OR.&
                &RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'42A') THEN
                    SICK_CLOSESEL_QT(pers_j) = 1
            END IF
        END IF
    END DO        

    !SICK_ELTHSELF_QT: Individual receives a supplementary benefit for help of a third person and is insured as self employed          
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            IF ( RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'11T'.OR.RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'21T'.OR.&
                &RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'31T'.OR.RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'41A'.OR.&
                &RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'42T'.OR.RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'A1T'.OR.&
                &RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'B1T'.OR.RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'C1T'.OR.&
                &RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'D1A'.OR.RIZIVINA_BETALING_INVAL_2008(pers_j)(1:3).EQ.'D2T') THEN
                    SICK_ELTHSELF_QT(pers_j) = 1
            END IF
        END IF
    END DO        

    !Maternity benefits self employed              
    !SICK_ELIGSEL2_QT: Individual receives a maternity benefit and is insured as self employed         
    !1 the individual is not in disability and entitled to maternity benefits 
    !2 the individual is in disability and entitled to maternity benefits
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            IF (InNIC(pers_j).EQ.1.AND.NIC_STELSEL_2008(pers_j).EQ.'2') THEN
!Merk op: procedure ANY werkt hier niet met karakters terwijl ze in FAMAL by Handamount wel werkt            
                IF (&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'03'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'05'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'06'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'07'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'08'.OR.NIC_SRTDAGEN_2008(pers_j).EQ.'09'.OR.&
                &NIC_SRTDAGEN_2008(pers_j).EQ.'10') THEN
                    SICK_ELIGSEL2_QT(pers_j) = 1
                END IF    
            END IF
            !if in disability
            IF (RIZIVINA_STELSEL_MAT_2008(pers_j)(1:1).EQ.'4') THEN
                IF (AANWIJZ_2008(pers_j)(1:2).EQ.'14'.OR.&
                   &AANWIJZ_2008(pers_j)(1:2).EQ.'15') THEN
                    SICK_ELIGSEL2_QT(pers_j) = 2
                END IF
            END IF
        END IF
    END DO        

    !SICK_WEEKSEL2_QT: Number of weeks the individual receives a maternity benefit in the given quarter for individual insured as a self employed          
    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            !if sick
            IF (SICK_ELIGSEL2_QT(pers_j).EQ.1) THEN
                !No further identification information
            !if disabled                
            ELSE IF (SICK_ELIGSEL2_QT(pers_j).EQ.2) THEN
                !Compute the value of the first and last registered day for which a benefit was received with DAY_FUNC
                DAY_START = DAY_FUNC(RIZIVINA_AANVANGS_MAT_2008(pers_j),'MM-YYYY', START_DAYQT)
!We use the last day of the quarter instead of RIZIVINA_EINDBETA_MAT_QT(pers_j) since the use of this variable performs better
!                DAY_STOP = DAY_FUNC(RIZIVINA_EINDBETA_MAT_QT(pers_j),'DDMMMYY', STOP_DAYQT)
                DAY_STOP = STOP_DAYQT
                !Compute the number of weeks with the function WEEKS_COMPUTE
                SICK_WEEKSEL2_QT(pers_j) = WEEKS_COMPUTE(DAY_START, DAY_STOP, 7, 1, 0, 3)
                !If result is still 0 after this operation, we assume that the individual received the benefit for the maximum number of weeks
                IF (SICK_WEEKSEL2_QT(pers_j).EQ.0) THEN
                    SICK_WEEKSEL2_QT(pers_j) = 3
                END IF
            END IF
        END IF
    END DO        

END SUBROUTINE SICK_CHARAC

!===================================================================================================================!
!CONSTRUCTION OF SICK_FAMSIT_QT                                                                                     !
!   = The family situation of the unemployed according to the unemployment legislation                              !
!   SICK_FAMSIT_QT =  1 the sick or disabled has dependent family                                                   !
!   SICK_FAMSIT_QT =  2 the sick or disabled is single                                                              !
!   SICK_FAMSIT_QT =  3 the sick or disabled is without dependent family and has not lost his sole source of income ! 
!and some variables required for the construction of this variable or for later use                                 !
!   SICK_REL_QT                                                                                                     !
!   SICK_DEPINC_QT                                                                                                  !
!===================================================================================================================!

SUBROUTINE SICK_FAMSIT_CONSTRUC

    INTEGER                         :: hh_i,i
    INTEGER                         :: pers_j,lijn_j,pers_k
    INTEGER                         :: PARTNER                  !indicate whether the sick or disabled has a partner yes or no 
    INTEGER                         :: NUMOTHER                 !count the number of others in the household

    !loop over all households
    DO hh_i=1,N_HH_SICK
        !loop over all members of the sociological household
        DO pers_j=SICK_START_HH(hh_i),SICK_END_HH(hh_i)                      
            !Check whether pers_j is sick or disabled and lives in a sociological household with more than 1 member
            IF (SICK_RISKGLOB_QT(pers_j).NE.0.AND.SICK_HH_SIZE(hh_i).GT.1) THEN
                !Determine the line in the parameter SICK_FAMREL_QT to convert relationship codes
                DO i = 1,SIZE(SICK_FAMREL_QT,1)
                    IF (FAMREL_RELATION(pers_j).EQ.SICK_FAMREL_QT(i,1)) THEN
                        lijn_j = i
                    END IF
                END DO
                !Initialise all relationship codes and determine the relation of each member of the sociological
                !household with the sick or disabled
                PARTNER = 0
                NUMOTHER = 0
                DO pers_k=SICK_START_HH(hh_i),SICK_END_HH(hh_i)
                    !initialise variables that will be reused for each individual determination of SICK_FAMSIT_QT
                    SICK_REL_QT(pers_k) = 0
                    SICK_DEPINC_QT(pers_k) = 0
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                        DO i = 1,SIZE(SICK_FAMREL_QT,2)
                            IF (FAMREL_RELATION(pers_k).EQ.SICK_FAMREL_QT(1,i)) THEN
                                SICK_REL_QT(pers_k) = SICK_FAMREL_QT(lijn_j,i)
                                !If the child is 25 or older clasify it as relative up till the third degree and not as child                                
                                IF (SICK_YEAR_AGE(pers_k).GE.25) THEN
                                    SICK_REL_QT(pers_k) = 5
                                END IF
                                !If there is a partner, store an indicator of this in the variable PARTNER
                                IF (SICK_FAMREL_QT(lijn_j,i).EQ.1) THEN
                                    PARTNER = pers_k
                                END IF
                                !count number of other household members
                                IF (SICK_FAMREL_QT(lijn_j,i).EQ.MAXVAL(SICK_FAMREL_QT(lijn_j,:),2)) THEN
                                    NUMOTHER = NUMOTHER + 1
                                END IF
                            END IF
                        END DO
                    !Members of the sociological household that are not family member are considered as non relatives
                    ELSE IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).NE.FAMREL_FAMILY(pers_k)) THEN
                        SICK_REL_QT(pers_k) = MAXVAL(SICK_FAMREL_QT(lijn_j,:),2)
                        NUMOTHER = NUMOTHER + 1
                    END IF
                END DO
                !Check the income condition per household member, conditional on the relationship code
                DO pers_k=SICK_START_HH(hh_i),SICK_END_HH(hh_i)
                    IF (ANY(SICK_REL_QT(pers_k).EQ.(/1, 2, 3, 4, 5, 6/))) THEN
                        !double income ceiling for dependents up to the third degree
                        IF (ANY(SICK_REL_QT(pers_k).EQ.(/1, 2, 3, 4, 5/))) THEN
                            IF   ((&
                                &(CONTRIB_REVPRIV_QT(pers_k) - CONTRIB_SSPRIV_EMEE_QT(pers_k)) +&
                                &(CONTRIB_REVPUB_QT(pers_k) - CONTRIB_SSPUB_EMEE_QT(pers_k)) + &
                                &(CONTRIB_INCSELF_QT(pers_k) - CONTRIB_SSSELF_EMER_QT(pers_k)) + &
                                &PENSWELF_AMOUNTP_QT(pers_k) + SICK_TOTBENPR_QT(pers_k) + UNEM_BENUN_QT(pers_k))&
                                &.GT.SICK_WAGEDFQ1_QT(1).AND.&
                                  (&
                                &(CONTRIB_REVPRIV_QT(pers_k) - CONTRIB_SSPRIV_EMEE_QT(pers_k)) +&
                                &(CONTRIB_REVPUB_QT(pers_k) - CONTRIB_SSPUB_EMEE_QT(pers_k)) + &
                                &(CONTRIB_INCSELF_QT(pers_k) - CONTRIB_SSSELF_EMER_QT(pers_k)) + &
                                &PENSWELF_AMOUNTP_QT(pers_k) + SICK_TOTBENPR_QT(pers_k) + UNEM_BENUN_QT(pers_k))&
                                &.LE.SICK_WAGEDFQ1_QT(2)) THEN
                                    SICK_DEPINC_QT(pers_k) = 1                            
                            ELSE IF ((&
                                &(CONTRIB_REVPRIV_QT(pers_k) - CONTRIB_SSPRIV_EMEE_QT(pers_k)) +&
                                &(CONTRIB_REVPUB_QT(pers_k) - CONTRIB_SSPUB_EMEE_QT(pers_k)) + &
                                &(CONTRIB_INCSELF_QT(pers_k) - CONTRIB_SSSELF_EMER_QT(pers_k)) + &
                                &PENSWELF_AMOUNTP_QT(pers_k) + SICK_TOTBENPR_QT(pers_k) + UNEM_BENUN_QT(pers_k))&
                                &.GT.SICK_WAGEDFQ1_QT(2)) THEN
                                    SICK_DEPINC_QT(pers_k) = 2 
                            END IF                            
                        ELSE
                            IF    ((&
                                &(CONTRIB_REVPRIV_QT(pers_k) - CONTRIB_SSPRIV_EMEE_QT(pers_k)) +&
                                &(CONTRIB_REVPUB_QT(pers_k) - CONTRIB_SSPUB_EMEE_QT(pers_k)) + &
                                &(CONTRIB_INCSELF_QT(pers_k) - CONTRIB_SSSELF_EMER_QT(pers_k)) + &
                                &PENSWELF_AMOUNTP_QT(pers_k) + SICK_TOTBENPR_QT(pers_k) + UNEM_BENUN_QT(pers_k))&
                                &.GT.SICK_WAGEDFQ1_QT(1)) THEN
                                    SICK_DEPINC_QT(pers_k) = 2 
                            END IF
                        END IF
                    END IF                        
                END DO
                !Determine the value of SICK_FAMSIT_QT(pers_j)
                !Distinguish between the values 1 and 3
                !Check whether the sick or disabled has a partner yes or no
                IF (PARTNER.NE.0) THEN
                    IF (SICK_DEPINC_QT(PARTNER).EQ.2) THEN
                        SICK_FAMSIT_QT(pers_j) = 3
                    ELSE IF (SICK_DEPINC_QT(PARTNER).EQ.1) THEN
                        SICK_FAMSIT_QT(pers_j) = 2
                    ELSE 
                        SICK_FAMSIT_QT(pers_j) = 1
                    END IF
                !if no partner                    
                ELSE                        
                    !A: Check whether there is a household member that satisfies the income conditions to classify the sick
                    !as cohabitating
                    DO pers_k=SICK_START_HH(hh_i),SICK_END_HH(hh_i)
                        IF (pers_j.NE.pers_k.AND.SICK_DEPINC_QT(pers_k).EQ.2) THEN
                            SICK_FAMSIT_QT(pers_j) = 3
                            EXIT
                        END IF
                    END DO
                    !B: If the sick or disabled cohabits with people with limited income he can be classified
                    !as single
                    IF (SICK_FAMSIT_QT(pers_j).EQ.0) THEN
                        DO pers_k=SICK_START_HH(hh_i),SICK_END_HH(hh_i)
                            IF (pers_j.NE.pers_k.AND.SICK_DEPINC_QT(pers_k).EQ.1) THEN
                                SICK_FAMSIT_QT(pers_j) = 2
                                EXIT
                            END IF
                        END DO
                    END IF
                    !In all other cases where the sick or disabled cohabits with other people we consider him to have family charge
                    IF (SICK_FAMSIT_QT(pers_j).EQ.0) THEN
                        SICK_FAMSIT_QT(pers_j) = 1
                    END IF
                END IF
            !if the sick or disabled is single, set SICK_FAMSIT_QT = 2
            ELSE IF (SICK_RISKGLOB_QT(pers_j).NE.0.AND.SICK_HH_SIZE(hh_i).EQ.1) THEN
                SICK_FAMSIT_QT(pers_j) = 2
            END IF
        END DO
    END DO  

END SUBROUTINE SICK_FAMSIT_CONSTRUC

!=================================================================================!
!Routine to compute amount that has to be subtracted from daily benefit in
!case of labour income
!=================================================================================!

SUBROUTINE SICK_DAILYLAB

    REAL(KIND=KIND_REAL)                                :: SICK_GTINC_DAY               !Gross taxable income per day
    REAL(KIND=KIND_REAL)                                :: SICK_NTINC_DAY               !Net taxable income per day

    !loop over all households
    DO hh_i=1,N_HH_SICK
        !loop over all members of the sociological household
        DO pers_j=SICK_START_HH(hh_i),SICK_END_HH(hh_i)                      
            SICK_GTINC_DAY =    (CONTRIB_REVPRIV_QT(pers_j) + CONTRIB_REVPUB_QT(pers_j))/78.D0
            SICK_NTINC_DAY =    ((CONTRIB_REVPRIV_QT(pers_j) - CONTRIB_SSPRIV_EMEE_QT(pers_j)) + &
                                &(CONTRIB_REVPUB_QT(pers_j) - CONTRIB_SSPUB_EMEE_QT(pers_j)))/78.D0
            IF (SICK_NTINC_DAY.GT.0.D0) THEN
                IF (SIMYEAR.LT.2003) THEN
                    SICK_LABDEDUC_DAY(pers_j) = SICK_GTINC_DAY
                ELSE
                    SICK_LABDEDUC_DAY(pers_j) = &
                    &SICK_BRACRATE_3COLUMNS(SICK_NTINC_DAY,SIZE(SICK_LABINCRA_DAY,1),SICK_LABINCRA_DAY,pers_j)
                END IF
            END IF
        END DO
    END DO  

END SUBROUTINE SICK_DAILYLAB

!==========================================================!
!With the subroutine SICK_BENPRIV_CONSTRUC we compute      !
!the sickness and disability benefits for those insured    !
!as wage earner on the private labour market               !
!Variables constructed in the routine                      !
!Sickness and disability benefits                          !
!   SICK_BENPRI1_DAY                                       !
!   SICK_BENPRI1_QT                                        !
!   SICK_EMPBPRI1_QT                                       !
!   SICK_SUPPRI1_DAY                                       !
!   SICK_SUPPRI1_QT                                        !
!   SICK_EMPSPRI1_QT                                       !
!   SICK_BETHPRIV_DAY                                      !
!   SICK_BETHPRIV_QT                                       !
!Maternity benefits                                        !
!   SICK_BENPRI2_DAY                                       !
!   SICK_BENPRI2_QT                                        !
!Industrial accidents                                      !
!   SICK_BENPRI3_DAY                                       !
!   SICK_BENPRI3_QT                                        !
!   SICK_EMPBPRI3_QT                                       !
!Occupational diseases                                     !
!   SICK_BENPRI4_DAY                                       !
!   SICK_BENPRI4_QT                                        !
!   SICK_EMPBPRI4_QT                                       !
!==========================================================!

SUBROUTINE SICK_BENPRIV_CONSTRUC(pers_j)

    INTEGER                                 :: pers_j,i,j
    INTEGER                                 :: WEEK_LOOP            !Number of weeks for which the loop with which the benefits are computed should run
    INTEGER                                 :: WEEK_START           !First week for which a benefit is computed
    REAL(KIND=KIND_REAL)                    :: LDW                  !Variable to store the limited daily wage (value selected conditional on working week regime)  
    INTEGER                                 :: TYPE_SICK            !Identifier to identify the type of sickness or disability benefit (used to reduce the check on minima and maxima)
    INTEGER                                 :: DAYS_REST            !Remaining number of days in the last week for which daily benefits are computed
    REAL(KIND=KIND_REAL),DIMENSION(2)       :: LDW_BLUE             !Variable to store the wage of blue collar workers above or below a given threshold
    REAL(KIND=KIND_REAL)                    :: SICK_GTINC_ANN       !Gross taxable income per year           
    REAL(KIND=KIND_REAL)                    :: SICK_PREPAY_DAY      !daily amount of prepayments
    REAL(KIND=KIND_REAL)                    :: SICK_BASE_DAY        !base to compute allowed cumulation between sick benefits and labour income for system before 2003
    REAL(KIND=KIND_REAL)                    :: SumInc               !sum income components

    !Sickness and disability benefits
    !Compute regular benefits:
    !SICK_BENPRI1_DAY: Daily benefit for sickness or disability for individual in the wage earner regime of the private labour market          
    !SICK_BENPRI1_QT:  Quarterly benefit for sickness or disability for individual in the wage earner regime of the private labour market          
    !SICK_EMPBPRI1_QT: Part of the quarterly benefit for sickness or disability paid by the employer for individual insured as wage earner on the private labour market            
    IF (ANY(SICK_ELIGPRI1_QT(pers_j).EQ.(/1, 2/))) THEN
        WEEK_LOOP = 0
        WEEK_START = 0
        LDW = 0
        DAYS_REST = 0
        !Determine the number of weeks in sickness or disability in the given quarter
        WEEK_LOOP = CEILING(DBLE(SICK_DAYSPRI1_QT(pers_j))/6.D0)
        IF (WEEK_LOOP.GT.13) THEN
            WEEK_LOOP = 13
        END IF
        !Determine the first week in sickness or disability in the given quarter
        WEEK_START = SICK_WEEKPRI1_QT(pers_j) - WEEK_LOOP  
        IF (WEEK_START.LT.0) THEN
            WEEK_START = 0
        ELSE IF (WEEK_START.EQ.0) THEN
            WEEK_START = 1
        END IF
        IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.0) THEN
            !Compute the limited daily wage
            IF (SICK_WOWEPRIV_QT(pers_j).EQ.1) THEN
                LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(1))
            ELSE IF (SICK_WOWEPRIV_QT(pers_j).EQ.2) THEN
                LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(2))
            END IF
            !Start loop to compute daily and quarterly benefits
            DO i=1,WEEK_LOOP
                !distinguish the relevant cases for the computation of the daily amount on the family charge of the sick or disabled
                !daily amount before correcting minima and maxima
                TYPE_SICK = 0
                SICK_BASE_DAY = 0.D0
                !If the employee has dependent family
                IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 1
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(2,1)/100.D0)*LDW
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(2,1)/100.D0)*LDW
                                TYPE_SICK = 2
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2) THEN
                            SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            SICK_BASE_DAY = (SICK_PERCPRIV_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 3
                        END IF
                    !In month 2 till 6
                    ELSE IF ((WEEK_START+i-1).GT.4.AND.(WEEK_START+i-1).LE.26) THEN
                        SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(4,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(4,1)/100.D0)*LDW
                        TYPE_SICK = 4
                    !In month 7 till 12
                    ELSE IF ((WEEK_START+i-1).GT.26.AND.(WEEK_START+i-1).LT.53) THEN
                        SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(5,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(5,1)/100.D0)*LDW
                        TYPE_SICK = 5
                    !If in disability
                    ELSE IF ((WEEK_START+i-1).GE.53) THEN
                        SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(6,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(6,1)/100.D0)*LDW
                        TYPE_SICK = 6
                    END IF
                !If employee has no family charge but lost his sole source of income
                ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPRIV_QT(pers_j).EQ.1) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(7,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 7
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(8,1)/100.D0)*LDW
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(2,1)/100.D0)*LDW
                                TYPE_SICK = 8
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2) THEN
                            SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(9,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            SICK_BASE_DAY = (SICK_PERCPRIV_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 9
                        END IF
                    !In month 2 till 6
                    ELSE IF ((WEEK_START+i-1).GT.4.AND.(WEEK_START+i-1).LE.26) THEN
                        SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(10,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(4,1)/100.D0)*LDW
                        TYPE_SICK = 10
                    !In month 7 till 12
                    ELSE IF ((WEEK_START+i-1).GT.26.AND.(WEEK_START+i-1).LT.53) THEN
                        SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(11,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(5,1)/100.D0)*LDW
                        TYPE_SICK = 11
                    !If in disability
                    ELSE IF ((WEEK_START+i-1).GE.53) THEN
                        SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(12,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(6,1)/100.D0)*LDW
                        TYPE_SICK = 12
                    END IF
                !If employee has no family charge and did not loose his sole source of income
                ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPRIV_QT(pers_j).EQ.0) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(13,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 13
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(14,1)/100.D0)*LDW
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(2,1)/100.D0)*LDW
                                TYPE_SICK = 14
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2) THEN
                            SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(15,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            SICK_BASE_DAY = (SICK_PERCPRIV_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 15
                        END IF
                    !In month 2 till 6
                    ELSE IF ((WEEK_START+i-1).GT.4.AND.(WEEK_START+i-1).LE.26) THEN
                        SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(16,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(4,1)/100.D0)*LDW
                        TYPE_SICK = 16
                    !In month 7 till 12
                    ELSE IF ((WEEK_START+i-1).GT.26.AND.(WEEK_START+i-1).LT.53) THEN
                        SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(17,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(5,1)/100.D0)*LDW
                        TYPE_SICK = 17
                    !If in disability
                    ELSE IF ((WEEK_START+i-1).GE.53) THEN
                        SICK_BENPRI1_DAY(pers_j) = (SICK_PERCPRIV_DAY(18,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(6,1)/100.D0)*LDW
                        TYPE_SICK = 18
                    END IF
                END IF
                !correct minima and maxima
                IF (TYPE_SICK.LE.18) THEN
                    !loop over different disability types
                    DO j=1,18
                        !Corrections for basic amount
                        IF (TYPE_SICK.EQ.j) THEN
                            !Correct minimum 
                            !If a regular worker
                            IF (SICK_REGWPRIV_QT(pers_j).EQ.1) THEN
                                IF (SICK_BENPRI1_DAY(pers_j).LT.SICK_PERCPRIV_DAY(j,3)) THEN
                                    SICK_BENPRI1_DAY(pers_j) = SICK_PERCPRIV_DAY(j,3)
                                END IF
                            !If an irregular worker    
                            ELSE IF (SICK_REGWPRIV_QT(pers_j).EQ.2) THEN
                                IF (SICK_BENPRI1_DAY(pers_j).LT.SICK_PERCPRIV_DAY(j,2)) THEN
                                    SICK_BENPRI1_DAY(pers_j) = SICK_PERCPRIV_DAY(j,2)
                                END IF
                            END IF
                            !Correct maximum
                            IF (SICK_BENPRI1_DAY(pers_j).GT.SICK_PERCPRIV_DAY(j,4).AND.&
                            &   INT(100.D0*SICK_PERCPRIV_DAY(j,4)).NE.0) THEN
                                SICK_BENPRI1_DAY(pers_j) = SICK_PERCPRIV_DAY(j,4)
                            END IF
                        END IF
                        !Corrections for SICK_BASE_DAY
                        IF (TYPE_SICK.EQ.j.AND.TYPE_SICK.LT.7) THEN
                            !Correct minimum
                            !If a regular worker
                            IF (SICK_REGWPRIV_QT(pers_j).EQ.1) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j,3)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j,3)
                                END IF
                            !If an irregular worker    
                            ELSE IF (SICK_REGWPRIV_QT(pers_j).EQ.2) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j,2)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j,2)
                                END IF
                            END IF
                            !Correct maximum
                            IF (SICK_BASE_DAY.GT.SICK_PERCPRIV_DAY(j,4).AND.&
                            &   INT(100.D0*SICK_PERCPRIV_DAY(j,4)).NE.0) THEN
                                SICK_BASE_DAY = SICK_PERCPRIV_DAY(j,4)
                            END IF
                        ELSE IF (TYPE_SICK.EQ.j.AND.TYPE_SICK.GE.7.AND.TYPE_SICK.LT.13) THEN
                            !Correct minimum
                            !If a regular worker
                            IF (SICK_REGWPRIV_QT(pers_j).EQ.1) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j-6,3)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-6,3)
                                END IF
                            !If an irregular worker    
                            ELSE IF (SICK_REGWPRIV_QT(pers_j).EQ.2) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j-6,2)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-6,2)
                                END IF
                            END IF
                            !Correct maximum
                            IF (SICK_BASE_DAY.GT.SICK_PERCPRIV_DAY(j-6,4).AND.&
                            &   INT(100.D0*SICK_PERCPRIV_DAY(j-6,4)).NE.0) THEN
                                SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-6,4)
                            END IF
                        ELSE IF (TYPE_SICK.EQ.j.AND.TYPE_SICK.GE.13) THEN
                            !Correct minimum
                            !If a regular worker
                            IF (SICK_REGWPRIV_QT(pers_j).EQ.1) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j-12,3)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-12,3)
                                END IF
                            !If an irregular worker    
                            ELSE IF (SICK_REGWPRIV_QT(pers_j).EQ.2) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j-12,2)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-12,2)
                                END IF
                            END IF
                            !Correct maximum
                            IF (SICK_BASE_DAY.GT.SICK_PERCPRIV_DAY(j-12,4).AND.&
                            &   INT(100.D0*SICK_PERCPRIV_DAY(j-12,4)).NE.0) THEN
                                SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-12,4)
                            END IF
                        END IF
                    END DO
                END IF
                !Correct the daily amount when labour income is simulate 
                IF (MIMOSIS_SIMNUMMT(pers_j).NE.0) THEN
                    IF (SIMYEAR.LT.2003) THEN
                        IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN
                            SICK_BENPRI1_DAY(pers_j) = (SICK_BASE_DAY)*(SICK_LABCUMUL_DAY(1)/100.D0) - SICK_LABDEDUC_DAY(pers_j)
                        ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/))) THEN
                            SICK_BENPRI1_DAY(pers_j) = (SICK_BASE_DAY)*(SICK_LABCUMUL_DAY(2)/100.D0) - SICK_LABDEDUC_DAY(pers_j)
                        END IF  
                    ELSE
                        SICK_BENPRI1_DAY(pers_j) = SICK_BENPRI1_DAY(pers_j) - SICK_LABDEDUC_DAY(pers_j)
                    END IF
                    !Correct if negative
                    IF (SICK_BENPRI1_DAY(pers_j).LT.0.D0) THEN
                        SICK_BENPRI1_DAY(pers_j) = 0.D0
                    END IF
                END IF
                !Add the daily amount to the quarterly amount
                IF (i.LT.WEEK_LOOP) THEN
                    SICK_BENPRI1_QT(pers_j) = SICK_BENPRI1_QT(pers_j) + SICK_BENPRI1_DAY(pers_j)*6.D0
                    !distinguish disability and primary disablement for comparison with exertnal sources
                    IF (ANY(TYPE_SICK.EQ.(/6,12,18/))) THEN
                        SICK_DISEMPNM_QT(pers_j,1) = SICK_DISEMPNM_QT(pers_j,1) + 6.D0
!                        SICK_DISEMPNM_QT(pers_j,2) = SICK_DISEMPNM_QT(pers_j,2) + SICK_BENPRI1_DAY(pers_j)*6.D0
                    ELSE IF (ANY(TYPE_SICK.EQ.(/1,2,3,4,5,7,8,9,10,11,13,14,15,16,17/))) THEN
                        !In case of a blue collar worker
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.AND.(WEEK_START+i-1).GT.2) THEN
                            SICK_SICEMPNM_QT(pers_j,1) = SICK_SICEMPNM_QT(pers_j,1) + 6.D0
!                            SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BENPRI1_DAY(pers_j)*6.D0
                        END IF
                        !In case of white collar worker
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.AND.(WEEK_START+i-1).GT.4) THEN
                            SICK_SICEMPNM_QT(pers_j,1) = SICK_SICEMPNM_QT(pers_j,1) + 6.D0
!                            SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BENPRI1_DAY(pers_j)*6.D0
                        END IF
                    END IF
                ELSE IF (i.EQ.WEEK_LOOP) THEN
                    DAYS_REST = CEILING(DBLE(SICK_DAYSPRI1_QT(pers_j)) - DBLE(WEEK_LOOP-1)*6.D0)
                    IF (DAYS_REST.LT.0) THEN
                        DAYS_REST = 0
                    END IF 
                    SICK_BENPRI1_QT(pers_j) = SICK_BENPRI1_QT(pers_j) + SICK_BENPRI1_DAY(pers_j)*DBLE(DAYS_REST)
                    !distinguish disability and primary disablement for comparison with exertnal sources
                    IF (ANY(TYPE_SICK.EQ.(/6,12,18/))) THEN
                        SICK_DISEMPNM_QT(pers_j,1) = SICK_DISEMPNM_QT(pers_j,1) + DBLE(DAYS_REST)
!                        SICK_DISEMPNM_QT(pers_j,2) = SICK_DISEMPNM_QT(pers_j,2) + SICK_BENPRI1_DAY(pers_j)*DBLE(DAYS_REST)
                    ELSE IF (ANY(TYPE_SICK.EQ.(/1,2,3,4,5,7,8,9,10,11,13,14,15,16,17/))) THEN
                        !In case of a blue collar worker
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.AND.(WEEK_START+i-1).GT.2) THEN
                            SICK_SICEMPNM_QT(pers_j,1) = SICK_SICEMPNM_QT(pers_j,1) + DBLE(DAYS_REST)
!                            SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BENPRI1_DAY(pers_j)*DBLE(DAYS_REST)
                        END IF
                        !In case of white collar worker
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.AND.(WEEK_START+i-1).GT.4) THEN
                            SICK_SICEMPNM_QT(pers_j,1) = SICK_SICEMPNM_QT(pers_j,1) + DBLE(DAYS_REST)
!                            SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BENPRI1_DAY(pers_j)*DBLE(DAYS_REST)
                        END IF
                    END IF
                END IF
                !Compute the amount paid by the employer
                !In case of a blue collar worker
                IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.AND.(WEEK_START+i-1).LE.2) THEN
                    SICK_EMPBPRI1_QT(pers_j) = SICK_BENPRI1_QT(pers_j)
                END IF
                !In case of white collar worker
                IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.AND.(WEEK_START+i-1).LE.4) THEN
                    SICK_EMPBPRI1_QT(pers_j) = SICK_BENPRI1_QT(pers_j)
                END IF
            END DO
        END IF
!Aangevuld bij aanpassingen september 2011: later bijstellen bij grondigere testing
!individu kan in geven jaar invalide zijn maar uitkeringen NIC ontvangen
        IF (ANY(SICK_ELIGPRI1_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (NIC_BEDRAG_2008(pers_j).NE.0.D0) THEN
                SICK_BENPRI1_QT(pers_j) = NIC_BEDRAG_2008(pers_j)/4.D0
                SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BENPRI1_QT(pers_j) 
            END IF
        END IF
        IF (SICK_ELIGPRI1_QT(pers_j).EQ.2) THEN
            IF (RIZIVINA_BEDRAG_INVAL_2008(pers_j).NE.0.D0) THEN
                SICK_BENPRI1_QT(pers_j) = RIZIVINA_BEDRAG_INVAL_2008(pers_j)/4.D0                
                SICK_DISEMPNM_QT(pers_j,2) = SICK_DISEMPNM_QT(pers_j,2) + SICK_BENPRI1_QT(pers_j) 
            END IF
        END IF
    END IF

    !Compute supplementary benefit for blue collar workers
    !SICK_SUPPRI1_DAY: Additional daily supplement for blue collar workers insured as wage earner on the private labour market         
    !SICK_SUPPRI1_QT:  Additional quarterly supplement for blue collar worker who is insured as wage earner on the private labour market           
    !SICK_EMPSPRI1_QT: Part of the additional quarterly supplement paid by the employer for blue collar worker who is insured as wage earner on the private labour market          
    IF (SICK_ELIGPRI1_QT(pers_j).EQ.1.AND.SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1) THEN
        IF (ANY(SICK_WEEKPRI1_QT(pers_j).EQ.(/2, 3, 4/))) THEN
            LDW_BLUE(1:2) = 0.D0
            WEEK_LOOP = 0
            WEEK_START = 0
            SICK_GTINC_ANN = 0
            SICK_PREPAY_DAY = 0
            !Determine the number of weeks in sickness or disability in the given quarter
            WEEK_LOOP = CEILING(DBLE(SICK_DAYSPRI1_QT(pers_j))/6.D0)
            IF (WEEK_LOOP.GT.4) THEN
                WEEK_LOOP = 4
            END IF
            !Determine the first week in sickness or disability in the given quarter
            WEEK_START = SICK_WEEKPRI1_QT(pers_j) - WEEK_LOOP  
            IF (WEEK_START.LT.0) THEN
                WEEK_START = 0
            ELSE IF (WEEK_START.EQ.0) THEN
                WEEK_START = 1
            END IF
            !start computation only from the second week on
            IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.1) THEN
                !Compute the daily wage above and below the daily wage threshold
                IF (SICK_LDAYWAGE_DAY(pers_j).LT.SICK_LIMPRIV_DAY(1)) THEN
                    LDW_BLUE(1) = SICK_LDAYWAGE_DAY(pers_j) 
                    LDW_BLUE(2) = 0                    
                ELSE 
                    LDW_BLUE(1) = SICK_LIMPRIV_DAY(1)
                    LDW_BLUE(2) = SICK_LDAYWAGE_DAY(pers_j) - SICK_LIMPRIV_DAY(1)                  
                END IF
                !Compute the prepayments per day
                !First compute annual income
                SICK_GTINC_ANN = SICK_LDAYWAGE_DAY(pers_j)*260.D0 - &
                &(SICK_SSCONT_QT/100.D0)*SICK_LDAYWAGE_DAY(pers_j)*260.D0
                !Then use the function SICK_PREPAY_COMPUTE to compute prepayments per day paid on this income
                SICK_PREPAY_DAY = SICK_PREPAY_COMPUTE(SICK_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),&
                &GECODEERD_INSZ_INPUT(pers_j))
                !Compute the daily benefit
                SICK_SUPPRI1_DAY(pers_j) =  &
                                            &(SICK_SUPPRIV_DAY(1)/100.D0)*LDW_BLUE(1) + &
                                            &(SICK_SUPPRIV_DAY(2)/100.D0)*LDW_BLUE(2) - &
                                            &SICK_PREPAY_DAY
                IF (SICK_SUPPRI1_DAY(pers_j).LT.0) THEN
                    SICK_SUPPRI1_DAY(pers_j) = 0
                END IF
                !Start loop to compute quarterly benefits: only count from second week on
                DO i=2,WEEK_LOOP
                    !Add the daily amount to the quarterly amount
                    IF (i.LT.WEEK_LOOP) THEN
                        SICK_SUPPRI1_QT(pers_j) = SICK_SUPPRI1_QT(pers_j) + SICK_SUPPRI1_DAY(pers_j)*6.D0
                    ELSE IF (i.EQ.WEEK_LOOP) THEN
                        DAYS_REST = CEILING(DBLE(SICK_DAYSPRI1_QT(pers_j)) - DBLE(WEEK_LOOP-1)*6.D0)
                        IF (DAYS_REST.LT.0) THEN
                            DAYS_REST = 0
                        END IF 
                        SICK_SUPPRI1_QT(pers_j) = SICK_SUPPRI1_QT(pers_j) + SICK_SUPPRI1_DAY(pers_j)*DBLE(DAYS_REST)
                    END IF
                    !Compute the amount paid by the employer
                    IF ((WEEK_START+i-1).EQ.2) THEN
                        SICK_EMPSPRI1_QT(pers_j) = SICK_SUPPRI1_QT(pers_j)
                    END IF
                END DO
            END IF    
        END IF
        !add supplement to the quarterly benefit
        SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_SUPPRI1_QT(pers_j) - SICK_EMPSPRI1_QT(pers_j)
!        IF (SICK_SUPPRI1_QT(pers_j).LT.SICK_EMPSPRI1_QT(pers_j)) THEN
!            WRITE(15,*) 'TestA',SICK_SUPPRI1_QT(pers_j),SICK_EMPSPRI1_QT(pers_j)
!        END IF
    END IF

    !Compute supplementary benefit for help of a third person
    !SICK_BETHPRIV_DAY: Daily supplement for help of a third person (individual is insured as wage earner on the private labour market)         
    !SICK_BETHPRIV_QT:  Quarterly supplement for help of a third person (individual is insured as wage earner on the private labour market)         
    IF (SICK_ELTHPRIV_QT(pers_j).EQ.1) THEN
        WEEK_LOOP = 0
        WEEK_START = 0
        LDW = 0
        DAYS_REST = 0
        !Determine the number of weeks in sickness or disability in the given quarter
        WEEK_LOOP = CEILING(DBLE(SICK_DAYSPRI1_QT(pers_j))/6.D0)
        IF (WEEK_LOOP.GT.13) THEN
            WEEK_LOOP = 13
        END IF
        !Determine the first week in sickness or disability in the given quarter
        WEEK_START = SICK_WEEKPRI1_QT(pers_j) - WEEK_LOOP  
        IF (WEEK_START.LT.0) THEN
            WEEK_START = 0
        ELSE IF (WEEK_START.EQ.0) THEN
            WEEK_START = 1
        END IF
        IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.0) THEN
            !Compute the limited daily wage
            IF (SICK_WOWEPRIV_QT(pers_j).EQ.1) THEN
                LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(1))
            ELSE IF (SICK_WOWEPRIV_QT(pers_j).EQ.2) THEN
                LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(2))
            END IF
            !Start loop to compute daily and quarterly benefits
            DO i=1,WEEK_LOOP
                !distinguish the relevant cases for the computation of the daily amount on the family charge of the sick or disabled
                !daily amount before correcting minima and maxima
                TYPE_SICK = 0
                !Distinguish way of computing help for third persons: system is different before and after 2007
                IF (SIMYEAR.LT.2007) THEN
                    !If the employee has dependent family
                    IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(1,1)
                            TYPE_SICK = 1
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(2,1)
                            TYPE_SICK = 2
                        END IF
                    !If employee has no family charge but lost his sole source of income
                    ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPRIV_QT(pers_j).EQ.1) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(3,1)
                            TYPE_SICK = 3
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = (SICK_THIRPRIV_DAY(4,1)/100.D0)*LDW
                            TYPE_SICK = 4
                        END IF
                    !If employee has no family charge and did not loose his sole source of income
                    ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPRIV_QT(pers_j).EQ.0) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(5,1)
                            TYPE_SICK = 5
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = (SICK_THIRPRIV_DAY(6,1)/100.D0)*LDW
                            TYPE_SICK = 6
                        END IF
                    END IF
                    !correct minima and maxima
                    IF (TYPE_SICK.LE.6) THEN
                        !loop over different disability types
                        DO j=1,6
                            IF (TYPE_SICK.EQ.j) THEN
                                !Correct minimum
                                IF (SICK_BETHPRIV_DAY(pers_j).LT.SICK_THIRPRIV_DAY(j,2)) THEN
                                    SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(j,2)
                                END IF
                                !Correct maximum
                                IF (SICK_BETHPRIV_DAY(pers_j).GT.SICK_THIRPRIV_DAY(j,3).AND.&
                                &   INT(100.D0*SICK_THIRPRIV_DAY(j,3)).NE.0) THEN
                                    SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(j,3)
                                END IF
                            END IF
                        END DO
                    END IF
                ELSE
                    !If the employee has dependent family
                    IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(1,1)
                            TYPE_SICK = 1
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(2,1)
                            TYPE_SICK = 2
                        END IF
                    !If employee has no family charge but lost his sole source of income
                    ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPRIV_QT(pers_j).EQ.1) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(3,1)
                            TYPE_SICK = 3
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(4,1)
                            TYPE_SICK = 4
                        END IF
                    !If employee has no family charge and did not loose his sole source of income
                    ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPRIV_QT(pers_j).EQ.0) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(5,1)
                            TYPE_SICK = 5
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(6,1)
                            TYPE_SICK = 6
                        END IF
                    END IF
                    !correct minima and maxima
                    IF (TYPE_SICK.LE.6) THEN
                        !loop over different disability types
                        DO j=1,6
                            IF (TYPE_SICK.EQ.j) THEN
                                !Correct minimum
                                IF (SICK_BETHPRIV_DAY(pers_j).LT.SICK_THIRPRIV_DAY(j,2)) THEN
                                    SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(j,2)
                                END IF
                                !Correct maximum
                                IF (SICK_BETHPRIV_DAY(pers_j).GT.SICK_THIRPRIV_DAY(j,3).AND.&
                                &   INT(100.D0*SICK_THIRPRIV_DAY(j,3)).NE.0) THEN
                                    SICK_BETHPRIV_DAY(pers_j) = SICK_THIRPRIV_DAY(j,3)
                                END IF
                            END IF
                        END DO
                    END IF
                END IF                    
                !Add the daily amount to the quarterly amount
                IF (i.LT.WEEK_LOOP) THEN
                    SICK_BETHPRIV_QT(pers_j) = SICK_BETHPRIV_QT(pers_j) + SICK_BETHPRIV_DAY(pers_j)*6.D0
                    !distinguish disability and primary disablement for comparison with exertnal sources
                    IF (ANY(TYPE_SICK.EQ.(/2,4,6/))) THEN
                        SICK_DISEMPNM_QT(pers_j,2) = SICK_DISEMPNM_QT(pers_j,2) + SICK_BETHPRIV_DAY(pers_j)*6.D0
                    ELSE IF (ANY(TYPE_SICK.EQ.(/1,3,5/))) THEN
                        SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BETHPRIV_DAY(pers_j)*6.D0
                    END IF
                ELSE IF (i.EQ.WEEK_LOOP) THEN
                    DAYS_REST = CEILING(DBLE(SICK_DAYSPRI1_QT(pers_j)) - DBLE(WEEK_LOOP-1)*6.D0)
                    IF (DAYS_REST.LT.0) THEN
                        DAYS_REST = 0
                    END IF 
                    SICK_BETHPRIV_QT(pers_j) = SICK_BETHPRIV_QT(pers_j) + SICK_BETHPRIV_DAY(pers_j)*DBLE(DAYS_REST)
                    !distinguish disability and primary disablement for comparison with exertnal sources
                    IF (ANY(TYPE_SICK.EQ.(/2,4,6/))) THEN
                        SICK_DISEMPNM_QT(pers_j,2) = SICK_DISEMPNM_QT(pers_j,2) + SICK_BETHPRIV_DAY(pers_j)*DBLE(DAYS_REST)
                    ELSE IF (ANY(TYPE_SICK.EQ.(/1,3,5/))) THEN
                        SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BETHPRIV_DAY(pers_j)*DBLE(DAYS_REST)
                    END IF
                END IF
            END DO
        END IF
    END IF

    !Maternity benefits
    !SICK_BENPRI2_DAY: Maternity benefit per day for individual insured as wage earner on the private labour market            
    !SICK_BENPRI2_QT: Maternity benefit per quarter for individual insured as wage earner on the private labour market            
    IF (ANY(SICK_ELIGPRI2_QT(pers_j).EQ.(/1, 2/))) THEN
        !Compute the limited daily wage
        IF (SICK_WOWEPRIV_QT(pers_j).EQ.1) THEN
            LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(1))
        ELSE IF (SICK_WOWEPRIV_QT(pers_j).EQ.2) THEN
            LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(2))
        END IF
        !Distinguish cases on the basis of the number of days already consumed
        IF (SICK_MATLPRI2_QT(pers_j).LT.31) THEN
            !Compute the first daily benefit
            !If the person, eligible for maternity benefits has a labour market contract
            IF (INT(100.D0*CONTRIB_REVPRIV_QT(pers_j)).NE.0) THEN
                SICK_BENPRI2_DAY(pers_j) = (SICK_MATPRIV_DAY(3)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
            !If the person, eligible for maternity benefits is unemployed
            ELSE IF (INT(100.D0*CONTRIB_REVPRIV_QT(pers_j)).EQ.0.AND.INT(100.D0*UNEM_BENUN_QT(pers_j)).NE.0) THEN
                SICK_BENPRI2_DAY(pers_j) = (SICK_MATPRIV_DAY(5)/100.D0)*LDW
            !In other cases follow the general rule
            ELSE
                SICK_BENPRI2_DAY(pers_j) = (SICK_MATPRIV_DAY(1)/100.D0)*LDW
            END IF
            !Compute the quarterly benefits with the first daily value
            SICK_BENPRI2_QT(pers_j) = SICK_BENPRI2_DAY(pers_j)*(DBLE(31)-DBLE(SICK_MATLPRI2_QT(pers_j)))
            !Compute the second daily benefit
            !If the person, eligible for maternity benefits has a labour market contract
            IF (INT(100.D0*CONTRIB_REVPRIV_QT(pers_j)).NE.0) THEN
                SICK_BENPRI2_DAY(pers_j) = (SICK_MATPRIV_DAY(4)/100.D0)*LDW
            !If the person, eligible for maternity benefits is unemployed
            ELSE IF (INT(100.D0*CONTRIB_REVPRIV_QT(pers_j)).EQ.0.AND.INT(100.D0*UNEM_BENUN_QT(pers_j)).NE.0) THEN
                SICK_BENPRI2_DAY(pers_j) = (SICK_MATPRIV_DAY(6)/100.D0)*LDW
            !In other cases follow the general rule
            ELSE
                SICK_BENPRI2_DAY(pers_j) = (SICK_MATPRIV_DAY(2)/100.D0)*LDW
            END IF
            !Compute the quarterly benefits with the second daily value
            SICK_BENPRI2_QT(pers_j) = SICK_BENPRI2_QT(pers_j) + SICK_BENPRI2_DAY(pers_j)*(DBLE(45))
        ELSE
            !Compute the daily benefit
            !If the person, eligible for maternity benefits has a labour market contract
            IF (INT(100.D0*CONTRIB_REVPRIV_QT(pers_j)).NE.0) THEN
                SICK_BENPRI2_DAY(pers_j) = (SICK_MATPRIV_DAY(4)/100.D0)*LDW
            !If the person, eligible for maternity benefits is unemployed
            ELSE IF (INT(100.D0*CONTRIB_REVPRIV_QT(pers_j)).EQ.0.AND.INT(100.D0*UNEM_BENUN_QT(pers_j)).NE.0) THEN
                SICK_BENPRI2_DAY(pers_j) = (SICK_MATPRIV_DAY(6)/100.D0)*LDW
            !In other cases follow the general rule
            ELSE
                SICK_BENPRI2_DAY(pers_j) = (SICK_MATPRIV_DAY(2)/100.D0)*LDW
            END IF
            !Compute the quarterly benefits with the daily value
            SICK_BENPRI2_QT(pers_j) = SICK_BENPRI2_DAY(pers_j)*(DBLE(75)-DBLE(SICK_MATLPRI2_QT(pers_j)))
        END IF
!Aangevuld bij aanpassingen september 2011: later bijstellen bij grondigere testing
        IF (SICK_ELIGPRI2_QT(pers_j).EQ.1) THEN
            IF (NIC_BEDRAG_2008(pers_j).NE.0.D0) THEN
                SICK_BENPRI2_QT(pers_j) = NIC_BEDRAG_2008(pers_j)/4.D0
            END IF
        ELSE IF (SICK_ELIGPRI2_QT(pers_j).EQ.2) THEN
            IF (RIZIVINA_BEDRAG_MAT_2008(pers_j).NE.0.D0) THEN
                SICK_BENPRI2_QT(pers_j) = RIZIVINA_BEDRAG_MAT_2008(pers_j)/4.D0                
            END IF
        END IF
    END IF

    !Benefits in case of industrial accidents
    !SICK_BENPRI3_DAY: Benefit per day received in case of an industrial accident (individual is insured as wage earner on the private labour market)          
    !SICK_BENPRI3_QT: Benefit per quarter received in case of an industrial accident (individual is insured as wage earner on the private labour market)          
    !SICK_EMPBPRI3_QT: Part of the benefit per quarter received in case of an industrial accident and paid by the employer (individual is insured as wage earner on the private labour market)         
    IF (SICK_ELIGPRI3_QT(pers_j).EQ.1) THEN
        WEEK_LOOP = 0
        WEEK_START = 0
        LDW = 0
        DAYS_REST = 0
        !Determine the number of weeks in benefit for industrial accident in the given quarter
        WEEK_LOOP = CEILING(DBLE(SICK_DAYSPRI3_QT(pers_j))/7.D0)
        IF (WEEK_LOOP.GT.13) THEN
            WEEK_LOOP = 13
        END IF
        !Determine the first week in benefit for industrial accident in the given quarter
        WEEK_START = SICK_WEEKPRI3_QT(pers_j) - WEEK_LOOP  
        IF (WEEK_START.LT.0) THEN
            WEEK_START = 0
        ELSE IF (WEEK_START.EQ.0) THEN
            WEEK_START = 1
        END IF
        IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.0) THEN
            !Compute the limited daily wage
            LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_INDACCLW_DAY)
            !Start loop to compute daily and quarterly benefits
            DO i=1,WEEK_LOOP
                !distinguish the relevant cases for the computation of the daily amount
                !daily amount before correcting minima and maxima
                TYPE_SICK = 0
                !If the employee is in temporary full disablement
                IF (SICK_TYDBPRI3_QT(pers_j).EQ.1) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPRI3_DAY(pers_j) = (SICK_INDACCPC_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 1
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPRI3_DAY(pers_j) = (SICK_INDACCPC_DAY(2,1)/100.D0)*LDW
                                TYPE_SICK = 2
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2) THEN
                            SICK_BENPRI3_DAY(pers_j) = (SICK_INDACCPC_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 3
                        END IF
                    !From the second month of sickness
                    ELSE IF ((WEEK_START+i-1).GT.4) THEN
                        SICK_BENPRI3_DAY(pers_j) = (SICK_INDACCPC_DAY(4,1)/100.D0)*LDW
                        TYPE_SICK = 4
                    END IF
                !If the employee is in temporary partial disablement
                ELSE IF (SICK_TYDBPRI3_QT(pers_j).EQ.2) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPRI3_DAY(pers_j) = (SICK_INDACCPC_DAY(5,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 5
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPRI3_DAY(pers_j) = (SICK_INDACCPC_DAY(6,1)/100.D0)*LDW
                                TYPE_SICK = 6
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2) THEN
                            SICK_BENPRI3_DAY(pers_j) = (SICK_INDACCPC_DAY(7,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 7
                        END IF
                    !From the second month of sickness
                    ELSE IF ((WEEK_START+i-1).GT.4) THEN
                        SICK_BENPRI3_DAY(pers_j) = (SICK_INDACCPC_DAY(8,1)/100.D0)*LDW
                        TYPE_SICK = 8
                    END IF
                !If the employee is in permanent disablement
                ELSE IF (SICK_TYDBPRI3_QT(pers_j).EQ.3) THEN
                    SICK_BENPRI3_DAY(pers_j) = (SICK_INDACCPC_DAY(9,1)/100.D0)*LDW*SICK_DEGRPRI3_QT(pers_j)
                    TYPE_SICK = 9
                END IF
                !correct minima and maxima
                IF (TYPE_SICK.LE.9) THEN
                    !loop over different disability types
                    DO j=1,9
                        IF (TYPE_SICK.EQ.j) THEN
                            !Correct minimum
                            IF (SICK_BENPRI3_DAY(pers_j).LT.SICK_INDACCPC_DAY(j,2)) THEN
                                SICK_BENPRI3_DAY(pers_j) = SICK_INDACCPC_DAY(j,2)
                            END IF
                            !Correct maximum
                            IF (SICK_BENPRI3_DAY(pers_j).GT.SICK_INDACCPC_DAY(j,3).AND.&
                            &   INT(100.D0*SICK_INDACCPC_DAY(j,3)).NE.0) THEN
                                SICK_BENPRI3_DAY(pers_j) = SICK_INDACCPC_DAY(j,3)
                            END IF
                        END IF
                    END DO
                END IF
                !Add the daily amount to the quarterly amount
                IF (i.LT.WEEK_LOOP) THEN
                    SICK_BENPRI3_QT(pers_j) = SICK_BENPRI3_QT(pers_j) + SICK_BENPRI3_DAY(pers_j)*7.D0
                ELSE IF (i.EQ.WEEK_LOOP) THEN
                    DAYS_REST = CEILING(DBLE(SICK_DAYSPRI3_QT(pers_j)) - DBLE(WEEK_LOOP-1)*7.D0)
                    IF (DAYS_REST.LT.0) THEN
                        DAYS_REST = 0
                    END IF 
                    SICK_BENPRI3_QT(pers_j) = SICK_BENPRI3_QT(pers_j) + SICK_BENPRI3_DAY(pers_j)*DBLE(DAYS_REST)
                END IF
                !Compute the amount paid by the employer
                !In case of a blue collar worker
                IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.AND.(WEEK_START+i-1).LE.1) THEN
                    SICK_EMPBPRI3_QT(pers_j) = SICK_BENPRI3_QT(pers_j)
                END IF
                !In case of white collar worker
                IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.AND.(WEEK_START+i-1).LE.4) THEN
                    SICK_EMPBPRI3_QT(pers_j) = SICK_BENPRI3_QT(pers_j)
                END IF
            END DO
        END IF
!Aangevuld bij aanpassingen september 2011: later bijstellen bij grondigere testing
        SumInc = MONTITT_2008(pers_j)/4.D0 + MONTITP_2008(pers_j)/4.D0
        IF (SumInc.NE.0.D0) THEN
            SICK_BENPRI3_QT(pers_j) = SumInc
        END IF
    END IF

    !Benefits in case of occupational diseases
    !SICK_BENPRI4_DAY: Benefit per day received in case of an occupational disease (individual is insured as wage earner on the private labour market)          
    !SICK_BENPRI4_QT: Benefit per quarter received in case of an occupational disease (individual is insured as wage earner on the private labour market)          
    !SICK_EMPBPRI4_QT: Part of the benefit per quarter received in case of an occupational disease and paid by the employer (individual is insured as wage earner on the private labour market)         
    IF (SICK_ELIGPRI4_QT(pers_j).EQ.1) THEN
        WEEK_LOOP = 0
        WEEK_START = 0
        LDW = 0
        DAYS_REST = 0
        !Determine the number of weeks in benefit for industrial accident in the given quarter
        WEEK_LOOP = CEILING(DBLE(SICK_DAYSPRI4_QT(pers_j))/7.D0)
        IF (WEEK_LOOP.GT.13) THEN
            WEEK_LOOP = 13
        END IF
        !Determine the first week in benefit for industrial accident in the given quarter
        WEEK_START = SICK_WEEKPRI4_QT(pers_j) - WEEK_LOOP  
        IF (WEEK_START.LT.0) THEN
            WEEK_START = 0
        ELSE IF (WEEK_START.EQ.0) THEN
            WEEK_START = 1
        END IF
        IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.0) THEN
            !Compute the limited daily wage
            LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_OCCDISLW_DAY)
            !Start loop to compute daily and quarterly benefits
            DO i=1,WEEK_LOOP
                !distinguish the relevant cases for the computation of the daily amount
                !daily amount before correcting minima and maxima
                TYPE_SICK = 0
                !If the employee is in temporary full disablement
                IF (SICK_TYDBPRI4_QT(pers_j).EQ.1) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPRI4_DAY(pers_j) = (SICK_OCCDISPC_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 1
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPRI4_DAY(pers_j) = (SICK_OCCDISPC_DAY(2,1)/100.D0)*LDW
                                TYPE_SICK = 2
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2) THEN
                            SICK_BENPRI4_DAY(pers_j) = (SICK_OCCDISPC_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 3
                        END IF
                    !From the second month of sickness
                    ELSE IF ((WEEK_START+i-1).GT.4) THEN
                        SICK_BENPRI4_DAY(pers_j) = (SICK_OCCDISPC_DAY(4,1)/100.D0)*LDW
                        TYPE_SICK = 4
                    END IF
                !If the employee is in temporary partial disablement
                ELSE IF (SICK_TYDBPRI4_QT(pers_j).EQ.2) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPRI4_DAY(pers_j) = (SICK_OCCDISPC_DAY(5,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 5
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPRI4_DAY(pers_j) = (SICK_OCCDISPC_DAY(6,1)/100.D0)*LDW
                                TYPE_SICK = 6
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2) THEN
                            SICK_BENPRI4_DAY(pers_j) = (SICK_OCCDISPC_DAY(7,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 7
                        END IF
                    !From the second month of sickness
                    ELSE IF ((WEEK_START+i-1).GT.4) THEN
                        SICK_BENPRI4_DAY(pers_j) = (SICK_OCCDISPC_DAY(8,1)/100.D0)*LDW
                        TYPE_SICK = 8
                    END IF
                !If the employee is in permanent disablement
                ELSE IF (SICK_TYDBPRI4_QT(pers_j).EQ.3) THEN
                    SICK_BENPRI4_DAY(pers_j) = (SICK_OCCDISPC_DAY(9,1)/100.D0)*LDW*SICK_DEGRPRI4_QT(pers_j)
                    TYPE_SICK = 9
                END IF
                !correct minima and maxima
                IF (TYPE_SICK.LE.9) THEN
                    !loop over different disability types
                    DO j=1,9
                        IF (TYPE_SICK.EQ.j) THEN
                            !Correct minimum
                            IF (SICK_BENPRI4_DAY(pers_j).LT.SICK_OCCDISPC_DAY(j,2)) THEN
                                SICK_BENPRI4_DAY(pers_j) = SICK_OCCDISPC_DAY(j,2)
                            END IF
                            !Correct maximum
                            IF (SICK_BENPRI4_DAY(pers_j).GT.SICK_OCCDISPC_DAY(j,3).AND.&
                            &   INT(100.D0*SICK_OCCDISPC_DAY(j,3)).NE.0) THEN
                                SICK_BENPRI4_DAY(pers_j) = SICK_OCCDISPC_DAY(j,3)
                            END IF
                        END IF
                    END DO
                END IF
                !Add the daily amount to the quarterly amount
                IF (i.LT.WEEK_LOOP) THEN
                    SICK_BENPRI4_QT(pers_j) = SICK_BENPRI4_QT(pers_j) + SICK_BENPRI4_DAY(pers_j)*7.D0
                ELSE IF (i.EQ.WEEK_LOOP) THEN
                    DAYS_REST = CEILING(DBLE(SICK_DAYSPRI4_QT(pers_j)) - DBLE(WEEK_LOOP-1)*7.D0)
                    IF (DAYS_REST.LT.0) THEN
                        DAYS_REST = 0
                    END IF 
                    SICK_BENPRI4_QT(pers_j) = SICK_BENPRI4_QT(pers_j) + SICK_BENPRI4_DAY(pers_j)*DBLE(DAYS_REST)
                END IF
                !Compute the amount paid by the employer
                !In case of a blue collar worker
                IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.AND.(WEEK_START+i-1).LE.1) THEN
                    SICK_EMPBPRI4_QT(pers_j) = SICK_BENPRI4_QT(pers_j)
                END IF
                !In case of white collar worker
                IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.AND.(WEEK_START+i-1).LE.4) THEN
                    SICK_EMPBPRI4_QT(pers_j) = SICK_BENPRI4_QT(pers_j)
                END IF
            END DO
        END IF
!Aangevuld bij aanpassingen september 2011: later bijstellen bij grondigere testing
        IF (FBZ_BEDRAG_2008(pers_j).NE.0.D0) THEN
            SICK_BENPRI4_QT(pers_j) = FBZ_BEDRAG_2008(pers_j)/4.D0
        END IF
    END IF

END SUBROUTINE SICK_BENPRIV_CONSTRUC

!==========================================================!
!With the subroutine SICK_BENPUBNS_CONSTRUC we compute     !
!the sickness and disability benefits for those insured    !
!as wage earner on the public labour market WITHOUT        ! 
!statutory service                                         ! 
!Variables constructed in the routine                      !
!Sickness and disability benefits                          !
!   SICK_BENPUB1_DAY                                       !
!   SICK_BENPUB1_QT                                        !
!   SICK_EMPBPUB1_QT                                       !
!   SICK_SUPPUB1_DAY                                       !
!   SICK_SUPPUB1_QT                                        !
!   SICK_EMPSPUB1_QT                                       !
!   SICK_BETHPUB_DAY                                       !
!   SICK_BETHPUB_QT                                        !
!Maternity benefits                                        !
!   SICK_BENPUB2_DAY                                       !
!   SICK_BENPUB2_QT                                        !
!Industrial accidents                                      !
!   SICK_BENPUB3_DAY                                       !
!   SICK_BENPUB3_QT                                        !
!   SICK_EMPBPUB3_QT                                       !
!Occupational diseases                                     !
!   SICK_BENPUB4_DAY                                       !
!   SICK_BENPUB4_QT                                        !
!   SICK_EMPBPUB4_QT                                       !
!==========================================================!

SUBROUTINE SICK_BENPUBNS_CONSTRUC(pers_j)

    INTEGER                                 :: pers_j,i,j
    INTEGER                                 :: WEEK_LOOP            !Number of weeks for which the loop with which the benefits are computed should run
    INTEGER                                 :: WEEK_START           !First week for which a benefit is computed
    REAL(KIND=KIND_REAL)                    :: LDW                  !Variable to store the limited daily wage (value selected conditional on working week regime)  
    INTEGER                                 :: TYPE_SICK            !Identifier to identify the type of sickness or disability benefit (used to reduce the check on minima and maxima)
    INTEGER                                 :: DAYS_REST            !Remaining number of days in the last week for which daily benefits are computed
    REAL(KIND=KIND_REAL),DIMENSION(2)       :: LDW_BLUE             !Variable to store the wage of blue collar workers above or below a given threshold
    REAL(KIND=KIND_REAL)                    :: SICK_GTINC_ANN       !Gross taxable income per year           
    REAL(KIND=KIND_REAL)                    :: SICK_PREPAY_DAY      !daily amount of prepayments
    REAL(KIND=KIND_REAL)                    :: SICK_BASE_DAY        !base to compute allowed cumulation between sick benefits and labour income for system before 2003
    REAL(KIND=KIND_REAL)                    :: SumInc               !sum income components

    !Sickness and disability benefits
    !Compute regular benefits:
    !SICK_BENPUB1_DAY: Daily benefit for sickness or disability for individual in the wage earner regime of the public labour market         
    !SICK_BENPUB1_QT:  Quarterly benefit for sickness or disability for individual in the wage earner regime of the public labour market         
    !SICK_EMPBPUB1_QT: Part of the quarterly benefit for sickness or disability paid by the employer for individual insured as wage earner on the public labour market            
    IF (ANY(SICK_ELIGPUB1_QT(pers_j).EQ.(/1, 2/))) THEN
        WEEK_LOOP = 0
        WEEK_START = 0
        LDW = 0
        DAYS_REST = 0
        !Determine the number of weeks in sickness or disability in the given quarter
        WEEK_LOOP = CEILING(DBLE(SICK_DAYSPUB1_QT(pers_j))/6.D0)
        IF (WEEK_LOOP.GT.13) THEN
            WEEK_LOOP = 13
        END IF
        !Determine the first week in sickness or disability in the given quarter
        WEEK_START = SICK_WEEKPUB1_QT(pers_j) - WEEK_LOOP  
        IF (WEEK_START.LT.0) THEN
            WEEK_START = 0
        ELSE IF (WEEK_START.EQ.0) THEN
            WEEK_START = 1
        END IF
        IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.0) THEN
            !Compute the limited daily wage
            IF (SICK_WOWEPUB_QT(pers_j).EQ.1) THEN
                LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(1))
            ELSE IF (SICK_WOWEPUB_QT(pers_j).EQ.2) THEN
                LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(2))
            END IF
            !Start loop to compute daily and quarterly benefits
            DO i=1,WEEK_LOOP
                !distinguish the relevant cases for the computation of the daily amount on the family charge of the sick or disabled
                !daily amount before correcting minima and maxima
                TYPE_SICK = 0
                SICK_BASE_DAY = 0.D0
                !If the employee has dependent family
                IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 1
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(2,1)/100.D0)*LDW
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(2,1)/100.D0)*LDW
                                TYPE_SICK = 2
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
                            SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            SICK_BASE_DAY = (SICK_PERCPRIV_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 3
                        END IF
                    !In month 2 till 6
                    ELSE IF ((WEEK_START+i-1).GT.4.AND.(WEEK_START+i-1).LE.26) THEN
                        SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(4,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(4,1)/100.D0)*LDW
                        TYPE_SICK = 4
                    !In month 7 till 12
                    ELSE IF ((WEEK_START+i-1).GT.26.AND.(WEEK_START+i-1).LT.53) THEN
                        SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(5,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(5,1)/100.D0)*LDW
                        TYPE_SICK = 5
                    !If in disability
                    ELSE IF ((WEEK_START+i-1).GE.53) THEN
                        SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(6,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(6,1)/100.D0)*LDW
                        TYPE_SICK = 6
                    END IF
                !If employee has no family charge but lost his sole source of income
                ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPUB_QT(pers_j).EQ.1) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(7,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 7
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(8,1)/100.D0)*LDW
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(2,1)/100.D0)*LDW
                                TYPE_SICK = 8
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
                            SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(9,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            SICK_BASE_DAY = (SICK_PERCPRIV_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 9
                        END IF
                    !In month 2 till 6
                    ELSE IF ((WEEK_START+i-1).GT.4.AND.(WEEK_START+i-1).LE.26) THEN
                        SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(10,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(4,1)/100.D0)*LDW
                        TYPE_SICK = 10
                    !In month 7 till 12
                    ELSE IF ((WEEK_START+i-1).GT.26.AND.(WEEK_START+i-1).LT.53) THEN
                        SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(11,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(5,1)/100.D0)*LDW
                        TYPE_SICK = 11
                    !If in disability
                    ELSE IF ((WEEK_START+i-1).GE.53) THEN
                        SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(12,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(6,1)/100.D0)*LDW
                        TYPE_SICK = 12
                    END IF
                !If employee has no family charge and did not loose his sole source of income
                ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPUB_QT(pers_j).EQ.0) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(13,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 13
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(14,1)/100.D0)*LDW
                                SICK_BASE_DAY = (SICK_PERCPRIV_DAY(2,1)/100.D0)*LDW
                                TYPE_SICK = 14
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
                            SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(15,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            SICK_BASE_DAY = (SICK_PERCPRIV_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 15
                        END IF
                    !In month 2 till 6
                    ELSE IF ((WEEK_START+i-1).GT.4.AND.(WEEK_START+i-1).LE.26) THEN
                        SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(16,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(4,1)/100.D0)*LDW
                        TYPE_SICK = 16
                    !In month 7 till 12
                    ELSE IF ((WEEK_START+i-1).GT.26.AND.(WEEK_START+i-1).LT.53) THEN
                        SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(17,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(5,1)/100.D0)*LDW
                        TYPE_SICK = 17
                    !If in disability
                    ELSE IF ((WEEK_START+i-1).GE.53) THEN
                        SICK_BENPUB1_DAY(pers_j) = (SICK_PERCPRIV_DAY(18,1)/100.D0)*LDW
                        SICK_BASE_DAY = (SICK_PERCPRIV_DAY(6,1)/100.D0)*LDW
                        TYPE_SICK = 18
                    END IF
                END IF
                !correct minima and maxima
                IF (TYPE_SICK.LE.18) THEN
                    !loop over different disability types
                    DO j=1,18
                        IF (TYPE_SICK.EQ.j) THEN
                            !Correct minimum
                            !If a regular worker
                            IF (SICK_REGWPUB_QT(pers_j).EQ.1) THEN
                                IF (SICK_BENPUB1_DAY(pers_j).LT.SICK_PERCPRIV_DAY(j,3)) THEN
                                    SICK_BENPUB1_DAY(pers_j) = SICK_PERCPRIV_DAY(j,3)
                                END IF
                            !If an irregular worker    
                            ELSE IF (SICK_REGWPUB_QT(pers_j).EQ.2) THEN
                                IF (SICK_BENPUB1_DAY(pers_j).LT.SICK_PERCPRIV_DAY(j,2)) THEN
                                    SICK_BENPUB1_DAY(pers_j) = SICK_PERCPRIV_DAY(j,2)
                                END IF
                            END IF
                            !Correct maximum
                            IF (SICK_BENPUB1_DAY(pers_j).GT.SICK_PERCPRIV_DAY(j,4).AND.&
                            &   INT(100.D0*SICK_PERCPRIV_DAY(j,4)).NE.0) THEN
                                SICK_BENPUB1_DAY(pers_j) = SICK_PERCPRIV_DAY(j,4)
                            END IF
                        END IF
                        !Corrections for SICK_BASE_DAY
                        IF (TYPE_SICK.EQ.j.AND.TYPE_SICK.LT.7) THEN
                            !Correct minimum
                            !If a regular worker
                            IF (SICK_REGWPRIV_QT(pers_j).EQ.1) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j,3)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j,3)
                                END IF
                            !If an irregular worker    
                            ELSE IF (SICK_REGWPRIV_QT(pers_j).EQ.2) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j,2)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j,2)
                                END IF
                            END IF
                            !Correct maximum
                            IF (SICK_BASE_DAY.GT.SICK_PERCPRIV_DAY(j,4).AND.&
                            &   INT(100.D0*SICK_PERCPRIV_DAY(j,4)).NE.0) THEN
                                SICK_BASE_DAY = SICK_PERCPRIV_DAY(j,4)
                            END IF
                        ELSE IF (TYPE_SICK.EQ.j.AND.TYPE_SICK.GE.7.AND.TYPE_SICK.LT.13) THEN
                            !Correct minimum
                            !If a regular worker
                            IF (SICK_REGWPRIV_QT(pers_j).EQ.1) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j-6,3)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-6,3)
                                END IF
                            !If an irregular worker    
                            ELSE IF (SICK_REGWPRIV_QT(pers_j).EQ.2) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j-6,2)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-6,2)
                                END IF
                            END IF
                            !Correct maximum
                            IF (SICK_BASE_DAY.GT.SICK_PERCPRIV_DAY(j-6,4).AND.&
                            &   INT(100.D0*SICK_PERCPRIV_DAY(j-6,4)).NE.0) THEN
                                SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-6,4)
                            END IF
                        ELSE IF (TYPE_SICK.EQ.j.AND.TYPE_SICK.GE.13) THEN
                            !Correct minimum
                            !If a regular worker
                            IF (SICK_REGWPRIV_QT(pers_j).EQ.1) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j-12,3)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-12,3)
                                END IF
                            !If an irregular worker    
                            ELSE IF (SICK_REGWPRIV_QT(pers_j).EQ.2) THEN
                                IF (SICK_BASE_DAY.LT.SICK_PERCPRIV_DAY(j-12,2)) THEN
                                    SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-12,2)
                                END IF
                            END IF
                            !Correct maximum
                            IF (SICK_BASE_DAY.GT.SICK_PERCPRIV_DAY(j-12,4).AND.&
                            &   INT(100.D0*SICK_PERCPRIV_DAY(j-12,4)).NE.0) THEN
                                SICK_BASE_DAY = SICK_PERCPRIV_DAY(j-12,4)
                            END IF
                        END IF
                    END DO
                END IF
                !Correct the daily amount when labour income is simulated 
                IF (MIMOSIS_SIMNUMMT(pers_j).NE.0) THEN
                    IF (SIMYEAR.LT.2003) THEN
                        IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN
                            SICK_BENPUB1_DAY(pers_j) = (SICK_BASE_DAY)*(SICK_LABCUMUL_DAY(1)/100.D0) - SICK_LABDEDUC_DAY(pers_j)
                        ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/))) THEN
                            SICK_BENPUB1_DAY(pers_j) = (SICK_BASE_DAY)*(SICK_LABCUMUL_DAY(2)/100.D0) - SICK_LABDEDUC_DAY(pers_j)
                        END IF  
                    ELSE
                        SICK_BENPUB1_DAY(pers_j) = SICK_BENPUB1_DAY(pers_j) - SICK_LABDEDUC_DAY(pers_j)
                    END IF
                    !Correct if negative
                    IF (SICK_BENPUB1_DAY(pers_j).LT.0.D0) THEN
                        SICK_BENPUB1_DAY(pers_j) = 0.D0
                    END IF
                END IF
                !Add the daily amount to the quarterly amount
                IF (i.LT.WEEK_LOOP) THEN
                    SICK_BENPUB1_QT(pers_j) = SICK_BENPUB1_QT(pers_j) + SICK_BENPUB1_DAY(pers_j)*6.D0
                    !distinguish disability and primary disablement for comparison with exertnal sources
                    IF (ANY(TYPE_SICK.EQ.(/6,12,18/))) THEN
                        SICK_DISEMPNM_QT(pers_j,1) = SICK_DISEMPNM_QT(pers_j,1) + 6.D0
!                        SICK_DISEMPNM_QT(pers_j,2) = SICK_DISEMPNM_QT(pers_j,2) + SICK_BENPUB1_DAY(pers_j)*6.D0
                    ELSE IF (ANY(TYPE_SICK.EQ.(/1,2,3,4,5,7,8,9,10,11,13,14,15,16,17/))) THEN
                        !In case of a blue collar worker
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1.AND.(WEEK_START+i-1).GT.2) THEN
                            SICK_SICEMPNM_QT(pers_j,1) = SICK_SICEMPNM_QT(pers_j,1) + 6.D0
!                            SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BENPUB1_DAY(pers_j)*6.D0
                        END IF
                        !In case of white collar worker
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2.AND.(WEEK_START+i-1).GT.4) THEN
                            SICK_SICEMPNM_QT(pers_j,1) = SICK_SICEMPNM_QT(pers_j,1) + 6.D0
!                            SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BENPUB1_DAY(pers_j)*6.D0
                        END IF
                    END IF
                ELSE IF (i.EQ.WEEK_LOOP) THEN
                    DAYS_REST = CEILING(DBLE(SICK_DAYSPUB1_QT(pers_j)) - DBLE(WEEK_LOOP-1)*6.D0)
                    IF (DAYS_REST.LT.0) THEN
                        DAYS_REST = 0
                    END IF 
                    SICK_BENPUB1_QT(pers_j) = SICK_BENPUB1_QT(pers_j) + SICK_BENPUB1_DAY(pers_j)*DBLE(DAYS_REST)
                    !distinguish disability and primary disablement for comparison with exertnal sources
                    IF (ANY(TYPE_SICK.EQ.(/6,12,18/))) THEN
                        SICK_DISEMPNM_QT(pers_j,1) = SICK_DISEMPNM_QT(pers_j,1) + DBLE(DAYS_REST)
!                        SICK_DISEMPNM_QT(pers_j,2) = SICK_DISEMPNM_QT(pers_j,2) + SICK_BENPUB1_DAY(pers_j)*DBLE(DAYS_REST)
                    ELSE IF (ANY(TYPE_SICK.EQ.(/1,2,3,4,5,7,8,9,10,11,13,14,15,16,17/))) THEN
                        !In case of a blue collar worker
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1.AND.(WEEK_START+i-1).GT.2) THEN
                            SICK_SICEMPNM_QT(pers_j,1) = SICK_SICEMPNM_QT(pers_j,1) + DBLE(DAYS_REST)
!                            SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BENPUB1_DAY(pers_j)*DBLE(DAYS_REST)
                        END IF
                        !In case of white collar worker
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2.AND.(WEEK_START+i-1).GT.4) THEN
                            SICK_SICEMPNM_QT(pers_j,1) = SICK_SICEMPNM_QT(pers_j,1) + DBLE(DAYS_REST)
!                            SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BENPUB1_DAY(pers_j)*DBLE(DAYS_REST)
                        END IF
                    END IF
                END IF
                !Compute the amount paid by the employer
                !In case of a blue collar worker
                IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1.AND.(WEEK_START+i-1).LE.2) THEN
                    SICK_EMPBPUB1_QT(pers_j) = SICK_BENPUB1_QT(pers_j)
                END IF
                !In case of white collar worker
                IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2.AND.(WEEK_START+i-1).LE.4) THEN
                    SICK_EMPBPUB1_QT(pers_j) = SICK_BENPUB1_QT(pers_j)
                END IF
            END DO
        END IF
!Aangevuld bij aanpassingen september 2011: later bijstellen bij grondigere testing
        IF (ANY(SICK_ELIGPUB1_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (NIC_BEDRAG_2008(pers_j).NE.0.D0) THEN
                SICK_BENPUB1_QT(pers_j) = NIC_BEDRAG_2008(pers_j)/4.D0
                SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BENPUB1_QT(pers_j)
            END IF
        END IF
        IF (SICK_ELIGPUB1_QT(pers_j).EQ.2) THEN
            IF (RIZIVINA_BEDRAG_INVAL_2008(pers_j).NE.0.D0) THEN
                SICK_BENPUB1_QT(pers_j) = RIZIVINA_BEDRAG_INVAL_2008(pers_j)/4.D0                
                SICK_DISEMPNM_QT(pers_j,2) = SICK_DISEMPNM_QT(pers_j,2) + SICK_BENPUB1_QT(pers_j)
            END IF
        END IF
    END IF

    !Compute supplementary benefit for blue collar workers
    !SICK_SUPPUB1_DAY: Additional daily supplement for blue collar workers insured as wage earner on the public labour market         
    !SICK_SUPPUB1_QT:  Additional quarterly supplement for blue collar worker who is insured as wage earner on the public labour market           
    !SICK_EMPSPUB1_QT: Part of the additional quarterly supplement paid by the employer for blue collar worker who is insured as wage earner on the public labour market          
    IF (SICK_ELIGPUB1_QT(pers_j).EQ.1.AND.SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
        IF (ANY(SICK_WEEKPUB1_QT(pers_j).EQ.(/2, 3, 4/))) THEN
            LDW_BLUE(1:2) = 0.D0
            WEEK_LOOP = 0
            WEEK_START = 0
            SICK_GTINC_ANN = 0
            SICK_PREPAY_DAY = 0
            !Determine the number of weeks in sickness or disability in the given quarter
            WEEK_LOOP = CEILING(DBLE(SICK_DAYSPUB1_QT(pers_j))/6.D0)
            IF (WEEK_LOOP.GT.4) THEN
                WEEK_LOOP = 4
            END IF
            !Determine the first week in sickness or disability in the given quarter
            WEEK_START = SICK_WEEKPUB1_QT(pers_j) - WEEK_LOOP  
            IF (WEEK_START.LT.0) THEN
                WEEK_START = 0
            ELSE IF (WEEK_START.EQ.0) THEN
                WEEK_START = 1
            END IF
            !start computation only from the second week on
            IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.1) THEN
                !Compute the daily wage above and below the daily wage threshold
                IF (SICK_LDAYWAGE_DAY(pers_j).LT.SICK_LIMPRIV_DAY(1)) THEN
                    LDW_BLUE(1) = SICK_LDAYWAGE_DAY(pers_j) 
                    LDW_BLUE(2) = 0                    
                ELSE 
                    LDW_BLUE(1) = SICK_LIMPRIV_DAY(1)
                    LDW_BLUE(2) = SICK_LDAYWAGE_DAY(pers_j) - SICK_LIMPRIV_DAY(1)                  
                END IF
                !Compute the prepayments per day
                !First compute annual income
                SICK_GTINC_ANN = SICK_LDAYWAGE_DAY(pers_j)*260.D0 - &
                &(SICK_SSCONT_QT/100.D0)*SICK_LDAYWAGE_DAY(pers_j)*260.D0
                !Then use the function SICK_PREPAY_COMPUTE to compute prepayments per day paid on this income
                SICK_PREPAY_DAY = SICK_PREPAY_COMPUTE(SICK_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),&
                &GECODEERD_INSZ_INPUT(pers_j))
                !Compute the daily benefit
                SICK_SUPPUB1_DAY(pers_j) =  &
                                            &(SICK_SUPPRIV_DAY(1)/100.D0)*LDW_BLUE(1) + &
                                            &(SICK_SUPPRIV_DAY(2)/100.D0)*LDW_BLUE(2) - &
                                            &SICK_PREPAY_DAY
                IF (SICK_SUPPUB1_DAY(pers_j).LT.0) THEN
                    SICK_SUPPUB1_DAY(pers_j) = 0
                END IF
                !Start loop to compute quarterly benefits: only count from second week on
                DO i=2,WEEK_LOOP
                    !Add the daily amount to the quarterly amount
                    IF (i.LT.WEEK_LOOP) THEN
                        SICK_SUPPUB1_QT(pers_j) = SICK_SUPPUB1_QT(pers_j) + SICK_SUPPUB1_DAY(pers_j)*6.D0
                    ELSE IF (i.EQ.WEEK_LOOP) THEN
                        DAYS_REST = CEILING(DBLE(SICK_DAYSPUB1_QT(pers_j)) - DBLE(WEEK_LOOP-1)*6.D0)
                        IF (DAYS_REST.LT.0) THEN
                            DAYS_REST = 0
                        END IF 
                        SICK_SUPPUB1_QT(pers_j) = SICK_SUPPUB1_QT(pers_j) + SICK_SUPPUB1_DAY(pers_j)*DBLE(DAYS_REST)
                    END IF
                    !Compute the amount paid by the employer
                    IF ((WEEK_START+i-1).EQ.2) THEN
                        SICK_EMPSPUB1_QT(pers_j) = SICK_SUPPUB1_QT(pers_j)
                    END IF
                END DO
            END IF    
        END IF
        !add supplement to the quarterly benefit
        SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_SUPPUB1_QT(pers_j) - SICK_EMPSPUB1_QT(pers_j)
!        IF (SICK_SUPPRI1_QT(pers_j).LT.SICK_EMPSPRI1_QT(pers_j)) THEN
!            WRITE(15,*) 'TestB',SICK_SUPPRI1_QT(pers_j),SICK_EMPSPRI1_QT(pers_j)
!        END IF
    END IF

    !Compute supplementary benefit for help of a third person
    !SICK_BETHPUB_DAY: Daily supplement for help of a third person (individual is insured as wage earner on the public labour market)         
    !SICK_BETHPUB_QT:  Quarterly supplement for help of a third person (individual is insured as wage earner on the public labour market)         
    IF (SICK_ELTHPUB_QT(pers_j).EQ.1) THEN
        WEEK_LOOP = 0
        WEEK_START = 0
        LDW = 0
        DAYS_REST = 0
        !Determine the number of weeks in sickness or disability in the given quarter
        WEEK_LOOP = CEILING(DBLE(SICK_DAYSPUB1_QT(pers_j))/6.D0)
        IF (WEEK_LOOP.GT.13) THEN
            WEEK_LOOP = 13
        END IF
        !Determine the first week in sickness or disability in the given quarter
        WEEK_START = SICK_WEEKPUB1_QT(pers_j) - WEEK_LOOP  
        IF (WEEK_START.LT.0) THEN
            WEEK_START = 0
        ELSE IF (WEEK_START.EQ.0) THEN
            WEEK_START = 1
        END IF
        IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.0) THEN
            !Compute the limited daily wage
            IF (SICK_WOWEPUB_QT(pers_j).EQ.1) THEN
                LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(1))
            ELSE IF (SICK_WOWEPUB_QT(pers_j).EQ.2) THEN
                LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(2))
            END IF
            !Start loop to compute daily and quarterly benefits
            DO i=1,WEEK_LOOP
                !distinguish the relevant cases for the computation of the daily amount on the family charge of the sick or disabled
                !daily amount before correcting minima and maxima
                TYPE_SICK = 0
                !Distinguish way of computing help for third persons: system is different before and after 2007
                IF (SIMYEAR.LT.2007) THEN
                    !If the employee has dependent family
                    IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(1,1)
                            TYPE_SICK = 1
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(2,1)
                            TYPE_SICK = 2
                        END IF
                    !If employee has no family charge but lost his sole source of income
                    ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPRIV_QT(pers_j).EQ.1) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(3,1)
                            TYPE_SICK = 3
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = (SICK_THIRPRIV_DAY(4,1)/100.D0)*LDW
                            TYPE_SICK = 4
                        END IF
                    !If employee has no family charge and did not loose his sole source of income
                    ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPRIV_QT(pers_j).EQ.0) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(5,1)
                            TYPE_SICK = 5
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = (SICK_THIRPRIV_DAY(6,1)/100.D0)*LDW
                            TYPE_SICK = 6
                        END IF
                    END IF
                    !correct minima and maxima
                    IF (TYPE_SICK.LE.6) THEN
                        !loop over different disability types
                        DO j=1,6
                            IF (TYPE_SICK.EQ.j) THEN
                                !Correct minimum
                                IF (SICK_BETHPUB_DAY(pers_j).LT.SICK_THIRPRIV_DAY(j,2)) THEN
                                    SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(j,2)
                                END IF
                                !Correct maximum
                                IF (SICK_BETHPUB_DAY(pers_j).GT.SICK_THIRPRIV_DAY(j,3).AND.&
                                &   INT(100.D0*SICK_THIRPRIV_DAY(j,3)).NE.0) THEN
                                    SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(j,3)
                                END IF
                            END IF
                        END DO
                    END IF
                ELSE
                    !If the employee has dependent family
                    IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(1,1)
                            TYPE_SICK = 1
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(2,1)
                            TYPE_SICK = 2
                        END IF
                    !If employee has no family charge but lost his sole source of income
                    ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPUB_QT(pers_j).EQ.1) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(3,1)
                            TYPE_SICK = 3
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = (SICK_THIRPRIV_DAY(4,1)/100.D0)*LDW
                            TYPE_SICK = 4
                        END IF
                    !If employee has no family charge and did not loose his sole source of income
                    ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/)).AND.SICK_SOLEPUB_QT(pers_j).EQ.0) THEN
                        !In month 4 till 12
                        IF ((WEEK_START+i-1).GT.13.AND.(WEEK_START+i-1).LT.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(5,1)
                            TYPE_SICK = 5
                        !If in disability
                        ELSE IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_BETHPUB_DAY(pers_j) = (SICK_THIRPRIV_DAY(6,1)/100.D0)*LDW
                            TYPE_SICK = 6
                        END IF
                    END IF
                    !correct minima and maxima
                    IF (TYPE_SICK.LE.6) THEN
                        !loop over different disability types
                        DO j=1,6
                            IF (TYPE_SICK.EQ.j) THEN
                                !Correct minimum
                                IF (SICK_BETHPUB_DAY(pers_j).LT.SICK_THIRPRIV_DAY(j,2)) THEN
                                    SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(j,2)
                                END IF
                                !Correct maximum
                                IF (SICK_BETHPUB_DAY(pers_j).GT.SICK_THIRPRIV_DAY(j,3).AND.&
                                &   INT(100.D0*SICK_THIRPRIV_DAY(j,3)).NE.0) THEN
                                    SICK_BETHPUB_DAY(pers_j) = SICK_THIRPRIV_DAY(j,3)
                                END IF
                            END IF
                        END DO
                    END IF
                END IF
                !Add the daily amount to the quarterly amount
                IF (i.LT.WEEK_LOOP) THEN
                    SICK_BETHPUB_QT(pers_j) = SICK_BETHPUB_QT(pers_j) + SICK_BETHPUB_DAY(pers_j)*6.D0
                    !distinguish disability and primary disablement for comparison with exertnal sources
                    IF (ANY(TYPE_SICK.EQ.(/2,4,6/))) THEN
                        SICK_DISEMPNM_QT(pers_j,2) = SICK_DISEMPNM_QT(pers_j,2) + SICK_BETHPUB_DAY(pers_j)*6.D0
                    ELSE IF (ANY(TYPE_SICK.EQ.(/1,3,5/))) THEN
                        SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BETHPUB_DAY(pers_j)*6.D0
                    END IF
                ELSE IF (i.EQ.WEEK_LOOP) THEN
                    DAYS_REST = CEILING(DBLE(SICK_DAYSPUB1_QT(pers_j)) - DBLE(WEEK_LOOP-1)*6.D0)
                    IF (DAYS_REST.LT.0) THEN
                        DAYS_REST = 0
                    END IF 
                    SICK_BETHPUB_QT(pers_j) = SICK_BETHPUB_QT(pers_j) + SICK_BETHPUB_DAY(pers_j)*DBLE(DAYS_REST)
                    !distinguish disability and primary disablement for comparison with exertnal sources
                    IF (ANY(TYPE_SICK.EQ.(/2,4,6/))) THEN
                        SICK_DISEMPNM_QT(pers_j,2) = SICK_DISEMPNM_QT(pers_j,2) + SICK_BETHPUB_DAY(pers_j)*DBLE(DAYS_REST)
                    ELSE IF (ANY(TYPE_SICK.EQ.(/1,3,5/))) THEN
                        SICK_SICEMPNM_QT(pers_j,2) = SICK_SICEMPNM_QT(pers_j,2) + SICK_BETHPUB_DAY(pers_j)*DBLE(DAYS_REST)
                    END IF
                END IF
            END DO
        END IF
    END IF

    !Maternity benefits
    !SICK_BENPUB2_DAY: Maternity benefit per day for individual insured as wage earner on the public labour market            
    !SICK_BENPUB2_QT: Maternity benefit per quarter for individual insured as wage earner on the public labour market            
    IF (ANY(SICK_ELIGPUB2_QT(pers_j).EQ.(/1, 2/))) THEN
        !Compute the limited daily wage
        IF (SICK_WOWEPUB_QT(pers_j).EQ.1) THEN
            LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(1))
        ELSE IF (SICK_WOWEPUB_QT(pers_j).EQ.2) THEN
            LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_LIMPRIV_DAY(2))
        END IF
        !Distinguish cases on the basis of the number of days already consumed
        IF (SICK_MATLPUB2_QT(pers_j).LT.31) THEN
            !Compute the first daily benefit
            !If the person, eligible for maternity benefits has a labour market contract
            IF (INT(100.D0*CONTRIB_REVPUB_QT(pers_j)).NE.0) THEN
                SICK_BENPUB2_DAY(pers_j) = (SICK_MATPRIV_DAY(3)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
            !If the person, eligible for maternity benefits is unemployed
            ELSE IF (INT(100.D0*CONTRIB_REVPUB_QT(pers_j)).EQ.0.AND.INT(100.D0*UNEM_BENUN_QT(pers_j)).NE.0) THEN
                SICK_BENPUB2_DAY(pers_j) = (SICK_MATPRIV_DAY(5)/100.D0)*LDW
            !In other cases follow the general rule
            ELSE
                SICK_BENPUB2_DAY(pers_j) = (SICK_MATPRIV_DAY(1)/100.D0)*LDW
            END IF
            !Compute the quarterly benefits with the first daily value
            SICK_BENPUB2_QT(pers_j) = SICK_BENPUB2_DAY(pers_j)*(DBLE(31)-DBLE(SICK_MATLPUB2_QT(pers_j)))
            !Compute the second daily benefit
            !If the person, eligible for maternity benefits has a labour market contract
            IF (INT(100.D0*CONTRIB_REVPUB_QT(pers_j)).NE.0) THEN
                SICK_BENPUB2_DAY(pers_j) = (SICK_MATPRIV_DAY(4)/100.D0)*LDW
            !If the person, eligible for maternity benefits is unemployed
            ELSE IF (INT(100.D0*CONTRIB_REVPUB_QT(pers_j)).EQ.0.AND.INT(100.D0*UNEM_BENUN_QT(pers_j)).NE.0) THEN
                SICK_BENPUB2_DAY(pers_j) = (SICK_MATPRIV_DAY(6)/100.D0)*LDW
            !In other cases follow the general rule
            ELSE
                SICK_BENPUB2_DAY(pers_j) = (SICK_MATPRIV_DAY(2)/100.D0)*LDW
            END IF
            !Compute the quarterly benefits with the second daily value
            SICK_BENPUB2_QT(pers_j) = SICK_BENPUB2_QT(pers_j) + SICK_BENPUB2_DAY(pers_j)*(DBLE(45))
        ELSE
            !Compute the daily benefit
            !If the person, eligible for maternity benefits has a labour market contract
            IF (INT(100.D0*CONTRIB_REVPUB_QT(pers_j)).NE.0) THEN
                SICK_BENPUB2_DAY(pers_j) = (SICK_MATPRIV_DAY(4)/100.D0)*LDW
            !If the person, eligible for maternity benefits is unemployed
            ELSE IF (INT(100.D0*CONTRIB_REVPUB_QT(pers_j)).EQ.0.AND.INT(100.D0*UNEM_BENUN_QT(pers_j)).NE.0) THEN
                SICK_BENPUB2_DAY(pers_j) = (SICK_MATPRIV_DAY(6)/100.D0)*LDW
            !In other cases follow the general rule
            ELSE
                SICK_BENPUB2_DAY(pers_j) = (SICK_MATPRIV_DAY(2)/100.D0)*LDW
            END IF
            !Compute the quarterly benefits with the daily value
            SICK_BENPUB2_QT(pers_j) = SICK_BENPUB2_DAY(pers_j)*(DBLE(75)-DBLE(SICK_MATLPUB2_QT(pers_j)))
        END IF
!Aangevuld bij aanpassingen september 2011: later bijstellen bij grondigere testing
        IF (SICK_ELIGPUB2_QT(pers_j).EQ.1) THEN
            IF (NIC_BEDRAG_2008(pers_j).NE.0.D0) THEN
                SICK_BENPUB2_QT(pers_j) = NIC_BEDRAG_2008(pers_j)/4.D0
            END IF
        ELSE IF (SICK_ELIGPUB2_QT(pers_j).EQ.2) THEN
            IF (RIZIVINA_BEDRAG_MAT_2008(pers_j).NE.0.D0) THEN
                SICK_BENPUB2_QT(pers_j) = RIZIVINA_BEDRAG_MAT_2008(pers_j)/4.D0                
            END IF
        END IF
    END IF

    !Benefits in case of industrial accidents
    !SICK_BENPUB3_DAY: Benefit per day received in case of an industrial accident (individual is insured as wage earner on the public labour market)          
    !SICK_BENPUB3_QT: Benefit per quarter received in case of an industrial accident (individual is insured as wage earner on the public labour market)          
    !SICK_EMPBPUB3_QT: Part of the benefit per quarter received in case of an industrial accident and paid by the employer (individual is insured as wage earner on the public labour market)         
    IF (SICK_ELIGPUB3_QT(pers_j).EQ.1) THEN
        WEEK_LOOP = 0
        WEEK_START = 0
        LDW = 0
        DAYS_REST = 0
        !Determine the number of weeks in benefit for industrial accident in the given quarter
        WEEK_LOOP = CEILING(DBLE(SICK_DAYSPUB3_QT(pers_j))/7.D0)
        IF (WEEK_LOOP.GT.13) THEN
            WEEK_LOOP = 13
        END IF
        !Determine the first week in benefit for industrial accident in the given quarter
        WEEK_START = SICK_WEEKPUB3_QT(pers_j) - WEEK_LOOP  
        IF (WEEK_START.LT.0) THEN
            WEEK_START = 0
        ELSE IF (WEEK_START.EQ.0) THEN
            WEEK_START = 1
        END IF
        IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.0) THEN
            !Compute the limited daily wage
            LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_INDACCLW_DAY)
            !Start loop to compute daily and quarterly benefits
            DO i=1,WEEK_LOOP
                !distinguish the relevant cases for the computation of the daily amount
                !daily amount before correcting minima and maxima
                TYPE_SICK = 0
                !If the employee is in temporary full disablement
                IF (SICK_TYDBPUB3_QT(pers_j).EQ.1) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPUB3_DAY(pers_j) = (SICK_INDACCPC_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 1
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPUB3_DAY(pers_j) = (SICK_INDACCPC_DAY(2,1)/100.D0)*LDW
                                TYPE_SICK = 2
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
                            SICK_BENPUB3_DAY(pers_j) = (SICK_INDACCPC_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 3
                        END IF
                    !From the second month of sickness
                    ELSE IF ((WEEK_START+i-1).GT.4) THEN
                        SICK_BENPUB3_DAY(pers_j) = (SICK_INDACCPC_DAY(4,1)/100.D0)*LDW
                        TYPE_SICK = 4
                    END IF
                !If the employee is in temporary partial disablement
                ELSE IF (SICK_TYDBPUB3_QT(pers_j).EQ.2) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPUB3_DAY(pers_j) = (SICK_INDACCPC_DAY(5,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 5
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPUB3_DAY(pers_j) = (SICK_INDACCPC_DAY(6,1)/100.D0)*LDW
                                TYPE_SICK = 6
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
                            SICK_BENPUB3_DAY(pers_j) = (SICK_INDACCPC_DAY(7,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 7
                        END IF
                    !From the second month of sickness
                    ELSE IF ((WEEK_START+i-1).GT.4) THEN
                        SICK_BENPUB3_DAY(pers_j) = (SICK_INDACCPC_DAY(8,1)/100.D0)*LDW
                        TYPE_SICK = 8
                    END IF
                !If the employee is in permanent disablement
                ELSE IF (SICK_TYDBPUB3_QT(pers_j).EQ.3) THEN
                    SICK_BENPUB3_DAY(pers_j) = (SICK_INDACCPC_DAY(9,1)/100.D0)*LDW*SICK_DEGRPUB3_QT(pers_j)
                    TYPE_SICK = 9
                END IF
                !correct minima and maxima
                IF (TYPE_SICK.LE.9) THEN
                    !loop over different disability types
                    DO j=1,9
                        IF (TYPE_SICK.EQ.j) THEN
                            !Correct minimum
                            IF (SICK_BENPUB3_DAY(pers_j).LT.SICK_INDACCPC_DAY(j,2)) THEN
                                SICK_BENPUB3_DAY(pers_j) = SICK_INDACCPC_DAY(j,2)
                            END IF
                            !Correct maximum
                            IF (SICK_BENPUB3_DAY(pers_j).GT.SICK_INDACCPC_DAY(j,3).AND.&
                            &   INT(100.D0*SICK_INDACCPC_DAY(j,3)).NE.0) THEN
                                SICK_BENPUB3_DAY(pers_j) = SICK_INDACCPC_DAY(j,3)
                            END IF
                        END IF
                    END DO
                END IF
                !Add the daily amount to the quarterly amount
                IF (i.LT.WEEK_LOOP) THEN
                    SICK_BENPUB3_QT(pers_j) = SICK_BENPUB3_QT(pers_j) + SICK_BENPUB3_DAY(pers_j)*7.D0
                ELSE IF (i.EQ.WEEK_LOOP) THEN
                    DAYS_REST = CEILING(DBLE(SICK_DAYSPUB3_QT(pers_j)) - DBLE(WEEK_LOOP-1)*7.D0)
                    IF (DAYS_REST.LT.0) THEN
                        DAYS_REST = 0
                    END IF 
                    SICK_BENPUB3_QT(pers_j) = SICK_BENPUB3_QT(pers_j) + SICK_BENPUB3_DAY(pers_j)*DBLE(DAYS_REST)
                END IF
                !Compute the amount paid by the employer
                !In case of a blue collar worker
                IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1.AND.(WEEK_START+i-1).LE.1) THEN
                    SICK_EMPBPUB3_QT(pers_j) = SICK_BENPUB3_QT(pers_j)
                END IF
                !In case of white collar worker
                IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2.AND.(WEEK_START+i-1).LE.4) THEN
                    SICK_EMPBPUB3_QT(pers_j) = SICK_BENPUB3_QT(pers_j)
                END IF
            END DO
        END IF
!Aangevuld bij aanpassingen september 2011: later bijstellen bij grondigere testing
        SumInc = MONTITT_2008(pers_j)/4.D0 + MONTITP_2008(pers_j)/4.D0
        IF (SumInc.NE.0.D0) THEN
            SICK_BENPUB3_QT(pers_j) = SumInc
        END IF
    END IF

    !Benefits in case of occupational diseases
    !SICK_BENPUB4_DAY: Benefit per day received in case of an occupational disease (individual is insured as wage earner on the public labour market)          
    !SICK_BENPUB4_QT: Benefit per quarter received in case of an occupational disease (individual is insured as wage earner on the public labour market)          
    !SICK_EMPBPUB4_QT: Part of the benefit per quarter received in case of an occupational disease and paid by the employer (individual is insured as wage earner on the public labour market)         
    IF (SICK_ELIGPUB4_QT(pers_j).EQ.1) THEN
        WEEK_LOOP = 0
        WEEK_START = 0
        LDW = 0
        DAYS_REST = 0
        !Determine the number of weeks in benefit for industrial accident in the given quarter
        WEEK_LOOP = CEILING(DBLE(SICK_DAYSPUB4_QT(pers_j))/7.D0)
        IF (WEEK_LOOP.GT.13) THEN
            WEEK_LOOP = 13
        END IF
        !Determine the first week in benefit for industrial accident in the given quarter
        WEEK_START = SICK_WEEKPUB4_QT(pers_j) - WEEK_LOOP  
        IF (WEEK_START.LT.0) THEN
            WEEK_START = 0
        ELSE IF (WEEK_START.EQ.0) THEN
            WEEK_START = 1
        END IF
        IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.0) THEN
            !Compute the limited daily wage
            LDW = MIN(SICK_LDAYWAGE_DAY(pers_j),SICK_OCCDISLW_DAY)
            !Start loop to compute daily and quarterly benefits
            DO i=1,WEEK_LOOP
                !distinguish the relevant cases for the computation of the daily amount
                !daily amount before correcting minima and maxima
                TYPE_SICK = 0
                !If the employee is in temporary full disablement
                IF (SICK_TYDBPUB4_QT(pers_j).EQ.1) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPUB4_DAY(pers_j) = (SICK_OCCDISPC_DAY(1,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 1
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPUB4_DAY(pers_j) = (SICK_OCCDISPC_DAY(2,1)/100.D0)*LDW
                                TYPE_SICK = 2
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
                            SICK_BENPUB4_DAY(pers_j) = (SICK_OCCDISPC_DAY(3,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 3
                        END IF
                    !From the second month of sickness
                    ELSE IF ((WEEK_START+i-1).GT.4) THEN
                        SICK_BENPUB4_DAY(pers_j) = (SICK_OCCDISPC_DAY(4,1)/100.D0)*LDW
                        TYPE_SICK = 4
                    END IF
                !If the employee is in temporary partial disablement
                ELSE IF (SICK_TYDBPUB4_QT(pers_j).EQ.2) THEN
                    !In the first month of sickness
                    IF ((WEEK_START+i-1).LE.4) THEN
                        !For blue collar workers
                        IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
                            !In the first week of sickness
                            IF ((WEEK_START+i-1).EQ.1) THEN
                                SICK_BENPUB4_DAY(pers_j) = (SICK_OCCDISPC_DAY(5,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                                TYPE_SICK = 5
                            !In week 2, 3 or 4
                            ELSE IF (ANY((WEEK_START+i-1).EQ.(/2, 3, 4/))) THEN
                                SICK_BENPUB4_DAY(pers_j) = (SICK_OCCDISPC_DAY(6,1)/100.D0)*LDW
                                TYPE_SICK = 6
                            END IF
                        !For white collar workers
                        ELSE IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
                            SICK_BENPUB4_DAY(pers_j) = (SICK_OCCDISPC_DAY(7,1)/100.D0)*SICK_LDAYWAGE_DAY(pers_j)
                            TYPE_SICK = 7
                        END IF
                    !From the second month of sickness
                    ELSE IF ((WEEK_START+i-1).GT.4) THEN
                        SICK_BENPUB4_DAY(pers_j) = (SICK_OCCDISPC_DAY(8,1)/100.D0)*LDW
                        TYPE_SICK = 8
                    END IF
                !If the employee is in permanent disablement
                ELSE IF (SICK_TYDBPUB4_QT(pers_j).EQ.3) THEN
                    SICK_BENPUB4_DAY(pers_j) = (SICK_OCCDISPC_DAY(9,1)/100.D0)*LDW*SICK_DEGRPUB4_QT(pers_j)
                    TYPE_SICK = 9
                END IF
                !correct minima and maxima
                IF (TYPE_SICK.LE.9) THEN
                    !loop over different disability types
                    DO j=1,9
                        IF (TYPE_SICK.EQ.j) THEN
                            !Correct minimum
                            IF (SICK_BENPUB4_DAY(pers_j).LT.SICK_OCCDISPC_DAY(j,2)) THEN
                                SICK_BENPUB4_DAY(pers_j) = SICK_OCCDISPC_DAY(j,2)
                            END IF
                            !Correct maximum
                            IF (SICK_BENPUB4_DAY(pers_j).GT.SICK_OCCDISPC_DAY(j,3).AND.&
                            &   INT(100.D0*SICK_OCCDISPC_DAY(j,3)).NE.0) THEN
                                SICK_BENPUB4_DAY(pers_j) = SICK_OCCDISPC_DAY(j,3)
                            END IF
                        END IF
                    END DO
                END IF
                !Add the daily amount to the quarterly amount
                IF (i.LT.WEEK_LOOP) THEN
                    SICK_BENPUB4_QT(pers_j) = SICK_BENPUB4_QT(pers_j) + SICK_BENPUB4_DAY(pers_j)*7.D0
                ELSE IF (i.EQ.WEEK_LOOP) THEN
                    DAYS_REST = CEILING(DBLE(SICK_DAYSPUB4_QT(pers_j)) - DBLE(WEEK_LOOP-1)*7.D0)
                    IF (DAYS_REST.LT.0) THEN
                        DAYS_REST = 0
                    END IF 
                    SICK_BENPUB4_QT(pers_j) = SICK_BENPUB4_QT(pers_j) + SICK_BENPUB4_DAY(pers_j)*DBLE(DAYS_REST)
                END IF
                !Compute the amount paid by the employer
                !In case of a blue collar worker
                IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.1.AND.(WEEK_START+i-1).LE.1) THEN
                    SICK_EMPBPUB4_QT(pers_j) = SICK_BENPUB4_QT(pers_j)
                END IF
                !In case of white collar worker
                IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.2.AND.(WEEK_START+i-1).LE.4) THEN
                    SICK_EMPBPUB4_QT(pers_j) = SICK_BENPUB4_QT(pers_j)
                END IF
            END DO
        END IF
!Aangevuld bij aanpassingen september 2011: later bijstellen bij grondigere testing
        IF (FBZ_BEDRAG_2008(pers_j).NE.0.D0) THEN
            SICK_BENPUB4_QT(pers_j) = FBZ_BEDRAG_2008(pers_j)/4.D0
        END IF
    END IF

END SUBROUTINE SICK_BENPUBNS_CONSTRUC

!==========================================================!
!With the subroutine SICK_BENPUBST_CONSTRUC we compute     !
!the sickness and disability benefits for those insured    !
!as wage earner on the public labour market WITH           ! 
!statutory service                                         ! 
!Variables constructed in the routine                      !
!Sickness and disability benefits                          !
!   SICK_BENPUB1_DAY                                       !
!   SICK_BENPUB1_QT                                        !
!   SICK_EMPBPUB1_QT                                       !
!Maternity benefits                                        !
!   SICK_BENPUB2_DAY                                       !
!   SICK_BENPUB2_QT                                        !
!   SICK_EMPBPUB2_QT                                       !
!Industrial accidents                                      !
!   SICK_BENPUB3_DAY                                       !
!   SICK_BENPUB3_QT                                        !
!   SICK_EMPBPUB3_QT                                       !
!Occupational diseases                                     !
!   SICK_BENPUB4_DAY                                       !
!   SICK_BENPUB4_QT                                        !
!   SICK_EMPBPUB4_QT                                       !
!==========================================================!

SUBROUTINE SICK_BENPUBST_CONSTRUC(pers_j)

    INTEGER                                 :: pers_j

    !Sickness and disability benefits
    !SICK_BENPUB1_DAY: Daily benefit for sickness or disability for individual in the wage earner regime of the public labour market         
    !SICK_BENPUB1_QT:  Quarterly benefit for sickness or disability for individual in the wage earner regime of the public labour market         
    !SICK_EMPBPUB1_QT: Part of the quarterly benefit for sickness or disability paid by the employer for individual insured as wage earner on the public labour market            
    IF (SICK_ELIGPUB1_QT(pers_j).EQ.1) THEN
        SICK_BENPUB1_DAY(pers_j)    = (SICK_PEPUBSIC_DAY/100.D0)*SICK_LDAYWAGE_DAY(pers_j) 
        !Correct the daily amount when labour income is simulated 
        IF (MIMOSIS_SIMNUMMT(pers_j).NE.0) THEN
            IF (SIMYEAR.LT.2003) THEN
                IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN
                    SICK_BENPUB1_DAY(pers_j) = SICK_BENPUB1_DAY(pers_j)*(SICK_LABCUMUL_DAY(1)/100.D0) - &
                    &SICK_LABDEDUC_DAY(pers_j)
                ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/))) THEN
                    SICK_BENPUB1_DAY(pers_j) = SICK_BENPUB1_DAY(pers_j)*(SICK_LABCUMUL_DAY(2)/100.D0) - &
                    &SICK_LABDEDUC_DAY(pers_j)
                END IF  
            ELSE
                SICK_BENPUB1_DAY(pers_j) = SICK_BENPUB1_DAY(pers_j) - SICK_LABDEDUC_DAY(pers_j)
            END IF
            !Correct if negative
            IF (SICK_BENPUB1_DAY(pers_j).LT.0.D0) THEN
                SICK_BENPUB1_DAY(pers_j) = 0.D0
            END IF
        END IF
        SICK_BENPUB1_QT(pers_j)     = SICK_BENPUB1_DAY(pers_j)*DBLE(SICK_DAYSPUB1_QT(pers_j))
        SICK_EMPBPUB1_QT(pers_j)    = SICK_BENPUB1_QT(pers_j) 
    END IF

    !Maternity benefits
    !SICK_BENPUB2_DAY: Maternity benefit per day for individual insured as wage earner on the public labour market            
    !SICK_BENPUB2_QT:  Maternity benefit per quarter for individual insured as wage earner on the public labour market            
    !SICK_EMPBPUB2_QT: Part of the quarterly benefit for sickness or disability paid by the employer for individual insured as wage earner on the public labour market            
    IF (SICK_ELIGPUB2_QT(pers_j).EQ.1) THEN
        SICK_BENPUB2_DAY(pers_j)    = (SICK_PEPUBMAT_DAY/100.D0)*SICK_LDAYWAGE_DAY(pers_j) 
        SICK_BENPUB2_QT(pers_j)     = SICK_BENPUB2_DAY(pers_j)*DBLE(SICK_DAYSPUB2_QT(pers_j))
        SICK_EMPBPUB2_QT(pers_j)    = SICK_BENPUB2_QT(pers_j) 
    END IF

    !Benefits in case of industrial accidents
    !SICK_BENPUB3_DAY: Benefit per day received in case of an industrial accident (individual is insured as wage earner on the public labour market)          
    !SICK_BENPUB3_QT:  Benefit per quarter received in case of an industrial accident (individual is insured as wage earner on the public labour market)          
    !SICK_EMPBPUB3_QT: Part of the benefit per quarter received in case of an industrial accident and paid by the employer (individual is insured as wage earner on the public labour market)         
    IF (SICK_ELIGPUB3_QT(pers_j).EQ.1) THEN
        SICK_BENPUB3_DAY(pers_j)    = (SICK_PEPUBIND_DAY/100.D0)*SICK_LDAYWAGE_DAY(pers_j)  
        SICK_BENPUB3_QT(pers_j)     = SICK_BENPUB3_DAY(pers_j)*DBLE(SICK_DAYSPUB3_QT(pers_j))
        SICK_EMPBPUB3_QT(pers_j)    = SICK_BENPUB3_QT(pers_j) 
    END IF

    !Benefits in case of occupational diseases
    !SICK_BENPUB4_DAY: Benefit per day received in case of an occupational disease (individual is insured as wage earner on the public labour market)          
    !SICK_BENPUB4_QT:  Benefit per quarter received in case of an occupational disease (individual is insured as wage earner on the public labour market)          
    !SICK_EMPBPUB4_QT: Part of the benefit per quarter received in case of an occupational disease and paid by the employer (individual is insured as wage earner on the public labour market)         
    IF (SICK_ELIGPUB4_QT(pers_j).EQ.1) THEN
        SICK_BENPUB4_DAY(pers_j)    = (SICK_PEPUBOCC_DAY/100.D0)*SICK_LDAYWAGE_DAY(pers_j) 
        SICK_BENPUB4_QT(pers_j)     = SICK_BENPUB4_DAY(pers_j)*DBLE(SICK_DAYSPUB4_QT(pers_j))
        SICK_EMPBPUB4_QT(pers_j)    = SICK_BENPUB4_QT(pers_j) 
    END IF

END SUBROUTINE SICK_BENPUBST_CONSTRUC

!==========================================================!
!With the subroutine SICK_BENSELF_CONSTRUC we compute      !
!the sickness and disability benefits for those insured    !
!as self employed                                          !
!Variables constructed in the routine                      !
!Sickness and disability self employed                     !
!   SICK_BENSEL1_DAY                                       !
!   SICK_BENSEL1_QT                                        !
!   SICK_BETHSELF_QT                                       !
!Maternity benefits self employed                          !
!   SICK_BENSEL2_QT                                        !
!==========================================================!

SUBROUTINE SICK_BENSELF_CONSTRUC(pers_j)

    INTEGER                                 :: pers_j
    INTEGER                                 :: WEEK_LOOP            !Number of weeks for which the loop with which the benefits are computed should run
    INTEGER                                 :: WEEK_START           !First week for which a benefit is computed
    INTEGER                                 :: DAYS_REST            !Remaining number of days in the last week for which daily benefits are computed
    INTEGER                                 :: TYPE_SICK            !Identifier to identify the type of sickness or disability benefit (used to reduce the check on minima and maxima)
    REAL(KIND=KIND_REAL)                    :: SICK_BASE_DAY        !base to compute allowed cumulation between sick benefits and labour income for system before 2003

    !Sickness and disability self employed               
    !SICK_BENSEL1_DAY: Daily benefit for sickness or disability for individual insured as self employed            
    !SICK_BENSEL1_QT:  Quarterly benefit for sickness or disability for individual insured as self employed            
    IF (ANY(SICK_ELIGSEL1_QT(pers_j).EQ.(/1, 2/))) THEN
        WEEK_LOOP = 0
        WEEK_START = 0
        DAYS_REST = 0
        !Determine the number of weeks in sickness or disability in the given quarter
        WEEK_LOOP = CEILING(DBLE(SICK_DAYSSEL1_QT(pers_j))/6.D0)
        IF (WEEK_LOOP.GT.13) THEN
            WEEK_LOOP = 13
        END IF
        !Determine the first week in sickness or disability in the given quarter
        WEEK_START = SICK_WEEKSEL1_QT(pers_j) - WEEK_LOOP  
        IF (WEEK_START.LT.0) THEN
            WEEK_START = 0
        ELSE IF (WEEK_START.EQ.0) THEN
            WEEK_START = 1
        END IF
        IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.0) THEN
            !Start loop to compute daily and quarterly benefits
            DO i=1,WEEK_LOOP
                TYPE_SICK = 0
                SICK_BASE_DAY = 0.D0
                !distinguish the relevant cases for the computation of the daily amount on the time in sickness or disability
                !If the self employed is in the second month until the 12th month of sickness
                IF ((WEEK_START+i-1).GT.4.AND.(WEEK_START+i-1).LT.53) THEN
                    TYPE_SICK = 1
                    !If the self employed has dependent family
                    IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN   
                        SICK_BENSEL1_DAY(pers_j) = SICK_LSSELSIC_DAY(1,1)
                        SICK_BASE_DAY = SICK_LSSELSIC_DAY(1,1)
                    !The self employed is "single"
                    ELSE IF (SICK_FAMSIT_QT(pers_j).EQ.2) THEN   
                        SICK_BENSEL1_DAY(pers_j) = SICK_LSSELSIC_DAY(1,2)
                        SICK_BASE_DAY = SICK_LSSELSIC_DAY(1,1)
                    ELSE
                        SICK_BENSEL1_DAY(pers_j) = SICK_LSSELSIC_DAY(1,3) 
                        SICK_BASE_DAY = SICK_LSSELSIC_DAY(1,1) 
                    END IF
                !From the 13th month on
                ELSE IF ((WEEK_START+i-1).GE.53) THEN
                    TYPE_SICK = 2
                    !The general rule
                    IF (SICK_CLOSESEL_QT(pers_j).EQ.0) THEN
                        !If the self employed has dependent family
                        IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN   
                            SICK_BENSEL1_DAY(pers_j) = SICK_LSSELSIC_DAY(2,1)
                            SICK_BASE_DAY = SICK_LSSELSIC_DAY(2,1)
                        !The self employed is "single"
                        ELSE IF (SICK_FAMSIT_QT(pers_j).EQ.2) THEN   
                            SICK_BENSEL1_DAY(pers_j) = SICK_LSSELSIC_DAY(2,2)
                            SICK_BASE_DAY = SICK_LSSELSIC_DAY(2,1)
                        ELSE
                            SICK_BENSEL1_DAY(pers_j) = SICK_LSSELSIC_DAY(2,3) 
                            SICK_BASE_DAY = SICK_LSSELSIC_DAY(2,1) 
                        END IF
                    !If the self employed had to close his business because of the accident
                    ELSE
                        !If the self employed has dependent family
                        IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN   
                            SICK_BENSEL1_DAY(pers_j) = SICK_LSSELSIC_DAY(3,1)
                            SICK_BASE_DAY = SICK_LSSELSIC_DAY(3,1)
                        !The self employed is "single"
                        ELSE IF (SICK_FAMSIT_QT(pers_j).EQ.2) THEN   
                            SICK_BENSEL1_DAY(pers_j) = SICK_LSSELSIC_DAY(3,2)
                            SICK_BASE_DAY = SICK_LSSELSIC_DAY(3,1)
                        ELSE
                            SICK_BENSEL1_DAY(pers_j) = SICK_LSSELSIC_DAY(3,3) 
                            SICK_BASE_DAY = SICK_LSSELSIC_DAY(3,1) 
                        END IF
                    END IF
                END IF
                !Correct the daily amount when labour income is simulate 
                IF (MIMOSIS_SIMNUMMT(pers_j).NE.0) THEN
                    IF (SIMYEAR.LT.2003) THEN
                        IF (SICK_FAMSIT_QT(pers_j).EQ.1) THEN
                            SICK_BENSEL1_DAY(pers_j) = (SICK_BASE_DAY)*(SICK_LABCUMUL_DAY(1)/100.D0) - SICK_LABDEDUC_DAY(pers_j)
                        ELSE IF (ANY(SICK_FAMSIT_QT(pers_j).EQ.(/2, 3/))) THEN
                            SICK_BENSEL1_DAY(pers_j) = (SICK_BASE_DAY)*(SICK_LABCUMUL_DAY(2)/100.D0) - SICK_LABDEDUC_DAY(pers_j)
                        END IF  
                    ELSE
                        SICK_BENSEL1_DAY(pers_j) = SICK_BENSEL1_DAY(pers_j) - SICK_LABDEDUC_DAY(pers_j)
                    END IF
                    !Correct if negative
                    IF (SICK_BENSEL1_DAY(pers_j).LT.0.D0) THEN
                        SICK_BENSEL1_DAY(pers_j) = 0.D0
                    END IF
                END IF
                !Add the daily amount to the quarterly amount
                IF (i.LT.WEEK_LOOP) THEN
                    SICK_BENSEL1_QT(pers_j) = SICK_BENSEL1_QT(pers_j) + SICK_BENSEL1_DAY(pers_j)*6.D0
                    !distinguish disability and primary disablement for comparison with exertnal sources
                    IF (TYPE_SICK.EQ.2) THEN
                        SICK_DISSELNM_QT(pers_j,1) = SICK_DISSELNM_QT(pers_j,1) + 6.D0
!                        SICK_DISSELNM_QT(pers_j,2) = SICK_DISSELNM_QT(pers_j,2) + SICK_BENSEL1_DAY(pers_j)*6.D0
                    ELSE IF (TYPE_SICK.EQ.1) THEN
                        SICK_SICSELNM_QT(pers_j,1) = SICK_SICSELNM_QT(pers_j,1) + 6.D0
!                        SICK_SICSELNM_QT(pers_j,2) = SICK_SICSELNM_QT(pers_j,2) + SICK_BENSEL1_DAY(pers_j)*6.D0
                    END IF
                ELSE IF (i.EQ.WEEK_LOOP) THEN
                    DAYS_REST = CEILING(DBLE(SICK_DAYSSEL1_QT(pers_j)) - DBLE(WEEK_LOOP-1)*6.D0)
                    IF (DAYS_REST.LT.0) THEN
                        DAYS_REST = 0
                    END IF 
                    SICK_BENSEL1_QT(pers_j) = SICK_BENSEL1_QT(pers_j) + SICK_BENSEL1_DAY(pers_j)*DBLE(DAYS_REST)
                    !distinguish disability and primary disablement for comparison with exertnal sources
                    IF (TYPE_SICK.EQ.2) THEN
                        SICK_DISSELNM_QT(pers_j,1) = SICK_DISSELNM_QT(pers_j,1) + DBLE(DAYS_REST)
!                        SICK_DISSELNM_QT(pers_j,2) = SICK_DISSELNM_QT(pers_j,2) + SICK_BENSEL1_DAY(pers_j)*DBLE(DAYS_REST)
                    ELSE IF (TYPE_SICK.EQ.1) THEN
                        SICK_SICSELNM_QT(pers_j,1) = SICK_SICSELNM_QT(pers_j,1) + DBLE(DAYS_REST)
!                        SICK_SICSELNM_QT(pers_j,2) = SICK_SICSELNM_QT(pers_j,2) + SICK_BENSEL1_DAY(pers_j)*DBLE(DAYS_REST)
                    END IF
                END IF
            END DO
        END IF
!Aangevuld bij aanpassingen september 2011: later bijstellen bij grondigere testing
        IF (ANY(SICK_ELIGSEL1_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (NIC_BEDRAG_2008(pers_j).NE.0.D0) THEN
                SICK_BENSEL1_QT(pers_j) = NIC_BEDRAG_2008(pers_j)/4.D0
                SICK_SICSELNM_QT(pers_j,2) = SICK_SICSELNM_QT(pers_j,2) + SICK_BENSEL1_QT(pers_j)
            END IF
        END IF
        IF (SICK_ELIGSEL1_QT(pers_j).EQ.2) THEN
            IF (RIZIVINA_BEDRAG_INVAL_2008(pers_j).NE.0.D0) THEN
                SICK_BENSEL1_QT(pers_j) = RIZIVINA_BEDRAG_INVAL_2008(pers_j)/4.D0                
                SICK_DISSELNM_QT(pers_j,2) = SICK_DISSELNM_QT(pers_j,2) + SICK_BENSEL1_QT(pers_j)
            END IF
        END IF
!        IF (NIC_STELSEL_2008(pers_j).EQ.'2'.AND.SICK_ELIGSEL1_QT(pers_j).EQ.1) THEN
!            WRITE(15,*) 'TestB',NIC_BEDRAG_2008(pers_j),NIC_SRTDAGEN_2008(pers_j),MIMOSIS_WEIGHT(pers_j)
!        END IF
!        IF (NIC_STELSEL_2008(pers_j).EQ.'2'.AND.NIC_SRTDAGEN_2008(pers_j).EQ.'01'.AND.NIC_BEDRAG_2008(pers_j).NE.0.D0) THEN
!            WRITE(15,*) 'TestA',NIC_BEDRAG_2008(pers_j),SICK_ELIGSEL1_QT(pers_j),MIMOSIS_WEIGHT(pers_j)
!        END IF
    END IF

    !SICK_BETHSELF_QT: Quarterly supplement for help of a third person (individual is insured as self employed)            
    IF (SICK_ELTHSELF_QT(pers_j).EQ.1) THEN
        WEEK_LOOP = 0
        WEEK_START = 0
        DAYS_REST = 0
        !Determine the number of weeks in sickness or disability in the given quarter
        WEEK_LOOP = CEILING(DBLE(SICK_DAYSSEL1_QT(pers_j))/6.D0)
        IF (WEEK_LOOP.GT.13) THEN
            WEEK_LOOP = 13
        END IF
        !Determine the first week in sickness or disability in the given quarter
        WEEK_START = SICK_WEEKSEL1_QT(pers_j) - WEEK_LOOP  
        IF (WEEK_START.LT.0) THEN
            WEEK_START = 0
        ELSE IF (WEEK_START.EQ.0) THEN
            WEEK_START = 1
        END IF
        IF (WEEK_START.GT.0.AND.WEEK_LOOP.GT.0) THEN
            !Start loop to compute daily and quarterly benefits
            DO i=1,WEEK_LOOP
                !If the self employed is in the 4th month of sickness or more
                IF ((WEEK_START+i-1).GT.12) THEN
                    !Add the daily amount to the quarterly amount
                    IF (i.LT.WEEK_LOOP) THEN
                        SICK_BETHSELF_QT(pers_j) = SICK_BETHSELF_QT(pers_j) + SICK_THIRSELF_DAY*6.D0
                        !distinguish disability and primary disablement for comparison with exertnal sources
                        IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_DISSELNM_QT(pers_j,2) = SICK_DISSELNM_QT(pers_j,2) + SICK_BENSEL1_DAY(pers_j)*6.D0
                        ELSE IF ((WEEK_START+i-1).LT.53) THEN
                            SICK_SICSELNM_QT(pers_j,2) = SICK_SICSELNM_QT(pers_j,2) + SICK_BENSEL1_DAY(pers_j)*6.D0
                        END IF
                    ELSE IF (i.EQ.WEEK_LOOP) THEN
                        DAYS_REST = CEILING(DBLE(SICK_DAYSSEL1_QT(pers_j)) - DBLE(WEEK_LOOP-1)*6.D0)
                        IF (DAYS_REST.LT.0) THEN
                            DAYS_REST = 0
                        END IF 
                        SICK_BETHSELF_QT(pers_j) = SICK_BETHSELF_QT(pers_j) + SICK_THIRSELF_DAY*DBLE(DAYS_REST)
                        !distinguish disability and primary disablement for comparison with exertnal sources
                        IF ((WEEK_START+i-1).GE.53) THEN
                            SICK_DISSELNM_QT(pers_j,2) = SICK_DISSELNM_QT(pers_j,2) + SICK_BENSEL1_DAY(pers_j)*DBLE(DAYS_REST)
                        ELSE IF ((WEEK_START+i-1).LT.53) THEN
                            SICK_SICSELNM_QT(pers_j,2) = SICK_SICSELNM_QT(pers_j,2) + SICK_BENSEL1_DAY(pers_j)*DBLE(DAYS_REST)
                        END IF
                    END IF
                END IF
            END DO
        END IF
    END IF
    
    !Maternity benefits self employed                
    !SICK_BENSEL2_QT:  Quarterly maternity benefit for individual insured as self employed         
    IF (ANY(SICK_ELIGSEL2_QT(pers_j).EQ.(/1, 2/))) THEN
        SICK_BENSEL2_QT(pers_j) = SICK_MATSELF_WEEK*DBLE(SICK_WEEKSEL2_QT(pers_j))
!Aangevuld bij aanpassingen september 2011: later bijstellen bij grondigere testing
        IF (ANY(SICK_ELIGSEL2_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (NIC_BEDRAG_2008(pers_j).NE.0.D0) THEN
                SICK_BENSEL2_QT(pers_j) = NIC_BEDRAG_2008(pers_j)/4.D0
            END IF
        END IF
        IF (SICK_ELIGSEL2_QT(pers_j).EQ.2) THEN
            IF (RIZIVINA_BEDRAG_MAT_2008(pers_j).NE.0.D0) THEN
                SICK_BENSEL2_QT(pers_j) = RIZIVINA_BEDRAG_MAT_2008(pers_j)/4.D0                
            END IF
        END IF
    END IF

END SUBROUTINE SICK_BENSELF_CONSTRUC

!===================================================================================!
!With subroutine SICK_EXCHANGE we construct the variables                           !
!based on the sickness computations, that have to be exchanged to                   !
!other modules                                                                      !
!   SICK_TOTBEN_QT                                                                  !
!   SICK_IND_QT                                                                     !
!   SICK_OCC_QT                                                                     !
!   SICK_DISABWEA_QT                                                                !
!   SICK_ALOTHBEN_QT                                                                !
!   SICK_EMPLBPRI_QT                                                                 !
!   SICK_EMPLBPUB_QT                                                                 !
!   SICK_BENINOC_QT                                                                 !
!   SICK_NONEINOC_QT                                                                !
!   SICK_DISABBEN_QT                                                                !
!===================================================================================!

SUBROUTINE SICK_EXCHANGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec

        !Variables constructed for the CONTRIB module
        !SICK_DISABWEA_QT: Disability benefits received by wage earners on the private or public labour market
        !Either received by blue or white collar workers
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.1.OR.&
           &SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
            SICK_DISABWEA_QT(pers_j) = SICK_DISEMPNM_QT(pers_j,2)
        END IF    

        !SICK_IND_QT: Disability benefits received by wage earners because of industrial accidents (benefits for help of a third person excluded)
        !Including benefits paid by the employer
        IF (ANY(SICK_TYDBPRI3_QT(pers_j).EQ.(/1,2,3/))) THEN
            SICK_IND_QT(pers_j)  = SICK_IND_QT(pers_j) + SICK_BENPRI3_QT(pers_j) 
        END IF    
        IF (ANY(SICK_TYDBPUB3_QT(pers_j).EQ.(/1,2,3/))) THEN
            SICK_IND_QT(pers_j)  = SICK_IND_QT(pers_j) + SICK_BENPUB3_QT(pers_j) 
        END IF    
 
        !SICK_OCC_QT: Disability benefits received by wage earners because of occupational diseases (benefits for help of a third person excluded)
        !Including benefits paid by the employer
        IF (ANY(SICK_TYDBPRI4_QT(pers_j).EQ.(/1,2,3/))) THEN
            SICK_OCC_QT(pers_j)  = SICK_OCC_QT(pers_j) + SICK_BENPRI4_QT(pers_j) 
        END IF    
        IF (ANY(SICK_TYDBPUB4_QT(pers_j).EQ.(/1,2,3/))) THEN
            SICK_OCC_QT(pers_j)  = SICK_OCC_QT(pers_j) + SICK_BENPUB4_QT(pers_j) 
        END IF    
 
        !SICK_ALOTHBEN_QT: Benefits in case of sickness or disability other than those covered by SICK_IND_QT, SICK_OCC_QT and SICK_DISABWEA_QT
        !Benefits for primary disablement
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.1.OR.&
           &SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
            SICK_ALOTHBEN_QT(pers_j) = SICK_ALOTHBEN_QT(pers_j) + SICK_SICEMPNM_QT(pers_j,2)
        END IF    
        !Employees, maternity benefits primary period
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPRI2_QT(pers_j).EQ.1) THEN
            SICK_ALOTHBEN_QT(pers_j) = SICK_ALOTHBEN_QT(pers_j) + SICK_BENPRI2_QT(pers_j)
        END IF    
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPUB2_QT(pers_j).EQ.1) THEN
            SICK_ALOTHBEN_QT(pers_j) = SICK_ALOTHBEN_QT(pers_j) + SICK_BENPUB2_QT(pers_j)
        END IF    
        !Employees, maternity benefits period of disability
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPRI2_QT(pers_j).EQ.2) THEN
            SICK_ALOTHBEN_QT(pers_j) = SICK_ALOTHBEN_QT(pers_j) + SICK_BENPRI2_QT(pers_j)
        END IF    
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPUB2_QT(pers_j).EQ.2) THEN
            SICK_ALOTHBEN_QT(pers_j) = SICK_ALOTHBEN_QT(pers_j) + SICK_BENPUB2_QT(pers_j)
        END IF    
        !Self employed, sickness benefits
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.ANY(SICK_ELIGSEL1_QT(pers_j).EQ.(/1, 2/))) THEN
            SICK_ALOTHBEN_QT(pers_j) = SICK_ALOTHBEN_QT(pers_j) + SICK_SICSELNM_QT(pers_j,2)
        END IF    
        !Self employed, disability benefits
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_ELIGSEL1_QT(pers_j).EQ.2) THEN
            SICK_ALOTHBEN_QT(pers_j) = SICK_ALOTHBEN_QT(pers_j) + SICK_DISSELNM_QT(pers_j,2)
        END IF    
        !Self employed, maternity benefits, primary period
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_ELIGSEL2_QT(pers_j).EQ.1) THEN
            SICK_ALOTHBEN_QT(pers_j) = SICK_ALOTHBEN_QT(pers_j) + SICK_BENSEL2_QT(pers_j)
        END IF    
        !Self employed, maternity benefits, disability
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_ELIGSEL2_QT(pers_j).EQ.2) THEN
            SICK_ALOTHBEN_QT(pers_j) = SICK_ALOTHBEN_QT(pers_j) + SICK_BENSEL2_QT(pers_j)
        END IF    

        !SICK_EMPLBPRI_QT: Benefits in case of sickness or disability paid by the employer in the private sector
        !SICK_EMPLBPUB_QT: Benefits in case of sickness or disability paid by the employer in the public sector
        SICK_EMPLBPRI_QT(pers_j) =   SICK_EMPBPRI1_QT(pers_j) + SICK_EMPSPRI1_QT(pers_j) 
        SICK_EMPLBPUB_QT(pers_j) =   SICK_EMPBPUB1_QT(pers_j) + SICK_EMPSPUB1_QT(pers_j) + &
                                  &  SICK_EMPBPUB2_QT(pers_j)

        !Variables constructed for the SICK and FAMAL module
        !SICK_TOTBEN_QT: Sum of benefits for sickness, disability, industrial accidents and occupational diseases            
            SICK_TOTBEN_QT(pers_j) = SICK_DISABWEA_QT(pers_j) + SICK_IND_QT(pers_j) + &
                                    &SICK_OCC_QT(pers_j) + SICK_ALOTHBEN_QT(pers_j) + &
                                    &SICK_EMPLBPRI_QT(pers_j) + SICK_EMPLBPUB_QT(pers_j)
            
        !Variables constructed for the UNEM module
        !SICK_BENINOC_QT: Sum of benefits for industrial accidents and occupational diseases          
            SICK_BENINOC_QT(pers_j) = SICK_IND_QT(pers_j) + SICK_OCC_QT(pers_j)

        !SICK_NONEINOC_QT: Sum of benefits for sickness and disability         
            SICK_NONEINOC_QT(pers_j) = SICK_TOTBEN_QT(pers_j) - SICK_BENINOC_QT(pers_j)
        
        !Variable constructed for the FAMAL module
        !SICK_DISABBEN_QT: Identification of the fact that the beneficiary receives disability benefits
        !1 the individual receives disability benefits as wage earner 
        !2 the individual receives disability benefits as non statutory civil servant 
        !3 the beneficiary receives disability benefits as self employed

!Aangepast revisie september 2011
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.OR.SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2) THEN
            !The individual receives benefits for sickness already for more than 6 months in the given quarter
            IF (InRIZIV(pers_j).EQ.1) THEN
                SICK_DISABBEN_QT(pers_j) = 1
            END IF
            IF (InNIC(pers_j).EQ.1.AND.NIC_DAGENAO_2008(pers_j).GT.156) THEN
                SICK_DISABBEN_QT(pers_j) = 1
            END IF
            !individual receives disability benefits, other than maternity benefits
            IF (SICK_DISEMPNM_QT(pers_j,2).GT.0) THEN
                SICK_DISABBEN_QT(pers_j) = 1
            END IF
            !individual receives disability maternity benefits
            IF (SICK_ELIGPRI2_QT(pers_j).EQ.2) THEN
                SICK_DISABBEN_QT(pers_j) = 1
            END IF
            !The individual receives benefits for industrial accidents already for more than 6 months in the given quarter
            IF (FATFAO_BENEF_2008(pers_j).EQ.1.AND.SICK_DAYSPRI3_QT(pers_j).GT.46) THEN
                SICK_DISABBEN_QT(pers_j) = 1
            END IF
            !The individual receives benefits for occupational diseases already for more than 6 months in the given quarter
            IF (FBZFMB_BENEF_2008(pers_j).EQ.1.AND.SICK_DAYSPRI4_QT(pers_j).GT.46) THEN
                SICK_DISABBEN_QT(pers_j) = 1
            END IF
        END IF    
        !public labour market: non statutory
        !remark: only identification if the status is still equal to 0, we give advantage to the wage earner scheme
        IF ((SICK_TYPEINSU_PUB_QT(pers_j).EQ.1.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.2).AND.SICK_DISABBEN_QT(pers_j).EQ.0) THEN
            !The individual receives benefits for sickness already for more than 6 months in the given quarter
            IF (InRIZIV(pers_j).EQ.1) THEN
                SICK_DISABBEN_QT(pers_j) = 2
            END IF
            IF (InNIC(pers_j).EQ.1.AND.NIC_DAGENAO_2008(pers_j).GT.156) THEN
                SICK_DISABBEN_QT(pers_j) = 2
            END IF
            !individual receives disability benefits, other than maternity benefits
            IF (SICK_DISEMPNM_QT(pers_j,2).GT.0) THEN
                SICK_DISABBEN_QT(pers_j) = 2
            END IF
            !individual receives disability maternity benefits
            IF (SICK_ELIGPUB2_QT(pers_j).EQ.2) THEN
                SICK_DISABBEN_QT(pers_j) = 2
            END IF
            !The individual receives benefits for industrial accidents already for more than 6 months in the given quarter
            IF (FATFAO_BENEF_2008(pers_j).EQ.1.AND.SICK_DAYSPUB3_QT(pers_j).GT.46) THEN
                SICK_DISABBEN_QT(pers_j) = 2
            END IF
            !The individual receives benefits for occupational diseases already for more than 6 months in the given quarter
            IF (FBZFMB_BENEF_2008(pers_j).EQ.1.AND.SICK_DAYSPUB4_QT(pers_j).GT.46) THEN
                SICK_DISABBEN_QT(pers_j) = 2
            END IF
        END IF    
        !Self employed, disability benefits or disability maternity benefits
        !remark: only identification if the status is still equal to 0, we give advantage to the wage earner scheme
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_DISABBEN_QT(pers_j).EQ.0) THEN
            !The individual receives benefits for sickness already for more than 6 months in the given quarter
            IF (InRIZIV(pers_j).EQ.1) THEN
                SICK_DISABBEN_QT(pers_j) = 3
            END IF
            IF (InNIC(pers_j).EQ.1.AND.NIC_DAGENAO_2008(pers_j).GT.156) THEN
                SICK_DISABBEN_QT(pers_j) = 3
            END IF
            !individual receives disability benefits, other than maternity benefits
            IF (SICK_DISSELNM_QT(pers_j,2).GT.0) THEN
                SICK_DISABBEN_QT(pers_j) = 3
            END IF
            !individual receives disability maternity benefits
            IF (SICK_ELIGSEL2_QT(pers_j).EQ.2) THEN
                SICK_DISABBEN_QT(pers_j) = 3
            END IF
        END IF    
        !public labour market: statutory
        !remark: only identification if the status is still equal to 0, we give advantage to the wage earner scheme
        IF ((SICK_TYPEINSU_PUB_QT(pers_j).EQ.3.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.4).AND.SICK_DISABBEN_QT(pers_j).EQ.0) THEN
            !The individual receives benefits for sickness already for more than 6 months in the given quarter
            IF (InRIZIV(pers_j).EQ.1) THEN
                SICK_DISABBEN_QT(pers_j) = 4
            END IF
            IF (InNIC(pers_j).EQ.1.AND.NIC_DAGENAO_2008(pers_j).GT.156) THEN
                SICK_DISABBEN_QT(pers_j) = 4
            END IF
            !individual receives disability benefits, other than maternity benefits
            IF (SICK_DISEMPNM_QT(pers_j,2).GT.0) THEN
                SICK_DISABBEN_QT(pers_j) = 4
            END IF
            !individual receives disability maternity benefits
            IF (SICK_ELIGPUB2_QT(pers_j).EQ.2) THEN
                SICK_DISABBEN_QT(pers_j) = 4
            END IF
            !The individual receives benefits for industrial accidents already for more than 6 months in the given quarter
            IF (FATFAO_BENEF_2008(pers_j).EQ.1.AND.SICK_DAYSPUB3_QT(pers_j).GT.46) THEN
                SICK_DISABBEN_QT(pers_j) = 4
            END IF
            !The individual receives benefits for occupational diseases already for more than 6 months in the given quarter
            IF (FBZFMB_BENEF_2008(pers_j).EQ.1.AND.SICK_DAYSPUB4_QT(pers_j).GT.46) THEN
                SICK_DISABBEN_QT(pers_j) = 4
            END IF
        END IF    

!Aangepast na herzieningen september 2011
!        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
!            !The individual receives benefits for sickness already for more than 6 months in the given quarter
!            IF (SICK_DAYSPRI1_QT(pers_j).GT.39) THEN
!                SICK_DISABBEN_QT(pers_j) = 1
!            END IF
!            !The individual receives maternity benefits during period of disability or receives maternity benefits and is
!            !sick for more than 6 months
!            IF ((SICK_ELIGPRI2_QT(pers_j).EQ.2).OR.(SICK_ELIGPRI2_QT(pers_j).EQ.1.AND.SICK_DAYSPRI2_QT(pers_j).GT.39)) THEN 
!                SICK_DISABBEN_QT(pers_j) = 1
!            END IF
!            !The individual receives benefits for industrial accidents already for more than 6 months in the given quarter
!            IF (SICK_DAYSPRI3_QT(pers_j).GT.46) THEN
!                SICK_DISABBEN_QT(pers_j) = 1
!            END IF
!            !The individual receives benefits for occupational diseases already for more than 6 months in the given quarter
!            IF (SICK_DAYSPRI4_QT(pers_j).GT.46) THEN
!                SICK_DISABBEN_QT(pers_j) = 1
!            END IF
!        END IF    
!        !public labour market: non statutory
!        !remark: only identification if the status is still equal to 0, we give advantage to the wage earner scheme
!        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/)).AND.SICK_DISABBEN_QT(pers_j).EQ.0) THEN
!            !The individual receives benefits for sickness already for more than 6 months in the given quarter
!            IF (SICK_DAYSPUB1_QT(pers_j).GT.39) THEN
!                SICK_DISABBEN_QT(pers_j) = 2
!            END IF
!            !The individual receives maternity benefits during period of disability or receives maternity benefits and is
!            !sick for more than 6 months
!            IF ((SICK_ELIGPUB2_QT(pers_j).EQ.2).OR.(SICK_ELIGPUB2_QT(pers_j).EQ.1.AND.SICK_DAYSPUB2_QT(pers_j).GT.39)) THEN 
!                SICK_DISABBEN_QT(pers_j) = 2
!            END IF
!            !The individual receives benefits for industrial accidents already for more than 6 months in the given quarter
!            IF (SICK_DAYSPUB3_QT(pers_j).GT.46) THEN
!                SICK_DISABBEN_QT(pers_j) = 2
!            END IF
!            !The individual receives benefits for occupational diseases already for more than 6 months in the given quarter
!            IF (SICK_DAYSPUB4_QT(pers_j).GT.46) THEN
!                SICK_DISABBEN_QT(pers_j) = 2
!            END IF
!        END IF    
!        !Self employed, disability benefits or disability maternity benefits
!        !remark: only identification if the status is still equal to 0, we give advantage to the wage earner scheme
!        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_DISABBEN_QT(pers_j).EQ.0) THEN
!            !The individual receives benefits for sickness already for more than 6 months in the given quarter
!            IF (SICK_DAYSSEL1_QT(pers_j).GT.39) THEN
!                SICK_DISABBEN_QT(pers_j) = 3
!            END IF
!            !The individual receives maternity benefits during period of disability 
!            IF (SICK_ELIGSEL2_QT(pers_j).EQ.2) THEN 
!                SICK_DISABBEN_QT(pers_j) = 3
!            END IF
!        END IF    
!        !public labour market: statutory
!        !remark: only identification if the status is still equal to 0, we give advantage to the wage earner scheme
!        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/)).AND.SICK_DISABBEN_QT(pers_j).EQ.0) THEN
!            !The individual receives benefits for sickness already for more than 6 months in the given quarter
!            IF (SICK_DAYSPUB1_QT(pers_j).GT.39) THEN
!                SICK_DISABBEN_QT(pers_j) = 4
!            END IF
!            !The individual receives maternity benefits during period of disability or receives maternity benefits and is
!            !sick for more than 6 months
!            IF (SICK_ELIGPUB2_QT(pers_j).EQ.2.OR.(SICK_ELIGPUB2_QT(pers_j).EQ.1.AND.SICK_DAYSPUB2_QT(pers_j).GT.39)) THEN 
!                SICK_DISABBEN_QT(pers_j) = 4
!            END IF
!            !The individual receives benefits for industrial accidents already for more than 6 months in the given quarter
!            IF (SICK_DAYSPUB3_QT(pers_j).GT.46) THEN
!                SICK_DISABBEN_QT(pers_j) = 4
!            END IF
!            !The individual receives benefits for occupational diseases already for more than 6 months in the given quarter
!            IF (SICK_DAYSPUB4_QT(pers_j).GT.46) THEN
!                SICK_DISABBEN_QT(pers_j) = 4
!            END IF
!       END IF    
        
    END DO

END SUBROUTINE SICK_EXCHANGE

!==========================================================!
!Produce some aggregate statistics to check the constuction
!==========================================================!

SUBROUTINE SICK_TEST

    INTEGER                                                 :: pers_j,i
    INTEGER                                                 :: NumAnal                              !Number of summary statistics computed 
    REAL(KIND=DPRECI)                                       :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: SICK_RESULT_TEST                     !matrix with test results
    CHARACTER(len=75),ALLOCATABLE,DIMENSION(:)              :: SICK_LAB_TEST                        !Labels of test variables
    REAL(KIND=DPRECI),DIMENSION(22)                         :: SumGross                             !Vector with quarterly amounts on benefits

    WRITE(15,'(/,''Print of test statistics'')')
    CALL DRAWLINE(15,136,'=')    

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))

    !Compute aggregate statistics on the number of sick or disabled

    NumAnal = 2
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not sick or disabled"
    SICK_LAB_TEST(2) = "Sick or disabled"

    DO pers_j=1,n_rec
        DO i=1,NumAnal
            IF (SICK_RISKGLOB_QT(pers_j).EQ.i-1) THEN
                SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_RISKGLOB_QT: identification of being sick or disabled'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)
    
    !Compute aggregate statistics on the family charge of the sick or disabled

    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not sick or disabled"
    SICK_LAB_TEST(2) = "sick or disabled with dependent family "
    SICK_LAB_TEST(3) = "sick or disabled who is single"
    SICK_LAB_TEST(4) = "sick or disabled without dependent family and who has not lost his sole sourc"

    DO pers_j=1,n_rec
        DO i=1,NumAnal
            IF (SICK_FAMSIT_QT(pers_j).EQ.i-1) THEN
                SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_FAMSIT_QT: family charge of the unemployed'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute aggregate statistics on those insured as private labour market wage earners
    
    NumAnal = 3
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not sick or disabled"
    SICK_LAB_TEST(2) = "Individual is insured as a blue collar worker "
    SICK_LAB_TEST(3) = "Individual is insured as a white collar worker"

    DO pers_j=1,n_rec
        DO i=1,NumAnal
            IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.i-1) THEN
                SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_TYPEINSU_PRIV_QT: insured as private labour market wage earner'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute aggregate statistics on those insured as public labour market wage earners
    
    NumAnal = 5
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not sick or disabled"
    SICK_LAB_TEST(2) = "Individual is insured as a blue collar worker without statutory service"
    SICK_LAB_TEST(3) = "Individual is insured as a white collar worker without statutory service"
    SICK_LAB_TEST(4) = "Individual is insured as a blue collar worker with statutory service"
    SICK_LAB_TEST(5) = "Individual is insured as a white collar worker with statutory service"

    DO pers_j=1,n_rec
        DO i=1,NumAnal
            IF (SICK_TYPEINSU_PUB_QT(pers_j).EQ.i-1) THEN
                SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_TYPEINSU_PUB_QT: insured as public labour market wage earner'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute aggregate statistics on those insured as self employed
    
    NumAnal = 2
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not sick or disabled"
    SICK_LAB_TEST(2) = "Individual is insured as a self employed"

    DO pers_j=1,n_rec
        DO i=1,NumAnal
            IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.i-1) THEN
                SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_TYPEINSU_SELF_QT: insured as self employed'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

!COMPUTE STATISTICS ON SICKNESS AND DISABILITY OF THOSE INSURED AS PRIVATE LABOUR MARKET WAGE EARNERS

    Write(15,'(/,''Some statistics on sickness and disability benefits of those insured as wage earner on the &
    &private labour market '')')
    CALL DRAWLINE(15,100,'-')    

    !Compute statistics on those eligible for benefits 
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is eligible for priv. lab. market sick or disab benefits"
    SICK_LAB_TEST(2) = "Individual is eligible for priv. lab. market maternity benefits"
    SICK_LAB_TEST(3) = "Individual is eligible for priv. lab. market benefits for industri acc"
    SICK_LAB_TEST(4) = "Individual is eligible for priv. lab. market benefits for occ. diss"

    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI1_QT(pers_j).NE.0) THEN
            SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
            SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (SICK_ELIGPRI2_QT(pers_j).NE.0) THEN
            SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
            SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (SICK_ELIGPRI3_QT(pers_j).NE.0) THEN
            SICK_RESULT_TEST(3,1) = SICK_RESULT_TEST(3,1) + 1
            SICK_RESULT_TEST(3,2) = SICK_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (SICK_ELIGPRI4_QT(pers_j).NE.0) THEN
            SICK_RESULT_TEST(4,1) = SICK_RESULT_TEST(4,1) + 1
            SICK_RESULT_TEST(4,2) = SICK_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those eligible for a benefit and insured as wage earner on the private labour market'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute statistics on those not eligible for benefits but with benefits different from 0
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is not eligible for priv. lab. market sick or disab benefits but benefit is <> 0"
    SICK_LAB_TEST(2) = "Individual is not  eligible for priv. lab. market maternity benefits but benefit is <> 0"
    SICK_LAB_TEST(3) = "Individual is not  eligible for priv. lab. market benefits for industri acc but benefit is <> 0"
    SICK_LAB_TEST(4) = "Individual is not  eligible for priv. lab. market benefits for occ. diss  but benefit is <> 0"

    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI1_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPRI1_QT(pers_j)).NE.0) THEN
            SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
            SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (SICK_ELIGPRI2_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPRI2_QT(pers_j)).NE.0) THEN
            SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
            SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (SICK_ELIGPRI3_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPRI3_QT(pers_j)).NE.0) THEN
            SICK_RESULT_TEST(3,1) = SICK_RESULT_TEST(3,1) + 1
            SICK_RESULT_TEST(3,2) = SICK_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (SICK_ELIGPRI4_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPRI4_QT(pers_j)).NE.0) THEN
            SICK_RESULT_TEST(4,1) = SICK_RESULT_TEST(4,1) + 1
            SICK_RESULT_TEST(4,2) = SICK_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those NOT eligible for a benefit but with computed benefit <> 0'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute statistics on those eligible for benefits but with benefits equal to 0
    
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is eligible for priv. lab. market sick or disab benefits but benefit is 0"
    SICK_LAB_TEST(2) = "Individual is eligible for priv. lab. market maternity benefits but benefit is 0"
    SICK_LAB_TEST(3) = "Individual is eligible for priv. lab. market benefits for industri acc but benefit is 0"
    SICK_LAB_TEST(4) = "Individual is eligible for priv. lab. market benefits for occ. diss  but benefit is 0"

    DO pers_j=1,n_rec
        IF (SICK_ELIGPRI1_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPRI1_QT(pers_j)).EQ.0) THEN
            SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
            SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (SICK_ELIGPRI2_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPRI2_QT(pers_j)).EQ.0) THEN
            SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
            SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (SICK_ELIGPRI3_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPRI3_QT(pers_j)).EQ.0) THEN
            SICK_RESULT_TEST(3,1) = SICK_RESULT_TEST(3,1) + 1
            SICK_RESULT_TEST(3,2) = SICK_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (SICK_ELIGPRI4_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPRI4_QT(pers_j)).EQ.0) THEN
            SICK_RESULT_TEST(4,1) = SICK_RESULT_TEST(4,1) + 1
            SICK_RESULT_TEST(4,2) = SICK_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those eligible for a benefit but with computed benefit equal to 0'')')
    Write(15,'(''0 possible for disabled because observed dates might be out of the observed quarter'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute aggregate statistics
    NumAnal = 18

    !Allocate and initialise test result
    ALLOCATE(SICK_RESULT_TEST(NumAnal,4))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    SICK_LAB_TEST(1) = "Last daily benefit computed for sick or disabled"
    SICK_LAB_TEST(2) = "Quarterly benefit paid for sickness or disability: total amount received"
    SICK_LAB_TEST(3) = "Quarterly benefit paid for sickness or disability: amount paid by the employer"
    SICK_LAB_TEST(4) = "Quarterly benefit paid for sickness paid by soc sec budget "
    SICK_LAB_TEST(5) = "Quarterly benefit paid for disability  paid by soc sec budget"
    SICK_LAB_TEST(6) = "Last daily benefit computed for supplement of blue collar workers"
    SICK_LAB_TEST(7) = "Quarterly benefit paid for supplement of blue collar workers: total amount received"
    SICK_LAB_TEST(8) = "Quarterly benefit paid for supplement of blue collar workers: amount paid by the employer"
    SICK_LAB_TEST(9) = "Daily supplement for help of a third person (individual is insured as wage earner on the private"         
    SICK_LAB_TEST(10)= "Quarterly supplement for help of a third person (individual is insured as wage earner on the private"         
    SICK_LAB_TEST(11)= "Last daily benefit computed for maternity leave"
    SICK_LAB_TEST(12) = "Quarterly benefit paid for maternity leave"
    SICK_LAB_TEST(13) = "Daily benefit computed for industrial accidents"
    SICK_LAB_TEST(14) = "Quarterly benefit paid for industrial accidents: total amount received"
    SICK_LAB_TEST(15) = "Quarterly benefit paid for industrial accidents: amount paid by the employer"
    SICK_LAB_TEST(16) = "Daily benefit computed for occupational diseases"
    SICK_LAB_TEST(17) = "Quarterly benefit paid for occupational diseases: total amount received"
    SICK_LAB_TEST(18) = "Quarterly benefit paid for occupational diseases: amount paid by the employer"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = SICK_BENPRI1_DAY(pers_j)
        SumGross(2) = SICK_BENPRI1_QT(pers_j)
        SumGross(3) = SICK_EMPBPRI1_QT(pers_j)
        IF (SICK_ELIGPRI1_QT(pers_j).EQ.1) THEN
            SumGross(4) = SICK_BENPRI1_QT(pers_j) - SICK_EMPBPRI1_QT(pers_j)
        ELSE IF (SICK_ELIGPRI1_QT(pers_j).EQ.2) THEN
            SumGross(5) = SICK_BENPRI1_QT(pers_j) - SICK_EMPBPRI1_QT(pers_j)
        END IF
        SumGross(6) = SICK_SUPPRI1_DAY(pers_j)
        SumGross(7) = SICK_SUPPRI1_QT(pers_j)
        SumGross(8) = SICK_EMPSPRI1_QT(pers_j)
        SumGross(9) = SICK_BETHPRIV_DAY(pers_j)
        SumGross(10) = SICK_BETHPRIV_QT(pers_j)
        SumGross(11) = SICK_BENPRI2_DAY(pers_j)
        SumGross(12) = SICK_BENPRI2_QT(pers_j)
        SumGross(13) = SICK_BENPRI3_DAY(pers_j)
        SumGross(14) = SICK_BENPRI3_QT(pers_j)
        SumGross(15) = SICK_EMPBPRI3_QT(pers_j)
        SumGross(16) = SICK_BENPRI4_DAY(pers_j)
        SumGross(17) = SICK_BENPRI4_QT(pers_j)
        SumGross(18) = SICK_EMPBPRI4_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                SICK_RESULT_TEST(j,1)=SICK_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                SICK_RESULT_TEST(j,2)=SICK_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(SICK_RESULT_TEST(i,2)).NE.0) THEN
            SICK_RESULT_TEST(i,3)=SICK_RESULT_TEST(i,1)/SICK_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,4)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST) 

    !Compute aggregate statistics on those eligible for industrial accidents benefit: private labour market
    
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not eligible for industrial accident benefit as private lab. market insured"
    SICK_LAB_TEST(2) = "the employee is in temporary full disablement"
    SICK_LAB_TEST(3) = "the employee is in temporary partial disablement"
    SICK_LAB_TEST(4) = "the employee is in permanent disablement"

    DO pers_j=1,n_rec
        DO i=1,NumAnal
            IF (SICK_TYDBPRI3_QT(pers_j).EQ.i-1) THEN
                SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_TYDBPRI3_QT: Type of benefit for an industrial accident the individual,&
    & insured as wage earner on the private labour market, is entitled to'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute aggregate statistics on those eligible for occupational diseases: private labour market
    
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not eligible for occupational disease benefit as private lab. market insured"
    SICK_LAB_TEST(2) = "the employee is in temporary full disablement"
    SICK_LAB_TEST(3) = "the employee is in temporary partial disablement"
    SICK_LAB_TEST(4) = "the employee is in permanent disablement"

    DO pers_j=1,n_rec
        DO i=1,NumAnal
            IF (SICK_TYDBPRI4_QT(pers_j).EQ.i-1) THEN
                SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_TYDBPRI4_QT: Type of benefit for an occupational disease the individual,&
    & insured as wage earner on the private labour market, is entitled to'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

!COMPUTE STATISTICS ON SICKNESS AND DISABILITY OF THOSE INSURED AS PUBLIC LABOUR MARKET WAGE EARNERS
!RESULTS FOR Civil servants without statutory service

    Write(15,'(/,''Results on Civil servants without statutory service '')')
    CALL DRAWLINE(15,100,'-')    

    !Compute statistics on those eligible for benefits 
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is eligible for pub. lab. market sick or disab benefits"
    SICK_LAB_TEST(2) = "Individual is eligible for pub. lab. market maternity benefits"
    SICK_LAB_TEST(3) = "Individual is eligible for pub. lab. market benefits for industri acc"
    SICK_LAB_TEST(4) = "Individual is eligible for pub. lab. market benefits for occ. diss"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (SICK_ELIGPUB1_QT(pers_j).NE.0) THEN
                SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
                SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB2_QT(pers_j).NE.0) THEN
                SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
                SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB3_QT(pers_j).NE.0) THEN
                SICK_RESULT_TEST(3,1) = SICK_RESULT_TEST(3,1) + 1
                SICK_RESULT_TEST(3,2) = SICK_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB4_QT(pers_j).NE.0) THEN
                SICK_RESULT_TEST(4,1) = SICK_RESULT_TEST(4,1) + 1
                SICK_RESULT_TEST(4,2) = SICK_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those eligible for a benefit and insured as wage earner on the private labour market'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute statistics on those not eligible for benefits but with benefits different from 0
    
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is not eligible for pub. lab. market sick or disab benefits but benefit is <> 0"
    SICK_LAB_TEST(2) = "Individual is not  eligible for pub. lab. market maternity benefits but benefit is <> 0"
    SICK_LAB_TEST(3) = "Individual is not  eligible for pub. lab. market benefits for industri acc but benefit is <> 0"
    SICK_LAB_TEST(4) = "Individual is not  eligible for pub. lab. market benefits for occ. diss  but benefit is <> 0"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (SICK_ELIGPUB1_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPUB1_QT(pers_j)).NE.0) THEN
                SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
                SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB2_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPUB2_QT(pers_j)).NE.0) THEN
                SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
                SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB3_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPUB3_QT(pers_j)).NE.0) THEN
                SICK_RESULT_TEST(3,1) = SICK_RESULT_TEST(3,1) + 1
                SICK_RESULT_TEST(3,2) = SICK_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB4_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPUB4_QT(pers_j)).NE.0) THEN
                SICK_RESULT_TEST(4,1) = SICK_RESULT_TEST(4,1) + 1
                SICK_RESULT_TEST(4,2) = SICK_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those NOT eligible for a benefit but with computed benefit <> 0'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute statistics on those eligible for benefits but with benefits equal to 0
    
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is eligible for pub. lab. market sick or disab benefits but benefit is 0"
    SICK_LAB_TEST(2) = "Individual is eligible for pub. lab. market maternity benefits but benefit is 0"
    SICK_LAB_TEST(3) = "Individual is eligible for pub. lab. market benefits for industri acc but benefit is 0"
    SICK_LAB_TEST(4) = "Individual is eligible for pub. lab. market benefits for occ. diss  but benefit is 0"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            IF (SICK_ELIGPUB1_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPUB1_QT(pers_j)).EQ.0) THEN
                SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
                SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB2_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPUB2_QT(pers_j)).EQ.0) THEN
                SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
                SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB3_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPUB3_QT(pers_j)).EQ.0) THEN
                SICK_RESULT_TEST(3,1) = SICK_RESULT_TEST(3,1) + 1
                SICK_RESULT_TEST(3,2) = SICK_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB4_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPUB4_QT(pers_j)).EQ.0) THEN
                SICK_RESULT_TEST(4,1) = SICK_RESULT_TEST(4,1) + 1
                SICK_RESULT_TEST(4,2) = SICK_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those eligible for a benefit but with computed benefit equal to 0'')')
    Write(15,'(''0 possible for disabled because observed dates might be out of the observed quarter'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    Write(15,'(/,''Some statistics on sickness and disability benefits of those insured as wage earner on the &
    &public labour market '')')

    !Compute aggregate statistics
    NumAnal = 18

    !Allocate and initialise test result
    ALLOCATE(SICK_RESULT_TEST(NumAnal,4))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    SICK_LAB_TEST(1) = "Last daily benefit computed for sick or disabled"
    SICK_LAB_TEST(2) = "Quarterly benefit paid for sickness or disability: total amount received"
    SICK_LAB_TEST(3) = "Quarterly benefit paid for sickness or disability: amount paid by the employer"
    SICK_LAB_TEST(4) = "Quarterly benefit paid for sickness paid by soc sec budget "
    SICK_LAB_TEST(5) = "Quarterly benefit paid for disability  paid by soc sec budget"
    SICK_LAB_TEST(6) = "Last daily benefit computed for supplement of blue collar workers"
    SICK_LAB_TEST(7) = "Quarterly benefit paid for supplement of blue collar workers: total amount received"
    SICK_LAB_TEST(8) = "Quarterly benefit paid for supplement of blue collar workers: amount paid by the employer"
    SICK_LAB_TEST(9) = "Daily supplement for help of a third person (individual is insured as wage earner on the public"         
    SICK_LAB_TEST(10)= "Quarterly supplement for help of a third person (individual is insured as wage earner on the public"         
    SICK_LAB_TEST(11)= "Last daily benefit computed for maternity leave"
    SICK_LAB_TEST(12) = "Quarterly benefit paid for maternity leave"
    SICK_LAB_TEST(13) = "Daily benefit computed for industrial accidents"
    SICK_LAB_TEST(14) = "Quarterly benefit paid for industrial accidents: total amount received"
    SICK_LAB_TEST(15) = "Quarterly benefit paid for industrial accidents: amount paid by the employer"
    SICK_LAB_TEST(16) = "Daily benefit computed for occupational diseases"
    SICK_LAB_TEST(17) = "Quarterly benefit paid for occupational diseases: total amount received"
    SICK_LAB_TEST(18) = "Quarterly benefit paid for occupational diseases: amount paid by the employer"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            SumGross(1:NumAnal)=0.D0
            !Compute quarterly amounts
            SumGross(1) = SICK_BENPUB1_DAY(pers_j)
            SumGross(2) = SICK_BENPUB1_QT(pers_j)
            SumGross(3) = SICK_EMPBPUB1_QT(pers_j)
            IF (SICK_ELIGPUB1_QT(pers_j).EQ.1) THEN
                SumGross(4) = SICK_BENPUB1_QT(pers_j) - SICK_EMPBPUB1_QT(pers_j)
            ELSE IF (SICK_ELIGPUB1_QT(pers_j).EQ.2) THEN
                SumGross(5) = SICK_BENPUB1_QT(pers_j) - SICK_EMPBPUB1_QT(pers_j)
            END IF
            SumGross(6) = SICK_SUPPUB1_DAY(pers_j)
            SumGross(7) = SICK_SUPPUB1_QT(pers_j)
            SumGross(8) = SICK_EMPSPUB1_QT(pers_j)
            SumGross(9) = SICK_BETHPUB_DAY(pers_j)
            SumGross(10)= SICK_BETHPUB_QT(pers_j)
            SumGross(11)= SICK_BENPUB2_DAY(pers_j)
            SumGross(12) = SICK_BENPUB2_QT(pers_j)
            SumGross(13) = SICK_BENPUB3_DAY(pers_j)
            SumGross(14) = SICK_BENPUB3_QT(pers_j)
            SumGross(15) = SICK_EMPBPUB3_QT(pers_j)
            SumGross(16) = SICK_BENPUB4_DAY(pers_j)
            SumGross(17) = SICK_BENPUB4_QT(pers_j)
            SumGross(18) = SICK_EMPBPUB4_QT(pers_j)
            DO j=1,NumAnal
                IF (INT(100.D0*SumGross(j)).NE.0) THEN
                    SICK_RESULT_TEST(j,1)=SICK_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                    SICK_RESULT_TEST(j,2)=SICK_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
                END IF
            END DO
        END IF
    END DO

    DO i=1,NumAnal
        IF (INT(SICK_RESULT_TEST(i,2)).NE.0) THEN
            SICK_RESULT_TEST(i,3)=SICK_RESULT_TEST(i,1)/SICK_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,4)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST) 

    !Compute aggregate statistics on those eligible for industrial accidents benefit: public labour market
    
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not eligible for industrial accident benefit as public lab. market insured"
    SICK_LAB_TEST(2) = "the employee is in temporary full disablement"
    SICK_LAB_TEST(3) = "the employee is in temporary partial disablement"
    SICK_LAB_TEST(4) = "the employee is in permanent disablement"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            DO i=1,NumAnal
                IF (SICK_TYDBPUB3_QT(pers_j).EQ.i-1) THEN
                    SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                    SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            END DO
        END IF
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_TYDBPUB3_QT: Type of benefit for an industrial accident the individual,&
    & insured as wage earner on the public labour market, is entitled to'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute aggregate statistics on those eligible for occupational diseases: public labour market
    
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not eligible for occupational disease benefit as public lab. market insured"
    SICK_LAB_TEST(2) = "the employee is in temporary full disablement"
    SICK_LAB_TEST(3) = "the employee is in temporary partial disablement"
    SICK_LAB_TEST(4) = "the employee is in permanent disablement"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
            DO i=1,NumAnal
                IF (SICK_TYDBPUB4_QT(pers_j).EQ.i-1) THEN
                    SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                    SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            END DO
        END IF
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_TYDBPUB4_QT: Type of benefit for an occupational disease the individual,&
    & insured as wage earner on the public labour market, is entitled to'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

!RESULTS FOR Civil servants with statutory service

    Write(15,'(/,''Results on Civil servants with statutory service '')')
    CALL DRAWLINE(15,100,'-')    

    !Compute statistics on those eligible for benefits 
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is eligible for pub. lab. market sick or disab benefits"
    SICK_LAB_TEST(2) = "Individual is eligible for pub. lab. market maternity benefits"
    SICK_LAB_TEST(3) = "Individual is eligible for pub. lab. market benefits for industri acc"
    SICK_LAB_TEST(4) = "Individual is eligible for pub. lab. market benefits for occ. diss"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            IF (SICK_ELIGPUB1_QT(pers_j).NE.0) THEN
                SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
                SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB2_QT(pers_j).NE.0) THEN
                SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
                SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB3_QT(pers_j).NE.0) THEN
                SICK_RESULT_TEST(3,1) = SICK_RESULT_TEST(3,1) + 1
                SICK_RESULT_TEST(3,2) = SICK_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB4_QT(pers_j).NE.0) THEN
                SICK_RESULT_TEST(4,1) = SICK_RESULT_TEST(4,1) + 1
                SICK_RESULT_TEST(4,2) = SICK_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those eligible for a benefit and insured as wage earner on the private labour market'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute statistics on those not eligible for benefits but with benefits different from 0
    
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is not eligible for pub. lab. market sick or disab benefits but benefit is <> 0"
    SICK_LAB_TEST(2) = "Individual is not  eligible for pub. lab. market maternity benefits but benefit is <> 0"
    SICK_LAB_TEST(3) = "Individual is not  eligible for pub. lab. market benefits for industri acc but benefit is <> 0"
    SICK_LAB_TEST(4) = "Individual is not  eligible for pub. lab. market benefits for occ. diss  but benefit is <> 0"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            IF (SICK_ELIGPUB1_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPUB1_QT(pers_j)).NE.0) THEN
                SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
                SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB2_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPUB2_QT(pers_j)).NE.0) THEN
                SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
                SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB3_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPUB3_QT(pers_j)).NE.0) THEN
                SICK_RESULT_TEST(3,1) = SICK_RESULT_TEST(3,1) + 1
                SICK_RESULT_TEST(3,2) = SICK_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB4_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENPUB4_QT(pers_j)).NE.0) THEN
                SICK_RESULT_TEST(4,1) = SICK_RESULT_TEST(4,1) + 1
                SICK_RESULT_TEST(4,2) = SICK_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those NOT eligible for a benefit but with computed benefit <> 0'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute statistics on those eligible for benefits but with benefits equal to 0
    
    NumAnal = 4
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is eligible for pub. lab. market sick or disab benefits but benefit is 0"
    SICK_LAB_TEST(2) = "Individual is eligible for pub. lab. market maternity benefits but benefit is 0"
    SICK_LAB_TEST(3) = "Individual is eligible for pub. lab. market benefits for industri acc but benefit is 0"
    SICK_LAB_TEST(4) = "Individual is eligible for pub. lab. market benefits for occ. diss  but benefit is 0"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            IF (SICK_ELIGPUB1_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPUB1_QT(pers_j)).EQ.0) THEN
                SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
                SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB2_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPUB2_QT(pers_j)).EQ.0) THEN
                SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
                SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB3_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPUB3_QT(pers_j)).EQ.0) THEN
                SICK_RESULT_TEST(3,1) = SICK_RESULT_TEST(3,1) + 1
                SICK_RESULT_TEST(3,2) = SICK_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGPUB4_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENPUB4_QT(pers_j)).EQ.0) THEN
                SICK_RESULT_TEST(4,1) = SICK_RESULT_TEST(4,1) + 1
                SICK_RESULT_TEST(4,2) = SICK_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those eligible for a benefit but with computed benefit equal to 0'')')
    Write(15,'(''0 possible for disabled because observed dates might be out of the observed quarter'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    Write(15,'(/,''Some statistics on sickness and disability benefits of those insured as wage earner on the &
    &public labour market '')')

    !Compute aggregate statistics
    NumAnal = 14

    !Allocate and initialise test result
    ALLOCATE(SICK_RESULT_TEST(NumAnal,4))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    SICK_LAB_TEST(1) = "Daily benefit computed for sick or disabled"
    SICK_LAB_TEST(2) = "Quarterly benefit paid for sickness or disability: total amount received"
    SICK_LAB_TEST(3) = "Quarterly benefit paid for sickness or disability: amount paid by the employer"
    SICK_LAB_TEST(4) = "Quarterly benefit paid for sickness paid by soc sec budget "
    SICK_LAB_TEST(5) = "Quarterly benefit paid for disability  paid by soc sec budget"
    SICK_LAB_TEST(6) = "Daily benefit computed for maternity leave"
    SICK_LAB_TEST(7)  = "Quarterly benefit paid for maternity leave"
    SICK_LAB_TEST(8)  = "Quarterly benefit paid by the employer for maternity leave"
    SICK_LAB_TEST(9)  = "Daily benefit computed for industrial accidents"
    SICK_LAB_TEST(10)  = "Quarterly benefit paid for industrial accidents: total amount received"
    SICK_LAB_TEST(11)  = "Quarterly benefit paid for industrial accidents: amount paid by the employer"
    SICK_LAB_TEST(12) = "Daily benefit computed for occupational diseases"
    SICK_LAB_TEST(13) = "Quarterly benefit paid for occupational diseases: total amount received"
    SICK_LAB_TEST(14) = "Quarterly benefit paid for occupational diseases: amount paid by the employer"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            SumGross(1:NumAnal)=0.D0
            !Compute quarterly amounts
            SumGross(1) = SICK_BENPUB1_DAY(pers_j)
            SumGross(2) = SICK_BENPUB1_QT(pers_j)
            SumGross(3) = SICK_EMPBPUB1_QT(pers_j)
            IF (SICK_ELIGPUB1_QT(pers_j).EQ.1) THEN
                SumGross(4) = SICK_BENPUB1_QT(pers_j) - SICK_EMPBPUB1_QT(pers_j)
            ELSE IF (SICK_ELIGPUB1_QT(pers_j).EQ.2) THEN
                SumGross(5) = SICK_BENPUB1_QT(pers_j) - SICK_EMPBPUB1_QT(pers_j)
            END IF
            SumGross(6) = SICK_BENPUB2_DAY(pers_j)
            SumGross(7) = SICK_BENPUB2_QT(pers_j)
            SumGross(8) = SICK_EMPBPUB2_QT(pers_j)
            SumGross(9) = SICK_BENPUB3_DAY(pers_j)
            SumGross(10)= SICK_BENPUB3_QT(pers_j)
            SumGross(11)= SICK_EMPBPUB3_QT(pers_j)
            SumGross(12) = SICK_BENPUB4_DAY(pers_j)
            SumGross(13) = SICK_BENPUB4_QT(pers_j)
            SumGross(14) = SICK_EMPBPUB4_QT(pers_j)
            DO j=1,NumAnal
                IF (INT(100.D0*SumGross(j)).NE.0) THEN
                    SICK_RESULT_TEST(j,1)=SICK_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                    SICK_RESULT_TEST(j,2)=SICK_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
                END IF
            END DO
        END IF
    END DO

    DO i=1,NumAnal
        IF (INT(SICK_RESULT_TEST(i,2)).NE.0) THEN
            SICK_RESULT_TEST(i,3)=SICK_RESULT_TEST(i,1)/SICK_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,4)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST) 

    !Compute aggregate statistics on those eligible for industrial accidents benefit: public labour market
    
    NumAnal = 2
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not eligible for industrial accident benefit as public lab. market insured"
    SICK_LAB_TEST(2) = "eligible for industrial accident benefit as public lab. market insured"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            DO i=1,NumAnal
                IF (SICK_ELIGPUB3_QT(pers_j).EQ.i-1) THEN
                    SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                    SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            END DO
        END IF
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_TYDBPUB3_QT: Type of benefit for an industrial accident the individual,&
    & insured as wage earner on the public labour market, is entitled to'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute aggregate statistics on those eligible for occupational diseases: public labour market
    
    NumAnal = 2
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Not eligible for occupational disease benefit as public lab. market insured"
    SICK_LAB_TEST(2) = "eligible for occupational disease benefit"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
            DO i=1,NumAnal
                IF (SICK_ELIGPUB4_QT(pers_j).EQ.i-1) THEN
                    SICK_RESULT_TEST(i,1) = SICK_RESULT_TEST(i,1) + 1
                    SICK_RESULT_TEST(i,2) = SICK_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            END DO
        END IF
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable SICK_TYDBPUB4_QT: Type of benefit for an occupational disease the individual,&
    & insured as wage earner on the public labour market, is entitled to'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

!COMPUTE STATISTICS ON SICKNESS AND DISABILITY OF THOSE INSURED AS SELF EMPLOYED

    Write(15,'(/,''Results on those insuered as self employed '')')
    CALL DRAWLINE(15,100,'-')    

    !Compute statistics on those eligible for benefits 
    NumAnal = 2
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is eligible for sick or disab benefits and insured as self empl"
    SICK_LAB_TEST(2) = "Individual is eligible for maternity benefits and insured as self empl"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            IF (SICK_ELIGSEL1_QT(pers_j).NE.0) THEN
                SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
                SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGSEL2_QT(pers_j).NE.0) THEN
                SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
                SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those eligible for a benefit and insured as self employed'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute statistics on those not eligible for benefits but with benefits different from 0
    
    NumAnal = 2
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is not eligible for sick or disab benefits but benefit is <> 0 and insured as self empl"
    SICK_LAB_TEST(2) = "Individual is not  eligible for maternity benefits but benefit is <> 0 and insured as self empl"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            IF (SICK_ELIGSEL1_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENSEL1_QT(pers_j)).NE.0) THEN
                SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
                SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGSEL2_QT(pers_j).EQ.0.AND.INT(100.D0*SICK_BENSEL2_QT(pers_j)).NE.0) THEN
                SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
                SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those NOT eligible for a benefit but with computed benefit <> 0'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute statistics on those eligible for benefits but with benefits equal to 0
    
    NumAnal = 2
    ALLOCATE(SICK_RESULT_TEST(NumAnal,3))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    SICK_LAB_TEST(1) = "Individual is eligible for sick or disab benefits but benefit is 0  and insured as self empl"
    SICK_LAB_TEST(2) = "Individual is eligible for maternity benefits but benefit is 0  and insured as self empl"

    DO pers_j=1,n_rec
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            IF (SICK_ELIGSEL1_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENSEL1_QT(pers_j)).EQ.0) THEN
                SICK_RESULT_TEST(1,1) = SICK_RESULT_TEST(1,1) + 1
                SICK_RESULT_TEST(1,2) = SICK_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (SICK_ELIGSEL2_QT(pers_j).NE.0.AND.INT(100.D0*SICK_BENSEL2_QT(pers_j)).EQ.0) THEN
                SICK_RESULT_TEST(2,1) = SICK_RESULT_TEST(2,1) + 1
                SICK_RESULT_TEST(2,2) = SICK_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END IF
    END DO                
    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,3)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts on those eligible for a benefit but with computed benefit equal to 0'')')
    Write(15,'(''0 possible for disabled because observed dates might be out of the observed quarter'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST)

    !Compute statistics on benefits received
    
    NumAnal = 6

    !Allocate and initialise test result
    ALLOCATE(SICK_RESULT_TEST(NumAnal,4))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    SICK_LAB_TEST(1) = "Daily benefit for sickness or disability for individual insured as self employed"            
    SICK_LAB_TEST(2) = "Quarterly benefit for sickness or disability for individual insured as self employed"            
    SICK_LAB_TEST(3) = "Quarterly benefit paid for sickness paid by soc sec budget "
    SICK_LAB_TEST(4) = "Quarterly benefit paid for disability  paid by soc sec budget"
    SICK_LAB_TEST(5) = "Quarterly supplement for help of a third person (individual is insured as self employed)"            
    SICK_LAB_TEST(6) = "Quarterly maternity benefit for individual insured as self employed"         

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
            SumGross(1:NumAnal)=0.D0
            !Compute quarterly amounts
            SumGross(1) = SICK_BENSEL1_DAY(pers_j)
            SumGross(2) = SICK_BENSEL1_QT(pers_j)
            IF (SICK_ELIGSEL1_QT(pers_j).EQ.1) THEN
                SumGross(3) = SICK_BENSEL1_QT(pers_j) 
            ELSE IF (SICK_ELIGSEL1_QT(pers_j).EQ.2) THEN
                SumGross(4) = SICK_BENSEL1_QT(pers_j) 
            END IF
            SumGross(5) = SICK_BETHSELF_QT(pers_j)
            SumGross(6) = SICK_BENSEL2_QT(pers_j)
            DO j=1,NumAnal
                IF (INT(100.D0*SumGross(j)).NE.0) THEN
                    SICK_RESULT_TEST(j,1)=SICK_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                    SICK_RESULT_TEST(j,2)=SICK_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
                END IF
            END DO
        END IF
    END DO

    DO i=1,NumAnal
        IF (INT(SICK_RESULT_TEST(i,2)).NE.0) THEN
            SICK_RESULT_TEST(i,3)=SICK_RESULT_TEST(i,1)/SICK_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,4)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Some statistics on sickness and disability benefits of those insured as self employed'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST) 

    !Compute global budget concepts: benefits for sickness and disability
    NumAnal = 12

    !Allocate and initialise test result
    ALLOCATE(SICK_RESULT_TEST(NumAnal,4))
    ALLOCATE(SICK_LAB_TEST(NumAnal))
    SICK_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    SICK_LAB_TEST(1) = "Wage earner regime: Sickness and disability benefits"
    SICK_LAB_TEST(2) = "Wage earner regime: Benefit paid for sickness paid by soc sec budget "
    SICK_LAB_TEST(3) = "Wage earner regime: Benefit paid for disability  paid by soc sec budget"
    SICK_LAB_TEST(4) = "Wage earner regime: Benefit paid for maternity in sickness paid by soc sec budget "
    SICK_LAB_TEST(5) = "Wage earner regime: Benefit paid for maternity in disability  paid by soc sec budget"
    SICK_LAB_TEST(6) = "Wage earner regime: Benefits in case of industrial accidents"
    SICK_LAB_TEST(7) = "Wage earner regime: Benefits in case of occupational diseases"
    SICK_LAB_TEST(8) = "Self employed regime: Sickness and disability benefits"
    SICK_LAB_TEST(9) = "Self employed regime: Benefit paid for sickness paid by soc sec budget "
    SICK_LAB_TEST(10) = "Self employed regime: Benefit paid for disability  paid by soc sec budget"
    SICK_LAB_TEST(11) = "Self employed regime: Benefit paid for maternity in sickness paid by soc sec budget "
    SICK_LAB_TEST(12) = "Self employed regime: Benefit paid for maternity in disability  paid by soc sec budget"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = (SICK_BENPRI1_QT(pers_j) + SICK_SUPPRI1_QT(pers_j) + SICK_BETHPRIV_QT(pers_j) &
                    &- SICK_EMPBPRI1_QT(pers_j) - SICK_EMPSPRI1_QT(pers_j)) + SICK_BENPRI2_QT(pers_j) +&
                    &(SICK_BENPUB1_QT(pers_j) + SICK_SUPPUB1_QT(pers_j) + SICK_BETHPUB_QT(pers_j) - &
                    &SICK_EMPBPUB1_QT(pers_j) - SICK_EMPSPUB1_QT(pers_j)) + SICK_BENPUB2_QT(pers_j) - SICK_EMPBPUB2_QT(pers_j)
        !Compute sickness and invalidity benefits separatly
        IF (SICK_ELIGPRI1_QT(pers_j).EQ.1) THEN
            SumGross(2) = (SICK_BENPRI1_QT(pers_j) + SICK_SUPPRI1_QT(pers_j) + SICK_BETHPRIV_QT(pers_j) &
                        &- SICK_EMPBPRI1_QT(pers_j) - SICK_EMPSPRI1_QT(pers_j)) 
        ELSE IF (SICK_ELIGPRI1_QT(pers_j).EQ.2) THEN
            SumGross(3) = (SICK_BENPRI1_QT(pers_j) + SICK_SUPPRI1_QT(pers_j) + SICK_BETHPRIV_QT(pers_j) &
                        &- SICK_EMPBPRI1_QT(pers_j) - SICK_EMPSPRI1_QT(pers_j)) 
        END IF
        IF (SICK_ELIGPUB1_QT(pers_j).EQ.1) THEN
            SumGross(2) = (SICK_BENPUB1_QT(pers_j) + SICK_SUPPUB1_QT(pers_j) + SICK_BETHPUB_QT(pers_j) &
                        &- SICK_EMPBPUB1_QT(pers_j) - SICK_EMPSPUB1_QT(pers_j)) 
        ELSE IF (SICK_ELIGPUB1_QT(pers_j).EQ.2) THEN
            SumGross(3) = (SICK_BENPUB1_QT(pers_j) + SICK_SUPPUB1_QT(pers_j) + SICK_BETHPUB_QT(pers_j) &
                        &- SICK_EMPBPUB1_QT(pers_j) - SICK_EMPSPUB1_QT(pers_j)) 
        END IF
        !Compute maternity benefits separatly
        IF (SICK_ELIGPRI2_QT(pers_j).EQ.1) THEN
            SumGross(4) = SICK_BENPRI2_QT(pers_j)
        ELSE IF (SICK_ELIGPRI2_QT(pers_j).EQ.2) THEN
            SumGross(5) = SICK_BENPRI2_QT(pers_j)
        END IF
        IF (SICK_ELIGPUB2_QT(pers_j).EQ.1) THEN
            SumGross(4) = SICK_BENPUB2_QT(pers_j) - SICK_EMPBPUB2_QT(pers_j)
        ELSE IF (SICK_ELIGPUB2_QT(pers_j).EQ.2) THEN
            SumGross(5) = SICK_BENPUB2_QT(pers_j) - SICK_EMPBPUB2_QT(pers_j)
        END IF
        SumGross(6) = SICK_BENPRI3_QT(pers_j) - SICK_EMPBPRI3_QT(pers_j)
        SumGross(7) = SICK_BENPRI4_QT(pers_j) - SICK_EMPBPRI4_QT(pers_j)
        SumGross(8) = SICK_BENSEL1_QT(pers_j) + SICK_BETHSELF_QT(pers_j) + SICK_BENSEL2_QT(pers_j)
        IF (SICK_ELIGSEL1_QT(pers_j).EQ.1) THEN
            SumGross(9) = SICK_BENSEL1_QT(pers_j) + SICK_BETHSELF_QT(pers_j) 
        ELSE IF (SICK_ELIGSEL1_QT(pers_j).EQ.2) THEN
            SumGross(10) = SICK_BENSEL1_QT(pers_j) + SICK_BETHSELF_QT(pers_j)
        END IF
        IF (SICK_ELIGSEL2_QT(pers_j).EQ.1) THEN
            SumGross(11) = SICK_BENSEL2_QT(pers_j) 
        ELSE IF (SICK_ELIGSEL2_QT(pers_j).EQ.2) THEN
            SumGross(12) = SICK_BENSEL2_QT(pers_j) 
        END IF
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                SICK_RESULT_TEST(j,1)=SICK_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                SICK_RESULT_TEST(j,2)=SICK_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    !convert values to annual values
    SICK_RESULT_TEST(1:NumAnal,1)=SICK_RESULT_TEST(1:NumAnal,1)*4.D0

    DO i=1,NumAnal
        IF (INT(SICK_RESULT_TEST(i,2)).NE.0) THEN
            SICK_RESULT_TEST(i,3)=SICK_RESULT_TEST(i,1)/SICK_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_TEST(i,4)=100.D0*SICK_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''Compute global budget concepts: benefits for sickness and disability'')')
    CALL DRAWLINE(15,100,'-')    
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')SICK_LAB_TEST(i),(SICK_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(SICK_LAB_TEST)
    DEALLOCATE(SICK_RESULT_TEST) 

END SUBROUTINE SICK_TEST

!==========================================================!
!Produce aggregate statistics that can be compared with
!statistics in other sources
!==========================================================!

SUBROUTINE SICK_COMPARE

    INTEGER                                                             :: pers_j,i,j
    INTEGER                                                             :: NumAnal                              !Number of summary statistics computed 
    !variables for computation of aggregates
    REAL(KIND=DPRECI)                                                   :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)                          :: SICK_AGGREGA                         !Vector with aggregates of SICK data comparable with other sources computed for 1 individual
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)                        :: SICK_RESULT_COMPARE                  !Vector with aggregates of SICK data comparable with other sources computed over all individuals
    CHARACTER(len=100),ALLOCATABLE,DIMENSION(:)                         :: SICK_LAB_COMPARE                     !Vector with labels of the different reconstructed aggregates                       
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)                          :: SICK_EXTERN                          !Vector with aggregates taken from external sources comparable with the reconstructed concepts
 
    WRITE(15,'(/,''Print statistics comparable with other sources'')')
    CALL DRAWLINE(15,136,'=')    

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))

    !Number of days registered
    NumAnal = 10
    ALLOCATE(SICK_AGGREGA(NumAnal))       
    ALLOCATE(SICK_RESULT_COMPARE(NumAnal,6))
    ALLOCATE(SICK_LAB_COMPARE(NumAnal))     
    ALLOCATE(SICK_EXTERN(NumAnal))        

    !Initialise results
    SICK_RESULT_COMPARE(1:SIZE(SICK_RESULT_COMPARE,1),1:6) = 0.D0
    SICK_EXTERN(1:SIZE(SICK_EXTERN,1)) = 0.D0           
    
    !Define labels
    SICK_LAB_COMPARE(1) = "Algemene regeling: primaire arbeidsongeschiktheid, arbeiders"
    SICK_LAB_COMPARE(2) = "Algemene regeling: primaire arbeidsongeschiktheid, bedienden"
    SICK_LAB_COMPARE(3) = "Algemene regeling: invaliditeit, arbeiders"
    SICK_LAB_COMPARE(4) = "Algemene regeling: invaliditeit, bedienden"
    SICK_LAB_COMPARE(5) = "Algemene regeling: moederschapsrust, primaire periode"
    SICK_LAB_COMPARE(6) = "Algemene regeling: moederschapsrust, invaliditeit"
    SICK_LAB_COMPARE(7) = "Regeling zelfstandigen: primaire arbeidsongeschiktheid"
    SICK_LAB_COMPARE(8) = "Regeling zelfstandigen: invaliditeit"
    SICK_LAB_COMPARE(9) = "Regeling zelfstandigen: moederschapsrust, primaire periode"
    SICK_LAB_COMPARE(10) = "Regeling zelfstandigen: moederschapsrust, invaliditeit"
    
    !Define values taken from external sources
    SICK_EXTERN(1)   = 0
    SICK_EXTERN(2)   = 0
    SICK_EXTERN(3)   = 0
    SICK_EXTERN(4)   = 0
    SICK_EXTERN(5)   = 0
    SICK_EXTERN(6)   = 0
    SICK_EXTERN(7)   = 0
    SICK_EXTERN(8)   = 0
    SICK_EXTERN(9)   = 0
    SICK_EXTERN(10)  = 0

    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !Make vector with local aggregates empty
        SICK_AGGREGA(1:SIZE(SICK_AGGREGA,1)) = 0.D0
        !Compute quarterly amounts
        !Algemene regeling: primaire arbeidsongeschiktheid, arbeiders
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(1)  = SICK_AGGREGA(1) + SICK_SICEMPNM_QT(pers_j,1)
        END IF    
        !Algemene regeling: primaire arbeidsongeschiktheid, bedienden
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(2)  = SICK_AGGREGA(2) + SICK_SICEMPNM_QT(pers_j,1)
        END IF    
        !Algemene regeling: invaliditeit, arbeiders
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(3)  = SICK_AGGREGA(3) + SICK_DISEMPNM_QT(pers_j,1)
        END IF    
        !Algemene regeling: invalideit, bedienden
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(4)  = SICK_AGGREGA(4) + SICK_DISEMPNM_QT(pers_j,1)
        END IF    
        !Algemene regeling: moederschapsrust, primaire periode
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPRI2_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(5)  = SICK_AGGREGA(5) + SICK_DAYSPRI2_QT(pers_j)
        END IF    
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPUB2_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(5)  = SICK_AGGREGA(5) + SICK_DAYSPUB2_QT(pers_j)
        END IF    
        !Algemene regeling: moederschapsrust, invaliditeit
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPRI2_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(6)  = SICK_AGGREGA(6) + SICK_DAYSPRI2_QT(pers_j)
        END IF    
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPUB2_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(6)  = SICK_AGGREGA(6) + SICK_DAYSPUB2_QT(pers_j)
        END IF    
        !Regeling zelfstandigen: primaire arbeidsongeschiktheid
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_ELIGSEL1_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(7)  = SICK_AGGREGA(7) + SICK_SICSELNM_QT(pers_j,1)
        END IF    
        !Regeling zelfstandigen: invaliditeit
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_ELIGSEL1_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(8)  = SICK_AGGREGA(8) + SICK_DISSELNM_QT(pers_j,1)
        END IF    
        !Regeling zelfstandigen: moederschapsrust, primaire periode
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_ELIGSEL2_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(9)  = SICK_AGGREGA(9) 
        END IF    
        !Regeling zelfstandigen: moederschapsrust, invaliditeit
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_ELIGSEL2_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(10)  = SICK_AGGREGA(10)
        END IF    
        !Add all the individual results to a vector that aggregates results over all individuals
        DO j=1,SIZE(SICK_AGGREGA,1)
            IF (INT(100.D0*SICK_AGGREGA(j)).NE.0) THEN
                SICK_RESULT_COMPARE(j,1)=SICK_RESULT_COMPARE(j,1)+MIMOSIS_WEIGHT(pers_j)*SICK_AGGREGA(j)
                SICK_RESULT_COMPARE(j,2)=SICK_RESULT_COMPARE(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    !Inflate contributions to annual values
    SICK_RESULT_COMPARE(1:SIZE(SICK_RESULT_COMPARE,1),1)=(SICK_RESULT_COMPARE(1:SIZE(SICK_RESULT_COMPARE,1),1)*4.D0)

    !Compute columns 3 to 6 of the vector with comparable aggregates
    DO i=1,SIZE(SICK_RESULT_COMPARE,1)
        !difference between real value and estimate
        SICK_RESULT_COMPARE(i,3)= SICK_RESULT_COMPARE(i,1) - SICK_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(SICK_EXTERN(i)).NE.0) THEN
            SICK_RESULT_COMPARE(i,4)= 100.D0*(SICK_RESULT_COMPARE(i,3)/SICK_EXTERN(i))    
        END IF
        !Average amount
        IF (INT(SICK_RESULT_COMPARE(i,2)).NE.0) THEN
            SICK_RESULT_COMPARE(i,5)=SICK_RESULT_COMPARE(i,1)/SICK_RESULT_COMPARE(i,2)
        END IF
        !Fraction of population with non zero value
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_COMPARE(i,6)=100.D0*SICK_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''Vergoede dagen voor ziekte en invaliditeit'')')
    Write(15,'(''Vergelijking met cijfers ivm algemene regeling en regeling zelfstandingen voor 2008'')')
    Write(15,'(''                                                                                                         &
    &         Extern              Sample   Number of individ            Abs dif       %Dif      Mean      %Pop'')')
    Write(15,'(''                                                                                                         &
    &             C1                  C2                                  C2-C1  100*C2/C1 per indiv'')')
    CALL DRAWLINE(15,210,'-')    
    DO i=1,SIZE(SICK_RESULT_COMPARE,1)
        Write(15,'(a100,F20.2,F20.2,F20.0,F20.0,F10.2,F10.2,F10.2)')SICK_LAB_COMPARE(i),SICK_EXTERN(i),&
        &(SICK_RESULT_COMPARE(i,j),j=1,6)
    END DO

    DEALLOCATE(SICK_AGGREGA)      
    DEALLOCATE(SICK_RESULT_COMPARE)
    DEALLOCATE(SICK_LAB_COMPARE)     
    DEALLOCATE(SICK_EXTERN)        

    !Benefits paid
    NumAnal = 14
    ALLOCATE(SICK_AGGREGA(NumAnal))       
    ALLOCATE(SICK_RESULT_COMPARE(NumAnal,6))
    ALLOCATE(SICK_LAB_COMPARE(NumAnal))     
    ALLOCATE(SICK_EXTERN(NumAnal))        

    !Initialise results
    SICK_RESULT_COMPARE(1:SIZE(SICK_RESULT_COMPARE,1),1:6) = 0.D0
    SICK_EXTERN(1:SIZE(SICK_EXTERN,1)) = 0.D0           
    
    !Define labels
    SICK_LAB_COMPARE(1) = "Algemene regeling: primaire arbeidsongeschiktheid, arbeiders"
    SICK_LAB_COMPARE(2) = "Algemene regeling: primaire arbeidsongeschiktheid, bedienden"
    SICK_LAB_COMPARE(3) = "Algemene regeling: invaliditeit, arbeiders"
    SICK_LAB_COMPARE(4) = "Algemene regeling: invaliditeit, bedienden"
    SICK_LAB_COMPARE(5) = "Algemene regeling: moederschapsrust, primaire periode"
    SICK_LAB_COMPARE(6) = "Algemene regeling: moederschapsrust, invaliditeit"
    SICK_LAB_COMPARE(7) = "Regeling zelfstandigen: primaire arbeidsongeschiktheid"
    SICK_LAB_COMPARE(8) = "Regeling zelfstandigen: invaliditeit"
    SICK_LAB_COMPARE(9) = "Regeling zelfstandigen: moederschapsrust, primaire periode"
    SICK_LAB_COMPARE(10) = "Regeling zelfstandigen: moederschapsrust, invaliditeit"
    SICK_LAB_COMPARE(11) = "Vergoedingen tijdelijke arbeidsongeschiktheid"
    SICK_LAB_COMPARE(12) = "Vergoedingen permanente arbeidsongeschiktheid"
    SICK_LAB_COMPARE(13) = "Vergoedingen tijdelijke beroepsziekte"
    SICK_LAB_COMPARE(14) = "Vergoedingen permanente beroepsziekte"
    
    !Define values taken from external sources
    SICK_EXTERN(1)   = 0
    SICK_EXTERN(2)   = 0
    SICK_EXTERN(3)   = 0
    SICK_EXTERN(4)   = 0
    SICK_EXTERN(5)   = 0
    SICK_EXTERN(6)   = 0
    SICK_EXTERN(7)   = 0 
    SICK_EXTERN(8)   = 0
    SICK_EXTERN(9)   = 0
    SICK_EXTERN(10)  = 0
    SICK_EXTERN(11)  = 0
    SICK_EXTERN(12)  = 0
    SICK_EXTERN(13)  = 0
    SICK_EXTERN(14)  = 0

    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !Make vector with local aggregates empty
        SICK_AGGREGA(1:SIZE(SICK_AGGREGA,1)) = 0.D0
        !Compute quarterly amounts
        !Algemene regeling: primaire arbeidsongeschiktheid, arbeiders
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(1)  = SICK_AGGREGA(1) + SICK_SICEMPNM_QT(pers_j,2)
        END IF    
        !Algemene regeling: primaire arbeidsongeschiktheid, bedienden
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(2)  = SICK_AGGREGA(2) + SICK_SICEMPNM_QT(pers_j,2)
        END IF    
        !Algemene regeling: invaliditeit, arbeiders
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.1.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(3)  = SICK_AGGREGA(3) + SICK_DISEMPNM_QT(pers_j,2)
        END IF    
        !Algemene regeling: invalideit, bedienden
        IF (SICK_TYPEINSU_PRIV_QT(pers_j).EQ.2.OR.SICK_TYPEINSU_PUB_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(4)  = SICK_AGGREGA(4) + SICK_DISEMPNM_QT(pers_j,2)
        END IF    
        !Algemene regeling: moederschapsrust, primaire periode
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPRI2_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(5)  = SICK_AGGREGA(5) + SICK_BENPRI2_QT(pers_j)
        END IF    
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPUB2_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(5)  = SICK_AGGREGA(5) + SICK_BENPUB2_QT(pers_j) - SICK_EMPBPUB2_QT(pers_j)
        END IF    
        !Algemene regeling: moederschapsrust, invaliditeit
        IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPRI2_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(6)  = SICK_AGGREGA(6) + SICK_BENPRI2_QT(pers_j)
        END IF    
        IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/)).AND.SICK_ELIGPUB2_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(6)  = SICK_AGGREGA(6) + SICK_BENPUB2_QT(pers_j) - SICK_EMPBPUB2_QT(pers_j)
        END IF    
        !Regeling zelfstandigen: primaire arbeidsongeschiktheid
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.ANY(SICK_ELIGSEL1_QT(pers_j).EQ.(/1, 2/))) THEN
            SICK_AGGREGA(7)  = SICK_AGGREGA(7) + SICK_SICSELNM_QT(pers_j,2)
        END IF    
        !Regeling zelfstandigen: invaliditeit
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_ELIGSEL1_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(8)  = SICK_AGGREGA(8) + SICK_DISSELNM_QT(pers_j,2)
        END IF    
        !Regeling zelfstandigen: moederschapsrust, primaire periode
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_ELIGSEL2_QT(pers_j).EQ.1) THEN
            SICK_AGGREGA(9)  = SICK_AGGREGA(9) + SICK_BENSEL2_QT(pers_j)
        END IF    
        !Regeling zelfstandigen: moederschapsrust, invaliditeit
        IF (SICK_TYPEINSU_SELF_QT(pers_j).EQ.1.AND.SICK_ELIGSEL2_QT(pers_j).EQ.2) THEN
            SICK_AGGREGA(10)  = SICK_AGGREGA(10) + SICK_BENSEL2_QT(pers_j)
        END IF    
        !Uitgaven voor tijdelijke arbeidsongeschiktheid
        IF (ANY(SICK_TYDBPRI3_QT(pers_j).EQ.(/1,2/))) THEN
            SICK_AGGREGA(11)  = SICK_AGGREGA(11) + SICK_BENPRI3_QT(pers_j) - SICK_EMPBPRI3_QT(pers_j)
        END IF    
        IF (ANY(SICK_TYDBPUB3_QT(pers_j).EQ.(/1,2/))) THEN
            SICK_AGGREGA(11)  = SICK_AGGREGA(11) + SICK_BENPUB3_QT(pers_j) - SICK_EMPBPUB3_QT(pers_j)
        END IF    
        !Uitgaven voor permanente arbeidsongeschiktheid
        IF (ANY(SICK_TYDBPRI3_QT(pers_j).EQ.(/3/))) THEN
            SICK_AGGREGA(12)  = SICK_AGGREGA(12) + SICK_BENPRI3_QT(pers_j) - SICK_EMPBPRI3_QT(pers_j)
        END IF    
        IF (ANY(SICK_TYDBPUB3_QT(pers_j).EQ.(/3/))) THEN
            SICK_AGGREGA(12)  = SICK_AGGREGA(12) + SICK_BENPUB3_QT(pers_j) - SICK_EMPBPUB3_QT(pers_j)
        END IF    
        !Uitgaven voor tijdelijke beroepsziekte
        IF (ANY(SICK_TYDBPRI4_QT(pers_j).EQ.(/1,2/))) THEN
            SICK_AGGREGA(13)  = SICK_AGGREGA(13) + SICK_BENPRI4_QT(pers_j) - SICK_EMPBPRI4_QT(pers_j)
        END IF    
        IF (ANY(SICK_TYDBPUB4_QT(pers_j).EQ.(/1,2/))) THEN
            SICK_AGGREGA(13)  = SICK_AGGREGA(13) + SICK_BENPUB4_QT(pers_j) - SICK_EMPBPUB4_QT(pers_j)
        END IF    
        !Uitgaven voor permanente beroepsziekte
        IF (ANY(SICK_TYDBPRI4_QT(pers_j).EQ.(/3/))) THEN
            SICK_AGGREGA(14)  = SICK_AGGREGA(14) + SICK_BENPRI4_QT(pers_j) - SICK_EMPBPRI4_QT(pers_j)
        END IF    
        IF (ANY(SICK_TYDBPUB4_QT(pers_j).EQ.(/3/))) THEN
            SICK_AGGREGA(14)  = SICK_AGGREGA(14) + SICK_BENPUB4_QT(pers_j) - SICK_EMPBPUB4_QT(pers_j)
        END IF    
        !Add all the individual results to a vector that aggregates results over all individuals
        DO j=1,SIZE(SICK_AGGREGA,1)
            IF (INT(100.D0*SICK_AGGREGA(j)).NE.0) THEN
                SICK_RESULT_COMPARE(j,1)=SICK_RESULT_COMPARE(j,1)+MIMOSIS_WEIGHT(pers_j)*SICK_AGGREGA(j)
                SICK_RESULT_COMPARE(j,2)=SICK_RESULT_COMPARE(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    !Inflate contributions to annual values
    SICK_RESULT_COMPARE(1:SIZE(SICK_RESULT_COMPARE,1),1)=(SICK_RESULT_COMPARE(1:SIZE(SICK_RESULT_COMPARE,1),1)*4.D0)

    !Compute columns 3 to 6 of the vector with comparable aggregates
    DO i=1,SIZE(SICK_RESULT_COMPARE,1)
        !difference between real value and estimate
        SICK_RESULT_COMPARE(i,3)= SICK_RESULT_COMPARE(i,1) - SICK_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(SICK_EXTERN(i)).NE.0) THEN
            SICK_RESULT_COMPARE(i,4)= 100.D0*(SICK_RESULT_COMPARE(i,3)/SICK_EXTERN(i))    
        END IF
        !Average amount
        IF (INT(SICK_RESULT_COMPARE(i,2)).NE.0) THEN
            SICK_RESULT_COMPARE(i,5)=SICK_RESULT_COMPARE(i,1)/SICK_RESULT_COMPARE(i,2)
        END IF
        !Fraction of population with non zero value
        IF (INT(SumW).NE.0) THEN
            SICK_RESULT_COMPARE(i,6)=100.D0*SICK_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''Uitgekeerde bedragen voor ziekte en invaliditeit'')')
    Write(15,'(''Vergelijking met cijfers ivm algemene regeling, regeling zelfstandingen en AO en BZ voor 2008: bedragen &
    &in Euros'')')
    Write(15,'(''                                                                                                         &
    &         Extern              Sample   Number of individ            Abs dif       %Dif      Mean      %Pop'')')
    Write(15,'(''                                                                                                         &
    &             C1                  C2                                  C2-C1  100*C2/C1 per indiv'')')
    CALL DRAWLINE(15,210,'-')    
    DO i=1,SIZE(SICK_RESULT_COMPARE,1)
        Write(15,'(a100,F20.2,F20.2,F20.0,F20.0,F10.2,F10.2,F10.2)')SICK_LAB_COMPARE(i),SICK_EXTERN(i),&
        &(SICK_RESULT_COMPARE(i,j),j=1,6)
    END DO

    DEALLOCATE(SICK_AGGREGA)      
    DEALLOCATE(SICK_RESULT_COMPARE)
    DEALLOCATE(SICK_LAB_COMPARE)     
    DEALLOCATE(SICK_EXTERN)        

END SUBROUTINE SICK_COMPARE

!============= SUBROUTINE OUTPUT_FILEWRIT ==============!
!This subroutine open output file and write output      !
!=================================================++====!

SUBROUTINE SICK_OUTPUT_FILEWRIT

    INTEGER :: pers_j
    
    OPEN (UNIT=12, FILE=UNEM_SICK,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_SICK
        WRITE(15,'("Could not find ", a100)') UNEM_SICK
        PAUSE
        STOP
    END IF
    OPEN (UNIT=13, FILE=FAMAL_SICK,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMAL_SICK
        WRITE(15,'("Could not find ", a100)') FAMAL_SICK
        PAUSE
        STOP
    END IF
    OPEN (UNIT=14, FILE=SICK_OUTP,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_OUTP
        WRITE(15,'("Could not find ", a100)') SICK_OUTP
        PAUSE
        STOP
    END IF
    OPEN (UNIT=16, FILE=PENSWELF_SICK,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PENSWELF_SICK
        WRITE(15,'("Could not find ", a100)') PENSWELF_SICK
        PAUSE
        STOP
    END IF
    OPEN (UNIT=17, FILE=SOCBEN_SICK,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_SICK
        WRITE(15,'("Could not find ", a100)') SOCBEN_SICK
        PAUSE
        STOP
    END IF

    DO pers_j=1,n_rec
        WRITE(12,'(A14,F15.2,F15.2)')   GECODEERD_INSZ_CHAR(pers_j),SICK_BENINOC_QT(pers_j),SICK_NONEINOC_QT(pers_j)
        WRITE(13,'(A14,I10,I10,I10,F15.2,I10)') &
        &GECODEERD_INSZ_CHAR(pers_j),SICK_TYPEINSU_PRIV_QT(pers_j),SICK_TYPEINSU_PUB_QT(pers_j),&
        &SICK_TYPEINSU_SELF_QT(pers_j),SICK_TOTBEN_QT(pers_j),SICK_DISABBEN_QT(pers_j)
        WRITE(14,'(A14,F15.2)')         GECODEERD_INSZ_CHAR(pers_j),SICK_TOTBEN_QT(pers_j)   
        WRITE(16,'(A14,F15.2)')         GECODEERD_INSZ_CHAR(pers_j),SICK_TOTBEN_QT(pers_j)   
        WRITE(17,'(A14,I10)')           GECODEERD_INSZ_CHAR(pers_j),SICK_DISABBEN_QT(pers_j)
    END DO

    CLOSE (UNIT=12)
    CLOSE (UNIT=13)
    CLOSE (UNIT=14)
    CLOSE (UNIT=16)
    CLOSE (UNIT=17)

    WRITE(*,*) 'Subroutine SICK_OUTPUT_FILEWRIT of PRE RUN............runned correctly'

END SUBROUTINE SICK_OUTPUT_FILEWRIT

!==========================================================!
!The subroutine SICK_PRERUN_CALC is used to call the other ! 
!subroutines and to carry out the necessary computations   ! 
!==========================================================!

SUBROUTINE SICK_PRERUN_CALC

!Program heading

!=============================================================================
    WRITE (*,*) ' '
    WRITE (*,*) 'MIMOSIS SICK PRERUN IS RUNNING '
    WRITE (*,*) ' '
!=============================================================================

    !open log file
    OPEN(UNIT=15, FILE=SICK_PRERUN_LOG, STATUS='REPLACE', ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_PRERUN_LOG
        WRITE(15,'("Could not find ", a100)') SICK_PRERUN_LOG
        PAUSE
        STOP
    END IF

    !Allocate all exogenous vectors with dimension n_rec
    CALL SICK_ALLOC_EXOG(n_rec)
    
    !Reads in PARAMETER file
    CALL SICK_PARAMETER_FILEREAD
    !Read in files with input data
    CALL SICK_INPUT_FILEREAD
    CALL SICK_CONTRIB_FILEREAD
    CALL SICK_UNEM_FILEREAD
    CALL SICK_PIT_FILEREAD
    CALL SICK_PENSWELF_FILEREAD
    CALL SICK_FAMAL_FILEREAD
    CALL SICK_SICK_FILEREAD
    CALL SICK_FAMREL_FILEREAD
    CALL SICK_WAGE_FILEREAD
    CALL SICK_WEIGHT_FILEREAD
    
    !Compute the value of the first and the last day of the quarter under analysis
    CALL SICK_SSDATE

    WRITE(15,'(/,''Print of function calls for which no selection was carried out'')')
    CALL DRAWLINE(15,100,'=')                

    !Allocate all endogenous vectors with dimension n_rec
    CALL SICK_ALLOC_ENDO(n_rec)
    !Initialise endogenous variables
    CALL SICK_INITENDO(n_rec)
    
    !Rank and Sort all variables read in
    CALL SICK_RANKANDSORT

    !Recode missing values
    CALL SICK_RECODE
    !Inflate nominal values
    CALL SICK_INFLATE

    !Determine household size  
    CALL SICK_HOUSEHOLD_SIZE
    !Determine age of all individuals
    CALL SICK_AGE

    !Construct variables necessary to run the different SICK computation subroutines 
    CALL SICK_CHARAC            !construction of different characteristics of the sick or disabled (mainly eligibility variables)
    CALL SICK_FAMSIT_CONSTRUC   !determine the household charge of each sick or disabled
    CALL SICK_DAILYLAB          !determine daily labour benefit that reduces disability benefit

    !Loop over all households  
    DO hh_i=1,N_HH_SICK
        !Loop over all persons within the household
        DO pers_j=SICK_START_HH(hh_i),SICK_END_HH(hh_i)
            !Compute sickness and disability benefits for those insured as wage earner on the private labour market
            IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN            
                CALL SICK_BENPRIV_CONSTRUC(pers_j)
            END IF
            !Compute sickness and disability benefits for those insured as wage earner on the public labour market WITHOUT statutory service
            IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN            
                CALL SICK_BENPUBNS_CONSTRUC(pers_j)
            END IF
            !Compute sickness and disability benefits for those insured as wage earner on the public labour market WITH statutory service
            IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN            
                CALL SICK_BENPUBST_CONSTRUC(pers_j)
            END IF
            !Compute sickness and disability benefits for those insured as self employed
            IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN            
                CALL SICK_BENSELF_CONSTRUC(pers_j)
            END IF
        END DO
    END DO  

    !Construct exchange concepts
    CALL SICK_EXCHANGE

    !test construction of family relations
    CALL SICK_TEST
    CALL SICK_COMPARE

    !Write output to output file
    CALL SICK_OUTPUT_FILEWRIT

    !Deallocate allocated variables
    CALL SICK_DEALLOC
    CLOSE(15)

END SUBROUTINE SICK_PRERUN_CALC

END MODULE SICK_PRERUN_SUBROUTINES
