!===================================================================!
!  Record of revisions:                                             !
!                                                                   !
!    Date      Programmer               Description of changes      !
!  ========    ====================     ======================      !
!  01/03/06    Consortium AG/01/086     Original code               !
!  01/10/11    FPS Social Security      Revision                    !
!===================================================================!

!=====================================================!
!This module declare all variables used in the        !
!   programme:                                        !
!  -Parameters                                        !
!  -Exogenous variables                               !
!  -Endogenous variables                              !
!         - defined in module notes                   !
!         - not defined in module notes               !
!  -Control variables                                 !
!======================================================

MODULE SOCBEN_DATA

    !========================================================================!
    !Database specific parameters:                                           !
    ! number of lines in parameter file                                      !
    INTEGER, PARAMETER:: n_par=210                                           !
    ! year of database                                                       !
    INTEGER, PARAMETER:: MIMOSIS_YEAR=2008                                   !
    !(first quarter=1, second quarter=4, third quarter=9, fourth quarter=12) !  
    INTEGER, PARAMETER:: month=1                                             !
    !Define kind of Integer*4 in a portable way                              !                       
    INTEGER, PARAMETER :: INT4 = SELECTED_INT_KIND(18)                       !                       
    !Define Double precision, i.e. REAL*8 in a portable way                  !
    INTEGER, PARAMETER :: DPRECI = SELECTED_REAL_KIND(15,307)                !
    !Define Single precision, i.e. REAL*4 in a portable way                  !
    INTEGER, PARAMETER :: SPRECI = SELECTED_REAL_KIND(6,37)                  !
    !Define parameter that stores the precision value used for large vectors !
    INTEGER, PARAMETER :: KIND_REAL = SPRECI                                 !   
    !========================================================================!

    !========================================================================!
    !Define variables to read locations and input parameters
    !Parameters for locating input 
    CHARACTER(len=150)                                      :: SOCBEN_INPUT_PATH                    !Default path for ALL input files
    CHARACTER(len=150)                                      :: SOCBEN_INPUT_FILE                    !Default name of INPUT file (i.e. INPUT from NATIONAL REGISTERS)
    CHARACTER(len=150)                                      :: SOCBEN_INPUT                         !Path+Name of input file
    CHARACTER(len=150)                                      :: SOCBEN_PIT_FILE                      !Default name of PIT file (i.e. the OUTPUT FILE of the PIT module)
    CHARACTER(len=150)                                      :: SOCBEN_PIT                           !Path+Name of contrib file
    CHARACTER(len=150)                                      :: SOCBEN_FAMAL_FILE                    !Default name of FAMAL file (i.e. the OUTPUT FILE of the FAMAL module)
    CHARACTER(len=150)                                      :: SOCBEN_FAMAL                         !Path+Name of famal file
    CHARACTER(len=150)                                      :: SOCBEN_CONTRIB_FILE                  !Default name of CONTRIB file (i.e. the OUTPUT FILE of the CONTRIB module)
    CHARACTER(len=150)                                      :: SOCBEN_CONTRIB                       !Path+Name of contrib file
    CHARACTER(len=150)                                      :: SOCBEN_UNEM_FILE                     !Default name of UNEM file (i.e. the OUTPUT FILE of the UNEM module)
    CHARACTER(len=150)                                      :: SOCBEN_UNEM                          !Path+Name of UNEM file
    CHARACTER(len=150)                                      :: SOCBEN_SICK_FILE                     !Default name of SICK file (i.e. the OUTPUT FILE of the SICK module)
    CHARACTER(len=150)                                      :: SOCBEN_SICK                          !Path+Name of SICK file
    CHARACTER(len=150)                                      :: SOCBEN_CONSTRUCT_FILE                !Default name of CONSTRUCT file 
    CHARACTER(len=150)                                      :: SOCBEN_CONSTRUCT                     !Path+Name of construct file
    CHARACTER(len=150)                                      :: FAMREL_INPUT_PATH                    !Path name where file with FAMREL input data is stored
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_FILE                  !File name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL                       !Internal name to store Path+Name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: CONSTANT_INPUT_PATH                  !Path name where the file EVAL_INPUT.txt and other simulation independent information is stored
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG_FILE                !File name of the MIMOSIS_WEIGHT file                    
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG                     !full location of MIMOSIS_WEIGHT.txt
    !Parameters for locating output (local variables)
    CHARACTER(len=150)                                      :: EVAL_SOCBEN_FILE                     !Name of standard output file
    CHARACTER(len=150)                                      :: EVAL_SOCBEN                          !Path+Name of output file (standard)
    !Tools to read input, output and parameters
    CHARACTER(len=150)                                      :: SOCBEN_HEADING                       !Heading of parameter file
    CHARACTER(len=20)                                       :: SOCBEN_PAR_FORMAT                    !Parameters' format
    CHARACTER(len=1)                                        :: SOCBEN_PAR_BREAK='@'                 !Character used to identify end of string
    CHARACTER(len=10)                                       :: SOCBEN_PAR_CURR                      !Currency
    CHARACTER(len=40),ALLOCATABLE,DIMENSION(:)              :: SOCBEN_PAR_NAMES                     !Vector of parameter names
    INTEGER                                                 :: SOCBEN_PAR_L                         !Number of lines (if parameter is a matrix)
    INTEGER                                                 :: SOCBEN_PAR_C                         !Number of columns (if parameter is a matrix)
    INTEGER,DIMENSION(12)                                   :: SOCBEN_PAR_BREAK_POINT               !Vector registering end of string
    !==========================================================================
    !Parameters of the SOCBEN module                             
    !General parameters
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: SOCBEN_FAMREL_ANN                    !Matrix used to convert relationships of family members
    !Parameters to determine the leefloon
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_MINTYM65_ANN                  !Maximum amount of social assistance by household typology for people below the age of 65
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_TRAIN_PROG_MONTH              !Special disregard on professional income for people following a "back to work" programme or an individualized training programme
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADL_RATE                     !Rate used to multiply gross cadastral income (land property) 
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADB_RATE                     !Rate used to multiply gross cadastral income (building)
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADL_BASICD_ANN               !Basic disregard on gross cadastral income (land)
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADB_BASICD_ANN               !Basic disregard on gross cadastral income (building)
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADB_SPOUSED_ANN              !Disregard on gross cadastral income (building) for dependent spouse
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADB_CHILDD_ANN               !Disregard on gross cadastral income (building) for (each) dependent child
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_CAPRATES_ANN                  !Rates used to impute income from financial assets
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_DISREGAR_ANN                 !Income disregard by household typology
    !Parameters to determine the IGO
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_MINTYP65_ANN                  !Maximum amount of social assistance by household typology for people 65 or older
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_GWB_ANN                       !Maximum amount of social assistance by household typology for people 65 or older (if still gewaarborgd inkomen voor bejaarden)
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_LABIGO_RATE                   !Rate used to determine the amount of labour income that has to be subtracted from the IGO 
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_PENSIGO_RATE                  !Rate used to determine the amount of pension income that has to be subtracted from the IGO 
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADIGO_RATE                   !Rate used to multiply gross cadastral income 
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADBIGO_BASICD_ANN            !Basic deduction used to reduce cadastral income (building)
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADIGO_CHILDD_ANN             !Deduction used to reduce cadastral income in function of number of children in charge (building)
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADLIGO_BASICD_ANN            !Basic deduction used to reduce cadastral income (land)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_CAPRAIGO_ANN                  !Rates used to impute income from financial assets to determine IGO
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_DISREIGO_ANN                 !Income disregard by household typology to determine IGO
    !Parameters to determine the benefits for disabled people
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_BAMIVT_ANN                    !Basic amount on which a disabled person is entitled in the IVT regime
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADIVT_BASICD_ANN             !Basic deduction used to exempt part of real estate income when computing the IVT benefit
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADIVT_CHILDD_ANN             !Deduction for dependent children used to exempt part of real estate income when computing the IVT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_LABIVT_ANN                    !Exemption applied on labour income of the disabled person when computing the IVT benefit
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_OTHIVT_ANN                    !Exemption applied on other income of the disabled person when computing the IVT benefit
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_PARTIVT_ANN                   !Exemption applied on income of the partner of a disabled person when computing the IVT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_DISREIVT_ANN                  !Basic disregard applied when computing the IVT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_BAMIT_ANN                     !Basic amount on which a disabled person is entitled in the IT regime
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_PARTITA_ANN                   !Exemption applied on income of the partner of a disabled person when computing the IT benefit (when disabled person is of medical type 1 or 2)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_PARTITB_ANN                   !Exemption applied on income of the partner of a disabled person when computing the IT benefit (when disabled person is of medical type 3, 4 or 5)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_LABIT_ANN                     !Exemption applied on labour income of the disabled person when computing the IT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_REPIT_ANN                     !Exemption applied on replacement income of the disabled person when computing the IT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_DISREIT_ANN                   !Basic disregard applied when computing the IT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_BAMTHB_ANN                    !Basic amount on which a disabled person is entitled in the THB regime
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_LABTHB_RATE                   !Rate used to determine the amount of labour income that has to be subtracted from the THB 
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_PENSTHB_RATE                  !Rate used to determine the amount of pension income that has to be subtracted from the THB
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADLTHB_RATE                  !Rate used to multiply gross cadastral income from land
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADBTHB_RATE                  !Rate used to multiply gross cadastral income from buildings
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADBTHB_BASICD_ANN            !Basic deduction used to exempt part of real estate income from buildings when computing the THB benefit
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADBTHB_CHILDD_ANN            !Deduction for dependent children used to exempt part of real estate income from buildings when computing the THB benefit
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADLTHB_BASICD_ANN            !Basic deduction used to exempt part of real estate income from land when computing the THB benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_CAPRATHB_ANN                  !Rates used to impute income from financial assets to determine THB
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_DISRETHB_ANN                  !Basic disregard applied when computing the THB benefit
    !Parameters for health benefit creation
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADBVT_BASICD_ANN             !Basic deduction used to exempt part of real estate income from buildings when computing the VT health benefit
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_CADBVT_CHILDD_ANN             !Deduction for dependent children used to exempt part of real estate income from buildings when computing the VT health benefit
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_VT_BASICD_ANN                 !Basic amount used to determine the income threshold for the VT health benefit
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_VT_CHILDD_ANN                 !Amount used per dependent child to determine the income threshold for the VT health benefit
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_OMNIO_BASICD_ANN              !Basic amount used to determine the income threshold for the OMNIO health benefit
    REAL(KIND=KIND_REAL)                                    :: SOCBEN_OMNIO_CHILDD_ANN              !Amount used per dependent child to determine the income threshold for the OMNIO health benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: SOCBEN_INCMAF_ANN                    !Thresholds for the income MAF
    !Parameters for simulations of the DG PMH
    INTEGER                                                 :: SOCBEN_SIMDGPMH_QT                   !Index to start simulations requested by the DG Personen met een Handicap
    !==========================================================================
    !Exogenous variables
    !SOCBEN_INPUT_Write.sas
    !/*popchar*/
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: GECODEERD_INSZ_CHAR                  ! Geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_INPUT                 ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_RELATION                    ! Verband tussen het individu en het huishoudhoofd op 1 januari 2009
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_SEX                         ! Geslacht van het individu op 1 januari 2009
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_HH_ID                       ! Geanonimiseerd identificatienummer van het gezin waartoe het individu behoort
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NAREGNIS_DATBIRTH                    ! Geboortedatum van het individu in jaar en maand
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_CIVIL                       ! Burgerlijke staat van het individu op 1 januari 2009
    !/*podmi*/
    CHARACTER(len=13),ALLOCATABLE,DIMENSION(:)              :: PODMI_BEGDATUM_2008                  ! Begindatum betaling recht POD MI (2008)
    CHARACTER(len=13),ALLOCATABLE,DIMENSION(:)              :: PODMI_ENDDATUM_2008                  ! Einddatum betaling recht POD MI (2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NumMonth_Hulp_2008                   ! Aantal maanden gedurende dewelke individu financile steun OCMW ontvangt in 2008
    CHARACTER(len=13),ALLOCATABLE,DIMENSION(:)              :: PODMI_BEGDATUM_2008_NOSEL            ! Begindatum betaling recht POD MI (2008): op basis van niet geselecteerde lijnen
    CHARACTER(len=13),ALLOCATABLE,DIMENSION(:)              :: PODMI_ENDDATUM_2008_NOSEL            ! Einddatum betaling recht POD MI (2008): op basis van niet geselecteerde lijnen
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InPODMI_SEL                          ! Individu geselecteerd uit basisdata PODMI voor verwerking steekproef
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InPODMI_NOSEL                        ! Individu in basisdata PODMI maar niet geselecteerd voor verwerking steekproef
    !/*fodSZ*/
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: date_start_recon                     ! Begindatum erkenning (=beslissing medische commissie)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: date_end_recon                       ! Einddatum erkenning (=beslissing medische commissie)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: arr                                  ! IVT
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: critere_un                           ! punten criterium 1
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: critere_deux                         ! punten criterium 2
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: critere_trois                        ! punten criterium 3
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: critere_quatre                       ! punten criterium 4
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: critere_cinq                         ! punten criterium 5
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: critere_six                          ! punten criterium 6
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: date_start_2008                      ! Begindatum recht
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: date_end_2008                        ! Einddatum recht
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NUM_Indiv_PAY                        ! Aantal keer dat een individu een betaling ontving gedurende het jaar
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: per_start_pay_2008                   ! Begindatum betaling
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: per_end_pay_2008                     ! Einddatum betaling
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: date_start_ptn_2008                  ! Begindatum partnerschap
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: date_end_ptn_2008                    ! Einddatum partnerschap
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: gecodeerd_insz_partner               ! Gecodeerd INSZ van de partner van het individu
    CHARACTER(len=12),ALLOCATABLE,DIMENSION(:)              :: IT_Type_PC_2008                      ! IT-IVT-THB status bij de start van iedere maand in 2008
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InFODSZ                              ! Individu geselecteerd uit n van de drie basisbestanden FODSZ voor verwerking steekproef
    !/*Pensioenkadaster*/
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: IGO_2008                             ! GIB/IGO (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: IGO_PDATE_2008                       ! startdatum betaling GIB/IGO (betalingsjaar = 2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: begindatum_pensioen_c_2008           ! begindatum_pensioen_c (geregistreerd in 2008)
    !Exogenous variables read from the file SOCBEN_PIT.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_PIT                   !Numerical part of the Personal identification number read from the PIT data set
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_FINTAX_ANN                       !Final taxes due (sum of state taxes and crises surcharges)     
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTOI_ANN_SP1                     !Net taxable occupational income of spouse 1 after applying marital splitting
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTOI_ANN_SP2                     !Net taxable occupational income of spouse 2 after applying marital splitting
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTI_ANN_SP1                      !Net taxable income of spouse 1
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTI_ANN_SP2                      !Net taxable income of spouse 2
    !Exogenous variables read from the file SOCBEN_FAMAL.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMAL                 !Numerical part of the Personal identification number read from the FAMAL data set  
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_CHILD_ELIG                     !Dummyvariable for child receiving child enefit (0=does not receive; 1=receives)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_ORPHAN_QT                      !Individual is considered to be an orphan       
    !Exogenous variables read from the file SOCBEN_CONTRIB.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_CONTRIB               !Numerical part of the Personal identification number read from the CONTRIB data set
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPRIV_QT                   !Gross taxable labour income earned as wage earner on the private labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPUB_QT                    !Gross taxable labour income earned as wage earner on the public labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISELF_QT                   !Gross taxable labour income earned as self employed
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPENS_QT                   !Gross taxable pension benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEARNE_QT                  !Gross taxable early retirement benefits of the new type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEAROL_QT                  !Gross taxable early retirement benefits of the old type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOLDUN_QT                  !Gross taxable unemployment benefits of older unemployed with seniority supplement
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOTHUN_QT                  !Gross taxable unemployment benefits, other than early retirement benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIDISAB_QT                  !Gross taxable disability benefits of wage earners
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTINDOCC_QT                  !Gross taxable benefits in case of industrial accidents or occupational diseases
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISICK_QT                   !Gross taxable benefits in case of sickness other than disability benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIFAMAL_QT                  !Gross taxable income from family allowances
    !Exogenous variables read from the file SOCBEN_UNEM.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_UNEM                  !Numerical part of the Personal identification number read from the UNEM data set
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_NUMMINU_QT                      !Number of months the unemployed is already in unemployment          
    !Exogenous variables read from the file SOCBEN_SICK.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_SICK                  !Numerical part of the Personal identification number read from the SICK data set
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DISABBEN_QT                     !Identification of the fact that the beneficiary receives disability benefits
    !Exogenous variables read from the file SOCBEN_CONSTRUCT.txt
    !Remark: SOCBEN construction variables are now treated seperatly but maybe we should consider to integrate them in the PIT module and consequently adapt their treatment in the SOCBEN module
    !SOCBEN_CONSTRUCT_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_CONSTRUCT             ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CADASTRAL_GINCOME                    ! Gross cadastral income (land property)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CADASTRAB_GINCOME                    ! Gross cadastral income (building)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FINANCIAL_WEALTH                     ! Financial wealth
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PRIVATE_PENSIONS                     ! Private pensions
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: ALIMONY_RECEIVED                     ! Alimony received
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: BENEFITS_NATURE                      ! Benefits in nature
    !Variables read from file MIMOSIS_FAMREL.txt                                                   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMREL                !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMILY                        !Unique identifier identifying the family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_RELATION                      !Relation of the individual with respect to the head of the family
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMTYPE                       !Type of family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_COUPLE                        !Type of couple to which the individual belongs
    !MIMOSIS_SAMPWEIG_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WEIGHT                ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_WEIGHT                       ! Gewicht dat corrigeert voor niet random selectie van de observatie-eenheden
    !==========================================================================
    !Endogeneous variables
    !to rank variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NEW_RANK                             !New rank after sorting and re-ranking       
    !to construct age variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_YEAR_AGE                      !Age of the individual expressed in years
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_MONTH_AGE                     !Age of the individual expressed in moths
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_MONTH_BIRTH                   !Month of birth(mm)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_YEAR_BIRTH                    !Year of birth (yyyy)
    !to construct information at the level of the sociological household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_HH_SIZE                       !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_START_HH                      !Variable stores an identification of the first household member (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_END_HH                        !Variable stores an identification of the last household member (maximal size vector = number of individuals)   
    INTEGER                                                 :: N_HH_SOCBEN                          !number of households analysed
    !To construct variables listed in the module note
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPRIV_ANN                  !Gross taxable labour income earned as wage earner on the private labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPUB_ANN                   !Gross taxable labour income earned as wage earner on the public labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISELF_ANN                  !Gross taxable labour income earned as self employed
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPENS_ANN                  !Gross taxable pension benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEARNE_ANN                 !Gross taxable early retirement benefits of the new type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEAROL_ANN                 !Gross taxable early retirement benefits of the old type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOLDUN_ANN                 !Gross taxable unemployment benefits of older unemployed with seniority supplement
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOTHUN_ANN                 !Gross taxable unemployment benefits, other than early retirement benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIDISAB_ANN                 !Gross taxable disability benefits of wage earners
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTINDOCC_ANN                 !Gross taxable benefits in case of industrial accidents or occupational diseases
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISICK_ANN                  !Gross taxable benefits in case of sickness other than disability benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIFAMAL_ANN                 !Gross taxable income from family allowances
    !instruments
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_IGOLEEFL_ANN                  !Indicator that states whether an individual is entitled to open the leefloon or igo-right on age conditions
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: SOCBEN_ENTLEIGO_ANN                  !Entitled to leefloon or IGO (matrix will store both real observed and reconstructed values)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_DEPCHILD_ANN                  !Counter storing the number of children that depend on the leefloon-receiver         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_CHILDNOB_ANN                  !Number of children for which the leefloon-receiver does not receive child benefits but who are with him now and then or for which he pays alimony (see legislation 2004)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_TEST_ANN                      !identifier to guarantee that an individual is tested only once in certain tests
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_TESTP_ANN                     !identifier to guarantee that an individual is tested only once in certain tests
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_PARTNID_ANN                   !store the numerical ID of the partner of an individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_HHDIINC_ANN                   !Disposable income of the sociological household on annual basis
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_HHNOLAB_ANN                   !Household without labour income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_NMLEEFL_ANN                   !Store the net means used to execute the means test when determining the leefloon
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_NMIGO_ANN                     !Store the net means used to execute the means test when determining the IGO
    !leefloon-creation
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_FAMSIT_2001_ANN               !Family charge position of the leefloon receiver according to the 2001 rules            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_FAMSIT_2003_ANN               !Family charge position of the leefloon receiver according to the 2003 rules
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_FAMSIT_2005_ANN               !Family charge position of the leefloon receiver according to the 2005 rules
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_LEEFLOON_ANN                  !Amount of minimum income transfer to those entitled to the leefloon
    !IGO creation
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_FAMIGO_2001_ANN               !Family charge position of the IGO receiver according to the 2001 rules            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_IGO_ANN                       !Amount of income transfer to elderly
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_IGO_NMONTHS                   !Number of months during which IGO is paid
    !Handicap-creation
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_ENTIT_ANN                     !Entitled to IT benefit 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_ENTIVT_ANN                    !Entitled to IVT benefit 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_ENTTHB_ANN                    !Entitled to THB benefit 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_PARITIVT_ANN                  !Partner for application of IT/IVT rules
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_PARTHB_ANN                    !Partner for application of THB rules
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MEDIVT_ANN                    !Medical categorie for IVT benefit (reductie in verdienvermogen)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_MEDIT_ANN                     !Medical categorie for IT benefit (mate van zelfredzaamheid) 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_MEDTHB_ANN                    !Medical categorie for THB benefit (mate van zelfredzaamheid)  
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_FAMITIVT_ANN                  !Family type for determination of IT/IVT
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_FAMTHB_ANN                    !Family type for determination of THB
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MEANSIVT_2001_ANN             !Individual means needed when executing the means test for the IVT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MEANSIT_2001_ANN              !Individual means needed when executing the means test for the IT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MEANSIVT_2004_ANN             !Individual means needed when executing the means test for the IVT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MEANSIT_2004_ANN              !Individual means needed when executing the means test for the IT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MEANSTHB_ANN                  !Individual means needed when executing the means test for the THB benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_IVT_ANN                       !Amount of IVT attributed to person with a disability
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_IT_ANN                        !Amount of IT attributed to person with a disability
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_THB_ANN                       !Amount of THB attributed to person with a disability
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_IT_NMONTHS                    !Number of months during which IT is paid
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_IVT_NMONTHS                   !Number of months during which IVT is paid
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_THB_NMONTHS                   !Number of months during which THB is paid
    !HealthBen-creation
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MEVT_ANN                      !Individual means needed when executing the means test for the VT health benefit
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_VTSTAT_ANN                    !Individual status that could generate the VT benefit
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_VT_ANN                        !Individual categorie for VT benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MEOMNIO_ANN                   !Individual means needed when executing the means test for the OMNIO health benefit
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_OMNIOSTA_ANN                  !Individual categorie for OMNIO benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MEINCMAF_ANN                  !Individual means needed when executing the means test for the Income MAF
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_INCMAFST_ANN                  !Individual categorie for Income MAF benefit (setting the threshold for out of pocket payments)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SOCBEN_SOCMAFST_ANN                  !Individual categorie for Social MAF benefit
    !overall storing variable
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MINIMEX_ANN                   !Amount of Minimum income transfer

    !Instruments
    INTEGER                                                 :: pers_j,hh_i,i,j,k
    INTEGER                                                 :: ierror

END MODULE SOCBEN_DATA                                                                
!==========================================================================

!=============================================!
!This module contains all the subroutines and !
!functions used in the SOCBEN MODULE          ! 
!=============================================!
MODULE SOCBEN_SUBROUTINES

    USE SOCBEN_DATA
    USE MIMOSIS_DATA    
    !use routine for sorting and ranking data
    USE RANKING_SORTING_ROUTINES

    IMPLICIT NONE

    CONTAINS

!=====================================================!
!Allocate all exogenous vectors of dimension DIM_ALLOC!
!=====================================================!

SUBROUTINE SOCBEN_ALLOC_EXOG(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !Exogenous variables
    !SOCBEN_INPUT_Write.sas
    !/*popchar*/
    ALLOCATE(GECODEERD_INSZ_CHAR(DIM_ALLOC))
    ALLOCATE(GECODEERD_INSZ_INPUT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_RELATION(DIM_ALLOC))
    ALLOCATE(NAREGNIS_SEX(DIM_ALLOC))
    ALLOCATE(NAREGNIS_HH_ID(DIM_ALLOC))
    ALLOCATE(NAREGNIS_DATBIRTH(DIM_ALLOC))
    ALLOCATE(NAREGNIS_CIVIL(DIM_ALLOC))
    !/*podmi*/
    ALLOCATE(PODMI_BEGDATUM_2008(DIM_ALLOC))
    ALLOCATE(PODMI_ENDDATUM_2008(DIM_ALLOC))
    ALLOCATE(NumMonth_Hulp_2008(DIM_ALLOC))
    ALLOCATE(PODMI_BEGDATUM_2008_NOSEL(DIM_ALLOC))
    ALLOCATE(PODMI_ENDDATUM_2008_NOSEL(DIM_ALLOC))
    ALLOCATE(InPODMI_SEL(DIM_ALLOC))
    ALLOCATE(InPODMI_NOSEL(DIM_ALLOC))
    !/*fodSZ*/
    ALLOCATE(date_start_recon(DIM_ALLOC))
    ALLOCATE(date_end_recon(DIM_ALLOC))
    ALLOCATE(arr(DIM_ALLOC))
    ALLOCATE(critere_un(DIM_ALLOC))
    ALLOCATE(critere_deux(DIM_ALLOC))
    ALLOCATE(critere_trois(DIM_ALLOC))
    ALLOCATE(critere_quatre(DIM_ALLOC))
    ALLOCATE(critere_cinq(DIM_ALLOC))
    ALLOCATE(critere_six(DIM_ALLOC))
    ALLOCATE(date_start_2008(DIM_ALLOC))
    ALLOCATE(date_end_2008(DIM_ALLOC))
    ALLOCATE(NUM_Indiv_PAY(DIM_ALLOC))
    ALLOCATE(per_start_pay_2008(DIM_ALLOC))
    ALLOCATE(per_end_pay_2008(DIM_ALLOC))
    ALLOCATE(date_start_ptn_2008(DIM_ALLOC))
    ALLOCATE(date_end_ptn_2008(DIM_ALLOC))
    ALLOCATE(gecodeerd_insz_partner(DIM_ALLOC))
    ALLOCATE(IT_Type_PC_2008(DIM_ALLOC))
    ALLOCATE(InFODSZ(DIM_ALLOC))
    !/*Pensioenkadaster*/
    ALLOCATE(IGO_2008(DIM_ALLOC))
    ALLOCATE(IGO_PDATE_2008(DIM_ALLOC))
    ALLOCATE(begindatum_pensioen_c_2008(DIM_ALLOC))
    !Exogenous variables read from the file SOCBEN_PIT.txt
    ALLOCATE(GECODEERD_INSZ_PIT(DIM_ALLOC))
    ALLOCATE(PIT_FINTAX_ANN(DIM_ALLOC))
    ALLOCATE(PIT_NTOI_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTOI_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTI_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTI_ANN_SP2(DIM_ALLOC))
    !Exogenous variables read from the file SOCBEN_FAMAL.txt
    ALLOCATE(GECODEERD_INSZ_FAMAL(DIM_ALLOC))
    ALLOCATE(FAMAL_CHILD_ELIG(DIM_ALLOC))
    ALLOCATE(FAMAL_ORPHAN_QT(DIM_ALLOC))
    !Exogenous variables read from the file SOCBEN_CONTRIB.txt
    ALLOCATE(GECODEERD_INSZ_CONTRIB(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPRIV_QT(DIM_ALLOC))   
    ALLOCATE(CONTRIB_GTIPUB_QT(DIM_ALLOC))    
    ALLOCATE(CONTRIB_GTISELF_QT(DIM_ALLOC))   
    ALLOCATE(CONTRIB_GTIPENS_QT(DIM_ALLOC))   
    ALLOCATE(CONTRIB_GTIEARNE_QT(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTIEAROL_QT(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTIOLDUN_QT(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTIOTHUN_QT(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTIDISAB_QT(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTINDOCC_QT(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTISICK_QT(DIM_ALLOC))   
    ALLOCATE(CONTRIB_GTIFAMAL_QT(DIM_ALLOC))
    !Exogenous variables read from the file SOCBEN_UNEM.txt
    ALLOCATE(GECODEERD_INSZ_UNEM(DIM_ALLOC))
    ALLOCATE(UNEM_NUMMINU_QT(DIM_ALLOC))
    !Exogenous variables read from the file SOCBEN_SICK.txt
    ALLOCATE(GECODEERD_INSZ_SICK(DIM_ALLOC))
    ALLOCATE(SICK_DISABBEN_QT(DIM_ALLOC))
    !SOCBEN_CONSTRUCT_Write.sas
    ALLOCATE(GECODEERD_INSZ_CONSTRUCT(DIM_ALLOC))
    ALLOCATE(CADASTRAL_GINCOME(DIM_ALLOC))
    ALLOCATE(CADASTRAB_GINCOME(DIM_ALLOC))
    ALLOCATE(FINANCIAL_WEALTH(DIM_ALLOC))
    ALLOCATE(PRIVATE_PENSIONS(DIM_ALLOC))
    ALLOCATE(ALIMONY_RECEIVED(DIM_ALLOC))
    ALLOCATE(BENEFITS_NATURE(DIM_ALLOC))
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    ALLOCATE(GECODEERD_INSZ_FAMREL(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMILY(DIM_ALLOC))
    ALLOCATE(FAMREL_RELATION(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMTYPE(DIM_ALLOC))
    ALLOCATE(FAMREL_COUPLE(DIM_ALLOC))
    !MIMOSIS_SAMPWEIG_Write.sas
    ALLOCATE(GECODEERD_INSZ_WEIGHT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WEIGHT(DIM_ALLOC))

END SUBROUTINE SOCBEN_ALLOC_EXOG

!=====================================================!
!Allocate all endogenous vectors of dimension DIM_ALLOC!
!=====================================================!

SUBROUTINE SOCBEN_ALLOC_ENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !Endogeneous variables
    !to rank variables
    ALLOCATE(NEW_RANK(DIM_ALLOC))
    !to construct age variables
    ALLOCATE(SOCBEN_YEAR_AGE(DIM_ALLOC))
    ALLOCATE(SOCBEN_MONTH_AGE(DIM_ALLOC))
    ALLOCATE(SOCBEN_MONTH_BIRTH(DIM_ALLOC))
    ALLOCATE(SOCBEN_YEAR_BIRTH(DIM_ALLOC))
    !to construct information at the level of the sociological household
    ALLOCATE(SOCBEN_HH_SIZE(DIM_ALLOC))
    ALLOCATE(SOCBEN_START_HH(DIM_ALLOC))
    ALLOCATE(SOCBEN_END_HH(DIM_ALLOC))
    !To construct variables listed in the module note
    !annual values
    ALLOCATE(CONTRIB_GTIPRIV_ANN(DIM_ALLOC))   
    ALLOCATE(CONTRIB_GTIPUB_ANN(DIM_ALLOC))    
    ALLOCATE(CONTRIB_GTISELF_ANN(DIM_ALLOC))   
    ALLOCATE(CONTRIB_GTIPENS_ANN(DIM_ALLOC))   
    ALLOCATE(CONTRIB_GTIEARNE_ANN(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTIEAROL_ANN(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTIOLDUN_ANN(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTIOTHUN_ANN(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTIDISAB_ANN(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTINDOCC_ANN(DIM_ALLOC))  
    ALLOCATE(CONTRIB_GTISICK_ANN(DIM_ALLOC))   
    ALLOCATE(CONTRIB_GTIFAMAL_ANN(DIM_ALLOC))
   !instruments
    ALLOCATE(SOCBEN_IGOLEEFL_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_ENTLEIGO_ANN(DIM_ALLOC,2))
    ALLOCATE(SOCBEN_DEPCHILD_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_CHILDNOB_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_TEST_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_TESTP_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_PARTNID_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_HHDIINC_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_HHNOLAB_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_NMLEEFL_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_NMIGO_ANN(DIM_ALLOC))
    !leefloon-creation
    ALLOCATE(SOCBEN_FAMSIT_2001_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_FAMSIT_2003_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_FAMSIT_2005_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_LEEFLOON_ANN(DIM_ALLOC))
    !IGO creation
    ALLOCATE(SOCBEN_FAMIGO_2001_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_IGO_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_IGO_NMONTHS(DIM_ALLOC))
    !Handicap-creation
    ALLOCATE(SOCBEN_ENTIT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_ENTIVT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_ENTTHB_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_PARITIVT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_PARTHB_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MEDIVT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MEDIT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MEDTHB_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_FAMITIVT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_FAMTHB_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MEANSIVT_2001_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MEANSIT_2001_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MEANSIVT_2004_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MEANSIT_2004_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MEANSTHB_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_IVT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_IT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_THB_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_IT_NMONTHS(DIM_ALLOC))
    ALLOCATE(SOCBEN_IVT_NMONTHS(DIM_ALLOC))
    ALLOCATE(SOCBEN_THB_NMONTHS(DIM_ALLOC))
    !HealthBen-creation
    ALLOCATE(SOCBEN_MEVT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_VTSTAT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_VT_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MEOMNIO_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_OMNIOSTA_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MEINCMAF_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_INCMAFST_ANN(DIM_ALLOC))
    ALLOCATE(SOCBEN_SOCMAFST_ANN(DIM_ALLOC))
    !overall storing variable
    ALLOCATE(SOCBEN_MINIMEX_ANN(DIM_ALLOC))

END SUBROUTINE SOCBEN_ALLOC_ENDO

!===============================================!
!Deallocate all allocated variables (including parameters)                        
!===============================================!

SUBROUTINE SOCBEN_DEALLOC

    !Deallocate allocated exogenous and endogenous variables
    !Exogenous variables
    !SOCBEN_INPUT_Write.sas
    !/*popchar*/
    DEALLOCATE(GECODEERD_INSZ_CHAR)
    DEALLOCATE(GECODEERD_INSZ_INPUT)
    DEALLOCATE(NAREGNIS_RELATION)
    DEALLOCATE(NAREGNIS_SEX)
    DEALLOCATE(NAREGNIS_HH_ID)
    DEALLOCATE(NAREGNIS_DATBIRTH)
    DEALLOCATE(NAREGNIS_CIVIL)
    !/*podmi*/
    DEALLOCATE(PODMI_BEGDATUM_2008)
    DEALLOCATE(PODMI_ENDDATUM_2008)
    DEALLOCATE(NumMonth_Hulp_2008)
    DEALLOCATE(PODMI_BEGDATUM_2008_NOSEL)
    DEALLOCATE(PODMI_ENDDATUM_2008_NOSEL)
    DEALLOCATE(InPODMI_SEL)
    DEALLOCATE(InPODMI_NOSEL)
    !/*fodSZ*/
    DEALLOCATE(date_start_recon)
    DEALLOCATE(date_end_recon)
    DEALLOCATE(arr)
    DEALLOCATE(critere_un)
    DEALLOCATE(critere_deux)
    DEALLOCATE(critere_trois)
    DEALLOCATE(critere_quatre)
    DEALLOCATE(critere_cinq)
    DEALLOCATE(critere_six)
    DEALLOCATE(date_start_2008)
    DEALLOCATE(date_end_2008)
    DEALLOCATE(NUM_Indiv_PAY)
    DEALLOCATE(per_start_pay_2008)
    DEALLOCATE(per_end_pay_2008)
    DEALLOCATE(date_start_ptn_2008)
    DEALLOCATE(date_end_ptn_2008)
    DEALLOCATE(gecodeerd_insz_partner)
    DEALLOCATE(IT_Type_PC_2008)
    DEALLOCATE(InFODSZ)
    !/*Pensioenkadaster*/
    DEALLOCATE(IGO_2008)
    DEALLOCATE(IGO_PDATE_2008)
    DEALLOCATE(begindatum_pensioen_c_2008)
    !Exogenous variables read from the file SOCBEN_PIT.txt
    DEALLOCATE(GECODEERD_INSZ_PIT)
    DEALLOCATE(PIT_FINTAX_ANN)
    DEALLOCATE(PIT_NTOI_ANN_SP1)
    DEALLOCATE(PIT_NTOI_ANN_SP2)
    DEALLOCATE(PIT_NTI_ANN_SP1)
    DEALLOCATE(PIT_NTI_ANN_SP2)
    !Exogenous variables read from the file SOCBEN_FAMAL.txt
    DEALLOCATE(GECODEERD_INSZ_FAMAL)
    DEALLOCATE(FAMAL_CHILD_ELIG)
    DEALLOCATE(FAMAL_ORPHAN_QT)
    !Exogenous variables read from the file SOCBEN_CONTRIB.txt
    DEALLOCATE(GECODEERD_INSZ_CONTRIB)
    DEALLOCATE(CONTRIB_GTIPRIV_QT)   
    DEALLOCATE(CONTRIB_GTIPUB_QT)    
    DEALLOCATE(CONTRIB_GTISELF_QT)   
    DEALLOCATE(CONTRIB_GTIPENS_QT)   
    DEALLOCATE(CONTRIB_GTIEARNE_QT)  
    DEALLOCATE(CONTRIB_GTIEAROL_QT)  
    DEALLOCATE(CONTRIB_GTIOLDUN_QT) 
    DEALLOCATE(CONTRIB_GTIOTHUN_QT) 
    DEALLOCATE(CONTRIB_GTIDISAB_QT)  
    DEALLOCATE(CONTRIB_GTINDOCC_QT)  
    DEALLOCATE(CONTRIB_GTISICK_QT)   
    DEALLOCATE(CONTRIB_GTIFAMAL_QT)
    !Exogenous variables read from the file SOCBEN_UNEM.txt
    DEALLOCATE(GECODEERD_INSZ_UNEM)
    DEALLOCATE(UNEM_NUMMINU_QT)
    !Exogenous variables read from the file SOCBEN_SICK.txt
    DEALLOCATE(GECODEERD_INSZ_SICK)
    DEALLOCATE(SICK_DISABBEN_QT)
    !SOCBEN_CONSTRUCT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_CONSTRUCT)
    DEALLOCATE(CADASTRAL_GINCOME)
    DEALLOCATE(CADASTRAB_GINCOME)
    DEALLOCATE(FINANCIAL_WEALTH)
    DEALLOCATE(PRIVATE_PENSIONS)
    DEALLOCATE(ALIMONY_RECEIVED)
    DEALLOCATE(BENEFITS_NATURE)
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    DEALLOCATE(GECODEERD_INSZ_FAMREL)
    DEALLOCATE(FAMREL_FAMILY)
    DEALLOCATE(FAMREL_RELATION)
    DEALLOCATE(FAMREL_FAMTYPE)
    DEALLOCATE(FAMREL_COUPLE)
    !MIMOSIS_SAMPWEIG_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WEIGHT)
    DEALLOCATE(MIMOSIS_WEIGHT)
    !==========================================================================
    !Endogenous variables 
    !to rank variables
    DEALLOCATE(NEW_RANK)
    !to construct age variables
    DEALLOCATE(SOCBEN_YEAR_AGE)
    DEALLOCATE(SOCBEN_MONTH_AGE)
    DEALLOCATE(SOCBEN_MONTH_BIRTH)
    DEALLOCATE(SOCBEN_YEAR_BIRTH)
    !to construct information at the level of the sociological household
    DEALLOCATE(SOCBEN_HH_SIZE)
    DEALLOCATE(SOCBEN_START_HH)
    DEALLOCATE(SOCBEN_END_HH)
    !To construct variables listed in the module note
    !annual values
    DEALLOCATE(CONTRIB_GTIPRIV_ANN)   
    DEALLOCATE(CONTRIB_GTIPUB_ANN)    
    DEALLOCATE(CONTRIB_GTISELF_ANN)   
    DEALLOCATE(CONTRIB_GTIPENS_ANN)   
    DEALLOCATE(CONTRIB_GTIEARNE_ANN)  
    DEALLOCATE(CONTRIB_GTIEAROL_ANN)  
    DEALLOCATE(CONTRIB_GTIOLDUN_ANN) 
    DEALLOCATE(CONTRIB_GTIOTHUN_ANN) 
    DEALLOCATE(CONTRIB_GTIDISAB_ANN)  
    DEALLOCATE(CONTRIB_GTINDOCC_ANN)  
    DEALLOCATE(CONTRIB_GTISICK_ANN)   
    DEALLOCATE(CONTRIB_GTIFAMAL_ANN)
    !instruments
    DEALLOCATE(SOCBEN_IGOLEEFL_ANN)
    DEALLOCATE(SOCBEN_ENTLEIGO_ANN)
    DEALLOCATE(SOCBEN_DEPCHILD_ANN)
    DEALLOCATE(SOCBEN_CHILDNOB_ANN)
    DEALLOCATE(SOCBEN_TEST_ANN)
    DEALLOCATE(SOCBEN_TESTP_ANN)
    DEALLOCATE(SOCBEN_PARTNID_ANN)
    DEALLOCATE(SOCBEN_HHDIINC_ANN)
    DEALLOCATE(SOCBEN_HHNOLAB_ANN)
    DEALLOCATE(SOCBEN_NMLEEFL_ANN)
    DEALLOCATE(SOCBEN_NMIGO_ANN)
    !leefloon-creation
    DEALLOCATE(SOCBEN_FAMSIT_2001_ANN)
    DEALLOCATE(SOCBEN_FAMSIT_2003_ANN)
    DEALLOCATE(SOCBEN_FAMSIT_2005_ANN)
    DEALLOCATE(SOCBEN_LEEFLOON_ANN)
    !IGO creation
    DEALLOCATE(SOCBEN_FAMIGO_2001_ANN)
    DEALLOCATE(SOCBEN_IGO_ANN)
    DEALLOCATE(SOCBEN_IGO_NMONTHS)
    !Handicap-creation
    DEALLOCATE(SOCBEN_ENTIT_ANN)
    DEALLOCATE(SOCBEN_ENTIVT_ANN)
    DEALLOCATE(SOCBEN_ENTTHB_ANN)
    DEALLOCATE(SOCBEN_PARITIVT_ANN)
    DEALLOCATE(SOCBEN_PARTHB_ANN)
    DEALLOCATE(SOCBEN_MEDIVT_ANN)
    DEALLOCATE(SOCBEN_MEDIT_ANN)
    DEALLOCATE(SOCBEN_MEDTHB_ANN)
    DEALLOCATE(SOCBEN_FAMITIVT_ANN)
    DEALLOCATE(SOCBEN_FAMTHB_ANN)
    DEALLOCATE(SOCBEN_MEANSIVT_2001_ANN)
    DEALLOCATE(SOCBEN_MEANSIT_2001_ANN)
    DEALLOCATE(SOCBEN_MEANSIVT_2004_ANN)
    DEALLOCATE(SOCBEN_MEANSIT_2004_ANN)
    DEALLOCATE(SOCBEN_MEANSTHB_ANN)
    DEALLOCATE(SOCBEN_IVT_ANN)
    DEALLOCATE(SOCBEN_IT_ANN)
    DEALLOCATE(SOCBEN_THB_ANN)
    DEALLOCATE(SOCBEN_IT_NMONTHS)
    DEALLOCATE(SOCBEN_IVT_NMONTHS)
    DEALLOCATE(SOCBEN_THB_NMONTHS)
    !HealthBen-creation
    DEALLOCATE(SOCBEN_MEVT_ANN)
    DEALLOCATE(SOCBEN_VTSTAT_ANN)
    DEALLOCATE(SOCBEN_VT_ANN)
    DEALLOCATE(SOCBEN_MEOMNIO_ANN)
    DEALLOCATE(SOCBEN_OMNIOSTA_ANN)
    DEALLOCATE(SOCBEN_MEINCMAF_ANN)
    DEALLOCATE(SOCBEN_INCMAFST_ANN)
    DEALLOCATE(SOCBEN_SOCMAFST_ANN)
    !overall storing variable
    DEALLOCATE(SOCBEN_MINIMEX_ANN)

    !Deallocate parameters
    DEALLOCATE(SOCBEN_FAMREL_ANN)
    DEALLOCATE(SOCBEN_MINTYM65_ANN)
    DEALLOCATE(SOCBEN_CAPRATES_ANN)
    DEALLOCATE(SOCBEN_DISREGAR_ANN)
    DEALLOCATE(SOCBEN_MINTYP65_ANN)
    DEALLOCATE(SOCBEN_GWB_ANN)
    DEALLOCATE(SOCBEN_CAPRAIGO_ANN)
    DEALLOCATE(SOCBEN_DISREIGO_ANN)
    DEALLOCATE(SOCBEN_BAMIVT_ANN)    
    DEALLOCATE(SOCBEN_LABIVT_ANN)    
    DEALLOCATE(SOCBEN_DISREIVT_ANN)   
    DEALLOCATE(SOCBEN_BAMIT_ANN)     
    DEALLOCATE(SOCBEN_PARTITA_ANN)   
    DEALLOCATE(SOCBEN_PARTITB_ANN)   
    DEALLOCATE(SOCBEN_LABIT_ANN)     
    DEALLOCATE(SOCBEN_REPIT_ANN)     
    DEALLOCATE(SOCBEN_DISREIT_ANN)   
    DEALLOCATE(SOCBEN_BAMTHB_ANN)    
    DEALLOCATE(SOCBEN_CAPRATHB_ANN)  
    DEALLOCATE(SOCBEN_DISRETHB_ANN)  
    DEALLOCATE(SOCBEN_INCMAF_ANN)  

END SUBROUTINE SOCBEN_DEALLOC

!===============================================!
!Initialise endogenous variables                        
!===============================================!

SUBROUTINE SOCBEN_INITENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors
    INTEGER                                                 :: pers_j                                    !index for do loop

    !==========================================================================
    !Endogenous variables (defined in module notes)

    DO pers_j=1,DIM_ALLOC
        !to rank variables
        NEW_RANK(pers_j) = 0
        !to construct age variables
        SOCBEN_YEAR_AGE(pers_j) = 0
        SOCBEN_MONTH_AGE(pers_j) = 0
        SOCBEN_MONTH_BIRTH(pers_j) = 0
        SOCBEN_YEAR_BIRTH(pers_j) = 0
        !to construct information at the level of the sociological household
        SOCBEN_HH_SIZE(pers_j) = 0
        SOCBEN_START_HH(pers_j) = 0
        SOCBEN_END_HH(pers_j) = 0
        !To construct variables listed in the module note
        !annual values
        CONTRIB_GTIPRIV_ANN(pers_j) = 0   
        CONTRIB_GTIPUB_ANN(pers_j) = 0    
        CONTRIB_GTISELF_ANN(pers_j) = 0   
        CONTRIB_GTIPENS_ANN(pers_j) = 0   
        CONTRIB_GTIEARNE_ANN(pers_j) = 0  
        CONTRIB_GTIEAROL_ANN(pers_j) = 0  
        CONTRIB_GTIOLDUN_ANN(pers_j) = 0 
        CONTRIB_GTIOTHUN_ANN(pers_j) = 0 
        CONTRIB_GTIDISAB_ANN(pers_j) = 0  
        CONTRIB_GTINDOCC_ANN(pers_j) = 0  
        CONTRIB_GTISICK_ANN(pers_j) = 0   
        CONTRIB_GTIFAMAL_ANN(pers_j) = 0
        !instruments
        SOCBEN_IGOLEEFL_ANN(pers_j) = 0
        SOCBEN_ENTLEIGO_ANN(pers_j,1:2) = 0
        SOCBEN_DEPCHILD_ANN(pers_j) = 0
        SOCBEN_CHILDNOB_ANN(pers_j) = 0
        SOCBEN_TEST_ANN(pers_j) = 0
        SOCBEN_TESTP_ANN(pers_j) = 0
        SOCBEN_PARTNID_ANN(pers_j) = 0
        SOCBEN_HHDIINC_ANN(pers_j) = 0
        SOCBEN_HHNOLAB_ANN(pers_j) = 0
        SOCBEN_NMLEEFL_ANN(pers_j) = 0
        SOCBEN_NMIGO_ANN(pers_j) = 0
        !leefloon-creation
        SOCBEN_FAMSIT_2001_ANN(pers_j) = 0
        SOCBEN_FAMSIT_2003_ANN(pers_j) = 0
        SOCBEN_FAMSIT_2005_ANN(pers_j) = 0
        SOCBEN_LEEFLOON_ANN(pers_j) = 0
        !IGO creation
        SOCBEN_FAMIGO_2001_ANN(pers_j) = 0
        SOCBEN_IGO_ANN(pers_j) = 0
        SOCBEN_IGO_NMONTHS(pers_j) = 0
        !Handicap-creation
        SOCBEN_ENTIT_ANN(pers_j) = 0
        SOCBEN_ENTIVT_ANN(pers_j) = 0
        SOCBEN_ENTTHB_ANN(pers_j) = 0
        SOCBEN_PARITIVT_ANN(pers_j) = 0
        SOCBEN_PARTHB_ANN(pers_j) = 0
        SOCBEN_MEDIVT_ANN(pers_j) = 0
        SOCBEN_MEDIT_ANN(pers_j) = 0
        SOCBEN_MEDTHB_ANN(pers_j) = 0
        SOCBEN_FAMITIVT_ANN(pers_j) = 0
        SOCBEN_FAMTHB_ANN(pers_j) = 0
        SOCBEN_MEANSIVT_2001_ANN(pers_j) = 0
        SOCBEN_MEANSIT_2001_ANN(pers_j) = 0
        SOCBEN_MEANSIVT_2004_ANN(pers_j) = 0
        SOCBEN_MEANSIT_2004_ANN(pers_j) = 0
        SOCBEN_MEANSTHB_ANN(pers_j) = 0
        SOCBEN_IVT_ANN(pers_j) = 0
        SOCBEN_IT_ANN(pers_j) = 0
        SOCBEN_THB_ANN(pers_j) = 0
        SOCBEN_IT_NMONTHS(pers_j) = 0
        SOCBEN_IVT_NMONTHS(pers_j) = 0
        SOCBEN_THB_NMONTHS(pers_j) = 0
        !HealthBen-creation
        SOCBEN_MEVT_ANN(pers_j) = 0
        SOCBEN_VTSTAT_ANN(pers_j) = 0
        SOCBEN_VT_ANN(pers_j) = 0
        SOCBEN_MEOMNIO_ANN(pers_j) = 0
        SOCBEN_OMNIOSTA_ANN(pers_j) = 0
        SOCBEN_MEINCMAF_ANN(pers_j) = 0
        SOCBEN_INCMAFST_ANN(pers_j) = 0
        SOCBEN_SOCMAFST_ANN(pers_j) = 0
        !overall storing variable
        SOCBEN_MINIMEX_ANN(pers_j) = 0
    END DO

END SUBROUTINE SOCBEN_INITENDO

!========== SUBROUTINE FAMAL_RANKANDSORT ===============!
!RANK and SORT exogenous variables 
!   Step 1: rank and sort all variables according to their 
!           individual identification number which is read in as an integer
!   Step 2: rank and sort all variables according to the 
!           household identification number
!======================================================!

SUBROUTINE SOCBEN_RANKANDSORT

!STEP1: ranking and sorting with the individuals identification number
    !Rank individual identification number from the INPUT data file
    CALL RANKING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    !SOCBEN_INPUT_Write.sas
    !/*popchar*/
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_CIVIL(1:n_rec),NEW_RANK)
    !/*podmi*/
    CALL SORTING(PODMI_BEGDATUM_2008(1:n_rec),NEW_RANK)
    CALL SORTING(PODMI_ENDDATUM_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NumMonth_Hulp_2008(1:n_rec),NEW_RANK)
    CALL SORTING(PODMI_BEGDATUM_2008_NOSEL(1:n_rec),NEW_RANK)
    CALL SORTING(PODMI_ENDDATUM_2008_NOSEL(1:n_rec),NEW_RANK)
    CALL SORTING(InPODMI_SEL(1:n_rec),NEW_RANK)
    CALL SORTING(InPODMI_NOSEL(1:n_rec),NEW_RANK)
    !/*fodSZ*/
    CALL SORTING(date_start_recon(1:n_rec),NEW_RANK)
    CALL SORTING(date_end_recon(1:n_rec),NEW_RANK)
    CALL SORTING(arr(1:n_rec),NEW_RANK)
    CALL SORTING(critere_un(1:n_rec),NEW_RANK)
    CALL SORTING(critere_deux(1:n_rec),NEW_RANK)
    CALL SORTING(critere_trois(1:n_rec),NEW_RANK)
    CALL SORTING(critere_quatre(1:n_rec),NEW_RANK)
    CALL SORTING(critere_cinq(1:n_rec),NEW_RANK)
    CALL SORTING(critere_six(1:n_rec),NEW_RANK)
    CALL SORTING(date_start_2008(1:n_rec),NEW_RANK)
    CALL SORTING(date_end_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NUM_Indiv_PAY(1:n_rec),NEW_RANK)
    CALL SORTING(per_start_pay_2008(1:n_rec),NEW_RANK)
    CALL SORTING(per_end_pay_2008(1:n_rec),NEW_RANK)
    CALL SORTING(date_start_ptn_2008(1:n_rec),NEW_RANK)
    CALL SORTING(date_end_ptn_2008(1:n_rec),NEW_RANK)
    CALL SORTING(gecodeerd_insz_partner(1:n_rec),NEW_RANK)
    CALL SORTING(IT_Type_PC_2008(1:n_rec),NEW_RANK)
    CALL SORTING(InFODSZ(1:n_rec),NEW_RANK)
    !/*Pensioenkadaster*/
    CALL SORTING(IGO_2008(1:n_rec),NEW_RANK)
    CALL SORTING(IGO_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(begindatum_pensioen_c_2008(1:n_rec),NEW_RANK)
    !Rank individual identification number from the PIT data file
    CALL RANKING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    !Sort variables read from the file SOCBEN_PIT.txt
    CALL SORTING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_FINTAX_ANN(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_NTOI_ANN_SP1(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_NTOI_ANN_SP2(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_NTI_ANN_SP1(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_NTI_ANN_SP2(1:n_rec),NEW_RANK)
    !Rank individual identification number from the FAMAL data file
    CALL RANKING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    !Sort variables read from the file SOCBEN_FAMAL.txt
    CALL SORTING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMAL_CHILD_ELIG(1:n_rec),NEW_RANK)
    CALL SORTING(FAMAL_ORPHAN_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from the CONTRIB data file
    CALL RANKING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    !Sort variables read from the file SOCBEN_CONTRIB.txt
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from the UNEM data file
    CALL RANKING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    !Sort variables read from the file SOCBEN_UNEM.txt
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_NUMMINU_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from the SICK data file
    CALL RANKING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    !Sort variables read from the file SOCBEN_SICK.txt
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_DISABBEN_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from the CONSTRUCT data file
    CALL RANKING(GECODEERD_INSZ_CONSTRUCT(1:n_rec),NEW_RANK)
    !SOCBEN_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CONSTRUCT(1:n_rec),NEW_RANK)
    CALL SORTING(CADASTRAL_GINCOME(1:n_rec),NEW_RANK)
    CALL SORTING(CADASTRAB_GINCOME(1:n_rec),NEW_RANK)
    CALL SORTING(FINANCIAL_WEALTH(1:n_rec),NEW_RANK)
    CALL SORTING(PRIVATE_PENSIONS(1:n_rec),NEW_RANK)
    CALL SORTING(ALIMONY_RECEIVED(1:n_rec),NEW_RANK)
    CALL SORTING(BENEFITS_NATURE(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_FAMREL.txt                                           
    CALL RANKING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !Rank individual identification number from the WEIGHT data file
    CALL RANKING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL SOCBEN_CONSISTENCY_CHECK

!STEP2: ranking and sorting with the household identification number
    !Rank household number
    CALL RANKING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    !SOCBEN_INPUT_Write.sas
    !/*popchar*/
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_CIVIL(1:n_rec),NEW_RANK)
    !/*podmi*/
    CALL SORTING(PODMI_BEGDATUM_2008(1:n_rec),NEW_RANK)
    CALL SORTING(PODMI_ENDDATUM_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NumMonth_Hulp_2008(1:n_rec),NEW_RANK)
    CALL SORTING(PODMI_BEGDATUM_2008_NOSEL(1:n_rec),NEW_RANK)
    CALL SORTING(PODMI_ENDDATUM_2008_NOSEL(1:n_rec),NEW_RANK)
    CALL SORTING(InPODMI_SEL(1:n_rec),NEW_RANK)
    CALL SORTING(InPODMI_NOSEL(1:n_rec),NEW_RANK)
    !/*fodSZ*/
    CALL SORTING(date_start_recon(1:n_rec),NEW_RANK)
    CALL SORTING(date_end_recon(1:n_rec),NEW_RANK)
    CALL SORTING(arr(1:n_rec),NEW_RANK)
    CALL SORTING(critere_un(1:n_rec),NEW_RANK)
    CALL SORTING(critere_deux(1:n_rec),NEW_RANK)
    CALL SORTING(critere_trois(1:n_rec),NEW_RANK)
    CALL SORTING(critere_quatre(1:n_rec),NEW_RANK)
    CALL SORTING(critere_cinq(1:n_rec),NEW_RANK)
    CALL SORTING(critere_six(1:n_rec),NEW_RANK)
    CALL SORTING(date_start_2008(1:n_rec),NEW_RANK)
    CALL SORTING(date_end_2008(1:n_rec),NEW_RANK)
    CALL SORTING(NUM_Indiv_PAY(1:n_rec),NEW_RANK)
    CALL SORTING(per_start_pay_2008(1:n_rec),NEW_RANK)
    CALL SORTING(per_end_pay_2008(1:n_rec),NEW_RANK)
    CALL SORTING(date_start_ptn_2008(1:n_rec),NEW_RANK)
    CALL SORTING(date_end_ptn_2008(1:n_rec),NEW_RANK)
    CALL SORTING(gecodeerd_insz_partner(1:n_rec),NEW_RANK)
    CALL SORTING(IT_Type_PC_2008(1:n_rec),NEW_RANK)
    CALL SORTING(InFODSZ(1:n_rec),NEW_RANK)
    !/*Pensioenkadaster*/
    CALL SORTING(IGO_2008(1:n_rec),NEW_RANK)
    CALL SORTING(IGO_PDATE_2008(1:n_rec),NEW_RANK)
    CALL SORTING(begindatum_pensioen_c_2008(1:n_rec),NEW_RANK)
    !Sort variables read from the file SOCBEN_PIT.txt
    CALL SORTING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_FINTAX_ANN(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_NTOI_ANN_SP1(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_NTOI_ANN_SP2(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_NTI_ANN_SP1(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_NTI_ANN_SP2(1:n_rec),NEW_RANK)
    !Sort variables read from the file SOCBEN_FAMAL.txt
    CALL SORTING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMAL_CHILD_ELIG(1:n_rec),NEW_RANK)
    CALL SORTING(FAMAL_ORPHAN_QT(1:n_rec),NEW_RANK)
    !Sort variables read from the file SOCBEN_CONTRIB.txt
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT(1:n_rec),NEW_RANK)
    !Sort variables read from the file SOCBEN_UNEM.txt
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_NUMMINU_QT(1:n_rec),NEW_RANK)
    !Sort variables read from the file SOCBEN_SICK.txt
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_DISABBEN_QT(1:n_rec),NEW_RANK)
    !SOCBEN_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CONSTRUCT(1:n_rec),NEW_RANK)
    CALL SORTING(CADASTRAL_GINCOME(1:n_rec),NEW_RANK)
    CALL SORTING(CADASTRAB_GINCOME(1:n_rec),NEW_RANK)
    CALL SORTING(FINANCIAL_WEALTH(1:n_rec),NEW_RANK)
    CALL SORTING(PRIVATE_PENSIONS(1:n_rec),NEW_RANK)
    CALL SORTING(ALIMONY_RECEIVED(1:n_rec),NEW_RANK)
    CALL SORTING(BENEFITS_NATURE(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL SOCBEN_CONSISTENCY_CHECK

END SUBROUTINE SOCBEN_RANKANDSORT

!===============================================!
!Reads in all parameters from the parameter file!                         
!===============================================!

SUBROUTINE SOCBEN_PARAMETER_FILEREAD

    INTEGER:: i
    !open parameter file       
    OPEN (UNIT=3, FILE=SOCBEN_PARLOC, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
        
    DO i=1,n_loclines/2
        !Read default path names
        !Start reading the normal file names        
        IF (CONTROL_IO(2*i-1)=='SOCBEN_INPUT_PATH') THEN
            SOCBEN_INPUT_PATH = CONTROL_IO(2*i)   
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_PIT_FILE') THEN
            SOCBEN_PIT_FILE = CONTROL_IO(2*i)     
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_FAMAL_FILE') THEN
            SOCBEN_FAMAL_FILE = CONTROL_IO(2*i)     
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_INPUT_FILE') THEN
            SOCBEN_INPUT_FILE = CONTROL_IO(2*i)     
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_CONTRIB_FILE') THEN
            SOCBEN_CONTRIB_FILE = CONTROL_IO(2*i)     
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_UNEM_FILE') THEN
            SOCBEN_UNEM_FILE = CONTROL_IO(2*i)     
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_SICK_FILE') THEN
            SOCBEN_SICK_FILE = CONTROL_IO(2*i)     
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_CONSTRUCT_FILE') THEN
            SOCBEN_CONSTRUCT_FILE = CONTROL_IO(2*i)                 
        ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_INPUT_PATH') THEN
            FAMREL_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_FAMREL_FILE') THEN
            MIMOSIS_FAMREL_FILE = CONTROL_IO(2*i)
        !Read path names of output files
        ELSE IF (CONTROL_IO(2*i-1)=='OUTPUT_PATH')THEN
            OUTPUT_PATH = CONTROL_IO(2*i)   
        ELSE IF (CONTROL_IO(2*i-1)=='EVAL_SOCBEN_FILE')THEN
            EVAL_SOCBEN_FILE = CONTROL_IO(2*i)   
        !Read material to read sample weights                                                    
        ELSE IF (CONTROL_IO(2*i-1)=='CONSTANT_INPUT_PATH') THEN
            CONSTANT_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_SAMPWEIG_FILE') THEN
            MIMOSIS_SAMPWEIG_FILE = CONTROL_IO(2*i)
        END IF 
    END DO
    !otherwise check for length of default path and file name parameters 
    SOCBEN_PAR_BREAK_POINT(1)=INDEX(SOCBEN_INPUT_PATH,SOCBEN_PAR_BREAK)
    SOCBEN_PAR_BREAK_POINT(2)=INDEX(SOCBEN_INPUT_FILE,SOCBEN_PAR_BREAK)                          
    SOCBEN_PAR_BREAK_POINT(3)=INDEX(SOCBEN_PIT_FILE,SOCBEN_PAR_BREAK)                          
    SOCBEN_PAR_BREAK_POINT(4)=INDEX(SOCBEN_FAMAL_FILE,SOCBEN_PAR_BREAK)                          
    SOCBEN_PAR_BREAK_POINT(5)=INDEX(SOCBEN_CONTRIB_FILE,SOCBEN_PAR_BREAK)                          
    SOCBEN_PAR_BREAK_POINT(11)=INDEX(SOCBEN_UNEM_FILE,SOCBEN_PAR_BREAK)                          
    SOCBEN_PAR_BREAK_POINT(12)=INDEX(SOCBEN_SICK_FILE,SOCBEN_PAR_BREAK)                          
    SOCBEN_PAR_BREAK_POINT(6)=INDEX(SOCBEN_CONSTRUCT_FILE,SOCBEN_PAR_BREAK)                          
    SOCBEN_PAR_BREAK_POINT(9)=INDEX(FAMREL_INPUT_PATH,SOCBEN_PAR_BREAK)
    SOCBEN_PAR_BREAK_POINT(10)=INDEX(MIMOSIS_FAMREL_FILE,SOCBEN_PAR_BREAK)    
    SOCBEN_PAR_BREAK_POINT(7)=INDEX(CONSTANT_INPUT_PATH,SOCBEN_PAR_BREAK)
    SOCBEN_PAR_BREAK_POINT(8)=INDEX(MIMOSIS_SAMPWEIG_FILE,SOCBEN_PAR_BREAK)
    !merge infomration (path+name) to create default input file names     
    SOCBEN_INPUT=SOCBEN_INPUT_PATH(1:SOCBEN_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_INPUT_FILE(1:SOCBEN_PAR_BREAK_POINT(2)-1)
    SOCBEN_PIT=SOCBEN_INPUT_PATH(1:SOCBEN_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_PIT_FILE(1:SOCBEN_PAR_BREAK_POINT(3)-1)
    SOCBEN_FAMAL=SOCBEN_INPUT_PATH(1:SOCBEN_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_FAMAL_FILE(1:SOCBEN_PAR_BREAK_POINT(4)-1)
    SOCBEN_CONTRIB=SOCBEN_INPUT_PATH(1:SOCBEN_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_CONTRIB_FILE(1:SOCBEN_PAR_BREAK_POINT(5)-1)
    SOCBEN_UNEM=SOCBEN_INPUT_PATH(1:SOCBEN_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_UNEM_FILE(1:SOCBEN_PAR_BREAK_POINT(11)-1)
    SOCBEN_SICK=SOCBEN_INPUT_PATH(1:SOCBEN_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_SICK_FILE(1:SOCBEN_PAR_BREAK_POINT(12)-1)
    SOCBEN_CONSTRUCT=SOCBEN_INPUT_PATH(1:SOCBEN_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_CONSTRUCT_FILE(1:SOCBEN_PAR_BREAK_POINT(6)-1)
    MIMOSIS_FAMREL = FAMREL_INPUT_PATH(1:SOCBEN_PAR_BREAK_POINT(9)-1)//&
    &MIMOSIS_FAMREL_FILE(1:SOCBEN_PAR_BREAK_POINT(10)-1)    
    MIMOSIS_SAMPWEIG = CONSTANT_INPUT_PATH(1:SOCBEN_PAR_BREAK_POINT(7)-1)//&
    &MIMOSIS_SAMPWEIG_FILE(1:SOCBEN_PAR_BREAK_POINT(8)-1)
    !check for length of new path and file name parameters
    SOCBEN_PAR_BREAK_POINT(1)=INDEX(OUTPUT_PATH,SOCBEN_PAR_BREAK)
    SOCBEN_PAR_BREAK_POINT(2)=INDEX(EVAL_SOCBEN_FILE,SOCBEN_PAR_BREAK)                          
    !merge infomration (path+name) to create default output file name     
    EVAL_SOCBEN=OUTPUT_PATH(1:SOCBEN_PAR_BREAK_POINT(1)-1)//&
    &EVAL_SOCBEN_FILE(1:SOCBEN_PAR_BREAK_POINT(2)-1)
    !Trim file names
    CALL TRIM@(SOCBEN_INPUT)
    CALL TRIM@(SOCBEN_PIT)
    CALL TRIM@(SOCBEN_FAMAL)
    CALL TRIM@(SOCBEN_CONTRIB)    
    CALL TRIM@(SOCBEN_UNEM)    
    CALL TRIM@(SOCBEN_SICK)    
    CALL TRIM@(SOCBEN_CONSTRUCT)
    CALL TRIM@(MIMOSIS_FAMREL)    
    CALL TRIM@(MIMOSIS_SAMPWEIG)
    CALL TRIM@(EVAL_SOCBEN)
    !write names to log file
    WRITE(15,'(''Print of reconstructed input and output file names'')')
    CALL DRAWLINE(15,100,'=')
    WRITE (15,*) 'INPUT FILE=  ',       SOCBEN_INPUT
    WRITE (15,*) 'PIT FILE=',           SOCBEN_PIT
    WRITE (15,*) 'FAMAL FILE=',         SOCBEN_FAMAL
    WRITE (15,*) 'CONTRIB FILE=',       SOCBEN_CONTRIB
    WRITE (15,*) 'UNEM FILE=',          SOCBEN_UNEM
    WRITE (15,*) 'SICK FILE=',          SOCBEN_SICK
    WRITE (15,*) 'CONSTRUCT FILE=',     SOCBEN_CONSTRUCT
    WRITE (15,*) 'FAMREL FILE= ',       MIMOSIS_FAMREL   
    WRITE (15,*) 'WEIGHT FILE= ',       MIMOSIS_SAMPWEIG   
    WRITE (15,*) 'EVAL FILE= ',         EVAL_SOCBEN

    ALLOCATE(SOCBEN_PAR_NAMES(n_par))     
    !read in all parameters by looping over the lines in the parameter file
    WRITE(15,'(/,''Print of read in operation of parameter file'')')
    CALL DRAWLINE(15,100,'=')    
    i=1
    DO 
        READ(3,'(a40, 2i10, a10)') SOCBEN_PAR_NAMES(i), SOCBEN_PAR_L, SOCBEN_PAR_C,SOCBEN_PAR_CURR  
        IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_FAMREL_ANN') THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE(SOCBEN_FAMREL_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C))
            READ(3,'(a20)') SOCBEN_PAR_FORMAT
            READ(3,SOCBEN_PAR_FORMAT)   ((SOCBEN_FAMREL_ANN(j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_FAMREL_ANN(j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)        
        !leefloon-parameters
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_MINTYM65_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_MINTYM65_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_MINTYM65_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_MINTYM65_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_TRAIN_PROG_MONTH')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_TRAIN_PROG_MONTH 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_TRAIN_PROG_MONTH
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADL_RATE')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_CADL_RATE 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_CADL_RATE
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADB_RATE')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADB_RATE 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADB_RATE
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADL_BASICD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADL_BASICD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADL_BASICD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADB_BASICD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADB_BASICD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADB_BASICD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADB_SPOUSED_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADB_SPOUSED_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADB_SPOUSED_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADB_CHILDD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADB_CHILDD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADB_CHILDD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CAPRATES_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_CAPRATES_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_CAPRATES_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_CAPRATES_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_DISREGAR_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_DISREGAR_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_DISREGAR_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_DISREGAR_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)        
        !IGO parameters
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_MINTYP65_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_MINTYP65_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_MINTYP65_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_MINTYP65_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_GWB_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_GWB_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_GWB_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_GWB_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_LABIGO_RATE')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_LABIGO_RATE 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_LABIGO_RATE
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_PENSIGO_RATE')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_PENSIGO_RATE 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_PENSIGO_RATE
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADIGO_RATE')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADIGO_RATE 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADIGO_RATE
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADBIGO_BASICD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADBIGO_BASICD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADBIGO_BASICD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADIGO_CHILDD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADIGO_CHILDD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADIGO_CHILDD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADLIGO_BASICD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADLIGO_BASICD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADLIGO_BASICD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CAPRAIGO_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_CAPRAIGO_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_CAPRAIGO_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_CAPRAIGO_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)        
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_DISREIGO_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_DISREIGO_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_DISREIGO_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_DISREIGO_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)        
        !Parameters to determine the benefits for disabled people
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_BAMIVT_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_BAMIVT_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_BAMIVT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_BAMIVT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADIVT_BASICD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_CADIVT_BASICD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_CADIVT_BASICD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADIVT_CHILDD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_CADIVT_CHILDD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_CADIVT_CHILDD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_LABIVT_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_LABIVT_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_LABIVT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_LABIVT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_OTHIVT_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_OTHIVT_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_OTHIVT_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_PARTIVT_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_PARTIVT_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_PARTIVT_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_DISREIVT_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_DISREIVT_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_DISREIVT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_DISREIVT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_BAMIT_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_BAMIT_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_BAMIT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_BAMIT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_PARTITA_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_PARTITA_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_PARTITA_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_PARTITA_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_PARTITB_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_PARTITB_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_PARTITB_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_PARTITB_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_LABIT_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_LABIT_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_LABIT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_LABIT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_REPIT_ANN') THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE(SOCBEN_REPIT_ANN(SOCBEN_PAR_L))
            READ(3,'(a20)') SOCBEN_PAR_FORMAT
            READ(3,SOCBEN_PAR_FORMAT) (SOCBEN_REPIT_ANN(j),j=1,SOCBEN_PAR_L)
            WRITE(15,SOCBEN_PAR_FORMAT) (SOCBEN_REPIT_ANN(j),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_DISREIT_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_DISREIT_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_DISREIT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_DISREIT_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_BAMTHB_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_BAMTHB_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_BAMTHB_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_BAMTHB_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_LABTHB_RATE')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_LABTHB_RATE 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_LABTHB_RATE
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_PENSTHB_RATE')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_PENSTHB_RATE 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_PENSTHB_RATE
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADLTHB_RATE')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_CADLTHB_RATE 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_CADLTHB_RATE
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADBTHB_RATE')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_CADBTHB_RATE 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_CADBTHB_RATE
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADBTHB_BASICD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_CADBTHB_BASICD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_CADBTHB_BASICD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADBTHB_CHILDD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_CADBTHB_CHILDD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_CADBTHB_CHILDD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADLTHB_BASICD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  SOCBEN_CADLTHB_BASICD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_CADLTHB_BASICD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CAPRATHB_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_CAPRATHB_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_CAPRATHB_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_CAPRATHB_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_DISRETHB_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_DISRETHB_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_DISRETHB_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_DISRETHB_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        !Parameters for health benefit creation
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADBVT_BASICD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADBVT_BASICD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADBVT_BASICD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_CADBVT_CHILDD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_CADBVT_CHILDD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_CADBVT_CHILDD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_VT_BASICD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_VT_BASICD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_VT_BASICD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_VT_CHILDD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_VT_CHILDD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_VT_CHILDD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_OMNIO_BASICD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_OMNIO_BASICD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_OMNIO_BASICD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_OMNIO_CHILDD_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)SOCBEN_OMNIO_CHILDD_ANN 
            WRITE(15,SOCBEN_PAR_FORMAT)SOCBEN_OMNIO_CHILDD_ANN
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_INCMAF_ANN')THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            ALLOCATE (SOCBEN_INCMAF_ANN(SOCBEN_PAR_L,SOCBEN_PAR_C)) 
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT)  ((SOCBEN_INCMAF_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L) 
            WRITE(15,SOCBEN_PAR_FORMAT) ((SOCBEN_INCMAF_ANN (j,k),k=1,SOCBEN_PAR_C),j=1,SOCBEN_PAR_L)
        !Parameters for simulations of the DG PMH
        ELSE IF (SOCBEN_PAR_NAMES(i)=='SOCBEN_SIMDGPMH_QT') THEN
            WRITE(15,'(a40)') SOCBEN_PAR_NAMES(i)
            READ (3,'(a20)') SOCBEN_PAR_FORMAT
            READ (3,SOCBEN_PAR_FORMAT) SOCBEN_SIMDGPMH_QT
            WRITE(15,SOCBEN_PAR_FORMAT) SOCBEN_SIMDGPMH_QT
        ELSE IF (SOCBEN_PAR_NAMES(i)=='end_parameters')THEN
            EXIT
        END IF
        i=i+1
    END DO
    DEALLOCATE(SOCBEN_PAR_NAMES)

    CLOSE (UNIT=3)
    WRITE(*,*) 'Subroutine SOCBEN_PARAMETER_FILEREAD of NORMAL RUN....runned correctly'
    
END SUBROUTINE SOCBEN_PARAMETER_FILEREAD

!==================================!
!Reads in data from the INPUT file !
!==================================!

SUBROUTINE SOCBEN_INPUT_FILEREAD

    OPEN(UNIT=5, FILE=SOCBEN_INPUT, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_INPUT
        WRITE(15,'("Could not find ", a100)') SOCBEN_INPUT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(A14,I10,I10,I10,A7,I10,A13,A13,I10,A13,A13,I10,I10,A7,A7,I10,&
            &I10,I10,I10,I10,I10,I10,A7,A7,I10,A7,A7,A7,A7,A14,A12,I10,&
            &F15.2,A7,A7)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
            &NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),NAREGNIS_CIVIL(pers_j),&
            &PODMI_BEGDATUM_2008(pers_j),PODMI_ENDDATUM_2008(pers_j),NUMMONTH_HULP_2008(pers_j),&
            &PODMI_BEGDATUM_2008_NOSEL(pers_j),PODMI_ENDDATUM_2008_NOSEL(pers_j),INPODMI_SEL(pers_j),&
            &INPODMI_NOSEL(pers_j),DATE_START_RECON(pers_j),DATE_END_RECON(pers_j),&
            &ARR(pers_j),CRITERE_UN(pers_j),CRITERE_DEUX(pers_j),&
            &CRITERE_TROIS(pers_j),CRITERE_QUATRE(pers_j),CRITERE_CINQ(pers_j),&
            &CRITERE_SIX(pers_j),DATE_START_2008(pers_j),DATE_END_2008(pers_j),&
            &NUM_INDIV_PAY(pers_j),PER_START_PAY_2008(pers_j),PER_END_PAY_2008(pers_j),&
            &DATE_START_PTN_2008(pers_j),DATE_END_PTN_2008(pers_j),GECODEERD_INSZ_PARTNER(pers_j),&
            &IT_TYPE_PC_2008(pers_j),INFODSZ(pers_j),IGO_2008(pers_j),&
            &IGO_PDATE_2008(pers_j),BEGINDATUM_PENSIOEN_C_2008(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SOCBEN_INPUT
            END IF
            !Read in numeric part of the individual indentification character variable
            READ (GECODEERD_INSZ_CHAR(pers_j)(8:), '(I10)') GECODEERD_INSZ_INPUT(pers_j)
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),NAREGNIS_CIVIL(pers_j),&
                &PODMI_BEGDATUM_2008(pers_j),PODMI_ENDDATUM_2008(pers_j),NUMMONTH_HULP_2008(pers_j),&
                &PODMI_BEGDATUM_2008_NOSEL(pers_j),PODMI_ENDDATUM_2008_NOSEL(pers_j),INPODMI_SEL(pers_j),&
                &INPODMI_NOSEL(pers_j),DATE_START_RECON(pers_j),DATE_END_RECON(pers_j),&
                &ARR(pers_j),CRITERE_UN(pers_j),CRITERE_DEUX(pers_j),&
                &CRITERE_TROIS(pers_j),CRITERE_QUATRE(pers_j),CRITERE_CINQ(pers_j),&
                &CRITERE_SIX(pers_j),DATE_START_2008(pers_j),DATE_END_2008(pers_j),&
                &NUM_INDIV_PAY(pers_j),PER_START_PAY_2008(pers_j),PER_END_PAY_2008(pers_j),&
                &DATE_START_PTN_2008(pers_j),DATE_END_PTN_2008(pers_j),GECODEERD_INSZ_PARTNER(pers_j),&
                &IT_TYPE_PC_2008(pers_j),INFODSZ(pers_j),IGO_2008(pers_j),&
                &IGO_PDATE_2008(pers_j),BEGINDATUM_PENSIOEN_C_2008(pers_j)'')')
                WRITE(15,'(A14,I10,I10,I10,A7,I10,A13,A13,I10,A13,A13,I10,I10,A7,A7,I10,&
                &I10,I10,I10,I10,I10,I10,A7,A7,I10,A7,A7,A7,A7,A14,A12,I10,&
                &F15.2,A7,A7)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),NAREGNIS_CIVIL(pers_j),&
                &PODMI_BEGDATUM_2008(pers_j),PODMI_ENDDATUM_2008(pers_j),NUMMONTH_HULP_2008(pers_j),&
                &PODMI_BEGDATUM_2008_NOSEL(pers_j),PODMI_ENDDATUM_2008_NOSEL(pers_j),INPODMI_SEL(pers_j),&
                &INPODMI_NOSEL(pers_j),DATE_START_RECON(pers_j),DATE_END_RECON(pers_j),&
                &ARR(pers_j),CRITERE_UN(pers_j),CRITERE_DEUX(pers_j),&
                &CRITERE_TROIS(pers_j),CRITERE_QUATRE(pers_j),CRITERE_CINQ(pers_j),&
                &CRITERE_SIX(pers_j),DATE_START_2008(pers_j),DATE_END_2008(pers_j),&
                &NUM_INDIV_PAY(pers_j),PER_START_PAY_2008(pers_j),PER_END_PAY_2008(pers_j),&
                &DATE_START_PTN_2008(pers_j),DATE_END_PTN_2008(pers_j),GECODEERD_INSZ_PARTNER(pers_j),&
                &IT_TYPE_PC_2008(pers_j),INFODSZ(pers_j),IGO_2008(pers_j),&
                &IGO_PDATE_2008(pers_j),BEGINDATUM_PENSIOEN_C_2008(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(A14,I10,I10,I10,A7,I10,A13,A13,I10,A13,A13,I10,I10,A7,A7,I10,&
                &I10,I10,I10,I10,I10,I10,A7,A7,I10,A7,A7,A7,A7,A14,A12,I10,&
                &F15.2,A7,A7)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),NAREGNIS_CIVIL(pers_j),&
                &PODMI_BEGDATUM_2008(pers_j),PODMI_ENDDATUM_2008(pers_j),NUMMONTH_HULP_2008(pers_j),&
                &PODMI_BEGDATUM_2008_NOSEL(pers_j),PODMI_ENDDATUM_2008_NOSEL(pers_j),INPODMI_SEL(pers_j),&
                &INPODMI_NOSEL(pers_j),DATE_START_RECON(pers_j),DATE_END_RECON(pers_j),&
                &ARR(pers_j),CRITERE_UN(pers_j),CRITERE_DEUX(pers_j),&
                &CRITERE_TROIS(pers_j),CRITERE_QUATRE(pers_j),CRITERE_CINQ(pers_j),&
                &CRITERE_SIX(pers_j),DATE_START_2008(pers_j),DATE_END_2008(pers_j),&
                &NUM_INDIV_PAY(pers_j),PER_START_PAY_2008(pers_j),PER_END_PAY_2008(pers_j),&
                &DATE_START_PTN_2008(pers_j),DATE_END_PTN_2008(pers_j),GECODEERD_INSZ_PARTNER(pers_j),&
                &IT_TYPE_PC_2008(pers_j),INFODSZ(pers_j),IGO_2008(pers_j),&
                &IGO_PDATE_2008(pers_j),BEGINDATUM_PENSIOEN_C_2008(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SOCBEN_INPUT_FILEREAD of NORMAL RUN........runned correctly'
    WRITE(15,'(I7, " records uploaded from SOCBEN_INPUT_FILE")') pers_j-1

END SUBROUTINE SOCBEN_INPUT_FILEREAD 

!================================!
!Reads in data from the PIT file !
!================================!

SUBROUTINE SOCBEN_PIT_FILEREAD

    OPEN(UNIT=5, FILE=SOCBEN_PIT, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_PIT
        WRITE(15,'("Could not find ", a100)') SOCBEN_PIT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,5F15.2)',IOSTAT=ierror) GECODEERD_INSZ_PIT(pers_j),&
            &PIT_FINTAX_ANN(pers_j),PIT_NTOI_ANN_SP1(pers_j),PIT_NTOI_ANN_SP2(pers_j),PIT_NTI_ANN_SP1(pers_j),&
            &PIT_NTI_ANN_SP2(pers_j)   
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SOCBEN_PIT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_PIT(pers_j),&
                &PIT_FINTAX_ANN(pers_j),PIT_NTOI_ANN_SP1(pers_j),PIT_NTOI_ANN_SP2(pers_j),PIT_NTI_ANN_SP1(pers_j),&
                &PIT_NTI_ANN_SP2(pers_j)   '')')
                WRITE (15,'(7x,I7,5F15.2)') GECODEERD_INSZ_PIT(pers_j),&
                &PIT_FINTAX_ANN(pers_j),PIT_NTOI_ANN_SP1(pers_j),PIT_NTOI_ANN_SP2(pers_j),PIT_NTI_ANN_SP1(pers_j),&
                &PIT_NTI_ANN_SP2(pers_j)   
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE (15,'(7x,I7,5F15.2)') GECODEERD_INSZ_PIT(pers_j),&
                &PIT_FINTAX_ANN(pers_j),PIT_NTOI_ANN_SP1(pers_j),PIT_NTOI_ANN_SP2(pers_j),PIT_NTI_ANN_SP1(pers_j),&
                &PIT_NTI_ANN_SP2(pers_j)   
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SOCBEN_PIT_FILEREAD of NORMAL RUN..........runned correctly'
    WRITE(15,'(I7, " records uploaded from SOCBEN_PIT_FILE")') pers_j-1

END SUBROUTINE SOCBEN_PIT_FILEREAD 

!==================================!
!Reads in data from the FAMAL file !
!==================================!

SUBROUTINE SOCBEN_FAMAL_FILEREAD

    OPEN(UNIT=5, FILE=SOCBEN_FAMAL, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_FAMAL
        WRITE(15,'("Could not find ", a100)') SOCBEN_FAMAL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10,I10)',IOSTAT=ierror) GECODEERD_INSZ_FAMAL(pers_j),FAMAL_CHILD_ELIG(pers_j)&
            &,FAMAL_ORPHAN_QT(pers_j)   
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SOCBEN_FAMAL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMAL(pers_j),FAMAL_CHILD_ELIG(pers_j),FAMAL_ORPHAN_QT(pers_j)   '')')
                WRITE (15,'(7x,I7,I10,I10)') GECODEERD_INSZ_FAMAL(pers_j),FAMAL_CHILD_ELIG(pers_j),&
                &FAMAL_ORPHAN_QT(pers_j)   
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE (15,'(7x,I7,I10,I10)') GECODEERD_INSZ_FAMAL(pers_j),FAMAL_CHILD_ELIG(pers_j),&
                &FAMAL_ORPHAN_QT(pers_j)   
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SOCBEN_FAMAL_FILEREAD of NORMAL RUN........runned correctly'
    WRITE(15,'(I7, " records uploaded from SOCBEN_FAMAL_FILE")') pers_j-1

END SUBROUTINE SOCBEN_FAMAL_FILEREAD

!====================================!
!Reads in data from the CONTRIB file !
!====================================!

SUBROUTINE SOCBEN_CONTRIB_FILEREAD

    OPEN(UNIT=5, FILE=SOCBEN_CONTRIB, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_CONTRIB
        WRITE(15,'("Could not find ", a100)') SOCBEN_CONTRIB
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
            &F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
            &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
            &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
            &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
            &CONTRIB_GTIFAMAL_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SOCBEN_CONTRIB
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
                &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
                &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
                &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
                &CONTRIB_GTIFAMAL_QT(pers_j)  '')')
                WRITE (15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
                &F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
                &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
                &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
                &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
                &CONTRIB_GTIFAMAL_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE (15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
                &F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
                &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
                &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
                &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
                &CONTRIB_GTIFAMAL_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SOCBEN_CONTRIB_FILEREAD of NORMAL RUN......runned correctly'
    WRITE(15,'(I7, " records uploaded from SOCBEN_CONTRIB_FILE")') pers_j-1

END SUBROUTINE SOCBEN_CONTRIB_FILEREAD 

!====================================!
!Reads in data from the UNEM file    !
!====================================!

SUBROUTINE SOCBEN_UNEM_FILEREAD

    OPEN(UNIT=5, FILE=SOCBEN_UNEM, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_UNEM
        WRITE(15,'("Could not find ", a100)') SOCBEN_UNEM
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_UNEM(pers_j),UNEM_NUMMINU_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SOCBEN_UNEM
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_UNEM(pers_j),UNEM_NUMMINU_QT(pers_j)  '')')
                WRITE (15,'(7x,I7,I10)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_NUMMINU_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE (15,'(7x,I7,I10)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_NUMMINU_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SOCBEN_UNEM_FILEREAD of NORMAL RUN.........runned correctly'
    WRITE(15,'(I7, " records uploaded from SOCBEN_UNEM_FILE")') pers_j-1

END SUBROUTINE SOCBEN_UNEM_FILEREAD 

!====================================!
!Reads in data from the SICK file    !
!====================================!

SUBROUTINE SOCBEN_SICK_FILEREAD

    OPEN(UNIT=5, FILE=SOCBEN_SICK, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_SICK
        WRITE(15,'("Could not find ", a100)') SOCBEN_SICK
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_SICK(pers_j),SICK_DISABBEN_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SOCBEN_SICK
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_SICK(pers_j),SICK_DISABBEN_QT(pers_j)'')')
                WRITE (15,'(7x,I7,I10)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_DISABBEN_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE (15,'(7x,I7,I10)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_DISABBEN_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SOCBEN_SICK_FILEREAD of NORMAL RUN.........runned correctly'
    WRITE(15,'(I7, " records uploaded from SOCBEN_SICK_FILE")') pers_j-1

END SUBROUTINE SOCBEN_SICK_FILEREAD 

!======================================!
!Reads in data from the CONSTRUCT file !
!======================================!

SUBROUTINE SOCBEN_CONSTRUCT_FILEREAD

    OPEN(UNIT=5, FILE=SOCBEN_CONSTRUCT, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_CONSTRUCT
        WRITE(15,'("Could not find ", a100)') SOCBEN_CONSTRUCT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CONSTRUCT(pers_j),CADASTRAL_GINCOME(pers_j),CADASTRAB_GINCOME(pers_j),&
            &FINANCIAL_WEALTH(pers_j),PRIVATE_PENSIONS(pers_j),ALIMONY_RECEIVED(pers_j),&
            &BENEFITS_NATURE(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,SOCBEN_CONSTRUCT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CONSTRUCT(pers_j),CADASTRAL_GINCOME(pers_j),CADASTRAB_GINCOME(pers_j),&
                &FINANCIAL_WEALTH(pers_j),PRIVATE_PENSIONS(pers_j),ALIMONY_RECEIVED(pers_j),&
                &BENEFITS_NATURE(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_CONSTRUCT(pers_j),CADASTRAL_GINCOME(pers_j),CADASTRAB_GINCOME(pers_j),&
                &FINANCIAL_WEALTH(pers_j),PRIVATE_PENSIONS(pers_j),ALIMONY_RECEIVED(pers_j),&
                &BENEFITS_NATURE(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_CONSTRUCT(pers_j),CADASTRAL_GINCOME(pers_j),CADASTRAB_GINCOME(pers_j),&
                &FINANCIAL_WEALTH(pers_j),PRIVATE_PENSIONS(pers_j),ALIMONY_RECEIVED(pers_j),&
                &BENEFITS_NATURE(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SOCBEN_CONSTRUCT_FILEREAD of NORMAL RUN....runned correctly'
    WRITE(15,'(I7, " records uploaded from SOCBEN_CONSTRUCT_FILE")') pers_j-1

END SUBROUTINE SOCBEN_CONSTRUCT_FILEREAD 

!=====================================!
!Reads in data from the FAMREL file   !
!=====================================!

SUBROUTINE SOCBEN_FAMREL_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=MIMOSIS_FAMREL, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') MIMOSIS_FAMREL
        WRITE(15,'("Could not find ", a100)') MIMOSIS_FAMREL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
            &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_FAMREL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine SOCBEN_FAMREL_FILEREAD of NORMAL RUN.......runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_FAMREL")') pers_j-1

END SUBROUTINE SOCBEN_FAMREL_FILEREAD

!========================================!
!Reads in data from the MIMOSIS_SAMPWEIG file !
!========================================!

SUBROUTINE SOCBEN_WEIGHT_FILEREAD

    OPEN(UNIT=5, FILE=MIMOSIS_SAMPWEIG, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') MIMOSIS_SAMPWEIG
        WRITE(15,'("Could not find ", a100)') MIMOSIS_SAMPWEIG
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_SAMPWEIG
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine SOCBEN_WEIGHT_FILEREAD of NORMAL RUN.......runned correctly'
    WRITE(15,'(I7, " records uploaded from MIMOSIS_SAMPWEIG_FILE")') pers_j-1

END SUBROUTINE SOCBEN_WEIGHT_FILEREAD

!=======================================================================!
!Check whether individual identification numbers are sorted consistently!
!=======================================================================!

SUBROUTINE SOCBEN_CONSISTENCY_CHECK

    INTEGER:: pers_j
    DO pers_j=1, n_rec
        IF ((GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))          .OR.&
          & (GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))        .OR.&
          & (GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))      .OR.&
          & (GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_CONSTRUCT(pers_j))    .OR.&
          & (GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))         .OR.&
          & (GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))         .OR.&
          & (GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))       .OR.&
          & (GECODEERD_INSZ_PIT(pers_j)  .NE.GECODEERD_INSZ_FAMAL(pers_j))        .OR.&
          & (GECODEERD_INSZ_PIT(pers_j)  .NE.GECODEERD_INSZ_CONTRIB(pers_j))      .OR.&
          & (GECODEERD_INSZ_PIT(pers_j)  .NE.GECODEERD_INSZ_UNEM(pers_j))         .OR.&
          & (GECODEERD_INSZ_PIT(pers_j)  .NE.GECODEERD_INSZ_SICK(pers_j))         .OR.&
          & (GECODEERD_INSZ_PIT(pers_j)  .NE.GECODEERD_INSZ_CONSTRUCT(pers_j))    .OR.&
          & (GECODEERD_INSZ_PIT(pers_j)  .NE.GECODEERD_INSZ_WEIGHT(pers_j))       .OR.&
          & (GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))      .OR.&
          & (GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))         .OR.&
          & (GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))         .OR.&
          & (GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_CONSTRUCT(pers_j))    .OR.&
          & (GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))       .OR.&
          & (GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))       .OR.&
          & (GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))       .OR.&
          & (GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_CONSTRUCT(pers_j))  .OR.&
          & (GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))     .OR.&
          & (GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))          .OR.&
          & (GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_CONSTRUCT(pers_j))     .OR.&
          & (GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))        .OR.&
          & (GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_CONSTRUCT(pers_j))     .OR.&
          & (GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))        .OR.&
          & (GECODEERD_INSZ_CONSTRUCT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))       &
          &)   THEN
                WRITE (*,*) "Error: Ranking of external files does not match!"
                PAUSE
                STOP
        END IF
    END DO   

END SUBROUTINE SOCBEN_CONSISTENCY_CHECK 

!====================================================!
!Subroutine: to recode some exogenous variable values!
!====================================================!

SUBROUTINE SOCBEN_RECODE

    INTEGER:: pers_j

    DO pers_j=1, n_rec
        !SOCBEN_INPUT_WRITE.SAS
        !/*POPCHAR*/
        IF (INT(NAREGNIS_RELATION(pers_j)).EQ.9999999)              NAREGNIS_RELATION(pers_j)=0
        IF (INT(NAREGNIS_SEX(pers_j)).EQ.9999999)                   NAREGNIS_SEX(pers_j)=0
        IF (INT(NAREGNIS_HH_ID(pers_j)).EQ.9999999)                 NAREGNIS_HH_ID(pers_j)=0
        CALL TRIM@(NAREGNIS_DATBIRTH(pers_j))
        IF (INT(NAREGNIS_CIVIL(pers_j)).EQ.9999999)                 NAREGNIS_CIVIL(pers_j)=0
        !/*PODMI*/
        CALL TRIM@(PODMI_BEGDATUM_2008(pers_j))
        CALL TRIM@(PODMI_ENDDATUM_2008(pers_j))
        IF (INT(NUMMONTH_HULP_2008(pers_j)).EQ.9999999)             NUMMONTH_HULP_2008(pers_j)=0
        CALL TRIM@(PODMI_BEGDATUM_2008_NOSEL(pers_j))
        CALL TRIM@(PODMI_ENDDATUM_2008_NOSEL(pers_j))
        IF (INT(INPODMI_SEL(pers_j)).EQ.9999999)                    INPODMI_SEL(pers_j)=0
        IF (INT(INPODMI_NOSEL(pers_j)).EQ.9999999)                  INPODMI_NOSEL(pers_j)=0
        !/*FODSZ*/
        CALL TRIM@(DATE_START_RECON(pers_j))
        CALL TRIM@(DATE_END_RECON(pers_j))
        IF (INT(ARR(pers_j)).EQ.9999999)                            ARR(pers_j)=0
        IF (INT(CRITERE_UN(pers_j)).EQ.9999999)                     CRITERE_UN(pers_j)=0
        IF (INT(CRITERE_DEUX(pers_j)).EQ.9999999)                   CRITERE_DEUX(pers_j)=0
        IF (INT(CRITERE_TROIS(pers_j)).EQ.9999999)                  CRITERE_TROIS(pers_j)=0
        IF (INT(CRITERE_QUATRE(pers_j)).EQ.9999999)                 CRITERE_QUATRE(pers_j)=0
        IF (INT(CRITERE_CINQ(pers_j)).EQ.9999999)                   CRITERE_CINQ(pers_j)=0
        IF (INT(CRITERE_SIX(pers_j)).EQ.9999999)                    CRITERE_SIX(pers_j)=0
        CALL TRIM@(DATE_START_2008(pers_j))
        CALL TRIM@(DATE_END_2008(pers_j))
        IF (INT(NUM_INDIV_PAY(pers_j)).EQ.9999999)                  NUM_INDIV_PAY(pers_j)=0
        CALL TRIM@(PER_START_PAY_2008(pers_j))
        CALL TRIM@(PER_END_PAY_2008(pers_j))
        CALL TRIM@(DATE_START_PTN_2008(pers_j))
        CALL TRIM@(DATE_END_PTN_2008(pers_j))
        CALL TRIM@(GECODEERD_INSZ_PARTNER(pers_j))
        CALL TRIM@(IT_TYPE_PC_2008(pers_j))
        IF (INT(INFODSZ(pers_j)).EQ.9999999)                        INFODSZ(pers_j)=0
        !/*PENSIOENKADASTER*/
        IF (INT(IGO_2008(pers_j)).EQ.9999999)                       IGO_2008(pers_j)=0
        CALL TRIM@(IGO_PDATE_2008(pers_j))
        CALL TRIM@(BEGINDATUM_PENSIOEN_C_2008(pers_j))
        !Exogenous variables read from the file SOCBEN_PIT.txt
        IF (INT(PIT_FINTAX_ANN(pers_j)).EQ.9999999)                 PIT_FINTAX_ANN(pers_j)=0
        IF (INT(PIT_NTOI_ANN_SP1(pers_j)).EQ.9999999)               PIT_NTOI_ANN_SP1(pers_j)=0
        IF (INT(PIT_NTOI_ANN_SP2(pers_j)).EQ.9999999)               PIT_NTOI_ANN_SP2(pers_j)=0
        IF (INT(PIT_NTI_ANN_SP1(pers_j)).EQ.9999999)                PIT_NTI_ANN_SP1(pers_j)=0
        IF (INT(PIT_NTI_ANN_SP2(pers_j)).EQ.9999999)                PIT_NTI_ANN_SP2(pers_j)=0
        !Variables read from file SOCBEN_CONTRIB.txt
        IF (INT(CONTRIB_GTIPRIV_QT(pers_j)).EQ.9999999)             CONTRIB_GTIPRIV_QT(pers_j)=0
        IF (INT(CONTRIB_GTIPUB_QT(pers_j)).EQ.9999999)              CONTRIB_GTIPUB_QT(pers_j)=0
        IF (INT(CONTRIB_GTISELF_QT(pers_j)).EQ.9999999)             CONTRIB_GTISELF_QT(pers_j)=0
        IF (INT(CONTRIB_GTIPENS_QT(pers_j)).EQ.9999999)             CONTRIB_GTIPENS_QT(pers_j)=0
        IF (INT(CONTRIB_GTIEARNE_QT(pers_j)).EQ.9999999)            CONTRIB_GTIEARNE_QT(pers_j)=0
        IF (INT(CONTRIB_GTIEAROL_QT(pers_j)).EQ.9999999)            CONTRIB_GTIEAROL_QT(pers_j)=0
        IF (INT(CONTRIB_GTIOLDUN_QT(pers_j)).EQ.9999999)            CONTRIB_GTIOLDUN_QT(pers_j)=0
        IF (INT(CONTRIB_GTIOTHUN_QT(pers_j)).EQ.9999999)            CONTRIB_GTIOTHUN_QT(pers_j)=0
        IF (INT(CONTRIB_GTIDISAB_QT(pers_j)).EQ.9999999)            CONTRIB_GTIDISAB_QT(pers_j)=0
        IF (INT(CONTRIB_GTINDOCC_QT(pers_j)).EQ.9999999)            CONTRIB_GTINDOCC_QT(pers_j)=0
        IF (INT(CONTRIB_GTISICK_QT(pers_j)).EQ.9999999)             CONTRIB_GTISICK_QT(pers_j)=0
        IF (INT(CONTRIB_GTIFAMAL_QT(pers_j)).EQ.9999999)            CONTRIB_GTIFAMAL_QT(pers_j)=0
        !Variables read from file SOCBEN_UNEM.txt
        IF (INT(UNEM_NUMMINU_QT(pers_j)).EQ.9999999)                UNEM_NUMMINU_QT(pers_j)=0
        !SOCBEN_CONSTRUCT_WRITE.SAS
        IF (INT(CADASTRAL_GINCOME(pers_j)).EQ.9999999)              CADASTRAL_GINCOME(pers_j)=0
        IF (INT(CADASTRAB_GINCOME(pers_j)).EQ.9999999)              CADASTRAB_GINCOME(pers_j)=0
        IF (INT(FINANCIAL_WEALTH(pers_j)).EQ.9999999)               FINANCIAL_WEALTH(pers_j)=0
        IF (INT(PRIVATE_PENSIONS(pers_j)).EQ.9999999)               PRIVATE_PENSIONS(pers_j)=0
        IF (INT(ALIMONY_RECEIVED(pers_j)).EQ.9999999)               ALIMONY_RECEIVED(pers_j)=0
        IF (INT(BENEFITS_NATURE(pers_j)).EQ.9999999)                BENEFITS_NATURE(pers_j)=0
        !MIMOSIS_SAMPWEIG_WRITE.SAS
        IF (INT(MIMOSIS_WEIGHT(pers_j)).EQ.9999999)                 MIMOSIS_WEIGHT(pers_j)=0
    END DO

END SUBROUTINE SOCBEN_RECODE

!===============================================!
! HOUSEHOLD_SIZE computes household size        !
! and identifies starting and ending record of  !
! sociological household and stores it in a     !
! vector with household information             !
!Constructs variables                           !
!   SOCBEN_HH_SIZE                              !
!   SOCBEN_START_HH                             !
!   SOCBEN_END_HH                               !
!   N_HH_SOCBEN                                 !
!===============================================!

SUBROUTINE SOCBEN_HOUSEHOLD_SIZE

    INTEGER::  I
    INTEGER::  J
    INTEGER::  K
    INTEGER::  N

    I=1
    J=1
    K=0
    N=0

    DO
        J=J+N
        N=0
        K=K+1
        DO WHILE (NAREGNIS_HH_ID(i)==NAREGNIS_HH_ID(j))
            N=N+1
            I=I+1
            IF (i>n_rec) EXIT    
        END DO
        SOCBEN_HH_SIZE(k)=n
        SOCBEN_START_HH(k)=j
        SOCBEN_END_HH(k)=j+n-1
        IF (i>n_rec.OR.j>n_rec)EXIT
    END DO
    N_HH_SOCBEN=K

END SUBROUTINE SOCBEN_HOUSEHOLD_SIZE

!=================================================================
!Computes the age of each individual in terms of years and months!
!=================================================================

!============================================!
! Subroutine main input : NAREGNIS_DATBIRTH  ! 
! Subroutine main output: SOCBEN_YEAR_AGE    !
!                         SOCBEN_MONTH_AGE   !
!                         SOCBEN_YEAR_BIRTH  !
!                         SOCBEN_MONTH_BIRTH !   
!============================================!

SUBROUTINE SOCBEN_AGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        SOCBEN_PAR_BREAK_POINT(1)=INDEX(NAREGNIS_DATBIRTH(pers_j),'-')
        READ (NAREGNIS_DATBIRTH(pers_j)(1:SOCBEN_PAR_BREAK_POINT(1)-1),  '(I10)') SOCBEN_MONTH_BIRTH(pers_j)
        READ (NAREGNIS_DATBIRTH(pers_j)(SOCBEN_PAR_BREAK_POINT(1)+1:),   '(I10)') SOCBEN_YEAR_BIRTH(pers_j)          
        !determine age as number of years at the end of the year for which data are registered
        SOCBEN_YEAR_AGE(pers_j)  =MIMOSIS_YEAR-SOCBEN_YEAR_BIRTH(pers_j)
        !determine number of months as year age plus number of months passed until the end of the year
        SOCBEN_MONTH_AGE(pers_j) =SOCBEN_YEAR_AGE(pers_j)*12+(12-SOCBEN_MONTH_BIRTH(pers_j))
    END DO

END SUBROUTINE SOCBEN_AGE

!===================================================================================!
!Draw a line of a give number of characters
!===================================================================================!

SUBROUTINE DRAWLINE(NUNIT,N,WRITECHAR)

    ! DRAWS A LINE OF N CHARACTERS OF TYPE WRITECHAR ON UNIT NUNIT
    INTEGER                                                 :: N                                    !Number of characters to be drawn
    INTEGER                                                 :: NUNIT                                    !Number of characters to be drawn
    CHARACTER(len=1)                                        :: WRITECHAR                            !Type of character that has to be reproduced N times
    CHARACTER(len=1),DIMENSION(N)                           :: A                                    !Vector of dimension N containing WRITECHAR in the N positions
    CHARACTER(len=15)                                       :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=7)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK

    A(1:N)=WRITECHAR
    !convert the integer value N into a character value NCHAR with the Fortran internal Write function                                                        
    WRITE (NCHAR, '(I7)') N
    !Construct the output format by aid of the internal Fortran function NONBLK which identifies the first non blank character
    !and read the remainder of the string, i.e. :
    OUTPUTFMT = '(' // NCHAR(NONBLK(NCHAR):) // 'A1)'
    WRITE(NUNIT,OUTPUTFMT)(A(I),I=1,N)

END SUBROUTINE DRAWLINE

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as a result of different rates applied on an input amount.             !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The rate is in column 3 of this matrix.              !
!An example of this procedure is the social security contribution of    ! 
!self employed in principal activity before retirement age              !
!In this module to be applied with parameters                           !
!   SOCBEN_CAPRATES_ANN                                                 !
!   SOCBEN_LABIVT_ANN                                                   !
!=======================================================================!

FUNCTION SOCBEN_BRACRATE_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,3)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !apply the rate to the difference between income and lower limit (and add it up to the deductions computed in the previous brackets)
            OUTPUT_AMOUNT=OUTPUT_AMOUNT+(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        !otherwise the amount is higher than the upper bracket
        ELSE IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                                                                     
            !apply rate to the difference between upper and lower limit (and add it up to the amount computed with previous brackets) 
            OUTPUT_AMOUNT=OUTPUT_AMOUNT+(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)
            FUNCSELE = 1
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=OUTPUT_AMOUNT+(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)                                                       
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function PIT_BRACRATE_3COLUMNS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION SOCBEN_BRACRATE_3COLUMNS

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as a result of comparing an input amount with a lower and upper bracket!
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The nominal amount is in column 3 of this matrix.    !
!An example of this procedure is the Income MAF amount                  !
!In this module to be applied with parameters                           !
!   SOCBEN_INCMAF_ANN                                                   !
!=======================================================================!

FUNCTION SOCBEN_BRACAMOU_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,3)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            OUTPUT_AMOUNT=MAT_BRACRATE(i,3)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 
    !in the last bracket
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=MAT_BRACRATE(i,3)                                                       
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function PIT_BRACAMOU_3COLUMNS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION SOCBEN_BRACAMOU_3COLUMNS

!============= SUBROUTINE SOCBEN_INFLATE===============================!
!This subroutines inflates nominal values to an appropriate price level !
!=======================================================================!
SUBROUTINE SOCBEN_INFLATE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        !Exogenous variables
        !Exogenous variables read from the file SOCBEN_INPUT.txt
        !Exogenous variables read from the file SOCBEN_PIT.txt
!        PIT_FINTAX_ANN(pers_j)       = INFLATE_NOMVAL(1)*PIT_FINTAX_ANN(pers_j)                  !Final taxes due (sum of state taxes and crises surcharges)     
!        PIT_NTOI_ANN_SP1(pers_j)     = INFLATE_NOMVAL(1)*PIT_NTOI_ANN_SP1(pers_j)                !Net taxable occupational income of spouse 1 after applying marital splitting
!        PIT_NTOI_ANN_SP2(pers_j)     = INFLATE_NOMVAL(1)*PIT_NTOI_ANN_SP2(pers_j)                !Net taxable occupational income of spouse 2 after applying marital splitting
!        PIT_NTI_ANN_SP1(pers_j)      = INFLATE_NOMVAL(1)*PIT_NTI_ANN_SP1(pers_j)                 !Net taxable income of spouse 1
!        PIT_NTI_ANN_SP2(pers_j)      = INFLATE_NOMVAL(1)*PIT_NTI_ANN_SP2(pers_j)                 !Net taxable income of spouse 2
        !Exogenous variables read from the file SOCBEN_FAMAL.txt
        !Exogenous variables read from the file SOCBEN_CONTRIB.txt
!        CONTRIB_GTIPRIV_QT(pers_j)   = INFLATE_NOMVAL(1)*CONTRIB_GTIPRIV_QT(pers_j)              !Gross taxable labour income earned as wage earner on the private labour market
!        CONTRIB_GTIPUB_QT(pers_j)    = INFLATE_NOMVAL(1)*CONTRIB_GTIPUB_QT(pers_j)               !Gross taxable labour income earned as wage earner on the public labour market
!        CONTRIB_GTISELF_QT(pers_j)   = INFLATE_NOMVAL(1)*CONTRIB_GTISELF_QT(pers_j)              !Gross taxable labour income earned as self employed
!        CONTRIB_GTIPENS_QT(pers_j)   = INFLATE_NOMVAL(1)*CONTRIB_GTIPENS_QT(pers_j)              !Gross taxable pension benefits
!        CONTRIB_GTIEARNE_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_GTIEARNE_QT(pers_j)             !Gross taxable early retirement benefits of the new type
!        CONTRIB_GTIEAROL_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_GTIEAROL_QT(pers_j)             !Gross taxable early retirement benefits of the old type
!        CONTRIB_GTIOLDUN_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_GTIOLDUN_QT(pers_j)             !Gross taxable unemployment benefits of older unemployed with seniority supplement
!        CONTRIB_GTIOTHUN_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_GTIOTHUN_QT(pers_j)             !Gross taxable unemployment benefits, other than early retirement benefits
!        CONTRIB_GTIDISAB_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_GTIDISAB_QT(pers_j)             !Gross taxable disability benefits of wage earners
!        CONTRIB_GTINDOCC_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_GTINDOCC_QT(pers_j)             !Gross taxable benefits in case of industrial accidents or occupational diseases
!        CONTRIB_GTISICK_QT(pers_j)   = INFLATE_NOMVAL(1)*CONTRIB_GTISICK_QT(pers_j)              !Gross taxable benefits in case of sickness other than disability benefits
!        CONTRIB_GTIFAMAL_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_GTIFAMAL_QT(pers_j)             !Gross taxable income from family allowances
        !Exogenous variables read from the file SOCBEN_CONSTRUCT.txt
        CADASTRAL_GINCOME(pers_j)    = INFLATE_NOMVAL(3)*CADASTRAL_GINCOME(pers_j)               !Gross cadastral income (land property)   
        CADASTRAB_GINCOME(pers_j)    = INFLATE_NOMVAL(3)*CADASTRAB_GINCOME(pers_j)               !Gross cadastral income (building)   
        FINANCIAL_WEALTH(pers_j)     = INFLATE_NOMVAL(3)*FINANCIAL_WEALTH(pers_j)                !Financial wealth   
        PRIVATE_PENSIONS(pers_j)     = INFLATE_NOMVAL(3)*PRIVATE_PENSIONS(pers_j)                !Private pensions
        ALIMONY_RECEIVED(pers_j)     = INFLATE_NOMVAL(3)*ALIMONY_RECEIVED(pers_j)                !Alimony received
        BENEFITS_NATURE(pers_j)      = INFLATE_NOMVAL(3)*BENEFITS_NATURE(pers_j)                 !Benefits in nature
        !Define exogenous variables read from MIMOSIS_WEIGHT.txt
    END DO   

END SUBROUTINE SOCBEN_INFLATE

!=================================================!
!  Transforms quarterly data into annual values   !
!=================================================!

!====================================================!
! Subroutine main input : gross incomes (quarterly)  !
! Subroutine main output: gross incomes (yearly)     !
!====================================================!

SUBROUTINE SOCBEN_TRANSFORM_QUARTERLY_DATA

    DO pers_j=1,n_rec
        CONTRIB_GTIPRIV_ANN(pers_j) =4.D0*CONTRIB_GTIPRIV_QT(pers_j)
        CONTRIB_GTIPUB_ANN(pers_j)  =4.D0*CONTRIB_GTIPUB_QT(pers_j)
        CONTRIB_GTISELF_ANN(pers_j) =4.D0*CONTRIB_GTISELF_QT(pers_j)
        CONTRIB_GTIPENS_ANN(pers_j) =4.D0*CONTRIB_GTIPENS_QT(pers_j)
        CONTRIB_GTIEARNE_ANN(pers_j)=4.D0*CONTRIB_GTIEARNE_QT(pers_j)
        CONTRIB_GTIEAROL_ANN(pers_j)=4.D0*CONTRIB_GTIEAROL_QT(pers_j)
        CONTRIB_GTIOLDUN_ANN(pers_j)=4.D0*CONTRIB_GTIOLDUN_QT(pers_j)
        CONTRIB_GTIOTHUN_ANN(pers_j)=4.D0*CONTRIB_GTIOTHUN_QT(pers_j)
        CONTRIB_GTIDISAB_ANN(pers_j)=4.D0*CONTRIB_GTIDISAB_QT(pers_j)
        CONTRIB_GTINDOCC_ANN(pers_j)=4.D0*CONTRIB_GTINDOCC_QT(pers_j)
        CONTRIB_GTISICK_ANN(pers_j) =4.D0*CONTRIB_GTISICK_QT(pers_j)
        CONTRIB_GTIFAMAL_ANN(pers_j)=4.D0*CONTRIB_GTIFAMAL_QT(pers_j)
    END DO   

END SUBROUTINE SOCBEN_TRANSFORM_QUARTERLY_DATA

!===========================================================================!
!Construct some instruments to carry out the later social benefit tests     !
!   SOCBEN_IGOLEEFL_ANN                                                     !
!   SOCBEN_ENTLEIGO_ANN                                                     !
!   SOCBEN_DEPCHILD_ANN                                                     !
!   SOCBEN_CHILDNOB_ANN                                                     !
!   SOCBEN_PARTNID_ANN                                                      !
!   SOCBEN_HHDIINC_ANN                                                      !
!   SOCBEN_HHNOLAB_ANN                                                      !
!   SOCBEN_NMLEEFL_ANN                                                      !
!   SOCBEN_NMIGO_ANN                                                        !
!Handicap-creation                                                          !
!   SOCBEN_ENTIT_ANN                                                        !
!   SOCBEN_ENTIVT_ANN                                                       !
!   SOCBEN_ENTTHB_ANN                                                       !
!   SOCBEN_PARITIVT_ANN                                                     !
!   SOCBEN_PARTHB_ANN                                                       !
!   SOCBEN_MEDIVT_ANN                                                       !
!   SOCBEN_MEDIT_ANN                                                        !
!   SOCBEN_MEDTHB_ANN                                                       !
!   SOCBEN_FAMITIVT_ANN                                                     !
!   SOCBEN_FAMTHB_ANN                                                       !
!   SOCBEN_MEANSIVT_2001_ANN                                                !
!   SOCBEN_MEANSIVT_2004_ANN                                                !
!===========================================================================!

SUBROUTINE SOCBEN_INSTRUM_A(hh_i)

    INTEGER                         :: hh_i,i,j
    INTEGER                         :: pers_j,lijn_j,pers_k,TestStat,TestPart,SumZelfR
    REAL(KIND=KIND_REAL)            :: HHINC,SUMLAB   
    REAL(KIND=KIND_REAL)            :: LABINC_J,SHARELAB_J,PENSINC_J,SHAREPENS_J,TOTTAXPROFINC_J,OTHTAXINC_J,FINWEAL_J,REALEST_J
    REAL(KIND=KIND_REAL)            :: LABINC_K,SHARELAB_K,PENSINC_K,SHAREPENS_K,TOTTAXPROFINC_K,OTHTAXINC_K
    REAL(KIND=KIND_REAL)            :: SHAREINTOT_J,TOTFINW,TOTPIT    
    INTEGER                         :: NUM_CONTR,NUM_CS
    REAL(KIND=KIND_REAL)            :: V2,R1,R2,R3,REALEST_K
    INTEGER                         :: MAAND_IGO,JAAR_IGO,MAAND_PENS,JAAR_PENS

    !Create SOCBEN_IGOLEEFL_ANN                                                     
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        !Remarks:
        !Determine with the AGE value and the year of simulation whether the individual will open a leefloon or an IGO right
        !although the strict age condition for the leefloon only applies from October 1st 2002 onwards we apply it for all cases
        !In the leefloon rules one generally only applies a lower age limit and not an upper limit
        !In case the income would be insufficient after application of the IGO rules one can apply for a leefloon
        !We exclude this possibility here.
        IF (SIMYEAR.LT.2003) THEN
            IF (SOCBEN_YEAR_AGE(pers_j).GE.25.AND.SOCBEN_YEAR_AGE(pers_j).LT.62) THEN
                SOCBEN_IGOLEEFL_ANN(pers_j) = 1
            ELSE IF (SOCBEN_YEAR_AGE(pers_j).GE.62) THEN
                SOCBEN_IGOLEEFL_ANN(pers_j) = 2
            END IF
        ELSE IF (SIMYEAR.GE.2003.AND.SIMYEAR.LT.2006) THEN
            IF (SOCBEN_YEAR_AGE(pers_j).GE.25.AND.SOCBEN_YEAR_AGE(pers_j).LT.63) THEN
                SOCBEN_IGOLEEFL_ANN(pers_j) = 1
            ELSE IF (SOCBEN_YEAR_AGE(pers_j).GE.63) THEN
                SOCBEN_IGOLEEFL_ANN(pers_j) = 2
            END IF
        ELSE IF (SIMYEAR.GE.2006.AND.SIMYEAR.LT.2009) THEN
            IF (SOCBEN_YEAR_AGE(pers_j).GE.25.AND.SOCBEN_YEAR_AGE(pers_j).LT.64) THEN
                SOCBEN_IGOLEEFL_ANN(pers_j) = 1
            ELSE IF (SOCBEN_YEAR_AGE(pers_j).GE.64) THEN
                SOCBEN_IGOLEEFL_ANN(pers_j) = 2
            END IF
        ELSE
            IF (SOCBEN_YEAR_AGE(pers_j).GE.25.AND.SOCBEN_YEAR_AGE(pers_j).LT.65) THEN
                SOCBEN_IGOLEEFL_ANN(pers_j) = 1
            ELSE IF (SOCBEN_YEAR_AGE(pers_j).GE.65) THEN
                SOCBEN_IGOLEEFL_ANN(pers_j) = 2
            END IF
        END IF
    END DO        

    !Create SOCBEN_DEPCHILD_ANN 
    !Create SOCBEN_CHILDNOB_ANN : remark: currently insufficient information to indentify this variable
    !Create SOCBEN_PARTNID_ANN                                                      !
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        !Determine the line in the parameter SOCBEN_FAMREL_ANN to convert relationship codes
        DO i = 1,SIZE(SOCBEN_FAMREL_ANN,1)
            IF (FAMREL_RELATION(pers_j).EQ.SOCBEN_FAMREL_ANN(i,1)) THEN
                lijn_j = i
            END IF
        END DO
        DO pers_k=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
            IF (pers_j.NE.pers_k) THEN
                !the partner and the children in charge are determined on the level of the family
                IF (FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                    DO i = 1,SIZE(SOCBEN_FAMREL_ANN,2)
                        IF (FAMREL_RELATION(pers_k).EQ.SOCBEN_FAMREL_ANN(1,i)) THEN
                            !Count the number of dependent children (dependence is determined by the fact that the child generates child benefits
                            !guarantee that children are classified only depending on one of the two partners
                            IF (SOCBEN_FAMREL_ANN(lijn_j,i).EQ.2.AND.FAMAL_CHILD_ELIG(pers_k).EQ.1.AND.&
                            &SOCBEN_TEST_ANN(pers_k).EQ.0) THEN
                                SOCBEN_DEPCHILD_ANN(pers_j) = SOCBEN_DEPCHILD_ANN(pers_j) + 1
                                SOCBEN_TEST_ANN(pers_k) = 1
                            END IF
                            !If there is a partner, store an indicator of this in the variable PARTNER
                            IF (SOCBEN_FAMREL_ANN(lijn_j,i).EQ.1.AND.SOCBEN_TESTP_ANN(pers_j).EQ.0) THEN
                                SOCBEN_PARTNID_ANN(pers_j) = pers_k                               
                                SOCBEN_PARTNID_ANN(pers_k) = pers_j                               
                                SOCBEN_TESTP_ANN(pers_j) = 1
                                SOCBEN_TESTP_ANN(pers_k) = 1
                            END IF
                        END IF
                    END DO
                END IF
            END IF                    
        END DO
    END DO        

    !Create SOCBEN_HHDIINC_ANN                                                      !
    !Create SOCBEN_HHNOLAB_ANN                                                      !
    HHINC = 0.D0
    SUMLAB = 0.D0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        !Compute household income as the sum of the components over all individuals
        HHINC = HHINC + &
                &CONTRIB_GTIPRIV_ANN(pers_j)                 + &
                &CONTRIB_GTIPUB_ANN(pers_j)                  + &
                &CONTRIB_GTISELF_ANN(pers_j)                 + &
                &CONTRIB_GTIPENS_ANN(pers_j)                 + &
                &CONTRIB_GTIEARNE_ANN(pers_j)                + &
                &CONTRIB_GTIEAROL_ANN(pers_j)                + &
                &CONTRIB_GTIOLDUN_ANN(pers_j)                + &
                &CONTRIB_GTIOTHUN_ANN(pers_j)                + &
                &CONTRIB_GTIDISAB_ANN(pers_j)                + &
                &CONTRIB_GTINDOCC_ANN(pers_j)                + &
                &CONTRIB_GTISICK_ANN(pers_j)                 - & 
                &PIT_FINTAX_ANN(pers_j)                      + &
                &FINANCIAL_WEALTH(pers_j)                    + &
                &SOCBEN_CADL_RATE*CADASTRAL_GINCOME(pers_j)  + &
                &SOCBEN_CADB_RATE*CADASTRAB_GINCOME(pers_j)  + &
                &PRIVATE_PENSIONS(pers_j)                    + &
                &ALIMONY_RECEIVED(pers_j)                    + &
                &BENEFITS_NATURE(pers_j)
        !Compute the sum of labour incomes as the sum of the components over all individuals
        SUMLAB = SUMLAB + &
                &CONTRIB_GTIPRIV_ANN(pers_j)                 + &
                &CONTRIB_GTIPUB_ANN(pers_j)                  + &
                &CONTRIB_GTISELF_ANN(pers_j)
        !If last individual of the household then attribute the sums to all household members                
        IF (pers_j.EQ.SOCBEN_END_HH(hh_i)) THEN
            DO pers_k=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
                SOCBEN_HHDIINC_ANN(pers_k) = HHINC
                SOCBEN_HHNOLAB_ANN(pers_k) = SUMLAB
            END DO
        END IF
    END DO        
    
    !Create SOCBEN_ENTLEIGO_ANN: identifier that indicates whether individual is entitled to Leefloon or IGO: 
    !distinction is made between status based on coditions and observations 
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        !Store conditions based on administrative data in position 1 of matrix
        IF (InPODMI_SEL(pers_j).EQ.1) THEN
            SOCBEN_ENTLEIGO_ANN(pers_j,1) = 1
        END IF
        IF (IGO_2008(pers_j).GT.0.D0) THEN
            SOCBEN_ENTLEIGO_ANN(pers_j,1) = 2
        END IF
        !Store old conditions on position 2 of matrix
        !If both Leefloon and IGO right then IGO gets value
        !Remark that the old condtions contained a test to limit the take up (test used in absence of lacking administrative information on take up
        !Procedure is only started if disposable income of the sociological household is below the highest IGO-level one can receive 
        !and if the individual is not living in a collective household
        IF (SOCBEN_IGOLEEFL_ANN(pers_j).EQ.1.AND.SOCBEN_HHDIINC_ANN(pers_j).LT.SOCBEN_MINTYM65_ANN(1,2)&
        &.AND.NAREGNIS_RELATION(pers_j).NE.20) THEN
            SOCBEN_ENTLEIGO_ANN(pers_j,2) = 1
        END IF
        IF (SOCBEN_IGOLEEFL_ANN(pers_j).EQ.2.AND.SOCBEN_HHDIINC_ANN(pers_j).LT.SOCBEN_MINTYP65_ANN(1,2)&
        &.AND.NAREGNIS_RELATION(pers_j).NE.20) THEN
            SOCBEN_ENTLEIGO_ANN(pers_j,2) = 2
        END IF
    END DO

    !Switch test back to 0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        SOCBEN_TEST_ANN(pers_j) = 0
    END DO

    !Create SOCBEN_NMLEEFL_ANN
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        IF (SOCBEN_TEST_ANN(pers_j).EQ.0) THEN
            !Determine the income level necessary to execute the means test
            !Take into account income of both partners if they are married
            !Remark that we only take into account these amount although the CPAS can decide itself to
            !take into account resource of other household members as well
            !But as compared to the IGO the leefloon rules are much more restrictive in the sense that for the IGO
            !it is stated that resources of all household members should be taken into account
            IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
                SOCBEN_TEST_ANN(pers_j) = 1
                SOCBEN_TEST_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 1
                !Use instruments to determine net income from labour and other taxable sources
                !Compute total amount of taxes paid (or received in case of credit) by the couple 
                !Since taxes per couple are stored for only one of the two partners they have to be added
                TOTPIT = PIT_FINTAX_ANN(pers_j) + PIT_FINTAX_ANN(SOCBEN_PARTNID_ANN(pers_j))
                !compute amount for j
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                LABINC_J =          CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) 
                SHARELAB_J = 0.D0
                IF (TOTTAXPROFINC_J.NE.0.D0) THEN
                    SHARELAB_J = LABINC_J/TOTTAXPROFINC_J
                END IF                    
                !compute amount for partner
                TOTTAXPROFINC_K =   CONTRIB_GTIPRIV_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                   &CONTRIB_GTIPUB_ANN(SOCBEN_PARTNID_ANN(pers_j))  + &
                                   &CONTRIB_GTISELF_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                   &CONTRIB_GTIPENS_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                   &CONTRIB_GTIEARNE_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                   &CONTRIB_GTIEAROL_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                   &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                   &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                   &CONTRIB_GTIDISAB_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                   &CONTRIB_GTINDOCC_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                   &CONTRIB_GTISICK_ANN(SOCBEN_PARTNID_ANN(pers_j))  
                LABINC_K =          CONTRIB_GTIPRIV_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                   &CONTRIB_GTIPUB_ANN(SOCBEN_PARTNID_ANN(pers_j))  + &
                                   &CONTRIB_GTISELF_ANN(SOCBEN_PARTNID_ANN(pers_j)) 
                SHARELAB_K = 0.D0
                IF (TOTTAXPROFINC_K.NE.0.D0) THEN
                    SHARELAB_K = LABINC_K/TOTTAXPROFINC_K
                END IF                    
                SHAREINTOT_J = 0.D0
                IF ((TOTTAXPROFINC_J+TOTTAXPROFINC_K).NE.0.D0) THEN
                    SHAREINTOT_J = TOTTAXPROFINC_J/(TOTTAXPROFINC_J+TOTTAXPROFINC_K)
                END IF                    
                !Determine income other than labour income net of taxes
                OTHTAXINC_J = (TOTTAXPROFINC_J - LABINC_J) - (1.D0 - SHARELAB_J)*SHAREINTOT_J*TOTPIT
                IF (OTHTAXINC_J.LT.0.D0) THEN
                    OTHTAXINC_J = 0.D0
                END IF
                OTHTAXINC_K = (TOTTAXPROFINC_K - LABINC_K) - (1.D0 - SHARELAB_K)*(1.D0 - SHAREINTOT_J)*TOTPIT
                IF (OTHTAXINC_K.LT.0.D0) THEN
                    OTHTAXINC_K = 0.D0
                END IF
                !subtract the training program amount from individual labour income
                LABINC_J = LABINC_J - SHARELAB_J*SHAREINTOT_J*TOTPIT - SOCBEN_TRAIN_PROG_MONTH*12.D0
                IF (LABINC_J.LT.0.D0) THEN
                    LABINC_J = 0.D0
                END IF
                LABINC_K = LABINC_K - SHARELAB_K*(1.D0 - SHAREINTOT_J)*TOTPIT - &
                &SOCBEN_TRAIN_PROG_MONTH*12.D0
                IF (LABINC_K.LT.0.D0) THEN
                    LABINC_K = 0.D0
                END IF
                !Determine net resources from financial wealth
                TOTFINW = FINANCIAL_WEALTH(pers_j) + FINANCIAL_WEALTH(SOCBEN_PARTNID_ANN(pers_j))
                FINWEAL_J = SOCBEN_BRACRATE_3COLUMNS(TOTFINW,SIZE(SOCBEN_CAPRATES_ANN,1),&
                &SOCBEN_CAPRATES_ANN,pers_j)
                IF (FINWEAL_J.LT.0.D0) THEN
                    FINWEAL_J = 0.D0
                END IF
                !Determine net resources from real estate income 
                REALEST_J =  SOCBEN_CADL_RATE*(CADASTRAL_GINCOME(pers_j)+CADASTRAL_GINCOME(SOCBEN_PARTNID_ANN(pers_j))&
                            &-SOCBEN_CADL_BASICD_ANN) + &
                            &SOCBEN_CADB_RATE*(CADASTRAB_GINCOME(pers_j)+CADASTRAB_GINCOME(SOCBEN_PARTNID_ANN(pers_j))&
                            &-SOCBEN_CADB_BASICD_ANN-SOCBEN_CADB_SPOUSED_ANN-SOCBEN_CADB_CHILDD_ANN*&
                            &(SOCBEN_DEPCHILD_ANN(pers_j)+SOCBEN_DEPCHILD_ANN(SOCBEN_PARTNID_ANN(pers_j))))
                IF (REALEST_J.LT.0.D0) THEN
                    REALEST_J = 0.D0
                END IF
                !Determine total amount of income
                !Remark: means are attributed to both partners. Those that are currently tax included are divided
                !in proportion of their contribution to the taxable income. This division is necessary since
                !the tax amount for married couples is still determined jointly while for all others it is not
                !The other income amounts are divided equally over both partners after applying the deduction rules
                !on the joint amount of income
                SOCBEN_NMLEEFL_ANN(pers_j) =                        OTHTAXINC_J + LABINC_J + (FINWEAL_J+REALEST_J)/2.D0 + &
                                                                    &PRIVATE_PENSIONS(pers_j)+ALIMONY_RECEIVED(pers_j)+&
                                                                    &BENEFITS_NATURE(pers_j)
                SOCBEN_NMLEEFL_ANN(SOCBEN_PARTNID_ANN(pers_j)) =    OTHTAXINC_K + LABINC_K + (FINWEAL_J+REALEST_J)/2.D0 + &
                                                                    &PRIVATE_PENSIONS(SOCBEN_PARTNID_ANN(pers_j))+&
                                                                    &ALIMONY_RECEIVED(SOCBEN_PARTNID_ANN(pers_j))+&
                                                                    &BENEFITS_NATURE(SOCBEN_PARTNID_ANN(pers_j))
            ELSE
                SOCBEN_TEST_ANN(pers_j) = 1
                !Use instruments to determine net income from labour and other taxable sources
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                LABINC_J =          CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) 
                SHARELAB_J = 0.D0
                IF (TOTTAXPROFINC_J.NE.0.D0) THEN
                    SHARELAB_J = LABINC_J/TOTTAXPROFINC_J
                END IF                    
                !Determine income other than labour income net of taxes
                OTHTAXINC_J = (TOTTAXPROFINC_J - LABINC_J) - (1.D0 - SHARELAB_J)*PIT_FINTAX_ANN(pers_j)
                IF (OTHTAXINC_J.LT.0.D0) THEN
                    OTHTAXINC_J = 0.D0
                END IF
                !subtract the training program amount from individual labour income
                LABINC_J = LABINC_J - SHARELAB_J*PIT_FINTAX_ANN(pers_j) - SOCBEN_TRAIN_PROG_MONTH*12.D0
                IF (LABINC_J.LT.0.D0) THEN
                    LABINC_J = 0.D0
                END IF
                !Determine net resources from financial wealth
                FINWEAL_J = SOCBEN_BRACRATE_3COLUMNS(FINANCIAL_WEALTH(pers_j),SIZE(SOCBEN_CAPRATES_ANN,1),&
                &SOCBEN_CAPRATES_ANN,pers_j)
                IF (FINWEAL_J.LT.0.D0) THEN
                    FINWEAL_J = 0.D0
                END IF
                !Determine net resources from real estate income 
                REALEST_J =  SOCBEN_CADL_RATE*(CADASTRAL_GINCOME(pers_j)-SOCBEN_CADL_BASICD_ANN) + &
                            &SOCBEN_CADB_RATE*(CADASTRAB_GINCOME(pers_j) - &
                            &SOCBEN_CADB_BASICD_ANN                      - &
                            &SOCBEN_CADB_CHILDD_ANN*SOCBEN_DEPCHILD_ANN(pers_j))                  
                IF (REALEST_J.LT.0.D0) THEN
                    REALEST_J = 0.D0
                END IF
                !Determine total amount of income
                SOCBEN_NMLEEFL_ANN(pers_j) = OTHTAXINC_J+LABINC_J+FINWEAL_J+REALEST_J+&
                                            &PRIVATE_PENSIONS(pers_j)+ALIMONY_RECEIVED(pers_j)+BENEFITS_NATURE(pers_j)
            END IF
        END IF
    END DO

    !Switch test back to 0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        SOCBEN_TEST_ANN(pers_j) = 0
    END DO

    !Create SOCBEN_NMIGO_ANN
    !Step 1: application of the accounting rules for individual income components
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        IF (SOCBEN_TEST_ANN(pers_j).EQ.0) THEN
            !Determine the income level necessary to execute the means test
            !Disentangle the income of both partners if they are married to compute their net amount
            IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
                SOCBEN_TEST_ANN(pers_j) = 1
                SOCBEN_TEST_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 1
                !Use instruments to determine net income from labour and other taxable sources
                !Compute total amount of taxes paid (or received in case of credit) by the couple 
                !Since taxes per couple are stored for only one of the two partners they have to be added
                TOTPIT = PIT_FINTAX_ANN(pers_j) + PIT_FINTAX_ANN(SOCBEN_PARTNID_ANN(pers_j))
                !compute amount for j
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                LABINC_J =          CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) 
                PENSINC_J = CONTRIB_GTIPENS_ANN(pers_j)           
                SHARELAB_J = 0.D0
                SHAREPENS_J = 0.D0
                IF (TOTTAXPROFINC_J.NE.0.D0) THEN
                    SHARELAB_J = LABINC_J/TOTTAXPROFINC_J
                    SHAREPENS_J = PENSINC_J/TOTTAXPROFINC_J
                END IF                    
                !compute amount for partner
                TOTTAXPROFINC_K =    CONTRIB_GTIPRIV_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIPUB_ANN(SOCBEN_PARTNID_ANN(pers_j))  + &
                                    &CONTRIB_GTISELF_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIPENS_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIEARNE_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIEAROL_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIDISAB_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTINDOCC_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTISICK_ANN(SOCBEN_PARTNID_ANN(pers_j))  
                LABINC_K =           CONTRIB_GTIPRIV_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIPUB_ANN(SOCBEN_PARTNID_ANN(pers_j))  + &
                                    &CONTRIB_GTISELF_ANN(SOCBEN_PARTNID_ANN(pers_j)) 
                PENSINC_K = CONTRIB_GTIPENS_ANN(SOCBEN_PARTNID_ANN(pers_j))           
                SHARELAB_K = 0.D0
                SHAREPENS_K = 0.D0
                IF (TOTTAXPROFINC_K.NE.0.D0) THEN
                    SHARELAB_K = LABINC_K/TOTTAXPROFINC_K
                    SHAREPENS_K = PENSINC_K/TOTTAXPROFINC_K
                END IF
                SHAREINTOT_J = 0.D0
                IF ((TOTTAXPROFINC_J+TOTTAXPROFINC_K).NE.0.D0) THEN
                    SHAREINTOT_J = TOTTAXPROFINC_J/(TOTTAXPROFINC_J+TOTTAXPROFINC_K)
                END IF                    
                !Determine income other than labour income and pension income net of taxes
                OTHTAXINC_J = (TOTTAXPROFINC_J - LABINC_J - PENSINC_J) - (1.D0 - SHARELAB_J - SHAREPENS_J)*SHAREINTOT_J*&
                &TOTPIT
                IF (OTHTAXINC_J.LT.0.D0) THEN
                    OTHTAXINC_J = 0.D0
                END IF
                OTHTAXINC_K = (TOTTAXPROFINC_K - LABINC_K - PENSINC_K) - (1.D0 - SHARELAB_K - SHAREPENS_K)*(1.D0 - SHAREINTOT_J)*&
                &TOTPIT
                IF (OTHTAXINC_K.LT.0.D0) THEN
                    OTHTAXINC_K = 0.D0
                END IF
                !take % of the net labour income
                LABINC_J = (SOCBEN_LABIGO_RATE/100.D0)*(LABINC_J - SHARELAB_J*SHAREINTOT_J*TOTPIT)
                IF (LABINC_J.LT.0.D0) THEN
                    LABINC_J = 0.D0
                END IF
                LABINC_K = (SOCBEN_LABIGO_RATE/100.D0)*(LABINC_K - SHARELAB_K*(1.D0 - SHAREINTOT_J)*TOTPIT)
                IF (LABINC_K.LT.0.D0) THEN
                    LABINC_K = 0.D0
                END IF
                !take % of the net pension income
                PENSINC_J = (SOCBEN_PENSIGO_RATE/100.D0)*(PENSINC_J - SHAREPENS_J*SHAREINTOT_J*TOTPIT)
                IF (PENSINC_J.LT.0.D0) THEN
                    PENSINC_J = 0.D0
                END IF
                PENSINC_K = (SOCBEN_PENSIGO_RATE/100.D0)*(PENSINC_K - SHAREPENS_K*(1.D0 - SHAREINTOT_J)*TOTPIT)
                IF (PENSINC_K.LT.0.D0) THEN
                    PENSINC_K = 0.D0
                END IF
                !Determine net resources from financial wealth
                TOTFINW = FINANCIAL_WEALTH(pers_j) + FINANCIAL_WEALTH(SOCBEN_PARTNID_ANN(pers_j))
                FINWEAL_J = SOCBEN_BRACRATE_3COLUMNS(TOTFINW,SIZE(SOCBEN_CAPRAIGO_ANN,1),&
                &SOCBEN_CAPRAIGO_ANN,pers_j)
                IF (FINWEAL_J.LT.0.D0) THEN
                    FINWEAL_J = 0.D0
                END IF
                !Determine net resources from real estate income 
                REALEST_J =  SOCBEN_CADIGO_RATE*(CADASTRAL_GINCOME(pers_j)+CADASTRAL_GINCOME(SOCBEN_PARTNID_ANN(pers_j))&
                            &-SOCBEN_CADLIGO_BASICD_ANN) + &
                            &SOCBEN_CADIGO_RATE*(CADASTRAB_GINCOME(pers_j)+CADASTRAB_GINCOME(SOCBEN_PARTNID_ANN(pers_j))&
                            &-SOCBEN_CADBIGO_BASICD_ANN-SOCBEN_CADIGO_CHILDD_ANN*&
                            &(SOCBEN_DEPCHILD_ANN(pers_j)+SOCBEN_DEPCHILD_ANN(SOCBEN_PARTNID_ANN(pers_j))))
                IF (REALEST_J.LT.0.D0) THEN
                    REALEST_J = 0.D0
                END IF
                !Determine total amount of income per individual
                !Remark: means are attributed to both partners. Those that are currently tax included are divided
                !in proportion of their contribution to the taxable income. This division is necessary since
                !the tax amount for married couples is still determined jointly while for all others it is not
                !The other income amounts are divided equally over both partners after applying the deduction rules
                !on the joint amount of income
                SOCBEN_NMIGO_ANN(pers_j) =                      OTHTAXINC_J + LABINC_J + PENSINC_J + &
                                                                &(FINWEAL_J+REALEST_J)/2.D0 + &
                                                                &PRIVATE_PENSIONS(pers_j)+ALIMONY_RECEIVED(pers_j)+&
                                                                &BENEFITS_NATURE(pers_j)
                SOCBEN_NMIGO_ANN(SOCBEN_PARTNID_ANN(pers_j)) =  OTHTAXINC_K + LABINC_K + PENSINC_K + &
                                                                &(FINWEAL_J+REALEST_J)/2.D0 + &
                                                                &PRIVATE_PENSIONS(SOCBEN_PARTNID_ANN(pers_j))+&
                                                                &ALIMONY_RECEIVED(SOCBEN_PARTNID_ANN(pers_j))+&
                                                                &BENEFITS_NATURE(SOCBEN_PARTNID_ANN(pers_j))
            ELSE
                SOCBEN_TEST_ANN(pers_j) = 1
                !Use instruments to determine net income from labour and other taxable sources
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                LABINC_J =          CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) 
                PENSINC_J = CONTRIB_GTIPENS_ANN(pers_j)           
                SHARELAB_J = 0.D0
                SHAREPENS_J = 0.D0
                IF (TOTTAXPROFINC_J.NE.0.D0) THEN
                    SHARELAB_J = LABINC_J/TOTTAXPROFINC_J
                    SHAREPENS_J = PENSINC_J/TOTTAXPROFINC_J
                END IF                    
                !Determine income other than labour income net of taxes
                OTHTAXINC_J = (TOTTAXPROFINC_J - LABINC_J - PENSINC_J) - (1.D0 - SHARELAB_J - SHAREPENS_J)*PIT_FINTAX_ANN(pers_j)
                IF (OTHTAXINC_J.LT.0.D0) THEN
                    OTHTAXINC_J = 0.D0
                END IF
                !take % of the net labour income
                LABINC_J = (SOCBEN_LABIGO_RATE/100.D0)*(LABINC_J - SHARELAB_J*PIT_FINTAX_ANN(pers_j))
                IF (LABINC_J.LT.0.D0) THEN
                    LABINC_J = 0.D0
                END IF
                !take % of the net pension income
                PENSINC_J = (SOCBEN_PENSIGO_RATE/100.D0)*(PENSINC_J - SHAREPENS_J*PIT_FINTAX_ANN(pers_j))
                IF (PENSINC_J.LT.0.D0) THEN
                    PENSINC_J = 0.D0
                END IF
                !Determine net resources from financial wealth
                FINWEAL_J = SOCBEN_BRACRATE_3COLUMNS(FINANCIAL_WEALTH(pers_j),SIZE(SOCBEN_CAPRAIGO_ANN,1),&
                &SOCBEN_CAPRAIGO_ANN,pers_j)
                IF (FINWEAL_J.LT.0.D0) THEN
                    FINWEAL_J = 0.D0
                END IF
                !Determine net resources from real estate income 
                REALEST_J =  SOCBEN_CADIGO_RATE*(CADASTRAL_GINCOME(pers_j)-SOCBEN_CADLIGO_BASICD_ANN) + &
                            &SOCBEN_CADIGO_RATE*(CADASTRAB_GINCOME(pers_j)&
                            &-SOCBEN_CADBIGO_BASICD_ANN-SOCBEN_CADIGO_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(pers_j)))
                IF (REALEST_J.LT.0.D0) THEN
                    REALEST_J = 0.D0
                END IF
                !Determine total amount of income
                SOCBEN_NMIGO_ANN(pers_j) = OTHTAXINC_J + LABINC_J + PENSINC_J + FINWEAL_J + REALEST_J + &
                                           &PRIVATE_PENSIONS(pers_j)+ALIMONY_RECEIVED(pers_j)+BENEFITS_NATURE(pers_j)
            END IF
        END IF
    END DO

    !Create SOCBEN_NMIGO_ANN
    !Step 2: computation of the amount accounted per contributing individual in the household
    !Remark: with IGO one can apply the following division rule since there is only the distinction
    !single with others
    HHINC = 0.D0
    NUM_CONTR = 0
!    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
!        !Compute household income as the sum of all net amounts that deviate from 0
!        IF (SOCBEN_NMIGO_ANN(pers_j).NE.0.D0) THEN
!            HHINC       = HHINC + SOCBEN_NMIGO_ANN(pers_j)
!            NUM_CONTR   = NUM_CONTR + 1
!        END IF
!        !If last individual of the household then attribute the sum divided by the number of contributors to all household members                
!        IF (pers_j.EQ.SOCBEN_END_HH(hh_i)) THEN
!            IF (NUM_CONTR.NE.0) THEN
!                DO pers_k=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
!                    SOCBEN_NMIGO_ANN(pers_k) = HHINC/NUM_CONTR
!                END DO
!            END IF                    
!        END IF
!    END DO        

    !Handicap-creation
    !SOCBEN_ENTIT_ANN                       !Entitled to IT benefit 
    !SOCBEN_ENTIVT_ANN                      !Entitled to IVT benefit 
    !SOCBEN_ENTTHB_ANN                      !Entitled to THB benefit 
    !SOCBEN_IT_NMONTHS(pers_j)              !Number of months during wich IT is paid  
    !SOCBEN_IVT_NMONTHS(pers_j)             !Number of months during wich IVT is paid  
    !SOCBEN_THB_NMONTHS(pers_j)             !Number of months during wich THB is paid
!Oorspronkelijke regel
!    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
!        !Voorlopig enkel gewerkt met de laatste maand
!        READ (IT_Type_PC_2008(pers_j)(12:), '(I10)') TestStat
!        IF (ANY(TestStat.EQ.(/1, 2, 3/))) THEN
!            SOCBEN_ENTITIVT_ANN(pers_j) = TestStat
!        ELSE IF (TestStat.EQ.4) THEN
!            SOCBEN_ENTTHB_ANN(pers_j) = 1
!        END IF
!    END DO
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        DO j=1,12
            !Voorlopig enkel gewerkt met de laatste maand
            READ (IT_Type_PC_2008(pers_j)(j:j), '(I10)') TestStat
            IF (ANY(TestStat.EQ.(/1, 3/))) THEN
                SOCBEN_ENTIT_ANN(pers_j) = 1
                SOCBEN_IT_NMONTHS(pers_j) = SOCBEN_IT_NMONTHS(pers_j) + 1
            END IF
            IF (ANY(TestStat.EQ.(/2, 3/))) THEN
                SOCBEN_ENTIVT_ANN(pers_j) = 1
                SOCBEN_IVT_NMONTHS(pers_j) = SOCBEN_IVT_NMONTHS(pers_j) + 1
            END IF
            IF (TestStat.EQ.4) THEN
                SOCBEN_ENTTHB_ANN(pers_j) = 1
                SOCBEN_THB_NMONTHS(pers_j) = SOCBEN_THB_NMONTHS(pers_j) + 1
            END IF
        END DO
    END DO

    !Switch test back to 0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        SOCBEN_TESTP_ANN(pers_j) = 0
    END DO

    !SOCBEN_PARITIVT_ANN                  !Partner for application of IT/IVT rules
    !SOCBEN_PARTHB_ANN                    !Partner for application of THB rules
    !Appoint for each individual its possible partner (this is different then what was applied for leefloon and IGO)
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        !Read partner id if observed
        IF (gecodeerd_insz_partner(pers_j).NE.'niet in stkprf') THEN
            READ (gecodeerd_insz_partner(pers_j)(8:), '(I10)') TestPart
        END IF
        !Determine the line in the parameter SOCBEN_FAMREL_ANN to convert relationship codes
        DO i = 1,SIZE(SOCBEN_FAMREL_ANN,1)
            IF (FAMREL_RELATION(pers_j).EQ.SOCBEN_FAMREL_ANN(i,1)) THEN
                lijn_j = i
            END IF
        END DO
        !look for a partner as the individual identified with observed data
        !but select only if the individual is living in the same household
        DO pers_k=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
            IF (pers_j.NE.pers_k) THEN
                IF (GECODEERD_INSZ_INPUT(pers_k).EQ.TestPart.AND.SOCBEN_TESTP_ANN(pers_j).EQ.0) THEN
                    SOCBEN_PARITIVT_ANN(pers_j) = pers_k
                    SOCBEN_PARTHB_ANN(pers_j) = pers_k
                    SOCBEN_TESTP_ANN(pers_j) = 1
                END IF
            END IF
        END DO
        !If no partner is found in the first step, then check whether there is a partner in the family
        !look for a partner as the married or unmarried member of the family with which the individual cohabits
        IF (SOCBEN_TESTP_ANN(pers_j).EQ.0) THEN
            DO pers_k=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
                IF (pers_j.NE.pers_k) THEN
                    IF (FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                        DO i = 1,SIZE(SOCBEN_FAMREL_ANN,2)
                            IF (FAMREL_RELATION(pers_k).EQ.SOCBEN_FAMREL_ANN(1,i)) THEN
                                IF (SOCBEN_FAMREL_ANN(lijn_j,i).EQ.1.AND.SOCBEN_TESTP_ANN(pers_j).EQ.0) THEN
                                    SOCBEN_PARITIVT_ANN(pers_j) = pers_k
                                    SOCBEN_PARTHB_ANN(pers_j) = pers_k
                                    SOCBEN_TESTP_ANN(pers_j) = 1
                                END IF
                            END IF
                        END DO
                    END IF
                END IF                    
            END DO
        END IF
        !If no partner is found in the first two steps, then check whether there is a member of the smaller family
        !that is not a relative of the first, second or third degree
        !the first individual of this type is taken as partner
        IF (SOCBEN_TESTP_ANN(pers_j).EQ.0) THEN
            DO pers_k=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
                IF (pers_j.NE.pers_k) THEN
                    IF (FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                        DO i = 1,SIZE(SOCBEN_FAMREL_ANN,2)
                            IF (FAMREL_RELATION(pers_k).EQ.SOCBEN_FAMREL_ANN(1,i)) THEN
                                IF (SOCBEN_FAMREL_ANN(lijn_j,i).EQ.MAXVAL(SOCBEN_FAMREL_ANN(lijn_j,:),2)) THEN
                                    SOCBEN_PARITIVT_ANN(pers_j) = pers_k
                                    SOCBEN_PARTHB_ANN(pers_j) = pers_k
                                    SOCBEN_TESTP_ANN(pers_j) = 1
                                END IF
                            END IF
                        END DO
                    END IF
                END IF                    
            END DO
        END IF
    END DO        

    !SOCBEN_MEDIVT_ANN                    !Medical categorie for IVT benefit (reductie in verdienvermogen)
    !SOCBEN_MEDIT_ANN                     !Medical categorie for IT benefit (mate van zelfredzaamheid) 
    !SOCBEN_MEDTHB_ANN                    !Medical categorie for THB benefit (mate van zelfredzaamheid)  
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        SumZelfR = critere_un(pers_j) + critere_deux(pers_j) + critere_trois(pers_j) + &
        &critere_quatre(pers_j) + critere_cinq(pers_j) + critere_six(pers_j)   
        !Medical categorie IVT
        IF (SOCBEN_ENTIVT_ANN(pers_j).EQ.1) THEN
            SOCBEN_MEDIVT_ANN(pers_j) = 0.D0
        END IF
        !Medical categorie IT
        IF (SOCBEN_ENTIT_ANN(pers_j).EQ.1) THEN
            IF (SumZelfR.LT.9) THEN
                SOCBEN_MEDIT_ANN(pers_j) = 1
            ELSE IF (SumZelfR.GE.9.AND.SumZelfR.LT.12) THEN
                SOCBEN_MEDIT_ANN(pers_j) = 2
            ELSE IF (SumZelfR.GE.12.AND.SumZelfR.LT.15) THEN
                SOCBEN_MEDIT_ANN(pers_j) = 3
            ELSE IF (SumZelfR.GE.15.AND.SumZelfR.LT.17) THEN
                SOCBEN_MEDIT_ANN(pers_j) = 4
            ELSE IF (SumZelfR.GE.17) THEN
                SOCBEN_MEDIT_ANN(pers_j) = 5
            END IF
        END IF
        !Medical categorie THB
        IF (SOCBEN_ENTTHB_ANN(pers_j).EQ.1) THEN
            IF (SumZelfR.LT.9) THEN
                SOCBEN_MEDTHB_ANN(pers_j) = 1
            ELSE IF (SumZelfR.GE.9.AND.SumZelfR.LT.12) THEN
                SOCBEN_MEDTHB_ANN(pers_j) = 2
            ELSE IF (SumZelfR.GE.12.AND.SumZelfR.LT.15) THEN
                SOCBEN_MEDTHB_ANN(pers_j) = 3
            ELSE IF (SumZelfR.GE.15.AND.SumZelfR.LT.17) THEN
                SOCBEN_MEDTHB_ANN(pers_j) = 4
            ELSE IF (SumZelfR.GE.17) THEN
                SOCBEN_MEDTHB_ANN(pers_j) = 5
            END IF
        END IF
    END DO        

    !SOCBEN_FAMITIVT_ANN                  !Family type for determination of IT/IVT
    !SOCBEN_FAMITIVT_ANN = 1 a person with a disability not belonging to class to 2 or 3 
    !SOCBEN_FAMITIVT_ANN = 2 a single person with a disability 
    !SOCBEN_FAMITIVT_ANN = 3 a person with a disability that cohabits with a partner or who has children in charge 
    !SOCBEN_FAMTHB_ANN                    ! Family type for determination of THB
    !SOCBEN_FAMTHB_ANN = 1 a person with a disability not belonging to class to 2 or 3 
    !SOCBEN_FAMTHB_ANN = 2 a single person with a disability 
    !SOCBEN_FAMTHB_ANN = 3 a person with a disability that cohabits with a partner or who has children in charge 
    !Loop over all members of the sociological household to determine the family situation for IT/IVT
    !is independent of the means of the individual
    !The maximum number of people with the C-status (=value 3) can be 1 per sociological household
    !if there is more than 1 individual then all get the B-status
    NUM_CS = 0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                      
        !Check whether pers_j is entitled to a IT/IVT benefit
        IF (SOCBEN_ENTIT_ANN(pers_j).EQ.1.OR.SOCBEN_ENTIVT_ANN(pers_j).EQ.1) THEN
            IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.0) THEN
                !Either the individual lives alone
                IF (SOCBEN_HH_SIZE(hh_i).EQ.1) THEN
                    SOCBEN_FAMITIVT_ANN(pers_j) = 2
                !or cohabits with other people
                ELSE
                    !Either the individual cohabits with a parnter or has children in charge
                    IF (SOCBEN_PARITIVT_ANN(pers_j).GT.0.OR.SOCBEN_DEPCHILD_ANN(pers_j).GT.0.OR.&
                    &SOCBEN_CHILDNOB_ANN(pers_j).GT.0) THEN
                        SOCBEN_FAMITIVT_ANN(pers_j) = 3
                        NUM_CS = NUM_CS + 1
                    !In all other cases the individual is cohabitating with other people
                    ELSE
                        SOCBEN_FAMITIVT_ANN(pers_j) = 1
                    END IF
                END IF
            END IF 
        END IF
        !Check whether pers_j is entitled to a THB benefit
        IF (SOCBEN_ENTTHB_ANN(pers_j).EQ.1) THEN
            IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.0) THEN
                !Either the individual lives alone
                IF (SOCBEN_HH_SIZE(hh_i).EQ.1) THEN
                    SOCBEN_FAMTHB_ANN(pers_j) = 2
                !or cohabits with other people
                ELSE
                    !Either the individual cohabits with a parnter or has children in charge
                    IF (SOCBEN_PARTHB_ANN(pers_j).GT.0.OR.SOCBEN_DEPCHILD_ANN(pers_j).GT.0.OR.&
                    &SOCBEN_CHILDNOB_ANN(pers_j).GT.0) THEN
                        SOCBEN_FAMTHB_ANN(pers_j) = 3
                    !In all other cases the individual is cohabitating with other people
                    ELSE
                        SOCBEN_FAMTHB_ANN(pers_j) = 1
                    END IF
                END IF
            END IF 
        END IF
    END DO
    
    !Restart loop to switch back the people with the C-status if there is more than 1 individual with C-status in the household
    IF (NUM_CS.GT.1) THEN
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                      
            IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.3) THEN
                SOCBEN_FAMITIVT_ANN(pers_j) = 2
            END IF
        END DO    
    END IF

    !SOCBEN_MEANSIVT_2001_ANN                  !Individual means needed when executing the means test for the IVT benefit
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        !compute amount for j
        TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                           &CONTRIB_GTIPUB_ANN(pers_j)  + &
                           &CONTRIB_GTISELF_ANN(pers_j) + &
                           &CONTRIB_GTIPENS_ANN(pers_j) + &
                           &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                           &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                           &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                           &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                           &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                           &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                           &CONTRIB_GTISICK_ANN(pers_j)  
        !Compute the amount of real estate income of individual j
        REALEST_J =  ((CADASTRAL_GINCOME(pers_j) + CADASTRAB_GINCOME(pers_j)) - SOCBEN_CADIVT_BASICD_ANN - &
        &SOCBEN_CADIVT_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(pers_j)))
        IF (REALEST_J.LT.0.D0) THEN
            REALEST_J = 0.D0
        END IF
        !Check whether there is a partner or not
        R1 = 0.D0
        TOTTAXPROFINC_K = 0.D0
        REALEST_K = 0.D0
        IF (SOCBEN_PARITIVT_ANN(pers_j).NE.0) THEN
            !Compute the total amount of taxable income of individual k
            TOTTAXPROFINC_K =    CONTRIB_GTIPRIV_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                                &CONTRIB_GTIPUB_ANN(SOCBEN_PARITIVT_ANN(pers_j))  + &
                                &CONTRIB_GTISELF_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                                &CONTRIB_GTIPENS_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                                &CONTRIB_GTIEARNE_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTIEAROL_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTIDISAB_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTINDOCC_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTISICK_ANN(SOCBEN_PARITIVT_ANN(pers_j))  
            !Compute the amount of real estate income of individual k
            REALEST_K =  ((CADASTRAL_GINCOME(SOCBEN_PARITIVT_ANN(pers_j)) + CADASTRAB_GINCOME(SOCBEN_PARITIVT_ANN(pers_j))) &
            &- SOCBEN_CADIVT_BASICD_ANN - SOCBEN_CADIVT_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(SOCBEN_PARITIVT_ANN(pers_j))))
            IF (REALEST_K.LT.0.D0) THEN
                REALEST_K = 0.D0
            END IF
            R1 = TOTTAXPROFINC_K + REALEST_K - SOCBEN_PARTIVT_ANN
            IF (R1.LT.0.D0) THEN
                R1 = 0.D0
            END IF
        END IF
        !Compute means to be subtracted later
        IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.1) THEN
            SOCBEN_MEANSIVT_2001_ANN(pers_j) = (R1 + TOTTAXPROFINC_J + REALEST_J) - SOCBEN_DISREIVT_ANN(1,2)
        ELSE IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.2) THEN
            SOCBEN_MEANSIVT_2001_ANN(pers_j) = (R1 + TOTTAXPROFINC_J + REALEST_J) - SOCBEN_DISREIVT_ANN(2,2)
        ELSE IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.3) THEN
            SOCBEN_MEANSIVT_2001_ANN(pers_j) = (R1 + TOTTAXPROFINC_J + REALEST_J) - SOCBEN_DISREIVT_ANN(3,2)
        END IF        
        IF (SOCBEN_MEANSIVT_2001_ANN(pers_j).LT.0.D0) THEN
            SOCBEN_MEANSIVT_2001_ANN(pers_j) = 0.D0
        END IF
    END DO    
    
    !SOCBEN_MEANSIVT_2004_ANN                  !Individual means needed when executing the means test for the IVT benefit
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        !compute amount for j
        TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                           &CONTRIB_GTIPUB_ANN(pers_j)  + &
                           &CONTRIB_GTISELF_ANN(pers_j) + &
                           &CONTRIB_GTIPENS_ANN(pers_j) + &
                           &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                           &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                           &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                           &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                           &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                           &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                           &CONTRIB_GTISICK_ANN(pers_j)  
        LABINC_J =          CONTRIB_GTIPRIV_ANN(pers_j) + &
                           &CONTRIB_GTIPUB_ANN(pers_j)  + &
                           &CONTRIB_GTISELF_ANN(pers_j) 
        !Compute amount of labour income of the disabled person that is exempted
        V2 = SOCBEN_BRACRATE_3COLUMNS(LABINC_J,SIZE(SOCBEN_LABIVT_ANN,1),&
        &SOCBEN_LABIVT_ANN,pers_j)
        R2 = 0.D0
        IF (LABINC_J.GT.V2) THEN
            R2 = LABINC_J - V2
        END IF
        !Compute the amount of real estate income of individual j
        REALEST_J =  ((CADASTRAL_GINCOME(pers_j) + CADASTRAB_GINCOME(pers_j)) - SOCBEN_CADIVT_BASICD_ANN - &
        &SOCBEN_CADIVT_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(pers_j)))
        IF (REALEST_J.LT.0.D0) THEN
            REALEST_J = 0.D0
        END IF
        !Compute the amount of other income sources of the disabled person that are not exempted
        R3 = (TOTTAXPROFINC_J - LABINC_J) + REALEST_J - SOCBEN_OTHIVT_ANN
        IF (R3.LT.0.D0) THEN
            R3 = 0.D0
        END IF
        !Check whether there is a partner or not
        R1 = 0.D0
        TOTTAXPROFINC_K = 0.D0
        REALEST_K = 0.D0
        IF (SOCBEN_PARITIVT_ANN(pers_j).NE.0) THEN
            !Compute the total amount of taxable income of individual k
            TOTTAXPROFINC_K =    CONTRIB_GTIPRIV_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                                &CONTRIB_GTIPUB_ANN(SOCBEN_PARITIVT_ANN(pers_j))  + &
                                &CONTRIB_GTISELF_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                                &CONTRIB_GTIPENS_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                                &CONTRIB_GTIEARNE_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTIEAROL_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTIDISAB_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTINDOCC_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                                &CONTRIB_GTISICK_ANN(SOCBEN_PARITIVT_ANN(pers_j))  
            !Compute the amount of real estate income of individual k
            REALEST_K =  ((CADASTRAL_GINCOME(SOCBEN_PARITIVT_ANN(pers_j)) + CADASTRAB_GINCOME(SOCBEN_PARITIVT_ANN(pers_j))) &
            &- SOCBEN_CADIVT_BASICD_ANN - SOCBEN_CADIVT_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(SOCBEN_PARITIVT_ANN(pers_j))))
            IF (REALEST_K.LT.0.D0) THEN
                REALEST_K = 0.D0
            END IF
            R1 = TOTTAXPROFINC_K + REALEST_K - SOCBEN_PARTIVT_ANN
            IF (R1.LT.0.D0) THEN
                R1 = 0.D0
            END IF
            !Compute means to be subtracted later
            SOCBEN_MEANSIVT_2004_ANN(pers_j) = R1 + R2 + R3
        ELSE 
            !Compute means to be subtracted later
            SOCBEN_MEANSIVT_2004_ANN(pers_j) = R2 + R3
        END IF
    END DO    
    
    !Limit number of months leefloon is paid per year to a maximum of 12
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        !If individual is entitled to leefloon then
        IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.1.OR.SOCBEN_ENTLEIGO_ANN(pers_j,2).EQ.1) THEN
            !If too much months limit them
            IF (NumMonth_Hulp_2008(pers_j).GT.12) THEN
                NumMonth_Hulp_2008(pers_j) = 12
            END IF
            !if no payment months are registered, assume at least one month the benefit is paid
            IF (NumMonth_Hulp_2008(pers_j).EQ.0) THEN
                NumMonth_Hulp_2008(pers_j) = 1
            END IF            
        END IF
    END DO
    
    !Compute number of months during which IGO is paid
    !and identify those who receive GWB instead of IGO
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        !If individual is entitled to IGO then
        IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.2.OR.SOCBEN_ENTLEIGO_ANN(pers_j,2).EQ.2) THEN
            !Number of months IGO
            SOCBEN_PAR_BREAK_POINT(1)=INDEX(IGO_PDATE_2008(pers_j),'-')
            READ (IGO_PDATE_2008(pers_j)(1:SOCBEN_PAR_BREAK_POINT(1)-1),  '(I10)') MAAND_IGO
            READ (IGO_PDATE_2008(pers_j)(SOCBEN_PAR_BREAK_POINT(1)+1:),   '(I10)') JAAR_IGO  
            IF (JAAR_IGO.LT.2008) THEN
                SOCBEN_IGO_NMONTHS(pers_j) = 12
            ELSE IF (JAAR_IGO.EQ.2008) THEN                 
                SOCBEN_IGO_NMONTHS(pers_j) = 12 + 1 - MAAND_IGO
            END IF
            !Correct impossible values
            IF (SOCBEN_IGO_NMONTHS(pers_j).LE.0) THEN
                SOCBEN_IGO_NMONTHS(pers_j) = 0 
            END IF
            IF (SOCBEN_IGO_NMONTHS(pers_j).GT.12) THEN
                SOCBEN_IGO_NMONTHS(pers_j) = 12 
            END IF
            !GWB entitlement: based on year of retirement, which can precede last IGO computation date
!            SOCBEN_PAR_BREAK_POINT(1)=INDEX(begindatum_pensioen_c_2008(pers_j),'-')
!            READ (begindatum_pensioen_c_2008(pers_j)(1:SOCBEN_PAR_BREAK_POINT(1)-1),  '(I10)') MAAND_PENS
!            READ (begindatum_pensioen_c_2008(pers_j)(SOCBEN_PAR_BREAK_POINT(1)+1:),   '(I10)') JAAR_PENS 
            IF ((JAAR_IGO.EQ.2001.AND.MAAND_IGO.LT.6).OR.JAAR_IGO.LT.2001) THEN
                IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.2) THEN
                    SOCBEN_ENTLEIGO_ANN(pers_j,1) = 3
                END IF                    
                IF (SOCBEN_ENTLEIGO_ANN(pers_j,2).EQ.2) THEN
                    SOCBEN_ENTLEIGO_ANN(pers_j,2) = 3
                END IF                    
            END IF
        END IF
    END DO
    
END SUBROUTINE SOCBEN_INSTRUM_A

!===========================================================================!
!Construct some instruments to carry out the later social benefit tests     !
!Handicap-creation                                                          !
!   SOCBEN_MEANSIT_2001_ANN                                                 !
!   SOCBEN_MEANSIT_2004_ANN                                                 !
!   SOCBEN_MEANSTHB_ANN                                                     !
!===========================================================================!

SUBROUTINE SOCBEN_INSTRUM_B(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j
    REAL(KIND=KIND_REAL)            :: LABINC_J,SHARELAB_J,PENSINC_J,SHAREPENS_J,TOTTAXPROFINC_J,OTHTAXINC_J,FINWEAL_J,REALEST_J
    REAL(KIND=KIND_REAL)            :: LABINC_K,SHARELAB_K,PENSINC_K,SHAREPENS_K,TOTTAXPROFINC_K,OTHTAXINC_K,TOTINC_K
    REAL(KIND=KIND_REAL)            :: SHAREINTOT_J,TOTFINW,TOTPIT    
    REAL(KIND=KIND_REAL)            :: V2,V3,V4,R1A,R1B,R2,R3,R4,REALEST_K,REPINC_J
    REAL(KIND=KIND_REAL)            :: MEANSA,MEANSB    

    !SOCBEN_MEANSIT_2001_ANN                   !Individual means needed when executing the means test for the IT benefit
    !Remark: for computation of IT benefit the IGO is considered as a replacement income
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        !compute amount for j
        TOTTAXPROFINC_J =    CONTRIB_GTIPRIV_ANN(pers_j) + &
                            &CONTRIB_GTIPUB_ANN(pers_j)  + &
                            &CONTRIB_GTISELF_ANN(pers_j) + &
                            &CONTRIB_GTIPENS_ANN(pers_j) + &
                            &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                            &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                            &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                            &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                            &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                            &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                            &CONTRIB_GTISICK_ANN(pers_j) 
        LABINC_J =           CONTRIB_GTIPRIV_ANN(pers_j) + &
                            &CONTRIB_GTIPUB_ANN(pers_j)  + &
                            &CONTRIB_GTISELF_ANN(pers_j) 
        REPINC_J = (TOTTAXPROFINC_J - LABINC_J) + SOCBEN_IGO_ANN(pers_j)                  
        !Compute amount of labour income of the disabled person that is exempted
        V2 = SOCBEN_BRACRATE_3COLUMNS(LABINC_J,SIZE(SOCBEN_LABIT_ANN,1),&
        &SOCBEN_LABIT_ANN,pers_j)
        R2 = 0.D0
        IF (LABINC_J.GT.V2) THEN
            R2 = LABINC_J - V2
        END IF
        !Compute the amount of replacement income of the disabled person that is not exempted
        !if system 2 is applied
        !Remark: we treat labour income and replacement income separatly for the computation of
        !the exemptions
        R3 = 0.D0
        IF (REPINC_J.GT.SOCBEN_REPIT_ANN(1)) THEN
            R3 = REPINC_J - SOCBEN_REPIT_ANN(1)
        END IF
        IF (R3.GT.SOCBEN_REPIT_ANN(2)) THEN
            R3 = SOCBEN_REPIT_ANN(2)
        END IF
        !Compute the amount of real estate income of individual j
        REALEST_J =  ((CADASTRAL_GINCOME(pers_j) + CADASTRAB_GINCOME(pers_j)) - SOCBEN_CADIVT_BASICD_ANN - &
        &SOCBEN_CADIVT_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(pers_j)))
        IF (REALEST_J.LT.0.D0) THEN
            REALEST_J = 0.D0
        END IF
        !Check whether there is a partner or not
        R1 = 0.D0
        IF (SOCBEN_PARITIVT_ANN(pers_j).NE.0) THEN
            !Compute the total amount of taxable income of individual k
            TOTTAXPROFINC_K =   CONTRIB_GTIPRIV_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                               &CONTRIB_GTIPUB_ANN(SOCBEN_PARITIVT_ANN(pers_j))  + &
                               &CONTRIB_GTISELF_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                               &CONTRIB_GTIPENS_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                               &CONTRIB_GTIEARNE_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTIEAROL_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTIDISAB_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTINDOCC_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTISICK_ANN(SOCBEN_PARITIVT_ANN(pers_j)) 
            !Compute the amount of real estate income of individual k
            REALEST_K =  ((CADASTRAL_GINCOME(SOCBEN_PARITIVT_ANN(pers_j)) + CADASTRAB_GINCOME(SOCBEN_PARITIVT_ANN(pers_j))) &
            &- SOCBEN_CADIVT_BASICD_ANN - SOCBEN_CADIVT_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(SOCBEN_PARITIVT_ANN(pers_j))))
            IF (REALEST_K.LT.0.D0) THEN
                REALEST_K = 0.D0
            END IF
            TOTINC_K = TOTTAXPROFINC_K + REALEST_K + SOCBEN_IGO_ANN(SOCBEN_PARITIVT_ANN(pers_j))
            !Compute exempted amount of the income of the partner
            IF (ANY(SOCBEN_MEDIT_ANN(pers_j).EQ.(/1, 2/))) THEN
                R1 = TOTINC_K - SOCBEN_BRACRATE_3COLUMNS(TOTINC_K,SIZE(SOCBEN_PARTITA_ANN,1),&
                &SOCBEN_PARTITA_ANN,pers_j)
            ELSE IF (ANY(SOCBEN_MEDIT_ANN(pers_j).EQ.(/3, 4, 5/))) THEN
                !Remark: before 2004 one probably only had 4 classes for MEDIT
                !Has to be checked
                R1 = TOTINC_K - SOCBEN_BRACRATE_3COLUMNS(TOTINC_K,SIZE(SOCBEN_PARTITB_ANN,1),&
                &SOCBEN_PARTITB_ANN,pers_j)
            END IF
            IF (R1.LT.0.D0) THEN
                R1 = 0.D0
            END IF
        END IF
        !Compute means according to system A: sum minus general disregard
        MEANSA = 0.D0
        IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.1) THEN
            MEANSA = (LABINC_J + REPINC_J + REALEST_J + R1) - SOCBEN_DISREIT_ANN(1,2)
        ELSE IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.2) THEN
            MEANSA = (LABINC_J + REPINC_J + REALEST_J + R1) - SOCBEN_DISREIT_ANN(2,2)
        ELSE IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.3) THEN
            MEANSA = (LABINC_J + REPINC_J + REALEST_J + R1) - SOCBEN_DISREIT_ANN(3,2)
        END IF        
        IF (MEANSA.LT.0.D0) THEN
            MEANSA = 0.D0
        END IF
        MEANSB = R2 + R3 + REALEST_J + R1 
        IF (MEANSA.GT.MEANSB) THEN
            SOCBEN_MEANSIT_2001_ANN(pers_j) = MEANSB
        ELSE
            SOCBEN_MEANSIT_2001_ANN(pers_j) = MEANSA
        END IF
    END DO    
    
    !SOCBEN_MEANSIT_2004_ANN                   !Individual means needed when executing the means test for the IT benefit
    !Remark: for computation of IT benefit the IGO is considered as a replacement income
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        !compute amount for j
        TOTTAXPROFINC_J =    CONTRIB_GTIPRIV_ANN(pers_j) + &
                            &CONTRIB_GTIPUB_ANN(pers_j)  + &
                            &CONTRIB_GTISELF_ANN(pers_j) + &
                            &CONTRIB_GTIPENS_ANN(pers_j) + &
                            &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                            &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                            &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                            &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                            &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                            &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                            &CONTRIB_GTISICK_ANN(pers_j) 
        LABINC_J =           CONTRIB_GTIPRIV_ANN(pers_j) + &
                            &CONTRIB_GTIPUB_ANN(pers_j)  + &
                            &CONTRIB_GTISELF_ANN(pers_j) 
        REPINC_J = (TOTTAXPROFINC_J - LABINC_J) + SOCBEN_IGO_ANN(pers_j)                  
        !Compute amount of labour income of the disabled person that is exempted
        V2 = SOCBEN_BRACRATE_3COLUMNS(LABINC_J,SIZE(SOCBEN_LABIT_ANN,1),&
        &SOCBEN_LABIT_ANN,pers_j)
        R2 = 0.D0
        IF (LABINC_J.GT.V2) THEN
            R2 = LABINC_J - V2
        END IF
        !Compute the amount of replacement income of the disabled person that is exempted
        V3 = 0.D0
        IF (V2.LT.SOCBEN_REPIT_ANN(1)) THEN
            V3 = SOCBEN_REPIT_ANN(2)
        ELSE
            V3 = SOCBEN_REPIT_ANN(2) - (V2 - SOCBEN_REPIT_ANN(1))
        END IF
        IF (V3.LT.0.D0) THEN
            V3 = 0.D0
        END IF
        R3 = 0.D0
        IF (REPINC_J.GT.V3) THEN
            R3 = REPINC_J - V3
        END IF
        !Compute the amount of real estate income of individual j
        REALEST_J =  ((CADASTRAL_GINCOME(pers_j) + CADASTRAB_GINCOME(pers_j)) - SOCBEN_CADIVT_BASICD_ANN - &
        &SOCBEN_CADIVT_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(pers_j)))
        IF (REALEST_J.LT.0.D0) THEN
            REALEST_J = 0.D0
        END IF
        !Compute the amount of other income sources of the disabled person that could be exempted
        V4 = 0.D0
        IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.1) THEN
            V4 = SOCBEN_DISREIT_ANN(1,2) - (V2 + V3)
        ELSE IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.2) THEN
            V4 = SOCBEN_DISREIT_ANN(2,2) - (V2 + V3)
        ELSE IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.3) THEN
            V4 = SOCBEN_DISREIT_ANN(3,2) - (V2 + V3)
        END IF        
        IF (V4.LT.0.D0) THEN
            V4 = 0.D0
        END IF
        R4 = R3 + REALEST_J - V4
        IF (R4.LT.0.D0) THEN
            R4 = 0.D0
        END IF
        !Check whether there is a partner or not
        IF (SOCBEN_PARITIVT_ANN(pers_j).NE.0) THEN
            !Compute the total amount of taxable income of individual k
            TOTTAXPROFINC_K =   CONTRIB_GTIPRIV_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                               &CONTRIB_GTIPUB_ANN(SOCBEN_PARITIVT_ANN(pers_j))  + &
                               &CONTRIB_GTISELF_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                               &CONTRIB_GTIPENS_ANN(SOCBEN_PARITIVT_ANN(pers_j)) + &
                               &CONTRIB_GTIEARNE_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTIEAROL_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTIDISAB_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTINDOCC_ANN(SOCBEN_PARITIVT_ANN(pers_j))+ &
                               &CONTRIB_GTISICK_ANN(SOCBEN_PARITIVT_ANN(pers_j)) 
            !Compute the amount of real estate income of individual k
            REALEST_K =  ((CADASTRAL_GINCOME(SOCBEN_PARITIVT_ANN(pers_j)) + CADASTRAB_GINCOME(SOCBEN_PARITIVT_ANN(pers_j))) &
            &- SOCBEN_CADIVT_BASICD_ANN - SOCBEN_CADIVT_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(SOCBEN_PARITIVT_ANN(pers_j))))
            IF (REALEST_K.LT.0.D0) THEN
                REALEST_K = 0.D0
            END IF
            TOTINC_K = TOTTAXPROFINC_K + REALEST_K + SOCBEN_IGO_ANN(SOCBEN_PARITIVT_ANN(pers_j))
            R1A = 0.D0
            R1B = 0.D0
            !Compute exempted amount of the income of the partner
            IF (ANY(SOCBEN_MEDIT_ANN(pers_j).EQ.(/1, 2/))) THEN
                R1A = TOTINC_K - SOCBEN_BRACRATE_3COLUMNS(TOTINC_K,SIZE(SOCBEN_PARTITA_ANN,1),&
                &SOCBEN_PARTITA_ANN,pers_j)
                IF (R1A.LT.0.D0) THEN
                    R1A = 0.D0
                END IF
                !Recompute the amount of other income sources of the person with a disability
                R4 = R3 + REALEST_J + R1A - V4
                IF (R4.LT.0.D0) THEN
                    R4 = 0.D0
                END IF
            ELSE IF (ANY(SOCBEN_MEDIT_ANN(pers_j).EQ.(/3, 4, 5/))) THEN
                R1B = TOTINC_K - SOCBEN_BRACRATE_3COLUMNS(TOTINC_K,SIZE(SOCBEN_PARTITB_ANN,1),&
                &SOCBEN_PARTITB_ANN,pers_j)
                IF (R1B.LT.0.D0) THEN
                    R1B = 0.D0
                END IF
            END IF
            !Compute means to be subtracted later
            SOCBEN_MEANSIT_2004_ANN(pers_j) = R2 + R4 + R1B
        ELSE 
            !Compute means to be subtracted later
            SOCBEN_MEANSIT_2004_ANN(pers_j) = R2 + R4
        END IF
!        IF (GECODEERD_INSZ_INPUT(pers_j).EQ.7349) THEN
!            WRITE(15,*) 'TestMeanIT',LABINC_J,V2,V3,V4,R2,R4,R1B
!        END IF
    END DO    
    
    !Switch test back to 0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        SOCBEN_TEST_ANN(pers_j) = 0
    END DO

    !Create SOCBEN_MEANSTHB_ANN
    !Step 1: application of the accounting rules for individual income components
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        IF (SOCBEN_TEST_ANN(pers_j).EQ.0) THEN
            !Determine the income level necessary to execute the means test
            !Take into account means of both partners
            !Remark that the partner with which the personal income taxes are shared
            !is the first one apointed as partner of the possbile THB benefit receiver
            IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
                SOCBEN_TEST_ANN(pers_j) = 1
                SOCBEN_TEST_ANN(SOCBEN_PARTHB_ANN(pers_j)) = 1
                !Use instruments to determine net income from labour and other taxable sources
                !Compute total amount of taxes paid (or received in case of credit) by the couple 
                !Since taxes per couple are stored for only one of the two partners they have to be added
                TOTPIT = PIT_FINTAX_ANN(pers_j) + PIT_FINTAX_ANN(SOCBEN_PARTHB_ANN(pers_j))
                !compute amount for j
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                LABINC_J =          CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) 
                PENSINC_J = CONTRIB_GTIPENS_ANN(pers_j)           
                SHARELAB_J = 0.D0
                SHAREPENS_J = 0.D0
                IF (TOTTAXPROFINC_J.NE.0.D0) THEN
                    SHARELAB_J = LABINC_J/TOTTAXPROFINC_J
                    SHAREPENS_J = PENSINC_J/TOTTAXPROFINC_J
                END IF                    
                !compute amount for partner
                TOTTAXPROFINC_K =    CONTRIB_GTIPRIV_ANN(SOCBEN_PARTHB_ANN(pers_j)) + &
                                    &CONTRIB_GTIPUB_ANN(SOCBEN_PARTHB_ANN(pers_j))  + &
                                    &CONTRIB_GTISELF_ANN(SOCBEN_PARTHB_ANN(pers_j)) + &
                                    &CONTRIB_GTIPENS_ANN(SOCBEN_PARTHB_ANN(pers_j)) + &
                                    &CONTRIB_GTIEARNE_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                    &CONTRIB_GTIEAROL_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                    &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                    &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                    &CONTRIB_GTIDISAB_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                    &CONTRIB_GTINDOCC_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                    &CONTRIB_GTISICK_ANN(SOCBEN_PARTHB_ANN(pers_j))  
                LABINC_K =           CONTRIB_GTIPRIV_ANN(SOCBEN_PARTHB_ANN(pers_j)) + &
                                    &CONTRIB_GTIPUB_ANN(SOCBEN_PARTHB_ANN(pers_j))  + &
                                    &CONTRIB_GTISELF_ANN(SOCBEN_PARTHB_ANN(pers_j)) 
                PENSINC_K = CONTRIB_GTIPENS_ANN(SOCBEN_PARTHB_ANN(pers_j))           
                SHARELAB_K = 0.D0
                SHAREPENS_K = 0.D0
                IF (TOTTAXPROFINC_K.NE.0.D0) THEN
                    SHARELAB_K = LABINC_K/TOTTAXPROFINC_K
                    SHAREPENS_K = PENSINC_K/TOTTAXPROFINC_K
                END IF
                SHAREINTOT_J = 0.D0
                IF ((TOTTAXPROFINC_J+TOTTAXPROFINC_K).NE.0.D0) THEN
                    SHAREINTOT_J = TOTTAXPROFINC_J/(TOTTAXPROFINC_J+TOTTAXPROFINC_K)
                END IF                    
                !Determine income other than labour income and pension income net of taxes
                OTHTAXINC_J = (TOTTAXPROFINC_J - LABINC_J - PENSINC_J) - (1.D0 - SHARELAB_J - SHAREPENS_J)*SHAREINTOT_J*&
                &TOTPIT
                IF (OTHTAXINC_J.LT.0.D0) THEN
                    OTHTAXINC_J = 0.D0
                END IF
                OTHTAXINC_K = (TOTTAXPROFINC_K - LABINC_K - PENSINC_K) - (1.D0 - SHARELAB_K - SHAREPENS_K)*(1.D0 - SHAREINTOT_J)*&
                &TOTPIT
                IF (OTHTAXINC_K.LT.0.D0) THEN
                    OTHTAXINC_K = 0.D0
                END IF
                !take % of the net labour income
                LABINC_J = (SOCBEN_LABTHB_RATE/100.D0)*(LABINC_J - SHARELAB_J*SHAREINTOT_J*TOTPIT)
                IF (LABINC_J.LT.0.D0) THEN
                    LABINC_J = 0.D0
                END IF
                LABINC_K = (SOCBEN_LABTHB_RATE/100.D0)*(LABINC_K - SHARELAB_K*(1.D0 - SHAREINTOT_J)*TOTPIT)
                IF (LABINC_K.LT.0.D0) THEN
                    LABINC_K = 0.D0
                END IF
                !take % of the net pension income
                PENSINC_J = (SOCBEN_PENSTHB_RATE/100.D0)*(PENSINC_J - SHAREPENS_J*SHAREINTOT_J*TOTPIT)
                IF (PENSINC_J.LT.0.D0) THEN
                    PENSINC_J = 0.D0
                END IF
                PENSINC_K = (SOCBEN_PENSTHB_RATE/100.D0)*(PENSINC_K - SHAREPENS_K*(1.D0 - SHAREINTOT_J)*TOTPIT)
                IF (PENSINC_K.LT.0.D0) THEN
                    PENSINC_K = 0.D0
                END IF
                !Determine net resources from financial wealth
                TOTFINW = FINANCIAL_WEALTH(pers_j) + FINANCIAL_WEALTH(SOCBEN_PARTHB_ANN(pers_j))
                FINWEAL_J = SOCBEN_BRACRATE_3COLUMNS(TOTFINW,SIZE(SOCBEN_CAPRATHB_ANN,1),&
                &SOCBEN_CAPRATHB_ANN,pers_j)
                IF (FINWEAL_J.LT.0.D0) THEN
                    FINWEAL_J = 0.D0
                END IF
                !Determine net resources from real estate income 
                REALEST_J =  SOCBEN_CADLTHB_RATE*(CADASTRAL_GINCOME(pers_j)+CADASTRAL_GINCOME(SOCBEN_PARTHB_ANN(pers_j))&
                            &-SOCBEN_CADLTHB_BASICD_ANN) + &
                            &SOCBEN_CADBTHB_RATE*(CADASTRAB_GINCOME(pers_j)+CADASTRAB_GINCOME(SOCBEN_PARTHB_ANN(pers_j))&
                            &-SOCBEN_CADBTHB_BASICD_ANN-SOCBEN_CADBTHB_CHILDD_ANN*&
                            &(SOCBEN_DEPCHILD_ANN(pers_j)+SOCBEN_DEPCHILD_ANN(SOCBEN_PARTHB_ANN(pers_j))))
                IF (REALEST_J.LT.0.D0) THEN
                    REALEST_J = 0.D0
                END IF
                !Determine total amount of income per individual
                !Remark: means are attributed to both partners. Those that are currently tax included are divided
                !in proportion of their contribution to the taxable income. This division is necessary since
                !the tax amount for married couples is still determined jointly while for all others it is not
                !The other income amounts are divided equally over both partners after applying the deduction rules
                !on the joint amount of income
                SOCBEN_MEANSTHB_ANN(pers_j) =                       OTHTAXINC_J + LABINC_J + PENSINC_J + &
                                                                    &(FINWEAL_J+REALEST_J)/2.D0 + &
                                                                    &PRIVATE_PENSIONS(pers_j)+ALIMONY_RECEIVED(pers_j)+&
                                                                    &BENEFITS_NATURE(pers_j)
                SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) =    OTHTAXINC_K + LABINC_K + PENSINC_K + &
                                                                    &(FINWEAL_J+REALEST_J)/2.D0 + &
                                                                    &PRIVATE_PENSIONS(SOCBEN_PARTHB_ANN(pers_j))+&
                                                                    &ALIMONY_RECEIVED(SOCBEN_PARTHB_ANN(pers_j))+&
                                                                    &BENEFITS_NATURE(SOCBEN_PARTHB_ANN(pers_j))
            ELSE
                SOCBEN_TEST_ANN(pers_j) = 1
                !Use instruments to determine net income from labour and other taxable sources
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                LABINC_J =          CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) 
                PENSINC_J = CONTRIB_GTIPENS_ANN(pers_j)           
                SHARELAB_J = 0.D0
                SHAREPENS_J = 0.D0
                IF (TOTTAXPROFINC_J.NE.0.D0) THEN
                    SHARELAB_J = LABINC_J/TOTTAXPROFINC_J
                    SHAREPENS_J = PENSINC_J/TOTTAXPROFINC_J
                END IF                    
                !Determine income other than labour income net of taxes
                OTHTAXINC_J = (TOTTAXPROFINC_J - LABINC_J - PENSINC_J) - (1.D0 - SHARELAB_J - SHAREPENS_J)*PIT_FINTAX_ANN(pers_j)
                IF (OTHTAXINC_J.LT.0.D0) THEN
                    OTHTAXINC_J = 0.D0
                END IF
                !take % of the net labour income
                LABINC_J = (SOCBEN_LABTHB_RATE/100.D0)*(LABINC_J - SHARELAB_J*PIT_FINTAX_ANN(pers_j))
                IF (LABINC_J.LT.0.D0) THEN
                    LABINC_J = 0.D0
                END IF
                !take % of the net pension income
                PENSINC_J = (SOCBEN_PENSTHB_RATE/100.D0)*(PENSINC_J - SHAREPENS_J*PIT_FINTAX_ANN(pers_j))
                IF (PENSINC_J.LT.0.D0) THEN
                    PENSINC_J = 0.D0
                END IF
                !Determine net resources from financial wealth
                FINWEAL_J = SOCBEN_BRACRATE_3COLUMNS(FINANCIAL_WEALTH(pers_j),SIZE(SOCBEN_CAPRATHB_ANN,1),&
                &SOCBEN_CAPRATHB_ANN,pers_j)
                IF (FINWEAL_J.LT.0.D0) THEN
                    FINWEAL_J = 0.D0
                END IF
                !Determine net resources from real estate income 
                REALEST_J =  SOCBEN_CADLTHB_RATE*(CADASTRAL_GINCOME(pers_j)-SOCBEN_CADLTHB_BASICD_ANN) + &
                            &SOCBEN_CADBTHB_RATE*(CADASTRAB_GINCOME(pers_j)&
                            &-SOCBEN_CADBTHB_BASICD_ANN-SOCBEN_CADBTHB_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(pers_j)))
                IF (REALEST_J.LT.0.D0) THEN
                    REALEST_J = 0.D0
                END IF
                !Determine total amount of income
                SOCBEN_MEANSTHB_ANN(pers_j) =   OTHTAXINC_J + LABINC_J + PENSINC_J + FINWEAL_J + REALEST_J + &
                                                &PRIVATE_PENSIONS(pers_j)+ALIMONY_RECEIVED(pers_j)+BENEFITS_NATURE(pers_j)
            END IF
        END IF
    END DO

    !Switch test back to 0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        SOCBEN_TEST_ANN(pers_j) = 0
    END DO

    !Create SOCBEN_MEANSTHB_ANN
    !Step 2: subtract general disregard
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        IF (SOCBEN_TEST_ANN(pers_j).EQ.0) THEN
            !If there is a partner, check whether both partners are of type C
            !Here it has to be condition: partner is not 0 instead of married couple
            IF (SOCBEN_PARTHB_ANN(pers_j).NE.0) THEN
                SOCBEN_TEST_ANN(pers_j) = 1
                SOCBEN_TEST_ANN(SOCBEN_PARTHB_ANN(pers_j)) = 1
                !When both partners are of type C and satisfy the medical conditions to receive a benefit
                !subtract half of the disregard
                !Medical condition has to be satisfied, otherwise one would disadvantage a person 
                !with a type 3 family member that is not entitled (this can appear due to our 
                !way of attributing the family types)
                IF (&
                    &SOCBEN_FAMTHB_ANN(pers_j).EQ.3.AND.&
                    &SOCBEN_FAMTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)).EQ.3.AND.&
                    &ANY(SOCBEN_MEDTHB_ANN(pers_j).EQ.(/1, 2, 3, 4, 5/)).AND.&
                    &ANY(SOCBEN_MEDTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)).EQ.(/1, 2, 3, 4, 5/))) THEN
                        SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(3,2)/2.D0
                        SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) = SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) &
                        &- SOCBEN_DISRETHB_ANN(3,2)/2.D0
                ELSE
                    !Loop over possibilities for disabled person
                    IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.1) THEN
                        SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(1,2)
                    ELSE IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.2) THEN
                        SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(2,2)
                    ELSE IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.3) THEN
                        SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(3,2)
                    END IF
                    !Loop over possibilities for partner 
                    IF (SOCBEN_FAMTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)).EQ.1) THEN
                        SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) = SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) &
                        &- SOCBEN_DISRETHB_ANN(1,2)
                    ELSE IF (SOCBEN_FAMTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)).EQ.2) THEN
                        SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) = SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) &
                        &- SOCBEN_DISRETHB_ANN(2,2)
                    ELSE IF (SOCBEN_FAMTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)).EQ.3) THEN
                        SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) = SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) &
                        &- SOCBEN_DISRETHB_ANN(3,2)
                    END IF
                END IF
            ELSE
                SOCBEN_TEST_ANN(pers_j) = 1
                !Loop over possibilities for disabled person
                IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.1) THEN
                    SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(1,2)
                ELSE IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.2) THEN
                    SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(2,2)
                ELSE IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.3) THEN
                    SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(3,2)
                END IF
            END IF
        END IF
        IF (SOCBEN_MEANSTHB_ANN(pers_j).LT.0.D0) THEN
            SOCBEN_MEANSTHB_ANN(pers_j) = 0.D0
        END IF
    END DO
        
END SUBROUTINE SOCBEN_INSTRUM_B

!===========================================================================!
!Construct some instruments to carry out the later social benefit tests     !
!Handicap-creation necessary to carry out the simulations for DG PMH        !
!Simulatie scenario's worden opgesomd in mail van GvdVelde van 2 mei 2010   !
!met als subject Mimosis en DG Personen met een handicap                    !
!   SOCBEN_MEANSIT_2004_ANN                                                 !
!   SOCBEN_MEANSTHB_ANN                                                     !
!===========================================================================!

SUBROUTINE SOCBEN_INSTRUM_C(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j
    REAL(KIND=KIND_REAL)            :: LABINC_J,SHARELAB_J,PENSINC_J,SHAREPENS_J,TOTTAXPROFINC_J,OTHTAXINC_J,FINWEAL_J,REALEST_J
    REAL(KIND=KIND_REAL)            :: LABINC_K,SHARELAB_K,PENSINC_K,SHAREPENS_K,TOTTAXPROFINC_K,OTHTAXINC_K,TOTINC_K
    REAL(KIND=KIND_REAL)            :: SHAREINTOT_J,TOTFINW   
    REAL(KIND=KIND_REAL)            :: V2,V3,V4,R1A,R1B,R2,R3,R4,REALEST_K,REPINC_J
    REAL(KIND=KIND_REAL)            :: MEANSA,MEANSB    

    IF (SOCBEN_SIMDGPMH_QT.EQ.4) THEN 
        !Create SOCBEN_MEANSTHB_ANN
        !Step 1: application of the accounting rules for individual income components
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
            IF (SOCBEN_TEST_ANN(pers_j).EQ.0) THEN
                !Determine the income level necessary to execute the means test
                !Take into account means of both partners
                !Remark that the partner with which the personal income taxes are shared
                !is the first one apointed as partner of the possbile THB benefit receiver
                IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
                    SOCBEN_TEST_ANN(pers_j) = 1
                    SOCBEN_TEST_ANN(SOCBEN_PARTHB_ANN(pers_j)) = 1
                    !Use instruments to determine net income from labour and other taxable sources
                    !compute amount for j
                    TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                       &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                       &CONTRIB_GTISELF_ANN(pers_j) + &
                                       &CONTRIB_GTIPENS_ANN(pers_j) + &
                                       &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                       &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                       &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                       &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                       &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                       &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                       &CONTRIB_GTISICK_ANN(pers_j)  
                    LABINC_J =          CONTRIB_GTIPRIV_ANN(pers_j) + &
                                       &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                       &CONTRIB_GTISELF_ANN(pers_j) 
                    PENSINC_J = CONTRIB_GTIPENS_ANN(pers_j)           
                    SHARELAB_J = 0.D0
                    SHAREPENS_J = 0.D0
                    IF (TOTTAXPROFINC_J.NE.0.D0) THEN
                        SHARELAB_J = LABINC_J/TOTTAXPROFINC_J
                        SHAREPENS_J = PENSINC_J/TOTTAXPROFINC_J
                    END IF                    
                    !compute amount for partner
                    TOTTAXPROFINC_K =    CONTRIB_GTIPRIV_ANN(SOCBEN_PARTHB_ANN(pers_j)) + &
                                        &CONTRIB_GTIPUB_ANN(SOCBEN_PARTHB_ANN(pers_j))  + &
                                        &CONTRIB_GTISELF_ANN(SOCBEN_PARTHB_ANN(pers_j)) + &
                                        &CONTRIB_GTIPENS_ANN(SOCBEN_PARTHB_ANN(pers_j)) + &
                                        &CONTRIB_GTIEARNE_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                        &CONTRIB_GTIEAROL_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                        &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                        &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                        &CONTRIB_GTIDISAB_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                        &CONTRIB_GTINDOCC_ANN(SOCBEN_PARTHB_ANN(pers_j))+ &
                                        &CONTRIB_GTISICK_ANN(SOCBEN_PARTHB_ANN(pers_j))  
                    LABINC_K =           CONTRIB_GTIPRIV_ANN(SOCBEN_PARTHB_ANN(pers_j)) + &
                                        &CONTRIB_GTIPUB_ANN(SOCBEN_PARTHB_ANN(pers_j))  + &
                                        &CONTRIB_GTISELF_ANN(SOCBEN_PARTHB_ANN(pers_j)) 
                    PENSINC_K = CONTRIB_GTIPENS_ANN(SOCBEN_PARTHB_ANN(pers_j))           
                    SHARELAB_K = 0.D0
                    SHAREPENS_K = 0.D0
                    IF (TOTTAXPROFINC_K.NE.0.D0) THEN
                        SHARELAB_K = LABINC_K/TOTTAXPROFINC_K
                        SHAREPENS_K = PENSINC_K/TOTTAXPROFINC_K
                    END IF
                    SHAREINTOT_J = 0.D0
                    IF ((TOTTAXPROFINC_J+TOTTAXPROFINC_K).NE.0.D0) THEN
                        SHAREINTOT_J = TOTTAXPROFINC_J/(TOTTAXPROFINC_J+TOTTAXPROFINC_K)
                    END IF                    
                    !Determine income other than labour income and pension income
                    OTHTAXINC_J = (TOTTAXPROFINC_J - LABINC_J - PENSINC_J) 
                    IF (OTHTAXINC_J.LT.0.D0) THEN
                        OTHTAXINC_J = 0.D0
                    END IF
                    OTHTAXINC_K = (TOTTAXPROFINC_K - LABINC_K - PENSINC_K) 
                    IF (OTHTAXINC_K.LT.0.D0) THEN
                        OTHTAXINC_K = 0.D0
                    END IF
                    !take % of the labour income
                    LABINC_J = (SOCBEN_LABTHB_RATE/100.D0)*(LABINC_J)
                    IF (LABINC_J.LT.0.D0) THEN
                        LABINC_J = 0.D0
                    END IF
                    LABINC_K = (SOCBEN_LABTHB_RATE/100.D0)*(LABINC_K)
                    IF (LABINC_K.LT.0.D0) THEN
                        LABINC_K = 0.D0
                    END IF
                    !take % of the pension income
                    PENSINC_J = (SOCBEN_PENSTHB_RATE/100.D0)*(PENSINC_J)
                    IF (PENSINC_J.LT.0.D0) THEN
                        PENSINC_J = 0.D0
                    END IF
                    PENSINC_K = (SOCBEN_PENSTHB_RATE/100.D0)*(PENSINC_K)
                    IF (PENSINC_K.LT.0.D0) THEN
                        PENSINC_K = 0.D0
                    END IF
                    !Determine net resources from financial wealth
                    TOTFINW = FINANCIAL_WEALTH(pers_j) + FINANCIAL_WEALTH(SOCBEN_PARTHB_ANN(pers_j))
                    FINWEAL_J = SOCBEN_BRACRATE_3COLUMNS(TOTFINW,SIZE(SOCBEN_CAPRATHB_ANN,1),&
                    &SOCBEN_CAPRATHB_ANN,pers_j)
                    IF (FINWEAL_J.LT.0.D0) THEN
                        FINWEAL_J = 0.D0
                    END IF
                    !Determine net resources from real estate income 
                    REALEST_J =  SOCBEN_CADLTHB_RATE*(CADASTRAL_GINCOME(pers_j)+CADASTRAL_GINCOME(SOCBEN_PARTHB_ANN(pers_j))&
                                &-SOCBEN_CADLTHB_BASICD_ANN) + &
                                &SOCBEN_CADBTHB_RATE*(CADASTRAB_GINCOME(pers_j)+CADASTRAB_GINCOME(SOCBEN_PARTHB_ANN(pers_j))&
                                &-SOCBEN_CADBTHB_BASICD_ANN-SOCBEN_CADBTHB_CHILDD_ANN*&
                                &(SOCBEN_DEPCHILD_ANN(pers_j)+SOCBEN_DEPCHILD_ANN(SOCBEN_PARTHB_ANN(pers_j))))
                    IF (REALEST_J.LT.0.D0) THEN
                        REALEST_J = 0.D0
                    END IF
                    !Determine total amount of income per individual
                    !Remark: means are attributed to both partners. Those that are currently tax included are divided
                    !in proportion of their contribution to the taxable income. This division is necessary since
                    !the tax amount for married couples is still determined jointly while for all others it is not
                    !The other income amounts are divided equally over both partners after applying the deduction rules
                    !on the joint amount of income
                    SOCBEN_MEANSTHB_ANN(pers_j) =                       OTHTAXINC_J + LABINC_J + PENSINC_J + &
                                                                        &(FINWEAL_J+REALEST_J)/2.D0 + &
                                                                        &PRIVATE_PENSIONS(pers_j)+ALIMONY_RECEIVED(pers_j)+&
                                                                        &BENEFITS_NATURE(pers_j)
                    SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) =    OTHTAXINC_K + LABINC_K + PENSINC_K + &
                                                                        &(FINWEAL_J+REALEST_J)/2.D0 + &
                                                                        &PRIVATE_PENSIONS(SOCBEN_PARTHB_ANN(pers_j))+&
                                                                        &ALIMONY_RECEIVED(SOCBEN_PARTHB_ANN(pers_j))+&
                                                                        &BENEFITS_NATURE(SOCBEN_PARTHB_ANN(pers_j))
                ELSE
                    SOCBEN_TEST_ANN(pers_j) = 1
                    !Use instruments to determine net income from labour and other taxable sources
                    TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                       &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                       &CONTRIB_GTISELF_ANN(pers_j) + &
                                       &CONTRIB_GTIPENS_ANN(pers_j) + &
                                       &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                       &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                       &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                       &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                       &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                       &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                       &CONTRIB_GTISICK_ANN(pers_j)  
                    LABINC_J =          CONTRIB_GTIPRIV_ANN(pers_j) + &
                                       &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                       &CONTRIB_GTISELF_ANN(pers_j) 
                    PENSINC_J = CONTRIB_GTIPENS_ANN(pers_j)           
                    SHARELAB_J = 0.D0
                    SHAREPENS_J = 0.D0
                    IF (TOTTAXPROFINC_J.NE.0.D0) THEN
                        SHARELAB_J = LABINC_J/TOTTAXPROFINC_J
                        SHAREPENS_J = PENSINC_J/TOTTAXPROFINC_J
                    END IF                    
                    !Determine income other than labour income net of taxes
                    OTHTAXINC_J = (TOTTAXPROFINC_J - LABINC_J - PENSINC_J)
                    IF (OTHTAXINC_J.LT.0.D0) THEN
                        OTHTAXINC_J = 0.D0
                    END IF
                    !take % of the labour income
                    LABINC_J = (SOCBEN_LABTHB_RATE/100.D0)*(LABINC_J)
                    IF (LABINC_J.LT.0.D0) THEN
                        LABINC_J = 0.D0
                    END IF
                    !take % of the pension income
                    PENSINC_J = (SOCBEN_PENSTHB_RATE/100.D0)*(PENSINC_J)
                    IF (PENSINC_J.LT.0.D0) THEN
                        PENSINC_J = 0.D0
                    END IF
                    !Determine net resources from financial wealth
                    FINWEAL_J = SOCBEN_BRACRATE_3COLUMNS(FINANCIAL_WEALTH(pers_j),SIZE(SOCBEN_CAPRATHB_ANN,1),&
                    &SOCBEN_CAPRATHB_ANN,pers_j)
                    IF (FINWEAL_J.LT.0.D0) THEN
                        FINWEAL_J = 0.D0
                    END IF
                    !Determine net resources from real estate income 
                    REALEST_J =  SOCBEN_CADLTHB_RATE*(CADASTRAL_GINCOME(pers_j)-SOCBEN_CADLTHB_BASICD_ANN) + &
                                &SOCBEN_CADBTHB_RATE*(CADASTRAB_GINCOME(pers_j)&
                                &-SOCBEN_CADBTHB_BASICD_ANN-SOCBEN_CADBTHB_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(pers_j)))
                    IF (REALEST_J.LT.0.D0) THEN
                        REALEST_J = 0.D0
                    END IF
                    !Determine total amount of income
                    SOCBEN_MEANSTHB_ANN(pers_j) =   OTHTAXINC_J + LABINC_J + PENSINC_J + FINWEAL_J + REALEST_J + &
                                                    &PRIVATE_PENSIONS(pers_j)+ALIMONY_RECEIVED(pers_j)+BENEFITS_NATURE(pers_j)
                END IF
            END IF
        END DO
    
        !Switch test back to 0
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
            SOCBEN_TEST_ANN(pers_j) = 0
        END DO
    
        !Create SOCBEN_MEANSTHB_ANN
        !Step 2: subtract general disregard
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
            IF (SOCBEN_TEST_ANN(pers_j).EQ.0) THEN
                !If there is a partner, check whether both partners are of type C
                !Here it has to be condition: partner is not 0 instead of married couple
                IF (SOCBEN_PARTHB_ANN(pers_j).NE.0) THEN
                    SOCBEN_TEST_ANN(pers_j) = 1
                    SOCBEN_TEST_ANN(SOCBEN_PARTHB_ANN(pers_j)) = 1
                    !When both partners are of type C and satisfy the medical conditions to receive a benefit
                    !subtract half of the disregard
                    !Medical condition has to be satisfied, otherwise one would disadvantage a person 
                    !with a type 3 family member that is not entitled (this can appear due to our 
                    !way of attributing the family types)
                    IF (&
                        &SOCBEN_FAMTHB_ANN(pers_j).EQ.3.AND.&
                        &SOCBEN_FAMTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)).EQ.3.AND.&
                        &ANY(SOCBEN_MEDTHB_ANN(pers_j).EQ.(/1, 2, 3, 4, 5/)).AND.&
                        &ANY(SOCBEN_MEDTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)).EQ.(/1, 2, 3, 4, 5/))) THEN
                            SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(3,2)/2.D0
                            SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) = SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) &
                            &- SOCBEN_DISRETHB_ANN(3,2)/2.D0
                    ELSE
                        !Loop over possibilities for disabled person
                        IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.1) THEN
                            SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(1,2)
                        ELSE IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.2) THEN
                            SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(2,2)
                        ELSE IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.3) THEN
                            SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(3,2)
                        END IF
                        !Loop over possibilities for partner 
                        IF (SOCBEN_FAMTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)).EQ.1) THEN
                            SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) = SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) &
                            &- SOCBEN_DISRETHB_ANN(1,2)
                        ELSE IF (SOCBEN_FAMTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)).EQ.2) THEN
                            SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) = SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) &
                            &- SOCBEN_DISRETHB_ANN(2,2)
                        ELSE IF (SOCBEN_FAMTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)).EQ.3) THEN
                            SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) = SOCBEN_MEANSTHB_ANN(SOCBEN_PARTHB_ANN(pers_j)) &
                            &- SOCBEN_DISRETHB_ANN(3,2)
                        END IF
                    END IF
                ELSE
                    SOCBEN_TEST_ANN(pers_j) = 1
                    !Loop over possibilities for disabled person
                    IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.1) THEN
                        SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(1,2)
                    ELSE IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.2) THEN
                        SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(2,2)
                    ELSE IF (SOCBEN_FAMTHB_ANN(pers_j).EQ.3) THEN
                        SOCBEN_MEANSTHB_ANN(pers_j) = SOCBEN_MEANSTHB_ANN(pers_j) - SOCBEN_DISRETHB_ANN(3,2)
                    END IF
                END IF
            END IF
            IF (SOCBEN_MEANSTHB_ANN(pers_j).LT.0.D0) THEN
                SOCBEN_MEANSTHB_ANN(pers_j) = 0.D0
            END IF
        END DO
    ELSE IF (ANY(SOCBEN_SIMDGPMH_QT.EQ.(/5, 6/))) THEN
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)  
            SOCBEN_MEANSIT_2004_ANN(pers_j) = 0.D0
            SOCBEN_MEANSTHB_ANN(pers_j) = 0.D0
        END DO
    END IF
            
END SUBROUTINE SOCBEN_INSTRUM_C

!===========================================================================!
!Construct some instruments to carry out the later tests for                !
!Health benefit-creation                                                    !
! SOCBEN_MEVT_ANN                                                           !
! SOCBEN_MEOMNIO_ANN                                                        !
! SOCBEN_MEINCMAF_ANN                                                       !
!===========================================================================!

SUBROUTINE SOCBEN_INSTRUM_HEALTH(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j
    REAL(KIND=KIND_REAL)            :: TOTTAXPROFINC_J,REALEST_J
    REAL(KIND=KIND_REAL)            :: TOTTAXPROFINC_K

    !Switch test back to 0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        SOCBEN_TEST_ANN(pers_j) = 0
    END DO

    !Create SOCBEN_MEVT_ANN                      !Individual means needed when executing the means test for the VT health benefit
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        IF (SOCBEN_TEST_ANN(pers_j).EQ.0) THEN
            !Determine the income level necessary to execute the means test
            !Take into account means of both partners
            !In principle division of couples and singles is not necessary here since we use
            IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
                SOCBEN_TEST_ANN(pers_j) = 1
                SOCBEN_TEST_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 1
                !Use instruments to determine net income from labour and other taxable sources
                !Compute total amount of taxes paid (or received in case of credit) by the couple 
                !Since taxes per couple are stored for only one of the two partners they have to be added
                !use basic partner id here SOCBEN_PARTNID_ANN(pers_j)
                !compute amount for j
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                !compute amount for partner
                TOTTAXPROFINC_K =    CONTRIB_GTIPRIV_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIPUB_ANN(SOCBEN_PARTNID_ANN(pers_j))  + &
                                    &CONTRIB_GTISELF_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIPENS_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIEARNE_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIEAROL_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIDISAB_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTINDOCC_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTISICK_ANN(SOCBEN_PARTNID_ANN(pers_j))  
                !Determine net resources from real estate income
                !REMARK: all builded income is taken: not adapted
                REALEST_J =  (CADASTRAB_GINCOME(pers_j)+CADASTRAB_GINCOME(SOCBEN_PARTNID_ANN(pers_j)))&
                            &-SOCBEN_CADBVT_BASICD_ANN-SOCBEN_CADBVT_CHILDD_ANN*&
                            &(SOCBEN_DEPCHILD_ANN(pers_j)+SOCBEN_DEPCHILD_ANN(SOCBEN_PARTNID_ANN(pers_j)))
                IF (REALEST_J.LT.0.D0) THEN
                    REALEST_J = 0.D0
                END IF
                !Determine total amount of income per individual
                SOCBEN_MEVT_ANN(pers_j) =                      TOTTAXPROFINC_J + &
                                                              &(REALEST_J)/2.D0 + &
                                                              &PRIVATE_PENSIONS(pers_j)
                SOCBEN_MEVT_ANN(SOCBEN_PARTNID_ANN(pers_j)) =  TOTTAXPROFINC_K + &
                                                              &(REALEST_J)/2.D0 + &
                                                              &PRIVATE_PENSIONS(SOCBEN_PARTNID_ANN(pers_j))
            ELSE
                SOCBEN_TEST_ANN(pers_j) = 1
                !Use instruments to determine net income from labour and other taxable sources
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                !Determine net resources from real estate income 
                REALEST_J =  CADASTRAB_GINCOME(pers_j)&
                            &-SOCBEN_CADBVT_BASICD_ANN-SOCBEN_CADBVT_CHILDD_ANN*(SOCBEN_DEPCHILD_ANN(pers_j))
                IF (REALEST_J.LT.0.D0) THEN
                    REALEST_J = 0.D0
                END IF
                !Determine total amount of income
                SOCBEN_MEVT_ANN(pers_j) =   TOTTAXPROFINC_J + &
                                           &REALEST_J + &
                                           &PRIVATE_PENSIONS(pers_j)
            END IF
        END IF
    END DO
    
    !Switch test back to 0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        SOCBEN_TEST_ANN(pers_j) = 0
    END DO

    !SOCBEN_MEOMNIO_ANN                   !Individual means needed when executing the means test for the OMNIO health benefit
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        IF (SOCBEN_TEST_ANN(pers_j).EQ.0) THEN
            !Determine the income level necessary to execute the means test
            !Take into account means of both partners
            !In principle division of couples and singles is not necessary here since we use
            IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
                SOCBEN_TEST_ANN(pers_j) = 1
                SOCBEN_TEST_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 1
                !Use instruments to determine net income from labour and other taxable sources
                !Compute total amount of taxes paid (or received in case of credit) by the couple 
                !Since taxes per couple are stored for only one of the two partners they have to be added
                !use basic partner id here SOCBEN_PARTNID_ANN(pers_j)
                !compute amount for j
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                !compute amount for partner
                TOTTAXPROFINC_K =    CONTRIB_GTIPRIV_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIPUB_ANN(SOCBEN_PARTNID_ANN(pers_j))  + &
                                    &CONTRIB_GTISELF_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIPENS_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIEARNE_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIEAROL_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIDISAB_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTINDOCC_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTISICK_ANN(SOCBEN_PARTNID_ANN(pers_j))  
                !REMARK: Status real estate income is not clear, maybe to be added later
                !Determine total amount of income per individual
                SOCBEN_MEOMNIO_ANN(pers_j) =                      TOTTAXPROFINC_J 
                SOCBEN_MEOMNIO_ANN(SOCBEN_PARTNID_ANN(pers_j)) =  TOTTAXPROFINC_K 
            ELSE
                SOCBEN_TEST_ANN(pers_j) = 1
                !Use instruments to determine net income from labour and other taxable sources
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                !Determine total amount of income
                SOCBEN_MEOMNIO_ANN(pers_j) =   TOTTAXPROFINC_J 
            END IF
        END IF
    END DO
    
    !Switch test back to 0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        SOCBEN_TEST_ANN(pers_j) = 0
    END DO

    !SOCBEN_MEINCMAF_ANN                  !Individual means needed when executing the means test for the Income MAF
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        IF (SOCBEN_TEST_ANN(pers_j).EQ.0) THEN
            !Determine the income level necessary to execute the means test
            !Take into account means of both partners
            !In principle division of couples and singles is not necessary here since we use
            IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
                SOCBEN_TEST_ANN(pers_j) = 1
                SOCBEN_TEST_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 1
                !Use instruments to determine net income from labour and other taxable sources
                !Compute total amount of taxes paid (or received in case of credit) by the couple 
                !Since taxes per couple are stored for only one of the two partners they have to be added
                !use basic partner id here SOCBEN_PARTNID_ANN(pers_j)
                !compute amount for j
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                !compute amount for partner
                TOTTAXPROFINC_K =    CONTRIB_GTIPRIV_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIPUB_ANN(SOCBEN_PARTNID_ANN(pers_j))  + &
                                    &CONTRIB_GTISELF_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIPENS_ANN(SOCBEN_PARTNID_ANN(pers_j)) + &
                                    &CONTRIB_GTIEARNE_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIEAROL_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIOLDUN_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIOTHUN_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTIDISAB_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTINDOCC_ANN(SOCBEN_PARTNID_ANN(pers_j))+ &
                                    &CONTRIB_GTISICK_ANN(SOCBEN_PARTNID_ANN(pers_j))  
                !REMARK: Status real estate income is not clear, maybe to be added later
                !Determine total amount of income per individual
                SOCBEN_MEINCMAF_ANN(pers_j) =                      TOTTAXPROFINC_J 
                SOCBEN_MEINCMAF_ANN(SOCBEN_PARTNID_ANN(pers_j)) =  TOTTAXPROFINC_K 
            ELSE
                SOCBEN_TEST_ANN(pers_j) = 1
                !Use instruments to determine net income from labour and other taxable sources
                TOTTAXPROFINC_J =   CONTRIB_GTIPRIV_ANN(pers_j) + &
                                   &CONTRIB_GTIPUB_ANN(pers_j)  + &
                                   &CONTRIB_GTISELF_ANN(pers_j) + &
                                   &CONTRIB_GTIPENS_ANN(pers_j) + &
                                   &CONTRIB_GTIEARNE_ANN(pers_j)+ &
                                   &CONTRIB_GTIEAROL_ANN(pers_j)+ &
                                   &CONTRIB_GTIOLDUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIOTHUN_ANN(pers_j)+ &
                                   &CONTRIB_GTIDISAB_ANN(pers_j)+ &
                                   &CONTRIB_GTINDOCC_ANN(pers_j)+ &
                                   &CONTRIB_GTISICK_ANN(pers_j)  
                !Determine total amount of income
                SOCBEN_MEINCMAF_ANN(pers_j) =   TOTTAXPROFINC_J 
            END IF
        END IF
    END DO

    !SOCBEN_VTSTAT_ANN                    !Individual categorie for VT benefit
    !SOCBEN_VTSTAT_ANN = 1 person receiving IVT/IT benefit
    !SOCBEN_VTSTAT_ANN = 2 person receiving THB benefit
    !SOCBEN_VTSTAT_ANN = 3 person receiving leefloon
    !SOCBEN_VTSTAT_ANN = 4 person receiving IGO
    !SOCBEN_VTSTAT_ANN = 5 person with disability but no benefit
    !SOCBEN_VTSTAT_ANN = 6 person in disability after work 
    !SOCBEN_VTSTAT_ANN = 7 widow or widower
    !SOCBEN_VTSTAT_ANN = 8 orphan (either one or two parents are dead)
    !SOCBEN_VTSTAT_ANN = 9 unemployed of 50 or older and more than 12 months in unemployment
    !SOCBEN_VTSTAT_ANN = 10 person above 65 (assumption is here that all people above 65 are entitled, even if not pensioned)
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        !Determine status without income condition
        IF (SOCBEN_VTSTAT_ANN(pers_j).EQ.0.AND.(SOCBEN_IVT_ANN(pers_j).GT.0.D0.&
        &OR.SOCBEN_IT_ANN(pers_j).GT.0.D0)) THEN
            SOCBEN_VTSTAT_ANN(pers_j) = 1
        ELSE IF (SOCBEN_VTSTAT_ANN(pers_j).EQ.0.AND.SOCBEN_THB_ANN(pers_j).GT.0.D0) THEN
            SOCBEN_VTSTAT_ANN(pers_j) = 2
        ELSE IF (SOCBEN_VTSTAT_ANN(pers_j).EQ.0.AND.SOCBEN_LEEFLOON_ANN(pers_j).GT.0.D0) THEN
            SOCBEN_VTSTAT_ANN(pers_j) = 3
        ELSE IF (SOCBEN_VTSTAT_ANN(pers_j).EQ.0.AND.SOCBEN_IGO_ANN(pers_j).GT.0.D0) THEN
            SOCBEN_VTSTAT_ANN(pers_j) = 4
        ELSE IF (SOCBEN_VTSTAT_ANN(pers_j).EQ.0.AND.(SOCBEN_MEDIVT_ANN(pers_j).LT.33.&
        &OR.ANY(SOCBEN_MEDIT_ANN(pers_j).EQ.(/1, 2, 3, 4, 5/)).OR.ANY(SOCBEN_MEDTHB_ANN(pers_j).EQ.(/1, 2, 3, 4, 5/)))) THEN
            SOCBEN_VTSTAT_ANN(pers_j) = 5
        ELSE IF (ANY(SICK_DISABBEN_QT(pers_j).EQ.(/1, 2, 3/))) THEN
            SOCBEN_VTSTAT_ANN(pers_j) = 6
        ELSE IF (SOCBEN_VTSTAT_ANN(pers_j).EQ.0.AND.NAREGNIS_CIVIL(pers_j).EQ.3) THEN
            SOCBEN_VTSTAT_ANN(pers_j) = 7
        ELSE IF (SOCBEN_VTSTAT_ANN(pers_j).EQ.0.AND.ANY(FAMAL_ORPHAN_QT(pers_j).EQ.(/1, 2/))) THEN
            SOCBEN_VTSTAT_ANN(pers_j) = 8
        ELSE IF (SOCBEN_VTSTAT_ANN(pers_j).EQ.0.AND.UNEM_NUMMINU_QT(pers_j).GE.12.&
        &AND.SOCBEN_YEAR_AGE(pers_j).GE.50) THEN
            SOCBEN_VTSTAT_ANN(pers_j) = 9
        ELSE IF (SOCBEN_VTSTAT_ANN(pers_j).EQ.0.AND.SOCBEN_YEAR_AGE(pers_j).GE.65) THEN
            SOCBEN_VTSTAT_ANN(pers_j) = 10
        END IF            
    END DO
    
END SUBROUTINE SOCBEN_INSTRUM_HEALTH

!=============================================================================================================!
!Construction of
!SOCBEN_FAMSIT_2001_ANN                                           
!   = The family situation of the person entitled to a leefloon
!   SOCBEN_FAMSIT_2001_ANN = 1 a person who is married to someone who has insuficient means itself 
!   SOCBEN_FAMSIT_2001_ANN = 2 a single parent 
!   SOCBEN_FAMSIT_2001_ANN = 3 a true single (= person living alone) 
!   SOCBEN_FAMSIT_2001_ANN = 4 a cohabitating person 
!   SOCBEN_FAMSIT_2001_ANN = 5 a person who is married and on which a means test is applied but who does not receive the leefloon
!and
!SOCBEN_LEEFLOON_ANN                       
!   = Amount of minimum income transfer
!as applied in the period 2001-2002
!=============================================================================================================!
!Tot 1 oktober 2002
!    categorie 1: koppel (=wettelijk gehuwden die samen voldoen aan de leefloontoets)
!    categorie 2: noudergezinnen (= alleenstaande met kinderen ten laste)
!    categorie 3: echte alleenstaande 
!    categorie 4: samenwonende (persoon die samenwoont met anderen die niet leefloongerechtigd zijn)
!=============================================================================================================!

SUBROUTINE SOCBEN_LEEFLOON_2001(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j
    REAL(KIND=KIND_REAL)            :: REST    

    !Loop over all members of the sociological household
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                      
        !Check whether pers_j satisfies the age condition to receive a leefloon
        !remark if SOCBEN_ENTLEIGO_ANN(pers_j,2) is used:
        !although the strict age condition only applies from October 1st 2002 onwards we apply it for all cases
        IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.1) THEN
            IF (SOCBEN_FAMSIT_2001_ANN(pers_j).EQ.0) THEN
                !Determine the family situation of the leefloon-receiver and the leefloon he receives
                !Either the individual lives alone
                !Take into account the overall disregard when determining the leefloon
                IF (SOCBEN_HH_SIZE(hh_i).EQ.1) THEN
                    REST = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(3,2)
                    IF (REST.LT.0.D0) THEN
                        REST = 0.D0
                    END IF
                    IF (REST.LT.SOCBEN_MINTYM65_ANN(3,2)) THEN
                        SOCBEN_FAMSIT_2001_ANN(pers_j) = 3
                        SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(3,2) - REST
                    END IF                        
                !or cohabits with other people
                ELSE
                    !Either the individual is married
                    IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
                        SOCBEN_FAMSIT_2001_ANN(pers_j) = 5    
                        SOCBEN_FAMSIT_2001_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 5    
                        REST = (SOCBEN_NMLEEFL_ANN(pers_j) + SOCBEN_NMLEEFL_ANN(SOCBEN_PARTNID_ANN(pers_j))) &
                        &- SOCBEN_DISREGAR_ANN(1,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (REST.LT.SOCBEN_MINTYM65_ANN(1,2)) THEN
                            SOCBEN_FAMSIT_2001_ANN(pers_j) = 1    
                            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(1,2) - REST
                        END IF                            
                    !Is not married and not cohabitating with a partner but has children in charge
                    ELSE IF (FAMREL_COUPLE(pers_j).EQ.0.AND.SOCBEN_DEPCHILD_ANN(pers_j).GT.0) THEN
                        REST = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(2,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (REST.LT.SOCBEN_MINTYM65_ANN(2,2)) THEN
                            SOCBEN_FAMSIT_2001_ANN(pers_j) = 2    
                            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(2,2) - REST
                        END IF                            
                    !In all other cases the individual is cohabitating with other people
                    ELSE
                        REST = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(4,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (REST.LT.SOCBEN_MINTYM65_ANN(4,2)) THEN
                            SOCBEN_FAMSIT_2001_ANN(pers_j) = 4
                            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(4,2) - REST
                        END IF                            
                    END IF
                END IF
            END IF 
            !Adapt amount on annual basis in function of number of months that a benefit was received
            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_LEEFLOON_ANN(pers_j)*(NumMonth_Hulp_2008(pers_j)/12.D0)
        END IF    
    END DO

END SUBROUTINE SOCBEN_LEEFLOON_2001

!=============================================================================================================!
!Construction of
!SOCBEN_FAMSIT_2003_ANN                                           
!   = The family situation of the person entitled to a leefloon
!   SOCBEN_FAMSIT_2003_ANN = 1 a single parent
!   SOCBEN_FAMSIT_2003_ANN = 2 a single parent with no children in direct chrage but with children that are with him now and then or for which he pays alimony
!   SOCBEN_FAMSIT_2003_ANN = 3 a true single (= person living alone) 
!   SOCBEN_FAMSIT_2003_ANN = 4 a cohabitating person (either married or not)
!   SOCBEN_FAMSIT_2003_ANN = 5 a person who is married and on which a means test is applied but who does not receive the leefloon
!and
!SOCBEN_LEEFLOON_ANN                       
!   = Amount of minimum income transfer
!as applied in the period 2003-2004
!=============================================================================================================!
!Vanaf oktober 2002 tot en met augustus 2005
!    categorie A: samenwonenden (=twee gehuwden die voldoen aan toets krijgen allebei het bedrag van samenwonende)
!    categorie B: echte alleenstaande
!    categorie C: echte alleenstaande + (= alleenstaande met speciale voorwaarden inzake kinderlast)
!    categorie D: noudergezinnen (= alleenstaande met kinderen ten laste)
!toepassingsgebied in de tijd is niet heel duidelijk. Herziening wet zou ingang gevonden hebben
!op 1 januari 2005 maar op site POD MI geeft men een overzicht met bedragen die van toepassing blijven tot
!en met augustus 2005
!=============================================================================================================!

SUBROUTINE SOCBEN_LEEFLOON_2003(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j
    REAL(KIND=KIND_REAL)            :: REST,REST_J,REST_K    

    !Loop over all members of the sociological household
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                      
        !Check whether pers_j satisfies the age condition to receive a leefloon
        IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.1) THEN
            IF (SOCBEN_FAMSIT_2003_ANN(pers_j).EQ.0) THEN
                !Determine the family situation of the leefloon-receiver and the leefloon he receives
                !Either the individual lives alone
                !Take into account the overall disregard when determining the leefloon
                IF (SOCBEN_HH_SIZE(hh_i).EQ.1) THEN
                    REST = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(3,2)
                    IF (REST.LT.0.D0) THEN
                        REST = 0.D0
                    END IF
                    IF (REST.LT.SOCBEN_MINTYM65_ANN(3,2)) THEN
                        SOCBEN_FAMSIT_2003_ANN(pers_j) = 3
                        SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(3,2) - REST
                    END IF                        
                !or cohabits with other people
                ELSE
                    !Either the individual is married
                    !they now both are entitled to half of the total the couple was entitled before
                    IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
                        SOCBEN_FAMSIT_2003_ANN(pers_j) = 5    
                        SOCBEN_FAMSIT_2003_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 5    
                        !execute seperate test for j and k
                        REST_J = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(4,2)
                        IF (REST_J.LT.0.D0) THEN
                            REST_J = 0.D0
                        END IF
                        REST_K = SOCBEN_NMLEEFL_ANN(SOCBEN_PARTNID_ANN(pers_j)) - SOCBEN_DISREGAR_ANN(4,2)
                        IF (REST_K.LT.0.D0) THEN
                            REST_K = 0.D0
                        END IF
                        !Following three lines new conditions since 15/05/2010
                        !assumption is still joint test on income of two spouses
                        REST = (REST_J + REST_K)/2.D0
                        REST_J = REST
                        REST_K = REST
                        !Determine benefit
                        IF (REST_J.LT.SOCBEN_MINTYM65_ANN(4,2)) THEN
                            SOCBEN_FAMSIT_2003_ANN(pers_j) = 4    
                            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(4,2) - REST_J
                        END IF                            
                        IF (REST_K.LT.SOCBEN_MINTYM65_ANN(4,2)) THEN
                            SOCBEN_FAMSIT_2003_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 4    
                            SOCBEN_LEEFLOON_ANN(SOCBEN_PARTNID_ANN(pers_j)) = SOCBEN_MINTYM65_ANN(4,2) - REST_K
                        END IF                            
                    !Is not married and not cohabitating with a partner but has children in charge
                    ELSE IF (FAMREL_COUPLE(pers_j).EQ.0.AND.SOCBEN_DEPCHILD_ANN(pers_j).GT.0) THEN
                        REST = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(1,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (REST.LT.SOCBEN_MINTYM65_ANN(1,2)) THEN
                            SOCBEN_FAMSIT_2003_ANN(pers_j) = 1    
                            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(1,2) - REST
                        END IF                            
                    !Is not married or cohabitating with a partner, has no children in charge but has children that are with him now and then
                    !or for which he pays alimony
                    ELSE IF (FAMREL_COUPLE(pers_j).EQ.0.AND.SOCBEN_DEPCHILD_ANN(pers_j).EQ.0.AND.&
                    &SOCBEN_CHILDNOB_ANN(pers_j).GT.0) THEN
                        REST = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(2,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (REST.LT.SOCBEN_MINTYM65_ANN(2,2)) THEN
                            SOCBEN_FAMSIT_2003_ANN(pers_j) = 2    
                            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(2,2) - REST
                        END IF                            
                    !In all other cases the individual is cohabitating with other people
                    ELSE
                        REST = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(4,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (REST.LT.SOCBEN_MINTYM65_ANN(4,2)) THEN
                            SOCBEN_FAMSIT_2003_ANN(pers_j) = 4
                            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(4,2) - REST
                        END IF                            
                    END IF
                END IF
            END IF 
            !Adapt amount on annual basis in function of number of months that a benefit was received
            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_LEEFLOON_ANN(pers_j)*(NumMonth_Hulp_2008(pers_j)/12.D0)
        END IF    
    END DO

END SUBROUTINE SOCBEN_LEEFLOON_2003

!=============================================================================================================!
!Construction of
!SOCBEN_FAMSIT_2005_ANN                                           
!   = The family situation of the person entitled to a leefloon
!   SOCBEN_FAMSIT_2005_ANN = 1 a person cohabitating with a partner or a single parent with no children in direct chrage but with children that are with him now and then or for which he pays alimony
!   SOCBEN_FAMSIT_2005_ANN = 3 a true single (= person living alone) 
!   SOCBEN_FAMSIT_2005_ANN = 4 a cohabitating person (either married or not)
!   SOCBEN_FAMSIT_2005_ANN = 5 a person who is married or cohabitating and on which a means test is applied but who does not receive the leefloon
!and
!SOCBEN_LEEFLOON_ANN                       
!   = Amount of minimum income transfer
!as applied from January 2005 on
!=============================================================================================================!
!Vanaf 1 januari 2005 (vooral categorie 3 is van belang)
!    categorie 1: samenwonend (= persoon die samenwoont met anderen die niet leefloongerechtigd zijn) of gehuwden zonder kinderen ten laste
!    categorie 2: echte alleenstaande
!    categorie 3: samenwonende met gezinslast (= samenwonende of alleenstaande die voldoen aan de inkomenstoets) en kinderen ten laste hebben
!=============================================================================================================!

SUBROUTINE SOCBEN_LEEFLOON_2005(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j
    REAL(KIND=KIND_REAL)            :: REST   

    !Loop over all members of the sociological household
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                      
        !Check whether pers_j satisfies the age condition to receive a leefloon
        IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.1) THEN
            IF (SOCBEN_FAMSIT_2005_ANN(pers_j).EQ.0) THEN
                !Determine the family situation of the leefloon-receiver and the leefloon he receives
                !Either the individual lives alone
                !Take into account the overall disregard when determining the leefloon
                IF (SOCBEN_HH_SIZE(hh_i).EQ.1) THEN
                    REST = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(3,2)
                    IF (REST.LT.0.D0) THEN
                        REST = 0.D0
                    END IF
                    IF (REST.LT.SOCBEN_MINTYM65_ANN(3,2)) THEN
                        SOCBEN_FAMSIT_2005_ANN(pers_j) = 3
                        SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(3,2) - REST
                    END IF                        
                !or cohabits with other people
                ELSE
                    !Either the individual is married or cohabitating
                    !a single test is applied in this case, the other partner has to be excluded for further tests
                    IF (ANY(FAMREL_COUPLE(pers_j).EQ.(/1, 2/))) THEN
                        SOCBEN_FAMSIT_2005_ANN(pers_j) = 5    
                        SOCBEN_FAMSIT_2005_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 5
                        !execute separate test for j and k
                        REST = (SOCBEN_NMLEEFL_ANN(pers_j) + SOCBEN_NMLEEFL_ANN(SOCBEN_PARTNID_ANN(pers_j))) - &
                        &SOCBEN_DISREGAR_ANN(1,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        !either both partners are entitled to a right
                        IF (SOCBEN_ENTLEIGO_ANN(SOCBEN_PARTNID_ANN(pers_j),1).EQ.1) THEN
                            IF (REST.LT.SOCBEN_MINTYM65_ANN(1,2)) THEN
                                SOCBEN_FAMSIT_2005_ANN(pers_j) = 1    
                                SOCBEN_FAMSIT_2005_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 6
                                SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(1,2) - REST
                                SOCBEN_LEEFLOON_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 0
                            END IF                            
                        !or only one of them is entitled    
                        ELSE
                            IF (REST.LT.SOCBEN_MINTYM65_ANN(4,2)) THEN
                                SOCBEN_FAMSIT_2005_ANN(pers_j) = 4    
                                SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(4,2) - REST
                            END IF                            
                        END IF
                    !Is not married and not cohabitating with a partner but has children in charge
                    ELSE IF (FAMREL_COUPLE(pers_j).EQ.0.AND.SOCBEN_DEPCHILD_ANN(pers_j).GT.0) THEN
                        REST = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(1,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (REST.LT.SOCBEN_MINTYM65_ANN(1,2)) THEN
                            SOCBEN_FAMSIT_2005_ANN(pers_j) = 1    
                            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(1,2) - REST
                        END IF                            
                    !In all other cases the individual is cohabitating with other people
                    ELSE
                        REST = SOCBEN_NMLEEFL_ANN(pers_j) - SOCBEN_DISREGAR_ANN(4,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (REST.LT.SOCBEN_MINTYM65_ANN(4,2)) THEN
                            SOCBEN_FAMSIT_2005_ANN(pers_j) = 4
                            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_MINTYM65_ANN(4,2) - REST
                        END IF                            
                    END IF
                END IF
            END IF
            !Adapt amount on annual basis in function of number of months that a benefit was received
            SOCBEN_LEEFLOON_ANN(pers_j) = SOCBEN_LEEFLOON_ANN(pers_j)*(NumMonth_Hulp_2008(pers_j)/12.D0)
        END IF    
    END DO

END SUBROUTINE SOCBEN_LEEFLOON_2005

!=============================================================================================================!
!Construction of
!SOCBEN_FAMIGO_2001_ANN                                           
!   = The family situation of the person entitled to a leefloon
!   SOCBEN_FAMIGO_2001_ANN = 1 a single
!   SOCBEN_FAMIGO_2001_ANN = 2 a cohabitating person
!and
!SOCBEN_IGO_ANN                       
!   = Amount of minimum income transfer for elderly
!as applied in the period 2001-2009
!=============================================================================================================!

SUBROUTINE SOCBEN_IGO_2001(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j
    REAL(KIND=KIND_REAL)            :: REST    
    INTEGER                         :: INDIV_SICK,PART_SICK

    !Loop over all members of the sociological household
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i) 
        INDIV_SICK = 0
        PART_SICK = 0
        !Check whether pers_j satisfies the age condition to receive the IGO
        IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).GT.1) THEN
            IF (SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.0) THEN
                !If IGO
                IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.2) THEN
                    !Take into account the overall disregard when determining the IGO
                    !Either the individual lives alone or not
                    IF (SOCBEN_HH_SIZE(hh_i).EQ.1) THEN
                        REST = SOCBEN_NMIGO_ANN(pers_j) - SOCBEN_DISREIGO_ANN(1,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (REST.LT.SOCBEN_MINTYP65_ANN(1,2)) THEN
                            SOCBEN_FAMIGO_2001_ANN(pers_j) = 1
                            SOCBEN_IGO_ANN(pers_j) = SOCBEN_MINTYP65_ANN(1,2) - REST
                        END IF                        
                    !or cohabits with other people
                    ELSE
                        REST = SOCBEN_NMIGO_ANN(pers_j) - SOCBEN_DISREIGO_ANN(2,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (REST.LT.SOCBEN_MINTYP65_ANN(2,2)) THEN
                            SOCBEN_FAMIGO_2001_ANN(pers_j) = 2
                            SOCBEN_IGO_ANN(pers_j) = SOCBEN_MINTYP65_ANN(2,2) - REST
                        END IF                        
                    END IF
!                IF (SOCBEN_IGO_ANN(pers_j).EQ.0.D0.AND.SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.0) THEN                    
!                WRITE(15,'(A5,I10,2F15.2,I10,12F15.2)') 'IGO',GECODEERD_INSZ_INPUT(pers_j),SOCBEN_IGO_ANN(pers_j),&
!                &SOCBEN_NMIGO_ANN(pers_j),SOCBEN_FAMIGO_2001_ANN(pers_j),REST,&
!                &CONTRIB_GTIPRIV_ANN(pers_j),CONTRIB_GTIPUB_ANN(pers_j),CONTRIB_GTISELF_ANN(pers_j),CONTRIB_GTIPENS_ANN(pers_j),& 
!                &CONTRIB_GTIEARNE_ANN(pers_j),CONTRIB_GTIEAROL_ANN(pers_j),CONTRIB_GTIOLDUN_ANN(pers_j),&
!                &CONTRIB_GTIOTHUN_ANN(pers_j),CONTRIB_GTIDISAB_ANN(pers_j),CONTRIB_GTINDOCC_ANN(pers_j),CONTRIB_GTISICK_ANN(pers_j)
!                WRITE(15,*) '2delijn',FAMREL_COUPLE(pers_j),INDIV_SICK,PART_SICK
!                END IF
                !GWB: needs another household treatement    
                ELSE IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.3) THEN
                    !Either the individual is married or cohabitating
                    !a single test is applied in this case, the other partner has to be excluded for further tests
                    IF (ANY(FAMREL_COUPLE(pers_j).EQ.(/1, 2/))) THEN
                        !avoid that later a double test is executed
!                        SOCBEN_FAMIGO_2001_ANN(pers_j) = 5    
!                        SOCBEN_FAMIGO_2001_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 5
                        !Identify sickness status of individual and partner
                        IF (SICK_DISABBEN_QT(pers_j).GT.0.OR.INFODSZ(pers_j).GT.0) THEN
                            INDIV_SICK = 1
                        END IF                                
                        IF (SICK_DISABBEN_QT(SOCBEN_PARTNID_ANN(pers_j)).GT.0.OR.INFODSZ(SOCBEN_PARTNID_ANN(pers_j)).GT.0) THEN
                            PART_SICK = 1
                        END IF
                        !execute test
                        REST = (SOCBEN_NMIGO_ANN(pers_j) + SOCBEN_NMIGO_ANN(SOCBEN_PARTNID_ANN(pers_j))) - &
                        &SOCBEN_DISREIGO_ANN(1,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        !Make sure that when an earlier right has been awarded to the partner that this right is
                        !switched off
                        !distinguish sickness cases
                        IF (INDIV_SICK.EQ.0.AND.PART_SICK.EQ.0) THEN
                            IF (REST.LT.SOCBEN_GWB_ANN(1,2)) THEN
                                SOCBEN_FAMIGO_2001_ANN(pers_j) = 1    
                                SOCBEN_FAMIGO_2001_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 6
                                SOCBEN_IGO_ANN(pers_j) = SOCBEN_GWB_ANN(1,2) - REST
                                SOCBEN_IGO_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 0
                            END IF                            
                        ELSE IF ((INDIV_SICK.EQ.1.AND.PART_SICK.EQ.0).OR.(INDIV_SICK.EQ.0.AND.PART_SICK.EQ.1)) THEN
                            IF (REST.LT.SOCBEN_GWB_ANN(2,2)) THEN
                                SOCBEN_FAMIGO_2001_ANN(pers_j) = 1    
                                SOCBEN_FAMIGO_2001_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 6
                                SOCBEN_IGO_ANN(pers_j) = SOCBEN_GWB_ANN(2,2) - REST
                                SOCBEN_IGO_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 0
                            END IF                            
                        ELSE IF (INDIV_SICK.EQ.1.AND.PART_SICK.EQ.1) THEN
                            IF (REST.LT.SOCBEN_GWB_ANN(3,2)) THEN
                                SOCBEN_FAMIGO_2001_ANN(pers_j) = 1    
                                SOCBEN_FAMIGO_2001_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 6
                                SOCBEN_IGO_ANN(pers_j) = SOCBEN_GWB_ANN(3,2) - REST
                                SOCBEN_IGO_ANN(SOCBEN_PARTNID_ANN(pers_j)) = 0
                            END IF                            
                        END IF
                    !If not in couple, in household larger than 1 without labour income, the individual is also treated as head    
                    ELSE IF (FAMREL_COUPLE(pers_j).EQ.0.AND.SOCBEN_HH_SIZE(hh_i).GT.1.AND.SOCBEN_HHNOLAB_ANN(pers_j).EQ.0.D0) THEN
                        !Identify sickness status of individual
                        IF (SICK_DISABBEN_QT(pers_j).GT.0.OR.INFODSZ(pers_j).GT.0) THEN
                            INDIV_SICK = 1
                        END IF                                
                        REST = SOCBEN_NMIGO_ANN(pers_j) - SOCBEN_DISREIGO_ANN(1,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (INDIV_SICK.EQ.0) THEN
                            IF (REST.LT.SOCBEN_GWB_ANN(1,2)) THEN
                                SOCBEN_FAMIGO_2001_ANN(pers_j) = 2
                                SOCBEN_IGO_ANN(pers_j) = SOCBEN_GWB_ANN(1,2) - REST
                            END IF                        
                        ELSE IF (INDIV_SICK.EQ.1) THEN
                            IF (REST.LT.SOCBEN_GWB_ANN(2,2)) THEN
                                SOCBEN_FAMIGO_2001_ANN(pers_j) = 2
                                SOCBEN_IGO_ANN(pers_j) = SOCBEN_GWB_ANN(2,2) - REST
                            END IF                        
                        END IF
                    !individual is entitled as single
                    ELSE
                        !Identify sickness status of individual
                        IF (SICK_DISABBEN_QT(pers_j).GT.0.OR.INFODSZ(pers_j).GT.0) THEN
                            INDIV_SICK = 1
                        END IF                                
                        REST = SOCBEN_NMIGO_ANN(pers_j) - SOCBEN_DISREIGO_ANN(2,2)
                        IF (REST.LT.0.D0) THEN
                            REST = 0.D0
                        END IF
                        IF (INDIV_SICK.EQ.0) THEN
                            IF (REST.LT.SOCBEN_GWB_ANN(4,2)) THEN
                                SOCBEN_FAMIGO_2001_ANN(pers_j) = 3
                                SOCBEN_IGO_ANN(pers_j) = SOCBEN_GWB_ANN(4,2) - REST
                            END IF                        
                        ELSE IF (INDIV_SICK.EQ.1) THEN
                            IF (REST.LT.SOCBEN_GWB_ANN(5,2)) THEN
                                SOCBEN_FAMIGO_2001_ANN(pers_j) = 3
                                SOCBEN_IGO_ANN(pers_j) = SOCBEN_GWB_ANN(5,2) - REST
                            END IF                        
                        END IF
                    END IF
!                IF (SOCBEN_IGO_ANN(pers_j).EQ.0.D0.AND.SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.0) THEN                    
!                WRITE(15,'(A5,I10,2F15.0,I10,12F15.0)') 'GWB',GECODEERD_INSZ_INPUT(pers_j),SOCBEN_IGO_ANN(pers_j),&
!                &SOCBEN_NMIGO_ANN(pers_j),SOCBEN_FAMIGO_2001_ANN(pers_j),REST,&
!                &CONTRIB_GTIPRIV_ANN(pers_j),CONTRIB_GTIPUB_ANN(pers_j),CONTRIB_GTISELF_ANN(pers_j),CONTRIB_GTIPENS_ANN(pers_j),& 
!                &CONTRIB_GTIEARNE_ANN(pers_j),CONTRIB_GTIEAROL_ANN(pers_j),CONTRIB_GTIOLDUN_ANN(pers_j),&
!                &CONTRIB_GTIOTHUN_ANN(pers_j),CONTRIB_GTIDISAB_ANN(pers_j),CONTRIB_GTINDOCC_ANN(pers_j),CONTRIB_GTISICK_ANN(pers_j)
!                WRITE(15,*) '2delijn',FAMREL_COUPLE(pers_j),INDIV_SICK,PART_SICK
!                END IF 
                END IF 
            END IF    
!            !If benefit was received but not indicated on the basis of the test, then attribute the benefit as fixed amount
!            IF (SOCBEN_IGO_ANN(pers_j).EQ.0.D0.AND.IGO_2008(pers_j).GT.0.D0) THEN
!                SOCBEN_IGO_ANN(pers_j) = IGO_2008(pers_j)
!            END IF
            !Adapt amount on annual basis in function of number of months that a benefit was received
            SOCBEN_IGO_ANN(pers_j) = SOCBEN_IGO_ANN(pers_j)*(SOCBEN_IGO_NMONTHS(pers_j)/12.D0)
        END IF    
    END DO

END SUBROUTINE SOCBEN_IGO_2001

!=============================================================================================================!
!Construction of
!SOCBEN_IVT_ANN                       
!   = Amount of IVT attributed to person with a disability
!SOCBEN_IT_ANN                        
!   = Amount of IT attributed to person with a disability
!according to the rules that applied from 2001 onwards
!=============================================================================================================!

SUBROUTINE SOCBEN_IVTIT_2001(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j

    !Loop over all members of the sociological household
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                      
        !Compute IVT amount
        IF (SOCBEN_ENTIVT_ANN(pers_j).EQ.1.AND.SOCBEN_MEDIVT_ANN(pers_j).LE.33.D0) THEN
            IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.1) THEN
                SOCBEN_IVT_ANN(pers_j) = SOCBEN_BAMIVT_ANN(1,2) - SOCBEN_MEANSIVT_2001_ANN(pers_j)
            ELSE IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.2) THEN
                SOCBEN_IVT_ANN(pers_j) = SOCBEN_BAMIVT_ANN(2,2) - SOCBEN_MEANSIVT_2001_ANN(pers_j)
            ELSE IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.3) THEN
                SOCBEN_IVT_ANN(pers_j) = SOCBEN_BAMIVT_ANN(3,2) - SOCBEN_MEANSIVT_2001_ANN(pers_j)
            END IF        
            IF (SOCBEN_IVT_ANN(pers_j).LT.0.D0) THEN
                SOCBEN_IVT_ANN(pers_j) = 0.D0
            END IF
        END IF    
        !Compute IT amount
        IF (SOCBEN_ENTIT_ANN(pers_j).EQ.1.AND.ANY(SOCBEN_MEDIT_ANN(pers_j).EQ.(/1, 2, 3, 4, 5/))) THEN
            IF (SOCBEN_MEDIT_ANN(pers_j).EQ.1) THEN
                SOCBEN_IT_ANN(pers_j) = SOCBEN_BAMIT_ANN(1,2) - SOCBEN_MEANSIT_2001_ANN(pers_j)
            ELSE IF (SOCBEN_MEDIT_ANN(pers_j).EQ.2) THEN
                SOCBEN_IT_ANN(pers_j) = SOCBEN_BAMIT_ANN(2,2) - SOCBEN_MEANSIT_2001_ANN(pers_j)
            ELSE IF (SOCBEN_MEDIT_ANN(pers_j).EQ.3) THEN
                SOCBEN_IT_ANN(pers_j) = SOCBEN_BAMIT_ANN(3,2) - SOCBEN_MEANSIT_2001_ANN(pers_j)
            ELSE IF (SOCBEN_MEDIT_ANN(pers_j).EQ.4) THEN
                SOCBEN_IT_ANN(pers_j) = SOCBEN_BAMIT_ANN(4,2) - SOCBEN_MEANSIT_2001_ANN(pers_j)
            ELSE IF (SOCBEN_MEDIT_ANN(pers_j).EQ.5) THEN
                SOCBEN_IT_ANN(pers_j) = SOCBEN_BAMIT_ANN(5,2) - SOCBEN_MEANSIT_2001_ANN(pers_j)
            END IF        
            IF (SOCBEN_IT_ANN(pers_j).LT.0.D0) THEN
                SOCBEN_IT_ANN(pers_j) = 0.D0
            END IF
        END IF
        !Adapt amount on annual basis in function of number of months that a benefit was received
        SOCBEN_IVT_ANN(pers_j) = SOCBEN_IVT_ANN(pers_j)*(DBLE(SOCBEN_IVT_NMONTHS(pers_j))/12.D0)
        SOCBEN_IT_ANN(pers_j) = SOCBEN_IT_ANN(pers_j)*(DBLE(SOCBEN_IT_NMONTHS(pers_j))/12.D0)
    END DO

END SUBROUTINE SOCBEN_IVTIT_2001

!=============================================================================================================!
!Construction of
!SOCBEN_IVT_ANN                       
!   = Amount of IVT attributed to person with a disability
!SOCBEN_IT_ANN                        
!   = Amount of IT attributed to person with a disability
!according to the rules that applied from 2004 onwards
!=============================================================================================================!

SUBROUTINE SOCBEN_IVTIT_2004(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j

    !Loop over all members of the sociological household
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                      
        !Compute IVT amount
        IF (SOCBEN_ENTIVT_ANN(pers_j).EQ.1.AND.SOCBEN_MEDIVT_ANN(pers_j).LE.33.D0) THEN
!            WRITE(15,'(A3,10x,F15.0,A7,I10,A7)') 'IVT',SOCBEN_MEANSIVT_2004_ANN(pers_j),begindatum_pensioen_c_2008(pers_j)&
!            &,GECODEERD_INSZ_INPUT(pers_j),date_start_2008(pers_j)
            IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.1) THEN
                SOCBEN_IVT_ANN(pers_j) = SOCBEN_BAMIVT_ANN(1,2) - SOCBEN_MEANSIVT_2004_ANN(pers_j)
            ELSE IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.2) THEN
                SOCBEN_IVT_ANN(pers_j) = SOCBEN_BAMIVT_ANN(2,2) - SOCBEN_MEANSIVT_2004_ANN(pers_j)
            ELSE IF (SOCBEN_FAMITIVT_ANN(pers_j).EQ.3) THEN
                SOCBEN_IVT_ANN(pers_j) = SOCBEN_BAMIVT_ANN(3,2) - SOCBEN_MEANSIVT_2004_ANN(pers_j)
            END IF        
            IF (SOCBEN_IVT_ANN(pers_j).LT.0.D0) THEN
                SOCBEN_IVT_ANN(pers_j) = 0.D0
            END IF
        END IF    
        !Compute IT amount
        IF (SOCBEN_ENTIT_ANN(pers_j).EQ.1.AND.ANY(SOCBEN_MEDIT_ANN(pers_j).EQ.(/1, 2, 3, 4, 5/))) THEN
!            WRITE(15,'(A3,I10,F15.0,A7,I10,A7)') ' IT',SOCBEN_MEDIT_ANN(pers_j),SOCBEN_MEANSIT_2004_ANN(pers_j),&
!            &begindatum_pensioen_c_2008(pers_j),GECODEERD_INSZ_INPUT(pers_j),date_start_2008(pers_j)
            IF (SOCBEN_MEDIT_ANN(pers_j).EQ.1) THEN
                SOCBEN_IT_ANN(pers_j) = SOCBEN_BAMIT_ANN(1,2) - SOCBEN_MEANSIT_2004_ANN(pers_j)
            ELSE IF (SOCBEN_MEDIT_ANN(pers_j).EQ.2) THEN
                SOCBEN_IT_ANN(pers_j) = SOCBEN_BAMIT_ANN(2,2) - SOCBEN_MEANSIT_2004_ANN(pers_j)
            ELSE IF (SOCBEN_MEDIT_ANN(pers_j).EQ.3) THEN
                SOCBEN_IT_ANN(pers_j) = SOCBEN_BAMIT_ANN(3,2) - SOCBEN_MEANSIT_2004_ANN(pers_j)
            ELSE IF (SOCBEN_MEDIT_ANN(pers_j).EQ.4) THEN
                SOCBEN_IT_ANN(pers_j) = SOCBEN_BAMIT_ANN(4,2) - SOCBEN_MEANSIT_2004_ANN(pers_j)
            ELSE IF (SOCBEN_MEDIT_ANN(pers_j).EQ.5) THEN
                SOCBEN_IT_ANN(pers_j) = SOCBEN_BAMIT_ANN(5,2) - SOCBEN_MEANSIT_2004_ANN(pers_j)
            END IF        
            IF (SOCBEN_IT_ANN(pers_j).LT.0.D0) THEN
                SOCBEN_IT_ANN(pers_j) = 0.D0
            END IF
        END IF
        !Adapt amount on annual basis in function of number of months that a benefit was received
        SOCBEN_IVT_ANN(pers_j) = SOCBEN_IVT_ANN(pers_j)*(DBLE(SOCBEN_IVT_NMONTHS(pers_j))/12.D0)
        SOCBEN_IT_ANN(pers_j) = SOCBEN_IT_ANN(pers_j)*(DBLE(SOCBEN_IT_NMONTHS(pers_j))/12.D0)
    END DO

END SUBROUTINE SOCBEN_IVTIT_2004

!=============================================================================================================!
!Construction of
!SOCBEN_THB_ANN                       
!   = Amount of THB attributed to person with a disability
!=============================================================================================================!

SUBROUTINE SOCBEN_THB_2001(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j

    !Loop over all members of the sociological household
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                      
        !Compute THB amount
        IF (ANY(SOCBEN_MEDTHB_ANN(pers_j).EQ.(/1, 2, 3, 4, 5/))) THEN
!            WRITE(15,'(A3,I10,F15.0,A7,I10)') 'THB',SOCBEN_MEDTHB_ANN(pers_j),SOCBEN_MEANSTHB_ANN(pers_j),&
!            &begindatum_pensioen_c_2008(pers_j),GECODEERD_INSZ_INPUT(pers_j)
            IF (SOCBEN_MEDTHB_ANN(pers_j).EQ.1) THEN
                SOCBEN_THB_ANN(pers_j) = SOCBEN_BAMTHB_ANN(1,2) - SOCBEN_MEANSTHB_ANN(pers_j)
            ELSE IF (SOCBEN_MEDTHB_ANN(pers_j).EQ.2) THEN
                SOCBEN_THB_ANN(pers_j) = SOCBEN_BAMTHB_ANN(2,2) - SOCBEN_MEANSTHB_ANN(pers_j)
            ELSE IF (SOCBEN_MEDTHB_ANN(pers_j).EQ.3) THEN
                SOCBEN_THB_ANN(pers_j) = SOCBEN_BAMTHB_ANN(3,2) - SOCBEN_MEANSTHB_ANN(pers_j)
            ELSE IF (SOCBEN_MEDTHB_ANN(pers_j).EQ.4) THEN
                SOCBEN_THB_ANN(pers_j) = SOCBEN_BAMTHB_ANN(4,2) - SOCBEN_MEANSTHB_ANN(pers_j)
            ELSE IF (SOCBEN_MEDTHB_ANN(pers_j).EQ.5) THEN
                SOCBEN_THB_ANN(pers_j) = SOCBEN_BAMTHB_ANN(5,2) - SOCBEN_MEANSTHB_ANN(pers_j)
            END IF        
            IF (SOCBEN_THB_ANN(pers_j).LT.0.D0) THEN
                SOCBEN_THB_ANN(pers_j) = 0.D0
            END IF
        END IF
        !Adapt amount on annual basis in function of number of months that a benefit was received
        SOCBEN_THB_ANN(pers_j) = SOCBEN_THB_ANN(pers_j)*(DBLE(SOCBEN_THB_NMONTHS(pers_j))/12.D0)
    END DO

END SUBROUTINE SOCBEN_THB_2001

!=============================================================================================================!
!Construction of
!  SOCBEN_VT_ANN
!  SOCBEN_OMNIOSTA_ANN
!  SOCBEN_INCMAFST_ANN
!=============================================================================================================!

SUBROUTINE SOCBEN_HEALTBEN_2008(hh_i)

    INTEGER                         :: hh_i
    INTEGER                         :: pers_j,pers_k,lijn_j,DEPCHILD
    REAL(KIND=KIND_REAL)            :: SUMHH_INC,HHTHRESHOLD    

    !SOCBEN_VT_ANN                        !Individual categorie for VT benefit
    !Loop over all members of the sociological household
    SUMHH_INC = 0.D0
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
        !Check whether an income test has already been executed
        IF (ANY(SOCBEN_VTSTAT_ANN(pers_j).EQ.(/1, 2, 3, 4/))) THEN
            SOCBEN_VT_ANN(pers_j) = 1
            !Loop over all members of the family and identify the possible partner and children in charge of the individual
            !Determine the line in the parameter SOCBEN_FAMREL_ANN to convert relationship codes
            DO i = 1,SIZE(SOCBEN_FAMREL_ANN,1)
                IF (FAMREL_RELATION(pers_j).EQ.SOCBEN_FAMREL_ANN(i,1)) THEN
                    lijn_j = i
                END IF
            END DO
            DO pers_k=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
                IF (pers_j.NE.pers_k) THEN
                    !the partner and the children in charge are determined on the level of the family
                    IF (FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                        DO i = 1,SIZE(SOCBEN_FAMREL_ANN,2)
                            IF (FAMREL_RELATION(pers_k).EQ.SOCBEN_FAMREL_ANN(1,i)) THEN
                                !Count the number of dependent children (dependence is determined by the fact that the child generates child benefits
                                IF (SOCBEN_FAMREL_ANN(lijn_j,i).EQ.2.AND.FAMAL_CHILD_ELIG(pers_k).EQ.1.AND.&
                                &SOCBEN_VT_ANN(pers_k).EQ.0) THEN
                                    SOCBEN_VT_ANN(pers_k) = 2
                                END IF
                                !If there is a partner, give him the right
                                IF (SOCBEN_FAMREL_ANN(lijn_j,i).EQ.1.AND.SOCBEN_VT_ANN(pers_k).EQ.0) THEN
                                    SOCBEN_VT_ANN(pers_k) = 2
                                END IF
                            END IF
                        END DO
                    END IF
                END IF                    
            END DO
        !if no income test executed but social status that generates right, execute test            
        ELSE IF (ANY(SOCBEN_VTSTAT_ANN(pers_j).EQ.(/5, 6, 7, 8, 9, 10/)).AND.SOCBEN_VT_ANN(pers_j).EQ.0) THEN
            SUMHH_INC = SOCBEN_MEVT_ANN(pers_j)
            DEPCHILD = 0
            !Loop over all members of the family and identify the possible partner and children in charge of the individual
            !Determine the line in the parameter SOCBEN_FAMREL_ANN to convert relationship codes
            DO i = 1,SIZE(SOCBEN_FAMREL_ANN,1)
                IF (FAMREL_RELATION(pers_j).EQ.SOCBEN_FAMREL_ANN(i,1)) THEN
                    lijn_j = i
                END IF
            END DO
            DO pers_k=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
                IF (pers_j.NE.pers_k) THEN
                    !the partner and the children in charge are determined on the level of the family
                    IF (FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                        DO i = 1,SIZE(SOCBEN_FAMREL_ANN,2)
                            IF (FAMREL_RELATION(pers_k).EQ.SOCBEN_FAMREL_ANN(1,i)) THEN
                                !Count the number of dependent children (dependence is determined by the fact that the child generates child benefits
                                IF (SOCBEN_FAMREL_ANN(lijn_j,i).EQ.2.AND.FAMAL_CHILD_ELIG(pers_k).EQ.1.AND.&
                                &SOCBEN_VT_ANN(pers_k).EQ.0) THEN
                                    DEPCHILD  = DEPCHILD + 1
                                    SUMHH_INC = SUMHH_INC + SOCBEN_MEVT_ANN(pers_k)
                                END IF
                                !If there is a partner, give him the right
                                IF (SOCBEN_FAMREL_ANN(lijn_j,i).EQ.1.AND.SOCBEN_VT_ANN(pers_k).EQ.0) THEN
                                    SUMHH_INC = SUMHH_INC + SOCBEN_MEVT_ANN(pers_k)
                                END IF
                            END IF
                        END DO
                    END IF
                END IF                    
            END DO
            !check whether household threshold is exceeded or not
            HHTHRESHOLD = SOCBEN_VT_BASICD_ANN + DEPCHILD*SOCBEN_VT_CHILDD_ANN
            IF (SUMHH_INC.LT.HHTHRESHOLD) THEN
                SOCBEN_VT_ANN(pers_j) = 3
                !Loop over all members of the family and identify the possible partner and children in charge of the individual
                !Determine the line in the parameter SOCBEN_FAMREL_ANN to convert relationship codes
                DO i = 1,SIZE(SOCBEN_FAMREL_ANN,1)
                    IF (FAMREL_RELATION(pers_j).EQ.SOCBEN_FAMREL_ANN(i,1)) THEN
                        lijn_j = i
                    END IF
                END DO
                DO pers_k=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
                    IF (pers_j.NE.pers_k) THEN
                        !the partner and the children in charge are determined on the level of the family
                        IF (FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                            DO i = 1,SIZE(SOCBEN_FAMREL_ANN,2)
                                IF (FAMREL_RELATION(pers_k).EQ.SOCBEN_FAMREL_ANN(1,i)) THEN
                                    !Count the number of dependent children (dependence is determined by the fact that the child generates child benefits
                                    IF (SOCBEN_FAMREL_ANN(lijn_j,i).EQ.2.AND.FAMAL_CHILD_ELIG(pers_k).EQ.1.AND.&
                                    &SOCBEN_VT_ANN(pers_k).EQ.0) THEN
                                        SOCBEN_VT_ANN(pers_k) = 4
                                    END IF
                                    !If there is a partner, give him the right
                                    IF (SOCBEN_FAMREL_ANN(lijn_j,i).EQ.1.AND.SOCBEN_VT_ANN(pers_k).EQ.0) THEN
                                        SOCBEN_VT_ANN(pers_k) = 4
                                    END IF
                                END IF
                            END DO
                        END IF
                    END IF                    
                END DO
            END IF
        END IF
    END DO    

    !SOCBEN_OMNIOSTA_ANN                  !Individual categorie for OMNIO benefit
    !Determine the household threshold and household income
    SUMHH_INC = 0.D0
    IF (SOCBEN_HH_SIZE(hh_i).EQ.1) THEN
        HHTHRESHOLD = SOCBEN_OMNIO_BASICD_ANN
        SUMHH_INC = SOCBEN_MEOMNIO_ANN(pers_j)
    ELSE
        HHTHRESHOLD = SOCBEN_OMNIO_BASICD_ANN + (SOCBEN_HH_SIZE(hh_i)-1)*SOCBEN_OMNIO_CHILDD_ANN
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
            SUMHH_INC = SUMHH_INC + SOCBEN_MEOMNIO_ANN(pers_j)
        END DO    
    END IF
    IF (SUMHH_INC.LT.HHTHRESHOLD) THEN
        !Loop over all members of the sociological household
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)
            SOCBEN_OMNIOSTA_ANN(pers_j) = 1                  
        END DO
    END IF        

    !SOCBEN_INCMAFST_ANN                  !Individual categorie for Income MAF benefit (setting the threshold for out of pocket payments)
    SUMHH_INC = 0.D0
    IF (SOCBEN_HH_SIZE(hh_i).EQ.1) THEN
        SUMHH_INC = SOCBEN_MEINCMAF_ANN(pers_j)
    ELSE
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
            SUMHH_INC = SUMHH_INC + SOCBEN_MEINCMAF_ANN(pers_j)
        END DO    
    END IF
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        SOCBEN_INCMAFST_ANN(pers_j) = SOCBEN_BRACAMOU_3COLUMNS(SUMHH_INC,SIZE(SOCBEN_INCMAF_ANN,1),&
        &SOCBEN_INCMAF_ANN,pers_j)
    END DO    

    !SOCBEN_SOCMAFST_ANN                  !Individual categorie for Social MAF benefit
    DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)   
        IF (ANY(SOCBEN_VT_ANN(pers_j).EQ.(/1, 3/))) THEN
            SOCBEN_SOCMAFST_ANN(pers_j) = 1
        ELSE IF (ANY(SOCBEN_VT_ANN(pers_j).EQ.(/2, 4/))) THEN
            SOCBEN_SOCMAFST_ANN(pers_j) = 2
        END IF
    END DO    
    
END SUBROUTINE SOCBEN_HEALTBEN_2008

!======================================!
!Calculate some aggregate test results
!======================================!
SUBROUTINE SOCBEN_TEST

    INTEGER                                                 :: i,pers_j 
    REAL(KIND=DPRECI),DIMENSION(1)                          :: SumMINIMEX                           !Vector with annual amounts on minimexes
    INTEGER                                                 :: NumAnal                              !Number of summary statistics computed 
    REAL(KIND=DPRECI)                                       :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: SOCBEN_RESULT_TEST                  
    INTEGER                                                 :: TestID

    !Set the number of statistics to be computed
    NumAnal = 1
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(SOCBEN_RESULT_TEST(NumAnal,4))
    SOCBEN_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Compute aggregate statistics
    DO hh_i=1,N_HH_SOCBEN
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                                                                  
            SumMINIMEX(1:NumAnal)=0.D0
            !Compute quarterly amounts
            SumMINIMEX(1) = SOCBEN_LEEFLOON_ANN(pers_j) + SOCBEN_IGO_ANN(pers_j)
            DO j=1,NumAnal
                IF (INT(SumMINIMEX(j)*100.D0).NE.0) THEN
                    SOCBEN_RESULT_TEST(j,1)=SOCBEN_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumMINIMEX(j)
                    SOCBEN_RESULT_TEST(j,2)=SOCBEN_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
                END IF
            END DO
        END DO
    END DO
    
    !Write aggregate statistics to log-file
    DO i=1,NumAnal
        IF (INT(SOCBEN_RESULT_TEST(i,2)).NE.0) THEN
            SOCBEN_RESULT_TEST(i,3)=SOCBEN_RESULT_TEST(i,1)/SOCBEN_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SOCBEN_RESULT_TEST(i,4)=100.D0*SOCBEN_RESULT_TEST(i,2)/SumW
        END IF        
    END DO
    Write(15,'(/,''                                                                                   AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    write(15,'(''level of annual guaranteed incomes                                        '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(1,j),j=1,4)    

    DEALLOCATE(SOCBEN_RESULT_TEST)

    !Set the number of statistics to be computed
    NumAnal = 6
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(SOCBEN_RESULT_TEST(NumAnal,4))
    SOCBEN_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Compute aggregate statistics
    DO hh_i=1,N_HH_SOCBEN
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                                                                  
            !Leefloon
            IF (INT(SOCBEN_LEEFLOON_ANN(pers_j)*100.D0).NE.0.OR.(INT(SOCBEN_LEEFLOON_ANN(pers_j)*100.D0).EQ.0.&
            &AND.SOCBEN_FAMSIT_2005_ANN(pers_j).EQ.6)) THEN
                SOCBEN_RESULT_TEST(1,1)=SOCBEN_RESULT_TEST(1,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_LEEFLOON_ANN(pers_j)
                SOCBEN_RESULT_TEST(1,2)=SOCBEN_RESULT_TEST(1,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IGO
            !Tel gevallen die benefit als koppel krijgen ook mee
            IF (INT(SOCBEN_IGO_ANN(pers_j)*100.D0).NE.0.OR.(INT(SOCBEN_IGO_ANN(pers_j)*100.D0).EQ.0.&
            &AND.SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.6)) THEN
                SOCBEN_RESULT_TEST(2,1)=SOCBEN_RESULT_TEST(2,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_IGO_ANN(pers_j)
                SOCBEN_RESULT_TEST(2,2)=SOCBEN_RESULT_TEST(2,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IVT
            IF (INT(SOCBEN_IVT_ANN(pers_j)*100.D0).NE.0.AND.INT(SOCBEN_IT_ANN(pers_j)*100.D0).EQ.0) THEN
                SOCBEN_RESULT_TEST(3,1)=SOCBEN_RESULT_TEST(3,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_IVT_ANN(pers_j)
                SOCBEN_RESULT_TEST(3,2)=SOCBEN_RESULT_TEST(3,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IT
            IF (INT(SOCBEN_IVT_ANN(pers_j)*100.D0).EQ.0.AND.INT(SOCBEN_IT_ANN(pers_j)*100.D0).NE.0) THEN
                SOCBEN_RESULT_TEST(4,1)=SOCBEN_RESULT_TEST(4,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_IT_ANN(pers_j)
                SOCBEN_RESULT_TEST(4,2)=SOCBEN_RESULT_TEST(4,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IT/IVT
            IF (INT(SOCBEN_IVT_ANN(pers_j)*100.D0).NE.0.AND.INT(SOCBEN_IT_ANN(pers_j)*100.D0).NE.0) THEN
                SOCBEN_RESULT_TEST(5,1)=SOCBEN_RESULT_TEST(5,1)+MIMOSIS_WEIGHT(pers_j)*&
                &(SOCBEN_IT_ANN(pers_j)+SOCBEN_IVT_ANN(pers_j))
                SOCBEN_RESULT_TEST(5,2)=SOCBEN_RESULT_TEST(5,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !THB
            IF (INT(SOCBEN_THB_ANN(pers_j)*100.D0).NE.0) THEN
                SOCBEN_RESULT_TEST(6,1)=SOCBEN_RESULT_TEST(6,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_THB_ANN(pers_j)
                SOCBEN_RESULT_TEST(6,2)=SOCBEN_RESULT_TEST(6,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO
    
    !Write aggregate statistics to log-file
    DO i=1,NumAnal
        IF (INT(SOCBEN_RESULT_TEST(i,2)).NE.0) THEN
            SOCBEN_RESULT_TEST(i,3)=SOCBEN_RESULT_TEST(i,1)/SOCBEN_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SOCBEN_RESULT_TEST(i,4)=100.D0*SOCBEN_RESULT_TEST(i,2)/SumW
        END IF        
    END DO
    Write(15,'(/,''                                                                                   AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    write(15,'(''level of annual guaranteed incomes for those receiving leefloon          '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(1,j),j=1,4)    
    write(15,'(''level of annual guaranteed incomes for those receiving IGO               '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(2,j),j=1,4)    
    write(15,'(''level of annual guaranteed incomes for those receiving IVT               '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(3,j),j=1,4)    
    write(15,'(''level of annual guaranteed incomes for those receiving IT                '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(4,j),j=1,4)    
    write(15,'(''level of annual guaranteed incomes for those receiving IT/IVT            '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(5,j),j=1,4)    
    write(15,'(''level of annual guaranteed incomes for those receiving THB               '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(6,j),j=1,4)    

    DEALLOCATE(SOCBEN_RESULT_TEST)

    !Leefloon naar gezinsstatuut
    NumAnal = 6
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(SOCBEN_RESULT_TEST(NumAnal,5))
    SOCBEN_RESULT_TEST(1:NumAnal,1:5)=0.D0

    !Compute aggregate statistics
    DO hh_i=1,N_HH_SOCBEN
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                                                                  
            !Leefloon
            IF (SOCBEN_FAMSIT_2005_ANN(pers_j).EQ.0) THEN
                SOCBEN_RESULT_TEST(1,1)=SOCBEN_RESULT_TEST(1,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_LEEFLOON_ANN(pers_j)
                SOCBEN_RESULT_TEST(1,2)=SOCBEN_RESULT_TEST(1,2)+MIMOSIS_WEIGHT(pers_j)
            !tel gevallen die behandeld werden als koppel
            ELSE IF (SOCBEN_FAMSIT_2005_ANN(pers_j).EQ.1.OR.SOCBEN_FAMSIT_2005_ANN(pers_j).EQ.6) THEN
                SOCBEN_RESULT_TEST(2,1)=SOCBEN_RESULT_TEST(2,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_LEEFLOON_ANN(pers_j)
                SOCBEN_RESULT_TEST(2,2)=SOCBEN_RESULT_TEST(2,2)+MIMOSIS_WEIGHT(pers_j)
                !subtract 10 from base amount = suboptimal procedure to deal with rounding errors from the treatement of the number of months a benefit was paid
                IF (NumMonth_Hulp_2008(pers_j).GT.0) THEN
                    IF (((SOCBEN_LEEFLOON_ANN(pers_j)*12.D0)/(DBLE(NumMonth_Hulp_2008(pers_j)))).&
                    &LT.(SOCBEN_MINTYM65_ANN(1,2)-10.D0)) THEN
                        SOCBEN_RESULT_TEST(2,5)=SOCBEN_RESULT_TEST(2,5)+MIMOSIS_WEIGHT(pers_j)
                    END IF
                END IF
            ELSE IF (SOCBEN_FAMSIT_2005_ANN(pers_j).EQ.2) THEN
                SOCBEN_RESULT_TEST(3,1)=SOCBEN_RESULT_TEST(3,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_LEEFLOON_ANN(pers_j)
                SOCBEN_RESULT_TEST(3,2)=SOCBEN_RESULT_TEST(3,2)+MIMOSIS_WEIGHT(pers_j)
            ELSE IF (SOCBEN_FAMSIT_2005_ANN(pers_j).EQ.3) THEN
                SOCBEN_RESULT_TEST(4,1)=SOCBEN_RESULT_TEST(4,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_LEEFLOON_ANN(pers_j)
                SOCBEN_RESULT_TEST(4,2)=SOCBEN_RESULT_TEST(4,2)+MIMOSIS_WEIGHT(pers_j)
                !subtract 10 from base amount = suboptimal procedure to deal with rounding errors from the treatement of the number of months a benefit was paid
                IF (NumMonth_Hulp_2008(pers_j).GT.0) THEN
                    IF (((SOCBEN_LEEFLOON_ANN(pers_j)*12.D0)/(DBLE(NumMonth_Hulp_2008(pers_j)))).&
                    &LT.(SOCBEN_MINTYM65_ANN(3,2)-10.D0)) THEN
                        SOCBEN_RESULT_TEST(4,5)=SOCBEN_RESULT_TEST(4,5)+MIMOSIS_WEIGHT(pers_j)
                    END IF
                END IF
            ELSE IF (SOCBEN_FAMSIT_2005_ANN(pers_j).EQ.4) THEN
                SOCBEN_RESULT_TEST(5,1)=SOCBEN_RESULT_TEST(5,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_LEEFLOON_ANN(pers_j)
                SOCBEN_RESULT_TEST(5,2)=SOCBEN_RESULT_TEST(5,2)+MIMOSIS_WEIGHT(pers_j)
                !subtract 10 from base amount = suboptimal procedure to deal with rounding errors from the treatement of the number of months a benefit was paid
                IF (NumMonth_Hulp_2008(pers_j).GT.0) THEN
                    IF (((SOCBEN_LEEFLOON_ANN(pers_j)*12.D0)/(DBLE(NumMonth_Hulp_2008(pers_j)))).&
                    &LT.(SOCBEN_MINTYM65_ANN(4,2)-10.D0)) THEN
                        SOCBEN_RESULT_TEST(5,5)=SOCBEN_RESULT_TEST(5,5)+MIMOSIS_WEIGHT(pers_j)
                    END IF
                END IF
            ELSE IF (SOCBEN_FAMSIT_2005_ANN(pers_j).EQ.5) THEN
                SOCBEN_RESULT_TEST(6,1)=SOCBEN_RESULT_TEST(6,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_LEEFLOON_ANN(pers_j)
                SOCBEN_RESULT_TEST(6,2)=SOCBEN_RESULT_TEST(6,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO
    
    !Write aggregate statistics to log-file
    DO i=1,NumAnal
        IF (INT(SOCBEN_RESULT_TEST(i,2)).NE.0) THEN
            SOCBEN_RESULT_TEST(i,3)=SOCBEN_RESULT_TEST(i,1)/SOCBEN_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SOCBEN_RESULT_TEST(i,4)=100.D0*SOCBEN_RESULT_TEST(i,2)/SumW
        END IF        
    END DO
    Write(15,'(/,''                                                                                   AmountEuro&
    &       NumIndividual   Average      %Pop   Partial'')')
    write(15,'(''geen gezinsstatuut voor leefloon                                         '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(1,j),j=1,5)    
    write(15,'(''a person cohabitating with a partner or a single parent                  '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(2,j),j=1,5)    
    write(15,'(''statuut niet toegewezen                                                  '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(3,j),j=1,5)    
    write(15,'(''a true single (= person living alone)                                    '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(4,j),j=1,5)    
    write(15,'(''a cohabitating person (either married or not)                            '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(5,j),j=1,5)    
    write(15,'(''a person who is married or cohabitating and on which a means test        '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(6,j),j=1,5)    

    DEALLOCATE(SOCBEN_RESULT_TEST)

    !IGO/GWB naar gezinsstatuut
    NumAnal = 5
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(SOCBEN_RESULT_TEST(NumAnal,5))
    SOCBEN_RESULT_TEST(1:NumAnal,1:5)=0.D0

    !Compute aggregate statistics
    DO hh_i=1,N_HH_SOCBEN
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i) 
            TestID = 0
            !IGO recht en alleenstaand
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.2.AND.SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.1) THEN
                SOCBEN_RESULT_TEST(1,1)=SOCBEN_RESULT_TEST(1,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_IGO_ANN(pers_j)
                SOCBEN_RESULT_TEST(1,2)=SOCBEN_RESULT_TEST(1,2)+MIMOSIS_WEIGHT(pers_j)
                TestID = 1
            !IGO recht en samenwonend
            !dit geval kan ook waarde 6 krijgen door test op niveau GWB
            ELSE IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.2.AND.(SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.2.OR.&
            &SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.6)) THEN
                SOCBEN_RESULT_TEST(2,1)=SOCBEN_RESULT_TEST(2,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_IGO_ANN(pers_j)
                SOCBEN_RESULT_TEST(2,2)=SOCBEN_RESULT_TEST(2,2)+MIMOSIS_WEIGHT(pers_j)
                TestID = 1
            !GWB recht en gehuwd
            ELSE IF ((SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.3.AND.(SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.1.OR.&
            &SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.6)).OR.(SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.0.AND.&
            &SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.6)) THEN
                SOCBEN_RESULT_TEST(3,1)=SOCBEN_RESULT_TEST(3,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_IGO_ANN(pers_j)
                SOCBEN_RESULT_TEST(3,2)=SOCBEN_RESULT_TEST(3,2)+MIMOSIS_WEIGHT(pers_j)
                TestID = 1
            !GWB recht en gezinslast
            ELSE IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.3.AND.SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.2) THEN
                SOCBEN_RESULT_TEST(4,1)=SOCBEN_RESULT_TEST(4,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_IGO_ANN(pers_j)
                SOCBEN_RESULT_TEST(4,2)=SOCBEN_RESULT_TEST(4,2)+MIMOSIS_WEIGHT(pers_j)
                TestID = 1
            !GWB recht en alleenstaand
            ELSE IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.3.AND.SOCBEN_FAMIGO_2001_ANN(pers_j).EQ.3) THEN
                SOCBEN_RESULT_TEST(5,1)=SOCBEN_RESULT_TEST(5,1)+MIMOSIS_WEIGHT(pers_j)*SOCBEN_IGO_ANN(pers_j)
                SOCBEN_RESULT_TEST(5,2)=SOCBEN_RESULT_TEST(5,2)+MIMOSIS_WEIGHT(pers_j)
                TestID = 1
            END IF
!            IF (TestID.EQ.0.AND.SOCBEN_ENTLEIGO_ANN(pers_j,1).GT.1.AND.SOCBEN_FAMIGO_2001_ANN(pers_j).GT.0) THEN
!                WRITE(15,*) 'TestIGO',SOCBEN_ENTLEIGO_ANN(pers_j,1),SOCBEN_FAMIGO_2001_ANN(pers_j)
!            END IF
        END DO
    END DO
    
    !Write aggregate statistics to log-file
    DO i=1,NumAnal
        IF (INT(SOCBEN_RESULT_TEST(i,2)).NE.0) THEN
            SOCBEN_RESULT_TEST(i,3)=SOCBEN_RESULT_TEST(i,1)/SOCBEN_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SOCBEN_RESULT_TEST(i,4)=100.D0*SOCBEN_RESULT_TEST(i,2)/SumW
        END IF        
    END DO
    Write(15,'(/,''                                                                                   AmountEuro&
    &       NumIndividual   Average      %Pop   Partial'')')
    write(15,'(''IGO recht en alleenstaand                                                '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(1,j),j=1,5)    
    write(15,'(''IGO recht en samenwonend                                                 '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(2,j),j=1,5)    
    write(15,'(''GWB recht en gehuwd                                                      '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(3,j),j=1,5)    
    write(15,'(''GWB recht en gezinslast maar niet gehudw                                 '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(4,j),j=1,5)    
    write(15,'(''GWB recht en alleenstaand                                                '',F20.2,F20.0,F10.2,F10.1,F10.0)')&
    &(SOCBEN_RESULT_TEST(5,j),j=1,5)    

    DEALLOCATE(SOCBEN_RESULT_TEST)

    !Set the number of statistics to be computed
    NumAnal = 4
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(SOCBEN_RESULT_TEST(NumAnal,4))
    SOCBEN_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Compute aggregate statistics
    DO hh_i=1,N_HH_SOCBEN
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                                                                  
            !IT
            IF (SOCBEN_ENTIT_ANN(pers_j).EQ.1.AND.SOCBEN_ENTIVT_ANN(pers_j).EQ.0) THEN
                SOCBEN_RESULT_TEST(1,2)=SOCBEN_RESULT_TEST(1,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IVT
            IF (SOCBEN_ENTIT_ANN(pers_j).EQ.0.AND.SOCBEN_ENTIVT_ANN(pers_j).EQ.1) THEN
                SOCBEN_RESULT_TEST(2,2)=SOCBEN_RESULT_TEST(2,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IT/IVT
            IF (SOCBEN_ENTIT_ANN(pers_j).EQ.1.AND.SOCBEN_ENTIVT_ANN(pers_j).EQ.1) THEN
                SOCBEN_RESULT_TEST(3,2)=SOCBEN_RESULT_TEST(3,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !THB
            IF (SOCBEN_ENTTHB_ANN(pers_j).EQ.1) THEN
                SOCBEN_RESULT_TEST(4,2)=SOCBEN_RESULT_TEST(4,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO
    
    !Write aggregate statistics to log-file
    DO i=1,NumAnal
        IF (INT(SOCBEN_RESULT_TEST(i,2)).NE.0) THEN
            SOCBEN_RESULT_TEST(i,3)=SOCBEN_RESULT_TEST(i,1)/SOCBEN_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SOCBEN_RESULT_TEST(i,4)=100.D0*SOCBEN_RESULT_TEST(i,2)/SumW
        END IF        
    END DO
    Write(15,'(/,''                                                                                   AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    write(15,'(''individuals only receiving IT benefits according to available data       '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(1,j),j=1,4)    
    write(15,'(''individuals only receiving IVT benefits according to available data      '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(2,j),j=1,4)    
    write(15,'(''individuals receiving IT/IVT benefits according to available data        '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(3,j),j=1,4)    
    write(15,'(''individuals receiving THB benefits according to available data           '',F20.2,F20.0,F10.2,F10.1)')&
    &(SOCBEN_RESULT_TEST(4,j),j=1,4)    

    DEALLOCATE(SOCBEN_RESULT_TEST)

    !Set the number of statistics to be computed
    NumAnal = 10
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(SOCBEN_RESULT_TEST(NumAnal,4))
    SOCBEN_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Compute aggregate statistics
    DO hh_i=1,N_HH_SOCBEN
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                                                                  
            !Leefloon observed
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.1) THEN
                SOCBEN_RESULT_TEST(1,2)=SOCBEN_RESULT_TEST(1,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IGO observed
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).GT.1) THEN
                SOCBEN_RESULT_TEST(2,2)=SOCBEN_RESULT_TEST(2,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !Leefloon identified
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,2).EQ.1) THEN
                SOCBEN_RESULT_TEST(3,2)=SOCBEN_RESULT_TEST(3,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IGO identified
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,2).GT.1) THEN
                SOCBEN_RESULT_TEST(4,2)=SOCBEN_RESULT_TEST(4,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !Leefloon observed and identified
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.1.AND.SOCBEN_ENTLEIGO_ANN(pers_j,2).EQ.1) THEN
                SOCBEN_RESULT_TEST(5,2)=SOCBEN_RESULT_TEST(5,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !Leefloon observed but not identified
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).EQ.1.AND.SOCBEN_ENTLEIGO_ANN(pers_j,2).NE.1) THEN
                SOCBEN_RESULT_TEST(6,2)=SOCBEN_RESULT_TEST(6,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !Leefloon not observed but identified
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).NE.1.AND.SOCBEN_ENTLEIGO_ANN(pers_j,2).EQ.1) THEN
                SOCBEN_RESULT_TEST(7,2)=SOCBEN_RESULT_TEST(7,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IGO observed and identified
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).GT.1.AND.SOCBEN_ENTLEIGO_ANN(pers_j,2).GT.1) THEN
                SOCBEN_RESULT_TEST(8,2)=SOCBEN_RESULT_TEST(8,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IGO observed but not identified
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).GT.1.AND.SOCBEN_ENTLEIGO_ANN(pers_j,2).LT.2) THEN
                SOCBEN_RESULT_TEST(9,2)=SOCBEN_RESULT_TEST(9,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !IGO not observed but identified
            IF (SOCBEN_ENTLEIGO_ANN(pers_j,1).LT.2.AND.SOCBEN_ENTLEIGO_ANN(pers_j,2).GT.2) THEN
                SOCBEN_RESULT_TEST(10,2)=SOCBEN_RESULT_TEST(10,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO
    
    !Write aggregate statistics to log-file
    DO i=1,NumAnal
        IF (INT(SOCBEN_RESULT_TEST(i,2)).NE.0) THEN
            SOCBEN_RESULT_TEST(i,3)=SOCBEN_RESULT_TEST(i,1)/SOCBEN_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SOCBEN_RESULT_TEST(i,4)=100.D0*SOCBEN_RESULT_TEST(i,2)/SumW
        END IF        
    END DO
    Write(15,'(/,''                                                                                                                &
    &            AmountEuro       NumIndividual   Average      %Pop'')')
    write(15,'(''individuals identified to be eligible for leefloon according to available data                                    &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(1,j),j=1,4)    
    write(15,'(''individuals identified to be eligible for IGO according to available data                                         &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(2,j),j=1,4)    
    write(15,'(''individuals identified to be eligible for leefloon according to old conditions                                    &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(3,j),j=1,4)    
    write(15,'(''individuals identified to be eligible for IGO according to old conditions                                         &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(4,j),j=1,4)    
    write(15,'(''individuals identified to be eligible for leelfoon according to available data and old conditions                 &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(5,j),j=1,4)    
    write(15,'(''individuals identified to be eligible for leefloon according to available data but not according to old conditions&
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(6,j),j=1,4)    
    write(15,'(''individuals identified to be eligible for leefloon according to old conditions but not according to available data&
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(7,j),j=1,4)    
    write(15,'(''individuals identified to be eligible for IGO according to available data and old conditions                      &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(8,j),j=1,4)    
    write(15,'(''individuals identified to be eligible for IGO according to available data but not according to old conditions     &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(9,j),j=1,4)    
    write(15,'(''individuals identified to be eligible for IGO according to old conditions but not according to available data     &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(10,j),j=1,4)    

    DEALLOCATE(SOCBEN_RESULT_TEST)

    !Set the number of statistics to be computed
    !medical class IT and THB
    NumAnal = 10
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(SOCBEN_RESULT_TEST(NumAnal,4))
    SOCBEN_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Compute aggregate statistics
    DO hh_i=1,N_HH_SOCBEN
        DO pers_j=SOCBEN_START_HH(hh_i),SOCBEN_END_HH(hh_i)                                                                  
            !MedCat1
            IF (SOCBEN_MEDIT_ANN(pers_j).EQ.1) THEN
                SOCBEN_RESULT_TEST(1,2)=SOCBEN_RESULT_TEST(1,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !MedCat2
            IF (SOCBEN_MEDIT_ANN(pers_j).EQ.2) THEN
                SOCBEN_RESULT_TEST(2,2)=SOCBEN_RESULT_TEST(2,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !MedCat3
            IF (SOCBEN_MEDIT_ANN(pers_j).EQ.3) THEN
                SOCBEN_RESULT_TEST(3,2)=SOCBEN_RESULT_TEST(3,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !MedCat4
            IF (SOCBEN_MEDIT_ANN(pers_j).EQ.4) THEN
                SOCBEN_RESULT_TEST(4,2)=SOCBEN_RESULT_TEST(4,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !MedCat5
            IF (SOCBEN_MEDIT_ANN(pers_j).EQ.5) THEN
                SOCBEN_RESULT_TEST(5,2)=SOCBEN_RESULT_TEST(5,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !MedCat1
            IF (SOCBEN_MEDTHB_ANN(pers_j).EQ.1) THEN
                SOCBEN_RESULT_TEST(6,2)=SOCBEN_RESULT_TEST(6,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !MedCat2
            IF (SOCBEN_MEDTHB_ANN(pers_j).EQ.2) THEN
                SOCBEN_RESULT_TEST(7,2)=SOCBEN_RESULT_TEST(7,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !MedCat3
            IF (SOCBEN_MEDTHB_ANN(pers_j).EQ.3) THEN
                SOCBEN_RESULT_TEST(8,2)=SOCBEN_RESULT_TEST(8,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !MedCat4
            IF (SOCBEN_MEDTHB_ANN(pers_j).EQ.4) THEN
                SOCBEN_RESULT_TEST(9,2)=SOCBEN_RESULT_TEST(9,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
            !MedCat5
            IF (SOCBEN_MEDTHB_ANN(pers_j).EQ.5) THEN
                SOCBEN_RESULT_TEST(10,2)=SOCBEN_RESULT_TEST(10,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO
    
    !Write aggregate statistics to log-file
    DO i=1,NumAnal
        IF (INT(SOCBEN_RESULT_TEST(i,2)).NE.0) THEN
            SOCBEN_RESULT_TEST(i,3)=SOCBEN_RESULT_TEST(i,1)/SOCBEN_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            SOCBEN_RESULT_TEST(i,4)=100.D0*SOCBEN_RESULT_TEST(i,2)/SumW
        END IF        
    END DO
    Write(15,'(/,''                                                          AmountEuro       &
    &NumIndividual   Average      %Pop'')')
    write(15,'(''MedCat IT 1                                     &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(1,j),j=1,4)    
    write(15,'(''MedCat IT 2                                     &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(2,j),j=1,4)    
    write(15,'(''MedCat IT 3                                     &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(3,j),j=1,4)    
    write(15,'(''MedCat IT 4                                     &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(4,j),j=1,4)    
    write(15,'(''MedCat IT 5                                     &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(5,j),j=1,4)    
    write(15,'(''MedCat THB 1                                    &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(6,j),j=1,4)    
    write(15,'(''MedCat THB 2                                    &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(7,j),j=1,4)    
    write(15,'(''MedCat THB 3                                    &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(8,j),j=1,4)    
    write(15,'(''MedCat THB 4                                    &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(9,j),j=1,4)    
    write(15,'(''MedCat THB 5                                    &
    &'',F20.2,F20.0,F10.2,F10.1)') (SOCBEN_RESULT_TEST(10,j),j=1,4)    

    DEALLOCATE(SOCBEN_RESULT_TEST)

END SUBROUTINE SOCBEN_TEST

!=================================================================================!
!Write some variables to execute some tests with SAS
!=================================================================================!

SUBROUTINE WRITE_SOCBEN

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\SOCBEN_TEST.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        WRITE(12,'(A14,3I10,3F15.2,2I10,3F15.2,I10)') GECODEERD_INSZ_CHAR(pers_j),FAMREL_RELATION(pers_j),FAMREL_COUPLE(pers_j),&
        &SOCBEN_FAMSIT_2001_ANN(pers_j),SOCBEN_LEEFLOON_ANN(pers_j),SOCBEN_IGO_ANN(pers_j),SOCBEN_NMLEEFL_ANN(pers_j),&
        &NAREGNIS_HH_ID(pers_j),NAREGNIS_RELATION(pers_j),CONTRIB_GTIPRIV_ANN(pers_j),CONTRIB_GTIPUB_ANN(pers_j),&
        &CONTRIB_GTISELF_ANN(pers_j),SOCBEN_YEAR_AGE(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine ...............................runned correctly'

END SUBROUTINE WRITE_SOCBEN

!======================================================================!
!Open output file and write output needed for the evaluation module    !
!======================================================================!
SUBROUTINE SOCBEN_OUTPUT_FILEWRIT

    INTEGER:: pers_j 

    OPEN (UNIT=12, FILE=EVAL_SOCBEN,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,N_REC
        SOCBEN_MINIMEX_ANN(pers_j) = SOCBEN_LEEFLOON_ANN(pers_j) + SOCBEN_IGO_ANN(pers_j) + &
        &SOCBEN_IVT_ANN(pers_j) + SOCBEN_IT_ANN(pers_j) + SOCBEN_THB_ANN(pers_j)
        WRITE(12,'(A14,F15.2)')  GECODEERD_INSZ_CHAR(pers_j),SOCBEN_MINIMEX_ANN(pers_j)/4.D0      
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine SOCBEN_OUTPUT_FILEWRIT of NORMAL RUN.......runned correctly'

END SUBROUTINE SOCBEN_OUTPUT_FILEWRIT 
               
SUBROUTINE SOCBEN_CALC
    
    !Programme headding
    !==========================================================================
    WRITE (*,*) ' '
    WRITE (*,*) 'MIMOSIS SOCBEN IS RUNNING'
    WRITE (*,*) ' '
    !==========================================================================    
    !open log file      
    OPEN(UNIT=15, FILE=SOCBEN_LOG, STATUS='REPLACE', ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_LOG
        WRITE(15,'("Could not find ", a100)') SOCBEN_LOG
        PAUSE
        STOP
    END IF

    !Allocate all vectors with dimension n_rec and initialise them to zero
    CALL SOCBEN_ALLOC_EXOG(n_rec)
    CALL SOCBEN_ALLOC_ENDO(n_rec)
    
    !Initalise endogenous variables
    CALL SOCBEN_INITENDO(n_rec)
    !Read in PARAMETER file
    CALL SOCBEN_PARAMETER_FILEREAD                                                           
    !Read in INPUT files and sort them according to individual ID
    CALL SOCBEN_INPUT_FILEREAD                                                               
    CALL SOCBEN_PIT_FILEREAD                                                               
    CALL SOCBEN_FAMAL_FILEREAD
    CALL SOCBEN_CONTRIB_FILEREAD                                                               
    CALL SOCBEN_UNEM_FILEREAD                                                               
    CALL SOCBEN_SICK_FILEREAD                                                               
    CALL SOCBEN_CONSTRUCT_FILEREAD
    CALL SOCBEN_FAMREL_FILEREAD
    CALL SOCBEN_WEIGHT_FILEREAD        
    !Rank and sort the different input data sets
    CALL SOCBEN_RANKANDSORT
    !Recode some exogenous variable values
    CALL SOCBEN_RECODE
    !Inflate values
    CALL SOCBEN_INFLATE
    !Determine household size  
    CALL SOCBEN_HOUSEHOLD_SIZE
    !Determine age of all individuals
    CALL SOCBEN_AGE

    WRITE(15,'(/,''Print of function calls for which no selection was carried out'')')
    CALL DRAWLINE(15,100,'=')                

    !Transform quarterly data into yearly data
    CALL SOCBEN_TRANSFORM_QUARTERLY_DATA          

    !Start loop over households
    DO hh_i=1,N_HH_SOCBEN
        CALL SOCBEN_INSTRUM_A(hh_i)
        !Compute value leefloon
        IF (SIMYEAR.LT.2003) THEN
            CALL SOCBEN_LEEFLOON_2001(hh_i)
        ELSE IF (SIMYEAR.GE.2003.AND.SIMYEAR.LT.2006) THEN
            CALL SOCBEN_LEEFLOON_2003(hh_i)    
        ELSE 
            CALL SOCBEN_LEEFLOON_2005(hh_i)    
        END IF    
        !Compute the IGO value
        !Remark: we always apply the IGO rules and not the GIB rules
        CALL SOCBEN_IGO_2001(hh_i)
        !Compute instruments for IT after IGO has been computed
        IF (ANY(SOCBEN_SIMDGPMH_QT.EQ.(/0, 1, 2, 3, 7/))) THEN 
            CALL SOCBEN_INSTRUM_B(hh_i)
        ELSE IF (ANY(SOCBEN_SIMDGPMH_QT.EQ.(/4, 5, 6/))) THEN 
            CALL SOCBEN_INSTRUM_C(hh_i)
        END IF            
        !Compute the IVT, IT and THB value
        !Remark that we apply the same rules from 2001 onwards
        !although the rules were applied in the cours of 2003
        IF (SIMYEAR.LT.2003) THEN
            CALL SOCBEN_IVTIT_2001(hh_i)
        ELSE
            CALL SOCBEN_IVTIT_2004(hh_i)
        END IF
        CALL SOCBEN_THB_2001(hh_i)
        !Run instruments and module to compute health benefit status
        CALL SOCBEN_INSTRUM_HEALTH(hh_i)
        CALL SOCBEN_HEALTBEN_2008(hh_i)
    END DO

    CALL SOCBEN_TEST

    !Writes output for use in the evaluation module
    CALL SOCBEN_OUTPUT_FILEWRIT
    
!    CALL WRITE_SOCBEN
    
    CALL SOCBEN_DEALLOC

END SUBROUTINE SOCBEN_CALC

END MODULE SOCBEN_SUBROUTINES
